--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

游戏声音管理类，定义声音管理相关操作方法及逻辑实现。

-   实现声音管理常用操作。

]]

local M = {}

setmetatable(M, { __index = function ( table, key )  
    return audio[key]
end})
-- [公共参数]
-- 常量
-- ..

-- [操作变量]
-- ..

local PATH_STORAGE = string.gsub(PTAH_STORAGE_RES, "/res/", "/")
local function getFilePath( fileName )
    local storagePath = PATH_STORAGE .. string.gsub(fileName, "res/sound/", "sound/")

    if io.exists(storagePath) then 
        return storagePath
    else
        return fileName
    end
end

-- audio.playMusic audio.playBackgroundMusic
local playMusic_ = audio.playMusic
function audio.playMusic(filename, isLoop)
    playMusic_( getFilePath(filename), isLoop )
end
audio.playBackgroundMusic = audio.playMusic
-- audio.preloadMusic audio.preloadBackgroundMusic
local preloadMusic_ = audio.preloadMusic
function audio.preloadMusic(filename)
    preloadMusic_( getFilePath(filename) )
end
audio.preloadBackgroundMusic = audio.preloadMusic
-- audio.playSound audio.playEffect
local playSound_ = audio.playSound
function audio.playSound(filename, isLoop)
    playSound_( getFilePath(filename), isLoop )
end
audio.playEffect = audio.playSound
-- audio.preloadSound audio.preloadEffect
local preloadSound_ = audio.preloadSound
function audio.preloadSound(filename)
    preloadSound_( getFilePath(filename) )
end
audio.preloadEffect = audio.preloadSound
-- audio.unloadSound audio.unloadEffect
local unloadSound_ = audio.unloadSound
function audio.unloadSound(filename)
    unloadSound_( getFilePath(filename) )
end
audio.unloadEffect = audio.unloadSound


-- [工具方法]
-- [工具方法][预加载]
local function preloadBackgroundMusic_( fileName )
    audio.preloadMusic(PHS_BACKGROUND .. fileName)
end
-- 预加载背景音乐[主页]
function M:preloadMusicHome() 
    preloadBackgroundMusic_("bg_home.mp3")
end

-- 预加载背景音乐[竞技场]
function M:preloadMusicArena() 
    preloadBackgroundMusic_("bg_arena.mp3")
end

-- 预加载背景音乐[地图]
function M:preloadMusicArena() 
    preloadBackgroundMusic_("bg_map.mp3")
end

-- 预加载背景音乐[战斗]
function M:preloadMusicFight() 
    preloadBackgroundMusic_( "bg_fight.mp3" )
end

-- 预加载背景音乐[四格漫画]
function M:preloadMusicFight() 
    preloadBackgroundMusic_( "bg_fight.mp3" )
end


---------


-- [工具方法][背景音乐]
local function playBackgroundMusic_( fileName, isLoop )
    M:playOrIgnoreMusic(PHS_BACKGROUND .. fileName, ifnil(isLoop, true))
end
-- 播放背景音乐[主页]
function M:playMusicHome(isLoop)
    playBackgroundMusic_( "bg_home.mp3", isLoop )
end
-- 播放背景音乐[主页]
function M:playMusicMap(isLoop)
    playBackgroundMusic_( "bg_map.mp3", isLoop )
end
-- 播放背景音乐[竞技场]
function M:playMusicArena(isLoop)
    playBackgroundMusic_( "bg_arena.mp3", isLoop )
end

-- 播放背景音乐[探索]
function M:playMusicExplore(isLoop)
    playBackgroundMusic_( "bg_explore.mp3", isLoop )
end

function M:playMusicBoss( isLoop )
    playBackgroundMusic_( "bg_team_war.mp3", isLoop )
    
end

-- 播放背景音乐[军团]
function M:playMusicArmy(isLoop)
    playBackgroundMusic_( "bg_army.mp3", isLoop )
end

-- 播放背景音乐[战斗]
function M:playMusicFight(isLoop)
    playBackgroundMusic_( "bg_fight.mp3", isLoop )
end

-- 播放背景音乐[战斗]
function M:playMusicComicOpening(isLoop)
    playBackgroundMusic_( "bg_opening.mp3", isLoop )
end

-- 播放背景音乐[许愿池]
function M:playMusicWishPool(isLoop)
    playBackgroundMusic_( "bg_wish_pool.mp3", isLoop )
end

-- 播放背景音乐[战队]
function M:playMusicTeamWar(isLoop)
    playBackgroundMusic_( "bg_boss.mp3", isLoop )
end

-- 播放背景音乐[大转盘]
function M:playMusicBigWheel(isLoop)
    playBackgroundMusic_( "bg_big_wheel.mp3", isLoop )
end

---------


-- [工具方法][音效]
local function playButtonEffect_( fileName )
    audio.playEffect(PHS_BUTTON .. fileName)
end
-- 播放音效[按钮音1]
function M:playEffectButton1()
    playButtonEffect_("btn_1.mp3")
end
-- 播放音效[按钮音2]
function M:playEffectButton2()
    playButtonEffect_("btn_2.mp3")
end

local function playMapEffect_( fileName )
    audio.playEffect(PHS_MAP .. fileName)
end
-- 播放音效[地图－新关卡]
function M:playEffectMapNewFlag()
    playMapEffect_("m_newflag.mp3")
end

local function playFightResultEffect_( fileName )
    audio.playEffect(PHS_FIGHT_RESULT .. fileName)
end
-- 播放音效[战斗－胜利]
function M:playEffectFightWin()
    playFightResultEffect_("f_res_win.mp3")
end
function M:playSoundCardShow() 
    playFightResultEffect_("sfx-notenoughmp.mp3")
end
-- 播放音效[战斗－失败]
function M:playEffectFightLose()
    playFightResultEffect_("f_res_lose.mp3")
end

-- 播放音效[战斗－升级]
function M:playEffectFightLevelUp()
    playFightResultEffect_("level_up.mp3")
end

-- 播放音效[战斗－解锁]
function M:playEffectFightUnlock()
    playFightResultEffect_("unlock.mp3")
end

-- 播放音效[战斗－获得高级卡牌]
function M:playEffectFightRewardAdvanced()
    playFightResultEffect_("reward_advanced.mp3")
end

-- 播放音效[获得奖励]
function M:playEffectFightReward()
    playFightResultEffect_("f_res_reward.mp3")
end

-- 播放音效[门－关]
function M:playFightDoorClose()
    audio.playEffect(PHS_FIGHT_DOOR .. "f_d_close.mp3")
end

-- 播放音效[门－开]
function M:playFightDoorOpen()
    audio.playEffect(PHS_FIGHT_DOOR .. "f_d_open.mp3")
end

-- 播放音效[星星]
function M:playStar()
    playButtonEffect_("star.mp3")
end

-- 播放音效[星星－掠过]
function M:playStarFlit()
    playButtonEffect_("star_flit.mp3")
end

-- 播放音效[武器装备]
function M:playEquip()
    playButtonEffect_("weapon.mp3")
end
-- 播放音效[金币]
function M:playGold()
    playButtonEffect_("gold.mp3")
end
-- 播放音效[铁匠]
function M:playBlackSmith()
    playButtonEffect_("smith.mp3")
end
-- 播放音效[挖矿]
function M:playMining()
    playButtonEffect_("mine.mp3")
end
---------


-- [工具方法][内部调用]
function M:playOrIgnoreMusic(fileName, isLoop)
	if audio.getFileNameMusic() ~= fileName then 
		-- audio.setFileNameMusic(fileName)
		audio.playMusic(fileName, ifnil(isLoop, true))
	end
end

-- 停止背景音乐
function M:playMusicStop()
    audio.pauseMusic()
end





return M