local UpdateHelper = {}


--[[ 

载入lua打包的zip文件,只是放在这有点尴尬...没处可调用

**Parameters:**

-   fileName:        string                  zip文件名
-   androidPath:     string                  android下的文件所在目录的绝对路径,通常是sd卡上
-   iosPath:         string                  ios下文件所在目录的相对路径,处于documentsPath下,通常不填写
-   appPath:         string                  app下的相对路径,不填写则默认res下

--]]
function UpdateHelper.loadLuaZip( fileName, androidPath, iosPath, appPath )
	if ENABLED_LUA_ZIP_SOURCE then 
        if device.platform == "android" then
            if io.exists(androidPath .. fileName) then
                CCLuaLoadChunksFromZip(androidPath .. fileName)
                return
            end
        else
            local path = device.documentsPath .. (iosPath and iosPath or "") .. fileName
            if io.exists(path) then
                CCLuaLoadChunksFromZip(path)
                return
            end
        end
        CCLuaLoadChunksFromZip((appPath and appPath or PATH_RES) .. fileName)
    end
end

function UpdateHelper.fetchDBPath( dbFile )
    local path = PTAH_STORAGE_RES .. dbFile
    if not io.exists(path) then 
        path = PATH_RES .. dbFile
    end
    return path
end

--[[ 

下载文件

**Parameters:**

-   url:        string                  文件下载地址
-   callback:   fn(fileName, bool)      下载后的回调,第二个参数表示下载是否成功
-   progressCallback:   fn(event{name,request,length})      进度回调
-   timeout:    int                     超时时间,可不填写
-   filePathInDocPath: string           文件在documentsPath下的路径,不包括文件名,如"sd/"

**Returns:**

-   return return

e.g.
network.downloadfile("http://ww1.sinaimg.cn/large/7f639ebfjw1e455va2z9xj20c715en13.jpg", function ( fileName, successful )
    print("file", fileName, "download", successful)
end)

--]]
function UpdateHelper.downloadfile( url, callback, progressCallback, timeout, filePathInDocPath )
    local function downloadCallback( event )
        local request = event.request
        local fileName = io.pathinfo(url).filename
        filePathInDocPath = filePathInDocPath or ""
        try(
            function ( ... )
                local filePath = device.documentsPath .. filePathInDocPath .. fileName
                FileManager:createDirectory(device.documentsPath .. filePathInDocPath)
                if io.writefile(filePath, request:getResponseDataLua(), "wb") then
                    if callback then callback(fileName, true) end
                else
                    if callback then callback(fileName, false) end
                    if io.remove then 
                        io.remove(filePath)
                    else
                        os.execute("rm -r -f \"" .. filePath .. "\"")
                    end
                end
            end,
            function ( ... )
                if callback then callback(fileName, false) end
            end
            )
    end
    local request = network.createHTTPRequest(downloadCallback, url, "GET")
    if type(timeout) == "number" then request:setTimeout(timeout) end
    if progressCallback and request.setProgressCallback then
        request:setProgressCallback(progressCallback) 
    end
    request:start()
end
network.downloadfile = UpdateHelper.downloadfile
--[[ 

解压zip文件

**Parameters:**

-   zipFilePath:        string       zip文件绝对路径
-   destPath:           string       解压路径

**Returns:**

-   return 是否成功

--]]
function UpdateHelper.unzip( zipFilePath, destPath )
    return FileManager:uncompressZipFile( zipFilePath, destPath )
end

--[[ 

下载并解压zip文件

**Parameters:**

-   url:        string                  文件下载地址
-   callback:   fn(fileName, bool)      结果回调,第二个参数表示下载是否成功
-   progressCallback:   fn(event{name,request,length})      进度回调
-   times:      int                     尝试次数,超过测试次数还未成功就为失败,默认为5
-   deleteZip:  boolean                 是否删除下载的zip文件,默认为true
-   timeout:    int                     超时时间,可不填写

e.g.
UpdateHelper.downloadAndUnzipFile("http://cocos2d-x.googlecode.com/files/cocos2d-2.0-x-2.0.3.zip",
    function ( filename, successful )
        print("download " , tostring(successful))
    end)
--]]
function UpdateHelper.downloadAndUnzipFile( url, callback, progressCallback, times, deleteZip, timeout )
    times = times or 5
    -- Log.d(TAG, "downloadAndUnzipFile()=>%s, times:%d", url, times)
    local function downloadAndUnzipFile_( url )
        network.downloadfile(url, function ( filename, successful )
            -- Log.d(TAG, "downloadAndUnzipFile_()=>filename:%s, successful:%s, times:%d", 
            --     filename, tostring(successful), times)
            if successful then
                --下载成功,解压
                if not UpdateHelper.unzip(device.documentsPath .. filename, PTAH_STORAGE_RES) then
                    --解压失败又得重新下啊
                    times = times - 1
                    if times > 0 then 
                        downloadAndUnzipFile_(url, times)
                    else
                        -- Log.d(TAG, "downloadAndUnzipFile() FAIL!!!")
                        if callback then callback(filename, false) end
                        return
                    end
                else
                    if callback then callback(filename, true) end
                end
                --删除掉zip吧
                if deleteZip ~= false then FileManager:removeFile(device.documentsPath .. filename) end
            else
                --下载失败,继续下
                times = times - 1
                if times > 0 then 
                    downloadAndUnzipFile_(url)
                else
                    -- Log.d(TAG, "downloadAndUnzipFile() FAIL!!!")
                    if callback then callback(filename, false) end
                    return 
                end
            end
        end, progressCallback, timeout or 1800)
    end
    downloadAndUnzipFile_(url)
end

local function removeStorageFiles()
    local function remove_(fileName)
        local fullpath = PTAH_STORAGE_RES .. fileName
        if device.platform == "ios" then 
            local luaoc = require("framework.client.luaoc")
            -- 进行每一项的删除
            luaoc.callStaticMethod( "AppController", "removeFile", {file = fullpath})
        else
            os.execute("rm -r -f \"" .. fullpath .. "\"")
        end
    end
    remove_("data")
    remove_("data.rdb")
    remove_("data_cn.rdb")
    remove_("data_tw.rdb")
    remove_("gamepackage.bin")
    remove_("VERSION")
    remove_(FILE_VERSION)
    remove_("sound")
    remove_("sd")
    remove_("ccb")
end
--[[ 

判定版本号,若APP的版本号比LUA增量包的来得新,则清除document下的指定文件
APP版本由config中的APP_VERSION定义
LUA增量包版本由每次增量包中带的VERSION文件定义,里面只写版本号

若不存在VERSION文件,则直接清除document下的指定文件

**Returns:**

-   return 是否进行了清空操作

--]]
function UpdateHelper.clearDocument()
    if APP_VERSION then 
        local lua_version = io.readfile(PTAH_STORAGE_RES .. FILE_VERSION)
        if not lua_version then 
            lua_version = io.readfile(PTAH_STORAGE_RES .. "VERSION")
        end
        if lua_version then 
            if APP_VERSION > lua_version then
                removeStorageFiles()
                return true
            end
        elseif lua_version == "" or lua_version == nil then 
            removeStorageFiles()
            return true
        end
    end
    return false
end


return UpdateHelper






















