---[===[
local CloudView = class("CloudView", function ( ... )
	return widget.newLayer{ 
		-- color = ccc4(120,120,120,120), 
		ignoreAnchor = false 
	}
end)

local MOVE_UNIT = 4
CloudView.MOVE_DIFF = {x = MOVE_UNIT, y = MOVE_UNIT}
CloudView.CLOUD_BIG = 2
CloudView.CLOUD_SMALL = 1

local cur_position = CCPointZero

function CloudView.changeMoveDiff()
	local ran = math.random(1, 2)
	local dx, dy 
	dx = math.randomset{1, -1} * MOVE_UNIT
	dy = math.randomset{1, -1} * MOVE_UNIT
	-- if dx ~= CloudView.MOVE_DIFF.x or dy ~= CloudView.MOVE_DIFF.y then 
	-- 	CloudView.MOVE_DIFF.x = dx
	-- 	CloudView.MOVE_DIFF.y = dy
	-- else
	-- 	CloudView.changeMoveDiff()
	-- end
	local p = ccpAdd(cur_position, ccp(dx, dy))
	if ccpDistance(CCPointZero, p) < 16 then 
		cur_position = p
		CloudView.MOVE_DIFF.x = dx
		CloudView.MOVE_DIFF.y = dy
	else
		CloudView.changeMoveDiff()
	end
end

local function createMore( self, size, count )
	self._size = size
	self:setContentSize(size)

	count = count or 4

	self._count = count == 6 and 6 or 4

	self._clouds = {
		"map/clouds/cloud_01.png",
		"map/clouds/cloud_02.png",
		"map/clouds/cloud_03.png",
		"map/clouds/cloud_04.png",
		"map/clouds/cloud_05.png",
		"map/clouds/cloud_06.png",
	}
	-- 随机4张放入其中
	self._clouds = math.randomset(self._clouds, self._count)
	for i, v in ipairs(self._clouds) do
		local sprite = display.newSprite(v)
		self:addChild(sprite)
		self._clouds[i] = sprite
	end
	local ranMinX, ranMaxX = -2, 2
	local ranMinY, ranMaxY = -2, 2

	if self._count == 6 then 
		util.alignNode(self._clouds[1], self, display.TOP_LEFT
			, { top = math.random(ranMinX, ranMaxX), left = math.random(ranMinY, ranMaxY) })

		util.alignNode(self._clouds[2], self, display.CENTER_LEFT
			, { left = math.random(ranMinY, ranMaxY) })

		util.alignNode(self._clouds[3], self, display.BOTTOM_LEFT
			, { bottom = math.random(ranMinX, ranMaxX), left = math.random(ranMinY, ranMaxY) })

		util.alignNode(self._clouds[4], self, display.TOP_RIGHT
			, { top = math.random(ranMinX, ranMaxX), right = math.random(ranMinY, ranMaxY) })
		
		util.alignNode(self._clouds[5], self, display.CENTER_RIGHT
			, { right = math.random(ranMinY, ranMaxY) })

		util.alignNode(self._clouds[6], self, display.BOTTOM_RIGHT
			, { bottom = math.random(ranMinX, ranMaxX), right = math.random(ranMinY, ranMaxY) }) 
	elseif self._count == 4 then 
		util.alignNode(self._clouds[1], self, display.TOP_LEFT
			, { top = math.random(ranMinX, ranMaxX), left = math.random(ranMinY, ranMaxY) })

		-- util.alignNode(self._clouds[2], self, display.CENTER_LEFT
		-- 	, { left = math.random(ranMinY, ranMaxY) })

		util.alignNode(self._clouds[3], self, display.BOTTOM_LEFT
			, { bottom = math.random(ranMinX, ranMaxX), left = math.random(ranMinY, ranMaxY) })

		util.alignNode(self._clouds[4], self, display.TOP_RIGHT
			, { top = math.random(ranMinX, ranMaxX), right = math.random(ranMinY, ranMaxY) })
		
		-- util.alignNode(self._clouds[5], self, display.CENTER_RIGHT
		-- 	, { right = math.random(ranMinY, ranMaxY) })

		util.alignNode(self._clouds[2], self, display.BOTTOM_RIGHT
			, { bottom = math.random(ranMinX, ranMaxX), right = math.random(ranMinY, ranMaxY) }) 

		-- util.alignNode(self._clouds[7], self, display.TOP_CENTER
		-- 	, { top = math.random(ranMinX, ranMaxX)})
		
		-- util.alignNode(self._clouds[8], self, display.CENTER)

		-- util.alignNode(self._clouds[9], self, display.BOTTOM_CENTER
		-- 	, { bottom = math.random(ranMinX, ranMaxX)}) 
	end
end

local function createSingle( self, cloudType )
	self._count = 1
	if cloudType == CloudView.CLOUD_SMALL then 
		-- 小云
		self._clouds = {
			"map/clouds/cloud_01.png",
			"map/clouds/cloud_04.png",
			"map/clouds/cloud_06.png",
		}
	else 
		-- 大云
		self._clouds = {
			"map/clouds/cloud_02.png",
			"map/clouds/cloud_03.png",
			"map/clouds/cloud_05.png",
		}
	end
	self._clouds = math.randomset(self._clouds, self._count)
	self._clouds = display.newSprite(self._clouds)
	self:setContentSize(self._clouds:getContentSize())
	self:addChild(self._clouds)
	util.alignNode(self._clouds, self)
end

function CloudView:ctor(size, count)
	if isnumber(size) then 
		-- 单云
		createSingle( self, size )-- 1.小云;2.大云
	else
		-- 多云
		createMore( self, size, count )
	end

	self:move()
end

function CloudView:disperse()
	self:stopAllActions()
	-- 一半的云向左,另一半的向右,移动过程中渐渐消失
	local function disperse_( target, left )
		if left then 
			transition.moveBy(target, { x = -target:getContentSize().width / 2, time = 2.0 })
		else
			transition.moveBy(target, { x = target:getContentSize().width / 2, time = 2.0 })
		end
		transition.fadeOut(target, {time = 2.0} )
	end

	for i = 1, math.floor(self._count / 2) do
		disperse_(self._clouds[i], true)
	end
	for i = math.floor(self._count / 2) + 1, self._count do
		disperse_(self._clouds[i], false)
	end

	self:performWithDelay(function ()
		self:removeSelf()
	end, 2.5)
end

local function singleCloudMove_( target )
	transition.moveBy(target
		, {
			x = math.random(-5, 5)
			, y = math.random(-5, 5)
			, time = 2
		})
end

function CloudView:move()
	transition.moveBy(self
		, {
			x = CloudView.MOVE_DIFF.x
			, y = CloudView.MOVE_DIFF.y
			, time = 2
			, onComplete = function (  )
				self:move()
			end
		})
	if self._count ~= 1 then 
		for i = 1, self._count do
			singleCloudMove_(self._clouds[i])
		end
	end
end

return CloudView
--]===]