---[==[
local EffectButton = class("EffectButton", function ( ... )
	return widget.newLayer()
end)

EffectButton.TAG = "[EffectButton]"

local picformat = "main/effect_buttons/%s"

function EffectButton:ctor( filename, text, callback, params )
	util.checkParams(EffectButton.TAG, "ctor", {
		{ filename, "filename"}
		, { filename, "text"}
		})
	self.params_ = params or {}
	self.params_.filename = filename
	self.params_.bgfilename = self.params_.bgfilename or string.format(picformat, "bg_main_button.png")
	self.params_.text = text
	self.params_.callback = callback
	self.views_ = {}
	-- 我们来构建下整个基本样式吧
	-- 以最大的光为尺寸大小吧 -2
	-- local sprite = display.newSprite(string.format(picformat, "img_light.png"))
	-- self:setContentSize(sprite:getContentSize())
	-- self:addChild(sprite, -2)
	-- util.alignNode(sprite, self)
	-- sprite:setVisible(false)
	display.addSpriteFramesWithFile(string.format(picformat, "img_light.plist"),string.format(picformat, "img_light.png"))
    local sprite = display.newSprite("#img_light_1.png")
    local animation = anim.createAnimation(
      string.format(picformat, "img_light")
        , "img_light_%d.png"
        , 1, 12,0.05
        , {1, 12})
    transition.playAnimationForever(sprite,animation)
    util.alignNode(sprite,self)
    self:setContentSize(sprite:getContentSize())
    self:addChild(sprite,1000)
    util.alignNode(sprite,self)
    sprite:setVisible(false)

	self.views_.light = sprite
	-- 放个圆盘背景 -1
	if not self.params_.noBgSprite then
		sprite = display.newSprite(self.params_.bgfilename)
		self:addChild(sprite, -1)
		util.alignNode(sprite, self)
		self.views_.bg = sprite
	end
	
	-- 来个layer放图,加触控 0
	local layer = display.newLayer()
	self.views_.buttonlayer = layer
	-- 放入正式图片的button
	sprite = widget.newButton{
		normalImage = string.format(picformat, self.params_.filename .. ".png"),
		scale = 1.2,
		swallowsTouches = true,
		listener = function ( button )
			SOUND:playEffectButton1()
			if self.params_.callback then 
				self.params_.callback(self)
			end
		end
	} 
	layer:setContentSize(sprite:getContentSize())
	layer:addChild(sprite, 0)
	if istable(self.params_.offset) then
		util.alignNode(sprite, layer, display.CENTER, self.params_.offset)
	else
		util.alignNode(sprite, layer)
	end
	self.views_.button = sprite
	if self.params_.text then
		-- 放上文字背景
		sprite = display.newSprite("main/bg_button_title.png")
		sprite:setScale(1.1)
		-- 来点文字 
		local label = ui.newTTFLabel{ text = self.params_.text, size = 18}
		self.views_.label = label
		-- 标题相关的容器
		local textLayer = widget.newLayer()
		textLayer:setContentSize(sprite:getContentSize())
		textLayer:addChild(sprite, 0)
		util.alignNode(sprite, textLayer)
		textLayer:addChild(label, 1)
		util.alignNodeWithPadding(label, textLayer, display.CENTER, {top = 2})
		self:addChild(textLayer, 1)
		util.alignNode(textLayer, self, display.BOTTOM_CENTER, 20)
	end
	
	self:addChild(layer, 0)
	util.alignNode(layer, self)
end

function EffectButton:flashOn( force )
	if force == true and self.params_.flash ~= true then 
		self.params_.flash = true 
	end
	if self.params_.flash and not self._flashOn then 
		-- 开始放动画吧
		self.views_.button:stopAllActions()
		local animation = anim.createAnimation(
			string.format(picformat, "anim_" .. self.params_.filename)
			, self.params_.filename .. "_%02d.png"
			, 1, 10, 0.05 
            , {1, 10}, {10, 1}, {1})
        self.views_.button.playAnimation(animation, true)
        self._flashOn = true
	end
end

function EffectButton:flashOff()
	if self._flashOn then 
		self._flashOn = nil
		self.views_.button.stopAnimation()
	end
end

function EffectButton:lightOn( force )
	if force == true and self.params_.light ~= true then 
		self.params_.light = true 
	end
	if self.params_.light and not self._lightOn then
		self.views_.light:setVisible(true)
		--anim.circling(self.views_.light, {forever = true, forward = true})
        -- 创建新消息渐隐动画
        --anim.fadeFade(self.views_.light, {min = 170, forever = true, playTime = 0.4})
		self._lightOn = true
	end
end

function EffectButton:lightOff()
	if self._lightOn then 
		self._lightOn = nil
		self.views_.light:stopAllActions()
		self.views_.light:setVisible(false)
	end
end

function EffectButton:markOn( force )
	if force == true and self.params_.mark ~= true then 
		self.params_.mark = true 
	end
	if self.params_.mark and not self.views_.mark then
		local sprite = widget.newSprite{file = "main/img_sign.png",}
		sprite:setScale(0.8)
		self.views_.buttonlayer:addChild(sprite, 3)
		util.alignNode(sprite, self.views_.buttonlayer, display.BOTTOM_RIGHT, self.params_.markPadding or { right = 4})
		sprite:setRotation(20)
        transition.execute(sprite, CCRepeatForever:create(transition.sequence{
        	CCScaleTo:create(0.2 , 0.8, 1)
        	, CCScaleTo:create(0.2 , 0.8, 0.8)
        	}))
		self.views_.mark = sprite
		self._mark = true
	end
end

function EffectButton:markOff()
	if self._mark then 
		self._mark = nil
		self.views_.mark:stopAllActions()
		self.views_.mark:removeSelf()
		self.views_.mark = nil
	end
end

function EffectButton:shakeOn( force )
	if force == true and self.params_.shake ~= true then 
		self.params_.shake = true 
	end
	if self.params_.shake and not self._shake then
		anim.shake(self.views_.buttonlayer, {
			forever = true
			, endDelay = 3
			, startDelay = 2
			})
		self._shake = true
	end
end

function EffectButton:shakeOff()
	if self._shake then 
		self._shake = nil
		self.views_.buttonlayer:stopAllActions()
	end
end

function EffectButton:refresh()
	if self.params_.checkRefresh then 
		if self.params_.checkRefresh() then 
			self:flashOn()
			self:lightOn()
			self:markOn()
			self:shakeOn()
		else
			self:flashOff()
			self:lightOff()
			self:markOff()
			self:shakeOff()
		end
	end
end

return EffectButton

--]==]