--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

滚动视图[幸运转盘]类，定义滚动视图[幸运转盘]相关操作方法及逻辑实现。

-   存储滚动视图[幸运转盘]相关信息。
-   示例：
    -- 引用类
    local FortuneWheelRewardPanel = require("cardui.FortuneWheelRewardPanel")
    -- 创建标签
    local panel = FortuneWheelRewardPanel.new{contentSize = parent:getContentSize(),}
    parent:addChild(panel)
    util.alignNode(panel, parent:getContentSize(), display.CENTER)
]]

local M = {}
M.TAG = "[cardui.FortuneWheelRewardPanel]"



-- [包含文件]
-- ..



-- [公共参数]
-- 常量


-- 操作变量
-- 验证
local TDAssertUtils             = require(PH_MDL_UTIL .. "TDAssertUtils")
local ChatUtil                  = require("ccb.m.chat.ChatUtil")

-- 枚举
-- 枚举[类型]
-- ..

-- 枚举[模式]
M.MODE_TIMES                    = 1;M.TIMES_MODE                    = 1;










-- [构造方法]
--[[ 

## 首页显示聊天界面

**Parameters:**

-   params: 数组包含

            [必填项]
            CCSize      contentSize             尺寸

            [选填项]
            CGSize      cellSize                单元格尺寸
            table       labelParams             字体属性
            function    onComplete              缓存中都播放完成事件
            
--]]
function M.new(params)
	--#必填项#
	local _contentSize                 = params.contentSize
	--#选填项#
 	local _cellSize                    = params.cellSize or CCSize(_contentSize.width, 25)
	--#内部参数#
	local _clipNode                    = display.newClippingRegionNode(CCRect(0, 0, _contentSize.width, _contentSize.height))
	local _numMessagesLast             = 0                             -- 旧行数
 	local _offsetLast                  = ccp(0, 0)                     -- 底行位置
    local _offsetY                     = -25                           -- 单行高度
 	local _gridTable                                                   -- Table实体




    -- 重置table数据
	function  _clipNode.setEntities( params )
    	local messagesFilter = {}
    	local messages = params.messages
    	
        -- 过滤消息类型
        for i = 1, #messages do
            messagesFilter[#messagesFilter + 1] = messages[i]
        end

        -- 1、缓存之前偏移位置
        -- 2、设置消息集
        if messagesFilter then
            if not (_gridTable and _gridTable.getContentOffset) then
                return
            end

            -- FIXME: 
            try(function() 
                _offsetLast = _gridTable:getContentOffset()
                -- 刷新Table
                _gridTable:setItems(messagesFilter)
            end)
        end

        -- 超过一页滚动到最下面
        if #messagesFilter > 0 then
            -- 小于一行的位移自动移动
            if _offsetLast.y > _offsetY then
                _gridTable:setContentOffset(_gridTable:maxContainerOffset())
                _offsetLast = _gridTable:getContentOffset()
            else
                -- 新消息数
                local numMessagesNew = #messagesFilter - _numMessagesLast
                -- 设置到缓存位置
                _gridTable:setContentOffset(ccpAdd(_offsetLast, ccp(0, (_offsetY * numMessagesNew))))
            end
        end

        _numMessagesLast = #messagesFilter
    end





    -- [实体构建]
    -- 加载结点[背景]
    local function loadNodeBackground()
        local sprite = widget.newScale9({ file = "hook_ac.png", })
        sprite:setContentSize(_clipNode:getContentSize())
        _clipNode:addChild(sprite, 0)
        util.alignNode(sprite, _clipNode:getContentSize(), display.CENTER)
    end

    -- 加载结点[滚动表格]
    local function loadScrollTable()
        -- [事件定义]
        -- 创建cell
        local function createCell(grid, index, item)
            local cell = widget.newLayer()
            cell:setContentSize(grid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)
            -- x坐标
            local curX = 0
            local c_item = item.cellItem
            local ctype = item.ctype
            -- local key = 1
            local color = COLOR3.WHITE
            local color4 = ccc4(255,255,255,200) 
            if ctype and ctype == ChatUtil.ChatSystem then 
                color = COLOR3.YELLOW 
                color4 = ccc4(255,242,0,200)
            end
                   
            -- 循环添加表格元素
            for i = 1, #c_item do
                local content = c_item[i]
                if content and content.title then
                    if content.flag == 1 then 
                        local ly = widget.newLayer{
                                             x = curX,
                                             y = 0
                                    }
                        local cttf = widget.newTTFLabelWithShadow{
                                text = content.title, 
                                shadow = true,
                                size = 20, 
                                color = color, 
                                offset = 1
                            }
                        ly:setContentSize(cttf:getContentSize())
                        ly:addChild(cttf, 1)
                        cell:addChild(ly, 1)
                        curX = curX + cttf:getContentSize().width

                        local rtext = content.text

                        local cardEntity
                        -- 去除"[]"
                        if rtext and string.find(rtext,"%[") and string.find(rtext,"%]") then 
                            rtext = string.gsub(rtext,"%[","")
                            rtext = string.gsub(rtext,"%]","")
                            cardEntity = CardDB.getByName(rtext)
                        end



                        if content.uid and content.uid ~= ME:getUserid()  then
                            cttf:setColor(COLOR3.GREEN)
                        end
                        if cardEntity then 
                            cttf:setColor(COLOR3.ORANGE)
                        end

                        --[[
                        widget.bindTouchEvent{
                            layer = ly,
                            clickListener = function ( layer )
                                if canuserchick and content.uid and content.uid ~= ME:getUserid() then
                                    canuserchick = false
                                    Log.d(TAG,"~~~~~~~~~~~~~~~~",content.uid) 
                                    NetHelper.showLoading(true,true)
                                    --   创建http请求
                                    NetHelper.createRequest(
                                        
                                       ACTID.MSG_GET_PLAYERDETAIL,                              -- 请求ID
                                       function(properties, params)                             -- 回调函数
                                            _node.toggleSendMsgEnable()
                                           canuserchick = true
                                           NetHelper.hideLoading(true)
                                           local entityFriend = require("ccb.m.friend.EntityFriend")
                                           local user = entityFriend.new(properties)
                                           CCBCenter.loadCCB("MSG_FRIENDPANELLAYER_MODEL", "ccb.c.friend.FriendPanelLayer", {entity = user, key = MSG_ID.MSG_CHATLAYER_MODEL, fnNextView = function() CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH) end })
                                       end,
                                       {                                                        -- 请求信息实体或参数Table
                                            pid = content.uid
                                       },
                                       {
                                            failedCb = function()
                                                canuserchick = true
                                                NetHelper.hideLoading(true)
                                            end
                                        }
                                    )
                                    
                                else
                                    if cardEntity then
                                        _node.toggleSendMsgEnable( )
                                        cardEntity._KEY = MSG_ID.MSG_CHATLAYER_MODEL
                                        CCBCenter.loadCCB("MSG_CARDDETAILLAYER_MODEL", "ccb.c.cardgroup.CardDetailLayer", cardEntity)
                                        Log.d(TAG,"have card ",content.text)
                                    else 
                                        Log.d(TAG,"no card~~~~~~~~~~~~~~~~",content.text)
                                    end
                                end
                            end
                        }
                        ]]
                    else
                        local cttf = widget.newTTFLabelWithShadow{
                                                    text = content.title, 
                                                    shadow = true,
                                                    size = 20, 
                                                    color = color, 
                                                    offset = 1,
                                                    x = curX,
                                                    y = 0
                                                }
                        cell:addChild(cttf, 1)
                        curX = curX + cttf:getContentSize().width
                    end
                end 
            end
            
            return cell
        end

        local function createItemCellListener(grid, index, item)
            Log.d(TAG, "createItemCellListener")
            return createCell(grid,index,item)
        end
        local function modifyItemCellListener(grid, index, item, itemCell)
            Log.d(TAG, "modifyItemCellListener")
        end
        local function tapListener( grid,index,item )
            Log.d(TAG, "tapListener")
        end


        -- [实体定义]
        local grid = widget.newScrollGrid({
            contentSize     = _clipNode:getContentSize(),
            parentNode      = _clipNode,
            direction       = widget.TABLEVIEW_DIRECTION_VERTICAL,
            contentParams = {
                cellSize                = _cellSize,
                padding                 = { 
                    middleH = 0, middleV = 0,
                    left = 0, right = 0, top = 0, bottom = 0,
                },
                column                  = 1,
                createItemCellListener  = createItemCellListener,
                modifyItemCellListener  = modifyItemCellListener,
                tapListener             = tapListener,
                selectedParams          = nil,
            },
        })

        _gridTable = grid
    end

	local function _init()
        -- 加载结点[背景]
        loadNodeBackground()
        -- 加载结点[滚动表格]
    	loadScrollTable()
	end

	_init()
	


	return _clipNode
end



return M