--[[ 

创建碎片

--]]
local FragmentView = class("FragmentView", function ( ... )
	return display.newNode()
end)
-- 行列,可在外部再定义
FragmentView.ROW, FragmentView.COLUMN = 6, 4
-- 显示序列,第一个序列为内部一table,可有多个
FragmentView.TILE_ORDERS = {
	
}
-- 碎片块信息
local _fragments = {
	--==第一行==--
	--24
	{ img = "24", dx = 10, dy = -4 },
	--25
	{ img = "25", dx = 1, dy = -10 },
	--26
	{ img = "26", dy = -4 },
	--27 (24)
	{ img = "24", rotate = 90, dx = -3, dy = -10 },

	--==第二行==--
	--28
	{ img = "26", rotate = -90, dx = 4, dy = -1 },
	--29
	{ img = "29", dx = 1, dy = -1},
	--30
	{ img = "30" },
	--31 (25)
	{ img = "25", rotate = 90, dx = -9, dy = -1},

	--==第三行==--
	--32 (25)
	{ img = "25", rotate = -90, dx = 10, dy = -1 },
	--33 (30)
	{ img = "30", dx = 1 },
	--34 (29)
	{ img = "29" },
	--35 (26)
	{ img = "26", rotate = 90, dx = -2 },

	--==第四行==-- 同第二行
	--36 (26)
	{ img = "26", rotate = -90, dx = 4 },
	--37 (29)
	{ img = "29", dx = 1 },
	--38 (30)
	{ img = "30" },
	--39 (25)
	{ img = "25", rotate = 90, dx = -9 },

	--==第五行==-- 同第三行
	--40 (25)
	{ img = "25", rotate = -90, dx = 10 },
	--41 (30)
	{ img = "30", dx = 1 },
	--42 (29)
	{ img = "29" },
	--43 (26)
	{ img = "26", rotate = 90, dx = -2},

	--==第六行==--
	--44 (24)
	{ img = "24", rotate = -90, dx = 4, dy = 9 },
	--45 (26)
	{ img = "26", rotate = 180, dx = 2, dy = 3 },
	--46 (25)
	{ img = "25", rotate = 180, dx = 1, dy = 10 },
	--47 (24)
	{ img = "24", rotate = 180, dx = -9, dy = 3 },
}
--[[ 

构造函数

**Parameters:**

-   percentage:        float/int 	     百分比,可以是[0 .. 1]的float,也可以是int,默认为1
- 	orderIndex: 	   int 				 使用的序列下标,默认为1.当为0时,使用随机序列,其它的若在FragmentView.TILE_ORDERS找到则使用,否则按[1..24]显示

--]]
function FragmentView:ctor( percentage, orderIndex )
	self.orderIndex = orderIndex or 1
	local bgSprite = widget.newSprite{ 
			file = "fragments/fragment_bg.png",
			ignoreAnchor = true,
		}
	self:setContentSize(bgSprite:getContentSize())
	self:addChild(bgSprite)
	self:setPercentage(percentage)
end
--[[ 

创建碎片块

**Parameters:**

-   i:        int     要创建的碎片块下标
- 	size: 	  CCSize  本view的尺寸

**Returns:**

-   return 碎片块精灵

--]]
local function createFragment( i, size )
	local fileInfo = _fragments[i]
	local sprite = display.newSprite( "fragments/fragment_part_" .. fileInfo.img .. ".png")
	if fileInfo.rotate then 
		sprite:setRotation(fileInfo.rotate)
	end
	local row = math.floor((i - 1) / FragmentView.COLUMN) + 1
	local column = i - 1 - math.floor((i - 1) / FragmentView.COLUMN) * FragmentView.COLUMN + 1
	local x = size.width / FragmentView.COLUMN * column - size.width  / FragmentView.COLUMN / 2 + (fileInfo.dx or 0) * SCALE_FOR_SPECIAL_LABEL
	local y = size.height - size.height / FragmentView.ROW * row + size.height / FragmentView.ROW / 2 + (fileInfo.dy or 0) * SCALE_FOR_SPECIAL_LABEL
	sprite:setPosition(x, y)
	return sprite
end
--[[ 

设置百分比,并刷新显示

**Parameters:**

-   percentage:        float/int 	     百分比,可以是[0 .. 1]的float,也可以是int,默认为1

--]]
function FragmentView:setPercentage( percentage )
	self.percentage = percentage or 1
	if self.percentage > 1 then self.percentage = self.percentage % 100 / 100 end
	self:refresh()
end
--[[ 

刷新界面

--]]
function FragmentView:refresh()
	local fragmentIndexs = FragmentView.TILE_ORDERS[self.orderIndex]
	if not fragmentIndexs then
		local tempIndexs = {}
		for i = 1, FragmentView.ROW * FragmentView.COLUMN do
			table.insert(tempIndexs, i)
		end
		if self.orderIndex == 0 and self.percentage < 1 then 
			-- random
			fragmentIndexs = {}
			while #tempIndexs > 0 do
				local temp = math.random(#tempIndexs)
				table.insert(fragmentIndexs, tempIndexs[temp])
				table.remove(tempIndexs, temp)
			end
		else 
			fragmentIndexs = tempIndexs
		end
	end
	local count = FragmentView.ROW * FragmentView.COLUMN -  math.floor(#fragmentIndexs * self.percentage)
	local size = self:getContentSize()
	for i = 1, count do
		local fragment = createFragment(fragmentIndexs[i], size)
		self:addChild(fragment)
	end
end

return FragmentView
