
local GoodsMaster = {}
GoodsMaster.TAG = "[GoodsMaster]"
-- type
-- 素材
GoodsMaster.MATERIAL      = 1
GoodsMaster.CUREQUIP      = 2
GoodsMaster.BAGEQUIP      = 3

local LinearNum = require("cardui.LinearNum")


-- 获取星级图片
local function getStarLevelImage( level )
    local layer = widget.newLinearLayer{padding = -3 ,}
    for i = 1, level do
        local sprite = widget.newSprite{file = IMAGE_goods_STAR,}
        layer:addChild(sprite)
    end
    return layer
end

-- create GoodsMaster

--[[**Parameters:**

            [必填项]
            goodsEntity      params              物品entity

            [选填项]
            int             params.type                绑定类型(默认不绑定)
                                                            GoodsMaster.MATERIAL
--]]

function GoodsMaster.new(params)
--[[ ## local params --]]
    local _goods = display.newLayer()
--#必填项#
    local _goodsParams = params
--#选填项#
    -- local _listener = params.listener
--#内部参数#   

--[[ ## local fuction --]]
    local function init( )
        util.checkParams(GoodsMaster.TAG, "new()", {
            { istable(_goodsParams), "is not a table" },
        })
        if _goodsParams.type == GoodsMaster.MATERIAL then
            GoodsMaster.bindMaterial(_goods)
        elseif _goodsParams.type == GoodsMaster.CUREQUIP then
            GoodsMaster.bindEquipUse(_goods)
        elseif _goodsParams.type == GoodsMaster.BAGEQUIP then
            GoodsMaster.bindEquipUse(_goods)
        end
    end

--[[ ## public fuction --]]
    function _goods.getParams( )
        return _goodsParams
    end

    function _goods.setParams( goodsEntity )
        _goodsParams = goodsEntity
    end
    
--[[ ## running --]]
    init()
    return _goods
end

--[[ 

绑定成小卡片, 可点击

--]]

function GoodsMaster.bindMaterial( goods )
--[[ ## local params --]]
    local _goods             = goods
    local _params            = _goods.getParams()
    local _no                = _params:getNo()
    local _showDetail        = _params.showDetail
    local _showFrame         = _params.showFrame
    local _handler           = _params.handler               -- 点击事件处理
    local _disenableTouch    = _params.disenableTouch        -- 点击事件关闭
--#必填项#

--#选填项#

--#内部参数#   

--[[ ## local fuction --]]
    local function onClick( goods )
        if _handler then
            _handler(goods)
        elseif _showDetail then
            if _no > 20000 then
                CCBCenter.loadCCB("MSG_EQUIPALERTLAYER_MODEL", "ccb.c.goods.EquipAlertLayer", _params)
            else
                CCBCenter.loadCCB("MSG_GOODSALERTLAYER_MODEL", "ccb.c.goods.GoodsAlertLayer", _params)
            end
            
        end
    end
    local function init( )
        -- 清空容器
        _goods:removeAllChildrenWithCleanup(true)
        -- 设置Size
        _goods:setContentSize(CCSize(87, 87))
        -- 取值
        if _no then
            -- 背景
            if _showFrame then
                local bgSprite = widget.newSprite{ file = _GAME.CONFIG.GOODS.QUALITY[_params:getQuality()].frame}
                bgSprite:setScale(SCALE_FOR_SPECIAL_NODE)
                _goods:addChild(bgSprite)
                util.alignNode(bgSprite, _goods:getContentSize(), display.CENTER)
            end
            -- 生成精灵
            local sprite = widget.newSprite{
                file = _params:getGoodsIcon(),
                ignoreAnchor = true,
            }
            -- 加入容器
            _goods:addChild(sprite)
            
            -- 设置位置
            util.alignNode(sprite, _goods:getContentSize(), display.CENTER)
           
        end
        if not _disenableTouch then
            -- 绑定点击
            widget.bindTouchEvent{
                layer = _goods,
                clickListener = onClick,
                container = _params.container,
            }
        end
        
    end
--[[ ## public fuction --]]
    function _goods:showGoodsDetail( )
        if _params.type == GoodsMaster.CUREQUIP then
            CCBCenter.loadCCB("MSG_EQUIPOFFALERTLAYER_MODEL", "ccb.c.goods.EquipOffAlertLayer", _params)
        else
            CCBCenter.loadCCB("MSG_EQUIPUSEALERTLAYER_MODEL", "ccb.c.goods.EquipUseAlertLayer", _params)
        end
    end
--[[ ## running --]]
    init()
    return _goods
end

--[[ 

绑定成装备带按钮, 可点击

--]]

function GoodsMaster.bindEquipUse( goods )
--[[ ## local params --]]
    local _goods          = goods
    local _params         = _goods.getParams()
    local _no             = _params:getNo()
    local _showDetail     = _params.showDetail
    local _showFrame      = _params.showFrame
    local _handler        = _params.handler               -- 点击事件处理
    local _onConfirm      = _params.onConfirm             -- 确认按钮
    local _disenableTouch    = _params.disenableTouch        -- 点击事件关闭
--#必填项#

--#选填项#

--#内部参数#   

--[[ ## local fuction --]]
    local function onClick( goods )
        if _handler then
            _handler(goods)
        elseif _showDetail then
            if _params.type == GoodsMaster.CUREQUIP then
                CCBCenter.loadCCB("MSG_EQUIPOFFALERTLAYER_MODEL", "ccb.c.goods.EquipOffAlertLayer", _params)
            else
                CCBCenter.loadCCB("MSG_EQUIPUSEALERTLAYER_MODEL", "ccb.c.goods.EquipUseAlertLayer", _params)
            end
        end
    end
    local function init( )
        -- 清空容器
        _goods:removeAllChildrenWithCleanup(true)
        -- 设置Size
        _goods:setContentSize(CCSize(87, 87))
        _params.shareButton = true
        -- 取值
        if _no then
            -- 背景
            if _showFrame then
                local bgSprite = widget.newSprite{ file = _GAME.CONFIG.GOODS.QUALITY[_params:getQuality()].frame}
                bgSprite:setScale(SCALE_FOR_SPECIAL_NODE)
                _goods:addChild(bgSprite)
                util.alignNode(bgSprite, _goods:getContentSize(), display.CENTER)
            end
            -- 生成精灵
            local sprite = widget.newSprite{
                file = _params:getGoodsIcon(),
                ignoreAnchor = true,
            }
            -- 加入容器
            _goods:addChild(sprite)
            
            -- 设置位置
            util.alignNode(sprite, _goods:getContentSize(), display.CENTER)
        end
        if not _disenableTouch then
            -- 绑定点击
            widget.bindTouchEvent{
                layer = _goods,
                clickListener = onClick,
                container = _params.container,
            }
        end
    end
--[[ ## public fuction --]]
    function _goods:showDetail( )
        if _params.type == GoodsMaster.CUREQUIP then
            CCBCenter.loadCCB("MSG_EQUIPOFFALERTLAYER_MODEL", "ccb.c.goods.EquipOffAlertLayer", _params)
        else
            CCBCenter.loadCCB("MSG_EQUIPUSEALERTLAYER_MODEL", "ccb.c.goods.EquipUseAlertLayer", _params)
        end
    end
--[[ ## running --]]
    init()
    return _goods
end

return GoodsMaster
