--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

标签[跑马灯]类，定义标签[跑马灯]相关操作方法及逻辑实现。

-   存储标签[跑马灯]相关信息。
-   示例：
    -- 引用类
    local LabelMarquee = require("cardui.LabelMarquee")
    -- 创建标签
    local label = LabelMarquee.new({ 
        classType  = LabelMarquee.CLASSTYPE_TTF_SHADOW      -- 类型
        text       = "300",                                 -- 标签文字
        size       = 50,                                    -- 字体大小
        color      = ccc3(255, 255, 255),                   -- 颜色
    })
    self:addChild(label, 10000)
    label:setPosition(ccp(display.cx, display.cy))

    -- 动画显示
    -- MODE: LabelMarquee.MODE_TIMES
    label:animateText({
         valueTo       = 1,                                 -- 到达值
         times         = 30,                                -- 动画次数
         duration      = 10,                                -- 动画时长
    })
    -- MODE: LabelMarquee.MODE_INCREMENTAL
    -- label:animateText({
    --      valueTo       = 1,                              -- 到达值
    --      duration      = 10,                             -- 动画时长
    --      mode          = LabelMarquee.MODE_INCREMENTAL,  -- 动画模式
    --      increase      = -10,                            -- 增长值
    -- })
]]

local M = {}



-- [包含文件]
-- ..



-- [公共参数]
-- 常量
-- 默认间隔
M.DFT_INTERVAL                  = 1
-- 默认增量
M.DFT_INCREASE                  = 0
-- 默认过渡时间
M.DFT_DURATION                  = 2.0
-- 默认次数
M.DFT_TIMES                     = 20
-- 默认是否允许次数超过总增量区间
M.DFT_IS_TIMES_OVERFLOW         = false


-- 操作变量
-- 验证
local TDAssertUtils             = require(PH_MDL_UTIL .. "TDAssertUtils")

-- 枚举
-- 枚举[类型]
M.CLASSTYPE_TTF                 = 1;M.TTF_CLASSTYPE                 = 1;
M.CLASSTYPE_TTF_SHADOW          = 2;M.TTF_SHADOW_CLASSTYPE          = 2;
M.CLASSTYPE_TTF_OUTLINE         = 3;M.TTF_OUTLINE_CLASSTYPE         = 3;
M.CLASSTYPE_BMFONT              = 4;M.BMFONT_CLASSTYPE              = 4;
M.CLASS_TYPES = {
    ui.newTTFLabel,
    ui.newTTFLabelWithShadow,
    ui.newTTFLabelWithOutline,
    ui.newBMFontLabel,
}

-- 枚举[模式]
M.MODE_TIMES                    = 1;M.TIMES_MODE                    = 1;
M.MODE_INCREMENTAL              = 2;M.INCREMENTAL_MODE              = 2;










-- [构造方法]
--[[--
    创建对象
]]
function M.new(params)
    -- [属性定义]
    local _classType            -- 类型
    local _mode                 -- 模式
    local _interval             -- 走马灯增量间隔
    local _increase             -- 走马灯增量
    local _times                -- 走马灯次数
    local _duration             -- 走马灯过渡时间
    local _isTimesOverflow      -- 是否允许次数超过总增量区间
    local _cctor                -- 构造方法
    local _node                 -- 结点

    -- [私有方法定义]
    -- 验证合法性
    local function assertParameters(p)
        TDAssertUtils:assertParameters(p)
    end

    -- 参数赋值
    local function assignParameters(p)
        _classType             = p.classType              or M.CLASSTYPE_TTF          -- 类型
        _mode                  = p.mode                   or M.MODE_TIMES             -- 模式
        _interval              = p.interval               or M.DFT_INTERVAL           -- 走马灯增量间隔
        _increase              = p.increase               or M.DFT_INCREASE           -- 走马灯增量
        _times                 = p.times                  or M.DFT_TIMES              -- 走马灯次数
        _duration              = p.duration               or M.DFT_DURATION           -- 走马灯过渡时间
        _isTimesOverflow       = p.isTimesOverflow        or M.DFT_IS_TIMES_OVERFLOW  -- 是否允许次数超过总增量区间CLASSTYPE
        _cctor                 = M.CLASS_TYPES[_mode]                                 -- 构造方法
    end




    





    -- [具体操作]
    -- [验证参数合法性]
    assertParameters(params)
    -- [参数赋值]
    assignParameters(params)

    -- [构建实体]
    if _cctor then 
        -- [实例化]
        _node = _cctor(params)


        -- [公有方法定义]
        -- 获得文本
        function _node:getText()
            local text = nil

            if     _classType == M.CLASSTYPE_TTF            then 
                text = self:getString()
            elseif _classType == M.CLASSTYPE_TTF_SHADOW     then 
                text = self.label:getString()
            elseif _classType == M.CLASSTYPE_TTF_OUTLINE    then 
                text = self.label:getString()
            elseif _classType == M.CLASSTYPE_BMFONT         then 
                text = self:getString()
            end

            return text
        end
        -- 设置文本
        function _node:setText(text)
            if     _classType == M.CLASSTYPE_TTF            then 
                self:setString(text)
            elseif _classType == M.CLASSTYPE_TTF_SHADOW     then 
                self:setString(text)
            elseif _classType == M.CLASSTYPE_TTF_OUTLINE    then 
                self:setString(text)
            elseif _classType == M.CLASSTYPE_BMFONT         then 
                self:setString(text)
            end
        end

        -- 动画文本显示
        function _node:animateText(params)
            -- [验证参数合法性]
            assertParameters(params)
            -- [参数赋值]
            assignParameters(params)

            -- [模式选择]
            if _mode == M.MODE_TIMES then 
                self:animateTextModeTimes(params)
            else
                self:animateTextModeIncremental(params)
            end
        end

        -- 动画文本显示[模式：次数]
        function _node:animateTextModeTimes(params)
            -- [参数赋值]
            local valueFrom     = params.valueFrom      or tonumber(self:getText())
            local valueTo       = params.valueTo
            local difference    = (valueTo - valueFrom)
            local times         = _times
            local duration      = _duration

            -- 处理增量
            local increase      = math.floor(difference / times)
            if not _isTimesOverflow then 
                if math.abs(difference) <= times then 
                    increase = (difference > 0) and 1 or -1
                end
            end
            -- 处理间隔时间
            local interval      = 1.0 * math.abs(duration) / times


            -- 调度输出
            -- print("valueFrom",  valueFrom)
            -- print("valueTo",    valueTo)
            -- print("difference", difference)
            -- print("duration",   duration)
            -- print("increase",   increase)
            -- print("times",      times)
            -- print("interval",   interval)



            -- [构建动画]
            local actions = {}
            for i = 1, times do
                actions[#actions + 1] = CCCallFunc:create(function() self:setText( (i == times) and valueTo or (valueFrom + i * increase) ) end)
                actions[#actions + 1] = CCDelayTime:create(interval)
            end
            -- 运行动作
            self:runAction(transition.sequence(actions))
        end

        -- 动画文本显示[模式：增量]
        function _node:animateTextModeIncremental(params)
            -- [参数验证]
            assert(_increase ~= 0, "params.incrase must not nil or 0!")

            -- [参数赋值]
            local valueFrom     = params.valueFrom      or tonumber(self:getText())
            local valueTo       = params.valueTo
            local difference    = (valueTo - valueFrom)
            local duration      = _duration
            local increase      = _increase
            local times         = 0

            -- 处理次数
            local integer, decimal = math.modf(difference / increase)
            if decimal ~= 0 then 
                times = integer + 1
            else 
                times = integer
            end
            -- 处理间隔时间
            local interval      = 1.0 * math.abs(duration) / times


            -- 调度输出
            -- print("valueFrom",  valueFrom)
            -- print("valueTo",    valueTo)
            -- print("difference", difference)
            -- print("duration",   duration)
            -- print("increase",   increase)
            -- print("times",      times)
            -- print("interval",   interval)



            -- [构建动画]
            local actions = {}
            for i = 1, times do
                actions[#actions + 1] = CCCallFunc:create(function() self:setText( (i == times) and valueTo or (valueFrom + i * increase) ) end)
                actions[#actions + 1] = CCDelayTime:create(interval)
            end
            -- 运行动作
            self:runAction(transition.sequence(actions))
        end
    end





    return _node
end



return M