--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

线性排列星星类，定义线性排列星星相关操作方法及逻辑实现。

-   提供星星排列的公共方法。
-   示例：
    local sprite = require("cardui.LinearStar").new({ fileName = "map/popup/lp_icon_star.png", count = 4, unitScale = 1, })
    _node:addChild(sprite)
    sprite:setPosition(ccp(300, 300))
]]

local M = class("LinearStar", TDNode)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- [默认值]数量
M.DFT_COUNT                     = 1
-- [默认值]外边距
M.DFT_MARGIN                    = ccp(10, 0)
-- [默认值]单元缩放比例
M.DFT_UNIT_SCALE                = 1

-- 枚举
M.STRATEGY_A_ENLARGED_DISPLAY_ONE_BY_ONE    = 1
M.STRATEGY_A_FADE_IN                        = 3
M.STRATEGY_A_FADE_OUT                       = 4

-- 操作变量
-- ..










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    self:assertParameters(params)



    -- [参数赋值]
    self._fileName          = params.fileName                               -- 文件名[图片]
    self._count             = params.count          or M.DFT_COUNT          -- 数量
    self._margin            = params.margin         or M.DFT_MARGIN         -- 边距
    self._unitScale         = params.unitScale      or M.DFT_UNIT_SCALE     -- 边距
    self._x                 = params._x             or 0                    -- 起始坐标x
    self._y                 = params._y             or 0                    -- 起始坐标y


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载星星
    self:loadStars()

    -- 加载实体渲染其他操作
    self:loadRenderOtherwise()
end

-- 加载星星
function M:loadStars()
    local sprite
    
    local positionPrev = ccp(0, 0)
    local size         = self:getSizeUnit()

    for i = 1, self._count do
        sprite = display.newSprite(self._fileName)
        self:addChild(sprite)

        sprite:setScale(self._unitScale)
        sprite:setPosition(ccpAdd(positionPrev, ccp(size.width * self._unitScale + self._margin.x, self._margin.y)))
        positionPrev = ccp(sprite:getPositionX(), 0)
    end
end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()
    self:setPosition(self:getPosition())
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()

end


---------


-- [动画]
function M:animate(strategy)
    -- 策略一：逐个放大显示

    -- 策略二：逐个放大显示    
end


---------


-- [实例方法][外部调用]
-- 获得尺寸[星星]
function M:getSizeUnit()
    local sprite = display.newSprite(self._fileName)
    return sprite:getContentSize()
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数合法性(参数集)
    -- TDAssertUtils.assertParameters(params)
    -- 验证参数合法性(文件名)
    -- TDAssertUtils.assertTypeString(params.fileName)
    -- 验证参数合法性(数量)
    -- TDAssertUtils.assert(params.fileName)
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得文件名[图片]
function M:getFileName()
    return self._fileName
end
-- 设置文件名[图片]
function M:setFileName(fileName)
    self._fileName = fileName
end

-- 获得数量
function M:getCount()
    return self._count
end
-- 设置数量
function M:setFileName(count)
    self._count = count
end

-- 获得边距
function M:getMargin()
    return self._margin
end
-- 设置边距
function M:setMargin(margin)
    self._margin = margin
end





return M