local MapView = {}


---------------
-- 包含文件
---------------
local LevelDao 						= require("db.leveldb")
local TDAnimationUtils 				= require(PH_MDL_UTIL .. "TDAnimationUtils")
local CloudView 					= require("cardui.CloudView")
local GoodsMaster 					= require("cardui.GoodsMaster")
local CardMaster 					= require("cardui.CardMaster")
local GoodsEntity 					= require("model.goods.GoodsEntity")
local CardEntity 					= require("model.card.CardEntity")

---------------
-- 公共常量定义
---------------
MapView.NUM_LEVELS 					= LevelDao.NUM_MAP_LEVELS
MapView.NUM_MAP_SUBLEVELS 			= LevelDao.NUM_MAP_SUBLEVELS
MapView.NUM_TILES 					= 18
MapView.DEBUG 	 					= false


---------------
-- 关卡难度定义
---------------
MapView.GATE_FLAG_TYPE_NONE 		= 0
MapView.GATE_FLAG_TYPE_EASY 		= 1
MapView.GATE_FLAG_TYPE_NORMAL	 	= 2
MapView.GATE_FLAG_TYPE_HARD 	 	= 3


---------------
-- 结点标识定义
---------------
local NODETAG_LABEL_FLAG_NUM 		= "LABEL_FLAG_NUM"
local NODETAG_LABEL_STAR_NUM 		= "LABEL_STAR_NUM"
local NODETAG_AREA_START 			= 3000


---------------
-- 结点颜色定义
---------------
local COLOR_LIGHT 					= ccc3(255, 255, 255)
local COLOR_DARK 					= ccc3(80, 80, 80)


---------------
-- 层深度定义
---------------
local Z_TILE_BACKGROUND 		= 10
local Z_TILE_MIDDLE 			= 20
local Z_TILE_FOREGROUND 		= 30
local Z_WAVE 					= 100
local Z_AREA 					= 200
local Z_AREA_NAME 				= 250
local Z_STARS	 				= 300
local Z_ANIM_ITEM 				= 400
local Z_COLUD 					= 500
local REWARD_TYPE_CARD  		= 0
local REWARD_TYPE_CHIP  		= 1
local REWARD_TYPE_GOODS 		= 2
local REWARD_TYPE_SP			= 3
local REWARD_TYPE_GOLD  		= 4
local REWARD_TYPE_FP    		= 5
local REWARD_TYPE_DIAMOND  		= 6
local REWARD_TYPE_FREEBUY  		= 7
local REWARD_TYPE_STAR  		= 8
local REWARD_TYPE_EXP   		= 9

---------------
-- 关卡难度定义
---------------
local map_info = {--404
	-- 大地图上的每个区域的可点击信息
	tiles = {
		{ -- 1
			index = 1,
			rect = CCRectMake(225, 105, 330, 101),
		},
		{ -- 2
			index = 1,
			rect = CCRectMake(174, 204, 362, 141),
		},
		{ -- 3
			index = 1,
			rect = CCRectMake(142, 341, 349, 82),
		},
		{ -- 4
			index = 1,
			rect = CCRectMake(168, 424, 319, 70),
		},
		{ -- 5
			index = 2,
			rect = CCRectMake(56, 507, 287, 312),
		},
		{ -- 6
			index = 2,
			rect = CCRectMake(338, 497, 90, 122),
		},
		{ -- 7
			index = 3,
			rect = CCRectMake(534, 229, 303, 115),
		},
		{ -- 8
			index = 3,
			rect = CCRectMake(491, 344, 352, 94),
		},
		{ -- 9
			index = 3,
			rect = CCRectMake(520, 435, 414, 93),
		},
		{ -- 10
			index = 3,
			rect = CCRectMake(574, 527, 365, 57),
		},
		{ -- 11
			index = 3,
			rect = CCRectMake(841, 379, 89, 60),
		},
		{ -- 12
			index = 4,
			rect = CCRectMake(388, 695, 96, 103),
		},
		{ -- 13
			index = 4,
			rect = CCRectMake(495, 699, 400, 133),
		},
		{ -- 14
			index = 4,
			rect = CCRectMake(500, 609, 435, 90),
		},
		{ -- 15
			index = 4,
			rect = CCRectMake(638, 584, 292, 26),
		},
		{ -- 16
			index = 5,
			rect = CCRectMake(931, 380, 383, 52),
		},
		{ -- 17
			index = 5,
			rect = CCRectMake(841, 256, 473, 124),
		},
		{ -- 18
			index = 5,
			rect = CCRectMake(835, 174, 447, 80),
		},
		{ -- 19
			index = 5,
			rect = CCRectMake(895, 61, 281, 118),
		},
		{ -- 20
			index = 6,
			rect = CCRectMake(933, 428, 378, 277),
		},
		{ -- 21
			index = 6,
			rect = CCRectMake(1313, 558, 100, 145),
		},
		{ -- 22
			index = 7,
			rect = CCRectMake(912, 704, 463, 76),
		},
		{ -- 23
			index = 7,
			rect = CCRectMake(912, 780, 462, 70),
		},
		{ -- 24
			index = 7,
			rect = CCRectMake(987, 850, 380, 59),
		},
		{ -- 25
			index = 7,
			rect = CCRectMake(1047, 909, 267, 41),
		},
		{ -- 26
			index = 8,
			rect = CCRectMake(1520, 871, 305, 58),
		},
		{ -- 27
			index = 8,
			rect = CCRectMake(1429, 831, 395, 47),
		},
		{ -- 28
			index = 8,
			rect = CCRectMake(1374, 704, 481, 128),
		},
		{ -- 29
			index = 8,
			rect = CCRectMake(1414, 579, 436, 135),
		},
		{ -- 30
			index = 9,
			rect = CCRectMake(1336, 502, 291, 58),
		},
		{ -- 31
			index = 9,
			rect = CCRectMake(1318, 462, 322, 48),
		},
		{ -- 32
			index = 9,
			rect = CCRectMake(1313, 363, 428, 110),
		},
		{ -- 33
			index = 9,
			rect = CCRectMake(1313, 310, 260, 54),
		},
		{ -- 34
			index = 9,
			rect = CCRectMake(1343, 280, 120, 30),
		},
		{ -- 35
			index = 10,
			rect = CCRectMake(1570, 67, 241, 265),
		},
		{ -- 36
			index = 10,
			rect = CCRectMake(1463, 55, 107, 253),
		},
		{ -- 37
			index = 10,
			rect = CCRectMake(1365, 58, 100, 221),
		},
		{ -- 38
			index = 11,
			rect = CCRectMake(1853, 179, 528, 192),
		},
		{ -- 39
			index = 11,
			rect = CCRectMake(1973, 371, 386, 123),
		},
		{ -- 40
			index = 12,
			rect = CCRectMake(1873, 630, 479, 111),
		},
		{ -- 41
			index = 12,
			rect = CCRectMake(1928, 550, 393, 81),
		},
		{ -- 42
			index = 12,
			rect = CCRectMake(2073, 741, 95, 124),
		},
		{ -- 43
			index = 12,
			rect = CCRectMake(1967, 741, 106, 39),
		},
		{ -- 44
			index = 12,
			rect = CCRectMake(2168, 741, 135, 47),
		},
	}, -- tiles
	-- 小地图信息:map表示所用小地图图片;center_point为小地图居中点;points为每个旗的信息
	areas = 	{
		[1] = 	{
			map = "map/submaps/submap_01.jpg",
			area_point 	 = ccp(109, 70),
			center_point = ccp(365, 322), 				-- x2: ccp(504, 382)
			cloud_rect = CCRectMake(168, 134, 362, 375),
			points = 	{
				[1] = { index = 1, x = 800, y = 430, },
				[2] = { index = 2, x = 540, y = 520, },
				[3] = { index = 3, x = 323, y = 455, },
				[4] = { index = 4, x = 351, y = 346, },
				[5] = { index = 5, x = 292, y = 210, },
				[6] = { index = 6, x = 440, y = 218, },
				[7] = { index = 7, x = 537, y = 289, },
				[8] = { index = 8, x = 699, y = 277, },
				[9] = { index = 9, x = 838, y = 253, },
				[10] = { index = 10, x = 778, y = 161, },
			}, -- points 1
		},
		[2] = 	{
			map = "map/submaps/submap_02.jpg",
			area_point 	 = ccp(0,470),
			center_point = ccp(200, 684), 				-- ccp(714, 1529)
			cloud_rect = CCRectMake(71, 528, 336, 305),
			points = 	{
				[1] = { index = 1, x = 347, y = 59, },
				[2] = { index = 2, x = 167, y = 163, },
				[3] = { index = 3, x = 202, y = 318, },
				[4] = { index = 4, x = 333, y = 384, },
				[5] = { index = 5, x = 495, y = 399, },
				[6] = { index = 6, x = 513, y = 252, },
				[7] = { index = 7, x = 641, y = 235, },
				[8] = { index = 8, x = 824, y = 260, },
				[9] = { index = 9, x = 765, y = 106, },
				[10] = { index = 10, x = 702, y = 14, },
			}, -- points 2
		},
		[3] = 	{
			map = "map/submaps/submap_03.jpg",
			area_point 	 = ccp(500, 68.5),
			center_point = ccp(699, 394),  				-- ccp(1298, 1367)
			cloud_rect = CCRectMake(517, 297, 389, 292),
			points = 	{
				[1] = { index = 1, x = 131, y = 440, },
				[2] = { index = 2, x = 151, y = 337, },
				[3] = { index = 3, x = 103, y = 215, },
				[4] = { index = 4, x = 235, y = 115, },
				[5] = { index = 5, x = 446, y = 89, },
				[6] = { index = 6, x = 517, y = 202, },
				[7] = { index = 7, x = 708, y = 183, },
				[8] = { index = 8, x = 671, y = 325, },
				[9] = { index = 9, x = 517, y = 409, },
				[10] = { index = 10, x = 700, y = 491, },
			}, -- points 4
		},
		[4] = 	{
			map = "map/submaps/submap_04.jpg",
			area_point 	 = ccp(500, 559),
			center_point = ccp(685, 716),  				-- ccp(1190, 1917)
			cloud_rect = CCRectMake(489, 601, 404, 254),
			points = 	{
				[1] = { index = 1, x = 350, y = 64, },
				[2] = { index = 2, x = 279, y = 218, },
				[3] = { index = 3, x = 149, y = 324, },
				[4] = { index = 4, x = 323, y = 434, },
				[5] = { index = 5, x = 446, y = 301, },
				[6] = { index = 6, x = 581, y = 408, },
				[7] = { index = 7, x = 746, y = 416, },
				[8] = { index = 8, x = 804, y = 302, },
				[9] = { index = 9, x = 759, y = 173, },
				[10] = { index = 10, x = 815, y = 61, },
			}, -- points 3
		},
		[5] = 	{
			map = "map/submaps/submap_05.jpg",
			area_point 	 = ccp(904, 36.5),
			center_point = ccp(1040, 216), 				-- ccp(2180, 1032)
			cloud_rect = CCRectMake(847, 83, 422, 319),
			points = 	{
				[1] = { index = 1, x = 156, y = 458, },
				[2] = { index = 2, x = 318, y = 452, },
				[3] = { index = 3, x = 481, y = 478, },
				[4] = { index = 4, x = 547, y = 348, },
				[5] = { index = 5, x = 414, y = 223, },
				[6] = { index = 6, x = 511, y = 92, },
				[7] = { index = 7, x = 684, y = 154, },
				[8] = { index = 8, x = 791, y = 230, },
				[9] = { index = 9, x = 842, y = 342, },
				[10] = { index = 10, x = 751, y = 460, },
			}, -- points 5
		},
		[6] = 	{
			map = "map/submaps/submap_06.jpg",
			area_point 	 = ccp(904, 325),
			center_point = ccp(1083, 548),  			-- ccp(2536, 1938)
			cloud_rect = CCRectMake(894, 416, 463, 270),
			points = 	{
				[1] = { index = 1, x = 345, y = 95, },
				[2] = { index = 2, x = 237, y = 275, },
				[3] = { index = 3, x = 215, y = 395, },
				[4] = { index = 4, x = 345, y = 360, },
				[5] = { index = 5, x = 470, y = 220, },
				[6] = { index = 6, x = 555, y = 315, },
				[7] = { index = 7, x = 618, y = 101, },
				[8] = { index = 8, x = 697, y = 250, },
				[9] = { index = 9, x = 835, y = 326, },
				[10] = { index = 10, x = 800, y = 465, },
			}, -- points 6
		},
		[7] = 	{
			map = "map/submaps/submap_07.jpg",
			area_point 	 = ccp(904, 652.5),
			center_point = ccp(1129, 810),   			-- ccp(3746, 1502)
			cloud_rect = CCRectMake(933, 680, 473, 289),
			points = 	{
				[1] = { index = 1, x = 164, y = 178, },
				[2] = { index = 2, x = 168, y = 306, },
				[3] = { index = 3, x = 247, y = 388, },
				[4] = { index = 4, x = 338, y = 286, },
				[5] = { index = 5, x = 491, y = 141, },
				[6] = { index = 6, x = 628, y = 222, },
				[7] = { index = 7, x = 451, y = 322, },
				[8] = { index = 8, x = 349, y = 463, },
				[9] = { index = 9, x = 487, y = 484, },
				[10] = { index = 10, x = 651, y = 450, },
			}, -- points 7
		},
		[8] = 	{
			map = "map/submaps/submap_08.jpg",
			area_point 	 = ccp(1345, 572),
			center_point = ccp(1593, 786),    			-- ccp(3878, 2148)
			cloud_rect = CCRectMake(1400, 615, 475, 328),
			points = 	{
				[1] = { index = 1, x = 314, y = 227, },
				[2] = { index = 2, x = 316, y = 127, },
				[3] = { index = 3, x = 391, y = 65, },
				[4] = { index = 4, x = 718, y = 73, },
				[5] = { index = 5, x = 811, y = 228, },
				[6] = { index = 6, x = 705, y = 363, },
				[7] = { index = 7, x = 613, y = 345, },
				[8] = { index = 8, x = 478, y = 387, },
				[9] = { index = 9, x = 280, y = 401, },
				[10] = { index = 10, x = 126, y = 322, },
			}, -- points 8
		},
		[9] = 	{
			map = "map/submaps/submap_09.jpg",
			area_point 	 = ccp(1345, 309),
			center_point = ccp(1466, 441),     			-- ccp(3383, 722)
			cloud_rect = CCRectMake(1257, 290, 496, 315),
			points = 	{
				[1] = { index = 1, x = 200, y = 380, },
				[2] = { index = 2, x = 120, y = 265, },
				[3] = { index = 3, x = 205, y = 160, },
				[4] = { index = 4, x = 340, y = 215, },
				[5] = { index = 5, x = 489, y = 155, },
				[6] = { index = 6, x = 495, y = 265, },
				[7] = { index = 7, x = 509, y = 415, },
				[8] = { index = 8, x = 724, y = 348, },
				[9] = { index = 9, x = 830, y = 275, },
				[10] = { index = 10, x = 830, y = 100, },
			}, -- points 9
		},
		[10] = 	{
			map = "map/submaps/submap_10.jpg",
			area_point 	 = ccp(1345, 4.5),
			center_point = ccp(1630, 210),      		-- ccp(4162, 374)
			cloud_rect = CCRectMake(1368, 55, 486, 286),
			points = 	{
				[1] = { index = 1, x = 99, y = 347, },
				[2] = { index = 2, x = 191, y = 237, },
				[3] = { index = 3, x = 315, y = 197, },
				[4] = { index = 4, x = 387, y = 91, },
				[5] = { index = 5, x = 500, y = 55, },
				[6] = { index = 6, x = 569, y = 184, },
				[7] = { index = 7, x = 698, y = 242, },
				[8] = { index = 8, x = 543, y = 300, },
				[9] = { index = 9, x = 456, y = 408, },
				[10] = { index = 10, x = 546, y = 498, },
			}, -- points 10
		},
		[11] = 	{
			map = "map/submaps/submap_11.jpg",
			area_point 	 = ccp(1830, 135),
			center_point = ccp(2147, 315),      		-- ccp(4162, 374)
			cloud_rect = CCRectMake(1850, 165, 480, 286),
			points = 	{
				[1] = { index = 1, x = 105, y = 85, },
				[2] = { index = 2, x = 151, y = 184, },
				[3] = { index = 3, x = 142, y = 352, },
				[4] = { index = 4, x = 244, y = 463, },
				[5] = { index = 5, x = 308, y = 372, },
				[6] = { index = 6, x = 361, y = 460, },
				[7] = { index = 7, x = 665, y = 460, },
				[8] = { index = 8, x = 732, y = 407, },
				[9] = { index = 9, x = 778, y = 318, },
				[10] = { index = 10, x = 850, y = 264, },
				[11] = { index = 11, x = 888, y = 121, },
				[12] = { index = 12, x = 585, y = 127, },
				[13] = { index = 13, x = 682, y = 278, },
				[14] = { index = 14, x = 540, y = 236, },
				[15] = { index = 15, x = 462, y = 311, },
			}, -- points 11
		},
		[12] = 	{
			map = "map/submaps/submap_12.jpg",
			area_point 	 = ccp(1850, 530),
			center_point = ccp(2147, 664),      		-- ccp(4162, 374)
			cloud_rect = CCRectMake(1850, 564, 494, 286),
			points = 	{
				[1] = { index = 1, x = 129, y = 31, },
				[2] = { index = 2, x = 345, y = 51, },
				[3] = { index = 3, x = 506, y = 22, },
				[4] = { index = 4, x = 717, y = 49, },
				[5] = { index = 5, x = 806, y = 124, },
				[6] = { index = 6, x = 755, y = 249, },
				[7] = { index = 7, x = 636, y = 145, },
				[8] = { index = 8, x = 476, y = 240, },
				[9] = { index = 9, x = 342, y = 225, },
				[10] = { index = 10, x = 217, y = 211, },
				[11] = { index = 11, x = 223, y = 302, },
				[12] = { index = 12, x = 306, y = 364, },
				[13] = { index = 13, x = 642, y = 368, },
				[14] = { index = 14, x = 545, y = 450, },
				[15] = { index = 15, x = 617, y = 512, },
			}, -- points 12
		},
	}, -- areas
}





---------------
-- 对象构建
---------------
--[[ 

创建MapView

**Parameters:**

-   containerNode:        CCNode 	     放置MapView的容器
- 	initIndex: 			  int 			 初始显示的地图区域下标,默认为1

**Returns:**

-   return MapView

--]]
function MapView.new(containerNode, initIndex, clickFlagCallback)
	local imgPrefix = "map/bottom/map_big_bg__"								-- 地图块路径字符串前缀
	local items = {} 													-- 地图块路径集
	local _map 															-- 地图对象
	local _curSubmap													-- 当前子地图
	local _curSubmapIndex												-- 当前子地图索引
	local _nodeTable = {}												-- 结点集
	local _mapStarLayer 												-- 星星层
	local _mapStarTable = {}											-- 所有星星
	local _mapAreaNames = {}											-- 地图区块名称集
	local _cloudViews = {} 					-- 天上顠的云层们


--[[ ## initialize fuction --]]
	-- 初始化地图块路径集
	for i = 1, MapView.NUM_TILES do
		table.insert(items, format("%s%02d.png", imgPrefix, i))
	end


--[[ ## local fuction --]]
	-- [网络请求]
	-- 处理网络请求失败的回调函数
	local function netReceiveFail_(result, cbParams)
	    -- 显示[购买体力]对话框
	    AlertLayer.new{
	        titleStr    = _s("COMMON_NEWS_TIPS"),                       	-- 标题
	        content     = _s("NETWORK_CONNECT_FAILED2"),   -- 内容
	        btnStr      = { _s("COMMON_OK") },        				-- 按钮文字数组
	        btnColor    = { AlertLayer.BTN_COLOR_YELLOW },
	        -- 按钮颜色数组
	        btnClickEvent = {                   -- 按钮事件数组
	            function()                      -- 按钮[确定]回调
	                -- print("出错[保存新手帮助步骤]回调")
	            end,
	        }, 
	    }
	end

	-- 发起网络请求，保存新手帮助步骤
	local function netSend_( update )
	    NetHelper.createRequest(
	       ACTID.MSG_UPDATE_EVEN,              -- 请求ID
	       nil,                                -- 回调函数
	       {                                   -- 请求信息实体或参数Table
	            tno = update, -- _curIndex,             	-- 步骤索引
	            typ = 3,
	       },
	       {
	            failedCb = netReceiveFail_,       	-- 请求失败回调函数
	            cbParams = { update = update },
	            -- showLoading = false,
	        }
	    )
	end

	local function saveCurIndex_( update )
		-- 保存下一个将要显示的下标
		-- ISGSVALUES()[TAG_HELPERVIEW_CURINDEX] = _curIndex
		-- GS.save(ISGSVALUES())

		-- 这里还要保存到服务器中
		netSend_(update)
	end 
    local function createRewardForHelper( params )
		local layer = widget.newLayer{contentSize = CCSize(960, 640), }--color = ccc4(255, 255, 100, 255)}
		-- [[创建items层]]
		-- 创建items
		local itemsLayer = widget.newLayer{
			ignoreAnchor = false,
		}
		widget.bindTouchEvent{
            layer = layer,
            priority = -1001,
            -- clickListener = function ( )
            	
            -- end,
            swallowsTouches = true,
            -- container = _params.container,
        }
		-- 背景
		local sprite = widget.newSprite{ file = "reward/bg_reward_items.png" }
		itemsLayer:setContentSize(sprite:getContentSize())
		itemsLayer:addChild(sprite, 0)
		util.alignNode(sprite, itemsLayer)

		local gridLayer = widget.newLayer{ contentSize = CCSize(437, 120)}
		itemsLayer:addChild(gridLayer)
		util.alignNode(gridLayer, itemsLayer)

		-- 奖励类容
		local function createRewardGrid_( )
			local function cellAtIndex( scrollGrid, idx , item)
	    		local cell = widget.newLayer{}--color = ccc4(255, 255, 100, 255),}
	            cell:setContentSize(scrollGrid:getCellSize())
	            cell:ignoreAnchorPointForPosition(false)

	            -- 物品背景
	            local bgSprite = widget.newSprite{file = "goods/goodBack.png"}

	            local spriteLayer = widget.newNode{contentSize = bgSprite:getContentSize()}
	            cell:addChild(spriteLayer)
	            util.alignNode(spriteLayer, cell, display.CENTER_TOP)

	            spriteLayer:addChild(bgSprite, 0)
	            util.alignNode(bgSprite, spriteLayer:getContentSize(), display.CENTER)

	            local rewardSprite
	            local count = 1
	    		if item._TYPE == REWARD_TYPE_CARD then
	    			-- 卡牌
	                item.type = CardMaster.MINISTARCARD
	                rewardSprite = CardMaster.new(item)
	                rewardSprite:setScale(0.76)
	                count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_CHIP then
					-- 碎片
					item.type = CardMaster.MINISTARCARD
	                rewardSprite = CardMaster.new(item)
	                rewardSprite:setScale(0.76)
	                local chipSprite = widget.newSprite{file = "fight/result3/f_r3_cardchip.png"}
	                spriteLayer:addChild(chipSprite, 2)
					util.alignNode(chipSprite, spriteLayer:getContentSize(), display.CENTER)
	                count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_GOODS then
					-- 物品
	                item.type = GoodsMaster.MATERIAL
	                item.showFrame = true
	                rewardSprite = GoodsMaster.new(item)
	                -- count = rewardSprite:getCount()
	                count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_SP then
					-- 体力
					rewardSprite = widget.newSprite{file = "mission/img_sign_burette.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_GOLD then
					-- 金币
					rewardSprite = widget.newSprite{file = "reward/icon/icon_gold.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_FP then
					-- 友情点
					rewardSprite = widget.newSprite{file = "main/img_sign_friendpoint.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_DIAMOND then
					-- 钻石
					rewardSprite = widget.newSprite{file = "main/img_sign_diamond.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_FREEBUY then
					-- 免费抽奖
					rewardSprite = widget.newSprite{file = "mission/img_sign_freebuy.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_STAR then
					-- 星星
					rewardSprite = widget.newSprite{file = "main/img_main_star.png"}
					rewardSprite:setScale(1.4)
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_EXP then
					-- 经验
					rewardSprite = widget.newSprite{file = "main/img_sign_exp.png"}
					count = item._VALUE
				end
				if rewardSprite then
					spriteLayer:addChild(rewardSprite, 1)
					util.alignNode(rewardSprite, spriteLayer:getContentSize(), display.CENTER)
				end
				local countLayer = widget.newLayer{contentSize = CCSize(93, 26)}
				cell:addChild(countLayer)
				util.alignNode(countLayer, cell, display.BOTTOM_CENTER)
				local countBg = widget.newSprite{file = "reward/bg_reward_items_count.png"}
				countLayer:addChild(countBg, 0)
				util.alignNode(countBg, countLayer)
				local countLabel = ui.newTTFLabel{text = "x " .. count, size = 22,}
				countLayer:addChild(countLabel, 1)
				util.alignNode(countLabel, countLayer)

				return cell
	    	end
	    	local function onBtnClick(grid, index, item, x, y)
	            -- item.shareButton = true
	    		if item._TYPE == REWARD_TYPE_CARD or item._TYPE == REWARD_TYPE_CHIP then
	    			CCBCenter.loadCCB("MSG_CARDDETAILLAYER_MODEL", "ccb.c.cardgroup.CardDetailLayer", item)
				elseif item._TYPE == REWARD_TYPE_GOODS then
	                if item:getNo() > 20000 then
	                    CCBCenter.loadCCB("MSG_EQUIPALERTLAYER_MODEL", "ccb.c.goods.EquipAlertLayer", item)
	                else
	                    CCBCenter.loadCCB("MSG_GOODSALERTLAYER_MODEL", "ccb.c.goods.GoodsAlertLayer", item)
	                end
				end
	    	end
			local scrollGrid = widget.newScrollGrid{
	    		contentSize = gridLayer:getContentSize(),
		        contentParams = {
		            cellSize = CCSize(95, gridLayer:getContentSize().height ),
		            padding = { 
		                middleH = 20,
		            },
		            row = 1,
		            createItemCellListener = cellAtIndex,
		            tapListener = onBtnClick,
		        },
		        bounceable = false,
		        direction = widget.TABLEVIEW_DIRECTION_HORIZONTAL,
		        -- parentNode = layer,
		    }
	    	return scrollGrid
		end
		local function rewardAdapter( reward )
	    	local rewardTable = {}
			-- 物品或卡牌碎片等
			if reward.brlist and #reward.brlist > 0 then
	            local _cache = {}
				for i,v in ipairs(reward.brlist) do
                    local entity
                    if v.no < 5000 then
                        -- 绑成卡牌
                        entity = CardEntity.new{id = v.no}
                        entity._TYPE = REWARD_TYPE_CARD
                    elseif v.no < 10000 then
                        -- 碎片
                        entity = CardEntity.new{id = v.no}
                        entity._TYPE = REWARD_TYPE_CHIP
                    else
                        -- 物品
                        entity = GoodsEntity.new(v)
                        entity._TYPE = REWARD_TYPE_GOODS
                    end
                    if _cache[v.no] then
                        entity = _cache[v.no]
                        entity._VALUE = entity._VALUE + v.count
                    else
                        entity._VALUE = v.count
                        rewardTable[#rewardTable + 1] = entity
                        _cache[v.no] = entity
                    end
				end
			end
			-- 体力
			if reward.sp and reward.sp > 0 then
				local sp = {_VALUE = reward.sp, _TYPE = REWARD_TYPE_SP }
				rewardTable[#rewardTable + 1] = sp
			end
			-- 金币
			if reward.gold and reward.gold > 0 then
				local gold = {_VALUE = reward.gold, _TYPE = REWARD_TYPE_GOLD }
				rewardTable[#rewardTable + 1] = gold
			end
			-- 友情点
			if reward.fp and reward.fp > 0 then
				local fp = {_VALUE = reward.fp, _TYPE = REWARD_TYPE_FP }
				rewardTable[#rewardTable + 1] = fp
			end
			-- 钻石
			if reward.diamond and reward.diamond > 0 then
				local diamond = {_VALUE = reward.diamond, _TYPE = REWARD_TYPE_DIAMOND }
				rewardTable[#rewardTable + 1] = diamond
			end
			-- 免费抽奖
			if reward.freebuy and reward.freebuy > 0 then
				local freebuy = {_VALUE = reward.freebuy, _TYPE = REWARD_TYPE_FREEBUY }
				rewardTable[#rewardTable + 1] = freebuy
			end
			-- 经验
			if reward.exp and reward.exp > 0 then
				local exp = {_VALUE = reward.exp, _TYPE = REWARD_TYPE_EXP }
				rewardTable[#rewardTable + 1] = exp
			end
			-- 星星
			if reward.star and reward.star > 0 then
				local star = {_VALUE = reward.star, _TYPE = REWARD_TYPE_STAR }
				rewardTable[#rewardTable + 1] = star
			end
			return rewardTable
		end
		

		local rewardGrid = createRewardGrid_()
		gridLayer:addChild(rewardGrid, 1)
		rewardGrid:setItems(rewardAdapter(params.rewards))
		-- 创建标题
		local titleLabel = ui.newTTFLabel{
			text = params.title,
			size = 30,
			font = FONT_1,
			-- color = BUTTON_COLOR3.RED,
		}
		itemsLayer:addChild(titleLabel)
		util.alignNode(titleLabel, itemsLayer:getContentSize(), display.TOP_CENTER, 12)
		-- 创建button
		local button = widget.newButton{
			normalImage = "reward/reward_button.png",
			scale = 1.1,
			labelParams = {
				text = _s("HELPER_TEXT_REWARD_BUTTON"),
				size = 30,
				color = BUTTON_COLOR3.BROWN,
			},
			priority = -1002,
			listener = function ( button )
				-- if MSG_ID.MSG_MAPCONTAINERLAYER_ONACTIONUPDATESTATUSDATA then
	   --              CCBCenter.sendLuaMessage(MSG_ID.MSG_MAPCONTAINERLAYER_ONACTIONUPDATESTATUSDATA)
	   --          end
	   			HelperView2.showNext()
	   			if not HelperView2.isNextIndex(790) then
	   				saveCurIndex_(19)
	   			end
	   			-- HelperView2.showBetween(_curSubmap, nil, {790, 790})
				
				-- CCBCenter.popLastLevelCCBNode()
				_curSubmap:removeChildByTag(1000, true)
			end
		}
		itemsLayer:addChild(button)
		util.alignNode(button, itemsLayer:getContentSize(), display.BOTTOM_CENTER) 
		layer:addChild(itemsLayer)
		util.alignNodeWithPadding(itemsLayer, layer:getContentSize(), display.CENTER, 
			{ top = 0, left = 20, })

		local x, y = itemsLayer:getPosition()

		-- [[建个小妞放左边,依嘿嘿嘿嘿]]
		if params.showGirl then
			local girlSprite = widget.newSprite{
				file = "reward/reward_girl.png",
				x = x - 326, 
				y = y - 20,
			}
			layer:addChild(girlSprite)
		end
		-- [[放箱钻石在右边,哇哈哈哈哈]]
		local boxSprite = widget.newSprite{
			file = "reward/reward_diamond_box.png",
			x = x + 190,
			y = y - 110,
		}	
		layer:addChild(boxSprite)

		return layer
    end
    
	-- 创建单元格
	local function createItemCellListener(grid, index, item)
		local tile = display.newLayer() --display.newSprite(item)--widget.newNode{ contentSize = grid:getCellSize() } --
		tile:setContentSize(grid:getCellSize()) --(CCSize(CONFIG_SCREEN_WIDTH / 2, CONFIG_SCREEN_HEIGHT / 2))
		tile:ignoreAnchorPointForPosition(false)
		return tile
	end

	-- 处理单元格点击事件响应
	local function tapListener(grid, index, item, x, y)
		local point = grid:convertToNodeSpace(ccp(x, y))
		for i, tile in ipairs(map_info.tiles) do
			if util.rectContainsPoint(tile.rect, point) then 
				-- print(string.format("========you click a area %d==========", tile.index))
				-- 我们要跳到小地图去
				SOUND:playEffectButton2() 											-- 播放音效
				-- 测试记得注释，并打开下面的注释
				--_map.showSubmap( tile.index )
				if _map.canViewSubmap(tile.index) then
					_map.showSubmap( tile.index )										-- 跳转至小地图，同时存储变量
				end
				break
			end
		end
	end

	-- 初始化地图
	_map = widget.newScrollGrid{
		contentSize 				= containerNode:getContentSize(),  				-- 显示尺寸，960x640
		direction 					= widget.TABLEVIEW_DIRECTION_BOTH,
		needRelocate 				= false, 										-- 是否停止时进行定位调整
		disableAutoCalcContentSize 	= true, 										-- 是否禁用自动计算尺寸
		bounceable 					= false, 										-- 是否启用边界阻尼效果
		contentParams = {
			cellSize 		= CCSize(405,342),--CCSize(CONFIG_SCREEN_WIDTH / 2, CONFIG_SCREEN_HEIGHT / 2),	-- 单元格尺寸，960x640
			isMultiTouches 	= true,
			items 			= items,
			column 			= 6,
			row 			= 3,
			-- minScale = 0.5, 
			tapListener 	= tapListener,
			createItemCellListener = createItemCellListener,
		},
		x = 0,
		y = 0,
	}
	function _map.clickFirstArea()
		tapListener(_map:getGrid(), 1, nil, 330, 330)
	end
	-- 初始化波纹层
	if true then 
	    local sprite = widget.newTileSprite({ file = "map/tiles/map_tile_wave2.png", rect = CCRectMake(0, 0, 2400 + 420, 1280 + 420), offset = ccp(-210, -210) })
		_map:getGrid():addChild(sprite, Z_WAVE)

		local action = TDAnimationUtils.sequenceRepeat({
			CCEaseSineInOut:create(CCMoveBy:create(2, ccp(-15, -10))),
			CCEaseSineInOut:create(CCMoveBy:create(2, ccp(0, 10))),
			CCDelayTime:create(.2),
			CCEaseSineInOut:create(CCMoveBy:create(2, ccp(10, -10))),
			CCEaseSineInOut:create(CCMoveBy:create(2, ccp(5, 10))),
		})
		sprite:runAction(action)
	end
	-- 加载地图块
	if true then 
		local lastLevel = game.getGameState("MAX_LEVEL") or 0

		if not isnumber(lastLevel) then lastLevel = 0 end
		for i = 1, MapView.NUM_MAP_SUBLEVELS do
			local sprite = display.newSprite(format("map/tiles/map_%02d_l.png", i))
			_map:getGrid():addChild(sprite, Z_AREA, NODETAG_AREA_START + i)
			-- sprite:setColor(COLOR_DARK)
			sprite:ignoreAnchorPointForPosition(true)
			sprite:setPosition(map_info.areas[i].area_point)
			if i > lastLevel then 
				local cloud = CloudView.new(map_info.areas[i].cloud_rect.size) --(sprite:getContentSize())
				_map:getGrid():addChild(cloud, Z_COLUD + 10 - i)
				cloud:ignoreAnchorPointForPosition(true)
				-- cloud:setPosition(ccpAdd(map_info.areas[i].area_point, ccp(0, 20)))
				cloud:setPosition(map_info.areas[i].cloud_rect.origin)
				_cloudViews[i] = cloud
			end
		end
		_map:runAction(CCRepeatForever:create(transition.sequence{
			CCDelayTime:create(6.0),
			CCCallFunc:create(function (  )
				CloudView.changeMoveDiff()
			end)
			}))
	end
--[===[
	-- 加载装饰动画
	if false then
		-- [海鸥]
		if true then 
			-- 资源加载
			local PREFIX_BIRD = "map_bird"
			local POSITION_START = ccp(400, 1000)
			local POSITION_BY    = ccp(400, -320)
			local OFFSETS = {
				ccp(0, 0),
				ccp(-30, -2),
				ccp(-13, 30),
				ccp(-55, 18),
				ccp(-35, 70),
				ccp(20, 50),
			}
			TDAnimationUtils.addSpriteFramesWithResource("map/anim/" .. PREFIX_BIRD)

			for i = 1, 6 do
				local sprite = display.newSprite("#" .. TDAnimationUtils.frameNameAtFirst(PREFIX_BIRD))
				sprite:setPosition(ccpAdd(POSITION_START, OFFSETS[i]))
				_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

				sprite:runAction(TDAnimationUtils.animateWithFrameRepeat({ prefix = PREFIX_BIRD, frameCount = 6, delay = (1.0/12), }))
				sprite:runAction(TDAnimationUtils.sequenceRepeat({
					CCMoveBy:create(15.0, POSITION_BY),
					CCCallFunc:create(function() sprite:setPosition(ccpAdd(POSITION_START, OFFSETS[i])) end),
				}))
			end
		end

		-- [鱼1]
		if true then 
			local sprite01 = display.newSprite("map/anim/map_fish01_body.png")
			local sprite02 = display.newSprite("map/anim/map_fish01_tail.png")
			sprite01:addChild(sprite02)
			_map:getGrid():addChild(sprite01, Z_ANIM_ITEM)

			local POSITION_START = ccp(702.0, 265.8)
			local POSITION_END   = ccp(682.0, 245.8)

			sprite01:setPosition(POSITION_START)
			sprite02:setPosition(ccp(78.0, 50))

			sprite01:runAction(TDAnimationUtils.sequenceRepeat({
				CCEaseSineIn:create(CCMoveTo:create(8.0, POSITION_END)),
				CCScaleTo:create(.04, 0),
				CCDelayTime:create(3.0),
				CCPlace:create(POSITION_START),
				CCScaleTo:create(.2, 1),
				CCDelayTime:create(1.0),
			}))
		end

		-- [鱼2]
		if true then 
			local sprite = display.newSprite("map/anim/map_fish02.png")
			_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

			sprite:setPosition(ccp(192.7, 469.4))
			sprite:runAction(TDAnimationUtils.sequenceRepeat({
				CCFadeOut:create(4.0),
				CCDelayTime:create(2.0),
				CCFadeIn:create(4.0),
				CCDelayTime:create(3.0),
			}))
		end

		-- [鱼3]
		if true then 
			local sprite = display.newSprite("map/anim/map_fish03.png")
			_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

			local POSITION_START = ccp(2309.2, 381.0)
			local POSITION_END 	 = ccp(2109.2, 441.0)
			sprite:setPosition(POSITION_START)

			sprite:runAction(TDAnimationUtils.sequenceRepeat({
				CCEaseSineIn:create(CCMoveTo:create(8.0, POSITION_END)),
				CCFadeOut:create(1.0),
				CCDelayTime:create(3.0),
				CCPlace:create(POSITION_START),
				CCFadeIn:create(1.0),
				CCDelayTime:create(1.0),
			}))
		end

		-- [鱼4]
		if true then 
			local sprite = display.newSprite("map/anim/map_fish04.png")
			_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

			sprite:setPosition(ccp(913.4, 1067.3))

			sprite:runAction(TDAnimationUtils.sequenceRepeat({
				CCFadeOut:create(2.0),
				CCDelayTime:create(1.0),
				CCFadeIn:create(2.0),
				CCDelayTime:create(1.0),
			}))
		end

		-- [鱼5]
		if true then 
			local sprite = display.newSprite("map/anim/map_fish05.png")
			_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

			sprite:setAnchorPoint(ccp(.5, 0))
			sprite:setPosition(ccp(1307.2, 69.0))

			sprite:runAction(TDAnimationUtils.sequenceRepeat({
				CCCallFunc:create(function()
					sprite:setScaleY(0)
				end),
				TDAnimationUtils.spawn({
					CCEaseBackOut:create(CCScaleTo:create(.3, 1.0)),
					CCFadeIn:create(.3),
				}),
				CCDelayTime:create(6.0),
			}))
		end

		-- [飞艇]
		if true then 
			local sprite = display.newSprite("map/anim/map_airship.png")
			_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

			local POSITION_START = ccp(1540.3, 1136.6)
			local POSITION_END	 = ccp(1540.3, 1116.6)
			sprite:setPosition(POSITION_START)

			sprite:runAction(TDAnimationUtils.sequenceRepeat({
				CCMoveTo:create(5.0, POSITION_END),
				CCMoveTo:create(5.0, POSITION_START),
			}))
		end

		-- [美人鱼]
		if true then 
			local sprite = display.newSprite("map/anim/map_mermaid.png")
			_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

			sprite:setPosition(ccp(2120.3, 1136.6))
		end

		-- [船1]
		if true then 
			local sprite = display.newSprite("map/anim/map_boat01.png")
			_map:getGrid():addChild(sprite, Z_ANIM_ITEM)

			local POSITION_START = ccp(214.1, 1047.2)
			local POSITION_END   = ccp(154.1, 1097.2)
			sprite:setPosition(POSITION_START)

			sprite:runAction(TDAnimationUtils.sequenceRepeat({
				CCEaseSineIn:create(CCMoveTo:create(4.0, POSITION_START)),
				CCCallFunc:create(function() sprite:setFlipX(true); sprite:setRotation(40) end),
				CCDelayTime:create(1.0),
				CCEaseSineOut:create(CCMoveTo:create(4.0, POSITION_END)),
				CCDelayTime:create(1.0),
				CCCallFunc:create(function() sprite:setFlipX(false); sprite:setRotation(0) end),
				CCDelayTime:create(1.0),
			}))
		end
	end
--]===]

	-- 放点碎云
	if true then 
		local function createCloud( cloudType, position )
			local cloud = CloudView.new(cloudType)
			cloud:setPosition(position)
			_map:getGrid():addChild(cloud, Z_COLUD)
		end
		local size = _map:getGrid():getContentSize()
		local w, h = size.width, size.height

		createCloud(CloudView.CLOUD_BIG, ccp(100, h - 100))
		createCloud(CloudView.CLOUD_SMALL, ccp(360, h - 160))
		createCloud(CloudView.CLOUD_BIG, ccp(600, h - 130))
		createCloud(CloudView.CLOUD_SMALL, ccp(900, h - 80))
		createCloud(CloudView.CLOUD_BIG, ccp(w - 460, h - 70))
		createCloud(CloudView.CLOUD_BIG, ccp(w - 60, h - 80))

		createCloud(CloudView.CLOUD_BIG, ccp(40, 380))
		createCloud(CloudView.CLOUD_SMALL, ccp(480, 620))
		createCloud(CloudView.CLOUD_SMALL, ccp(520, 520))
		createCloud(CloudView.CLOUD_BIG, ccp(w - 100, 520))

		createCloud(CloudView.CLOUD_BIG, ccp(100, 100))
		createCloud(CloudView.CLOUD_BIG, ccp(700, 130))
		createCloud(CloudView.CLOUD_BIG, ccp(w - 50, 90))
		createCloud(CloudView.CLOUD_BIG, ccp(1850, 483))
	end

	-- 初始化星星层
	_mapStarLayer = widget.newLayer{}--color = COLOR_MASK}
	_mapStarLayer:setContentSize(_map:getGrid():getContentSize())
	_map:getGrid():addChild(_mapStarLayer, Z_STARS)

	-------- 以下是用来进行点击区域的可视调试,不要删啊
	if MapView.DEBUG then 
		local areaButtonRect = CCRectMake(0, 0, 28, 28) 				-- 图片尺寸(213x315) -- CCRectMake(0, 0, 65, 65)
		local function addAreaClickButton(params, i)
			local area = widget.newButton{
				normalImage = "map/img_rect.png", -- "bg_transparent.png", -- 
				contentSize = params.rect.size,
				rect 		= areaButtonRect, 
				listener 	= function ( button )
					-- print(string.format("========area %d==========", params.index))
				end, 
				x 			= params.rect.origin.x, 					-- 坐标x
				y 			= params.rect.origin.y,						-- 坐标y
				cancelOnMove 	= true,									-- 是否启用移动后取消其可点击模式
				swallowsTouches = false, 								-- 是否阻止点击事件传递
				labelParams 	= {
					text = params.index .. " | " .. tostring(i), 		-- 标签文字
					size = 40, 											-- 标签字体大小
				},
			}
			area:ignoreAnchorPointForPosition(true)
			_map:getGrid():addChild(area, 1000 + i, 1000 + i)
		end

		-- 构建区域边框集
		for i,tile in ipairs(map_info.tiles) do
			addAreaClickButton(tile, i)
		end
	end




--[[ ## public fuction --]]
	local _screenCenterPoint = ccp(CONFIG_SCREEN_WIDTH / 2, CONFIG_SCREEN_HEIGHT / 2)
	--[[ 
	
	大地图上从当前位置移动到指定下标区域
	
	**Parameters:**
	
	-   index:        int       将要移动到的地图区域下标
	- 	anim: 		  float 	移动时间,默认为0.2,为0时无动画
	
	**Returns:**
	
	-   return 是否移动了,若位置没有改变,则为false表示未移动
	
	--]]
	function _map.moveTo( index, anim )
		if isnumber(index) and 0 < index and index <= MapView.NUM_MAP_SUBLEVELS then 
			anim = anim or 0.2
			-- 计算计算,该死的计算
			local grid = _map:getGrid()
			local centerPoint = ccpMult(map_info.areas[index].center_point, grid:getScale())
			local position = ccpMult(grid:convertToNodeSpace(_screenCenterPoint), grid:getScale())
			local curContentOffset = _map:getContentOffset()
			local destContentOffset = ccp(curContentOffset.x - centerPoint.x + position.x, curContentOffset.y - centerPoint.y + position.y)
			-- 超界处理
			local offsetX, offsetY = destContentOffset.x, destContentOffset.y
			if offsetX > 0 then offsetX = 0 end
			if offsetY > 0 then offsetY = 0 end
			local tempX = (-grid:getContentSize().width + CONFIG_SCREEN_WIDTH) * grid:getScale()
			if offsetX < tempX then offsetX = tempX end
			local tempY = (-grid:getContentSize().height + CONFIG_SCREEN_HEIGHT) * grid:getScale()
			if offsetY < tempY then offsetY = tempY end

			destContentOffset = ccp(offsetX, offsetY)
			if destContentOffset.x == curContentOffset.x and destContentOffset.y == curContentOffset.y then 
				return false 
			end
			if anim > 0 then 
				_map:setContentOffsetInDuration(destContentOffset, anim)
			else
				_map:setContentOffset(destContentOffset)
			end
		else
			_map:setContentOffset(CCPointZero)
		end
		return true
	end
	
	--[[ 
	
	创建小地图左上角的信息块
	
	**Parameters:**
	
	-   gateNum:        int 	     通过的关卡个数
	- 	starNum: 		int 		 可收集的星星个数
	
	**Returns:**
	
	-   return 小地图左上角的信息块
	
	--]]
	local function createSubmapInfo(mapIndex)
		-- [创建层]
		local layer = widget.newLayer{ contentSize = CCSize(150, 80), }


		-- [默认赋值]
		local gateNum = _map.getPassedGateNum(mapIndex) 			-- 获得已经通关关卡个数
		local starNum = _map.getPassedStarNum(mapIndex) 			-- 获得已经通关关卡星星数

		local countNum = LevelDao.getCurrentLevels(mapIndex)
		-- [创建结点]
		-- 1、关卡
		local gateIcon 		= widget.newSprite{ file = "map/submaps/gate_icon.png" }
		local gateTitleBg 	= widget.newScale9{ file = "map/submaps/bg_title.png", contentSize = CCSize(96, 30), }
		local gateNode 		= widget.newNode{ contentSize = gateTitleBg:getContentSize() }
		util.alignNode(gateTitleBg, gateNode:getContentSize(), display.CENTER)
		local gateNumLabel = ui.newTTFLabel{
				text = gateNum .. "/" .. countNum, size = 28,
				textAlign = ui.TEXT_ALIGN_CENTER,
			}
		util.alignNode(gateNumLabel, gateNode:getContentSize(), display.CENTER)
		gateNumLabel:setPosition(ccpAdd(ccp(gateNumLabel:getPosition()), ccp(0, -2)))
		gateNode:addChild(gateTitleBg)
		gateNode:addChild(gateNumLabel)
		util.modifyPositionInAndroid(gateNumLabel, 0, 2)

		-- 2、星星
		local starIcon = widget.newSprite{ file = "map/submaps/star_icon.png", }
		local starBg = widget.newScale9{ file = "map/submaps/bg_title.png", contentSize = CCSize(96, 30), }
		local starNode = widget.newNode{ contentSize = starBg:getContentSize() }
		util.alignNode(starBg, starNode:getContentSize(), display.CENTER)
		local starNumLabel = ui.newTTFLabel{
				text = starNum .. "/" .. (countNum * 3), size = 28,
				textAlign = ui.TEXT_ALIGN_CENTER,
			}
		util.alignNode(starNumLabel, starNode:getContentSize(), display.CENTER)
		starNumLabel:setPosition(ccpAdd(ccp(starNumLabel:getPosition()), ccp(0, -2)))
		starNode:addChild(starBg)
		starNode:addChild(starNumLabel)
		util.modifyPositionInAndroid(starNumLabel, 0, 2)


		-- [加入到层]
		layer:addChild(gateIcon)
		util.alignNode(gateIcon, layer:getContentSize(), display.TOP_LEFT)
		layer:addChild(starIcon)
		util.alignNode(starIcon, layer:getContentSize(), display.BOTTOM_LEFT)
		layer:addChild(gateNode)
		util.alignNodeWithPadding(gateNode, layer:getContentSize(), display.TOP_LEFT, 
			{ left = gateIcon:getContentSize().width + 8, top = 6 })
		layer:addChild(starNode)
		util.alignNodeWithPadding(starNode, layer:getContentSize(), display.BOTTOM_LEFT, 
			{ left = starIcon:getContentSize().width + 8, bottom = 2 })

		-- 小登记下以后方便改
		_nodeTable[NODETAG_LABEL_FLAG_NUM] = gateNumLabel
		_nodeTable[NODETAG_LABEL_STAR_NUM] = starNumLabel

		return layer
	end

	--[[ 
	
	创建小地图关卡
	
	**Parameters:**
	
	-   mapIndex:        int 	     区域下标
	- 	flagIndex: 		 int 		 关卡旗帜下标
	- 	gateFlagType: 	 emnu 		 关卡旗帜类型
										MapView.GATE_FLAG_TYPE_NONE (默认)
										MapView.GATE_FLAG_TYPE_EASY 
										MapView.GATE_FLAG_TYPE_NORMAL	
										MapView.GATE_FLAG_TYPE_HARD 
	- 	anim: 			 bool 		 是否使用动画,默认为false	
	
	**Returns:**
	
	-   return 小地图关卡
	
	--]]
	function _map.createSubmapGate( mapIndex, flagIndex, anim )
		-- [创建层]
		local layer = widget.newLayer{ contentSize = CCSize(86, 110), ignoreAnchor = false }


		-- [重置子结点]
		if _curSubmap:getChildByTag(1000 + flagIndex) then 
			_curSubmap:getChildByTag(1000 + flagIndex):removeFromParentAndCleanup(true)
		end

		-- [数据源]
		-- 旗帜PO
		local flagInfo 	   = map_info.areas[mapIndex].points[flagIndex]
		-- 关卡旗帜类型
		local gateFlagType = _map.getFlagStar(mapIndex, flagIndex)
		-- 关卡名称
		local title    	   = _map.getFlagTitle(mapIndex, flagIndex)
		-- [创建结点]
		-- 阴影
		local gateShadow = widget.newSprite{ file = "map/submaps/gate_flag_shadow.png", }
		-- 旗
		local gateFlag = widget.newSprite{ file = string.format("map/submaps/gate_flag_%d.png", iff(gateFlagType == 0, 0, gateFlagType)) }
		-- 字
		local titleNode = widget.newNode{ contentSize = CCSize(layer:getContentSize().width, 20) }
		local titleBg = widget.newScale9{ file = "map/submaps/bg_gate_name.png", contentSize = titleNode:getContentSize(), }
		local titleLabel = ui.newTTFLabel{
				text = title, size = 12,
				textAlign = ui.TEXT_ALIGN_CENTER,
			}
		util.alignNode(titleBg, titleNode:getContentSize(), display.CENTER)
		util.alignNode(titleLabel, titleNode:getContentSize(), display.CENTER)
		titleLabel:setPosition(ccpAdd(ccp(titleLabel:getPosition()), ccp(0, -2)))
		titleNode:addChild(titleBg)
		titleNode:addChild(titleLabel)
		-- 星
		local starsLayer
		if gateFlagType > 0 then
			starsLayer = widget.newLinearLayer{ }
			for i = 1, gateFlagType do
				local starSprite = widget.newSprite{ file = "map/submaps/gate_star.png" }
				starsLayer:addChild(starSprite)
			end
		end


		-- [加入到层]
		util.alignNode(titleNode, layer:getContentSize(), display.BOTTOM_CENTER)
		layer:addChild(titleNode)
		util.alignNode(gateShadow, layer:getContentSize(), display.BOTTOM_CENTER, titleNode:getContentSize().height)
		layer:addChild(gateShadow)
		util.alignNode(gateFlag, layer:getContentSize(), display.BOTTOM_CENTER, titleNode:getContentSize().height)
		layer:addChild(gateFlag)
		if starsLayer then
			util.alignNode(starsLayer, layer:getContentSize(), display.TOP_CENTER)
			layer:addChild(starsLayer)
		end


		-- [动画]
		-- 要不要用动画啊,很麻烦啊
		display.align(layer, display.BOTTOM_CENTER, flagInfo.x, flagInfo.y)
		if anim then 
			local positionGateFlag = ccp(gateFlag:getPosition())
			local scaleGateFlag    = gateFlag:getScale()
			local scaleGateShadow  = gateShadow:getScale()
			local scaleTitleNode   = titleNode:getScale()
			gateFlag:setPosition(ccpAdd(positionGateFlag, ccp(0, CONFIG_SCREEN_HEIGHT)))
			titleNode:setScaleY(0)
			gateShadow:setScale(scaleGateShadow * .7)

			gateFlag:runAction(
				TDAnimationUtils.sequence({
					CCEaseExponentialIn:create(CCMoveTo:create(0.3, positionGateFlag)),
					CCCallFunc:create(function() SOUND:playEffectMapNewFlag() end),
					CCScaleTo:create(0.1, scaleGateFlag, scaleGateFlag * .8),
					CCEaseBackOut:create(CCScaleTo:create(0.1, scaleGateFlag)),
					CCCallFunc:create(function()
						titleNode:setScale(scaleTitleNode)
					end),
				})
			)
			gateShadow:runAction(
				TDAnimationUtils.sequence({
					CCScaleTo:create(0.3, scaleGateShadow),
					CCScaleTo:create(0.1, scaleGateShadow * 1.5, scaleGateShadow),
					CCEaseBackOut:create(CCScaleTo:create(0.15, scaleGateShadow)),
				})
			)
		end
		layer:setTag(1000 + flagIndex)

		if not _curSubmap.GATE_LAYER then 
			local gateLayer = display.newLayer()
			_curSubmap:addChild(gateLayer)
			_curSubmap.GATE_LAYER = gateLayer
		end
		
		_curSubmap.GATE_LAYER:addChild(layer)
		-- 绑定点击事件
		widget.bindTouchEvent{
			layer = layer,
			-- sound = "",
			clickListener = function ( gate )
			HelperView2.showNext()
			gate:runAction(
				TDAnimationUtils.sequence{
					CCScaleTo:create(0.1, 1.2),
					CCScaleTo:create(0.15, 0.8),
					CCScaleTo:create(0.08, 1.0),
					CCCallFunc:create(function()
						if clickFlagCallback then 
							if HelperView2.showBetween(_curSubmap, 
								function ( index )
									if index == 590 then 
										clickFlagCallback(mapIndex, flagIndex)
									end
								end, {510, 590}) ~= nil then
							elseif HelperView2.showBetween(_curSubmap, 
								function ( index )
									if index == 760 then
										-- 弹出领取新手奖励
										local layer = createRewardForHelper{title = _s("HELPER_TEXT_REWARD_TITLE"), rewards = {diamond = 50, fp = 100, star = 200, brlist = {{no = 72, count = 1},}}}
										_curSubmap:addChild(layer, 10, 1000)
									elseif index == 810 then 
										-- if HelperView2.showBetween(_curSubmap,
											-- showHelpCallBack, {790, 810}) == nil then
											clickFlagCallback(mapIndex, flagIndex)
										-- end
									end
								end, {700, 810}) ~= nil then
							elseif HelperView2.showBetween(_curSubmap,
								function ( index )
									if index == 810 then 
										clickFlagCallback(mapIndex, flagIndex)
									end
								end, {790, 810}) ~= nil then 
							else 
								clickFlagCallback(mapIndex, flagIndex)
							end
						end
					end)
				})
			end,
		}

		return layer
	end
	-- 初始小地图上的所有关卡信息
	function _map.initSubmapGateFlags(curSubmapIndex)
		if _curSubmap then 
			for i = 1, LevelDao.getCurrentLevels(curSubmapIndex) do
				--测试注释，记得还原回去
				 if _map.isCreateSubmapGate(_curSubmapIndex, i) then 
					_map.createSubmapGate(_curSubmapIndex, i, _map.isAnimateSubmapGate(_curSubmapIndex, i))
				 end
			end
			-- HelperView2.showBetween(_curSubmap, function ( index )
			-- 	if index == 760 then
			-- 		-- 弹出领取新手奖励
			-- 		local layer = createRewardForHelper{title = _s("HELPER_TEXT_REWARD_TITLE"), rewards = {diamond = 50, fp = 100, star = 200, brlist = {{no = 72, count = 1},}}}
			-- 		_curSubmap:addChild(layer, 10, 1000)
			-- 	end
			-- end, {700, 810})
			-- local newFirstFlag = nil
			-- for i = 1, MapView.NUM_MAP_SUBLEVELS do
			-- 	if _map.isCreateSubmapGate(_curSubmapIndex, i) then 
			-- 		local anim = _map.isAnimateSubmapGate(_curSubmapIndex, i)
			-- 		if i == 1 and anim then 
			-- 			newFirstFlag = true
			-- 		end 
			-- 		_map.createSubmapGate(_curSubmapIndex, i, anim)
			-- 	end
			-- end
			-- TODO 这里进行故事引导的非点击触发时机 当前子地图下标为_curSubmapIndex
			-- if _map.isAnimateSubmapGate(_curSubmapIndex + 1, 1) then
			-- 	print("@@@@@@@@@@@@@@@@@ 新地图的第一次触发啊") 
			-- end
		end
	end
	--[[ 
	
	显示小地图
	
	**Parameters:**
	
	-   index:        int 	     区域下标
	- 	anim: 		  bool 		 是否动画显示,默认为true
	
	**Returns:**
	
	-   return 小地图
	
	--]]
	function _map.showSubmap( index, anim )
		-- [创建层]
		local submap = display.newLayer()
		submap:setCascadeOpacityEnabled(true)

		-- [状态存储]
		_curSubmapIndex = index 							-- 存储当前子地图索引
		_curSubmap 		= submap							-- 存储当前子地图


		-- [数据源]
		local areaInfo = map_info.areas[index] 				-- 子地图PO对象


		-- [创建结点]
		-- 1、背景
		local submapSprite = widget.newSprite{
			file = areaInfo.map,
		}
		submap:addChild(submapSprite)
		util.alignNode(submapSprite, submap:getContentSize(), display.CENTER)

		-- 2、返回按键
		local returnBtn = widget.newButton{
							normalImage = "map/submaps/btn_close_normal.png",
							selectImage = "map/submaps/btn_close_select.png",
							listener = function ( btn )
								submap.closeLayer()
							end,
						}
		submap:addChild(returnBtn)
		util.alignNode(returnBtn, submap:getContentSize(), display.TOP_RIGHT, 10)

		-- 3、左上角信息
		local submapInfoLayer = createSubmapInfo(index)
		submap:addChild(submapInfoLayer)
		util.alignNode(submapInfoLayer, submap:getContentSize(), display.TOP_LEFT, 12)

		-- 4、探索按钮
		if ME:getTriggerMsgEntity():isUnlockExplore() then
			local bgBtnExplore = widget.newSprite{file = "main/effect_buttons/bg_main_button.png"}

			local btnLayer = widget.newNode{contentSize = bgBtnExplore:getContentSize()}
			submap:addChild(btnLayer)
			util.alignNode(btnLayer, submap:getContentSize(), display.BOTTOM_RIGHT, 20)

			btnLayer:addChild(bgBtnExplore, 0)
			util.alignNode(bgBtnExplore, btnLayer:getContentSize(), display.CENTER)

			local btnExplore = widget.newButton{
				normalImage = "map/btn_map_explore.png", 
				listener = function ( btn )
					SOUND:playEffectButton1()
					HelperView2.showNext()
					CCBCenter.loadCCB("MSG_MAPEXPLORESELECTLAYER_MODEL", "ccb.c.map.MapExploreSelectLayer", {sn = _curSubmapIndex,})
				end,
				scale = 1.1,
				-- labelParams = {text = _s("MAP_EXPLORE"), size = 20, padding = {top = 24},},-- color = BUTTON_COLOR3.BROWN},
			}
			btnLayer:addChild(btnExplore, 1)
			util.alignNode(btnExplore, btnLayer:getContentSize(), display.CENTER)

			local textLabel = widget.newLayer{}
	        textLabel:setContentSize(CCSize(72, 22))
			btnLayer:addChild(textLabel, 2)
			util.alignNodeWithPadding(textLabel, btnLayer:getContentSize(), display.BOTTOM_CENTER, {bottom = 10})
			-- 文字背景
			local bgSprite = widget.newScale9{file = "main/bg_button_title.png", contentSize = textLabel:getContentSize(),}
			textLabel:addChild(bgSprite, 0)
			util.alignNodeWithPadding(bgSprite, textLabel:getContentSize(), display.CENTER)
			-- 文字
			local exploreLabel = ui.newTTFLabel{text = _s("MAP_EXPLORE"), size = 18,}
			textLabel:addChild(exploreLabel, 0)
			util.alignNodeWithPadding(exploreLabel, textLabel:getContentSize(), display.CENTER, {top = 2})
			if ME:getTriggerMsgEntity():isJumpToExploreHelper() then
				HelperView2.setCurIndex(110020)
				HelperView2.showBetween(submap, nil, {110020, 110020})
			end
		end

		if ME:getTriggerMsgEntity():isShowLevelStarRewardHelper() then
			-- 弹出领取新手奖励
			local layer = createRewardForHelper{title = _s("HELPER_TEXT_STAR_REWARD_TITLE"), showGirl = true, rewards = {diamond = 50, fp = 200, star = 300, brlist = {{no = 17, count = 1},}}}
			-- CCBCenter.pushLastLevelCCBNode(layer)
			_curSubmap:addChild(layer, 10, 1000)
		end

		

		-- [动画]
		submap:setVisible(false)
		submap.closeLayer = function ( ... )
			-- 刷新地图状态栏
			CCBCenter.sendLuaMessage(MSG_ID.MSG_MAPCONTAINERLAYER_ONACTIONUPDATESTATUSDATA)
			-- 处理地图
			transition.scaleTo(_map, { time = 0.5, scale = 1.0 })
			transition.fadeOut(submap, { time = 0.5,
				onComplete = function()
					CCBCenter.popSecondCCBNode()
					_curSubmapIndex = nil 
					_curSubmap = nil
				end  
			})

			-- 处理子结点
			if _curSubmap.GATE_LAYER then 
				_curSubmap.GATE_LAYER:setVisible(false)
				_curSubmap.GATE_LAYER = nil
			end
		end
		if anim == false then 
			_map.moveTo(index, 0)
			_map:setScale(4.0)
			submap:setVisible(true)
			_map.initSubmapGateFlags(_curSubmapIndex)
		else
			local delay = 0.2
			if _map.moveTo(index, delay) == false then delay = 0 end
			util.delayCall(function ()
				transition.scaleTo(_map, { time = 0.5, scale = 4.0 })
				submap:setVisible(true)
				transition.fadeIn(submapSprite, { time = 0.5, onComplete = function()
					_map.initSubmapGateFlags(_curSubmapIndex)
				end })
			end, delay)
		end
		
		CCBCenter.pushSecondCCBNode(submap)
		HelperView2.showBetween(submap, nil, { 555 , 555 }, { 657 , 657 }, { 780 , 780 }, { 1010 , 1010 })


		return submap
	end

	function _map.getSubmapNode(  )
		return _curSubmap
	end

	-- 求两点间直线距离
	local function getDistance( pointA, pointB )
		local result = ((pointA.x - pointB.x)^2 + (pointA.y - pointB.y)^2)^0.5
		return math.abs(result) 
	end
	-- [外部接口]
	--[[ 
	
	展示大地图星星
	
	**Parameters:**

	- 	starNum: 		int 		 可收集的星星个数
	
	**Returns:**
	
	--]]
	function _map.showMapStar( starNum )
		-- 清空
		_mapStarLayer:removeAllChildrenWithCleanup(true)
		_mapStarTable = {}
		_mapStarTable.times = 1
		-- 无星星可收, 木有
		if starNum <= 0 then
			return
		end
		-- 加入星星
		local OFFSET_BASE = ccp(-30, -50)
		for i, area in ipairs(map_info.areas) do
			local stars = _map.getPassedStarNum(i)
			if stars > 0 then
				-- 星星精灵
				_mapStarTable[i] = widget.newNode{}
				_mapStarTable[i].star = widget.newSprite{file = "map/submaps/star_icon.png"}
				_mapStarTable[i]:addChild(_mapStarTable[i].star)
				_mapStarTable[i]:setScale(1.5)
				_mapStarLayer:addChild(_mapStarTable[i])
				_mapStarTable[i]:setPosition(ccpAdd(area.center_point, OFFSET_BASE))
				-- 数量精灵
				_mapStarTable[i].label = ui.newTTFLabelWithShadow{text = "x" .. stars, size = 26,}
				_mapStarLayer:addChild(_mapStarTable[i].label)
				_mapStarTable[i].label:setPosition(ccpAdd(area.center_point, ccp(30 + OFFSET_BASE.x, OFFSET_BASE.y)))
			end
		end
	end

	-- [外部接口]
	--[[ 
	
	收集地图星星动画
	
	**Parameters:**

	- 	focusPoint: 		ccp 		 汇聚点
	- 	onComplete: 		function     完成
	**Returns:**
	
	--]]
	function _map.collectMapStar( focusPoint , onComplete)
		-- 无星星可收, 不理你
		if #_mapStarTable <= 0 then
			return
		end
		focusPoint = focusPoint or _screenCenterPoint
		local position = _map:getGrid():convertToNodeSpace(focusPoint)
		-- 星星动画
		for i, starNode in ipairs(_mapStarTable) do
			starNode.label:setVisible(false)
			starNode.particle = CCParticleSystemQuad:create("map/star.plist")
			local particleBatch = CCParticleBatchNode:createWithTexture(starNode.particle:getTexture())
	    	particleBatch:addChild(starNode.particle)
	    	starNode:addChild(particleBatch)
			local array     = {}
			-- 根据距离计算时间
			local distance = getDistance(ccp(starNode:getPositionX(), starNode:getPositionY()), position)
	        array[#array + 1] = CCMoveTo:create(distance / 1000, position)
	        -- SOUND:playStarFlit()
	        transition.execute(starNode, transition.sequence(array), {easing = "CCEaseSineIn", onComplete = function ()
	        	starNode.star:setVisible(false)
	        	starNode.particle:stopSystem()
	        	SOUND:playStar()
	        	_mapStarTable.times = _mapStarTable.times + 1
	        	if _mapStarTable.times > #_mapStarTable then
	        		_mapStarTable.times = 1
	        		onComplete()
	        	end
	        end })
		end
	end

	-- [外部接口]
	--[[ 
	
	设置小地图左上角信息
	
	**Parameters:**
	
	-   flagNum:        int 	     通过的关卡个数
	- 	starNum: 		int 		 可收集的星星个数
	
	**Returns:**
	
	--]]
	function _map.setSubmapInfo( flagNum, starNum )
		if isnumber(flagNum) and flagNum >= 0 then 
			_nodeTable[NODETAG_LABEL_FLAG_NUM].setString(flagNum .. "/" .. MapView.NUM_MAP_SUBLEVELS)
		end
		if isnumber(starNum) and starNum >= 0 then 
			_nodeTable[NODETAG_LABEL_STAR_NUM].setString(tostring(starNum))
		end
	end

	--[[ 
	
	设置地图区域显示
	
	**Parameters:**
	
	- 	names: 			table 		 地图区域名字集
	-   maxAreaIndex:   int 	     地图区域最大索引
	
	**Returns:**
	
	--]]
	function _map.renderAreas(names, maxAreaIndex)
		if names and type(names) == "table" then 
			for i = #_mapAreaNames, 1, -1 do
				_mapAreaNames[i]:removeFromParentAndCleanup(true)
			end
			for i = 1, #names do
				local layer = display.newLayer()
				local bg_map_tittle = display.newSprite("map/bg_map_tittle.png")
				layer:setContentSize(bg_map_tittle:getContentSize())

				layer:setPosition(ccpAdd(map_info.areas[i].center_point, ccp(-80, 0)))
				util.alignNode(bg_map_tittle, layer:getContentSize(), display.CENTER)
				layer:addChild(bg_map_tittle)

				local label = ui.newTTFLabel{text = names[i], size = 20, color = BUTTON_COLOR3.BROWN}--LevelDao.getIndexName(i) .. ". " .. names[i], size = 36,}
				-- util.alignNode(label, layer:getContentSize(), display.CENTER)
				util.alignNodeWithPadding(label, layer:getContentSize(), display.CENTER, 
				{top = 2})
				layer:addChild(label)

				_map:getGrid():addChild(layer, Z_AREA_NAME)
				_mapAreaNames[i] = label
			end
		end

		if type(maxAreaIndex) == "number" then 
			for i = 1, maxAreaIndex do
				-- Version1:
				-- _map:getGrid():getChildByTag(NODETAG_AREA_START + i):setTexture(TDAnimationUtils.addTextureCache(format("map/tiles/map_%02d_l.png", i)))

				-- Version2:
				-- local oldSprite = _map:getGrid():getChildByTag(NODETAG_AREA_START + i)
				-- local sprite = display.newSprite(format("map/tiles/map_%02d_l.png", i))
				-- _map:getGrid():addChild(sprite, Z_AREA, NODETAG_AREA_START + i)

				-- sprite:ignoreAnchorPointForPosition(false)
				-- sprite:setPosition(oldSprite:getPosition())
				-- oldSprite:removeFromParentAndCleanup(true)

				-- Version3:
				-- util.toCCSprite(_map:getGrid():getChildByTag(NODETAG_AREA_START + i)):setColor(COLOR_LIGHT)
				try(
					function ( )
						if _cloudViews[i] then 
							_cloudViews[i]:disperse()
						end
					end
				)
				
			end

			-- STORY 这里进行故事引导的非点击触发时机 当前子地图下标为_curSubmapIndex
			if _map.isShowNewMapArea and _map.isShowNewMapArea(maxAreaIndex) then
				local storyPlayed = game.getGameState("STORY_PLAYED")
				local lastMapIndex = maxAreaIndex - 1

				-- 在10-10普通过了却又没有记录,你重装的啊,cow,不给你弹出来了
				if not storyPlayed and maxAreaIndex == MapView.NUM_MAP_SUBLEVELS and _curSubmapIndex == LevelDao.getCurrentLevels(maxAreaIndex) then 
					return 
				end

				-- -- 有记录以来,到12-15普通过了,就当你是最后的了,给弹出来吧
				if storyPlayed and maxAreaIndex == MapView.NUM_MAP_SUBLEVELS + 1 and string.startwith(storyPlayed,  string.format("%02d-", maxAreaIndex)) then 

					lastMapIndex = maxAreaIndex - 1
				end

				if storyPlayed and storyPlayed == string.format("%02d-%02d-0", lastMapIndex,LevelDao.getCurrentLevels(lastMapIndex)) then 
					-- print("@@@@@@@@@@@@@@@@@ 新地图的第一次触发啊", maxAreaIndex)

					_map.moveTo( lastMapIndex)
					HelperView2.playStoryEnd(containerNode:getParent(), lastMapIndex, function ( mapIndex )
						if maxAreaIndex <= MapView.NUM_MAP_SUBLEVELS then
							_map.moveTo( maxAreaIndex)
						else
							_map.moveTo( lastMapIndex)
						end
						game.addGameState("STORY_PLAYED", string.format("%02d-%02d", mapIndex,LevelDao.getCurrentLevels(mapIndex) + 1), true)
					end)
					return true
				end

			end 
			
		end
	end

	--[[ 
	
	根据地图索引、关卡索引获得某关卡名称
	
	**Parameters:**
	
	-   mapIndex:        int 	     地图索引
	- 	flagIndex: 		 int 		 关卡索引
	
	**Returns:**
	
	--]]
	function _map.getFlagTitle(mapIndex, flagIndex)
		-- TODO: throw new UmimplementException! It will reset by External call.
		return ""
	end

	--[[ 
	
	根据地图索引、关卡索引获得某关卡星星数目
	
	**Parameters:**
	
	-   mapIndex:        int 	     地图索引
	- 	flagIndex: 		 int 		 关卡索引
	
	**Returns:**
	
	--]]
	function _map.getFlagStar(mapIndex, flagIndex)
		-- TODO: throw new UmimplementException! It will reset by External call.
		return -1
	end

	--[[ 
	
	根据地图索引获得某地图已经通过的关卡数量
	
	**Parameters:**
	
	-   mapIndex:        int 	     地图索引
	
	**Returns:**
	
	--]]
	function _map.getPassedGateNum(mapIndex)
		-- TODO: throw new UmimplementException! It will reset by External call.
		return -1
	end


	--[[ 
	
	根据地图索引获得某地图已经通关的星星数量
	
	**Parameters:**
	
	-   mapIndex:        int 	     地图索引
	
	**Returns:**
	
	--]]
	function _map.getPassedStarNum(mapIndex)
		-- TODO: throw new UmimplementException! It will reset by External call.
		return -1
	end

	--[[ 
	
	根据地图索引、关卡索引判断是否要生成子关卡
	
	**Parameters:**
	
	-   mapIndex:        int 	     地图索引
	- 	flagIndex: 		 int 		 关卡索引
	
	**Returns:**
	
	--]]
	function _map.isCreateSubmapGate(mapIndex, flagIndex)
		-- TODO: throw new UmimplementException! It will reset by External call.
		return false
	end

	--[[ 
	
	根据地图索引、关卡索引判断是否要对子地图关卡进行动画显示
	
	**Parameters:**
	
	-   mapIndex:        int 	     地图索引
	- 	flagIndex: 		 int 		 关卡索引
	
	**Returns:**
	
	--]]
	function _map.isAnimateSubmapGate(mapIndex, flagIndex)
		-- TODO: throw new UmimplementException! It will reset by External call.
		return false
	end

	--[[ 
	
	判断地图区域是否可点击
	
	**Parameters:**
	
	-   mapIndex:        int 	     地图索引
	
	**Returns:**
	
	--]]
	function _map.canViewSubmap(mapIndex)
		-- TODO: throw new UmimplementException! It will reset by External call.
		return false
	end

	
	-- 初始缩放级别
	_map:getGrid():initScale4Multi( 1 )
	_map.moveTo(initIndex or 1, 0)

	containerNode:addChild( _map )
	return _map
end

return MapView




























