
local M = {}
M.TAG = "[cardui.ParallaxView]"
-- 主要场景(快速获得)
M.LAYER_CATEGORY_MAIN = 0
-- 部件:普通精灵
M.PATS_CATEGORY_SPRITE = 0
-- 部件:帧动画
M.PATS_CATEGORY_SPRITEFRAMES = 1
--[[ 

## ParallaxView
TODO 垂直的还没有
**Parameters:**

-   params: 数组包含
            [必填项]
            table                                   bgConfigTable          尺寸
            CCSize                                  contentSize             cell尺寸
--]]
function M.new(params)
    
--[=[ ## local params --]=]
    --#必填项#
    -- 层配置表
    local _bgConfigTable      = params.bgConfigTable
    Log.a(_bgConfigTable, M.TAG, " ParallaxView.new() invalid params.bgConfigTable")
    -- 容器尺寸
    local _contentSize        = params.contentSize
    Log.a(_contentSize, M.TAG, " ParallaxView.new() invalid params.contentSize")
    -- 移动时间
    local _moveTime           = params.moveTime or 0
    -- 视差容器(时差容器)
    local _parallaxNode        = widget.newNode{contentSize = _contentSize, ignoreAnchor = true}
    local _count = 2
--[=[ ## local fuction --]=]

    -- 循环背景
    local function createCycleMoveLayer( config )
        local layer 
        local cellSize
        local imageSize = CCSize(0, 0)
        local offset = config.offset or { left = 0, right = 0, down = 0, up = 0, } 
        if config.files then
            -- 有图片直接贴背景
            local image
            for i, v in ipairs(config.files) do
                image = widget.newSprite{file = v, ignoreAnchor = true}
                imageSize = CCSizeAdd(imageSize, image:getContentSize().width, image:getContentSize().height)
            end
            cellSize = CCSizeAdd(imageSize, (offset.left or 0) + (offset.right or 0), (offset.up or 0) + (offset.down or 0))
        else
            cellSize = CCSizeAdd(config.contentSize or config.node:getContentSize(), (offset.left or 0) + (offset.right or 0), (offset.up or 0) + (offset.down or 0))
        end

        if config.rate and config.rate > 0 then
            -- 创建循环数
            if cellSize.width > _contentSize.width then
                _count = math.ceil(cellSize.width / _contentSize.width)
            elseif cellSize.width < _contentSize.width then
                _count = math.ceil(_contentSize.width / cellSize.width)
            end

            -- 创建层
            layer = widget.newLayer{ 
                contentSize = CCSize(cellSize.width * _count, cellSize.height),
                ignoreAnchor = false,
            }
            -- 加入层
            local imageViewTable = {}
            for i = 1, _count do
                if config.files then
                    local imageLayer = widget.newLayer{
                        contentSize = imageSize,
                        ignoreAnchor = true,
                    }
                    local imageViewTable = {}
                    for i,v in ipairs(config.files) do
                        local imageSprite = widget.newSprite{file = v, ignoreAnchor = true}
                        imageLayer:addChild(imageSprite)
                        imageViewTable[i] = imageSprite
                        if i == 1 then
                            imageSprite:setPosition(0, 0)
                        else
                            imageSprite:setPosition(imageViewTable[i - 1]:getPositionX() + imageViewTable[i - 1]:getContentSize().width, 0)
                        end
                    end
                    layer:addChild(imageLayer, 0, i)
                    imageLayer:setPosition((offset.left or 0) + (i - 1) * cellSize.width , offset.up or 0)
                else
                    local node = config.node 
                    or widget.newLayer{ 
                        contentSize = config.contentSize,
                        ignoreAnchor = true,
                        -- color = ccc4(255, 100, 132, 152),
                    }
                    layer:addChild(node, 0, i)
                    node:setPosition((offset.left or 0) + (i - 1) * cellSize.width , offset.up or 0)
                end
            end
        else
            -- 创建层
            layer = widget.newLayer{ 
                contentSize = CCSize(cellSize.width, cellSize.height),
                ignoreAnchor = false,
            }
            if config.files then
                local imageLayer = widget.newLayer{
                    contentSize = imageSize,
                    ignoreAnchor = true,
                }
                local imageViewTable = {}
                for i,v in ipairs(config.files) do
                    local imageSprite = widget.newSprite{file = v, ignoreAnchor = true}
                    imageLayer:addChild(imageSprite)
                    imageViewTable[i] = imageSprite
                    if i == 1 then
                        imageSprite:setPosition(0, 0)
                    else
                        imageSprite:setPosition(imageViewTable[i - 1]:getPositionX() + imageViewTable[i - 1]:getContentSize().width, 0)
                    end
                end
                layer:addChild(imageLayer, 0, 1)
                imageLayer:setPosition(offset.left or 0, offset.up or 0)
            else
                local node = config.node 
                or widget.newLayer{ 
                    contentSize = config.contentSize,
                    ignoreAnchor = true,
                    -- color = ccc4(255, 100, 132, 152),
                }
                layer:addChild(node, 0, 1)
                node:setPosition(offset.left or 0, offset.up or 0)
            end
        end

        if config.parts then
            -- 加入部件
            local nums = 0
            local position = ccp(0, 0)
            
            for i, part in ipairs(config.parts) do
                local partSprite
                -- 需要设置的位置数
                nums = _count * #part.positions
                for i = 1, nums do
                    if part.category == M.PATS_CATEGORY_SPRITEFRAMES then
                        partSprite = anim.playAnimation(
                            part.params.target, 
                            part.params.filename, 
                            part.params.pattern, 
                            part.params.nums, 
                            part.params.playTime,
                            part.params.defaultDisplayIndex,
                            part.params.forever,
                            part.params.removeWhenFinished,
                            part.params.onComplete,
                            part.params.delayTime
                        )
                    elseif part.category == M.PATS_CATEGORY_SPRITE then
                        partSprite = widget.newSprite(part.params)
                    end
                    layer:addChild(partSprite, 1)
                    if i <= #part.positions then
                        position = part.positions[i]
                    else
                        position = ccpAdd(part.positions[i - #part.positions], ccp(cellSize.width, 0))
                    end
                    partSprite:setPosition(position)
                end
            end
        end
        
        return layer
    end

    -- 创建动作
    local function createMoveAnim( )
        for i, v in ipairs(_bgConfigTable) do
            if v.rate and v.rate > 0 then
                local array     = {}
                array[#array + 1] = CCMoveTo:create(_moveTime / v.rate, ccp( - v._VIEW:getContentSize().width / 2, v._VIEW:getPositionY()))
                array[#array + 1] = CCCallFunc:create(
                                        function ( ... )
                                            v._VIEW:setPosition(ccp(0, v._VIEW:getPositionY()))
                                        end
                                    )
                local action = transition.sequence(array)
                transition.execute(v._VIEW, CCRepeatForever:create(action))
            end
        end
    end
    local function addParts( bgConfig )
        local nums = 0
        local partSprite
        local position = ccp(0, 0)
        for i, part in ipairs(bgConfig.parts) do
            if part.category == M.PATS_CATEGORY_SPRITEFRAMES then
                partSprite = anim.playAnimation(
                    part.params.target, 
                    part.params.filename, 
                    part.params.pattern, 
                    part.params.nums, 
                    part.params.playTime,
                    part.params.defaultDisplayIndex,
                    part.params.forever,
                    part.params.removeWhenFinished,
                    part.params.onComplete,
                    part.params.delayTime
                )
            end
            nums = _count * #part.positions
            for i = 1, nums do
                bgConfig._VIEW:addChild(partSprite)
                if i <= #part.positions then
                    position = part.positions[i]
                else
                    position = ccpAdd(part.positions[i - part.positions], offset)
                end
                
                partSprite:setPosition(part.positions[i])
            end
            
        end
        
    end
    -- 创建背景
    local function fillParallaxNode( )
        -- 填充容器
        for i, bgConfig in ipairs(_bgConfigTable) do
            bgConfig._VIEW = createCycleMoveLayer(bgConfig)
            _parallaxNode:addChild(bgConfig._VIEW, bgConfig.zorder)
            if bgConfig.x and bgConfig.y then
                bgConfig._VIEW:setPosition(ccp(bgConfig.x, bgConfig.y))
            end
            if bgConfig.anchorPoint then
                bgConfig._VIEW:setAnchorPoint(bgConfig.anchorPoint)
            end
            -- 加上小部件
            -- addParts( bgConfig )
        end
    end
    local function init( )
        -- 创建背景
        fillParallaxNode( )
        -- 创建动画
        createMoveAnim()
        -- 动画先停停
        util.delayCall(function ( ... )
            _parallaxNode:pauseMoveAnim()
        end, 0.01)
    end
--[=[ ## public fuction --]=]
    -- 继续动作
    function _parallaxNode:resumeMoveAnim( )
        for i, v in ipairs(_bgConfigTable) do
            if v.rate and v.rate > 0 then
                CCDirector:sharedDirector():getActionManager():resumeTarget(v._VIEW)
            end
        end
    end
    -- 暂停动作
    function _parallaxNode:pauseMoveAnim( )
        for i, v in ipairs(_bgConfigTable) do
            if v.rate and v.rate > 0 then
                CCDirector:sharedDirector():getActionManager():pauseTarget(v._VIEW)
            end
            
        end
    end
    -- 取得下标层
    function _parallaxNode:getLayerByIndex( index )
        return _bgConfigTable[index]._VIEW
    end
    -- 取得主要层
    function _parallaxNode:getMainLayer( )
        for i, bgConfig in ipairs(_bgConfigTable) do
            if bgConfig.category == M.LAYER_CATEGORY_MAIN then
                return bgConfig._VIEW
            end
        end
    end
    init()
    return _parallaxNode
end

return M
