--[[ 

玩家角色形象类

--]]

local Role = require("cardui.Role2")
local Player = class("Player", Role)

-- 运动变量
Player.ACT = {}
Player.ACT.STAND            	= "stand"                   -- 站立
Player.ACT.WALK             	= "walk"                    -- 行走
Player.ACT.POSE             	= "pose"                    -- 摆POSE
Player.ACT.SAD              	= "sad"                     -- 悲伤 
Player.ACT.JUMP             	= "jump"                    -- 跳 
Player.ACT.ATTACK           	= "attack"                  -- 攻击
Player.ACT.RUN 			  		= "run" 					-- 跑
Player.ACT.SCOOP 				= "scoop" 					-- 挖
Player.ACT.SWEEP 			  	= "sweep" 					-- 扫荡


Player.RANDOM_ACT = {
	Player.ACT.POSE,
	Player.ACT.SAD,
	Player.ACT.ATTACK,
}

Player.PARTS = {}
Player.PARTS.HEAD 				= 1   						-- 头部
Player.PARTS.BODY 				= 2   						-- 胸甲
Player.PARTS.HAND 				= 3   						-- 双手
Player.PARTS.SHOES 				= 4   						-- 鞋子
Player.PARTS.WEAPON 			= 5   						-- 武器
Player.PARTS.SHIELD 			= 6   						-- 盾牌
Player.PARTS.AID 				= 7   						-- 辅助道具，生活工具

Player.PARTS.FACE  				= 101 						-- 脸
Player.PARTS.HAIR  				= 102 						-- 发型不能乱
Player.PARTS.SEX  				= 103 						-- 性别
Player.PARTS.BOOK 				= 104   					-- 书

local PARTS_INFO 				= {
	[Player.PARTS.HEAD] = {
		{ "parts_hat", sex = true }, 
		{ "parts_hair", "_hair", sex = true }, 
		{ "parts_hair_b", "_hair_b", sex = true }, 
	},
	[Player.PARTS.BODY] = {
		{ "parts_clothes" },
		{ "parts_belt", "_y" },
		{ "parts_shoulder_l", "_jl" }, 
		{ "parts_shoulder_r", "_jr" },
		{ "parts_trousers_l", "_kl" }, 
		{ "parts_trousers_r", "_kr" },
	},
	[Player.PARTS.HAND] = {
		{ "parts_glove_l", "_l" }, 
		{ "parts_glove_r", "_r" }, 
	},
	[Player.PARTS.SHOES] = {
		{ "parts_shoes_l", "_l" }, 
		{ "parts_shoes_r", "_r" }, 
	},
	[Player.PARTS.WEAPON] = {
		{ "parts_weapon" }, 
	},
	[Player.PARTS.SHIELD] = {
		{ "parts_shield" }, 
	},
	[Player.PARTS.FACE] = {
		{ "parts_face", "parts_face_%s_%s.png" }, 
	},
	[Player.PARTS.HAIR] = {
		{ "parts_hat", "parts_hat_%s_%s.png" }, 
		{ "parts_hair", "parts_hair_%s_%s.png" }, 
		{ "parts_hair_b", "parts_hair_b_%s_%s.png" }, 
	},
	[Player.PARTS.BOOK] = {
		{ "parts_book",  }, 
	},
}

local DEFAULT_WALK_SPEED 		= 100 						-- 默认行走速度
local DEFAULT_RUN_SPEED 		= 200 						-- 默认跑步速度

local SEX_FEMALE 				= "woman"
local SEX_MALE 					= "man"

local SHIELD_BOOK = { 
	["20105"] = true, 
}

--[[ 

转换装备格式为table

**Parameters:**

- 	equipment: 		table|string 		装备配置

**Returns:**

-   return table

--]]
local function convertEquipments_( equipment )
	local result
	if isstring(equipment) then 
		if "" ~= equipment then
			local equipUnits = string.split(equipment, ",")
			local equipTable = {}
			for i,v in ipairs(equipUnits) do
				if v and string.find(v, ":") then 
					local key, value = string.splitlast(v, ":")
					equipTable[tonumber(key)] = value 
				end
			end
			result = equipTable
		end
	elseif istable(equipment) then 
		result = equipment
	end
	result = result or {}
	return result
end

--[[ 

构造函数

**Parameters:**

- 	sex: 			int|enum 			性别: MALE | FEMALE
- 	head: 			string 				头发和脸的配置,如"1|3",历史问题,你懂的...
- 	equipment: 		table|string 		装备配置
- 	params: 		table 				附加的参数,不要问我干什么的,我也不知道,看父类去

--]]
function Player:ctor( sex, head, equipment, params )
	params = params or {}
	if not params.scale then params.scale = 0.8 end
	Player.super.ctor(self, 
		"role/role_player", 
		"role_player",
		params)
	self.params_ = self.params_ or {}
	self.params_.head = head
	self.params_.sex = tonumber(sex) == 1 and SEX_MALE or SEX_FEMALE
	self.params_.equipment = convertEquipments_( equipment )
	self.params_.cur_equipment = {}
	-- 化妆下
	if self.params_.head then 
		local headInfos = string.split(self.params_.head, "|") 
		self:dressUp(headInfos[1], headInfos[2], sex) 
	end
	-- 变身~~~
	if istable(self.params_.equipment) then 
		self:dress_(self.params_.sex, self.params_.equipment)
	end
	self:defaultAction()
	
	self.role:setVisible(false)
    self:performWithDelay(function (  )
    	self.role:setVisible(true)
    end, 0.1)
end

local function createEquipmentName_( id, str )
	return string.format("Img_EquipMax%d%s.png", tonumber(id), str or "")
end
--[[ 

化妆

**Parameters:**

- 	hair: 			int|string 			头发参数,[1..4]
- 	face: 			int|string 			脸部参数,你一定要说是眼睛也可以,[1..4]	

--]]
function Player:dressUp( hair, face, sex )
	self.params_.hair = hair
	self.params_.face = face
	if sex then 
		if self.params_.cur_equipment[Player.PARTS.SEX] ~= sex then 
			self.params_.cur_equipment[Player.PARTS.SEX] = sex
			self.params_.cur_equipment[Player.PARTS.FACE] = nil
			self.params_.cur_equipment[Player.PARTS.HAIR] = nil
			self.params_.sex = sex == MALE and SEX_MALE or SEX_FEMALE
			if self.params_.sex == SEX_MALE then 
				self:loadArmatureData_("role/parts_body/parts_body")
				self:changeBone("parts_body", "parts_body_man.png")
			else
				self:changeBone("parts_body", "parts_body.png")
			end
		end
	end
	-- 脸先来吧
	if self.params_.cur_equipment[Player.PARTS.FACE] ~= face then 
		self.params_.cur_equipment[Player.PARTS.FACE] = face
		self:dressSpecialPart_(Player.PARTS.FACE, face, self.params_.sex)
	end
	
	-- 然后是头发
	if istable(self.params_.equipment) and not self.params_.equipment[Player.PARTS.HEAD] then 
		if self.params_.cur_equipment[Player.PARTS.HAIR] ~= hair then 
			self.params_.cur_equipment[Player.PARTS.HAIR] = hair
			self:dressSpecialPart_(Player.PARTS.HAIR, hair, self.params_.sex)
		end
	end
end

local function isexistfile( file )
	local result = false
	try(
		function ( ... )
			local sprite = CCSprite:create(file)
			if sprite then 
				result = true
			end
		end
		)
	return result
end
--[[ 

特殊化妆部位

**Parameters:**

- 	partId: 		int 			部位id, 参看Player.PARTS

--]]
function Player:dressSpecialPart_( partId, ... )
	local partData = PARTS_INFO[partId]
	if partData then 
		for _, data in ipairs(partData) do
			local partname, framename_format = data[1], data[2]
			local framename = string.format(framename_format, ...)
			local file = string.format("role/%s/%s", partname, framename)
			if isexistfile(file) then
				local img, _ = string.splitlast(file, "%.")
				self:addArmatureFileInfo_(img, partname)
				local success = self:changeBone(partname, framename)
				if not success then 
					self:resetBone(partname)
				end
			else
				self:resetBone(partname)
			end
		end
	end
end


function Player:loadRoleArmatureData_( partNo, data )
	local partname, appendname = data[1], data[2] or ""
	-- 先判断不分性别的
	local framename = createEquipmentName_(partNo, appendname)
	local file = string.format("role/%s/%s", partname, framename)
	if not isexistfile(file) then 
		-- 居然没有,那看看区分性别的吧
		if data.sex == true then 
			framename = createEquipmentName_(partNo, string.format("%s_%s", appendname, self.params_.sex))
			file = string.format("role/%s/%s", partname, framename)
			if not isexistfile(file) then 
				framename = nil
			end
		end
	end
	-- 有存在这个图了,加载进来吧
	if framename then 
		local img, _ = string.splitlast(file, "%.")
		-- ArmatureDataManager:sharedArmatureDataManager():addArmatureFileInfo(
		-- 	img .. ".png", 
		-- 	img .. ".plist", 
		-- 	string.format("role/%s/%s.xml", partname, partname)
		-- 	)
		self:addArmatureFileInfo_(img, partname)
	end
	return framename 
end
function Player:addArmatureFileInfo_( filename, xmlname )
	ArmatureDataManager:sharedArmatureDataManager():addArmatureFileInfo(
			filename .. ".png", 
			filename .. ".plist", 
			string.format("role/%s/%s.xml", xmlname, xmlname)
			)
end
--[[ 

换装部位

**Parameters:**

- 	partId: 		int 			部位id, 参看Player.PARTS
- 	partNo: 		int 			装备编号,不设置则表示卸下

--]]
function Player:dressPart( partId, partNo, ... )
	-- 判断是不是book,是的话要当shield来处理
	if partId == Player.PARTS.SHIELD and partNo and SHIELD_BOOK[tostring(partNo)] then 
		partId = Player.PARTS.BOOK
		-- 如果装了shield,记得要扔掉先
		if self.params_.cur_equipment[Player.PARTS.SHIELD] then 
			self:resetPart(Player.PARTS.SHIELD)
		end
	end
	if partId == Player.PARTS.SHIELD and not partNo then 
		if self.params_.cur_equipment[Player.PARTS.BOOK] ~= nil then
			self:resetPart(Player.PARTS.BOOK)
		end 
	end
	if self.params_.cur_equipment[partId] ~= partNo then 
		self.params_.cur_equipment[partId] = partNo
		if partId == Player.PARTS.HEAD then
			self.params_.cur_equipment[Player.PARTS.HAIR] = nil
		end
		if partNo then
			local partData = PARTS_INFO[partId]
			if partData then 
				for _, data in ipairs(partData) do
					local framename = self:loadRoleArmatureData_(partNo, data)
					if framename then 
						local success = self:changeBone(data[1], framename)
						if not success then 
							self:resetBone(data[1])
						end
					else
						self:resetBone(data[1])
					end
				end
			end
		else
			self:resetPart(partId)
		end
	end
end
--[[ 

取得装备到身上的物品编号

**Parameters:**

- 	partId: 		int 			部位id, 参看Player.PARTS

**Returns:**

-   return 如果定义partId则返回对应的编号,无则为nil;若无partId,则全返回

--]]
function Player:getDressPart( partId )
	if isnumber(partId) then 
		return self.params_.equipment[partId]
	else
		return clone(self.params_.equipment)
	end
end
--[[ 

穿上装备

**Parameters:**

- 	equipment: 		table|string 		装备配置
-   force: 			boolean 			是否强制穿上装备,默认为false	

**Returns:**

-   return return

--]]
function Player:dressEquipments( equipment, force )
	if equipment then 
		if force then
			self.params_.cur_equipment = {} 
		end
		equipment = convertEquipments_( equipment )
		self.params_.equipment = equipment
		self:dress_(self.params_.sex, equipment)
	end
end
--[[ 

变身~~~

**Parameters:**

- 	sex: 			string  			性别: "man" | "woman"
- 	equipment: 		table|string 		装备配置 		

--]]
function Player:dress_( sex, equipment )
	local function doDress_( partId, ... )
		self:dressPart(partId, equipment[partId], ...)
	end
	-- 头部
	doDress_(Player.PARTS.HEAD)
	-- 胸甲
	-- if sex == SEX_MALE then 
	-- 	self:loadArmatureData_("role/role_parts_body")
	-- 	self:changeBone("parts_body", "parts_body_man.png")
	-- else
	-- 	self:changeBone("parts_body", "parts_body.png")
	-- end
	doDress_(Player.PARTS.BODY)
	-- 双手
	doDress_(Player.PARTS.HAND)
	-- 鞋子
	doDress_(Player.PARTS.SHOES)
	-- 武器
	doDress_(Player.PARTS.WEAPON)
	-- 盾牌
	doDress_(Player.PARTS.SHIELD)
	-- 辅助道具，生活工具
	-- doDress_(Player.PARTS.AID)
end
--[[ 

重置/卸下装备

**Parameters:**

- 	partId: 		int 			部位id, 参看Player.PARTS

--]]
function Player:resetPart( partId )
	local partData = PARTS_INFO[partId]
	if partData then 
		for _, data in ipairs(partData) do
			if data[1] then 
				self:resetBone(data[1])
			end
		end
		self.params_.cur_equipment[partId] = nil
	end
	if partId == Player.PARTS.HEAD then 
		self.params_.cur_equipment[Player.PARTS.HAIR] = nil
		self:dressSpecialPart_(Player.PARTS.HAIR, self.params_.hair, self.params_.sex)
	end
end
--[[ 

装备上挖矿行头

--]]
function Player:dressScoop()
	self.params_.cur_equipment[Player.PARTS.WEAPON] = nil
	self:addArmatureFileInfo_("role/parts_weapon/role_scoop", "parts_weapon")
	self:changeBone("parts_weapon", "role_scoop.png")
	self.params_.cur_equipment[Player.PARTS.SHIELD] = nil
	self:changeBone("parts_shield", "parts_shield.png")

	if self.params_.cur_equipment[Player.PARTS.BOOK] then 
		self:resetPart(Player.PARTS.BOOK)
	end
end
--[[ 

向左转

--]]
function Player:turnLeft()
	if self.role:getScaleX() < 0 then 
		self:flipX()
	end
end
--[[ 

向右转

--]]
function Player:turnRight()
	if self.role:getScaleX() > 0 then 
		self:flipX()
	end
end
--[[ 

来个默认动作,没事时都做这个动作

--]]
function Player:defaultAction()
	self:stand()
end
--[[ 

随机动作

--]]
function Player:randomAction()
	if not self.do_random_action then 
		self:defaultAction()
	end
	if self.state == Player.ACT.STAND then 
		local act = Player.RANDOM_ACT[math.random(1, #Player.RANDOM_ACT)]
		self.do_random_action = true
		self:do_( act, { callback = 
			function ( ... )
				self.do_random_action = nil
				self:defaultAction()
			end, 
			} 
		)
	end
end
--[[ 

站立

**Parameters:**

-   args:        table     附带参数,可不设置
							callback: 	fun(role, event, act, args)
							time:		float　		执行时间
							times:　		int 		执行次数

--]]
function Player:stand(args)
	self:do_( Player.ACT.STAND, args )
end
--[[ 

准备移动用参数,当然只有需要移动时才会用得上,仅可内部使用

**Parameters:**

- 	self: 			self 				这个不用说吧
- 	args: 			table 				被操作的参数们
- 	defaultSpeed: 	float 				默认的移动速度

**Returns:**

-   return 被准备好的参数,就这么被了

--]]
local function prepareArgsForMoveAction_( self, args, defaultSpeed )
	if args then
		-- 来看看不要不转向吧
		if tonumber(args.x) ~= 0 then  
			if args.x > 0 then 
				self:turnRight()
			else
				self:turnLeft()
			end
		end
		-- 处理下移动相关
		if tonumber(args.x) ~= 0 or tonumber(args.y) ~= 0 then 
			if not args.time and not args.speed then 
				args.speed = defaultSpeed
			end
		end
	end
	return args
end
--[[ 

行走

**Parameters:**

-   args:        table     附带参数,可不设置
							callback: 	fun(role, event, act, args)
							time:		float　		执行时间
							times:　		int 		执行次数
							speed: 		float 		跑步速度,当有设定time时无效,默认为DEFAULT_WALK_SPEED
							x: 			int 		x轴上移动距离,正数为向右,负数为向左
							y: 			int 		y轴上移动距离,正数为向上,负数为向下

--]]
function Player:walk(args)
	args = prepareArgsForMoveAction_( self, args, DEFAULT_WALK_SPEED )
	self:do_( Player.ACT.WALK, args )
end
--[[ 

跑步

**Parameters:**

-   args:        table     附带参数,可不设置
							callback: 	fun(role, event, act, args)
							time:		float　		执行时间
							times:　		int 		执行次数
							speed: 		float 		跑步速度,当有设定time时无效,默认为DEFAULT_RUN_SPEED
							x: 			int 		x轴上移动距离,正数为向右,负数为向左
							y: 			int 		y轴上移动距离,正数为向上,负数为向下

--]]
function Player:run(args)
	args = prepareArgsForMoveAction_( self, args, DEFAULT_RUN_SPEED )
	self:do_( Player.ACT.RUN, args )
end
--[[ 

扫荡跑步

**Parameters:**

-   args:        table     附带参数,可不设置
							callback: 	fun(role, event, act, args)
							time:		float　		执行时间
							times:　		int 		执行次数
							speed: 		float 		跑步速度,当有设定time时无效,默认为DEFAULT_RUN_SPEED
							x: 			int 		x轴上移动距离,正数为向右,负数为向左
							y: 			int 		y轴上移动距离,正数为向上,负数为向下

--]]
function Player:sweep(args)
	args = prepareArgsForMoveAction_( self, args, DEFAULT_RUN_SPEED )
	self:do_( Player.ACT.SWEEP, args )
end
--[[ 

耍帅

**Parameters:**

- 	callback: 	 fun(role, event, act, args) 	回调,可不设置
-   args:        table     						附带参数,可不设置
												time:		float　		执行时间
												times:　		int 		执行次数

--]]
function Player:pose(callback, args)
	if callback then 
		args = args or {}
		args.callback = callback
	end
	self:do_( Player.ACT.POSE, args)
end
--[[ 

小悲伤下

**Parameters:**

- 	callback: 	 fun(role, event, act, args) 	回调,可不设置
-   args:        table     						附带参数,可不设置
												time:		float　		执行时间
												times:　		int 		执行次数

--]]
function Player:sad(callback, args)
	if callback then 
		args = args or {}
		args.callback = callback
	end
	self:do_( Player.ACT.SAD, args )
end
--[[ 

我跳~~~

**Parameters:**

- 	callback: 	 fun(role, event, act, args) 	回调,可不设置
-   args:        table     						附带参数,可不设置
												time:		float　		执行时间
												times:　		int 		执行次数

--]]
function Player:jump(callback, args)
	args = args or {}
	args.durationTo = 0
	args.callback = callback
	self:do_( Player.ACT.JUMP, args )
end
--[[ 

打打打~~~

**Parameters:**

- 	callback: 	 fun(role, event, act, args) 	回调,可不设置
-   args:        table     						附带参数,可不设置
												time:		float　		执行时间
												times:　		int 		执行次数

--]]
function Player:attack(callback, args)
	if callback then 
		args = args or {}
		args.callback = callback
	end
	self:do_( Player.ACT.ATTACK, args )
end
--[[ 

挖挖挖~~~

**Parameters:**

- 	callback: 	 fun(role, event, act, args) 	回调,可不设置
-   args:        table     						附带参数,可不设置
												time:		float　		执行时间
												times:　		int 		执行次数

--]]
function Player:scoop(callback, args)
	if callback then 
		args = args or {}
		args.callback = callback
	end
	self:do_( Player.ACT.SCOOP, args )
end


return Player