--[[ 

角色基类

没什么好说的,反正就是基类,为具体角色提供支持

--]]
local Role = class("Role", function ()
	return widget.newLayer{
		-- color = ccc4(120, 120, 120, 120), 
		ignoreAnchor = false,
	}
end)
local TAG = "[Role]"
-- 事件变量
local EVENT_LOOP_COMPLETE       = "loopComplete"
local EVENT_COMPLETE            = "complete"
local EVENT_START               = "start"
--[[ 

构造函数

**Parameters:**

- armatureDatas:　		table|string 				骨骼的数据,只有一个时就用一个string,多个时用table,要求plist,png,xml同名,只写路径和名字不用后缀
- armatureName: 		string 						骨骼的名称
- params: 				table 						附带的参数:
													scale 		float 		缩放比例
													speedScale 	float 		动画播放速度比例

--]]
function Role:ctor( armatureDatas, armatureName, params )
	self.params_ = params or {}
	-- 解析下参数
	if isstring(armatureDatas) then 
		self:loadArmatureData_(armatureDatas)
	elseif istable(armatureDatas) then 
		for _, v in ipairs(armatureDatas) do
			self:loadArmatureData_(v)
		end
	end
	-- 开始创建真正的role对象
	self.role = Armature:create(armatureName)
	self.role = CCNodeExtend.extend(self.role)
	self:addChild(self.role)
	self:setContentSize(self.role:getContentSize())
	util.alignNode(self.role, self)
	-- 只是为了简化animation的取得
	self.roleAnimation = self.role:getAnimation()
	-- 设置下一些初始参数
	self.params_.speedScale = self.params_.speedScale or 0.4
	if istable(self.params_) then 
		-- 处理下点击 
		if self.params_.randomAction == true or isfunction(self.params_.clickListener) then 
			-- 创建点击区,加入点击事件
			local touchLayer = widget.newLayer{
				-- color = ccc4(255,0,0,120),
				contentSize = CCSize(160, 290),
				ignoreAnchor = false,
			}
			self:addChild(touchLayer, 10000, 10000)
			util.alignNode(touchLayer, self, display.BOTTOM_CENTER, { bottom = 30, } )
			local function clickListener( layer )
				self:stopAllActions()
				if self.params_.randomAction == true then 
					self:randomAction()
				else
					self.params_.clickListener(self)
				end
			end
			widget.bindTouchEvent{
				layer = touchLayer,
				clickListener = clickListener,
				swallowsTouches = true,
				-- priority = -129,
			}
		end
		-- 缩放比例
		if isnumber(self.params_.scale) then 
			self:setScale(self.params_.scale)
		end
		-- 动画播放速度(0..1]
		if isnumber(self.params_.speedScale) then 
			self:setAnimationSpeedScale(self.params_.speedScale)
		end
	end
	self:setCascadeOpacityEnabled(true)
end
--[[ 

执行默认动作,要在继承类中实现,要不鬼知道你要干什么

--]]
function Role:defaultAction()
	error("Role:defaultAction() - inherited class must override this method")
end
--[[ 

随机动作

--]]
function Role:randomAction()
	Log.i(TAG, "Role:randomAction() - inherited class must override this method")
end
--[[ 

加骨骼数据,protected

**Parameters:**

-   filename: 		string 			加载文件名称,要求plist,png,xml同名,只写路径和名字不用后缀

--]]
function Role:loadArmatureData_( filename )
	ArmatureDataManager:sharedArmatureDataManager():addArmatureFileInfo(
		filename .. ".png", 
		filename .. ".plist", 
		filename .. ".xml"
		)
end
--[[ 

注册事件

--]]
function Role:registerEventHandler()
	if not self.skeletonUtils_ then
		self.skeletonUtils_ = SkeletonUtils:create()
		self.skeletonUtils_:registerArmatureMovementEventHandler(self.role, 
			handler(self, self.eventHandler_))
	end
end
--[[ 

事件处理,protected

**Parameters:**

- role:　		Armature 			骨骼对象
- event: 		string 				事件名称
- act: 			string 				动作名称　

--]]
function Role:eventHandler_(role, event, act)
	if event == EVENT_LOOP_COMPLETE then 
		self:onLoopComplete(event, act)
	elseif event == EVENT_COMPLETE then 
		self:onComplete(event, act)
	elseif event == EVENT_START then 
		self:onStart(event, act)
	end
end
--[[ 

事件开始处理,子类中重载处理

**Parameters:**

- event: 		string 				事件名称
- act: 			string 				动作名称　

--]]
function Role:onStart( event, act )
	-- Log.d(TAG, "Role:onStart(event=%s, act=%s)", event, act)
end
--[[ 

事件循环单次处理结束,子类中重载处理,重载时记得要先执行一次基类的

**Parameters:**

- event: 		string 				事件名称
- act: 			string 				动作名称　

--]]
function Role:onLoopComplete( event, act )
	-- Log.d(TAG, "Role:onLoopComplete(event=%s, act=%s)", event, act)
	local args = self.actionArgs
	if args and args.times and not args.time then 
		args.times_ = args.times_ or args.times
		args.times_ = args.times_ - 1
		if args.singleCallback then 
			args.singleCallback(self, event, act, args)
		end
		if args.times_ <= 0 then 
			args.times_ = nil
			self:defaultAction()
			if args and args.callback then 
				args.callback(self, event, act, args)
			end
			self.actionArgs = nil
		end
	elseif args and args.time then 
		if args.singleCallback then 
			args.singleCallback(self, event, act, args)
		end
	end
end
--[[ 

事件结束处理,子类中重载处理,重载时记得要先执行一次基类的

**Parameters:**

- event: 		string 				事件名称
- act: 			string 				动作名称　

--]]
function Role:onComplete( event, act )
	-- Log.d(TAG, "Role:onComplete(event=%s, act=%s)", event, act)
	local args = self.actionArgs
	if args and isnumber(args.times) and args.times >= 1 then 
		if args.singleCallback then 
			args.singleCallback(self, event, act, args)
		end
		if args.times > 1 then 
			args.times = args.times - 1
			self:do_(act, args)
		else
			self:defaultAction()
			if args and args.callback then 
				args.callback(self, event, act, args)
				self.actionArgs = nil
			end
		end
	elseif (args and isnumber(args.time) and args.time > 0) or self.timeToDo then 
		if args and not self.timeToDo then self.timeToDo = args end
		if self.timeToDo and self.timeToDo.singleCallback then 
			self.timeToDo.singleCallback(self, event, act, self.timeToDo)
		end
		self:do_(act)
	else
		self:defaultAction()
		if args and args.callback then 
			args.callback(self, event, act, args)
			self.actionArgs = nil
		end
	end
end
--[[ 

设置动画播放速度

**Parameters:**

-   scale:        float      	动画播放速度倍率, (0 .. 1]

--]]
function Role:setAnimationSpeedScale( scale )
	self.roleAnimation:setAnimationScale(scale)
end
--[[ 

执行动作,protected

**Parameters:**

- act: 			string 				动作名称
- args: 		table 				附加属性
									callback: 			fun(role, event, act, args)
									singleCallback: 	fun(role, event, act, args) 单次运动结束后回调
									time:				float　						执行时间
									times:　				int 						执行次数
									speed: 				float 						移动速度,当有设定time时无效,仅需要移动时才需要设置
									x: 					int 						x轴上移动距离,正数为向右,负数为向左,仅需要移动时才需要设置
									y: 					int 						y轴上移动距离,正数为向上,负数为向下,仅需要移动时才需要设置

--]]
function Role:do_( act, args )
	local realdo = self.state ~= act
	self.state = act
	local durationTo = -1
	self.actionArgs = nil
	if args then 
		if args.durationTo then durationTo = args.durationTo end
		-- 首先来区分下是不是要移动,有移动的话,控制交给移动动作处理,没有的就要自己处理了...
		if (tonumber(args.x) ~= 0 or tonumber(args.y) ~= 0) and (args.time or args.speed) then 
			args.x, args.y = tonumber(args.x), tonumber(args.y)
			-- 用speed来计算下要运行的时间,当然,有time了speed就被忽视了
			if not args.time then
				local distance = 0
				if args.x ~= 0 and args.y == 0 then 
					distance = math.abs(args.x)
				elseif args.x == 0 and args.y ~= 0 then  
					distance = math.abs(args.y)
				else
					distance = math.sqrt(args.x * args.x + args.y * args.y)
				end
				args.time = distance / args.speed
			end
			self:stopAllActions()
			if args.singleCallback then 
				self:registerEventHandler() 
				self.actionArgs = args
			end
			transition.moveBy(self, {
				time = args.time,
				x = args.x, 
				y = args.y,
				onComplete = function ()
					self:defaultAction()
					if args.callback then 
						args.callback(self, EVENT_COMPLETE, act, args)
					end
					self.actionArgs = nil
				end,
				})
		else 
			if args.callback or args.times or args.time then 
				self:registerEventHandler() 
			end
			if isnumber(args.time) then 
				self:performWithDelay(function (  )
					self.timeToDo = nil
					self:defaultAction()
					self.actionArgs = nil
					if args.callback then 
						args.callback(self, EVENT_COMPLETE, act, args)
					end
				end, args.time)
			end
			self.actionArgs = args
		end
	end
	if (istable(args) and args.times) or realdo or self.timeToDo then 
		self.roleAnimation:play(act, durationTo)
	end
end

--[[ 

绑定粒子效果到指定骨骼上

**Parameters:**

- name: 			string 			名称
- particleName: 	string 			粒子文件名称(包含路径)
- boneName: 		string 			骨骼名称

**Returns:**

-   return 绑定上去的粒子效果

--]]
function Role:bindParticle( name, particleName, boneName )
	local bone = SkeletonUtils:createBoneParticle(name, particleName)
	self.role:addBone(bone, boneName)
	return bone
end
--[[ 

改变骨骼上的具体装备

**Parameters:**

- boneName: 		string 			骨骼名称
- changeName: 		string 			新装备的名称

--]]
function Role:changeBone( boneName, changeName )
	if CCSpriteFrameCache:sharedSpriteFrameCache():spriteFrameByName(changeName) then 
		local bone = self.role:getBone(boneName)
		SkeletonUtils:addBoneDisplay(bone, changeName, 1)
		SkeletonUtils:changeBoneDisplayByIndex(bone, 1, true)
		return true
	else
		Log.i(TAG, "Role:changeBone(), invalid frame, name %s", changeName)
	end
	return false
end
--[[ 

重置骨骼

**Parameters:**

- boneName: 		string 			骨骼名称

--]]
function Role:resetBone( boneName )
	local bone = self.role:getBone(boneName)
	SkeletonUtils:changeBoneDisplayByIndex(bone, 0, true)
end
--[[ 

左右翻转

--]]
function Role:flipX()
	self.role:setScaleX(-1 * self.role:getScaleX())
end
--[[ 

上下翻转

--]]
function Role:flipY()
	self.role:setScaleY(-1 * self.role:getScaleY())
end


return Role