
local ScrollLabel = {}
ScrollLabel.TAG = "[cardui.ScrollLabel]"
--[[ 

## 滚动文字

**Parameters:**

-   params: 数组包含

            [必填项]
            CCSize      contentSize            尺寸

            [选填项]
            table       labelParams            字体属性
            function    onComplete             缓存中都播放完成事件
            int         playTime               播放时间(默认: 10s)
--]]
function ScrollLabel.new( params )

--[=[ ## local params --]=]

    --#必填项#    
    local _contentSize              = params.contentSize
    Log.a(_contentSize, ScrollLabel.TAG, " ScrollLabel.new() invalid params.contentSize")

    --#选填项#
    local _labelParams          = params.labelParams or {}
    local _onComplete           = params.onComplete
    local _playTime             = params.playTime or 10
    --#内部参数#
    local _clipNode             = display.newClippingRegionNode(CCRect(0, 0, _contentSize.width, _contentSize.height))
    local _textCache            = {}
    local _textLabel
    local _isPlaying            = false
--[=[ ## local fuction --]=]
    local function doMoveAnim( )
        _isPlaying = true
        if _textLabel then
            _textLabel:removeFromParentAndCleanup(true)
        end

        if isstring(_textCache[1]) then
            _labelParams.text = _textCache[1]
            _textLabel = ui.newTTFLabel(_labelParams)
        elseif istable(_textCache[1]) then
            _textLabel = ui.newTTFLabel(_textCache[1])
        else
            _isPlaying = false
            return
        end
        
        _clipNode:addChild(_textLabel)
        _textLabel:setPosition(ccp(_clipNode:getContentSize().width + _textLabel:getContentSize().width, _clipNode:getContentSize().height/2))

        -- 计算播放时间
        local time = _playTime
        if _textLabel:getContentSize().width/_contentSize.width > 1 then
            time = time * _textLabel:getContentSize().width/_contentSize.width
        end
        local array     = {}
        array[#array + 1] = CCMoveTo:create(time, ccp(- _textLabel:getContentSize().width, _clipNode:getContentSize().height/2))
        transition.execute(_textLabel, transition.sequence(array), { onComplete = function ()
            table.remove(_textCache, 1)
            if #_textCache > 0 then
                doMoveAnim( )
            else
                _isPlaying = false
                _clipNode:setVisible(false)
                if _onComplete then
                    _onComplete(_clipNode)
                end
            end
        end })
    end
    local function init( )
        _clipNode:setVisible(false)
        -- 背景(好像不需要)
        -- local bgSprite = widget.newScale9{file = "main/img_postmessage_back.png",}
        -- bgSprite:setContentSize(CCSize(_clipNode:getContentSize().width + 30, _clipNode:getContentSize().height))
        -- _clipNode:addChild(bgSprite, 0)
        -- util.alignNode(bgSprite, _clipNode:getContentSize(), display.CENTER)
    end
--[=[ ## public fuction --]=]
    --[[ 
    
    播放文字
    
    **Parameters:**
    
    -   text:        labelParams or string     文字属性或者文字
    
    --]]
    function _clipNode.playText( text )
        -- 什么都没有, 施主请回
        if not text then
            return
        end
        table.insert(_textCache, text)
        if not _isPlaying then
            _clipNode:setVisible(true)
            doMoveAnim()
        end
    end
    -- 停止播放
    function _clipNode.stopWithCleanup( cleanup )
        _isPlaying = false
        _textLabel:stopAllActions()
        if cleanup then
            _textCache = nil
            if _textLabel then
                _textLabel:removeFromParentAndCleanup(true)
            end
        end
    end
    init()
    return _clipNode
end

return ScrollLabel