
local M = {}
M.TAG = "[cardui.ShopViewUtil]"
M.CURRENCY_TYPE_RMB     = 0
M.CURRENCY_TYPE_DIAMOND = 1
M.CURRENCY_TYPE_STAR    = 2
M.CURRENCY_TYPE_GOLD    = 3
M.CURRENCY_TYPE_91BEAN  = 4

local function getCurrencyLabel( value, ctype )
    ctype = ctype or 1
    local node = widget.newNode{contentSize = CCSize(113, 26)}
    local valueLabel = ui.newTTFLabel{
        text = value,
        size = 18,
    }
    node:addChild(valueLabel, 1)
    util.alignNode(valueLabel, node, display.CENTER_RIGHT, 10)
    local typeIcon
    if M.CURRENCY_TYPE_RMB == ctype then
        -- typeIcon = ui.newTTFLabel{text = device.language == "en" and "$" or "¥", size = 18}
        typeIcon = ui.newTTFLabel{text = "", size = 18}
    elseif M.CURRENCY_TYPE_91BEAN == ctype then
        -- typeIcon = ui.newTTFLabel{text = "91豆", size = 18}
        typeIcon = ui.newTTFLabel{text = "", size = 18}
    elseif M.CURRENCY_TYPE_DIAMOND == ctype then
        typeIcon = widget.newSprite{file = "reward/icon/icon_diamond.png",}
        typeIcon:setScale(0.5)
    elseif M.CURRENCY_TYPE_STAR == ctype then
        typeIcon = widget.newSprite{file = "reward/icon/icon_star.png",}
        typeIcon:setScale(0.5)
    elseif M.CURRENCY_TYPE_GOLD == ctype then
        typeIcon = widget.newSprite{file = "reward/icon/icon_gold.png",}
        typeIcon:setScale(0.5)
    end
    node:addChild(typeIcon, 1)
    util.alignNode(typeIcon, node, display.CENTER_LEFT, 10)
    function node:setValue( value )
        valueLabel:setString(value)
    end
    return node
end
--[[ 

## 卡牌弹出窗

--]]
local function showCardAlertView( item )
    local cardEntity = require("model.card.CardEntity").new{id = item.info.no}
    CCBCenter.loadCCB("MSG_CARDDETAILLAYER_MODEL", "ccb.c.cardgroup.CardDetailLayer", cardEntity)
end
--[[ 

## 装备弹出窗

--]]
local function showEquipAlertView( item )
    local equipEntity = require("model.goods.GoodsEntity").new(item.info)
    CCBCenter.loadCCB("MSG_EQUIPALERTLAYER_MODEL", "ccb.c.goods.EquipAlertLayer", equipEntity)
end
--[[ 

## 物品弹出窗

--]]
local function showGoodsAlertView( item )
    local goodsEntity = require("model.goods.GoodsEntity").new(item.info)
    CCBCenter.loadCCB("MSG_GOODSALERTLAYER_MODEL", "ccb.c.goods.GoodsAlertLayer", goodsEntity)
end

function M:showItemAlertView( item )
    if item.info.no > 20000 then
        showEquipAlertView(item)
    elseif item.info.no > 10000 then
        showGoodsAlertView(item)
    elseif item.info.no > 0 then
        showCardAlertView(item)
    end
end
--[[ 

## 获得物品和星星商店的图标

--]]
function M:getGoodsAndStarView( item )
    local goodsImageView
    local goodsType = item.starType
    if goodsType == STARTYPE.GOODS then
        local goods = item.info
        if goods.no > 20000 then
            if goods.pic and goods.pic ~= "" then
                goodsImageView = widget.newSprite{file = string.format(_GAME.CONFIG.GOODS.PATH, goods.pic)}
                goodsImageView:setScale(0.9)
            else
                goodsImageView = widget.newSprite{file = string.format(_GAME.CONFIG.GOODS.EQUIPICON, goods.no)}
                goodsImageView:setScale(0.9)
            end
        elseif goods.no > 10000 then
            if goods.pic and goods.pic ~= "" then
                goodsImageView = widget.newSprite{file = string.format(_GAME.CONFIG.GOODS.PATH, goods.pic)}
                goodsImageView:setScale(0.9)
            else
                goodsImageView = widget.newSprite{file = string.format(_GAME.CONFIG.GOODS.GOODSICON, goods.no)}
                goodsImageView:setScale(0.9)
            end
        elseif goods.no < 10000 then
            goodsImageView = widget.newSprite{file = string.format(CARD_ICON, goods.no)}
            -- 碎片
            if goods.no > 5000 then
                local chipSprite = widget.newSprite{file = "fight/result3/f_r3_cardchip.png"}
                goodsImageView:addChild(chipSprite)
                util.alignNode(chipSprite, goodsImageView:getContentSize(), display.CENTER)
            end
            goodsImageView:setScale(0.7)
        end
    else
        if goodsType == STARTYPE.FREEBUY then
            goodsImageView = widget.newSprite{file = "starshop/img_freebuy.png"}
        elseif goodsType == STARTYPE.SP then
            goodsImageView = widget.newSprite{file = "starshop/img_sp.png"}
        elseif goodsType == STARTYPE.EXPCARD1 then
            goodsImageView = widget.newSprite{file = "starshop/img_exp_1.png"}
        elseif goodsType == STARTYPE.EXPCARD2 then
            goodsImageView = widget.newSprite{file = "starshop/img_exp_2.png"}
        elseif goodsType == STARTYPE.EXPCARD3 then
            goodsImageView = widget.newSprite{file = "starshop/img_exp_3.png"}
        elseif goodsType == STARTYPE.CHIP then
            goodsImageView = widget.newSprite{file = "starshop/img_exp_3.png"}
        elseif goodsType == STARTYPE.FP then
            goodsImageView = widget.newSprite{file = "starshop/img_friendship.png"}
        elseif goodsType == STARTYPE.DIAMOND then
            goodsImageView = widget.newSprite{file = "main/img_sign_diamond.png"}
        elseif goodsType == STARTYPE.UNDEFINE then
            goodsImageView = widget.newSprite{file = "starshop/img_exp_3.png"}
        end
        goodsImageView:setScale(0.8)
    end
    return goodsImageView
end
--[[ 

## 创建商店物品页

--]]
function M:createShopScrollPage( params )
    local layer = params.parentNode
    local column = params.column or 6
    local indicatorHight = params.indicatorHight or 40
    local pageGrid
    local function createItemCellListener( grid, index, item )
        local gridCell = widget.newLayer{ contentSize = grid:getCellSize() }
        gridCell.VIEW_ = params.createShopGoodsViewListener( grid, index, item, gridCell )
        if gridCell.VIEW_ then
            gridCell:addChild(gridCell.VIEW_)
            util.alignNode(gridCell.VIEW_, gridCell)
            if grid.getPageGrid():isHighLight(item) then 
                gridCell.VIEW_:setSelected(true, false)
            end
        end

        gridCell:ignoreAnchorPointForPosition(false)
        return gridCell
    end
    local function modifyItemCellListener( grid, index, item, itemCell )
        -- itemCell:setString(tostring(item.index))
        local isHighLight = grid.getPageGrid():isHighLight(item)
        if not isHighLight and itemCell.VIEW_ then 
            -- 都叫你不要亮了还亮个P,给我灭掉!!!
            itemCell.VIEW_:setSelected(false)
        elseif isHighLight and not itemCell.VIEW_.light then 
            -- 要你亮时你又在这卖萌,找屎
            itemCell.VIEW_:setSelected(true)
        end
    end
    local function tapListener( grid, index, item, x, y )
        local cell = grid:cellAt(index)
        local view = cell.VIEW_
        if view then
            local image = view:getImage()
            local btnNode = view:getButton()
            local imageNode = view:getImageNode()
            if (image and util.isPointInNode(image, x, y )) 
            or ( imageNode and util.isPointInNode(imageNode, x, y )) then
                if params.onGoodsClick then
                    params.onGoodsClick( grid, index, item )
                end
            elseif util.isPointInNode(btnNode, x, y ) then
                if params.onBtnClick then
                    params.onBtnClick( grid, index, item )
                end
            end
            local pageIndex = pageGrid:getScrollLayer().getCurPageIndex()
            item.index = (pageIndex - 1)  * 12 + index
            grid.getPageGrid():highLight(pageIndex, index)
        end
    end
    pageGrid = widget.newPageGrid{
        pageParams = {
            rect = CCRect(0, 0, layer:getContentSize().width, layer:getContentSize().height),
            totalPages = 1,
            contentSize = layer:getContentSize(),
            ignoreAnchor = true,
            indicatorParams = {
                normal = "shop/container/img_shop_indicator.png",
                selected = "shop/container/img_shop_indicator_selected.png",
                height = indicatorHight,
                padding = 10,
            },
        },
        gridParams = {
            disableAutoCalcContentSize = true,
            row = 2,
            column = column,
            -- padding = {middleV = 6, left = 10, middleH = -10},
            createItemCellListener = createItemCellListener,
            modifyItemCellListener = modifyItemCellListener,
            tapListener = tapListener,
        },
    }
    layer:addChild(pageGrid)
    return pageGrid
end
--[[ 

## 创建商店物品按钮

--]]
function M:createShopGoodsView( params )
    if not params then
        params = {}
    end
    params.selectImage = params.selectImage or "shop/container/btn_shop_cell_orange_select.png"
    params.normalImage = params.normalImage or "shop/container/btn_shop_cell_orange_normal.png"
    -- 背景
    local normalImageBg = widget.newSprite{file = params.normalImage}
    local node = widget.newNode{contentSize = normalImageBg:getContentSize()}
    node:addChild(normalImageBg, 0)
    util.alignNode(normalImageBg, node)
    -- 图片
    local goodsImage
    if params.image then
        goodsImage = widget.newSprite{file = params.image}
        node:addChild(goodsImage, 1)
        util.alignNode(goodsImage, node, nil, {top = -36})
    end
    local goodsImageNode
    if params.imageNode then
        goodsImageNode = params.imageNode
        node:addChild(goodsImageNode, 1)
        util.alignNode(goodsImageNode, node, nil, {top = -36})
    end
    -- 图片上说明
    if params.imageDesc then
        local descLabel = ui.newTTFLabel{text = params.imageDesc, size = 20,}
        node:addChild(descLabel, 2)
        util.alignNode(descLabel, node, display.TOP_RIGHT, {right = 20, top = 20})
    end
    -- 物品标题
    if params.title then
        local titleLabel = ui.newTTFLabel{text = params.title, size = 16,}
        node:addChild(titleLabel, 3)
        util.alignNode(titleLabel, node, display.CENTER, {top = -4})
        local bgSprite = widget.newScale9{file = "main/bg_button_title.png", contentSize = CCSizeAdd(titleLabel:getContentSize(), 10, 0), rect = CCRect(0, 0, 72, 22),}
        node:addChild(bgSprite, 2)
        util.alignNode(bgSprite, node, display.CENTER, {top = -6})
    end
    -- 按钮
    local buySprite = widget.newSprite{file = "shop/container/btn_shop_buy.png",}
    local btnBuyNode = widget.newNode{contentSize = buySprite:getContentSize()}
    btnBuyNode:addChild(buySprite, 0)
    util.alignNode(buySprite, btnBuyNode)
    local btnBuyLabel = ui.newTTFLabel{text = _s("SHOP_BUY"), color = BUTTON_COLOR3.BROWN,}
    btnBuyNode:addChild(btnBuyLabel, 1)
    util.alignNode(btnBuyLabel, btnBuyNode)
    node:addChild(btnBuyNode, 1)
    util.alignNode(btnBuyNode, node, nil, {top = 60,})

    -- 花费文字
    local costLabel = getCurrencyLabel(params.value, params.ctype)
    node:addChild(costLabel, 1)
    util.alignNode(costLabel, node, nil, {top = 24,})
    -- 选中
    function node:setSelected( isSelect , anim )
        if anim == nil then
            anim = true
        end
        normalImageBg:removeFromParentAndCleanup(true)
        normalImageBg = widget.newSprite{file = isSelect and params.selectImage or params.normalImage,}
        node:addChild(normalImageBg, 0)
        util.alignNode(normalImageBg, node)
        if anim then
            transition.scaleTo(node, {easing = "CCEaseBackInOut", time = 0.1, scale = isSelect and 1.1 or 1})
        else
            node:setScale(isSelect and 1.1 or 1)
        end
        
    end
    -- 设置按钮文字
    function node:setButtonTTF( label )
        btnBuyLabel:setString(label)
    end
    -- 设置按钮背景
    function node:setButtonEnable( enable )
        local file = enable and "shop/container/btn_shop_buy.png" or "shop/container/btn_shop_buy_disible.png"
        buySprite:removeFromParentAndCleanup(true)
        buySprite = widget.newSprite{file = file,}
        btnBuyNode:addChild(buySprite, 0)
        util.alignNode(buySprite, btnBuyNode)
    end
    -- 取得按钮
    function node:getButton( )
        return btnBuyNode
    end
    -- 取得图片
    function node:getImage( )
        return goodsImage
    end
    -- 取得图片node
    function node:getImageNode( )
        return goodsImageNode
    end
    -- 改文字
    function node:setValue( value )
        costLabel:setValue(value)
    end
    return node
end

--[[

## 创建商店背景  

]]
function M:createShopBackgroup( params )
    if not params then
        params = {}
    end
    params.file = params.file or "bg_shop_orange.png"
    if params.parentNode then
        params.contentSize = params.contentSize or params.parentNode:getContentSize()
    else
        params.contentSize = params.contentSize or CCSize(960, 457)
    end

    display.addSpriteFramesWithFile("shop/container/bg_shop.plist", "shop/container/bg_shop.png")

    local batchNode = display.newBatchNode("shop/container/bg_shop.png", 25)
    batchNode:setContentSize(params.contentSize)
    local bgSprite = display.newSprite("#" .. params.file)

    local count = math.ceil(params.contentSize.width / bgSprite:getContentSize().width)
    for i = 1, count do
        bgSprite = display.newSprite("#" .. params.file)
        bgSprite:ignoreAnchorPointForPosition(true)
        batchNode:addChild(bgSprite)
        bgSprite:setPosition((i - 1) * (bgSprite:getContentSize().width - 1), 0)
    end
    if params.parentNode then
        params.parentNode:addChild(batchNode)
    end
    return batchNode
end
--[[

## 创建商店底部状态栏 

]]
function M:createShopPlayerStatus( params )
    local statusValues = {
        {file = "reward/icon/icon_star.png", value = "getStar"}, 
        {file = "reward/icon/icon_diamond.png", value = "getDiamond"}, 
        {file = "reward/icon/icon_gold.png", value = "getGold"}, 
        {file = "reward/icon/icon_friend_point.png", value = "getFp"},
    }

    local statusIcons = {}
    local function createStatus_( statusValue )
        local cellNode = widget.newNode{
            contentSize = CCSize(210, 60),
        }
        -- 图标
        local icon = widget.newSprite{file = statusValue.file}
        cellNode:addChild(icon)
        util.alignNode(icon, cellNode, display.CENTER_LEFT)
        -- 文字背景
        local bgCellSprite = widget.newScale9{
            file = "shop/container/bg_shop_status_value.png",
            contentSize = CCSize(150, 36),
        }
        cellNode:addChild(bgCellSprite)
        util.alignNode(bgCellSprite, cellNode, display.CENTER_RIGHT)
        -- 文字
        local statusLabel = ui.newTTFLabel{text = ME[statusValue.value](ME)}
        cellNode:addChild(statusLabel)
        util.alignNode(statusLabel, cellNode, display.CENTER_RIGHT, 10)
        statusValue.text = statusLabel

        return cellNode
    end
    local statusNode = widget.newNode{
        contentSize = CCSize(960, 62),
    }
    local bgSprite = widget.newScale9{
        file = "shop/container/bg_shop_status.png",
        contentSize = CCSize(962, 62),
        ignoreAnchor = true,
        x = -1,
        y = 0, 
    }
    statusNode:addChild(bgSprite, 0)

    local cell
    for i, v in ipairs(statusValues) do
        cell = createStatus_(v)
        statusNode:addChild(cell)
        cell:setPosition((i - 1) * cell:getContentSize().width + i * 24, 0)
    end

    if params.parentNode then
        params.parentNode:addChild(statusNode)
    end
    function statusNode:refresh( )
        for i, statusValue in ipairs(statusValues) do
            statusValue.text:setString(ME[statusValue.value](ME))
        end
    end
    return statusNode
end

--[[

## 创建商店底部状态栏仅有钻石和金币 

]]
function M:createShopGoldDiamondStatus( params )
    local statusValues = {
        -- {file = "reward/icon/icon_star.png", value = "getStar"}, 
        {file = "reward/icon/icon_diamond.png", value = "getDiamond"}, 
        {file = "reward/icon/icon_gold.png", value = "getGold"}, 
        -- {file = "reward/icon/icon_friend_point.png", value = "getFp"},
    }

    local statusIcons = {}
    local function createStatus_( statusValue )
        local cellNode = widget.newNode{
            contentSize = CCSize(210, 52),
        }
        -- 图标
        local icon = widget.newSprite{file = statusValue.file}
        cellNode:addChild(icon)
        util.alignNode(icon, cellNode, display.CENTER_LEFT)
        -- 文字背景
        local bgCellSprite = widget.newScale9{
            file = "shop/myster/bg_money.png",
            contentSize = CCSize(150, 36),
        }
        cellNode:addChild(bgCellSprite)
        util.alignNode(bgCellSprite, cellNode, display.CENTER_RIGHT)
        -- 文字
        local statusLabel = ui.newTTFLabel{text = ME[statusValue.value](ME)}
        cellNode:addChild(statusLabel)
        util.alignNode(statusLabel, cellNode, display.CENTER_RIGHT, {right = 8, bottom = -4})
        statusValue.text = statusLabel

        return cellNode
    end
    local statusNode = widget.newNode{
        contentSize = CCSize(640, 52),
    }
    -- local bgSprite = widget.newScale9{
    --     file = "shop/container/bg_shop_status.png",
    --     contentSize = CCSize(640, 52),
    --     ignoreAnchor = true,
    --     x = -1,
    --     y = 0, 
    -- }
    -- statusNode:addChild(bgSprite, 0)

    local cell
    for i, v in ipairs(statusValues) do
        cell = createStatus_(v)
        statusNode:addChild(cell)
        cell:setPosition((i - 1) * cell:getContentSize().width + i * 24, 0)
    end

    if params.parentNode then
        params.parentNode:addChild(statusNode)
    end
    function statusNode:refresh( )
        for i, statusValue in ipairs(statusValues) do
            statusValue.text:setString(ME[statusValue.value](ME))
        end
    end
    return statusNode
end

return M