--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

SP体力类，定义SP体力相关操作方法及逻辑实现。

-   存储标签[SP体力]相关信息。
-   示例：
    -- 1、发起SP体力请求
    local SpManager = require("cardui.SpManager")
    -- 发起添加sp请求
    SpManager.netAdd(fnCallback)

    ============================================================
    
]]

local M = {}



-- [包含文件]
-- ..


-- [公共参数]
-- 常量
-- ..


-- 操作变量
-- 断言
TDAssertUtils               = require("view.Fight.Util.TDAssertUtils")
-- 字符串
TDStringUtils               = require("view.Fight.Util.TDStringUtils")










-- [函数声明]
-- [网络]
-- 发送网络请求[购买体力]
local netSendAddSp
-- 接收网络请求[购买体力]
local netReceiveAddSp

-- [对话框]
-- 显示体力已满对话框
local showAlertFullSp
-- 显示购买体力对话框
local showAlertAddSp
-- 显示购买钻石对话框
local showAlertDiamond
-- [辅助]
-- 判断Sp值是否已满
local isSpFull
-- 获得当天日期
local getDateToday
-- 获得Sp体力次数
local getSpBuyTimes










-- [辅助功能]
-- [对话框]

function showAlertDiamond( )
    AlertLayer.new{
        titleStr    = _s("COMMON_NEWS_TIPS"), 
        content     = _s("BUY_DIAMOND_CONFIRM"), -- 内容
        btnStr      = { _s("COMMON_RECHARGE"), _s("COMMON_CANCEL") },          -- 按钮文字数组
        btnColor    = { AlertLayer.BTN_COLOR_BLUE, AlertLayer.BTN_COLOR_YELLOW,},
        btnClickEvent = {                   -- 按钮事件数组
            AlertLayer.buyDiamond,
        }, 
    }
end
-- 显示体力已满对话框
function showAlertFullSp()
    -- 显示[购买体力]对话框
      AlertLayer.new{
        titleStr    = _s("COMMON_NEWS_TIPS"),           -- 标题
        content     = _s("BUY_SP_FULL"), -- 内容
        btnStr      = { _s("COMMON_OK") },          -- 按钮文字数组
        btnColor    = { AlertLayer.BTN_COLOR_YELLOW, },
        -- 按钮颜色数组
        btnClickEvent = {                   -- 按钮事件数组
            function()                      -- 按钮[确定]回调
                -- print("按钮[确定]回调")
            end,
        }, 
      }
end

-- 显示购买体力对话框
function showAlertAddSp(callbackSuccess)
    -- 获得当天购买体力次数
    local times = getSpBuyTimes()
    -- 获得购买钻石数
    local spParams = TDStringUtils.split(GAMERATE:getSpDiamond(), "|")
    local cost     = spParams[1]
    local sp       = spParams[2]
    local rate     = spParams[3]

    local willCost = cost * (rate ^ (times - 1))
    -- 显示[购买体力]对话框
     AlertLayer.new4IAP{
        isUserDiamond = true,
        diamondIndex = 6,
        content = format(_s("BUY_SP_CONFIRM"), willCost, sp, times),     -- 内容
        btnClickEvent = {
            function ( )
                 -- print("按钮[确定]回调")
                -- 发起网络请请求
                if willCost > ME:getDiamond() then
                    -- 钻不够啊
                    showAlertDiamond( )
                    -- NetHelper.QuickToast(_s("SHOP_DIAMOND_NOT_ENOUGH"))
                else
                    netSendAddSp(callbackSuccess)
                end
            end,
        }
      }
end


---------


-- [辅助]
-- 判断Sp值是否已满
function isSpFull()
    return ME:getCursp() >= ME:getActlimit()
end

-- 获得当天日期
function getDateToday()
    return os.date("%m/%d/%Y")
end

-- 获得Sp体力次数
function getSpBuyTimes()
    return ME:getBuyCostCount()
end


---------


-- [网络]
-- 发送网络请求[购买体力]
function netSendAddSp(callbackSuccess)
    NetHelper.createRequest(
       ACTID.MSG_BUY_SP,                        -- 请求ID
       netReceiveAddSpSuccess,                  -- 回调函数[成功]
       {},                                      -- 请求实体
       {                                        -- 请求信息实体或参数Table
        failedCb = netReceiveAddSpFail,
        cbParams = {oldSp = ME:getCursp(), callbackSuccess = callbackSuccess, },     -- 回调参数
       }
    )
end

-- 接收网络请求[购买体力][成功]
function netReceiveAddSpSuccess(params, markProperties)
    -- [更新数值]
    --[[
    -- 1、更新SP进度条
    local sp = ME:getCursp() /  ME:getActlimit() * 100
    _ccbNode.progressBarSp:runProgressTo( 0.5 , sp )
    _ccbNode.progressBarSp:setString(ME:getCursp() .. "/" .. ME:getActlimit())
    -- 体力时间
    _ccbNode.ctlSpTime:modifyTime(ME:getSpNextTime())

    _nodeTable["lblSpTimeLabel"]:setVisible(ME:getSpNextTime() ~= 0)
    _ccbNode.ctlSpTime:setVisible(ME:getSpNextTime() ~= 0)
    _nodeTable["lblSpTimeFull"]:setVisible(ME:getSpNextTime() == 0)
    
    -- 2、更新钻石数
    _nodeTable["lblDiamond"]:setString(tostring(ME:getDiamond()))
    ]]
    



    -- 对话框提示
    AlertLayer.new4OK({
        content     = _s("BUY_SP_SUCCESS")
    })
    if markProperties.callbackSuccess then
        markProperties.callbackSuccess()
    end
end

-- 接收网络请求[购买体力][失败]
function netReceiveAddSpFail(params)
    --[[
        例子：
      AlertLayer.new{
        btnStr = {"确定","取消","充值" }, -- 按钮文字数组
        btnColor = {AlertLayer.BTN_COLOR_YELLOW,AlertLayer.BTN_COLOR_YELLOW,AlertLayer.BTN_COLOR_BLUE},
        -- 按钮颜色数组
        btnClickEvent = {
            function (  )  按钮1事件  默认为确定
              print("btn1ClickEvent")
            end,
            function (  ) 按钮2事件 默认为取消
              print("btn2ClickEvent")
            end,
            function (  ) 按钮3事件 一般为充值
              print("btn3ClickEvent")
            end
        }, -- 按钮事件数组
        titleStr = "标题",  -- 标题
        content = "内容"  -- 内容
      }

      注：全部参数都是选填，若只填一个btnStr那会只显示一个按钮
      ,若写2个则显示2个，若都不填则显示默认2个按钮，为"确定"和
      “取消”，内容默认为空，标题默认为“提示信息”


      -- 这个接口默认显示2个按钮“确定”和“取消”
      AlertLayer.new( params )
      -- 这个接口默认显示3个按钮“确定”，“取消”和“充值”
      AlertLayer.new4IAP( params )
      -- 这个接口默认显示1个按钮“取消”
      AlertLayer.new4Cancel( params )
      -- 这个接口默认显示1个按钮“确定”
      AlertLayer.new4OK( params )
    ]]
    AlertLayer.new4OK({
        content     = _s("NETWORK_CONNECT_FAILED")
    })
end










-- [函数定义]
--[[--
    发起添加SP体力请求
]]
function M.netAdd(callbackSuccess)
    if isSpFull() then
        -- 显示体力已满对话框
        showAlertFullSp()
    else
        -- 显示购买体力对话框
        showAlertAddSp(callbackSuccess)
    end
end



return M