
local M = {}
M.TAG = "[cardui.TeamViewUtil]"
local MAX_LENGTH    = 25
local CardMaster = require("cardui.CardMaster")
local CardEntity = require("model.card.CardEntity")
local STATUS_FIGHT = {
    [0] = _s("TEAM_NOT_BATTLE"),
    [1] = _s("TEAM_WAIT_BATTLE"),
    [2] = _s("TEAM_KNOCKOUT"),
    [3] = _s("TEAM_QUARTER_FINALS"),
    [4] = _s("TEAM_SEMIFINAL"),
    [5] = _s("TEAM_FINALS"),
    [6] = _s("TEAM_FINISH"),
}
M.FROM_KEY = {
    MAIN = 1,
    TEAMMAIN = 2,
    INTERSERVER = 3,
    SAMESERVER = 4,
    INTERSERVERLIST = 5,
    SAMESERVERLIST = 6,
    INTERSERVERRECORD = 7,
    SAMESERVERRECORD = 8,
    INTERSERVERRECORDLIST = 9,
    SAMESERVERRECORDLIST = 10,
}
-- 获取战斗状态文本信息
function M:getFightStatusText( status, interServer )
    local serverType = interServer and _s("TEAM_INTER_SERVER_WAR") or _s("TEAM_SAME_SERVER_WAR")
    -- 倒计时状态
    if FALSE == status then
        -- 未开赛
        return _s("TEAM_WAR_START", serverType)
    elseif 1 == status then
        -- 待战期
        return _s("TEAM_WAR_WAIT", serverType)
    elseif 6 == status then
        -- 赛季结束
        return _s("TEAM_WAR_FINISH", serverType)
    else
        -- 已开赛
        return _s("TEAM_WAR_END", serverType, STATUS_FIGHT[status])
    end
end
-- 带按钮的编辑框
function M:createEditBoxWithButton( params )
    local layer = params.parentNode
    local text = params.text or ""
    -- 背景图
    local bgSprite = widget.newScale9{
        file = "team/bg_team_text.png",
        rect = CCRect(0, 0, 90, 45),
        contentSize = CCSizeAdd(layer:getContentSize(), -80, 0),
    }
    layer:addChild(bgSprite, 0)
    util.alignNode(bgSprite, layer, display.CENTER_LEFT)
    -- 文本框
    local textLabel = ui.newTTFLabel{
        contentSize = CCSizeAdd(layer:getContentSize(), -84, -6),
        size = 28,
        text = text,
    }
    layer:addChild(textLabel, 1)
    util.alignNode(textLabel, layer, display.CENTER_LEFT, {left = 6, top = 4})
    -- 编辑框
    local editbox = widget.newEditBox{
        contentSize = CCSizeAdd(layer:getContentSize(), -84, -6),
        normalBgImage = {file = "hook_ac.png"},
        placeHolderText = _s("TEAM_EDIT_HIDE"),
        textMaxLength   = MAX_LENGTH,
    }
    editbox:setFontSize(18)
    editbox:setPlaceholderFontSize(18)
    editbox:setInputMode(kEditBoxInputModeSingleLine)
    editbox:setVisible(false)
    layer:addChild(editbox, 1)
    util.alignNode(editbox, layer, display.CENTER_LEFT, {left = 4, top = 4})
    -- 编辑按钮
    local editButton = widget.newCheckBox{
        normalImage = "common/buttons/btn_normal_yellow.png",
        selectImage = "common/buttons/btn_normal_yellow.png",
        contentSize = CCSize(80, 56),
        rect = CCRect(0, 0, 80, 58),
        scale = 1.1,
        labelParams = {text = _s("COMMON_EDIT"), color = BUTTON_COLOR3.BROWN},
        listener = function ( button )
            if button.isSelected() then
                button.setTextString(_s("COMMON_SAVE"))
                textLabel:setVisible(false)
                editbox:setVisible(true)
                editbox:setText(textLabel:getString())
                
            else
                local editString = editbox:getText()
                button.setTextString(_s("COMMON_EDIT"))
                editbox:setVisible(false)
                textLabel:setVisible(true)
                textLabel:setString(editString)
                if params.onSaveClick then
                    params.onSaveClick(editString)
                end
            end
        end,
    }
    layer:addChild(editButton, 2)
    util.alignNode(editButton, layer, display.CENTER_RIGHT)

    function editbox:reset( )
        editbox:setText("")
    end
    function editbox:setString( string )
        editbox:setText(string)
        textLabel:setString(string)
    end
    function editbox:setButtonVisible( visible )
        editButton:setVisible(visible)
    end

    return editbox
end

-- 卡牌小图列表
function M:createCardLayer( params )
    local layer = widget.newLayer()
    params.cards = params.cards or {}
    params.offset = params.offset or 10
    local cardNode, cardEntity, cardView, cardLvLabel, cardLvNode, cardLvBg
    for i = 1, 4 do
        cardNode = widget.newNode{contentSize = CCSize(91, 91), ignoreAnchor = true,}
        if i > #params.cards then
            local bgCard = widget.newSprite{file = "common/bg/bg_card_head.png"}
            local frame = widget.newSprite{file = "common/bg/bg_card_head_frame.png"}
            cardNode:addChild(bgCard, 0)
            cardNode:addChild(frame, 1)
            util.alignNode(bgCard, cardNode)
            util.alignNode(frame, cardNode)
        else
            cardEntity = CardEntity.new(params.cards[i])
            cardEntity.type = CardMaster.MINISTARCARD
            cardEntity.showLv = true
            if not params.handler then
                cardEntity.showDetail = true
            end
            cardEntity.container = params.parentNode
            cardView = CardMaster.new(cardEntity)
            cardView:setScale(0.8)
            cardNode:addChild(cardView)
            util.alignNode(cardView, cardNode)

            -- cardLvNode = widget.newNode{contentSize = CCSize(60, 30),}
            -- cardNode:addChild(cardLvNode)
            -- util.alignNode(cardLvNode, cardNode, display.RIGHT_TOP)
            -- -- cardLvBg = widget.newSprite{file = "friend/friendCardStartback.png"}
            -- -- cardLvNode:addChild(cardLvBg, 0)
            -- -- util.alignNode(cardLvBg, cardLvNode)

            -- cardLvLabel = widget.newTTFLabelWithShadow{
            --     text = "LV:" .. cardEntity:getLv(),
            --     size = 18,
            -- }
            -- cardLvNode:addChild(cardLvLabel, 1)
            -- util.alignNode(cardLvLabel, cardLvNode)
        end
        layer:addChild(cardNode)
        cardNode:setPosition( (i - 1) * (91 + params.offset), 0)
    end
    layer:setContentSize(CCSizeMul(CCSize(91 + params.offset, 91), 4, 1))
    if params.handler then
        -- 绑定点击
        widget.bindTouchEvent{
            layer = layer,
            clickListener = params.handler,
            -- container = params.container,
        }
    end
    return layer
end
return M