
local TopButtons = {}

TopButtons.INDEX_NONE = -1
TopButtons.INDEX_MAINLAYER = 1
TopButtons.INDEX_MAPLAYER = 2
TopButtons.INDEX_CARDGROUPLAYER = 3
TopButtons.INDEX_ARENASELECTLAYER = 4
TopButtons.INDEX_FRIENDLAYER = 5
TopButtons.INDEX_EQUIPLAYER = 6
TopButtons.INDEX_GOODSLAYER = 7
TopButtons.INDEX_SHOPLAYER = 8


-- create bubble button
function TopButtons.new(params)
    local _contentSize = params.contentSize or CCSize(960, 90.5)
    local _select = params.select
    local _width = _contentSize.width/8
    local _unlockLevel = require("static.unlock_level")
    -- 配置按钮
    local function configButtons( )
        -- 需要跳转到自己的页面在这里注册跳转页面的ID,PATH
        return {
                {poi = TopButtons.INDEX_MAINLAYER, title = _s("MAIN_ROLE"), id = MAINLAYER_ID, path = MAINLAYER_PATH, width = _width-1},
                {poi = TopButtons.INDEX_MAPLAYER, title = _s("MAIN_DUNGEON"), id = MAP_MESSAGE_ID, path = MAP_LAYER_ID,width = _width-1},
                {poi = TopButtons.INDEX_CARDGROUPLAYER, title = _s("MAIN_CARDGROUP"), id = "MSG_CARDGROUPLAYER_MODEL", path = "ccb.c.cardgroup.CardGroupLayer", width = _width-1},
                {poi = TopButtons.INDEX_ARENASELECTLAYER, title = _s("MAIN_ARENA"), id = "MSG_ARENASELECTLAYER_MODEL", path = "ccb.c.arena.ArenaSelectLayer",  width = _width-1},
                {poi = TopButtons.INDEX_FRIENDLAYER, title = _s("MAIN_FRIENDS"), id = "MSG_FRIENDLAYER_MODEL", path = "ccb.c.friend.FriendLayer", width = _width-1},
                {poi = TopButtons.INDEX_EQUIPLAYER, title = _s("MAIN_EQUIP"), id = "MSG_EQUIPMENTLAYER_MODEL" , path = "ccb.c.equipment.EquipmentLayer" , width = _width-1},
                {poi = TopButtons.INDEX_GOODSLAYER, title = _s("MAIN_BACKAGE"), id = "MSG_GOODSLAYER_MODEL" , path = "ccb.c.goods.GoodsLayer" , width = _width-1},
                {poi = TopButtons.INDEX_SHOPLAYER, title = _s("MAIN_SHOP"), id = "MSG_SHOPNEWLAYER_MODEL" , path = "ccb.c.shop.ShopNewLayer" , width = _width-1},
            }
    end

    local function onTab( scrollLinear, index, item ) 
        if index ~= _select then
            SOUND:playEffectButton2()
            
            -- ================== 竞技场未开放 ==================
            -- if index == 4 and ME:getLv() < LIMIT_JJC_LEVEL then 
            --     NetHelper.QuickToast(format(_s("ARENA_LEVEL_NOT_ENOUGH"), LIMIT_JJC_LEVEL))
            --     return
            -- end 
            local msgEntity = ME:getTriggerMsgEntity()
            if index == TopButtons.INDEX_ARENASELECTLAYER and not msgEntity:isUnlockArena() then
                NetHelper.QuickToast(_s("COMMON_UNOPEN", _unlockLevel.ARENA, _s("MAIN_ARENA")))
                return
            elseif index == TopButtons.INDEX_FRIENDLAYER and not msgEntity:isUnlockFriend() then
                NetHelper.QuickToast(_s("COMMON_UNOPEN", _unlockLevel.FRIEND, _s("MAIN_FRIENDS")))
                return
            elseif index == TopButtons.INDEX_EQUIPLAYER and not msgEntity:isUnlockEquip() then
                NetHelper.QuickToast(_s("COMMON_UNOPEN", _unlockLevel.EQUIP, _s("MAIN_EQUIP")))
                return
            end
            -- ================== 竞技场未开放 ==================

            util.enableCustomTouch( scrollLinear, false )
            -- 动画类型
            local transitionType = index == 1 and "SLIDEINL" or nil
            -- local transitionType = index > _select and "SLIDEINR" or "SLIDEINL"
            if (_select == TopButtons.INDEX_ARENASELECTLAYER and _select ~= TopButtons.INDEX_MAPLAYER) 
                or (_select ~= TopButtons.INDEX_ARENASELECTLAYER and _select == TopButtons.INDEX_MAPLAYER)  then
                SOUND:playMusicHome(true)
            end
            local anim = nil
            if _select > index then 
                anim = CCBCenter.ANIM_LAYER_LEFT_TO_RIGHT
            else
                anim = CCBCenter.ANIM_LAYER_RIGHT_TO_LEFT
            end
            _select = index
            -- 刷新
            scrollLinear:refresh()
            -- 替换scene
            -- CCBCenter.replaceScene(item.id, item.path, {fromMain = false, }, nil, transitionType, LOAD_DELAY_TIME)
            CCBCenter.replaceLayer(item.id, item.path, {fromMain = false, }, function (node)
                node:performWithDelay(function ( ... )
                    game.requestDateForLoad( node )
                    util.enableCustomTouch( scrollLinear, true )
                end, 0.01)
                node.onExit = function ( self )
                    util.removeUnusedTextures()
                end
            end, anim)
        end
    end

    local function onButtonClick( button ) 
        scrollLinear, index, item = button.params.scrollLinear, button.params.idx, button.params.item
        onTab(scrollLinear, index, item)
    end

    local function cellAtIndex( scrollLinear, idx , item)
        -- 创建容器
        -- local cell = display.newColorLayer(ccc4(255,100,255,100))
        local cell = widget.newLayer()
        cell:setContentSize(CCSize(item.width, 90.5))

        local textColor
        -- 选中效果
        if _select == idx then
            -- 背景
            local bgSprite = widget.newSprite{file = "topbar/bg_top_bar_select.png",}
            cell:addChild(bgSprite, 0)
            util.alignNode(bgSprite, cell:getContentSize(), display.CENTER, {top = -1})
            
            textColor = COLOR3.WHITE
            local particle = CCParticleSystemQuad:create("topbar/topFire.plist")
            local particleBatch = CCParticleBatchNode:createWithTexture(particle:getTexture())
            particleBatch:addChild(particle)
            cell:addChild(particleBatch, 1)
            particleBatch:setPosition(ccp(cell:getContentSize().width/2, 0))

        else
            -- 背景
            local bgSprite = widget.newSprite{file = "topbar/bg_top_bar_normal.png",}
            cell:addChild(bgSprite, 0)
            util.alignNode(bgSprite, cell:getContentSize(), display.CENTER, {top = -1})
            textColor = COLOR3.WHITE

            local btnSelect = widget.newButton{
                normalImage = "bg_transparent.png",
                selectImage = "topbar/bg_top_bar_select.png",
                contentSize = cell:getContentSize(),
                swallowsTouches = true,
                listener = onButtonClick,
            }
            btnSelect.params = {
                scrollLinear = scrollLinear, 
                idx = idx, 
                item = item
            }
            cell:addChild(btnSelect, 0)
            util.alignNode(btnSelect, cell:getContentSize(), display.CENTER, {top = -1, left = -1})
        end

        local imgFile = ""
        -- local offset = 0
        local padding = {}
        if item.poi == TopButtons.INDEX_MAINLAYER then
            imgFile = "topbar/img_bar_hero.png"
            -- offset = 1
            padding = {top = 1, left = -2}
        elseif item.poi == TopButtons.INDEX_MAPLAYER then
            imgFile = "topbar/img_bar_dugeon.png"
            -- offset = 3
            padding = {left = -2}
        elseif item.poi == TopButtons.INDEX_CARDGROUPLAYER then
            imgFile = "topbar/img_bar_freebuy.png"
            -- offset = 4
            padding = {top = 2, left = -2}
        elseif item.poi == TopButtons.INDEX_ARENASELECTLAYER then
            imgFile = "topbar/img_bar_arena.png"
            -- offset = 6
            padding = {top = 2, left = -2}
        elseif item.poi == TopButtons.INDEX_FRIENDLAYER then
            imgFile = "topbar/img_bar_friend.png"
            -- offset = 4
            padding = {top = 1}
        elseif item.poi == TopButtons.INDEX_SHOPLAYER then
            imgFile = "topbar/img_bar_diamond.png"
            -- offset = 9
            padding = {top = 9, left = -2}
        elseif item.poi == TopButtons.INDEX_EQUIPLAYER then
            imgFile = "topbar/img_bar_equip.png"
            -- offset = 4
            padding = {left = -2}
        elseif item.poi == TopButtons.INDEX_GOODSLAYER then
            imgFile = "topbar/img_bar_backpack.png"
            -- offset = 3
            padding = {left = -2}
        end
        -- 图标
        local imgSprite = widget.newSprite{file = imgFile,}
        cell:addChild(imgSprite, 1)
        util.alignNodeWithPadding(imgSprite, cell:getContentSize(), display.CENTER_TOP, padding)

        -- 内容
        -- local title = widget.newTTFLabelWithOutline{
        local title = ui.newTTFLabel{
                                                        text = item.title, 
                                                        shadow = true, 
                                                        font = FONT_1, 
                                                        size = 22, 
                                                        color = textColor, 
                                                        offset = 1,
                                                    }
        util.alignNode(title, cell:getContentSize(), display.CENTER_BOTTOM, 1)
        cell:addChild(title, 1)
        util.modifyPositionInAndroid( title, 0, 2 )

        -- 好友申请
        if (ME:getNfriend() == TRUE and item.poi == TopButtons.INDEX_FRIENDLAYER) 
        or (ME:getNshop() == TRUE and item.poi == TopButtons.INDEX_SHOPLAYER)
        or (ME:getNreward() == TRUE and item.poi == TopButtons.INDEX_MAINLAYER) then
            local TipTTF = ui.newTTFLabel{
                text = "new",
                color = COLOR3.GREEN,
                size = 16,
            }
            cell:addChild(TipTTF, 2)
            util.alignNodeWithPadding(TipTTF, cell:getContentSize(), display.CENTER_RIGHT , {top = -2, right = 4})
        end

        cell:ignoreAnchorPointForPosition(false)
        return cell
    end

    -- 生成
    local scrollLinear = ViewFactory.create{
                    viewType            = WIDGET_TYPE.ScrollLinear,
                    contentSize         = _contentSize,
                    cellAtIndexListener = cellAtIndex,
                    bounceable          = false,
                    contentParams       = { padding = 1, },
                    direction           = widget.TABLEVIEW_DIRECTION_HORIZONTAL,
                    -- tapListener         = onTab,
                    swallowsTouches     = true,
                }
    scrollLinear:setItems(configButtons())

    local topButtonsLayer = widget.newLayer{
        contentSize = _contentSize,
    }
    -- 背景
    local bg = widget.newSprite{file = "topbar/bg_top_bar.png", ignoreAnchor = true, }
    -- bg:setScaleY(1.25)
    bg:setPosition(ccp(0, -9))
    topButtonsLayer:addChild(bg)
    topButtonsLayer:addChild(scrollLinear)

    function topButtonsLayer.setSelect( select )
        if select == _select then return end 
        _select = select
        scrollLinear:refresh()
    end
    
    return topButtonsLayer
end

return TopButtons
