local M = {}
TAG = "[ChatUtil]"

M.ChatAll = 100
-- 系统
M.ChatSystem = 1000
-- 私人消息
M.ChatPrivate = 1001
-- 联盟
M.ChatUnion = 1002
-- 世界
M.ChatWorld = 1003

M.ChatTypeStr =
{
    [1000] = _s("CHAT_SYSTEM"),
    [1001] = _s("CHAT_PERSONAL"),
    [1002] = _s("CHAT_UNION"),
    [1003] = _s("CHAT_WORLD"),
}

-- 聊天一页最多多少条消息
M.maxMsgCountPerPage = 11

-- 最多多少条消息
M.maxMsgCount = M.maxMsgCountPerPage * 3

-- 一条消息最多多少字
M.maxSendMsgCount = 100

local MAXROWCOUNT = 1009

--[[

-   消息处理器

**Parameters:**

-   output:   table     输出消息集合
-   words:    string    主体内容

**return:**
-   strs:      table    输出消息集合
--]]
local function getChatStrs( output, words )

    --[[ 消息集合 
    -- 包含f,w 
    -- f 为0时 表示不可点击， 1为可点击
    -- w 为消息内容
    --]]
    local strs = output or {}
    -- 开始点
    local startp = 1
    -- 结束点
    local endp

    -- 循环查找"]"，直到结束
    while true do
        -- print("startp-----",startp)
        local endp =  string.find( words, "%]",startp)

        -- 若已经找不到"]",则组合字符串，为不可点击
        if endp == nil then

            if string.sub(words, startp, nil ) ~= "" then 
                -- 若前一个消息存在且为不可点击，则2个组合在一起 ,否则则另生成一个成员
                if strs[#strs] and strs[#strs].f == 0 then 
                    strs[#strs].w =  strs[#strs].w .. string.sub(words, startp, nil )
                else
                    strs[#strs + 1] =  { f = 0 , w = string.sub(words, startp, nil )}
                end 
            end

            break
        else
            -- 若找到"]"，则截取开始位到"]"的字符串
            local str = string.sub(words, startp, endp)
            -- print("str~~~",str)

            -- 该小段起始位
            local mstart = 0
            -- 该小段结束位
            local mend
            -- 
            local mstr = ""
            -- 可点击标识
            local flag = 0
            -- 在该小段寻找"[",直到结束
            while true do
                mend = string.find( str, "%[",mstart + 1)
                
                -- 若找不到"[",则开始组合
                if mend == nil then
                    -- print("mstr===",mstr)
                    -- 组合"["之前的字符串
                    if mstr ~= "" then 
                        -- 若前一个消息存在且为不可点击，则2个组合在一起 ,否则则另生成一个成员
                        if strs[#strs] and strs[#strs].f == 0 then 
                            strs[#strs].w =  strs[#strs].w .. mstr 
                        else 
                            strs[#strs + 1] =  { f = 0 , w = mstr }
                        end 

                    end 

                    -- 组合"[ ]"字符串
                    if string.sub(str, mstart, nil ) ~= "" then 
                        if flag == 0 then 
                            -- 一直找不到"["，则按不可点击组合
                            if strs[#strs] and strs[#strs].f == 0 then 
                                strs[#strs].w =  strs[#strs].w .. string.sub(str, mstart, nil ) 
                            else 
                                strs[#strs + 1] =  { f = 0 , w = string.sub(str, mstart, nil ) }
                            end 
                        else 
                            -- 组合“[]”内的字符串
                            local cstr = string.sub(str, mstart, nil )
                            if cstr and cstr ~= "[]" then 
                                -- print("cstr",cstr)
                                strs[#strs + 1] =  {f = flag , w = cstr }
                            else 
                                -- print("为空或不存在则按不可点击组合")
                                -- “[]”为空或不存在则按不可点击组合
                                if strs[#strs] and strs[#strs].f == 0 then 
                                    strs[#strs].w =  strs[#strs].w .. cstr  
                                else 
                                    strs[#strs + 1] =  { f = 0 , w = cstr }
                                end 
                            end
                            
                        end

                    end 
                    
                    break
                else
                    -- 若找到"[",则组合“[”之前的字符串,直到找到最后一个"[",标记为可点击
                    flag = 1
                    mstr = mstr .. string.sub(str, mstart, mend - 1 )
                    -- 设置起始位置为下一个(因为包含“[”,所以不加1)
                    mstart = mend 
                end
            end

            -- 设置起始位置为下一个
            startp =  endp + 1
        end
        
    end

    return strs
end

--[[

-   获得字宽度

**Parameters:**

-   word:      string    文字

**return:**
-   width:     number    宽度   
--]]

local testTTF = widget.newTTFLabelWithShadow{
    text = device.language == "en" and "a" or "字", 
    shadow = true,
    size = 30, 
    offset = 1
}
local testwidth = testTTF:getContentSize().width
local function getWordWidth( word )
    return testwidth                          
end

--[[

-   获得字数

**Parameters:**

-   word:      string    文字

**return:**
-   count:     number    字数   
--]]
local function getWordCount( word )
    local m = string.match( word, "[\128-\255]+")
    local n = string.match( word, "[%a%d_]+")
    local count = 0
    if m and m == word then 
        count = 3 -- 若是汉字 算3个
    elseif n == word then
        count = 2 -- 不是汉字 算2个
    else 
        count = -1
    end

    return count
end

local function getWordCount2( word )
    local m = string.match( word, "[\128-\255]+")
    local n = string.match( word, "[%a%d_]+")
    local count = 0
    if m and m == word then 
        count = 5 -- 若是汉字 算5个
    elseif n == word then
        count = 3 -- 不是汉字 算3个
    else 
        count = -1
    end

    return count
end

function M.getTextCount( textStr )
    local utextStr = UTF8(textStr)
    local count = 0
    if utextStr:len() < 2 then 
        return utextStr:len()
    end

    for i=1,utextStr:len() do
        local w = utextStr:sub(i,i)
        local wcount = getWordCount( tostring(w) )
        if wcount < 0 then 
            return wcount
        end
        count = count + wcount
    end

    return count
end

function M.getTextCount2( textStr )
    local utextStr = UTF8(textStr)
    local count = 0
    if utextStr:len() < 2 then 
        return utextStr:len()
    end

    for i=1,utextStr:len() do
        local w = utextStr:sub(i,i)
        local wcount = getWordCount2( tostring(w) )
        if wcount < 0 then 
            return wcount
        end
        count = count + wcount
    end

    return count
end

--[[

-   组合表格

**Parameters:**

-   wcount:    number    累计字数
-   textStr:   string    输入文本
-   vflag:     bool      是否可点击
-   items:     table     输出消息集合
-   newrow:    bool      是否新一行
-   lineWidth: number    行宽

**return:**
-   wcount:     number   累计字数    
-   items:      table    输出消息集合
-   newrow:     bool     是否新一行
--]]
local function createCellItem( wcount, textStr, vflag, items, newrow, oriText, uid, lineWidth)

    local utextStr = UTF8(textStr) -- 转化为utf8

    local flag = false
    local vtestStr = ""
    local vtestStr2 = ""
    -- local startX = wcount
    -- local endX = wcount
    lineWidth = lineWidth or MAXROWCOUNT

    for i=1,utextStr:len() do
        local w = utextStr:sub(i,i)

        local wWidth = getWordWidth( tostring(w) )

        -- 累加字宽度
        wcount = wcount + wWidth

        -- 超过一行
        if wcount > lineWidth then
            flag = true
            vtestStr2 = vtestStr2 .. tostring(w)
        else 
            vtestStr = vtestStr .. tostring(w)
            -- endX = endX + wWidth
        end

    end
    -- print("startX ",startX,"endX ",endX)

    -- if items[#items] and newrow == false then 

    --     local xxx = items[#items].x
    --     if vflag == 1 then 
    --         if xxx then 
    --             xxx[#xxx + 1] = { startX = startX, endX = endX, text = oriText} 
    --         else 
    --             xxx = {}
    --             xxx[#xxx + 1] = { startX = startX, endX = endX} 
    --         end 
    --     end

    --     items[#items] =  {title = items[#items].title .. vtestStr, flag = vflag, x = xxx }
    -- else 
    --     local xxx = {}
    --     if vflag == 1 then 
    --         xxx[#xxx + 1] = { startX = startX, endX = endX, text = oriText} 
    --     end
    --     items[#items + 1] =  {title = vtestStr, flag = vflag, x = xxx }
    -- end 


    if items[#items] and newrow == false then 
        local item = items[#items]
        item[#item + 1] =  {title = vtestStr, flag = vflag, text = oriText, uid = uid }
    else 
        local item = {}
        item[#item + 1] =  {title = vtestStr, flag = vflag, text = oriText, uid = uid }
        items[#items + 1] = item
    end 

    if flag then
        -- print("~~==================================================")
        newrow = true
        wcount,items,newrow = createCellItem( 0, vtestStr2, vflag, items, newrow, oriText, uid)
    else
        newrow = false
    end

    return wcount,items,newrow
end

--[[

-   生成表格集合

**Parameters:**

-   initItem:   table     输出消息集合
-   messages:   string    主体内容
-   lineWidth:  number    行宽

**return:**
-   items:      table    输出表格集合
--]]
function M.createCellItems( initItem, messages, lineWidth )
    
    -- 获得处理后的消息集合
    local cellItems = getChatStrs(initItem,messages)
    
    local wcount = 0  -- 字数
    local items = {} -- 生成后的表格集合
    local newrow = true -- 是否换行
    lineWidth = lineWidth or MAXROWCOUNT
    -- 循环消息集合生成表格集合

    for i=1,#cellItems do
        -- print("==================================================")

        -- 用来显示的文字，去除一些标识
        local showword = cellItems[i].w

        showword = M.filterFlag( showword )
                        
        wcount,items,newrow = createCellItem( wcount, showword, cellItems[i].f, items, newrow, cellItems[i].w, cellItems[i].o, lineWidth)
    end

    return items
end

function M.filterFlag( showword )
        -- 有卡牌标识，说明是卡牌，它的后面跟的是卡牌的等级
        if string.find( showword, GAMERATE:getChatCardFlag() .. ".+%]") then
            showword = string.gsub(showword, GAMERATE:getChatCardFlag() .. ".+%]", "]")
        end

        -- 有装备标识，说明是装备，它的后面跟的是装备的id，点击后触发请求事件
        if string.find(showword, GAMERATE:getChatEquipFlag() .. ".+%]") then 
            showword = string.gsub(showword, GAMERATE:getChatEquipFlag() .. ".+%]","]")
        end

        -- 有人名标识，说明是人名，它的后面跟的是玩家的id，点击后触发请求事件
        if string.find(showword, GAMERATE:getChatPlayerFlag() .. ".+%]") then 
            showword = string.gsub(showword, GAMERATE:getChatPlayerFlag() .. ".+%]","]")
        end

        -- 有物品标识，说明是物品，它的后面跟的是物品的编号，点击后触发请求事件
        if string.find(showword, GAMERATE:getChatGoodsFlag() .. ".+%]") then 
            showword = string.gsub(showword, GAMERATE:getChatGoodsFlag() .. ".+%]","]")
        end

        -- 有战队标识，说明是战队，它的后面跟的是战队的id，点击后触发请求事件
        if string.find(showword, GAMERATE:getChatTeamFlag() .. ".+%]") then 
            showword = string.gsub(showword, GAMERATE:getChatTeamFlag() .. ".+%]","]")
        end

        return showword
end

    -- function M.testtest()
    --     local words = "12345678[90]一二[三四五六][]][2adalds]]][[家阿123】]"
    --     -- local words = "[[[23]123[123]]]]"
    --     print(string.utf8len(words),"    ")
    --     local strs = M.getChatStrs({},words)
    --     print("strs ",json.encode(strs))

    --     print("------------------------------------")
        
    --     local items = M.createCellItems(strs)

    --     return items
    -- end

        -- for w in string.gmatch( item.title, "%[[^%u4e00-^%u9fa5]+%]") do
        --  print("index:",index," match:",w)
           --  local d = string.find(item.title, "%[什么%]")
        --  print("[]d:",d)
        -- end

return M