--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

地图难度按钮类，定义地图难度按钮相关操作方法及逻辑实现。

-   指定地图难度按钮公共方法。

]]

local M = class("MapDifficultyButton", TDNode)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
local FN_BTN_DIFFICULTY_SIMPLE          = PHI_MAP_POPUP .. "lp_d_n.png"
local FN_BTN_DIFFICULTY_ORDINARY        = PHI_MAP_POPUP .. "lp_d_h.png"
local FN_BTN_DIFFICULTY_DIFFICULT       = PHI_MAP_POPUP .. "lp_d_p.png"

-- 操作变量
-- ..










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    self:assertParameters(params)



    -- [参数赋值]
    self._fontSize          = params.fontSize                           -- 字体尺寸 
    self._kind              = params.kind                               -- 难度类型
    self._isLocked          = params.isLocked            or false       -- 解锁标识 
    self._isCurrent         = params.isCurrent           or false       -- 当前进行难度标识 
    self._handler           = params.handler                            -- 点击处理函数 


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载光效
    self:loadLight()
    -- 加载按钮
    self:loadButton()
    -- 加载锁
    self:loadLock()

    -- 加载实体渲染其他操作
    self:loadRenderOtherwise()
end

-- 加载光效
function M:loadLight()
    local sprite = display.newSprite(PHI_MAP .. "map_d_light.png")
    self:addChild(sprite)

    -- 定位
    display.align(sprite, display.CENTER)
    util.alignNode(sprite, self:getContentSize(), display.CENTER)

    -- 标记
    self.light = sprite
end

-- 加载按钮
function M:loadButton()
    local text              = self:getText()
    local handler           = self._handler
    local fileNameNormal    = self:getFileNameNormal()
    local fileNameSelect    = self:getFileNameSelect()
    local fontSize          = self._size            or 20
    local difficulty        = self._kind

    local button = widget.newButton({
        labelParams        = {text = "", size = fontSize, shadow = true , padding = {top = 0}},
        normalImage        = fileNameNormal,
        selectImage        = fileNameSelect,
        contentSize        = size,
        listener           = handler,
    })
    self:addChild(button)

    -- 定位
    util.alignNode(button, self:getContentSize(), display.CENTER)
    button.difficulty = difficulty

    -- 标记
    self.button = button
end

-- 加载锁
function M:loadLock()
    local sprite = display.newSprite(PHI_MAP .. "m_lock.png")
    self:addChild(sprite)

    -- 定位
    display.align(sprite, display.CENTER)
    util.alignNode(sprite, self:getContentSize(), display.CENTER)

    -- 标记
    self.lock = sprite
end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()
    -- 设置可见性[锁]
    self.lock:setVisible(self._isLocked)
    -- 设置可见性[光效]
    self.light:setVisible(self._isCurrent)
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()

end


---------


-- [实例方法][内部调用]
-- 获得血量
function M:getText()
    local text

    if     self._kind == DIFFICULTY_SIMPLE    then 
        text = _s("MAP_DIFFICULTY_SIMPLE")
    elseif self._kind == DIFFICULTY_ORDINARY  then 
        text = _s("MAP_DIFFICULTY_ORDINARY")
    elseif self._kind == DIFFICULTY_DIFFICULT then 
        text = _s("MAP_DIFFICULTY_DIFFICULT")
    end

    return text
end

-- 获得文件名[正常态]
function M:getFileNameNormal()
    local fileName

    if     self._kind == DIFFICULTY_SIMPLE    then 
        fileName = FN_BTN_DIFFICULTY_SIMPLE
    elseif self._kind == DIFFICULTY_ORDINARY  then 
        fileName = FN_BTN_DIFFICULTY_ORDINARY
    elseif self._kind == DIFFICULTY_DIFFICULT then 
        fileName = FN_BTN_DIFFICULTY_DIFFICULT
    end

    return fileName
end

-- 获得文件名[正常态]
function M:getFileNameSelect()
    return self:getFileNameNormal()
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数合法性(参数集)
    TDAssertUtils.assertParameters(params)
end





return M