local M = {} 

local TAG = "[MessageModel]"

local json = require("framework.shared.json")

local EntityMessage = require("ccb.m.message.EntityMessage")
local EntityTalks = require("ccb.m.message.EntityTalks")

local ChatUtil = require("ccb.m.chat.ChatUtil")
local EntityChatItem = require("ccb.m.chat.EntityChatItem")
local MsgDB = require("ccb.m.message.MsgDB")

-- 默认获取消息数量
MESSAGE_LIMIT_NUM = 50;

-- 系统消息默认 senderid
MESSAGE_SYSTEM_SENDERID = 0;
-- 消息发送失败状态
MESSAGE_SENDED_FAILED = 0;
-- 消息发送成功状态
MESSAGE_SENDED_SUCCEED = 1;
-- 消息正在发送中
MESSAGE_SENDING = 2;
-- 消息是否最新状态
MESSAGE_NEW_STATUS = 1;

-- 当前发送消息
-- local MESSAGE_SENDING = {}


-- [[ 数据库操作 ]]--

-- 初始化数据库，并打开数据库连接，
function M.openDB(  )
    -- body
    -- 初始化数据库
    if not MsgDB.isMessageDBOpen() then
        MsgDB.init();

        -- 更新所有上一次断开连接后，所有还处于发送状态的消息
        -- 将其状态改为失败
        M.updateAllSendingMessagesStatus();
    else
        MsgDB.init();
    end
    
    -- 判断是否有系统公告，如果不存在系统公告，默认写入
    -- 获取系统消息
    local sys = MsgDB.get({
                        tableName = MSGDB_TABLE_TALK ,
                        fileds    = {} ,
                        condition = "and senderId="..MESSAGE_SYSTEM_SENDERID.." order by uptime" ,
                    })

    -- print(" sys message ".. #sys);

    if not sys or #sys < 1 then
        local str = _s("MESSAGE_DEFAULT_INFO")

        if isSDK(_GAME.CONFIG.SDK.SDK_91) then 
             str = _s("MESSAGE_DEFAULT_INFO_91")
        end

        local msg   =  { 
                         {
                            senderId = MESSAGE_SYSTEM_SENDERID,
                            senderNickName = _s("MAIN_BULLETIN"),
                            receiverId = ME:getUserid(),
                            content  = str,
                            sendTime = os.time(),
                            messageType = ChatUtil.ChatSystem,
                            isSucceed = MESSAGE_SENDED_SUCCEED,
                            receiverName = ME:getName(),
                          }
                        }
        -- 写入数据库
        M.saveNewMessages({data = msg})
    end

end

-- 关闭数据库连接
function M.closeDB( )
    -- body
    MsgDB.finish();
end

-- 保存获取到的消息 
function M.saveNewMessages( params )
    -- body
    if not params or not istable(params.data) then
        -- Log.a(TAG , "saveNewMessages() invalid params data")
    end

    local _messages = params.data

    -- 判断与fid的对话是否存在
        -- 存在：删除旧的对话，插入新的对话
        -- 不存在：插入新对话
    MsgDB.insert({
            tableName = MSGDB_TABLE_MESSAGE ,
            values    = _messages
        })
    
    -- 写入对话表
    for k,v in pairs(_messages) do

            MsgDB.delete({
                        tableName = MSGDB_TABLE_TALK ,
                        condition = "senderId=" .. v.senderId ,
                    })
            MsgDB.insert({
                    tableName = MSGDB_TABLE_TALK ,
                    values    = {
                                    {
                                        senderId          = v.senderId,
                                        senderNickName    = v.senderNickName,
                                        isSucceed         = MESSAGE_SENDED_SUCCEED,
                                        isnew             = MESSAGE_NEW_STATUS,
                                        uptime            = v.sendTime,
                                        content           = v.content,
                                    },
                                }
                })

        -- if v.senderId ~= ME:getUserid() then

        -- end
    end

end



-- 保存自己发送的消息
function M.saveWhenMineTalking( msg )
    -- body
    -- if not msg or msg.receiverId==0 then
    --     -- Log.a(TAG , "saveWhenMineTalking() invalid params msg")
    -- end

    -- 判断是否已存在对话
    local talks = MsgDB.get({
                        tableName = MSGDB_TABLE_TALK ,
                        fileds    = {} ,
                        condition = "and senderId ="..msg.receiverId ,
                    })
    if not talks or #talks < 1 then

        MsgDB.delete({
                        tableName = MSGDB_TABLE_TALK ,
                        condition = "senderId=" .. msg.receiverId ,
                    })

        MsgDB.insert({
                    tableName = MSGDB_TABLE_TALK ,
                    values    = {
                                    {
                                        senderId          = msg.receiverId,
                                        senderNickName    = msg.receiverName,
                                        isSucceed         = MESSAGE_SENDED_SUCCEED,
                                        isnew             = 0,
                                        uptime            = msg.sendTime,
                                        content           = msg.content,
                                    },
                                }
                })
    end

    MsgDB.insert({
            tableName = MSGDB_TABLE_MESSAGE ,
            values    = {msg}
        })

    MsgDB.update({
                    tableName = MSGDB_TABLE_TALK ,
                    condition = "senderId="..msg.receiverId,
                    update    = " uptime="..msg.sendTime..",content='"..msg.content.."' "
                })
end


-- 从数据库获取所有会话
function M.getAllTalks( )
    -- body  
    -- 获取所有好友的对话  默认取前50条
    local talks = MsgDB.get({
                        tableName = MSGDB_TABLE_TALK ,
                        fileds    = {} ,
                        condition = "and senderId !=0 order by uptime desc" ,
                    })
    -- 获取系统消息
    local sys = MsgDB.get({
                        tableName = MSGDB_TABLE_TALK ,
                        fileds    = {} ,
                        condition = "and senderId="..MESSAGE_SYSTEM_SENDERID.." order by uptime desc limit 1" ,
                    })

    if not talks and not sys then return {} end

    local allTalks = {}

    -- 插入系统消息到第一条
    for index,value in pairs(sys) do
       local msg = EntityTalks.new(value)
       table.insert(allTalks, msg)
    end

    -- 插入所有好友对话
    for index,value in pairs(talks) do
        local msg = EntityTalks.new(value)
        table.insert(allTalks, msg)
    end

    return allTalks;
end


-- 更新是否最新、是否读取状态
function M.updateTalk4Readed( params )
    -- body

    MsgDB.update({
                    tableName = MSGDB_TABLE_TALK ,
                    condition = "senderId="..params.fid ,
                    update    = "isnew=0"
                })

end


-- 获取与好友的消息
function M.getMsgWithFriendById( params )
    -- body

    local _fid = params.fid;

    local msgs = MsgDB.get({
                        tableName = MSGDB_TABLE_MESSAGE ,
                        fileds    = {} ,
                        condition = "and senderId=".._fid.." or receiverId=".._fid.." limit 0,100",
                    })

    local allmsgs = {}

    for index,value in pairs(msgs) do
        local msg = EntityMessage.new(value)
        table.insert(allmsgs, msg)
    end

    return allmsgs;
end


-- 更新是否最新、是否读取状态
function M.updateSendedMessageStatus( senderId,receiverId,sendTime,isSucceed )
    -- body

    local status = MESSAGE_SENDED_FAILED;
    if isSucceed == 1 then status = MESSAGE_SENDED_SUCCEED end;

    MsgDB.update({
                    tableName = MSGDB_TABLE_MESSAGE ,
                    condition = "senderId="..senderId.." and sendTime="..sendTime,
                    update    = " isSucceed="..status
                })

    MsgDB.update({
                    tableName = MSGDB_TABLE_TALK ,
                    condition = "senderId="..receiverId.." and uptime="..sendTime,
                    update    = " isSucceed="..status
                })

end


-- 删除消息
function M.deleteMessage( senderId,sendTime )
    -- body
    MsgDB.delete({
            tableName = MSGDB_TABLE_MESSAGE ,
            condition = "senderId="..senderId.." and sendTime="..sendTime,
        })
end


--[[ 请求相关 ]]--


-- 获取最新消息
function M.fetchNewMessage( params )
    -- body
    local function responseItems( properties )

        if properties then

            local json = properties.items

            -- 如果获取到最新消息，才做相应操作
            if json and #json > 0 then
                -- 写入数据库
                M.saveNewMessages({data = json});

                if params.callback and isfunction(params.callback) then
                    params.callback(json)
                end 
            end

        else
            -- print("err")
        end
    end

    NetHelper.createRequest(
        
            ACTID.MSG_GET_CHATLIST,          --      请求id
            responseItems,                   --       回调函数
            { kind = ChatUtil.ChatPrivate, typ = 3, },
            {showLoading = false}
        )

end



-- 发送信息
function M.sendMessageToFriend( params )
    
    -- body
    local function responseItems( properties , cbParams)

        -- print("~~~~~~",properties);
        if cbParams then
            M.updateSendedMessageStatus(cbParams.senderId,cbParams.receiverId,cbParams.sendTime,1)
            -- for i,v in ipairs(MESSAGE_SENDING) do
            --     if v.senderId == cbParams.data.senderId and v.receiverId == cbParams.data.touserid and v.time == cbParams.data.sendTime then
            --         table.remove(MESSAGE_SENDING,i)
            --         break;
            --     end
            -- end
        end

        if params.callback and isfunction(params.callback) then
            params.callback({status = MESSAGE_SENDED_SUCCEED})
        end
    end

    local function responseItemsFailed( properties , cbParams)

        if cbParams then
            M.updateSendedMessageStatus(cbParams.senderId,cbParams.receiverId,cbParams.sendTime,0)
        
            -- for i,v in ipairs(MESSAGE_SENDING) do
            --     if v.senderId == cbParams.data.senderId and v.receiverId == cbParams.data.touserid and v.time == cbParams.data.sendTime then
            --         table.remove(MESSAGE_SENDING,i)
            --         break;
            --     end
            -- end
        end

        if params.callback and isfunction(params.callback) then
            params.callback({status = MESSAGE_SENDED_FAILED})
        end

    end

    -- -30009 给自己留言

    NetHelper.createRequest(
       ACTID.MSG_SEND_CHAT,         --      请求id
       responseItems,               --      回调函数
        {
            messagetype = ChatUtil.ChatPrivate,   -- 消息类型 默认都是世界
            content     = params.data.content,     -- 内容   
            touserid    = params.data.touserid
        },
        {
            failedCb = responseItemsFailed,
            cbParams = {senderId = params.data.senderId,
                        receiverId = params.data.touserid,
                        sendTime = params.data.sendTime},
            showLoading = false
        }                           
    )

    -- 保存当前发送
    -- local isContain = false
    -- for i,v in ipairs(MESSAGE_SENDING) do
    --     if v.senderId == params.data.senderId and v.receiverId == params.data.touserid and v.time == params.data.sendTime then
    --         isContain = true
    --     end
    -- end
    -- if not isContain then
    --     MESSAGE_SENDING[#MESSAGE_SENDING + 1] = {senderId = params.data.senderId,
    --                                              receiverId = params.data.touserid,
    --                                              time = params.data.sendTime}
    -- end
end


-- 更新所有还处于发送状态的消息
function M.updateAllSendingMessagesStatus(  )
    -- body

    local msgs = MsgDB.get({
                        tableName = MSGDB_TABLE_MESSAGE ,
                        fileds    = {} ,
                        condition = "and senderId != 0",
                    })

    for index,value in pairs(msgs) do

            -- print("######updateAllSendingMessagesStatus  value "..value.senderId);

            if value.senderId == ME:getUserid() and value.isSucceed == MESSAGE_SENDING then
                 
                    MsgDB.update({
                        tableName = MSGDB_TABLE_MESSAGE ,
                        condition = "senderId="..value.senderId.." and sendTime="..value.sendTime,
                        update    = " isSucceed="..MESSAGE_SENDED_FAILED
                        })
            end
    end
    -- 插入所有好友对话
    -- for index,value in ipairs(MESSAGE_SENDING) do

    --         print("###updateAllSendingMessagesStatus senderId : "..value.senderId.." receiverId:"..value.receiverId.." time :" .. value.time);

    --         MsgDB.update({
    --                 tableName = MSGDB_TABLE_MESSAGE ,
    --                 condition = "senderId="..value.senderId.." and sendTime="..value.time,
    --                 update    = " isSucceed="..MESSAGE_SENDED_FAILED
    --             })

    --         MsgDB.update({
    --                 tableName = MSGDB_TABLE_TALK ,
    --                 condition = "senderId="..value.receiverId.." and uptime="..value.time,
    --                 update    = " isSucceed="..MESSAGE_SENDED_FAILED
    --             })
    -- end
end



--[[ util ]]

-- 获取字符串大小
local function getWordSize( word,font,size,offset)
    local testTTF = widget.newTTFLabelWithShadow{
                                    text = word, 
                                    shadow = true,
                                    font = font,
                                    size = size, 
                                    offset = offset
                                }
    return testTTF:getContentSize()                          
end


-- 获取字符串宽度
function M.getWordWidth( word,font,size,offset)
    -- body
    local utextStr = UTF8(word) -- 转化为utf8

    local width = 0;

    local tsize = getWordSize( "字",font,size,offset)
    local tsizea_Z = getWordSize( "a",font,size,offset)
    local tsizep = getWordSize( ",",font,size,offset)

    for i=1,utextStr:len() do

        local str = tostring(utextStr:sub(i,i))
         -- .        任意字符  
         -- %a        字母  
         -- %c        控制字符  
         -- %d        数字  
         -- %l        小写字母  
         -- %p        标点字符  
         -- %s        空白符  
         -- %u        大写字母  
         -- %w        字母和数字  
         -- %x        十六进制数字  
         -- %z        代表0的字符 
         -- %A        非字母
        if string.match(str,"%w+") then
           width = width + tsizea_Z.width
        elseif string.match(str,"%p+") then
           width = width + tsizep.width
        elseif string.match(str, "[\128-\255]+") then
             width = width + tsize.width
        else
             width = width + tsize.width          
        end
    end
    return width;
end


-- 获取消息大小
function M.getMessageSize( word,font,size,offset,maxLength)

    local utextStr = UTF8(word) -- 转化为utf8
    local width = 0;
    local height = 0;
    local flag = 0;

    local tsize = getWordSize( "字",font,size,offset)
    local tsizea_Z = getWordSize( "a",font,size,offset)
    local tsizep = getWordSize( ",",font,size,offset)

    for i=1,utextStr:len() do

        local str = tostring(utextStr:sub(i,i))
         -- .        任意字符  
         -- %a        字母  
         -- %c        控制字符  
         -- %d        数字  
         -- %l        小写字母  
         -- %p        标点字符  
         -- %s        空白符  
         -- %u        大写字母  
         -- %w        字母和数字  
         -- %x        十六进制数字  
         -- %z        代表0的字符 
         -- %A        非字母
        if string.match(str,"%w+") then
           width = width + tsizea_Z.width
        elseif string.match(str,"%p+") then
           width = width + tsizep.width
        elseif string.match(str, "[\128-\255]+") then
             width = width + tsize.width
        else
             width = width + tsize.width          
        end

        -- 累加字宽度
        if width > maxLength then
            height = height + tsize.height
            width = 0
            flag = 1
        end

        if width > 0 and i == utextStr:len() then
            height = height + tsize.height
        end
    end

     if height >= display.widthInPixels*SCALE_FOR_SPECIAL_NODE then
            height = display.widthInPixels*SCALE_FOR_SPECIAL_NODE;
     end

    return iff(flag == 1,maxLength,width),height                         
end


--[[
-    如果字符串宽度超过 ＃maxWidth＃，截取字符串，并以‘...’结尾

**Parameters:**

-   str:        string    要截取的字符串
-   maxWidth:   number

**return:** 

--]]
function M.interceptMessage( str ,font,size,offset, maxWidth )
    -- body

    local utextStr = UTF8(str) -- 转化为utf8
    local result = str
    local width = 0
    local index = 1

    local tsize = getWordSize( "字",font,size,offset)
    local tsizea_Z = getWordSize( "a",font,size,offset)
    local tsizep = getWordSize( ",",font,size,offset)

    for i=1,utextStr:len() do

        local str = tostring(utextStr:sub(i,i))
         -- .        任意字符  
         -- %a        字母  
         -- %c        控制字符  
         -- %d        数字  
         -- %l        小写字母  
         -- %p        标点字符  
         -- %s        空白符  
         -- %u        大写字母  
         -- %w        字母和数字  
         -- %x        十六进制数字  
         -- %z        代表0的字符 
         -- %A        非字母
        if string.match(str,"%w+") then
           width = width + tsizea_Z.width
        elseif string.match(str,"%p+") then
           width = width + tsizep.width
        elseif string.match(str, "[\128-\255]+") then
             width = width + tsize.width
        else
             width = width + tsize.width          
        end
        if width > maxWidth then
            index = i;
            break;
        end
    end

    if index > 1 then 

        utextStr = utextStr:sub(1,index); 

        result = tostring(utextStr) .. "......";
    end 

    return result;
end



--[[ 测试 ]]--

function M.test4SendMessageToFriend(  )
    -- body

    local function callback_sendMessage(  )
        -- body
        -- print("test4SendMessageToFriend")
    end

    M.sendMessageToFriend({
                callback = callback_sendMessage,
                data     = {touserid = "11", content = "测试测试"}
        })

end

function M.test4FetchFriendMsgs( )
    -- body

    local function callback_sendMessage(  )
        -- body
        -- print("test4FetchFriendMsgs")
    end

    M.fetchFriendMsgs({
            callback = callback_sendMessage,
        })

end


return M