--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

local M = class("EntityTalent", BaseEntity) 

-- [构造方法]
-- 构造方法，定义实体初始化赋值操作。
function M:ctor(properties)
    -- [属性注入]
    self:setProperties(properties)
end

-- [辅助方法]
-- 属性定义，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.fields = {
    
    "lv", --//等级 int
    "typ", --//类型： 攻击、血、爆击等 int
    "camp"  ,    --阵营： 联盟、部落、亡灵、全阵营 int
    "curexp",    --//当前经验 int
    "nextexp",   --下等及经验  int
    "name",      --名称  str
    "title",     --说明  str
    "value",     --说明值
    "nextValue", --下等级说明  str
    "introduction", --介绍   str
    "nextNeedLv",    --下级需要的人物等级

}

-- 属性细则，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.schema = {
    
    lv    =   {"number"},
    typ    =   {"number"},
    camp = {"number"},
    curexp    =   {"number"},
    --数据库
    nextexp    =   {"number"},
    name    =   {"string"},
    title    =   {"string"},
    value    =   {"string"},
    nextValue    =   {"string"},
    introduction = {"string"}, 
    nextNeedLv = {"number"}
}





function M:getLv()
    return self.lv_
end
function M:setLv(newKey)
    self.lv_ = newKey
end

function M:getTyp()
    return self.typ_
end
function M:setTyp(newKey)
    self.typ_ = newKey
end

function M:getCamp()
    return self.camp_
end
function M:setCamp(newKey)
    self.camp_ = newKey
end

function M:getNextNeedLv()
    
    local lvvv
    if self:getLv() == 10 then
        lvvv = self:getLv()
    else 
        lvvv = self:getLv()+1
    end

    local talentDB = TalentDB.getByLvTypCamp({lv = lvvv, typ = self:getTyp(), camp = self:getCamp()})

    if talentDB then
        return talentDB.limitlv
    end
end

function M:getCurexp()
    return self.curexp_
end
function M:setCurexp(newKey)
    self.curexp_ = newKey
end

function M:getNextexp()

    local lvvv
    if self:getLv() == 10 then
        lvvv = self:getLv()
    else 
        lvvv = self:getLv()+1
    end
    local talentDB = TalentDB.getByLvTypCamp({lv = lvvv, typ = self:getTyp(), camp = self:getCamp()})

    if talentDB then
        return talentDB.needexp
    end
end
-- function M:setNextexp(newKey)
--     self.nextexp_ = newKey
-- end

function M:getName()

    
    local talentDB = TalentDB.getByLvTypCamp({lv = 1, typ = self:getTyp(), camp = self:getCamp()})

    if talentDB then
        return talentDB.name
    end
end
-- function M:setName(newKey)
--     self.name_ = newKey
-- end

function M:getTitle()
    local talentDB = TalentDB.getByLvTypCamp({lv = 1, typ = self:getTyp(), camp = self:getCamp()})

    if talentDB then
        return talentDB.introduce
    end
end
-- function M:setTitle(newKey)
--     self.title_ = newKey
-- end

function M:getValue()

    if not self:getLv() or self:getLv() == 0 then
        return 0
    end
    local talentDB = TalentDB.getByLvTypCamp({lv = self:getLv(), typ = self:getTyp(), camp = self:getCamp()})

    if talentDB then
        return talentDB.num
    end
end

function M:getNextValue()

    if not self:getLv() or self:getLv() == 10 then
        return 0
    end
    
    local talentDB = TalentDB.getByLvTypCamp({lv = self:getLv()+1, typ = self:getTyp(), camp = self:getCamp()})

    if talentDB then
        return talentDB.num
    end
end

function M:getIntroduction()
    local talentDB = TalentDB.getByLvTypCamp({lv = 1, typ = self:getTyp(), camp = self:getCamp()})

    if talentDB then
        return talentDB.meo
    end
end
-- function M:setIntroduction(newKey)
--     self.introduction_ = newKey
-- end

-- function M:getValue()
--     if self:getLv() and self:getTyp() and self:getCamp() then
--         local talent = TalentDB.getByLvTypCamp{lv = self:getLv(), typ = self:getTyp(), camp = self:getCamp()}
--         if talent then
--             return talent.Ext4
--         end
--     end
--     return 0
-- end

return M