--BigWheelLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[BigWheelLayerView]"
	local _node, _nodeTable = node, luaValue
	local ShopViewUtil = require("cardui.ShopViewUtil")
	local GoodsMaster = require("cardui.GoodsMaster")
    local GoodsEntity = require("model.goods.GoodsEntity")
    local CardEntity = require("model.card.CardEntity")
    local CardMaster = require("cardui.CardMaster")
	local REWARD_TYPE_CARD  	= 0
	local REWARD_TYPE_CHIP  	= 1
	local REWARD_TYPE_GOODS 	= 2
	local REWARD_TYPE_SP		= 3
	local REWARD_TYPE_GOLD  	= 4
	local REWARD_TYPE_FP    	= 5
	local REWARD_TYPE_DIAMOND  	= 6
	local REWARD_TYPE_FREEBUY  	= 7
	local REWARD_TYPE_STAR  	= 8
	local REWARD_TYPE_EXP   	= 9


	--左边灯
	local _lightLeft
	--右边灯
	local _lightRight
	--背景灯闪
	local _bgBigWheel 
	-- 玩家状态
	local _playerStatus
	--奖励排版界面
	local _mRewardsNode
	--奖励Grid
	local _rewardsGrid
	--黄灯
	local _light
	--目前选中的位置
	local _nowPoint = 0
	--高亮显示
	local _highLight
	--背景Laye
	local _bgLayer
	--钻石消费价格
	local _damPrice

	--灯光转动
	local function lightTurn( ... )

        --灯光闪烁
		local arrayBg     = {}
        arrayBg[#arrayBg + 1] = CCFadeTo:create(0.25, 0)
        arrayBg[#arrayBg + 1] = CCFadeTo:create(0.25, 255)
        action = CCRepeatForever:create(transition.sequence(arrayBg))
        _bgBigWheel:runAction(action)

        local arrayLight    = {}
        arrayLight[#arrayLight + 1] = CCFadeTo:create(0.5, 0)
        arrayLight[#arrayLight + 1] = CCFadeTo:create(0.5, 255)
        _highLight:runAction(CCRepeatForever:create(transition.sequence(arrayLight)))

	end

	local function setDiamondPrice( damPrice )
		_damPrice	=	damPrice
	end

	local function updateRewardInfo( data )
	 	--奖励Table
	 	local rewardTable = {} 

    	local rewardInfo = _s("REWARD_GET")		-- 卡牌
		if #data:getCards() > 0 then
			for i, card in ipairs(data:getCards()) do
				rewardTable[#rewardTable + 1] = card
				card._TYPE = REWARD_TYPE_CARD
                card._VALUE = 1
			end
		end
		-- 碎片
		if #data:getChips() > 0 then
			for i, chip in ipairs(data:getChips()) do
				rewardTable[#rewardTable + 1] = chip
				chip._TYPE = REWARD_TYPE_CHIP
                chip._VALUE = 1
			end
		end
		-- 物品或卡牌碎片等
		if #data:getBrlist() > 0 then

            local _cache = {}
			for i,v in ipairs(data:getBrlist()) do
                if v.getNo then
                    if v:getNo() < 5000 then
                        -- 绑成卡牌
                        local cardEntity = CardEntity.new{id = v:getNo()}
                        cardEntity._TYPE = REWARD_TYPE_CARD
                        cardEntity._VALUE = v:getCount()
                        rewardTable[#rewardTable + 1] = cardEntity
                    elseif v:getNo() < 10000 then
                        -- 绑成碎片
                        local cardEntity = CardEntity.new{id = v:getNo()}
                        cardEntity._TYPE = REWARD_TYPE_CHIP
                        cardEntity._VALUE = v:getCount()
                        rewardTable[#rewardTable + 1] = cardEntity
                    else
                        -- 物品
                        v._TYPE = REWARD_TYPE_GOODS
                        v._VALUE = v:getCount()
                        rewardTable[#rewardTable + 1] = v
                    end
                elseif v.getId then
                    if v:getId() < 5000 then
                        v._TYPE = REWARD_TYPE_CARD
                        v._VALUE = 1
                        rewardTable[#rewardTable + 1] = v
                    else
                        card._TYPE = REWARD_TYPE_CHIP
                        v._VALUE = 1
                        rewardTable[#rewardTable + 1] = v
                    end
                else
                    local entity
                    if v.no < 5000 then
                        -- 绑成卡牌
                        entity = CardEntity.new{id = v.no}
                        entity._TYPE = REWARD_TYPE_CARD
                    elseif v.no < 10000 then
                        -- 碎片
                        entity = CardEntity.new{id = v.no}
                        entity._TYPE = REWARD_TYPE_CHIP
                    else
                        -- 物品
                        entity = GoodsEntity.new(v)
                        entity._TYPE = REWARD_TYPE_GOODS
                    end
                    if _cache[v.no] then
                        entity = _cache[v.no]
                        entity._VALUE = entity._VALUE + v.count
                    else
                        entity._VALUE = v.count
                        rewardTable[#rewardTable + 1] = entity
                        _cache[v.no] = entity
                    end
                end
			end
		end
		-- 体力
		if data:getSp() > 0 then
			local sp = {_VALUE = data:getSp(), _TYPE = REWARD_TYPE_SP }
			rewardTable[#rewardTable + 1] = sp
		end
		-- 金币
		if data:getGold() > 0 then
			local gold = {_VALUE = data:getGold(), _TYPE = REWARD_TYPE_GOLD }
			rewardTable[#rewardTable + 1] = gold
		end
		-- 友情点
		if data:getFp() > 0 then
			local fp = {_VALUE = data:getFp(), _TYPE = REWARD_TYPE_FP }
			rewardTable[#rewardTable + 1] = fp
		end
		-- 钻石
		if data:getDiamond() > 0 then
			local diamond = {_VALUE = data:getDiamond(), _TYPE = REWARD_TYPE_DIAMOND }
			rewardTable[#rewardTable + 1] = diamond
		end
		-- 免费抽奖
		if data:getFreebuy() > 0 then
			local freebuy = {_VALUE = data:getFreebuy(), _TYPE = REWARD_TYPE_FREEBUY }
			rewardTable[#rewardTable + 1] = freebuy
		end
		-- 经验
		if data:getExp() > 0 then
			local exp = {_VALUE = data:getExp(), _TYPE = REWARD_TYPE_EXP }
			rewardTable[#rewardTable + 1] = exp
		end
		-- 星星
		if data:getStar() > 0 then
			local star = {_VALUE = data:getStar(), _TYPE = REWARD_TYPE_STAR }
			rewardTable[#rewardTable + 1] = star
		end
		return rewardTable[1]
    end

	local function setValues( rewardlst )
		
		local t = {}

		for i,v in ipairs(rewardlst) do

	  		local entity = updateRewardInfo(require("model.main.RewardEntity").new(v))
		    table.insert(t,entity)
			if i == 7 or i == 9 then
				for j=1,4 do
					table.insert(t,{})
				end
		    end
		end
		_rewardsGrid:setItems(t)
	end
	--设置按钮
	local function setButtons( DamPrice,GoldPrice )

		_node:setButtonTitle( "btnGoldBegin", _s("ACTIVITY_BIG_WHEEL_GOLD",GoldPrice))
		_node:setButtonTitle( "btnDiamondBegin", _s("ACTIVITY_BIG_WHEEL_DIAMOND",DamPrice))
		setDiamondPrice(DamPrice)
	end
	--奖励Grid
	local function createRewardsGrid( ... )

		local function createItemCellListener(grid, index, item)
			if item._TYPE then

				local cell = widget.newLayer()
	            cell:setContentSize(grid:getCellSize())
	            cell:ignoreAnchorPointForPosition(false)

	             -- 背景
	            local bgSprite = widget.newSprite{file = "goods/goodBack.png"}

	            local spriteLayer = widget.newNode{color = ccc4(255, 100, 255, 100), contentSize = bgSprite:getContentSize()}
	            cell:addChild(spriteLayer)
	            util.alignNode(spriteLayer, cell:getContentSize(), display.CENTER,{let = - 2,top = - 2})

	            spriteLayer:addChild(bgSprite, 0)
	            util.alignNode(bgSprite, spriteLayer:getContentSize(), display.CENTER)

	            local rewardSprite
	            local count = 1
	    		if item._TYPE == REWARD_TYPE_CARD then
	    			-- 卡牌
	                item.type = CardMaster.MINISTARCARD
	                item.showDetail = true
	                item.showFrame = true
	                rewardSprite = CardMaster.new(item)
	                rewardSprite:setScale(0.75)
	                count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_CHIP then
					-- 碎片
					item.type = CardMaster.MINISTARCARD
					item.showDetail = true
	                item.showFrame = true
	                rewardSprite = CardMaster.new(item)
	                item:setScale(0.75)
	                local chipSprite = widget.newSprite{file = "fight/result3/f_r3_cardchip.png"}
	                spriteLayer:addChild(chipSprite, 2)
					util.alignNode(chipSprite, spriteLayer:getContentSize(), display.CENTER)
	                count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_GOODS then
					-- 物品
	                item.type = GoodsMaster.MATERIAL
	                item.showDetail = true
	                item.showFrame = true
	                rewardSprite = GoodsMaster.new(item)
	                rewardSprite.showDetail = true
	                -- count = rewardSprite:getCount()
	                count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_SP then
					-- 体力
					rewardSprite = widget.newSprite{file = "mission/img_sign_burette.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_GOLD then
					-- 金币
					rewardSprite = widget.newSprite{file = "reward/icon/icon_gold.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_FP then
					-- 友情点
					rewardSprite = widget.newSprite{file = "main/img_sign_friendpoint.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_DIAMOND then
					-- 钻石
					rewardSprite = widget.newSprite{file = "main/img_sign_diamond.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_FREEBUY then
					-- 免费抽奖
					rewardSprite = widget.newSprite{file = "mission/img_sign_freebuy.png"}
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_STAR then
					-- 星星
					rewardSprite = widget.newSprite{file = "main/img_main_star.png"}
					rewardSprite:setScale(1.4)
					count = item._VALUE
				elseif item._TYPE == REWARD_TYPE_EXP then
					-- 经验
					rewardSprite = widget.newSprite{file = "main/img_sign_exp.png"}
					count = item._VALUE
				end
				if rewardSprite then
					spriteLayer:addChild(rewardSprite, 1)
					util.alignNode(rewardSprite, spriteLayer:getContentSize(), display.CENTER)
				end

				if count > 1 then 
					--数量
		          	local count = ui.newTTFLabel{
		                size = 30,
		                text = "x"..count
					}
		      		util.alignNode(count, cell, display.BOTTOM_RIGHT,{right = 10})
		          	cell:addChild(count)
		        end

          	return cell
          end
		end
	    --scrollview
	    local scrollGrid = widget.newGrid{
	    	contentSize = _mRewardsNode:getContentSize(),
	        cellSize = CCSize( _mRewardsNode:getContentSize().width / 6, _mRewardsNode:getContentSize().height / 4),
	        column = 6,
	        row = 4,
	        createItemCellListener = createItemCellListener,
	        selectedParams = nil,
	        direction = widget.TABLEVIEW_DIRECTION_HORIZONTAL,
	    }
	    _mRewardsNode:addChild(scrollGrid)

	    return scrollGrid
	end

	--通过point获取位置点
	local function getRowColumn( point )
		if point >= 0 and point <= 5 then 
			return 1,point + 1
		elseif point == 7 or point == 9  then
			return (point - 5) / 2 + 1,6
		elseif point == 6 or point == 8 then
			return (point - 4) / 2 + 1,1
		elseif point >= 10 and point <= 15 then
			return 4,point - 9
		end
	end

	--下一个位置点
	local function getNextRowColumn( row,column )	 
		 if row == 1 then
		 	if column + 1 <= 6 then
		 		return row,column + 1
		 	end
		 elseif row == 4 then
		 	if column - 1 > 0 then
		 		return row,column - 1
		 	end
		 end

		 if column == 6 then 
		 	if row + 1 <= 4 then
		 		return row + 1 ,column 
		 	else
		 		return row ,column - 1
		 	end
		 elseif column == 1 then
		 	if row - 1 >= 1 then
		 		return row - 1 ,column 
		 	else
		 		return row ,column + 1
		 	end
		 end
	end

	local function changeRightPoint( point )
		if point <= 5 then
			return point + 1
		elseif point == 6 then
			return 16
		elseif point == 7 then
			return point
		elseif point == 8 then
			return 15
		elseif point == 9 then
			return 8
		elseif point >= 10 and point <= 15  then
			return 24 - point
		end 
	end

	local function bingWheelRun( properties )

		local point = properties.point

		local nowRow,nowColumn = getRowColumn(_nowPoint)
		local toRow,toColumn = getRowColumn(point)
		local nowRightPoint = changeRightPoint(_nowPoint)
		local rightPoint = changeRightPoint(point)
		local goPoint = rightPoint - nowRightPoint

		if goPoint < 0 then
			goPoint =  16 + goPoint
		end

		local action

		function onAnimComplete()

			_nowPoint = point

			_highLight:setPosition(_rewardsGrid:getCellPosition(toRow,toColumn))
			_highLight:setVisible(true)

			util.delayCall(function ( )
                    
		        local rewardEntity = require("model.main.RewardEntity").new(properties.reward)
				CCBCenter.loadCCB(
			            "MSG_RECEIVEREWARDALERTLAYER_MODEL", 
		            	"ccb.c.main.ReceiveRewardAlertLayer", 
			            {
			                entity = rewardEntity,
			                onConfirm = function( )
			     --            	_nodeTable["btnGoldBegin"]:setEnabled(true)
								-- _nodeTable["btnDiamondBegin"]:setEnabled(true)
								-- _nodeTable["btnClose"]:setEnabled(true)
								util.enableCustomTouch(_node,false)
			                end
			            }
			        )

            end, 0.5) 
			_playerStatus:refresh()
		end

        local time1 = 0.5
        local time2 = 0.025
        local arrayLight     = {}
        local random = math.random(1, 2)

		for i = 1,64 + goPoint do
			local nextRow,nextColumn = getNextRowColumn(nowRow,nowColumn)
			local time
			if i <= 5 then
				time = time2 + ( 5 - i ) * 0.025
			elseif i >= 56 + goPoint - random and i < 64 + goPoint - random then
				time = time2 + ( i - 56 - goPoint + random ) * 0.025
			elseif i >= 64 + goPoint - random then
				time = 0.5 * random
			else
				time = time2
			end
			arrayLight[#arrayLight + 1] = CCMoveTo:create(time,ccp(_rewardsGrid:getCellPosition(nextRow,nextColumn)))
			nowRow = nextRow
			nowColumn = nextColumn
		end

        arrayLight[#arrayLight + 1] = CCCallFunc:create(onAnimComplete)
        
        _light:runAction(transition.sequence(arrayLight))

	end

--[[ ## local fuction --]]
	local function initViews()
		
		 --音效
        SOUND:playMusicBigWheel() 

        --背景灯闪
		_bgLayer = CCBCenter.getNodeByTag(_node, 1,"CCLayer")
		--背景灯闪
		_bgBigWheel = CCBCenter.getNodeByTag(_node, 102,"CCSprite")
		--奖励排版界面
		_mRewardsNode = CCBCenter.getNodeByTag(_node, 103)
		--移动框
		_light = CCBCenter.getNodeByTag(_node, 10301,"CCSprite")
		--移动框
		_highLight = CCBCenter.getNodeByTag(_node, 10302,"CCSprite")
		--奖励Grid
		_rewardsGrid = createRewardsGrid()

		-- 玩家状态
		_playerStatus = ShopViewUtil:createShopPlayerStatus{
            parentNode = CCBCenter.getNodeByTag(_node, 108),
        }

		lightTurn()

	end

	local function getDiamondPrice( ... )
		return _damPrice
	end

--[[ ## public fuction --]]
  	--获取装备关系id
  	_node.setValues = setValues
  	--获取装备关系id
  	_node.setButtons = setButtons
  	--获取装备关系id
  	_node.bingWheelRun = bingWheelRun
  	--钻石消费价格
  	_node.getDiamondPrice = getDiamondPrice
--[[ ## MAINs --]]
	initViews()
end

return M