--FestivalSignLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[FestivalSignLayerView]"
	local _node, _nodeTable = node, luaValue

	--时间条
	local _signBg
	--签到Node
	local _signNode
  --签到Grid
  local _signGrid
  --奖励Table
  local rewardTable = {}

  local RewardEntity = require("model.main.RewardEntity")
  local GoodsMaster = require("cardui.GoodsMaster")
  local GoodsEntity = require("model.goods.GoodsEntity")
  local CardEntity = require("model.card.CardEntity")
  local CardMaster = require("cardui.CardMaster")
  local REWARD_TYPE_CARD    = 0
  local REWARD_TYPE_CHIP    = 1
  local REWARD_TYPE_GOODS   = 2
  local REWARD_TYPE_SP    = 3
  local REWARD_TYPE_GOLD    = 4
  local REWARD_TYPE_FP      = 5
  local REWARD_TYPE_DIAMOND   = 6
  local REWARD_TYPE_FREEBUY   = 7
  local REWARD_TYPE_STAR    = 8
  local REWARD_TYPE_EXP     = 9

  -- 设置奖励内容
  local function updateRewardInfo( data )
    rewardTable = {}
    local rewardInfo = _s("REWARD_GET")   -- 卡牌
    if #data:getCards() > 0 then
      for i, card in ipairs(data:getCards()) do
        rewardTable[#rewardTable + 1] = card
        card._TYPE = REWARD_TYPE_CARD
                card._VALUE = 1
      end
    end
    -- 碎片
    if #data:getChips() > 0 then
      for i, chip in ipairs(data:getChips()) do
        rewardTable[#rewardTable + 1] = chip
        chip._TYPE = REWARD_TYPE_CHIP
                chip._VALUE = 1
      end
    end
    -- 物品或卡牌碎片等
    if #data:getBrlist() > 0 then
      local _cache = {}
      for i,v in ipairs(data:getBrlist()) do
          if v.getNo then
              if v:getNo() < 5000 then
                  -- 绑成卡牌
                  local cardEntity = CardEntity.new{id = v:getNo()}
                  cardEntity._TYPE = REWARD_TYPE_CARD
                  cardEntity._VALUE = v:getCount()
                  rewardTable[#rewardTable + 1] = cardEntity
              elseif v:getNo() < 10000 then
                  -- 绑成碎片
                  local cardEntity = CardEntity.new{id = v:getNo()}
                  cardEntity._TYPE = REWARD_TYPE_CHIP
                  cardEntity._VALUE = v:getCount()
                  rewardTable[#rewardTable + 1] = cardEntity
              else
                  -- 物品
                  v._TYPE = REWARD_TYPE_GOODS
                  v._VALUE = v:getCount()
                  rewardTable[#rewardTable + 1] = v
              end
          elseif v.getId then
              if v:getId() < 5000 then
                  v._TYPE = REWARD_TYPE_CARD
                  v._VALUE = 1
                  rewardTable[#rewardTable + 1] = v
              else
                  card._TYPE = REWARD_TYPE_CHIP
                  v._VALUE = 1
                  rewardTable[#rewardTable + 1] = v
              end
          else
              local entity
              if v.no < 5000 then
                  -- 绑成卡牌
                  entity = CardEntity.new{id = v.no}
                  entity._TYPE = REWARD_TYPE_CARD
              elseif v.no < 10000 then
                  -- 碎片
                  entity = CardEntity.new{id = v.no}
                  entity._TYPE = REWARD_TYPE_CHIP
              else
                  -- 物品
                  entity = GoodsEntity.new(v)
                  entity._TYPE = REWARD_TYPE_GOODS
              end
              if _cache[v.no] then
                  entity = _cache[v.no]
                  entity._VALUE = entity._VALUE + v.count
              else
                  entity._VALUE = v.count
                  rewardTable[#rewardTable + 1] = entity
                  _cache[v.no] = entity
              end
          end
        end
      end
      -- 体力
      if data:getSp() > 0 then
        local sp = {_VALUE = data:getSp(), _TYPE = REWARD_TYPE_SP }
        rewardTable[#rewardTable + 1] = sp
      end
      -- 金币
      if data:getGold() > 0 then
        local gold = {_VALUE = data:getGold(), _TYPE = REWARD_TYPE_GOLD }
        rewardTable[#rewardTable + 1] = gold
      end
      -- 友情点
      if data:getFp() > 0 then
        local fp = {_VALUE = data:getFp(), _TYPE = REWARD_TYPE_FP }
        rewardTable[#rewardTable + 1] = fp
      end
      -- 钻石
      if data:getDiamond() > 0 then
        local diamond = {_VALUE = data:getDiamond(), _TYPE = REWARD_TYPE_DIAMOND }
        rewardTable[#rewardTable + 1] = diamond
      end
      -- 免费抽奖
      if data:getFreebuy() > 0 then
        local freebuy = {_VALUE = data:getFreebuy(), _TYPE = REWARD_TYPE_FREEBUY }
        rewardTable[#rewardTable + 1] = freebuy
      end
      -- 经验
      if data:getExp() > 0 then
        local exp = {_VALUE = data:getExp(), _TYPE = REWARD_TYPE_EXP }
        rewardTable[#rewardTable + 1] = exp
      end
      -- 星星
      if data:getStar() > 0 then
        local star = {_VALUE = data:getStar(), _TYPE = REWARD_TYPE_STAR }
        rewardTable[#rewardTable + 1] = star
      end
    end

	--奖励grid
  local function createRewardGrid( rewardLayer )

    local function tapListener( grid, index, item )
      if item:getType() == RewardEntity.REWARD_TYPE.CARD or item:getType() == RewardEntity.REWARD_TYPE.CHIP then
          CCBCenter.loadCCB("MSG_CARDDETAILLAYER_MODEL", "ccb.c.cardgroup.CardDetailLayer", item:getEntity())
      elseif item:getType() == RewardEntity.REWARD_TYPE.GOODS then
          if item:getEntity():getNo() > 20000 then
              CCBCenter.loadCCB("MSG_EQUIPALERTLAYER_MODEL", "ccb.c.goods.EquipAlertLayer", item:getEntity())
          else
              CCBCenter.loadCCB("MSG_GOODSALERTLAYER_MODEL", "ccb.c.goods.GoodsAlertLayer", item:getEntity())
          end
      end
    end

    local function createItemCellListener(grid, index, item)

        local cell = widget.newLayer{contentSize = CCSize(rewardLayer:getContentSize().width,
        rewardLayer:getContentSize().height / 3)}
        cell:setContentSize(grid:getCellSize())
        cell:ignoreAnchorPointForPosition(false)

        cell:addChild(item)
        util.alignNode(item, cell)
        
      return cell
    end

    local scrollGrid = widget.newScrollGrid{
      x = 0,
      y = 0,
      contentSize = rewardLayer:getContentSize(),
      contentParams = {
          cellSize = CCSize(rewardLayer:getContentSize().width , rewardLayer:getContentSize().height / 3),
          column = 1,
          createItemCellListener = createItemCellListener,
          tapListener = tapListener,
      },
      direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
      parentNode = rewardLayer,
    }

    scrollGrid:setItems(rewardTable)
  end

  --签到列表
	local function createSignGrid( signTable )
    local cellLength
    if #signTable > 7 then
      cellLength = 7
    else
      cellLength = #signTable
    end 

		local function createItemCellListener(grid, index, item)
      local cell = widget.newLayer{contentSize = CCSize(_signNode:getContentSize().width / cellLength,_signNode:getContentSize().height)}

       -- 背景
      local rewardBg = widget.newScale9{
          file = "festivalsign/bg_sign_reward_nomal.png",
          rect = CCRectMake(0, 0, 112, 103) , 
          contentSize = CCSize(110,355)
      }
      util.alignNode(rewardBg, cell:getContentSize(), display.BOTTOM_CENTER)
      cell:addChild(rewardBg, 0)	

       --时间点
      local timeImg = display.newSprite("festivalsign/bg_festival_sign_1.png")
      timeImg:setPosition(rewardBg:getPositionX(), rewardBg:getPositionY() + rewardBg:getContentSize().height + timeImg:getContentSize().height / 2)
      cell:addChild(timeImg)  

      local initScale,isDisabled,i = 1,true,0

      if item.status == 3 or item.status == 1 then
      	initScale = 1.3
        if item.status == 1 then
      	   isDisabled = false
        end
      	i = 10
      end

      --领取奖励按钮
      local btnGetReward =  widget.newButton{
        	initScale = initScale,
            scale = initScale + 0.1,
            normalImage = "festivalsign/btn_festival_sign.png",
            selectImage = "festivalsign/btn_festival_sign.png",
            labelParams = {  
            	size = 20,
            	color = BUTTON_COLOR3.BROWN,
            	text = _s("ACTIVITY_FESTIVAL_SIGN_TITLE",item.tag),
            	padding = { top = - 5}
            },
            listener = function ( button )
                HelperView2.showNext()
                CCBCenter.sendLuaMessage(MSG_ID.MSG_FESTIVALSIGNLAYER_ONACTIONFETCHGETREWARD,nil,{tag = item.tag})
            end
      }
      btnGetReward.setStateDisabled(isDisabled)
      btnGetReward:setPosition(timeImg:getPositionX(),
      	timeImg:getPositionY() + timeImg:getContentSize().height + btnGetReward:getContentSize().height * initScale / 4 + i)
      
      if item.status == 2 or item.status == 3 then
          local signedImg = display.newSprite("festivalsign/img_festival_signed.png")
          util.alignNode(signedImg,btnGetReward)
          btnGetReward:addChild(signedImg,10000) 
      end

      if item.status == 3 or item.status == 1 then
      	local array     = {}
        array[#array + 1] = CCScaleTo:create(0.2 , initScale, initScale + 0.1)
        array[#array + 1] = CCScaleTo:create(0.2 , initScale, initScale )
        
        btnGetReward:runAction(CCRepeatForever:create(transition.sequence(array)))
      end
      
      cell:addChild(btnGetReward)

      --奖励layer
      local rewardLayer = widget.newLayer{contentSize = CCSize(96,318)}
      util.alignNode(rewardLayer, rewardBg,display.BOTTOM_CENTER,{bottom = 5})
      rewardBg:addChild(rewardLayer)  

      --奖励串
      local rewardEntity = RewardEntity.new(item.reward)
      rewardTable = rewardEntity:getRewardViews(rewardEntity)
      createRewardGrid(rewardLayer)


      return cell
    end

    local scrollGrid = widget.newScrollGrid{
      x = 0,
      y = 0,
      contentSize = _signNode:getContentSize(),
      contentParams = {
          cellSize = CCSize( _signNode:getContentSize().width / cellLength, _signNode:getContentSize().height),
          row = 1,
          createItemCellListener = createItemCellListener,
      },
      direction = widget.TABLEVIEW_DIRECTION_HORIZONTAL,
      parentNode = _signNode,
    }

    return scrollGrid
	end

	local function drawBg( properties )

    _signNode:removeAllChildrenWithCleanup(true)
    _signBg:setVisible(true)
    _nodeTable["mRuleLabel"]:setVisible(true)

    _nodeTable["mTitleLabel"]:setVisible(true)
    if properties.str and isstring(properties.str) and string.len(properties.str) > 0 then

      _nodeTable["mTitleLabel"]:setString(properties.str)
    end

    local width
    if 120 * #properties.lst > 960 then 
      width = 960
    else
      width = 120 * #properties.lst
    end
    _signBg:setContentSize(CCSize(width ,18))
    _signNode:setContentSize(CCSize(width , 565))

    _signGrid = createSignGrid(properties.lst)

    _signGrid:setItems(properties.lst)

	end

--[[ ## local fuction --]]
  local function checkHelperInfo( )
    HelperView2.showBetween(_node, nil, {930, 930}, {940, 940})
  end
	local function initViews()
    checkHelperInfo( )
    _signBg =  CCBCenter.getNodeByTag(_node, 3,"CCScale9Sprite")

    _signNode = CCBCenter.getNodeByTag(_node , 4)
	end

--[[ ## public fuction --]]
  _node.drawBg = drawBg

--[[ ## MAINs --]]
	initViews()
  _node.checkHelperInfo = checkHelperInfo
end

return M