--EquipAlertLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[EquipAlertLayerView]"
	local _node, _nodeTable = node, luaValue
    local _goodsEntity = _node._DATA
    local _type = _node._TYPE or 1
    local ChatUtil = require("ccb.m.chat.ChatUtil")
    local _scrollGrid
    local _scrollContentSize
    local ALERT_TYPE_NORMAL = 1
    local ALERT_TYPE_USE = 2
    local ALERT_TYPE_OFF = 3
    local TEXT_SIZE = 16
    local CELL_SIZE = 20
--[[ ## local fuction --]]
    local function createScrollGrid( )
        local node = CCBCenter.getNodeByTag(_node, 10402)
        local function cellAtIndex( scrollGrid, idx , item)
            local cell = widget.newLayer()
            cell:setContentSize(scrollGrid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)

            cell:addChild(item)
            util.alignNode(item, cell)

            return cell
        end
        local scrollGrid = widget.newScrollGrid{
            contentSize = _scrollContentSize,
            contentParams = {
                cellSize = CCSize(node:getContentSize().width, CELL_SIZE),
                column = 1,
                createItemCellListener = cellAtIndex,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
            parentNode = node,
        }
        return scrollGrid
    end
    -- 说明模式
    local function addDetailView( string )
        local node = CCBCenter.getNodeByTag(_node, 10402)
        node:removeAllChildrenWithCleanup(true)
        local scrollView = widget.newScrollView{
            -- bounceable = false, 
            contentSize = node:getContentSize(), 
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL, 
        }
        node:addChild(scrollView)
        local textLabel = ui.newTTFLabel{text = "" , color = COLOR_WHITE, size = 18}
        local size = scrollView:getContentSize()
        textLabel:setDimensions(CCSize(size.width, 0))
        textLabel:setVerticalAlignment(ui.TEXT_VALIGN_TOP)
        textLabel:setHorizontalAlignment(ui.TEXT_ALIGN_LEFT)
        textLabel:setString(string)
        scrollView:setContainer(textLabel)
        scrollView:updateInset()
        util.autoFixScrollView(scrollView)
    end
    -- 图标
    local function addGoodsIcon( file )
        local node = CCBCenter.getNodeByTag(_node, 1030102)
        local sprite = widget.newSprite{file = file}
        node:addChild(sprite, 1)
        util.alignNode(sprite, node:getContentSize(), display.CENTER)
    end
    -- 图标背景
    local function addGoodsIconBG( file )
        local node = CCBCenter.getNodeByTag(_node, 1030102)
        local bgSprite = widget.newSprite{ file = file}
        node:addChild(bgSprite, 0)
        util.alignNode(bgSprite, node:getContentSize(), display.CENTER)
    end
    -- 取得文本View
    local function getTextTitleView( params )
        local node = widget.newLayer{contentSize = _scrollGrid:getGrid():getCellSize()}
        params.size = params.size or TEXT_SIZE
        local ttfLabel = ui.newTTFLabel(params)
        node:addChild(ttfLabel)
        util.alignNode(ttfLabel, node, display.CENTER_LEFT)
        return node
    end
    -- 取得套装View
    local function getSetView( set )
        local node = widget.newLayer{contentSize = _scrollGrid:getGrid():getCellSize()}
        local name = EquipDB.getByID(set.setid).name
        local nameLabel = ui.newTTFLabel{text = string.format("    %s", name), size = TEXT_SIZE, color = set.ison == TRUE and COLOR3.WHITE or BUTTON_COLOR3.GREE }
        node:addChild(nameLabel)
        util.alignNode(nameLabel, node, display.CENTER_LEFT)
        return node
    end
    -- 取得装备技能
    local function getSkillsInfoView( skills )
        local skillsId = skills.id
        local skillsLv = skills.lv
        local isact = skills.isact

        local skillEntity = EquipSkillDB.getSkillEntity(skillsId, skillsLv)
        if skillEntity then
            local node = widget.newLayer{contentSize = _scrollGrid:getGrid():getCellSize()}
            local name = skillEntity:getName()
            local color = COLOR3.GREEN
            local lv   = skillEntity:getLevel()
            local infoText = ""
            if isact == 1 then
                infoText = string.format(_s("EQUIP_SET_SKILL"), name, lv)
            elseif 2 <= isact and isact <= 6 then
                infoText = string.format(_s("EQUIP_SET_SKILL_POS", isact, name, lv))
                color = BUTTON_COLOR3.GREE
            elseif 12 <= isact and isact <= 16 then
                infoText = string.format(_s("EQUIP_SET_SKILL_POS", isact - 10, name, lv))
                -- color = COLOR3.GREEN
            end

            local nameLabel = ui.newTTFLabel{text = infoText, size = TEXT_SIZE, color = color }
            node:addChild(nameLabel)
            util.alignNode(nameLabel, node, display.CENTER_LEFT)
            return node
        else
            return skillEntity
        end
    end
    -- 取得属性View
    local function getProsInfoView( pro )
        local node = widget.newLayer{contentSize = _scrollGrid:getGrid():getCellSize()}
        local infoText = ""
        local effect = pro.typ
        local race = pro.camp
        local value = pro.value
        local isact = pro.isact
        local color = COLOR3.WHITE

        local typText = TALENT_EFFECT_ATK == effect and _s("EQUIP_ATTACK") or _s("EQUIP_HP")
        local raceText
        if RACE_ALL == race then
            raceText = _s("EQUIP_ALLRACE")
        elseif RACE_ALLIANCE == race then
            raceText = _s("CARD_RACE_ALLIANCE")
        elseif RACE_TRIBE == race then
            raceText = _s("CARD_RACE_TRIBE")
        elseif RACE_UNDEAD == race then
            raceText = _s("CARD_RACE_UNDEAD")
        elseif RACE_HERO == race then
            raceText = _s("EQUIP_HERO")
        elseif RACE_MAIN == race then
            raceText = _s("CARD_MAIN")
        end
        if effect and race and value then
            infoText = string.format("%s%s + %s", raceText, typText, value)
            if 2 <= isact and isact <= 6 then
                infoText = string.format(_s("EQUIP_SET_POS", isact, infoText))
                color = BUTTON_COLOR3.GREE
            elseif 12 <= isact and isact <= 16 then
                infoText = string.format(_s("EQUIP_SET_POS", isact - 10, infoText))
                color = COLOR3.GREEN
            end
            local prosLabel = ui.newTTFLabel{size = TEXT_SIZE, text = infoText, color = color}
            node:addChild(prosLabel)
            util.alignNode(prosLabel, node, display.CENTER_LEFT)
        end
        
        return node
    end
    -- 数据更新
    local function updateGoodsInfo( )
        -- 物品名
        local goodsName = _goodsEntity:getName()
        if goodsName then
            _nodeTable["mEquipNameLabel"]:setString(goodsName)
            _nodeTable["mEquipNameLabel"]:setColor(_GAME.CONFIG.GOODS.QUALITY[_goodsEntity:getQuality()].rgb)
        end
        -- 追加
        local add = _goodsEntity:getAdd()
        if add and add > 0 then
            goodsName = string.format("%s + %d", goodsName, add)
            _nodeTable["mEquipNameLabel"]:setString(goodsName)
        end
        -- 类型
        local goodsType = _goodsEntity:getPartString()
        if goodsType then
            _nodeTable["mTypeLabel"]:setString(string.format(_s("GOODS_TYPE"), goodsType))
        end
        -- 等级
        local goodsLv = _goodsEntity:getStrlv()
        if goodsLv then
            _nodeTable["mLevelLabel"]:setString(string.format(_s("GOODS_LEVEL"), goodsLv))
        end
        -- 进阶等级
        local evolv = _goodsEntity:getEvolv()
        if evolv then
            _nodeTable["mAddLabel"]:setString(string.format(_s("GOODS_EVO_LEVEL"), evolv))
        end
        
        -- 售价
        local goodsPrice = _goodsEntity:getPrice()
        if goodsPrice then
            _nodeTable["mSellLabel"]:setString(string.format(_s("GOODS_PRICE"), goodsPrice))
        end
        -- 图标
        local goodsIcon = _goodsEntity:getGoodsIcon()
        if goodsIcon then
            addGoodsIcon(goodsIcon)
        end
        -- 背景图
        local bgGoodsIcon = _GAME.CONFIG.GOODS.QUALITY[_goodsEntity:getQuality()].frame
        if bgGoodsIcon then
            addGoodsIconBG(bgGoodsIcon)
        end

        local infoViews = {}
        -- 普通属性
        -- infoViews[#infoViews + 1] = getTextTitleView{text = _s("EQUIP_EFFECT"), size = TEXT_SIZE}
        local prosTable = _goodsEntity:getPros()
        for i, v in ipairs(prosTable) do
            if v.isact == 1 then
                infoViews[#infoViews + 1] = getProsInfoView(v)
            end
        end
        -- 装备技能
        local skillsTable = _goodsEntity:getSkills()
        for i, v in ipairs(skillsTable) do
            if v.isact == 1 then
                infoViews[#infoViews + 1] = getSkillsInfoView(v)
            end
        end
        -- 套装
        local setidsTable = _goodsEntity:getSetids()
        local setId = _goodsEntity:getSetid()
        local setDb = EquipSetDB.getByID(setId)
        if #setidsTable > 0 and setDb then
            local ison = 0
            -- 已经装备数量
            for i, v in ipairs(setidsTable) do
                if v.ison == TRUE then
                    ison = ison + 1
                end
            end
            -- 套装名
            local setName = setDb.name
            if setName then
                infoViews[#infoViews + 1] = getTextTitleView{text = _s("EQUIP_SET", setName, ison, #setidsTable), color = COLOR3.YELLOW, }
            end
            -- 套装部位
            for i, v in ipairs(setidsTable) do
                infoViews[#infoViews + 1] = getSetView(v)
            end
        end
        -- 套装属性
        for i, v in ipairs(prosTable) do
            if v.isact > 1 then
                infoViews[#infoViews + 1] = getProsInfoView(v)
            end
        end
        -- 套装技能
        for i, v in ipairs(skillsTable) do
            if v.isact > 1 then
                infoViews[#infoViews + 1] = getSkillsInfoView(v)
            end
        end
        if _goodsEntity:getNo() >= 29000 then
            -- 说明
            local goodsDetail = _goodsEntity:getDetails()
            if goodsDetail then
                addDetailView(goodsDetail)
            end
        else
            -- 滚动
            _scrollGrid:setItems(infoViews)
            util.autoFixScrollView(_scrollGrid)
        end
    end

    -- 封装聊天发送装备名字      
    local function getChatSendRealEquipName( equipname, eid, pid, quality)
        if equipname == nil then return "" end
        if eid == nil or pid == nil or quality == nil or not isnumber(eid) or not isnumber(pid) or not isnumber(quality) or quality < 1 or quality > 5 then return equipname end
        if eid <= 0 or pid <= 0 then return equipname end
        local str = "[" .. equipname .. GAMERATE:getChatEquipFlag() .. eid .. GAMERATE:getChatEquipFlag() .. quality .. GAMERATE:getChatEquipFlag() .. pid .. "]"
        return str
    end

    -- 封装聊天发送物品名字       
    local function getChatSendRealGoodsName( goodname, no, quality, pid )
        if goodname == nil then return "" end
        if no == nil or not isnumber(no) or quality == nil or not isnumber(quality) then return goodname  end
        if no <= 10000 or no > 20000 or quality < 1 or quality > 5 then return goodname end 
        if pid == nil or not isnumber(pid) then return goodname end
        if pid <= 0 then return goodname end
        local str = "[" .. goodname .. GAMERATE:getChatGoodsFlag() .. no .. GAMERATE:getChatGoodsFlag() .. quality .. GAMERATE:getChatGoodsFlag() .. pid .. "]"
        return str
    end
    local function shareToChat( button )
        button.setStateDisabled(true)
        local sendString 
        if 10000 < _goodsEntity:getNo() and _goodsEntity:getNo() < 20000 then
            sendString = getChatSendRealGoodsName(_goodsEntity:getName(), _goodsEntity:getNo(), _goodsEntity:getQuality(), ME:getUserid())
        elseif _goodsEntity:getNo() > 20000 then
            sendString = getChatSendRealEquipName(_goodsEntity:getName(), _goodsEntity:getEqid(), ME:getUserid(), _goodsEntity:getQuality())
        end
        if sendString and ME:CanSpeak() then
            --   创建http请求
            NetHelper.createRequest(
                
                ACTID.MSG_SEND_CHAT,                     -- 请求id
                function ( properties, params )
                    NetHelper.QuickToast(_s("CHAT_SEND_SUCCESS"))
                end,                       -- 回调函数
                {
                    messagetype = ChatUtil.ChatWorld,   -- 消息类型 默认都是世界
                    content = sendString,               -- 内容   
                    touserid = 0,                       -- 接收者id，这里为0
                },                                      -- 请求信息实体或参数table
                { 
                    showLoading = false,
                }                           
            )
        end
    end
    local function updateAlertView( )
        local padding ={left = -92}
        if _type == ALERT_TYPE_NORMAL then
            padding.top = -20 
        else
            padding.top = 30
            if _type == ALERT_TYPE_USE then
                -- 顶部文字
                _nodeTable["mEquipPoiLabel"]:setString(_s("EQUIP_BAG_EUQIP"))
                -- 按钮文字
                _node:setButtonTitle("btnEquip", _s("EQUIP_USE"))
            elseif _type == ALERT_TYPE_OFF then
                -- 顶部文字
                _nodeTable["mEquipPoiLabel"]:setString(_s("EQUIP_CUR_EUQIP"))
                -- 按钮文字
                _node:setButtonTitle("btnConfirm", _s("EQUIP_UNLOAD"))
            end
            -- 按钮不可点击
            if _node._DATA.buttonEnable == false then
                _nodeTable["btnConfirm"]:setEnabled(false)
                _nodeTable["btnEquip"]:setEnabled(false)
            end
        end
        -- 分享按钮
        if _node._DATA.shareButton and _goodsEntity:getQuality() >= 2 and _GAME.CONFIG.SDK_PLATFORM.chat ~= false then
            local shareButton = widget.newButton{
                normalImage = "btn_share.png",
                scale = 1.1,
                listener = shareToChat,
                labelParams = {text = _s("GOODS_SHOW_OFF"), color = BUTTON_COLOR3.BROWN, size = 16, padding = {top = 20}},
            }
            _node:addChild(shareButton)
            util.alignNodeWithPadding(shareButton, _node, display.TOP_CENTER, padding)
        end        
    end
    local function resetAlertSize( )
        -- 计算有几条
        local contentNode = CCBCenter.getNodeByTag(_node, 10402)
        local prosTable = _goodsEntity:getPros()
        local skillsTable = _goodsEntity:getSkills()
        local setidsTable = _goodsEntity:getSetids()
        local count = #prosTable + #skillsTable + #setidsTable + 1
        if count < 6 or _goodsEntity:getNo() > 29000 then
            count = 6
        end
        if _goodsEntity.userName and _goodsEntity.userName ~= "" and _type == ALERT_TYPE_NORMAL then
            _nodeTable["mEquipPoiLabel"]:setString(_s("EQUIP_HERO_HAVE", _goodsEntity.userName))
        end
        if count * CELL_SIZE < contentNode:getContentSize().height and _type == ALERT_TYPE_NORMAL then
            -- 小于缩小
            local node = CCBCenter.getNodeByTag(_node, 1)
            local node1 = _nodeTable["mEquipPoiLabel"]
            local node2 = CCBCenter.getNodeByTag(_node, 102)
            local node3 = CCBCenter.getNodeByTag(_node, 103)
            local node4 = CCBCenter.getNodeByTag(_node, 104)
            local node5 = CCBCenter.getNodeByTag(_node, 105)
            _scrollContentSize = CCSize(contentNode:getContentSize().width, count * CELL_SIZE)
            -- 背景尺寸缩小
            _nodeTable["mBgSprite"]:setContentSize(CCSizeAdd(_nodeTable["mBgSprite"]:getContentSize(), 0, - (node4:getContentSize().height - _scrollContentSize.height) + 8))
            
            -- 文本尺寸
            node4:setContentSize(_scrollContentSize)
            -- 容器尺寸重置
            _node:setContentSize(CCSize(_node:getContentSize().width, node1:getContentSize().height + node2:getContentSize().height + node3:getContentSize().height + node4:getContentSize().height + node5:getContentSize().height))
            node:setContentSize(_node:getContentSize())
            -- 位置重置
            -- 玩家名称位置
            node1:setAnchorPoint(ccp(0, 1))
            util.alignNode(node1, _node, display.TOP_LEFT, 10)
            local positionY1 = node1:getPositionY() - node1:getContentSize().height
            -- 名称位置
            node2:setAnchorPoint(ccp(0, 1))
            node2:setPosition(0, positionY1)
            local positionY2 = positionY1 - node2:getContentSize().height - 4 
            -- 图片位置
            node3:setAnchorPoint(ccp(0, 1))
            node3:setPosition(0, positionY2)
            local positionY3 = positionY2 - node3:getContentSize().height - 4 
            -- 文本位置
            node4:setAnchorPoint(ccp(0, 1))
            node4:setPosition(0, positionY3)
            local positionY4 = positionY3 - node4:getContentSize().height - 4
            -- 金币位置重置
            node5:setAnchorPoint(ccp(0, 1))
            node5:setPosition(0, positionY4)
            util.alignNode(_nodeTable["mBgSprite"], _node, display.TOP_CENTER)
        else
            -- 不变
            _scrollContentSize = contentNode:getContentSize()
        end
    end
	local function initViews()
        resetAlertSize()
        _scrollGrid = createScrollGrid()
		updateGoodsInfo()
        updateAlertView( )
	end

--[[ ## public fuction --]]

--[[ ## MAINs --]]
	initViews()
end

return M