--EquipUseAlertLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[EquipUseAlertLayerView]"
    local _node, _nodeTable = node, luaValue
    local _goodsEntity = _node._DATA
    local _scrollGrid
--[[ ## local fuction --]]
    -- 属性容器
    local function createScrollGrid( )
        local node = CCBCenter.getNodeByTag(_node, 10402)
        local function cellAtIndex( scrollGrid, idx , item)
            local cell = widget.newLayer()
            cell:setContentSize(scrollGrid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)

            cell:addChild(item)
            util.alignNode(item, cell)

            return cell
        end
        local scrollGrid = widget.newScrollGrid{
            contentSize = node:getContentSize(),
            contentParams = {
                cellSize = CCSize(node:getContentSize().width, 20),
                column = 1,
                createItemCellListener = cellAtIndex,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
            parentNode = node,
        }
        return scrollGrid
    end
    -- 图标
    local function addGoodsIcon( file )
        local node = CCBCenter.getNodeByTag(_node, 1030102)
        local sprite = widget.newSprite{file = file}
        node:addChild(sprite, 1)
        util.alignNode(sprite, node:getContentSize(), display.CENTER)
    end
    -- 图标背景
    local function addGoodsIconBG( file )
        local node = CCBCenter.getNodeByTag(_node, 1030102)
        local bgSprite = widget.newSprite{ file = file}
        node:addChild(bgSprite, 0)
        util.alignNode(bgSprite, node:getContentSize(), display.CENTER)
    end
    -- 取得文本View
    local function getTextTitleView( params )
        local node = widget.newLayer{contentSize = _scrollGrid:getGrid():getCellSize()}
        params.size = params.size or 14
        local ttfLabel = ui.newTTFLabel(params)
        node:addChild(ttfLabel)
        util.alignNode(ttfLabel, node, display.CENTER_LEFT)
        return node
    end
    -- 取得套装View
    local function getSetView( set )
        local node = widget.newLayer{contentSize = _scrollGrid:getGrid():getCellSize()}
        local name = EquipDB.getByID(set.setid).name
        local nameLabel = ui.newTTFLabel{text = name, size = 14, color = set.ison == TRUE and COLOR3.WHITE or BUTTON_COLOR3.GREE }
        node:addChild(nameLabel)
        util.alignNode(nameLabel, node, display.CENTER_LEFT)
        return node
    end
    -- 取得属性View
    local function getProsInfoView( pro )
        local node = widget.newLayer{contentSize = _scrollGrid:getGrid():getCellSize()}
        local infoText = ""
        local effect = pro.typ
        local race = pro.camp
        local value = pro.value
        local isact = pro.isact
        local color = COLOR3.WHITE

        local typText = TALENT_EFFECT_ATK == effect and _s("EQUIP_ATTACK") or _s("EQUIP_HP")
        local raceText
        if RACE_ALL == race then
            raceText = _s("EQUIP_ALLRACE")
        elseif RACE_ALLIANCE == race then
            raceText = _s("CARD_RACE_ALLIANCE")
        elseif RACE_TRIBE == race then
            raceText = _s("CARD_RACE_TRIBE")
        elseif RACE_UNDEAD == race then
            raceText = _s("CARD_RACE_UNDEAD")
        elseif RACE_HERO == race then
            raceText = _s("EQUIP_HERO")
        end
        if effect and race and value then
            infoText = string.format("%s%s + %s", raceText, typText, value)
            if 2 <= isact and isact <= 6 then
                infoText = string.format(_s("EQUIP_SET_POS", isact, infoText))
                color = BUTTON_COLOR3.GREE
            elseif 12 <= isact and isact <= 16 then
                infoText = string.format(_s("EQUIP_SET_POS", isact - 10, infoText))
                color = COLOR3.GREEN
            end
            local prosLabel = ui.newTTFLabel{size = 14, text = infoText, color = color}
            node:addChild(prosLabel)
            util.alignNode(prosLabel, node, display.CENTER_LEFT)
        end
        
        return node
    end
    -- 数据更新
    local function updateGoodsInfo( )
        -- 物品名
        local goodsName = _goodsEntity:getName()
        if goodsName then
            _nodeTable["mEquipNameLabel"]:setString(goodsName)
            _nodeTable["mEquipNameLabel"]:setColor(_GAME.CONFIG.GOODS.QUALITY[_goodsEntity:getQuality()].rgb)
        end
        -- 类型
        local goodsType = _goodsEntity:getPartString()
        if goodsType then
            _nodeTable["mTypeLabel"]:setString(string.format(_s("GOODS_TYPE"), goodsType))
        end
        -- 等级
        local goodsLv = _goodsEntity:getStrlv()
        if goodsLv then
            _nodeTable["mLevelLabel"]:setString(string.format(_s("GOODS_LEVEL"), goodsLv))
        end
        -- 需求等级
        -- local goodsType = _goodsEntity:getTypString()
        -- if goodsType then
        --     _nodeTable["mNeedLevelLabel"]:setString(string.format(_s("GOODS_NEED_LEVEL"), 0))
        -- end

        -- 售价
        local goodsPrice = _goodsEntity:getPrice()
        if goodsPrice then
            _nodeTable["mSellLabel"]:setString(string.format(_s("GOODS_PRICE"), goodsPrice))
        end
        -- 图标
        local goodsIcon = _goodsEntity:getGoodsIcon()
        if goodsIcon then
            addGoodsIcon(goodsIcon)
        end
        -- 背景图
        local bgGoodsIcon = _GAME.CONFIG.GOODS.QUALITY[_goodsEntity:getQuality()].frame
        if bgGoodsIcon then
            addGoodsIconBG(bgGoodsIcon)
        end

        local infoViews = {}
        -- 属性
        infoViews[#infoViews + 1] = getTextTitleView{text = _s("EQUIP_EFFECT"), size = 14}
        local prosTable = _goodsEntity:getPros()
        for i, v in ipairs(prosTable) do
            infoViews[#infoViews + 1] = getProsInfoView(v)
        end
        -- 套装
        local setidsTable = _goodsEntity:getSetids()
        local setId = _goodsEntity:getSetid()
        local setDb = EquipSetDB.getByID(setId)
        -- setId = 8
        -- setidsTable = {{setid = 20001, ison = TRUE}, {setid = 20002, ison = TRUE}, {setid = 20003, ison = TRUE}, {setid = 20004, ison = FALSE}, {setid = 20005, ison = FALSE}}
        if #setidsTable > 0 and setDb then
            local ison = 0
            -- 已经装备数量
            for i, v in ipairs(setidsTable) do
                if v.ison == TRUE then
                    ison = ison + 1
                end
            end
            -- 套装名
            local setName = setDb.name
            if setName then
                infoViews[#infoViews + 1] = getTextTitleView{text = _s("EQUIP_SET", setName, ison, #setidsTable), color = COLOR3.YELLOW, }
            end
            -- 套装部位
            for i, v in ipairs(setidsTable) do
                infoViews[#infoViews + 1] = getSetView(v)
            end
        end
        _scrollGrid:setItems(infoViews)
        util.autoFixScrollView(_scrollGrid)
    end
    local function updateAlertView( )
        -- 顶部文字
        _nodeTable["mEquipPoiLabel"]:setString(_s("EQUIP_BAG_EUQIP"))
        -- 按钮文字
        _node:setButtonTitle("btnEquip", _s("EQUIP_USE"))
        -- 按钮不可点击
        if _node._DATA.buttonEnable == false then
            _nodeTable["btnConfirm"]:setEnable(false)
            _nodeTable["btnEquip"]:setEnable(false)
        end
    end
    local function initViews()
        _scrollGrid = createScrollGrid()
        updateGoodsInfo()
        updateAlertView( )
    end

--[[ ## public fuction --]]

--[[ ## MAINs --]]
    initViews()
end

return M