--GoodsLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[GoodsLayerView]"
	local _node, _nodeTable = node, luaValue
    local _goodsTable = {}
    local _goodsGrid, _detailScrollView, _detailLabel
    local _tabKeys = {}
    local _curBackpack = 1
    local _curGoods
--[[ ## local fuction --]]
    -- 创建详情的滚动窗
    local function createDetailScrollView( )
        local node = CCBCenter.getNodeByTag(_node, 407)
        local scrollView = widget.newScrollView{
            -- bounceable = false, 
            contentSize = node:getContentSize(), 
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL, 
        }
        node:addChild(scrollView)

        local detailLabel = ui.newTTFLabel{text = "" , color = COLOR_WHITE}
        local size = scrollView:getContentSize()
        detailLabel:setDimensions(CCSize(size.width, 0))
        detailLabel:setVerticalAlignment(ui.TEXT_VALIGN_TOP)
        detailLabel:setHorizontalAlignment(ui.TEXT_ALIGN_LEFT)
        -- detailLabel:setString(goods:getDetails())
        scrollView:setContainer(detailLabel)
        scrollView:updateInset()
        util.autoFixScrollView(scrollView)

        return scrollView, detailLabel
    end
    -- 设置详情窗口
    local function setGoodsDetail( string )
        _detailLabel:setString(string)
        _detailScrollView:updateInset()
        util.autoFixScrollView(_detailScrollView)
    end
    -- 更新物品背包
    local function updateGoodsGrid( goodsType, isHoldPageIndex )
        local backpack = _goodsTable[goodsType]
        if backpack then
            _goodsGrid:setItems(backpack:getObjs(), true, isHoldPageIndex)
        end
    end
    -- 更新物品的信息
    local function updateGoodsInfo( goods )
        _curGoods = goods
        -- 名称
        _nodeTable["mGoodsNameLabel"]:setString(goods:getName())
        _nodeTable["mGoodsNameLabel"]:setColor(_GAME.CONFIG.GOODS.QUALITY[goods:getQuality()].rgb)
        _nodeTable["btnUse"]:setEnabled(goods:getUsetype() ~= _GAME.CONFIG.GOODS.USETYPE.NOTUSE)
        _nodeTable["btnSell"]:setEnabled(goods:getCansell() == TRUE)
        _nodeTable["mPriveLabel"]:setString(goods:getPrice())
        -- 按钮文字
        local useText = _s("GOODS_USE")
        if goods:getUsetype() == _GAME.CONFIG.GOODS.USETYPE.SYNTHESIS then
            useText = _s("GOODS_SYNTHESIS")
        end
        _node:setButtonTitle("btnUse", useText)
        -- 图标
        local iconNode = CCBCenter.getNodeByTag(_node, 40202)
        iconNode:removeAllChildrenWithCleanup(true)
        local bgSprite = widget.newSprite{ file = _GAME.CONFIG.GOODS.QUALITY[goods:getQuality()].frame}
        iconNode:addChild(bgSprite, 0)
        util.alignNode(bgSprite, iconNode:getContentSize(), display.CENTER)
        
        local goodsIcon = widget.newSprite{file = goods:getGoodsIcon()}
        iconNode:addChild(goodsIcon)
        util.alignNode(goodsIcon, iconNode:getContentSize(), display.CENTER)
        -- 详情
        setGoodsDetail(goods:getDetails())
    end
    local function cleanGoodsInfo( )
        _curGoods = nil
        _nodeTable["mGoodsNameLabel"]:setString("")
        _nodeTable["btnUse"]:setEnabled(false)
        _nodeTable["btnSell"]:setEnabled(false)
        _nodeTable["mPriveLabel"]:setString("")
        -- 图标
        local iconNode = CCBCenter.getNodeByTag(_node, 40202)
        iconNode:removeAllChildrenWithCleanup(true)
        -- 详情
        setGoodsDetail("")
    end
    -- 创建背包切换按钮
    local function updatePackageTab( backage )
        local layer = CCBCenter.getNodeByTag(_node, 502)
        layer:removeAllChildrenWithCleanup(true)
        local function onTabClick( index, button )
            _curBackpack = index
            -- 清空
            cleanGoodsInfo( )
            -- 切换背包
            updateGoodsGrid(_tabKeys[index], false)
        end 
        local function getTypText_( typ )
            for k, v in pairs(_GAME.CONFIG.GOODS.TYPE) do
                if v.KEY == typ then
                    return v.TEXT
                end
            end
            return ""
        end
        local buttons = {}
        _tabKeys = {}
        for i, v in ipairs(backage) do
            local button = widget.newButton{   
                normalImage = "cardgroup/btn_switch_normal.png",
                selectImage = "cardgroup/btn_switch_select.png",
                contentSize = CCSize(94, 49),
                discolor    = COLOR3.WHITE,
                labelParams = {text = getTypText_(v:getTyp()), size = 22, color = BUTTON_COLOR3.BROWN, padding = {top = 2}},
            }
            _tabKeys[i] = v:getTyp()
            table.insert(buttons, button)
        end

        local tab = widget.newTab{
            items = buttons,
            selectIndex = _curBackpack,
            listener = onTabClick,
        }
        layer:addChild(tab, 1)
        util.alignNode(tab, layer:getContentSize(), display.CENTER_LEFT)
    end

    local function createGoodsPages( ... )
        local layer = CCBCenter.getNodeByTag(_node, 503)

        local function createItemCellListener( grid, index, item )
            local gridCell = widget.newLayer{ contentSize = grid:getCellSize() }
            gridCell.VIEWS_ = {}
            
            local bgImageFile = "goods/goodBack.png"
            
            if item.getGoodsIcon then
                item.type = item:getTyp()
                -- 背景颜色
                bgImageFile = _GAME.CONFIG.GOODS.QUALITY[item:getQuality()].frame
                -- 图标
                local goodsIcon = widget.newSprite{file = item:getGoodsIcon()}
                util.alignNode(goodsIcon, gridCell:getContentSize(), display.CENTER)
                gridCell:addChild(goodsIcon, 1)
                gridCell.VIEWS_.goodsIcon = goodsIcon
                -- 数量
                local goodsNum = ui.newTTFLabel{
                    text = item:getCount(),
                    size = 20,
                }
                util.alignNode(goodsNum, gridCell:getContentSize(), display.BOTTOM_RIGHT, 10)
                gridCell:addChild(goodsNum, 2)
                gridCell.VIEWS_.goodsNum = goodsNum
                if grid.getPageGrid():isHighLight(item) then 
                    local light = widget.newSprite{file = "goods/bg_goods_select.png"}
                    util.alignNode(light, gridCell:getContentSize(), display.CENTER)
                    gridCell:addChild(light, -1)
                    gridCell.VIEWS_.light = light
                end
            end
            -- 背景
            local bg = display.newSprite(bgImageFile)
            gridCell:addChild(bg, 0)
            gridCell.VIEWS_.bg = bg
            util.alignNode(bg, gridCell:getContentSize(), display.CENTER)

            gridCell:ignoreAnchorPointForPosition(false)
            return gridCell
        end

        local function modifyItemCellListener( grid,index,item,itemCell )
            -- itemCell:setString(tostring(item.index))
            local isHighLight = grid.getPageGrid():isHighLight(item)
            if not isHighLight and itemCell.VIEWS_.light then 
                -- 都叫你不要亮了还亮个P,给我灭掉!!!
                itemCell.VIEWS_.light:removeSelf()
                itemCell.VIEWS_.light = nil
            elseif isHighLight and not itemCell.VIEWS_.light then 
                -- 要你亮时你又在这卖萌,找屎
                local light = widget.newSprite{file = "goods/bg_goods_select.png"}
                util.alignNode(light, itemCell:getContentSize(), display.CENTER)
                itemCell:addChild(light, -1)
                itemCell.VIEWS_.light = light
            end
        end

        local function tapListener( grid,index,item )
            -- print("@@@@@@ tapListener", item.index)
            local cell = grid:cellAt(index)
            local array     = {}
            array[#array + 1] = CCScaleTo:create(0.2, 1.1)
            array[#array + 1] = CCScaleTo:create(0.2, 1)
            transition.execute(cell, transition.sequence(array), {easing = "CCEaseBackInOut", onComplete = function ()
                -- playArrowShakeAnim( _node, isReverse )
            end })
            if item.type then
                local pageIndex = _goodsGrid:getScrollLayer().getCurPageIndex()
                item.index = (pageIndex - 1)  * 18 + index
                grid.getPageGrid():highLight(_goodsGrid:getScrollLayer().getCurPageIndex(), index)
                updateGoodsInfo(item)
            end
        end

        local pageGrid = widget.newPageGrid{
            pageParams = {
                    rect = CCRect(0, 0, layer:getContentSize().width, layer:getContentSize().height),
                    totalPages = 1,
                    contentSize = layer:getContentSize(),
                    ignoreAnchor = true,
                    indicatorParams = {
                        normal = "goods/goods_Indicator.png",
                        selected = "goods/goods_Indicator_selected.png",
                        height = 30,
                        padding = 10,
                    },
            },
            gridParams = {
                    disableAutoCalcContentSize = true,
                    row = 3, 
                    column = 6, 
                    createItemCellListener = createItemCellListener,
                    modifyItemCellListener = modifyItemCellListener,
                    tapListener = tapListener,
            },
        }

        layer:addChild(pageGrid)
        -- pageGrid:jumpToPage(3)
        -- pageGrid:setItems(items, false, true)
        
        -- -- 测试用
        -- util.delayCall(function ( ... )
        --     local items = {}
        --     for i = 1, 100 do
        --         items[i] = { index = "i:" .. i }
        --     end
        --     pageGrid:setItems(items, true, false)
        -- end, 5)
        return pageGrid
    end
    
	local function initViews()
        _nodeTable["btnUse"]:setEnabled(false)
        _nodeTable["btnSell"]:setEnabled(false)
		M.updatePlayerInfo( )
        _goodsGrid = createGoodsPages()
        _detailScrollView, _detailLabel = createDetailScrollView()
	end

--[[ ## public fuction --]]
    function M.getCurGoods( )
        return _curGoods
    end
    -- 更新玩家信息(钻石和金币)
    function M.updatePlayerInfo( )
        _nodeTable["mGoldLabel"]:setString(ME:getGold())
        _nodeTable["mDiamondLabel"]:setString(ME:getDiamond())
    end
    -- 更新背包信息
    function M.updatePackageInfo( backage )
        _goodsTable = {}
        for i,v in ipairs(backage) do
            _goodsTable[v:getTyp()] = v
        end
        updatePackageTab( backage )
        updateGoodsGrid(_tabKeys[_curBackpack], true)
        
        if _curGoods then
            local find = nil
            for i,v in ipairs(_goodsTable[_tabKeys[_curBackpack]]:getObjs()) do
                if _curGoods.index == i and _curGoods:getNo() == v:getNo() then
                    find = v 
                    find.index = i
                    break
                end
            end
            if find then
                updateGoodsInfo(find)
                _goodsGrid:highLight(find.index)
            else
                cleanGoodsInfo()
            end
        end
    end
--[[ ## MAINs --]]
	initViews()
    _node.getCurGoods           = M.getCurGoods
    _node.cleanGoodsInfo        = cleanGoodsInfo
    _node.updatePlayerInfo      = M.updatePlayerInfo
    _node.updatePackageInfo     = M.updatePackageInfo
end

return M