--MailAlertLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[MailAlertLayerView]"
	local _node, _nodeTable = node, luaValue
    local _messageTab, _listGrid, _editBox, _chatGrid
    local _messageSchedule
    local RewardEntity = require("model.main.RewardEntity")
    local STATE_IMAGE = {"main/activity/img_activity_sign_new.png", "main/activity/img_activity_sign_notice.png"}
    local MAIL_TYPE = {
        SYSTEM = 1,
        USER = 2,
    }
    local REFRESH_TIME = 3
    local _selectTab = _node._SELECT or MAIL_TYPE.SYSTEM
    local _mailData = {
        [MAIL_TYPE.SYSTEM] = {tabTitle = _s("MAIL_SYSTEM_INBOX"), selectIndex = nil, selectMid = nil, offset = nil, content = {}},
        [MAIL_TYPE.USER] = {tabTitle = _s("MAIL_INBOX"), selectIndex = nil, selectMid = nil, offset = nil, content = {}},
    }
    local _chatOffset = nil
    local MAX_POP_LENGTH = 300
--[[ ## local fuction --]]
    local function hideEditbox( hide )
        CCBCenter.getNodeByTag(_node, 306):setVisible(hide)
        if _editBox then
            _editBox:setVisible(hide)
        end
    end
    local function enableRefreshMessage( enable )
        local function recChatRequest_()
            if _chatGrid then
                _chatOffset = _chatGrid:getScrollView():getContentOffset()
            end
            _mailData[_selectTab].offset = _listGrid:getContentOffset()
            CCBCenter.sendLuaMessage(MSG_ID.MSG_MAILALERTLAYER_ACTION_ONFETCHMAILLIST, nil, {chat = true})
        end 
        if enable then
            -- 启动聊天刷新
            _messageSchedule = _node:schedule(recChatRequest_, REFRESH_TIME)
        else
            _node:removeAction(_messageSchedule)
        end
    end
    local function createEditBox( )
        local editNode = CCBCenter.getNodeByTag(_node, 5)
        local node = CCBCenter.getNodeByTag(_node, 306)
        -- 背景图
        local bgSprite = widget.newScale9{
            file = "mail/bg_mail_content.png",
            rect = CCRect(0, 0, 41, 35),
            contentSize = CCSizeAdd(node:getContentSize(), -80, 0),
        }
        node:addChild(bgSprite, 0)
        util.alignNode(bgSprite, node, display.CENTER_LEFT)
        -- 编辑框
        local editbox = widget.newEditBox{
            contentSize = CCSizeAdd(editNode:getContentSize(), 0, -6),
            normalBgImage = {file = "hook_ac.png"},
            placeHolderText = _s("MAIL_SEND_TIP"),
            -- textMaxLength   = 100,
            x = editNode:getPositionX() ,
            y = editNode:getPositionY() ,
        }
        -- editbox:setFontSize(18)
        -- editbox:setPlaceholderFontSize(18)
        -- editbox:setInputMode(kEditBoxInputModeAny)
        _node:addChild(editbox, 10)
        -- util.alignNode(editbox, node, display.CENTER_LEFT, {left = 4, top = 4})
        -- 发送按钮
        local editButton = widget.newButton{
            normalImage = "common/buttons/btn_normal_yellow.png",
            contentSize = CCSize(80, 56),
            rect = CCRect(0, 0, 80, 58),
            scale = 1.1,
            labelParams = {text = _s("COMMON_SEND"), color = BUTTON_COLOR3.BROWN},
            listener = function ( button )
                if editbox:getText() ~= "" and ME:CanSpeak( ) then
                    -- _mailData[_selectTab].offset = _listGrid:getContentOffset()
                    _mailData[_selectTab].offset = nil
                    _chatOffset = nil
                    local msgData = _mailData[_selectTab]
                    CCBCenter.sendLuaMessage(MSG_ID.MSG_MAILALERTLAYER_ACTION_ONFETCHSENDMAIL, nil, {fid = msgData.content[msgData.selectIndex].mid, content = editbox:getText()})
                    editbox:setText("")
                end
            end,
        }
        node:addChild(editButton, 2)
        util.alignNode(editButton, node, display.CENTER, {left = 180})
        return editbox
    end
    local function createChatGrid( )
        local node = CCBCenter.getNodeByTag(_node, 30302)
        local function cellAtIndex( scrollGrid, idx, item)

            local cell = widget.newLayer()
            cell:ignoreAnchorPointForPosition(false)
            try(
                function ( )
                    local tempLabel = ui.newTTFLabel{
                        text = item.content,
                        size = 18,
                    }
                    local lines = math.ceil( tempLabel:getContentSize().width/ MAX_POP_LENGTH)
                    local infoLabel = ui.newTTFLabel{
                        text = item.content,
                        size = 18,
                        color = BUTTON_COLOR3.BROWN,
                        dimensions = CCSize(lines == 1 and tempLabel:getContentSize().width or MAX_POP_LENGTH, lines * tempLabel:getContentSize().height),
                    }
                    cell:addChild(infoLabel, 1)
        
                    local bgFile
                    local poi
                    if item.sendid == ME:getUserid() then
                        bgFile = "mail/bg_msg_content_right.png"
                        poi = display.CENTER_RIGHT
                    else
                        bgFile = "mail/bg_msg_content_left.png"
                        poi = display.CENTER_LEFT
                    end
                    local bgSprite = widget.newScale9{
                        file = bgFile,
                        contentSize = CCSizeAdd(infoLabel:getContentSize(), 30, 20),
                        rect = CCRectMake(0, 0, 43, 48),
                        capInsets = CCRectMake(15, 35, 2.5, 2.5),
                    }
                    cell:addChild(bgSprite, 0)
        
                    local cellHeight = bgSprite:getContentSize().height + 10
        
                    cell:setContentSize(CCSize(node:getContentSize().width, cellHeight))
                    util.alignNode(bgSprite, cell, poi)
                    util.alignNode(infoLabel, cell, poi, 20)
                end
            )
            return cell
        end
        local scrollGrid = widget.newScrollLinear{
            contentSize = node:getContentSize(),
            cellAtIndexListener = cellAtIndex,
            contentParams = {
                column = 1,
                -- tapListener = tapListener,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
        }
        node:addChild(scrollGrid)
        return scrollGrid
    end
    local function createBulletinGrid( data )
        local node = CCBCenter.getNodeByTag(_node, 30302)
        node:removeAllChildrenWithCleanup(true)
        local function getTTFLabel( text )
            -- local label = ui.newTTFLabel{text = text, size = 20, }
            local tempLabel = ui.newTTFLabel{
                text = text,
                size = 20,
            }
            local lines = math.ceil( tempLabel:getContentSize().width / node:getContentSize().width)
            local label = ui.newTTFLabel{
                text = text,
                size = 20,
                dimensions = CCSize(node:getContentSize().width, lines * tempLabel:getContentSize().height),
            }
            return label
        end
        local function getRewardGrid( reward )
            local rewardGrid = nil
            local rewardEntity = RewardEntity.new(reward)
            if rewardEntity:notEmpty() then
                local function cellAtIndex( scrollGrid, idx, item)
                    local cell = widget.newLayer()
                    cell:setContentSize(scrollGrid:getCellSize())
                    cell:ignoreAnchorPointForPosition(false)
                    if item then
                        cell:addChild(item)
                        util.alignNode(item, cell)
                    end
                    return cell
                end
                local function onTabClick( grid, index, item, x, y )
                    if not util.isPointInNode(CCBCenter.getNodeByTag(_node, 30302), x, y) then 
                        return
                    end
                    if item:getType() == RewardEntity.REWARD_TYPE.CARD or item:getType() == RewardEntity.REWARD_TYPE.CHIP then
                        CCBCenter.loadCCB("MSG_CARDDETAILLAYER_MODEL", "ccb.c.cardgroup.CardDetailLayer", item:getEntity())
                    elseif item:getType() == RewardEntity.REWARD_TYPE.GOODS then
                        if item:getEntity():getNo() > 20000 then
                            CCBCenter.loadCCB("MSG_EQUIPALERTLAYER_MODEL", "ccb.c.goods.EquipAlertLayer", item:getEntity())
                        else
                            CCBCenter.loadCCB("MSG_GOODSALERTLAYER_MODEL", "ccb.c.goods.GoodsAlertLayer", item:getEntity())
                        end
                    end
                end
                rewardGrid = widget.newGrid{
                    cellSize = CCSize(87, 87),
                    column = 4,
                    createItemCellListener = cellAtIndex,
                    items = rewardEntity:getRewardViews(),
                    tapListener = onTabClick,
                    padding = {
                        middleH = 20,
                        middleV = 10,
                    },
                }
                -- rewardGrid:setItems(rewardEntity:getRewardViews())
            end
            return rewardGrid
        end
        local function cellAtIndex( scrollGrid, idx, item)
            local cell = widget.newLayer()
            cell:ignoreAnchorPointForPosition(false)
            cell:addChild(item)
            local cellHeight = item:getContentSize().height + 10
            cell:setContentSize(CCSize(node:getContentSize().width, cellHeight))
            util.alignNode(item, cell, display.CENTER_LEFT)
            return cell
        end
        local scrollGrid = widget.newScrollLinear{
            contentSize = node:getContentSize(),
            cellAtIndexListener = cellAtIndex,
            contentParams = {
                column = 1,
                -- tapListener = tapListener,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
        }
        node:addChild(scrollGrid)
        local items = {}
        items[#items + 1] = getTTFLabel(_s("MAIL_CONTENT"))
        items[#items + 1] = getTTFLabel(data.content)
        items[#items + 1] = getRewardGrid(data.reward)
        scrollGrid:setItems(items)
        return scrollGrid
    end
    -- 通知模板
    local function showNoticeTemplate( data )
        Log.d(TAG, "通知模板")
        -- 聊天列表
        if not _chatGrid then
            _chatGrid = createChatGrid()
        end

        _chatGrid:setItems(data.detailed)

        if _chatGrid:getLinearLayer():getContentSize().height > _chatGrid:getContentSize().height then
            _chatGrid:getScrollView():setContentOffset(ccp(0, 0))
        end
    end
    -- 公告模板
    local function showBulletinTemplate( data )
        Log.d(TAG, "公告模板")
        -- 公告内容
        local detail = data.detailed[1]
        _bulletin = createBulletinGrid(detail)
        -- if _bulletin:getLinearLayer():getContentSize().height > _bulletin:getContentSize().height then
        --     _bulletin:getScrollView():setContentOffset(ccp(0, 0))
        -- end
        if detail.state == 1 then
            _nodeTable["btnReceive"]:setVisible(true)
            _nodeTable["btnReceive"]:setEnabled(true)
            _node:setButtonTitle("btnReceive", _s("MAIL_RECEIVE"))
        elseif detail.state == 2 then
            _nodeTable["btnReceive"]:setVisible(true)
            _nodeTable["btnReceive"]:setEnabled(false)
            _node:setButtonTitle("btnReceive", _s("MAIL_ALREADY_RECEIVE"), nil, CCControlStateDisabled)
        else
            _nodeTable["btnReceive"]:setVisible(false)
        end
    end
    -- 聊天模板
    local function showChatTemplate( data, force )
        Log.d(TAG, "聊天模板")
        -- 聊天框
        hideEditbox(true)
        if not _editBox then
            _editBox = createEditBox()
        elseif force then
            _editBox:setText("")
        end
        -- 聊天列表
        if not _chatGrid then
            _chatGrid = createChatGrid()
        end
        if _chatGrid:getItems() and #_chatGrid:getItems() ~= #data.detailed then
            _chatOffset = nil
        end
        _chatGrid:setItems(data.detailed)
        if _chatOffset then
            _chatGrid:getScrollView():setContentOffset(_chatOffset)
        elseif _chatGrid:getLinearLayer():getContentSize().height > _chatGrid:getContentSize().height then
            _chatGrid:getScrollView():setContentOffset(ccp(0, 0))
        end
    end
    local function clearContent( )
        -- 发件人
        try(
            function ( )
                _nodeTable["mSenderLabel"]:setString("")
            end
        )
        -- 按钮状态
        _nodeTable["btnDelete"]:setVisible(false)
        _nodeTable["btnReceive"]:setVisible(false)
        -- 清空内容区域
        _chatGrid = nil
        _bulletin = nil
        CCBCenter.getNodeByTag(_node, 30302):removeAllChildrenWithCleanup(true)
        -- 输入框
        hideEditbox(false)
        if _editBox then
            _editBox:setText("")
        end
    end
    local function updateContent( data, force )
        -- 发件人
        try(
            function ( )
                _nodeTable["mSenderLabel"]:setString(_s("MAIL_SENDER_NAME", data.sendername))
            end
        )
        -- 按钮
        _nodeTable["btnDelete"]:setVisible(true)
        -- 清空内容区域
        _chatGrid = nil
        _bulletin = nil
        CCBCenter.getNodeByTag(_node, 30302):removeAllChildrenWithCleanup(true)
        if data.mtyp == MAIL_TYPE.SYSTEM then
            -- 输入框
            hideEditbox(false)
            if data.mid == 1 or data.mid == 2 or data.mid == 3 or data.mid == 4 then
                -- 赞美1 或打劫 2 軍團3 4
                -- 领取按钮
                _nodeTable["btnReceive"]:setVisible(false)
                showNoticeTemplate( data )
            else
                showBulletinTemplate( data )
            end
        elseif data.mtyp == MAIL_TYPE.USER then
            -- 领取按钮
            _nodeTable["btnReceive"]:setVisible(false)
            showChatTemplate(data, force)
        end
    end
    -- 更新左边列表状态
    local function updateList( force )
        _listGrid:setItems(_mailData[_selectTab].content)
        if not _mailData[_selectTab].selectIndex then
            clearContent()
        else
            updateContent(_mailData[_selectTab].content[_mailData[_selectTab].selectIndex], force)
        end
        -- 如果有offset就保持当前位置
        if _mailData[_selectTab].offset then
            _listGrid:setContentOffset(_mailData[_selectTab].offset)
        else
            util.autoFixScrollView(_listGrid)
            -- _listGrid:setContentOffset(ccp(0, 0))
        end
    end
    local function createTab( )
        local node = CCBCenter.getNodeByTag(_node, 2)
        node:removeAllChildrenWithCleanup(true)

        local function onTabClick( index, button )
            SOUND:playEffectButton1()
            _chatOffset = nil
            -- 保留旧list的位置
            if _selectTab and _mailData[_selectTab] then
                _mailData[_selectTab].offset = _listGrid:getContentOffset()
            end
            _selectTab = button._mailType
            -- 聊天刷新
            enableRefreshMessage(_selectTab == MAIL_TYPE.USER)

            -- 做tab的点击后刷新列表和内容页面
            updateList(true)
            local data = _mailData[_selectTab]
            if data.selectIndex and data.content[data.selectIndex] then
                updateContent(data.content[data.selectIndex], true)
            else
                clearContent( )
            end
        end
        -- 构建按钮集合
        local buttons = { }
        for k, v in pairs(_mailData) do
            local btnTab = widget.newButton{   
                selectImage = "mail/btn_tab_select.png",
                normalImage = "mail/btn_tab_normal.png",
                rect = CCRectMake(0, 0, 54, 60),
                contentSize = CCSize(160, 60),
                labelParams = {text = v.tabTitle, size = 20, color = BUTTON_COLOR3.BROWN, padding = {top = 4}},
                -- scale = 1.1,
            }
            btnTab._mailType = k 
            table.insert(buttons, btnTab)   
        end
        -- tab排序
        table.sort (buttons ,function(a, b) 
            return a._mailType < b._mailType
        end)
        -- 取得选中的tab下标
        local index
        for i, v in ipairs(buttons) do
            if v._mailType == _selectTab then
                index = i
            end
        end
        local tab = widget.newTab{
            items = buttons,
            selectIndex = index,
            padding = 10,
            listener = onTabClick,
            direction = DIRECTION_HORIZONTAL,
        }
        node:addChild(tab)
        util.alignNode(tab, node)
        return tab
    end
    local function createList( )
        --取得table容器
        local node = CCBCenter.getNodeByTag(_node, 301)
        node:removeAllChildrenWithCleanup(true)
        
        local function cellAtIndex( scrollLinear, idx , item)
            local cell = display.newLayer()
            -- local cell = display.newColorLayer(ccc4(255, 255, 255, 255))
            cell:setContentSize(CCSize(node:getContentSize().width, 90))

            -- 背景
            local imageFile = "mail/bg_cell_normal.png"
            if _mailData[_selectTab] and _mailData[_selectTab].selectIndex == idx then
                imageFile = "mail/bg_cell_select.png"
                -- updateContent(item)
            end
            local bgSprite = widget.newScale9{ file = imageFile, contentSize = scrollLinear:getCellSize(), rect = CCRectMake(0, 0, 190, 90)}
            util.alignNode(bgSprite, cell)
            cell:addChild(bgSprite, 0, 100)
            local padding = {top = 10, left = 10}
            if _selectTab == MAIL_TYPE.USER then
                padding.left = 56
                -- 头像
                local headNode = widget.newLayer{contentSize = CCSize(91, 91)}
                cell:addChild(headNode, 2)
                util.alignNode(headNode, cell, display.TOP_LEFT, 10)
                headNode:setScale(0.5)
                local bgHead = widget.newSprite{
                    file = "common/bg/bg_card_head.png",
                }
                headNode:addChild(bgHead, 0)
                util.alignNode(bgHead, headNode)
                local bgFrameHead = widget.newSprite{
                    file = "common/bg/bg_card_head_frame.png",
                }
                headNode:addChild(bgFrameHead, 1)
                util.alignNode(bgFrameHead, headNode) 

                local heroHead = PlayerHead.new(item.sex, item.head, item.equiplst):createRect()
                headNode:addChild(heroHead, 2)
                util.alignNode(heroHead, headNode, display.CENTER, {top = -6})
                heroHead:setScale(0.6)
                if item.mid ~= ME:getUserid() then
                    -- 绑定头像点击
                    widget.bindTouchEvent{
                        layer = headNode,
                        clickListener = function ( )
                            CCBCenter.loadCCB("MSG_FRIENDPANELLAYER_MODEL", "ccb.c.friend.FriendPanelLayer", {fid = item.mid, fnNextView = function ( )
                                CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
                            end})
                        end,
                        container = layer,
                    }
                end
            end

            -- 名称
            local sender = ui.newTTFLabel{text = item.sendername, size = 18, color = BUTTON_COLOR3.BROWN}
            util.alignNode(sender, cell, display.TOP_LEFT, padding)
            cell:addChild(sender, 1)
            -- 时间
            local sendTime = ui.newTTFLabel{text = item.sendtime, size = 18, color = BUTTON_COLOR3.BROWN}
            util.alignNode(sendTime, cell, display.CENTER_LEFT, {top = padding.top - 10, left = padding.left })
            cell:addChild(sendTime, 1)
            -- 标题
            local title = ui.newTTFLabel{text = item.title, size = 18, color = BUTTON_COLOR3.BROWN}
            util.alignNode(title, cell, display.BOTTOM_LEFT, 10)
            cell:addChild(title, 1)
            -- 消息状态
            -- local signFile = STATE_IMAGE[item.flag]
            if item.state == TRUE then
                local signSprite = widget.newSprite{file = "main/activity/img_activity_sign_new.png"}
                util.alignNode(signSprite, cell, display.CENTER_RIGHT)
                cell:addChild(signSprite, 2, 101)
                -- anim.fadeFade( signSprite, {playtime = 1.5, forever = true, endDelay = 1})
            end

            cell:ignoreAnchorPointForPosition(false)
            return cell
        end
        local function modifyCellAtIndex( grid, index, item, itemCell )
            local bgSprite = itemCell:getChildByTag(100)
            if bgSprite then
                bgSprite:removeFromParentAndCleanup(true)
            end
            local imageFile = "mail/bg_cell_normal.png"
            if _mailData[_selectTab] and _mailData[_selectTab].selectIndex == index then
                imageFile = "mail/bg_cell_select.png"
            end
            local bgSprite = widget.newScale9{ file = imageFile, contentSize = grid:getCellSize(), rect = CCRectMake(0, 0, 190, 90)}
            util.alignNode(bgSprite, itemCell)
            itemCell:addChild(bgSprite, 0, 100)

            -- 活动状态
            local signSprite = itemCell:getChildByTag(101)
            if signSprite and item.state ~= TRUE then
                signSprite:removeFromParentAndCleanup(true)
            end
        end
        local function onBtnClick( grid, index, item )
            SOUND:playEffectButton1()
            _chatOffset = nil

            if _mailData[_selectTab].selectIndex == index then
                return
            else
                _mailData[_selectTab].selectMid = item.mid
                _mailData[_selectTab].selectIndex = index
            end
            updateContent(item, true)
            
            -- 已读取状态, 发送已读请求
            if item.state == TRUE then
                item.state = 2
                CCBCenter.sendLuaMessage(MSG_ID.MSG_MAILALERTLAYER_ACTION_ONFETCHREADMAIL, nil, {mtyp = _selectTab, mid = item.mid})
            end
            grid:refresh()
            -- _activityGrid:scrollToItemIndex(_listSelectIndex, 0.2)
        end
        local scrollGrid = widget.newScrollGrid{
            contentSize = node:getContentSize(),
            contentParams = {
                cellSize = CCSize(node:getContentSize().width, 90),
                column = 1,
                createItemCellListener = cellAtIndex,
                modifyItemCellListener = modifyCellAtIndex,
                selectedParams = nil,
                tapListener = onBtnClick,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
            parentNode = node,
        }

        return scrollGrid
    end
	local function initViews()
		_messageTab = createTab()
        _listGrid = createList()
        -- _messageSchedule = _node:schedule(recChatRequest, REFRESH_TIME)
	end

--[[ ## public fuction --]]
    local function deleteMail( )
        if _selectTab and _mailData[_selectTab] and _mailData[_selectTab].selectIndex then
            local mid = _mailData[_selectTab].content[_mailData[_selectTab].selectIndex].mid
            CCBCenter.sendLuaMessage(MSG_ID.MSG_MAILALERTLAYER_ACTION_ONFETCHDELETEMAIL, nil, {mtyp = _selectTab, mid = mid})
            _mailData[_selectTab].selectIndex = nil
            _mailData[_selectTab].selectMid = nil
            _mailData[_selectTab].offset = nil
        end
    end
    local function receiveReward( )
        if _selectTab and _mailData[_selectTab] and _mailData[_selectTab].selectIndex then
            _mailData[_selectTab].offset = _listGrid:getContentOffset()
            local mid = _mailData[_selectTab].content[_mailData[_selectTab].selectIndex].mid
            CCBCenter.sendLuaMessage(MSG_ID.MSG_MAILALERTLAYER_ACTION_ONFETCHMAILREWARD, nil, {mtyp = _selectTab, mid = mid})
        end
    end
    -- 更新聊天消息内容
    local function updateChat( data )
        local curData = _mailData[_selectTab]
        for i, v in ipairs(data.mcontent) do
            if curData.selectIndex and curData.content[curData.selectIndex].mid == v.mid then
                curData.content[curData.selectIndex] = v
                updateContent(curData.content[curData.selectIndex], false)
                if v.state == TRUE then
                    Log.d(TAG, "read e-mail")
                    CCBCenter.sendLuaMessage(MSG_ID.MSG_MAILALERTLAYER_ACTION_ONFETCHREADMAIL, nil, {mtyp = _selectTab, mid = v.mid})
                end
                break
            end
        end
        -- 确定下标
        -- for k, v in pairs(_mailData) do
        --     for i, msg in ipairs(v.content) do
        --         if v.selectMid == msg.mid then
        --             v.selectIndex = i
        --             if _selectTab == MAIL_TYPE.USER and msg.mtyp == _selectTab then
        --                 updateContent(msg, false)
        --             end
        --         end
        --     end
        -- end
        -- updateList(false)
    end
    -- 更新全部
    local function updateData( data )
        -- 清空内容
        for k, v in pairs(_mailData) do
            v.content = {}
        end
        for i, v in ipairs(data.mcontent) do
            if _mailData[v.mtyp] then
                table.insert(_mailData[v.mtyp].content, v)
            end
        end
        -- 确定下标
        for k, v in pairs(_mailData) do
            for i, msg in ipairs(v.content) do
                if v.selectMid == msg.mid then
                    v.selectIndex = i
                end
            end
        end
        updateList(true)
    end
--[[ ## MAINs --]]
	initViews()
    _node.updateData = updateData
    _node.updateChat = updateChat
    _node.deleteMail = deleteMail
    _node.receiveReward = receiveReward
end

return M