--MessageLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[MessageLayerView]"
	local _node, _nodeTable = node, luaValue
    local _fid = _node._FID
    local _nickname = _node._NICKNAME
    local _chatGrid, _editBox
    local _chatData, _chatOffset, _messageSchedule
    local MAX_POP_LENGTH = 300
    local REFRESH_TIME = 3
    local MAIL_TYPE = {
        SYSTEM = 1,
        USER = 2,
    }
--[[ ## local fuction --]]
    -- local function hideEditbox( hide )
    --     CCBCenter.getNodeByTag(_node, 303):setVisible(hide)
    --     if _editBox then
    --         _editBox:setVisible(hide)
    --     end
    -- end
    local function enableRefreshMessage( enable )
        local function recChatRequest_()
            if _chatGrid then
                _chatOffset = _chatGrid:getScrollView():getContentOffset()
            end
            CCBCenter.sendLuaMessage(MSG_ID.MSG_MESSAGELAYER_ACTION_ONFETCHMAILLIST)
        end 
        if enable then
            -- 启动聊天刷新
            _messageSchedule = _node:schedule(recChatRequest_, REFRESH_TIME)
        else
            _node:removeAction(_messageSchedule)
        end
    end
    local function createEditBox( )
        local editNode = CCBCenter.getNodeByTag(_node, 5)
        local node = CCBCenter.getNodeByTag(_node, 303)
        -- 背景图
        local bgSprite = widget.newScale9{
            file = "mail/bg_mail_content.png",
            rect = CCRect(0, 0, 41, 35),
            contentSize = CCSizeAdd(node:getContentSize(), -80, 0),
        }
        node:addChild(bgSprite, 0)
        util.alignNode(bgSprite, node, display.CENTER_LEFT)
        -- 编辑框
        local editbox = widget.newEditBox{
            contentSize = CCSizeAdd(editNode:getContentSize(), -0, -6),
            normalBgImage = {file = "hook_ac.png"},
            placeHolderText = _s("MAIL_SEND_TIP"),
            -- textMaxLength   = MAX_LENGTH,
            x = editNode:getPositionX() ,
            y = editNode:getPositionY() ,
        }
        -- editbox:setFontSize(18)
        -- editbox:setPlaceholderFontSize(18)
        -- editbox:setInputMode(kEditBoxInputModeAny)
        _node:addChild(editbox, 10)
        -- util.alignNode(editbox, node, display.CENTER_LEFT, {left = 4, top = 4})
        -- 发送按钮
        local editButton = widget.newButton{
            normalImage = "common/buttons/btn_normal_yellow.png",
            contentSize = CCSize(80, 56),
            rect = CCRect(0, 0, 80, 58),
            scale = 1.1,
            labelParams = {text = _s("COMMON_SEND"), color = BUTTON_COLOR3.BROWN},
            listener = function ( button )
                if editbox:getText() ~= "" and ME:CanSpeak( ) then
                    CCBCenter.sendLuaMessage(MSG_ID.MSG_MESSAGELAYER_ACTION_ONFETCHSENDMAIL, nil, {fid = _fid, content = editbox:getText()})
                    editbox:setText("")
                end
            end,
        }
        node:addChild(editButton, 2)
        util.alignNode(editButton, node, display.CENTER, {left = 300})
        return editbox
    end
    local function createChatGrid( )
        local node = CCBCenter.getNodeByTag(_node, 30202)
        local function cellAtIndex( scrollGrid, idx, item)
            local cell = widget.newLayer()
            cell:ignoreAnchorPointForPosition(false)

            local tempLabel = ui.newTTFLabel{
                text = item.content,
                size = 18,
            }
            local lines = math.ceil( tempLabel:getContentSize().width/ MAX_POP_LENGTH)
            local infoLabel = ui.newTTFLabel{
                text = item.content,
                size = 18,
                color = BUTTON_COLOR3.BROWN,
                dimensions = CCSize(lines == 1 and tempLabel:getContentSize().width or MAX_POP_LENGTH, lines * tempLabel:getContentSize().height),
            }
            cell:addChild(infoLabel, 1)

            local bgFile
            local poi
            if item.sendid == ME:getUserid() then
                bgFile = "mail/bg_msg_content_right.png"
                poi = display.CENTER_RIGHT
            else
                bgFile = "mail/bg_msg_content_left.png"
                poi = display.CENTER_LEFT
            end
            local bgSprite = widget.newScale9{
                file = bgFile,
                contentSize = CCSizeAdd(infoLabel:getContentSize(), 30, 20),
                rect = CCRectMake(0, 0, 43, 48),
                capInsets = CCRectMake(15, 35, 2.5, 2.5),
            }
            cell:addChild(bgSprite, 0)

            local cellHeight = bgSprite:getContentSize().height + 10

            cell:setContentSize(CCSize(node:getContentSize().width, cellHeight))
            util.alignNode(bgSprite, cell, poi)
            util.alignNode(infoLabel, cell, poi, 20)
            return cell
        end
        local scrollGrid = widget.newScrollLinear{
            contentSize = node:getContentSize(),
            cellAtIndexListener = cellAtIndex,
            contentParams = {
                column = 1,
                -- tapListener = tapListener,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
        }
        node:addChild(scrollGrid)
        return scrollGrid
    end
	local function initViews()
		_chatGrid = createChatGrid()
        _editBox = createEditBox()
        enableRefreshMessage(true)
	end

--[[ ## public fuction --]]
    local function updateData( data )
        -- 清空内容
        local chatCount = _chatGrid:getItems() and #_chatGrid:getItems() or 0
        _chatData = {}
        for i, v in ipairs(data.mcontent) do
            if v.mtyp == MAIL_TYPE.USER and v.mid == _fid then
                _chatData = v
                _chatGrid:setItems(_chatData.detailed)
                if chatCount ~= #v.detailed then
                    _chatOffset = nil
                end
                if _chatOffset then
                    _chatGrid:getScrollView():setContentOffset(_chatOffset)
                elseif _chatGrid:getLinearLayer():getContentSize().height > _chatGrid:getContentSize().height then
                    _chatGrid:getScrollView():setContentOffset(ccp(0, 0))
                end
                break
            end
        end
    end
--[[ ## MAINs --]]
	initViews()
    _node.updateData = updateData
end

return M