--MapExploreSelectLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[MapExploreSelectLayerView]"
	local _node, _nodeTable = node, luaValue
    local _data = _node._DATA
    local _mapSn = _data.sn
    local LevelDao  = require("db.leveldb")
    local GoodsMaster = require("cardui.GoodsMaster")
    local GoodsEntity = require("model.goods.GoodsEntity")
    local CardEntity = require("model.card.CardEntity")
    local CardMaster = require("cardui.CardMaster")
    local _levelScrollGrid, _rewardScrollGrid, _progressBar, _cdTimer
    local _selectIndex
    local _scrollContentOffset

    local REWARD_STATUS_LOCK = 0
    local REWARD_STATUS_UNGET = 1
    local REWARD_STATUS_GET = 2
--[[ ## local fuction --]]
    -- 体力耗损
    local function costSp( sp, offset )
        local fontSize = 20
        offset = offset or 20
        local node = CCBCenter.getNodeByTag(_node, 50302)
        node:removeAllChildrenWithCleanup(true)

        local params = {{text = _s("MAP_EXPLORE_COST"), size = fontSize, },
            {text = sp, size = fontSize, color = ccc3(141, 198, 63)},
            {text = _s("MAP_EXPLORE_SP"), size = fontSize, },}

        local  linearLabel = widget.newLinearLabel({paramTable = params})
        util.alignNode(linearLabel,node,display.CENTER)
        node:addChild(linearLabel)
        
    end
    local function setBtnExplore( enable, time )
        _nodeTable["btnExplore"]:setEnabled(enable)
        local function cdTime( num )
            local str
            local minute = math.ceil(num / 60)
            if minute <= 1 then
                str = _s("MAP_FRIEND_LESS_THAN_ONE")
            else
                str = format(_s("MAP_FRIEND_TIME"), minute)
            end
            return str
        end
        if time > 0 and not enable then
            _nodeTable["mBtnExploreLabel"]:setString(cdTime(time))
        else
            _nodeTable["mBtnExploreLabel"]:setString(_s("MAP_EXPLORE_LEVEL"))
        end
    end
    -- 倒计时
    local function createCdTimer( )
        local layer = CCBCenter.getNodeByTag(_node, 603)
        local timer = CountDownView.new{ 
            format = "[h]:[mm]:[ss]",
            labelParams = {size = 24, color = BUTTON_COLOR3.BROWN},
            listener = function()
                _cdTimer:stop()
                CCBCenter.sendLuaMessage(MSG_ID.MSG_MAPEXPLORESELECTLAYER_ACTION_ONFETCHEXPLOREREWARD, nil, _data)
            end
        }
        layer:addChild(timer)
        util.alignNodeWithPadding(timer, layer:getContentSize(), display.CENTER)
        timer:setVisible(false)
        return timer
    end
    -- 完成度滚动条
    local function createProgress( )
        local layer = CCBCenter.getNodeByTag(_node, 502)
        --增加经验ProgressBar
        local progress = widget.newProgressBar{
            frontSprintParams = {file = "map/explore/selectexplore/img_progress_green.png",},
            bgSprintParams = {file = "map/explore/selectexplore/bg_progress.png"},
            percentage = 0,
            ttfLabelParams = {size = 20, },
        }
        progress:ignoreAnchorPointForPosition(true)
        layer:addChild(progress)
        return progress
    end
    -- 奖励格子
    local function createRewardScrollGrid( )
        local layer = CCBCenter.getNodeByTag(_node, 4)
        local function createItemCellListener( grid, index, item )
            local cell = widget.newLayer()
            cell:setContentSize(grid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)

            local bgSprite = widget.newSprite{file = "goods/goodBack.png"}
            cell:addChild(bgSprite, 0)
            util.alignNode(bgSprite, cell:getContentSize(), display.CENTER)

            local goodsView
            if item.status == REWARD_STATUS_GET then
                if item.brlist.no > 10000 then
                    local entity = GoodsEntity.new(item.brlist)
                    entity.type = GoodsMaster.MATERIAL
                    entity.showDetail = true
                    entity.showFrame = true
                    goodsView = GoodsMaster.new(entity)
                elseif item.brlist.no < 10000 then
                    local entity = CardEntity.new{id = item.brlist.no}
                    entity.type = CardMaster.MINISTARCARD
                    entity.showDetail = true
                    goodsView = CardMaster.new(entity)
                    goodsView:setScale(0.76)
                    -- 碎片
                    if item.brlist.no > 5000 then
                        local chipSprite = widget.newSprite{file = "fight/result3/f_r3_cardchip.png"}
                        goodsView:addChild(chipSprite)
                        util.alignNode(chipSprite, goodsView:getContentSize(), display.CENTER)
                    end
                end
                if item.brlist.count > 1 then
                    local countLabel = ui.newTTFLabel{text = item.brlist.count, size = 22}
                    cell:addChild(countLabel, 2)
                    util.alignNode(countLabel, cell, display.BOTTOM_RIGHT, 6)
                end
            elseif item.status == REWARD_STATUS_UNGET then
                goodsView = widget.newSprite{file = "map/explore/selectexplore/img_question_mask.png"}
            elseif item.status == REWARD_STATUS_LOCK then
                goodsView = widget.newSprite{file = "map/explore/selectexplore/img_lock.png"}
            end
            
            cell:addChild(goodsView, 1)
            util.alignNode(goodsView, cell, display.CENTER)

            return cell
        end
        local function onBtnClick(grid, index, item, x, y)
            if item.status == REWARD_STATUS_UNGET then
                NetHelper.QuickToast(_s("MAP_EXPLORE_UNGET"))
            elseif item.status == REWARD_STATUS_LOCK then
                if index == 3 then
                    NetHelper.QuickToast(_s("MAP_EXPLORE_UNLOCK_NORMAL", _mapSn, _data._RESULT[_selectIndex].subsn))
                elseif index == 4 then
                    NetHelper.QuickToast(_s("MAP_EXPLORE_UNLOCK_HARD", _mapSn, _data._RESULT[_selectIndex].subsn))
                end
            end
        end
        local scrollGrid = widget.newScrollGrid{
            contentSize = layer:getContentSize(),
            contentParams = {
                cellSize = CCSize( 91, 91),
                row = 1,
                createItemCellListener = createItemCellListener,
                tapListener = onBtnClick,
            },
            direction = widget.TABLEVIEW_DIRECTION_HORIZONTAL,
            parentNode = layer,
        }
        return scrollGrid
    end
    -- 关卡信息
    local function updateLevelInfo( item )
        _rewardScrollGrid:setItems(item.rewards)
        -- 完成度
        local progress = item.fin / _data._MAXLV 
        _progressBar:setPercentage( progress * 100 )
        _progressBar:setString(string.format("%d / %d", item.fin, _data._MAXLV))
        -- 体力耗损
        costSp(_data._SP)
        -- 倒计时
        setBtnExplore(progress < 1, item.cd)
    end
    -- 关卡的列表
    local function createLevelScrollGrid( )
        local layer = CCBCenter.getNodeByTag(_node, 3)
        local function createItemCellListener( grid, index, item )
            local cell = widget.newLayer()
            cell:setContentSize(grid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)

            -- 背景
            local bgFile = "map/explore/selectexplore/bg_level_cell_unopen.png"
            if index == _selectIndex then
                bgFile = "map/explore/selectexplore/bg_level_cell_select.png"
            elseif item.isopen == TRUE then
                bgFile = "map/explore/selectexplore/bg_level_cell_open.png"
            end
            
            local bgSprite = widget.newSprite{file = bgFile}
            cell:addChild(bgSprite, 0, 100)
            util.alignNode(bgSprite, cell:getContentSize(), display.CENTER)
            local name = _s("MAP_EXPLORE_UNLOCK")
            if item.isopen == TRUE then
                name = LevelDao.getByIndexes(_mapSn, item.subsn).subname
            end
            -- 名字
            local nameLabel = ui.newTTFLabel{
                text = string.format("%d - %d %s", _mapSn, item.subsn, name),
                color = BUTTON_COLOR3.BROWN,
                size = 22,
            }
            cell:addChild(nameLabel, 1, 100)
            util.alignNode(nameLabel, cell:getContentSize(), display.CENTER)

            return cell
        end
        local function onBtnClick( grid, index, item )
            if _selectIndex == index then
                return
            end
            if item.isopen == FALSE then
                NetHelper.QuickToast(_s("MAP_EXPLORE_UNLOCK_LEVEL", _mapSn, item.subsn - 1))
                return
            end
            -- 旧的背景重置
            local cell = grid:cellAt(_selectIndex)
            local bgSprite = cell:getChildByTag(100)
            if bgSprite then
                bgSprite:removeFromParentAndCleanup(true)
            end
            bgSprite = widget.newSprite{
                file = "map/explore/selectexplore/bg_level_cell_open.png",
            }
            cell:addChild(bgSprite, 0, 100)
            util.alignNode(bgSprite, cell:getContentSize(), display.CENTER)
            -- 新的背景修改
            local newCell = grid:cellAt(index)
            local bgSprite = newCell:getChildByTag(100)
            if bgSprite then
                bgSprite:removeFromParentAndCleanup(true)
            end
            bgSprite = widget.newSprite{
                file = "map/explore/selectexplore/bg_level_cell_select.png",
            }
            newCell:addChild(bgSprite, 0, 100)
            util.alignNode(bgSprite, newCell:getContentSize(), display.CENTER)

            _selectIndex = index
            updateLevelInfo(item)
        end
        local scrollGrid = widget.newScrollGrid{
            contentSize = layer:getContentSize(),
            contentParams = {
                cellSize = CCSize( 297, 59),
                column = 1,
                createItemCellListener = createItemCellListener,
                tapListener = onBtnClick,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
            parentNode = layer,
        }
        return scrollGrid
    end
    local function checkHelperInfo(  )
        if ME:getTriggerMsgEntity():isShowExploreHelper() then
            HelperView2.setCurIndex(110030)
        end
        HelperView2.showBetween(_node, nil, {110030, 110070})
    end
	local function initViews()
		_levelScrollGrid = createLevelScrollGrid( )
        _rewardScrollGrid = createRewardScrollGrid( )
        _progressBar = createProgress()
        _cdTimer = createCdTimer( )
        checkHelperInfo(  )
	end

--[[ ## public fuction --]]
    local function updateExploreData( result )
        if not _selectIndex then
            for i,v in ipairs(result) do
                if v.isopen == TRUE then
                    _selectIndex = i
                end
            end
        end
        _levelScrollGrid:setItems(result)
        if result[_selectIndex] then
            updateLevelInfo( result[_selectIndex] )
        end
        if _scrollContentOffset then
            _levelScrollGrid:setContentOffset(_scrollContentOffset)
        elseif #result > 0 then
            _levelScrollGrid:scrollToRow(_selectIndex, 0.2)
        end
        -- 取最短时间倒计时
        local cdTime
        local progress = 0
        for i, v in ipairs(result) do
            if v.fin >= _data._MAXLV and v.cd > 0 then
                if not cdTime or (cdTime and v.cd < cdTime) then
                    cdTime = v.cd
                end
            end
        end
        if cdTime and cdTime > 0 then
            Log.d(TAG, "start update in %d", cdTime)
            _cdTimer:start(cdTime)
        else
            _cdTimer:stop()
        end
    end
    local function getSelectLevel( )
        local result
        if _data._RESULT then
            local level = _data._RESULT[_selectIndex]
            result = {
                costSp    = _data._SP,          -- 挑战需要sp
                maxLv     = _data._MAXLV,       -- 挑战次数
                mapSn     = _mapSn,             -- 大地图id
                mapSubsn  = level.subsn,        -- 小地图id
                progress  = level.fin,          -- 进度
            }
        end
        return result
    end
    local function saveScrollOffset( first )
        if not first then
            _scrollContentOffset = _levelScrollGrid:getContentOffset()
        end
    end
--[[ ## MAINs --]]
	initViews()
    _node.updateExploreData   = updateExploreData
    _node.getSelectLevel   = getSelectLevel
    _node.saveScrollOffset   = saveScrollOffset
end

return M