--RankingsLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[RankingsLayerView]"
	local _node, _nodeTable = node, luaValue
    local _rankScrollGrid, _rankTab
    local _heroRole
    local _curRankTyp = 1
    local _maxNumLength = 1
    local _data
    -- 1 等级榜 2 天赋榜 3 总atk榜 4 总hp榜 5 财富榜 6 膜拜榜
    local RANK_TYPE = {
        {typ = 1, text = _s("RANK_TYPE_LV"), valueFun = "lv", unit = _s("RANK_TYPE_LV"),},
        {typ = 2, text = _s("RANK_TYPE_TALENT"), valueFun = "talvalue", unit = _s("RANK_TYPE_TALENT"),},
        {typ = 3, text = _s("RANK_TYPE_TOTALATK"), valueFun = "totalatk", unit = _s("RANK_UNIT_TOTALATK"),},
        {typ = 4, text = _s("RANK_TYPE_TOTALHP"), valueFun = "totalhp", unit = _s("RANK_UNIT_TOTALHP"),},
        {typ = 5, text = _s("RANK_TYPE_GOLD"), valueFun = "gold", unit = _s("RANK_UNIT_GOLD"),},
        {typ = 6, text = _s("RANK_TYPE_WORSHIPTIMES"), valueFun = "worshiptimes", unit = _s("RANK_TYPE_WORSHIPTIMES"),},
    }
    local updateListTitle
--[[ ## local fuction --]]
    
    -- 获得当前类型配置信息
    local function getCurTypeInfo( typ )
        for i, v in ipairs(RANK_TYPE) do
            if v.typ == typ then
                return v
            end
        end
        return RANK_TYPE[1]
    end
    local function createTabButtons( )
        local layer = CCBCenter.getNodeByTag(_node, 302)
        local function onTabClick( index, button )
            SOUND:playEffectButton1()
            _curRankTyp = RANK_TYPE[index].typ
            updateListTitle()
            CCBCenter.sendLuaMessage(MSG_ID.MSG_RANKINGSLAYER_ACTION_ONFETCHRANKINGS, nil, {typ = _curRankTyp})
        end

        local buttons = { }
        for i, v in ipairs(RANK_TYPE) do
            local btnRank = widget.newButton{   
                normalImage = "rankings/btn_tab_normal.png",
                selectImage = "rankings/btn_tab_select.png",
                disableImage = "rankings/btn_tab_normal.png",
                contentSize = CCSize(101, 68),
                labelParams = {text = _s("RANK_ANNOUNCEMENT", v.text), color = BUTTON_COLOR3.BLUE, padding = {left = -4}}
            } 
            table.insert(buttons, btnRank)   
        end

        local tab = widget.newTab{
            items = buttons,
            selectIndex = 1,
            listener = onTabClick,
            direction = DIRECTION_VERTICAL,
            padding = 4,
        }
        layer:addChild(tab)
        util.alignNode(tab, layer)
        return tab
    end
    local function createRankScrollGrid( )
        local layer = CCBCenter.getNodeByTag(_node, 30102)
        local function createItemCellListener( grid, index, item )
            local cell = widget.newLayer()
            cell:setContentSize(grid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)

            if item.id == ME:getUserid() then
                local bgSprite = widget.newSprite{file = "rankings/bg_ranking_list.png"}
                cell:addChild(bgSprite, 0)
                util.alignNode(bgSprite, cell)
            end

            -- 横线
            local lineLayer = widget.newLayer{color = ccc4(255, 255, 255, 255), contentSize = CCSize(layer:getContentSize().width - 20, 1)}
            cell:addChild(lineLayer)
            util.alignNode(lineLayer, cell, display.BOTTOM_CENTER, 2)
            -- 排名
            if index < 4 then
                local rankSprite = widget.newSprite{file = "rankings/rank_no" .. index .. ".png"}
                cell:addChild(rankSprite)
                util.alignNodeWithPadding(rankSprite, cell, display.CENTER_LEFT, {left = 42, top = -1} )    
            else
                local rankLabel = ui.newTTFLabel{
                    text = index,
                    size = 22,
                }
                cell:addChild(rankLabel)
                util.alignNode(rankLabel, cell, display.CENTER, {left = -202})
            end

            -- 姓名
            local nameLabel = ui.newTTFLabel{
                text = item.name,
                size = 22,
            }
            cell:addChild(nameLabel)
            util.alignNode(nameLabel, cell, display.CENTER_LEFT, {left = 190})
            local curRankTypInfo = getCurTypeInfo(_curRankTyp)
            -- 数值
            local valueLabel = ui.newTTFLabel{
                text = item[curRankTypInfo.valueFun],
                size = 22,
            }
            cell:addChild(valueLabel)
            util.alignNode(valueLabel, cell, display.CENTER_LEFT, {left = 435 - (5 * _maxNumLength)})

            return cell
        end
        local function onBtnClick(grid, index, item, x, y)
            if item.id ~= ME:getUserid() then
                CCBCenter.loadCCB("MSG_FRIENDPANELLAYER_MODEL", "ccb.c.friend.FriendPanelLayer", {fid = item.id, fnNextView = function ( )
                    CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
                end})
            end
        end
        local scrollGrid = widget.newScrollGrid{
            contentSize = layer:getContentSize(),
            contentParams = {
                cellSize = CCSize(layer:getContentSize().width, 59) ,
                column = 1,
                createItemCellListener = createItemCellListener,
                tapListener = onBtnClick,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
            parentNode = layer,
        }
        return scrollGrid
    end
    local function updateHeroImage( data )
        local layer = CCBCenter.getNodeByTag(_node, 201)
        layer:removeAllChildrenWithCleanup(true)
        -- 名字
        _nodeTable["mHeroNameLabel"]:setString(data.name)
        -- 形象
        local role = Player.new(data.sex, data.head, data.equiplst, {randomAction = true})
        layer:addChild(role)
        util.alignNode(role, layer, display.BOTTOM_CENTER, -26)
        role:flipX()
    end
    function updateListTitle( )
        _nodeTable["mValueLabel"]:setString(getCurTypeInfo(_curRankTyp).unit)
    end
    local function updateNumMaxLength( num )
        local str = tostring(num)
        _maxNumLength = string.utf8len(str)
    end
	local function initViews()
		_rankScrollGrid = createRankScrollGrid()
        _rankTab = createTabButtons()
        updateListTitle( )
	end

--[[ ## public fuction --]]
    local function canWorship( )
        if _data and _data.Players[1].id ~= ME:getUserid() then
            return true
        else
            NetHelper.QuickToast(_s("RANK_CANNOT_WORSHIP_SELF"))
            return false
        end
    end
    local function getRankType( )
        return _curRankTyp
    end
    local function updateData( data )
        _data = data
        -- 我的信息
        _nodeTable["mMyRankingLabel"]:setString(data.MyRank)
        _nodeTable["mMyNameLabel"]:setString(ME:getName())
        _nodeTable["mMyValueLabel"]:setString(data.Hero[getCurTypeInfo(_curRankTyp).valueFun])
        if #data.Players > 0 then
            updateHeroImage(data.Players[1])
            _nodeTable["btnWorship"]:setEnabled(true)
            local curRankTypInfo = getCurTypeInfo(_curRankTyp)
            local heroVlaue = data.Players[1][curRankTypInfo.valueFun]
            _nodeTable["mHeroValueLabel"]:setString(string.format("%s：%.0f", curRankTypInfo.unit, heroVlaue))
            updateNumMaxLength(heroVlaue)
        else
            _nodeTable["btnWorship"]:setEnabled(false)
            _nodeTable["mHeroValueLabel"]:setString("")
        end
        _rankScrollGrid:setItems(data.Players)
    end
--[[ ## MAINs --]]
	initViews()
    _node.updateData = updateData
    _node.getRankType = getRankType
    _node.canWorship = canWorship
end



return M