--ShopMysteryLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[ShopMysteryLayerView]"
	local _node, _nodeTable = node, luaValue
    local ShopViewUtil = require("cardui.ShopViewUtil")
    local _shopScrollGrid
    local _titleLabel, _playerStatus
    local HEAD_TYPE_MYSTERYSHOP = 4 -- 神秘商人
    local HEAD_TYPE_TRAVELSHOP  = 5 -- 旅行商人
    local BUY_STATE = {
        NOT_BUY = 2,
        ALREADY_BUY = 1,
    }
--[[ ## local fuction --]]
    local function isEnough( goods )
        if goods.state == BUY_STATE.ALREADY_BUY then
            NetHelper.QuickToast(_s("SHOP_ALREADY_BUY"))
        elseif goods.buytyp == 1 and goods.costvalue > ME:getDiamond() then
            NetHelper.QuickToast(_s("TEAM_DIAMOND_NOT_ENOUGH"))
            return false 
        elseif goods.buytyp == 2 and goods.costvalue > ME:getGold() then
            NetHelper.QuickToast(_s("TALENT_TOAST_NOT_ENOUGH_GOLD"))
            return false 
        else
            return true 
        end
    end
    local function createStarScrollPage( )
        local function createShopGoodsViewListener( grid, index, item, cell )
            local view 
            if item.id then
                local ctype = item.buytyp == 1 and ShopViewUtil.CURRENCY_TYPE_DIAMOND or ShopViewUtil.CURRENCY_TYPE_GOLD
                view = ShopViewUtil:createShopGoodsView{
                    title = item.title,
                    imageDesc = "X" .. item.count,
                    imageNode = ShopViewUtil:getGoodsAndStarView(item),
                    ctype = ctype,
                    value = item.costvalue,
                }
                view:setButtonTTF(item.state == BUY_STATE.NOT_BUY and _s("SHOP_BUY") or _s("SHOP_ALREADY_BUY"))
                view:setButtonEnable(item.state == BUY_STATE.NOT_BUY)
            end
            return view
        end
        local function onBtnClick( grid, index, goods )
            Log.d(TAG, "去购买")
            -- 购买
            if goods.id and goods.state == BUY_STATE.NOT_BUY then
                if isEnough(goods) then
                    if goods.starType == STARTYPE.SP and ME:getCursp() >= ME:getActlimit() then
                        NetHelper.QuickToast(_s("ROLE_SP_FULL"))
                    else
                        AlertLayer.new{
                            btnStr = {_s("COMMON_OK"), _s("COMMON_CANCEL")},
                            btnColor = {AlertLayer.BTN_COLOR_YELLOW, AlertLayer.BTN_COLOR_BLUE, },  
                            titleStr = _s("SHOP_BUY"),  --标题
                            content = string.format(_s("SHOP_BUY_CONFIRM"), goods.title, goods.count, goods.costvalue),  --内容
                            btnClickEvent = {
                                function (  )  --按钮1事件
                                    local requestData = {}
                                    requestData.proid = goods.id
                                    --TODO
                                    CCBCenter.sendLuaMessage(MSG_ID.MSG_SHOPMYSTERYLAYER_ACTION_ONFETCHMYSTERYSHOP, nil, requestData)
                                end,
                            },
                        }   
                    end
                        
                end
            end
        end
        local function onGoodsClick( grid, index, item )
            Log.d(TAG, "查看详细")
            ShopViewUtil:showItemAlertView(item)
        end
        local scroll = ShopViewUtil:createShopScrollPage{
            parentNode = CCBCenter.getNodeByTag(_node, 103),
            createShopGoodsViewListener = createShopGoodsViewListener,
            onBtnClick = onBtnClick,
            onGoodsClick = onGoodsClick,
            column = 4,
            indicatorHight = 30,
        }
        return scroll
    end
    local function createTitleLabel( )
        local node = CCBCenter.getNodeByTag(_node, 10502)
        local titleLabel = widget.newTTFLabelWithShadow{
            text = _s("MAP_EXPLORE_TRAVEL_SHOP"),
            size = 30,
        }
        node:addChild(titleLabel)
        util.alignNode(titleLabel, node)
        return titleLabel
    end
    -- 更新头像 & 文字
    local function updateShopType( )
        local headNode = CCBCenter.getNodeByTag(_node, 104)
        headNode:removeAllChildrenWithCleanup(true)
        local headFile = _nodeTable._TYPE == HEAD_TYPE_MYSTERYSHOP and "shop/myster/head_bear.png" or "shop/myster/head_car.png"
        local headSprite = widget.newSprite{file = headFile}
        headNode:addChild(headSprite)
        util.alignNode(headSprite, headNode)
        local title = _nodeTable._TYPE == HEAD_TYPE_MYSTERYSHOP and _s("MAP_EXPLORE_MYSTERY_SHOP") or _s("MAP_EXPLORE_TRAVEL_SHOP")
        if not _titleLabel then
            _titleLabel = createTitleLabel( )
        end
        _titleLabel:setString(title)
    end
	local function initViews()
        if _nodeTable._TYPE then
            updateShopType( )
        end
        -- 内容
        _shopScrollGrid = createStarScrollPage( )
        -- 玩家状态
        _playerStatus = ShopViewUtil:createShopGoldDiamondStatus{
            parentNode = CCBCenter.getNodeByTag(_node, 106),
        }
	end

--[[ ## public fuction --]]
    function refreshPlayerStatus( )
        _playerStatus.refresh()
    end
    local function updateData( data )
        if _nodeTable._TYPE ~= data.shoptyp then
            _nodeTable._TYPE = data.shoptyp
            updateShopType( )
        end
        _shopScrollGrid:setItems(data.shops)
    end
--[[ ## MAINs --]]
	initViews()
    _node.updateData = updateData
    _node.refreshPlayerStatus = refreshPlayerStatus
end

return M