--TeamMemberLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[TeamMemberLayerView]"
	local _node, _nodeTable = node, luaValue
    local _memberScrollGrid, _editbox
    local _data
    local TeamViewUtil = require("cardui.TeamViewUtil")
    local STATUS_SIGN_UP = {
        NOT            = 0,    -- 未报名（显示“报名”）
        ALREADY        = 1,    -- 已经报名（显示“取消报名”） 
        NOT_BATTLE     = 0,    -- 0 未开战（备战期）
        WAIT_BATTLE    = 1,    -- 1 待战期
        KNOCKOUT       = 2,    -- 2 已开战（淘汰赛）
        QUARTER_FINALS = 3,    -- 3 已开战（八强赛）
        SEMIFINAL      = 4,    -- 4 已开战（半决赛）
        FINALS         = 5,    -- 5 已开战（决赛）
    }  
    local STATUS_LEADER = {
        NOT   = 0,
        YES   = 1,
        OTHER = 2,
    }
    local STATUS_BUTTON = {
        EDIT = 1,   -- 1 可编辑
        KICK = 2,   -- 2 可踢出
    }   
    local STATUS_INTER_SERVICE = {
        NO = 0,         -- 0 没进入
        YES = 1,        -- 1 进入
        OPEN = 2,       -- 2 开始比赛
    } 
    -- local canDoEdit
--[[ ## local fuction --]]
    local function createMemberScrollGrid( )
        local layer = CCBCenter.getNodeByTag(_node, 3)
        local function createItemCellListener( grid, index, item )
            local cell = widget.newLayer()
            cell:setContentSize(grid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)

            -- 背景
            local bgFile = item.isleader == TRUE and "team/bg_team_captain.png" or "team/bg_team_member.png"
            local bgSprite = widget.newScale9{
                file = bgFile,
                rect = CCRect(0, 0, 117, 154),
                contentSize = CCSizeAdd(grid:getCellSize(), 0, -10),
            }
            cell:addChild(bgSprite, 0)
            util.alignNode(bgSprite, cell:getContentSize(), display.CENTER_BOTTOM)

            -- 编辑按钮
            if STATUS_BUTTON.EDIT == item.status then
                local editButton = widget.newButton{
                    normalImage = "common/buttons/btn_normal_yellow.png",
                    disableImage = "common/buttons/btn_normal_gray.png",
                    contentSize = CCSize(129, 56),
                    rect = CCRect(0, 0, 80, 58),
                    scale = 1.1,
                    container = layer,
                    labelParams = {text = _s("COMMON_EDIT"), color = BUTTON_COLOR3.BROWN, },
                    listener = function ( button )
                        if _node.canDoEdit( ) then
                            CCBCenter.loadCCB("MSG_CARDFILTERLAYER_MODEL", "ccb.c.cardgroup.CardFilterLayer", {_KEY = MSG_ID.MSG_TEAMMEMBERLAYER_MODEL})
                        end
                    end,
                }
                cell:addChild(editButton, 1)
                util.alignNode(editButton, cell:getContentSize(), display.BOTTOM_RIGHT, 20)
            elseif STATUS_BUTTON.KICK == item.status then
                local contentText = STATUS_INTER_SERVICE.YES == _data.status3 and _s("TEAM_SERVICR_REMOVE") or _s("TEAM_REMOVE_MEMBER_INFO")
                -- 踢人按钮
                local kickButton = widget.newButton{
                    normalImage = "common/buttons/btn_normal_red.png",
                    disableImage = "common/buttons/btn_normal_gray.png",
                    contentSize = CCSize(129, 56),
                    rect = CCRect(0, 0, 80, 58),
                    scale = 1.1,
                    container = layer,
                    discolor = BUTTON_COLOR3.BROWN, 
                    labelParams = {text = _s("TEAM_REMOVE_MEMBER"), color = COLOR3.WHITE, },
                    listener = function ( button )
                        if not _node.isNeedSelect() and _node.canDoEdit( ) then
                            AlertLayer.new{
                                btnStr = {_s("COMMON_OK"), _s("COMMON_CANCEL")},
                                btnColor = {AlertLayer.BTN_COLOR_YELLOW, AlertLayer.BTN_COLOR_BLUE, },  
                                titleStr = _s("TEAM_REMOVE_MEMBER"),  --标题
                                content = contentText,  --内容
                                btnClickEvent = {
                                    function ( )
                                        CCBCenter.sendLuaMessage(MSG_ID.MSG_TEAMMEMBERLAYER_ACTION_ONFETCHKICKTEAMMEMBER, nil, {pid = item.id})
                                    end,
                                },
                            }
                        end
                    end,
                }
                cell:addChild(kickButton, 1)
                util.alignNode(kickButton, cell:getContentSize(), display.BOTTOM_RIGHT, 20)
            end

            -- 头像
            local headNode = widget.newLayer{contentSize = CCSize(91, 91)}
            cell:addChild(headNode, 2)
            util.alignNode(headNode, cell:getContentSize(), display.TOP_LEFT, {left = 10})

            local bgHead = widget.newSprite{
                file = "common/bg/bg_card_head.png",
            }
            headNode:addChild(bgHead, 0)
            util.alignNode(bgHead, headNode:getContentSize(), display.CENTER)
            local bgFrameHead = widget.newSprite{
                file = "common/bg/bg_card_head_frame.png",
            }
            headNode:addChild(bgFrameHead, 1)
            util.alignNode(bgFrameHead, headNode:getContentSize(), display.CENTER) 

            local heroHead = PlayerHead.new(item.sex, item.head, item.equiplst):createRect()
            headNode:addChild(heroHead, 2)
            util.alignNode(heroHead, headNode:getContentSize(), display.CENTER, {top = -3})
            heroHead:setScale(0.56)
            if item.id ~= ME:getUserid() and _node.getSerId() == 0 then
                -- 绑定头像点击
                widget.bindTouchEvent{
                    layer = headNode,
                    clickListener = function ( )
                        CCBCenter.loadCCB("MSG_FRIENDPANELLAYER_MODEL", "ccb.c.friend.FriendPanelLayer", {fid = item.id, fnNextView = function ( )
                            CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
                        end})
                    end,
                    container = layer,
                }
            end
            -- 等级
            local lvLabel = widget.newTTFLabelWithShadow{
                text = "LV:" .. item.lv,
                size = 18,
            }
            headNode:addChild(lvLabel, 3)
            util.alignNode(lvLabel, headNode, display.TOP_RIGHT, 6)
            -- 名字
            local nameNode = widget.newNode{contentSize = CCSize(194, 45)}
            cell:addChild(nameNode, 1)
            util.alignNode(nameNode, cell:getContentSize(), display.TOP_LEFT, {left = 120, top = 20})
            local bgNameFile = item.isleader == TRUE and "team/bg_team_captain_text.png" or "team/bg_team_member_text.png"
            local bgName = widget.newScale9{
                file = bgNameFile,
                rect = CCRect(0, 0, 90, 45),
                contentSize = nameNode:getContentSize(),
            }
            nameNode:addChild(bgName, 0)
            util.alignNode(bgName, nameNode:getContentSize(), display.CENTER)
            local nameLabel = ui.newTTFLabel{text = item.name,}
            nameNode:addChild(nameLabel, 1)
            util.alignNode(nameLabel, nameNode:getContentSize(), display.CENTER_LEFT, 10)

            -- 上次登陆时间
            local dateNode = widget.newNode{contentSize = CCSize(540, 45)}
            cell:addChild(dateNode, 1)
            util.alignNode(dateNode, cell:getContentSize(), display.TOP_LEFT, {left = 380, top = 20})
            local bgDate = widget.newScale9{
                file = bgNameFile,
                rect = CCRect(0, 0, 90, 45),
                contentSize = dateNode:getContentSize(),
            }
            dateNode:addChild(bgDate, 0)
            util.alignNode(bgDate, dateNode:getContentSize(), display.CENTER)
            local dateLabel = ui.newTTFLabel{text = _s("TEAM_LASR_DATE", item.lastdate),}
            dateNode:addChild(dateLabel, 1)
            util.alignNode(dateLabel, dateNode:getContentSize(), display.CENTER_LEFT, 10)

            -- 卡牌
            item.parentNode = layer
            local cardsLayer = TeamViewUtil:createCardLayer(item)
            cell:addChild(cardsLayer)
            util.alignNode(cardsLayer, cell:getContentSize(), display.CENTER_LEFT, {left = 120, top = 28})

            return cell
        end
        -- local function onBtnClick(grid, index, item, x, y)
            
        -- end
        local scrollGrid = widget.newScrollGrid{
            contentSize = layer:getContentSize(),
            contentParams = {
                cellSize = CCSize( 952, 170),
                column = 1,
                createItemCellListener = createItemCellListener,
                -- tapListener = onBtnClick,
                padding = {top = 4, left = 4, right = 4,}
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
            parentNode = layer,
        }
        return scrollGrid 
    end

    local function updateTeamView( data )
        -- 文字
        _nodeTable["mTeamNameLabel"]:setString(data.tname)
        _nodeTable["mTeamIdLabel"]:setString(data.tno)
        _nodeTable["mTeamScoreLabel"]:setString(data.point)
        _editbox:setString(data.declaration)

        -- 按钮 status1 状态 0 未报名（队长显示“报名”，队员显示“未报名”） 1 已经报名（队长显示“取消报名”，队员显示“已报名”）
        local function setButtonInfo_( isleader )
            if isleader then
                if STATUS_SIGN_UP.NOT == data.status1 then
                    _node:setButtonTitle( "btnSignUp", _s("TEAM_WAR_SIGN_UP"))
                elseif STATUS_SIGN_UP.ALREADY == data.status1 then
                    _node:setButtonTitle( "btnSignUp", _s("TEAM_WAR_UN_SIGN_UP"))
                end
            else
                _nodeTable["btnSignUp"]:setEnabled(false)
                if STATUS_SIGN_UP.NOT == data.status1 then
                    _node:setButtonTitle( "btnSignUp", _s("TEAM_NOT_SIGN_UP"), nil, CCControlStateDisabled)
                elseif STATUS_SIGN_UP.ALREADY == data.status1 then
                    _node:setButtonTitle( "btnSignUp", _s("TEAM_ALREADY_SIGN_UP"), nil, CCControlStateDisabled)
                end
            end
        end
        
        if STATUS_LEADER.YES == data.isleader then
            _editbox:setButtonVisible(true)
            _nodeTable["btnViewApplication"]:setVisible(true)
            _nodeTable["btnWorldRecruits"]:setVisible(true)
            _nodeTable["btnDismissal"]:setVisible(false)
            _nodeTable["btnSignUp"]:setVisible(true)
            _nodeTable["btnExitTeam"]:setVisible(true)
            setButtonInfo_(true)
        elseif STATUS_LEADER.NOT == data.isleader then
            _editbox:setButtonVisible(false)
            _nodeTable["btnViewApplication"]:setVisible(false)
            _nodeTable["btnWorldRecruits"]:setVisible(false)
            _nodeTable["btnDismissal"]:setVisible(false)
            _nodeTable["btnSignUp"]:setVisible(true)
            _nodeTable["btnExitTeam"]:setVisible(true)
            setButtonInfo_(false)
        elseif STATUS_LEADER.OTHER == data.isleader then
            _editbox:setButtonVisible(false)
            _nodeTable["btnViewApplication"]:setVisible(false)
            _nodeTable["btnWorldRecruits"]:setVisible(false)
            _nodeTable["btnDismissal"]:setVisible(false)
            _nodeTable["btnSignUp"]:setVisible(false)
            _nodeTable["btnExitTeam"]:setVisible(false)
            _nodeTable["mServiceNameLabel"]:setVisible(true)
        end
        _nodeTable["mServiceNameLabel"]:setString(_s("TEAM_SERVICR", data.sername))
        -- if data.status2 ~= 0 then
            -- _editbox:setButtonVisible(false)
            -- _node:setButtonTitle( "btnSignUp", _s("TEAM_ALREADY_SIGN_UP"))
            -- _nodeTable["btnViewApplication"]:setEnabled(false)
            -- _nodeTable["btnWorldRecruits"]:setEnabled(false)
            -- _nodeTable["btnDismissal"]:setEnabled(false)
            -- _nodeTable["btnSignUp"]:setEnabled(false)
            -- _nodeTable["btnExitTeam"]:setEnabled(false)
        -- else
            -- _editbox:setButtonVisible(true)
            -- _nodeTable["btnViewApplication"]:setEnabled(true)
            -- _nodeTable["btnWorldRecruits"]:setEnabled(true)
            -- _nodeTable["btnDismissal"]:setEnabled(true)
            -- _nodeTable["btnSignUp"]:setEnabled(true)
            -- _nodeTable["btnExitTeam"]:setEnabled(true)
        -- end
        if TRUE == data.showadd then
            _nodeTable["btnJoin"]:setVisible(true)
        else
            _nodeTable["btnJoin"]:setVisible(false)
        end
        -- 需要配置卡牌
        if TRUE == data.needselect then
            CCBCenter.loadCCB("MSG_CARDFILTERLAYER_MODEL", "ccb.c.cardgroup.CardFilterLayer", {_KEY = MSG_ID.MSG_TEAMMEMBERLAYER_MODEL, _ISNEED = true,})
        end
        -- 需要pk
        _nodeTable["btnPk"]:setVisible(TRUE == data.showpk)
    end
    local function updateInfo( data )
        _data = data
        updateTeamView(data)
        _memberScrollGrid:setItems(data.teamers)
    end
    local function saveDeclaration( declaration )
        CCBCenter.sendLuaMessage(MSG_ID.MSG_TEAMMEMBERLAYER_ACTION_ONFETCHWRITETEAMDECLARATION, nil, {content = declaration,})
    end
    
	local function initViews()
		_editbox = TeamViewUtil:createEditBoxWithButton{
            parentNode = CCBCenter.getNodeByTag(_node, 206),
            onSaveClick = saveDeclaration,
        }
        _memberScrollGrid = createMemberScrollGrid()   
        -- 按钮
        _editbox:setButtonVisible(false)
        _nodeTable["btnViewApplication"]:setVisible(false)
        _nodeTable["btnWorldRecruits"]:setVisible(false)
        _nodeTable["btnDismissal"]:setVisible(false)
        _nodeTable["btnSignUp"]:setVisible(false)
        _nodeTable["btnJoin"]:setVisible(false)
        _nodeTable["btnExitTeam"]:setVisible(false)
        _nodeTable["btnPk"]:setVisible(false)
        _nodeTable["mServiceNameLabel"]:setVisible(false)
	end

--[[ ## public fuction --]]
    local function canDoEdit( )
        if not _data then
            NetHelper.QuickToast(_s("NETWORK_NOT_GOOD"))
            return false
        end
        if _data.status2 ~= 0 then
            NetHelper.QuickToast(_s("TEAM_CANNOT_EDIT"))
            return false
        else
            if STATUS_INTER_SERVICE.OPEN == _data.status3 then
                NetHelper.QuickToast(_s("TEAM_CANNOT_EDIT"))
                return false
            else
                return true
            end
            
        end
    end
    local function isInterServiceStatus( )
        return STATUS_INTER_SERVICE.YES == _data.status3
    end
    local function getCount( )
        local count = 0
        if _data and _data.teamers then
            count = #_data.teamers
        end
        return count
    end
    local function getTeamId( )
        if not _data or not _data.tid then
            return nil
        else
            return _data.tid
        end
    end
    local function getSerId( )
        if not _data or not _data.serid then
            return nil
        else
            return _data.serid
        end
    end
    local function getTeamNo( )
        if not _data or not _data.tno then
            return nil
        else
            return _data.tno
        end
    end
    local function getTeamName( )
        if not _data or not _data.tname then
            return nil
        else
            return _data.tname
        end
    end
    local function getTeamDeclaration( )
        if not _data or not _data.declaration or _data.declaration == "" then
            return ""
        else
            return _data.declaration .. "!"
        end
    end
    local function isNeedSelect( )
        if not _data then
            NetHelper.QuickToast(_s("NETWORK_NOT_GOOD"))
            return nil
        else
            if _data.needselect == TRUE then
                NetHelper.QuickToast(_s("TEAM_CONFIG_CARD_FIRST"))
                return true
            else
                return false
            end
            
        end
    end
    local function sendTeamRecruitsClick( )
        local function getTeamNameWithFlag_( )
            local result = getTeamName()
            result = string.format("[%s%s%d%s%d]", result, GAMERATE:getChatTeamFlag(), getTeamId(), GAMERATE:getChatTeamFlag(), getSerId())
            return result
        end
        CCBCenter.sendLuaMessage(MSG_ID.MSG_TEAMMEMBERLAYER_ACTION_ONFETCHSENDTEAMAD, nil, {ad = _s("TEAM_SEND_TEAM_AD", getTeamDeclaration(), getTeamNameWithFlag_(), getTeamNo())})
    end
    local function getSignUpStatus( )
        if not _data then
            return nil
        else
            if STATUS_SIGN_UP.NOT == _data.status1 then
                return TRUE
            elseif STATUS_SIGN_UP.ALREADY == _data.status1 then
                return FALSE
            end
        end
    end
--[[ ## MAINs --]]
	initViews()
    _node.updateInfo = updateInfo
    _node.getCount = getCount
    _node.getTeamId = getTeamId
    _node.getSerId = getSerId
    _node.getTeamNo = getTeamNo
    _node.getTeamName = getTeamName
    _node.getTeamDeclaration = getTeamDeclaration
    _node.isInterServiceStatus = isInterServiceStatus
    _node.canDoEdit = canDoEdit
    _node.isNeedSelect = isNeedSelect
    _node.getSignUpStatus = getSignUpStatus
    _node.sendTeamRecruitsClick = sendTeamRecruitsClick

end

return M