--TeamWarLayer.lua
local M = {}

function M.bindView(node, luaValue)
	local TAG = "[TeamWarLayerView]"
	local _node, _nodeTable = node, luaValue
    local _cdTimer, _scrollGrid
    local _data
    local TeamViewUtil = require("cardui.TeamViewUtil")

--[[ ## local fuction --]]
    -- 倒计时
    local function createCdTimer( )
        local layer = CCBCenter.getNodeByTag(_node, 3)
        local timer = CountDownView.new{ 
            format = "[d][hh]:[mm]:[ss]",
            labelParams = {size = 30},
            listener = function()
                _cdTimer:stop()
                CCBCenter.sendLuaMessage(MSG_ID.MSG_TEAMWARLAYER_ACTION_ONFETCHTEAMINDEX)
            end
        }
        layer:addChild(timer)
        util.alignNodeWithPadding(timer, layer:getContentSize(), display.CENTER)
        timer:setVisible(false)
        return timer
    end
    -- 队伍列表
    local function createScrollGrid( )
        local layer = CCBCenter.getNodeByTag(_node, 202)
        local function createItemCellListener( grid, index, item )
            local cell = widget.newLayer()
            cell:setContentSize(grid:getCellSize())
            cell:ignoreAnchorPointForPosition(false)

            local nameLabel = ui.newTTFLabel{
                text = item.name,
            }
            cell:addChild(nameLabel)
            util.alignNode(nameLabel, cell, display.CENTER_LEFT, 10)

            return cell
        end
        local function onBtnClick(grid, index, item, x, y)
            CCBCenter.loadCCB("MSG_FRIENDPANELLAYER_MODEL", "ccb.c.friend.FriendPanelLayer", {fid = item.id, fnNextView = function ( )
                CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
            end})
        end
        local scrollGrid = widget.newScrollGrid{
            contentSize = layer:getContentSize(),
            contentParams = {
                cellSize = CCSizeMul(layer:getContentSize(), 1, 0.2) ,
                column = 1,
                createItemCellListener = createItemCellListener,
                tapListener = onBtnClick,
            },
            direction = widget.TABLEVIEW_DIRECTION_VERTICAL,
            parentNode = layer,
        }
        return scrollGrid
    end
    -- 奖励状态tip
    local function createRewardTitle( title )
        local layer = CCBCenter.getNodeByTag(_node, 405)
        layer:removeAllChildrenWithCleanup(true)
        local titleLabel = ui.newTTFLabel{ text = title, size = 30, }
        local titleNode = widget.newNode{contentSize = titleLabel:getContentSize()}
        titleNode:addChild(titleLabel, 1)
        util.alignNode(titleLabel, titleNode, nil, {top = 2})
        local bgSprite = widget.newScale9{
            file = "map/bg_star.png", 
            rect = CCRect(0, 0, 88, 22),
            contentSize = CCSizeAdd(titleNode:getContentSize(), 10, 4)
        }
        titleNode:addChild(bgSprite, 0)
        util.alignNode(bgSprite, titleNode)
        layer:addChild(titleNode)
        util.alignNode(titleNode, layer, display.CENTER_RIGHT, 12)
    end
	local function initViews()
		_nodeTable["btnCreateTeam"]:setVisible(false)
        _nodeTable["btnJoinTeam"]:setVisible(false)
        _nodeTable["btnMyTeam"]:setVisible(false)
        _nodeTable["btnTeamList"]:setVisible(false)
        _nodeTable["btnReward"]:setVisible(false)
        _nodeTable["btnRewardInfo"]:setVisible(true)
        createRewardTitle(_s("TEAM_REWARD_INFO"))
        _cdTimer = createCdTimer()
        _scrollGrid = createScrollGrid()
	end

--[[ ## public fuction --]]
    local function getCreateLimit( )
        if not _data then
            return nil
        else
            local resule = {
                needlv = _data.needlv,
                needdiamond = _data.needdia,
            }   
            return resule
        end
    end
    local function getReward( )
        if not _data then
            return nil
        else
            return _data.reward
        end
    end
    -- 更新数据
    local function updateData( data )
        _data = data
        -- 上届冠军
        _scrollGrid:setItems(data.teamers)
        -- 冠军名称
        _nodeTable["mTeamNameLabel"]:setString(data.tname)
        -- 按钮状态
        if FALSE == data.btnstatus then
            _nodeTable["btnCreateTeam"]:setVisible(true)
            _nodeTable["btnJoinTeam"]:setVisible(true)
            _nodeTable["btnMyTeam"]:setVisible(false)
            _nodeTable["btnTeamList"]:setVisible(false)
        elseif TRUE == data.btnstatus then
            _nodeTable["btnCreateTeam"]:setVisible(false)
            _nodeTable["btnJoinTeam"]:setVisible(false)
            _nodeTable["btnMyTeam"]:setVisible(true)
            _nodeTable["btnTeamList"]:setVisible(true)
        end
        -- 倒计时状态
        _cdTimer:setVisible(true)
        _cdTimer:setFormat(TeamViewUtil:getFightStatusText(data.status) .. _s("TEAM_WAR_TIME_FORMAT"))

        if 0 == data.status then
            _node:setButtonTitle( "btnSeasonReview", _s("TEAM_LAST_SEASON_VIEW"))
            _nodeTable["mLastSeasonLabel"]:setString(_s("TEAM_LAST_SEASON_CHAMPIONS"))
        else
            _node:setButtonTitle( "btnSeasonReview", _s("TEAM_THIS_SEASON_VIEW"))
            _nodeTable["mLastSeasonLabel"]:setString(_s("TEAM_THIS_SEASON_CHAMPIONS"))
        end
        if data.time and data.time > 0 then
            _cdTimer:start(data.time)
        else
            _cdTimer:modifyTime(0)
            _cdTimer:stop()
        end
        -- 奖励状态
        if TRUE == data.rewardsta then
            createRewardTitle(_s("TEAM_GET_REWARD"))
            _nodeTable["btnReward"]:setVisible(true)
            _nodeTable["btnRewardInfo"]:setVisible(false)
        else
            createRewardTitle(_s("TEAM_REWARD_INFO"))
            _nodeTable["btnReward"]:setVisible(false)
            _nodeTable["btnRewardInfo"]:setVisible(true)
        end
        -- 上届排名
        local rankNode = CCBCenter.getNodeByTag(_node, 5)
        if data.rank and data.rank ~= "" then
            rankNode:setVisible(true)
            _nodeTable["mRankLabel"]:setString(data.rank)
        else
            rankNode:setVisible(false)
        end
    end
--[[ ## MAINs --]]
	initViews()
    _node.updateData = updateData
    _node.getCreateLimit = getCreateLimit
    _node.getReward = getReward
end

return M