-- 卡片数据库操作
require("sqlite3")
local SkillEntity = require("model.card.SkillEntity")
local CardEntity = require("model.card.CardEntity")
CardDB = {}

local TAG = "[CardDB]"
local DB_PATH = util.fullPath( PATH_CARD_DB )


--[=[ ## local params --]=]

--[[ ## local fuction --]]
local _db
local _cardCache = {}

-- 打开数据库
local function openDB( )
	_db = sqlite3.open(DB_PATH)
end

-- 关闭数据库
local function closeDB( )
	_db:close()
end

--[[ ## public fuction --]]

-- 通过id 获取技能等级相关
function CardDB.getSkills( id, evolv )
	if _cardCache[id] and _cardCache[id]["skills"] and _cardCache[id]["skills"][evolv] then
		-- 缓存有则直接返回
		return _cardCache[id]["skills"][evolv]
	else
		local result = {}
		local card
		-- 先获取card缓存
		if _cardCache[id] then
			-- 缓存有则直接返回
			card =  _cardCache[id]
		else
			-- 开
			openDB( )
			for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. id) do 
				card = v
				break
			end
			-- 关
			closeDB( )
		end
		local function getInfo_( skilldata, skilllv )
			local info = ""
			Log.d(TAG, "技能:" .. skilldata.name)
			if skilllv == 0 then
				Log.d(TAG, "进阶等级为0, 返回描述")
				info = skilldata.memo
			elseif evolv > 1 and skilldata.needevo == TRUE then
				Log.d(TAG, "进阶等级为:".. evolv ..", 开始计算")
				local value
				local valueTable = string.split(skilldata["g" .. skilllv], ",")
				local isRange = #valueTable > 1
				local evoRate = skilldata["evo" .. evolv]/100
				for i = 1, #valueTable do
					Log.d(TAG, "初始值:" .. valueTable[i])
					Log.d(TAG, "系数:" .. evoRate)
					value = math.round(tonumber(valueTable[i]) * evoRate)
					Log.d(TAG, "进阶值:" .. value)
					if isRange and i ~= #valueTable then
						value = value .. " ~ "
					end
					info = info .. value
				end
				info = string.format(skilldata.memo, info, info)
			else
				Log.d(TAG, "数值不受进阶影响")
				info = string.format(skilldata.memo, string.gsub(skilldata["g" .. skilllv], ",", " ~ "), string.gsub(skilldata["g" .. skilllv], ",", " ~ "))
			end

			return info
		end
		-- 开
		openDB( )
		-- 技能1
		for v in _db:nrows('SELECT * FROM skilltb WHERE skillno = ' .. card.s1) do 
			local skillEntity = SkillEntity.new{
				skillno = card.s1,
				name = v.name, -- 名称
				info = getInfo_(v, card.s1g),--string.format(v.memo, string.gsub(v["g" .. card.s1g], ",", " ~ "), string.gsub(v["g" .. card.s1g], ",", " ~ ")),	-- 说明	
				level = card.s1g,	
			}
			-- table.insert(result, skillEntity)
			result.skill1 = skillEntity
			break
		end
		-- 技能2
		for v in _db:nrows('SELECT * FROM skilltb WHERE skillno = ' .. card.s2) do 
			local skillEntity = SkillEntity.new{
				skillno = card.s2,
				name = v.name, -- 名称
				info = getInfo_(v, card.s2g),--string.format(v.memo, string.gsub(v["g" .. card.s2g], ",", " ~ "), string.gsub(v["g" .. card.s2g], ",", " ~ ")),	-- 说明	
				level = card.s2g,
			}
			-- table.insert(result, skillEntity)
			result.skill2 = skillEntity
			break
		end
		-- 技能3
		for v in _db:nrows('SELECT * FROM skilltb WHERE skillno = ' .. card.s3) do 
			local skillEntity = SkillEntity.new{
				skillno = card.s3,
				name = v.name, -- 名称
				info = getInfo_(v, card.s3g),--string.format(v.memo, string.gsub(v["g" .. card.s3g], ",", " ~ "), string.gsub(v["g" .. card.s3g], ",", " ~ ")),	-- 说明
				level = card.s3g,		
			}
			-- table.insert(result, skillEntity)
			result.skill3 = skillEntity
			break
		end
		-- 关
		closeDB( )
		
		-- 加入缓存
		_cardCache[id] = card
		if not _cardCache[id]["skills"] then
			_cardCache[id]["skills"] = {}
		end
		_cardCache[id]["skills"][evolv] = result
		return result
	end
end

-- 通过id 与 等级 获取经验
function CardDB.getExp( id, lv )
	if _cardCache[id] and _cardCache[id][lv] then
		-- 缓存有则直接返回
		return _cardCache[id][lv]
	else
		local result
		local card
		if _cardCache[id] then
			-- 缓存有则直接返回
			card =  _cardCache[id]
		else
			-- 开
			openDB( )
			for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. id) do 
				card = v
				break
			end
			-- 关
			closeDB( )
		end
		-- 开
		openDB( )
		for v in _db:nrows('SELECT * FROM cardupexptb WHERE level = ' .. lv) do 
			result = v[ "star" .. card.starlevel]--tonumber(v[card.starlevel + 1])
			break
		end
		-- 关
		closeDB( )
		
		-- 加入缓存
		_cardCache[id] = card
		_cardCache[id][lv] = result
		return result
	end
end

-- 通过id 与 等级 获取强化经验
function CardDB.getEnhancedExp( id )
	if _cardCache[id] and _cardCache[id].enhancedExp then
		-- 缓存有则直接返回
		return _cardCache[id].enhancedExp
	else
		local result
		local card
		if _cardCache[id] then
			-- 缓存有则直接返回
			card =  _cardCache[id]
		else
			-- 开
			openDB( )
			for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. id) do 
				card = v
				break
			end
			-- 关
			closeDB( )
		end
		-- 开
		openDB( )
		for v in _db:nrows('SELECT * FROM cardstrengthenexptb WHERE cardstar = ' .. card.starlevel) do 
			result = tonumber(v.strengthenexp)
			break
		end
		-- 关
		closeDB( )
		
		-- 加入缓存
		_cardCache[id] = card
		_cardCache[id].enhancedExp = result
		return result
	end
end
-- 通过id 获取军团天赋经验
function CardDB.getAExp( id )
	if _cardCache[id] and _cardCache[id].aExp then
		-- 缓存有则直接返回
		return _cardCache[id].aExp
	else
		local result
		local card
		if _cardCache[id] then
			-- 缓存有则直接返回
			card =  _cardCache[id]
		else
			-- 开
			openDB( )
			for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. id) do 
				card = v
				break
			end
			-- 关
			closeDB( )
		end
		-- 开
		openDB( )
		for v in _db:nrows('SELECT * FROM cardstrengthenexptb WHERE cardstar = ' .. card.starlevel) do 
			result = tonumber(v.aexp)
			break
		end
		-- 关
		closeDB( )
		
		-- 加入缓存
		_cardCache[id] = card
		_cardCache[id].aExp = result
		return result
	end
end
-- 通过id 获取军团天赋金币
function CardDB.getAGold( id )
	if _cardCache[id] and _cardCache[id].aGold then
		-- 缓存有则直接返回
		return _cardCache[id].aGold
	else
		local result
		local card
		if _cardCache[id] then
			-- 缓存有则直接返回
			card =  _cardCache[id]
		else
			-- 开
			openDB( )
			for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. id) do 
				card = v
				break
			end
			-- 关
			closeDB( )
		end
		-- 开
		openDB( )
		for v in _db:nrows('SELECT * FROM cardstrengthenexptb WHERE cardstar = ' .. card.starlevel) do 
			result = tonumber(v.agold)
			break
		end
		-- 关
		closeDB( )
		
		-- 加入缓存
		_cardCache[id] = card
		_cardCache[id].aGold = result
		return result
	end
end

-- 通过id 获取强化金币
function CardDB.getSellGlod( id )
	if _cardCache[id] and _cardCache[id].sellGlod then
		-- 缓存有则直接返回
		return _cardCache[id].sellGlod
	else
		local result
		local card
		if _cardCache[id] then
			-- 缓存有则直接返回
			card =  _cardCache[id]
		else
			-- 开
			openDB( )
			for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. id) do 
				card = v
				break
			end
			-- 关
			closeDB( )
		end
		-- 开
		openDB( )
		for v in _db:nrows('SELECT * FROM cardstrengthenexptb WHERE cardstar = ' .. card.starlevel) do 
			result = tonumber(v.sprice)
			break
		end
		-- 关
		closeDB( )
		
		-- 加入缓存
		_cardCache[id] = card
		_cardCache[id].sellGlod = result
		return result
	end
end

-- 通过name获取
function CardDB.getByName( name )
	if _cardCache[name] then
		-- 缓存有则直接返回
		return _cardCache[name]
	else
		-- 开
		openDB( )
		local result
		for v in _db:nrows('SELECT * FROM cardtb WHERE name = ' .. '"' .. name .. '"') do 
			result = CardEntity.new{
					id = v.cardno,
					name = v.name,
					star = v.starlevel,
			}
			break
		end
		-- 关
		closeDB( )
		-- 加入缓存
		if result then
			_cardCache[name] = result
		end
		return result
	end
end
-- 通过name获取
function CardDB.getByCardNo( no )
	if _cardCache[no] then
		-- 缓存有则直接返回
		return CardEntity.new{
			id = _cardCache[no].cardno,
			name = _cardCache[no].name,
			star = _cardCache[no].starlevel,
		}
	else
		-- 开
		openDB( )
		local result
		local dbDate
		for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. no) do 
			dbDate = v
			break
		end
		-- 关
		closeDB( )
		-- 加入缓存
		if dbDate then
			_cardCache[no] = dbDate
			result = CardEntity.new{
					id = dbDate.cardno,
					name = dbDate.name,
					star = dbDate.starlevel,
			}
		end
		return result
	end
end

-- 获取所有卡牌
function CardDB.getAllCardsID( )
	if _cardCache["AllID"] then
		-- 缓存有则直接返回
		return _cardCache["AllID"]
	else
		-- 开
		openDB( )
		local result = {}
		for v in _db:nrows('SELECT * FROM cardtb') do 
			-- result = v
			table.insert(result, v.cardno)
			-- break
		end
		-- 关
		closeDB( )
		-- 加入缓存
		if #result > 0 then
			_cardCache["AllID"] = result
		end
		return result
	end
end
-- 获取所有卡牌
function CardDB.getAllCards( )
	-- 开
	openDB( )
	local result = {}
	for v in _db:nrows('SELECT * FROM cardtb') do 
		-- result = v
		table.insert(result, v)
		-- break
	end
	-- 关
	closeDB( )
	return result
end
-- 通过id获取
function CardDB.getByID( id )
	if _cardCache[id] then
		-- 缓存有则直接返回
		return _cardCache[id]
	else
		-- 开
		openDB( )
		local result
		for v in _db:nrows('SELECT * FROM cardtb WHERE cardno = ' .. id) do 
			result = v
			break
		end
		-- 关
		closeDB( )
		-- 加入缓存
		if result then
			_cardCache[id] = result
		end
		return result
	end
end

-- 通过id获取
function CardDB.getByIDs( ids )
	local result = {}
	local noCachedIds = {}
	for i, id in ipairs(ids) do
		if _cardCache[id] then
			-- 缓存有则直接返回
			result[id] = _cardCache[id]
		else
			-- 收集没有的id
			table.insert(noCachedIds, id)
		end
	end
	-- 查数据
	if #noCachedIds > 0 then
		openDB( )
		for v in _db:nrows('SELECT * FROM cardtb WHERE cardno in (' .. table.concat(noCachedIds, ",") .. ')' ) do
			result[v.cardno] = v
			-- 加入缓存
			_cardCache[v.cardno] = v
		end
		closeDB( )
	end
	return result
end

-- 清空缓存
function CardDB.cleanAllCard( )
	_cardCache = nil
end