-- 卡片数据库操作
require("sqlite3")
local EntityMap   = require("model.entity.map.EntityMap")
local EntityLevel = require("model.entity.map.EntityLevel")
local M = {}

local TAG 					= "[LevelDB]"
local DB_PATH 				= util.fullPath( PATH_CARD_DB )
local TDArrayUtils      	= require(PH_MDL_UTIL .. "TDArrayUtils")
local TDStringUtils         = require(PH_MDL_UTIL .. "TDStringUtils")

M.INDEX_NAMES 	 			= { "I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX", "X" }
M.NUM_MAP_LEVELS 			= 10
M.NUM_MAP_SUBLEVELS 		= 12


--[=[ ## local params --]=]

--[[ ## local fuction --]]
local _db
local _cache = {}

-- 打开数据库
local function openDB( )
	_db = sqlite3.open(DB_PATH)
end

-- 关闭数据库
local function closeDB( )
	_db:close()
end










--[[ ## public fuction --]]

-- 获得索引名称
function M.getIndexName(index)
	if index >= 1 and index <= #M.INDEX_NAMES then 
		return M.INDEX_NAMES[index]
	else
		return ""
	end
end

-- 获取所有PO对象
function M.getAll()
	-- if _cache[id] then
	if false then 
		-- 缓存有则直接返回
		return nil
	else
		local result = {}
		-- 开
		openDB( )
		local result = {}
		for v in _db:nrows('SELECT * FROM leveltb') do 
			if result[v.levelid] == nil then 
				result[v.levelid] = {}
				_cache[v.levelid] = {}
			end

			result[v.levelid][v.sublevelid] = v
			-- 加入缓存
			if v then
				_cache[v.levelid][v.sublevelid] = v
			end
		end
		-- 关
		closeDB( )
		return result
	end
end

-- 获取所有地图名称
function M.getNames()
	if _cache["names"] then 
		-- 缓存有则直接返回
		return _cache["names"]
	else
		local result = {}
		-- 开
		openDB( )
		local result = {}
		for v in _db:nrows('SELECT distinct name FROM leveltb order by levelid') do 
			result[#result + 1] = v.name
		end

		-- 加入缓存
		_cache["names"] = result

		-- 关
		closeDB( )
		return result
	end
end

-- 通过id获取PO对象
function M.getByIndexes(id, subid)
	-- if _cache[id] then
	if false then 
		-- 缓存有则直接返回
		return _cache[id]
	else
		-- 开
		openDB( )
		local result
		for v in _db:nrows('SELECT * FROM leveltb WHERE levelid = ' .. id .. ' and sublevelid = ' .. subid) do 
			result = v
			break
		end
		-- 关
		closeDB( )
		-- 加入缓存
		if result then
			_cache[id] = result
		end
		return result
	end
end

-- 获得前一次现在进行关卡索引
function M.getLastLevelIndexes(entities)
	-- 如果不存在通关关卡数据，则当前关卡索引为1
	if not entities or TDArrayUtils.isEmpty(entities) then
		return 1, 1
	else
		local index1 = 0
		local index2 = 0
		for _, entity in ipairs(entities) do
			local i1 	= entity:getSn()
			local i2 	= entity:getSubsn()

			if i1 > index1 then
				index1 = i1
				index2 = i2
			elseif i1 == index1 then
				if i2 > index2 then 
					index2 = i2
				end
			end
		end

		return index1, index2
	end
end

-- 获得前一次现在进行关卡索引
function M.getLastLevelInfo(entities)
	-- 如果不存在通关关卡数据，则当前关卡索引为1
	if not entities or TDArrayUtils.isEmpty(entities) then
		return { 1, 1, 0 }
	else
		local index1 = 0
		local index2 = 0
		local difficulty = 0
		for _, entity in ipairs(entities) do
			local i1 	= entity:getSn()
			local i2 	= entity:getSubsn()

			if i1 > index1 then
				index1 = i1
				index2 = i2
				difficulty = entity:getStar()
			elseif i1 == index1 then
				if i2 > index2 then 
					index2 = i2
					difficulty = entity:getStar()
				end
			end
		end

		return { index1, index2, difficulty }
	end
end

-- 获得当前现在进行关卡索引
function M.getCurrentLevelIndexes(entities)
	-- 如果不存在通关关卡数据，则当前关卡索引为1
	local sn, subsn, maxsn = 1, 1, 1
	if not entities or TDArrayUtils.isEmpty(entities) then
		-- return 1, 1
	else
		local index1 = 0
		local index2 = 0
		for _, entity in ipairs(entities) do
			local i1 	= entity:getSn()
			local i2 	= entity:getSubsn()

			if i1 > index1 then
				index1 = i1
				index2 = i2
			elseif i1 == index1 then
				if i2 > index2 then 
					index2 = i2
				end
			end
		end
		maxsn = index1
		if not M.isLevelLast(index1, index2) then 
			index2 = index2 + 1
			if index2 > M.getCurrentLevels(index1) then 
				index1 = index1 + 1
				index2 = 1
			end
		end

		sn, subsn =  index1, index2
	end
	local maxLevel = game.getGameState("MAX_LEVEL") or 0
	if maxsn ~= maxLevel then 
		game.addGameState("MAX_LEVEL", maxsn, true)
	end
	return sn, subsn
end

-- 判断当前关卡是否是最新一关，展开新关卡判断
function M.isBreak(entities, index1, index2)
	local i1, i2 = M.getCurrentLevelIndexes(entities)
	-- local notLast = not M.isLevelLast(index1, index2)
	return index1 == i1 and index2 == i2 -- and notLast
end

-- 是否是最后一个关卡
function M.isLevelLast(index1, index2)
	return index1 == M.getCurrentLevels(index2) and index2 == M.NUM_MAP_SUBLEVELS
end

-- 获得主关卡对应的子关卡索引
function M.getCurrentLevelIndex2(entities, index1)
	local i1, i2 = M.getCurrentLevelIndexes(entities)
	if index1 == i1 then 
		return i2
	else
		return M.NUM_MAP_SUBLEVELS
	end
end

-- 获得关卡星级
function M.getStarByIndexes(entities, index1, index2)
	local star = 0

	for _, entity in pairs(entities) do
		local i1 	= entity:getSn()
		local i2 	= entity:getSubsn()

		if i1 == index1 and i2 == index2 then
			star = entity:getStar()
			break
		end
	end
	
	return star
end

-- 获得下一个关卡名称
function M.getNextLevelName(levelId, sublevelId)
	if type(levelId) ~= "number" or type(sublevelId) ~= "number" then 
		return ""
	end

	local index1 = levelId
	local index2 = sublevelId + 1
	if index2 > M.NUM_MAP_SUBLEVELS then 
		index1 = index1 + 1
		index2 = 1
	end


	-- 开
	openDB( )
	local result
	for v in _db:nrows('SELECT * FROM leveltb WHERE levelid = ' .. index1 .. ' and sublevelid = ' .. index2) do 
		result = v
		break
	end
	-- 关
	closeDB( )
	return result and result.subname or ""
end

-- 获得关卡名称
function M.getLevelName(id)
	-- if _cache[id] then
	if false then 
		-- 缓存有则直接返回
		return _cache[id]
	else
		-- 开
		openDB( )
		local result
		for v in _db:nrows('SELECT * FROM leveltb WHERE levelid = ' .. id .. ' and sublevelid = 1') do 
			result = v
			break
		end
		-- 关
		closeDB( )
		return result and result.name or ""
	end
end

-- 获得关卡英雄信息
function M.getHeroInfo(levelString)
	local levelParams = TDStringUtils.split(levelString, "-")
	local level1      = tonumber(levelParams[1])
	local level2	  = tonumber(levelParams[2])

	-- if _cache[id] then
	if false then 
		-- 缓存有则直接返回
		return _cache[id]
	else
		-- 开
		openDB( )
		local result
		for v in _db:nrows('SELECT * FROM leveltb WHERE levelid = ' .. level1 .. ' and sublevelid = ' .. level2) do 
			result = v
			break
		end
		-- 关
		closeDB( )

		if result then 
			return result.face, result.heroname
		else
			return 1, "未知战斗对象"
		end
	end
end

function M.getCurrentLevels( index )
	if index <= 10 then
		return 10
	else
		return 15
	end
end



return M