function isSDK( ... )
    local sdks = { ... }
    local cur = _GAME.CONFIG.SDK_PLATFORM
    for i, v in ipairs(sdks) do
        if v == cur then 
            return true
        end
    end
    return false
end

require("tandyfw.init")

-- 针对android做些功能初始
FILE_CARD_DB = "data/data_%s.rdb"
local FILE_LANGUAGE = "language.bin"
local FILE_SERVER = "server.txt"
local FILE_SERVER_URL = "server_url.txt"

if io.exists(PTAH_STORAGE_RES .. FILE_SERVER_URL) then
    _GAME.CONFIG.SDK_PLATFORM.domain = io.readfile(PTAH_STORAGE_RES .. FILE_SERVER_URL)
end

function game.prepareLanguage()
    if isstring(_GAME.CONFIG.DEFAULT_LANGUAGE) then 
        device.language = _GAME.CONFIG.DEFAULT_LANGUAGE
        return 
    end
    if isandroid() then 
        if device.language == "default" or device.language == "cn" then 
            local ok, result = luaj.callStaticMethod(
                                ANDROID_CLASS_NAME
                                , "getOSLanguage"
                                , nil
                                , "()Ljava/lang/String;"
                                ) 
            if ok then 
                result = string.lower(result)
                if string.startwith(result, "zh_") then 
                    if string.lower(result) == "zh_cn" then 
                        result = "cn"
                    else
                        result = "tw"
                    end
                else 
                    result = string.sub(result, 1, 2)
                end
                device.language = result
            else
                device.language = "cn"
            end
        end
        -- if device.language == "default" then device.language = "cn" end
    else
        if device.language == "default" or device.language == "cn" then
            local luaoc = require("framework.client.luaoc")
            local function setLanguage( result )
                local lan = result["language"]
                if lan == "zh-Hans" then 
                    device.language = "cn"
                elseif lan == "zh-Hant" then 
                    device.language = "tw"
                else
                    device.language = lan
                end
            end
            local ok, ret = luaoc.callStaticMethod( "AppController", "getLanguages", {callback = setLanguage})
        end
    end
    if table.isnotempty(_GAME.CONFIG.DEFAULT_LANGUAGE) then 
        local lan = nil
        for i, v in ipairs(_GAME.CONFIG.DEFAULT_LANGUAGE) do
            if device.language == v then 
                lan = v
                break 
            end
        end
        if not lan then 
            -- 没找到啊,只好拿默认的了
            device.language = _GAME.CONFIG.DEFAULT_LANGUAGE[1]
        end
    end
end

function game.setLanguage( language )
    if isstring(language) then 
        io.writefile(PTAH_STORAGE_RES .. FILE_LANGUAGE, language)
    end
end

try(function ()
    game.prepareLanguage()
    if isandroid() then 
        local luaj = require("framework.client.luaj")
        -- android时覆盖openudid
        local ok, udid = luaj.callStaticMethod(
            ANDROID_CLASS_NAME
            , "getOpenUDID"
            , nil
            , "()Ljava/lang/String;"
            )
        if ok then 
            device.getOpenUDID = function ()
                return udid
            end
        end

        
        FILE_CARD_DB = string.format(FILE_CARD_DB, device.language)

        -- 复制db
        if not io.exists(device.documentsPath .. FILE_CARD_DB) then
            local filepath = io.pathinfo(PTAH_STORAGE_RES .. FILE_CARD_DB)
            luaj.callStaticMethod(
                ANDROID_CLASS_NAME
                , "copyAssets"
                , { PATH_RES .. FILE_CARD_DB, filepath.dirname, filepath.filename }
                ) 
        end
        -- android上打开url
        device.openURL = function ( url )
            luaj.callStaticMethod(
                ANDROID_CLASS_NAME
                , "openURL"
                , { url }
                ) 
        end
    else
        FILE_CARD_DB = string.format(FILE_CARD_DB, device.language)
        util.copyfile( PATH_RES .. FILE_CARD_DB, PTAH_STORAGE_RES .. FILE_CARD_DB, false)
        -- util.copyfile( PATH_RES .. FILE_VERSION, PTAH_STORAGE_RES .. FILE_VERSION, false)
        
    end
end, nil, nil)

if io.exists(PTAH_STORAGE_RES .. "VERSION") then
    os.remove(PTAH_STORAGE_RES .. "VERSION")
end
if io.exists(PTAH_STORAGE_RES .. FILE_VERSION) then
    os.remove(PTAH_STORAGE_RES .. FILE_VERSION)
end
io.writefile(PTAH_STORAGE_RES .. FILE_VERSION, CVERSION)

-- 初始化多语言
-- device.language = "cn"
try(function ( ... )
    if io.exists(string.format("%slanguage/lan_game_%s.lua", device.writablePath, device.language)) then 
        local lanpath = string.gsub(device.writablePath, "/", ".")
        if string.startwith(lanpath, ".") then 
            lanpath = string.sub(lanpath, 2)
        end
        util.loadLanguage( lanpath .. "language.lan_game")
    else
        util.loadLanguage("language.lan_game")
    end
end, function ( ... )
    util.loadLanguage("language.lan_game")
end)

require("config_card")
if not _GAME.CONFIG.SDK then 
    require("config_sdk")--sdk充值平台全局c n
end
require("config_fighting")
_sdkPlatform          = require("sdk.SdkPlatform")
 
if _GAME.CONFIG.LANGUAGE_SWITCH and isios() then 
    local lan = device.language == "en" and 2 or 1
    _sdkPlatform.changeLanguage(
            function( )
                -- 清空信息  
            end,
            function() 
            end,
            {language = lan}
        )
end

if not DIAMONDPLATFORM then 
    if isSDK(_GAME.CONFIG.SDK.SDK_APPSTORE, 
    	_GAME.CONFIG.SDK.SDK_APPSTORE_EN, 
    	_GAME.CONFIG.SDK.SDK_FACEBOOK, 
    	_GAME.CONFIG.SDK.SDK_TWITTER, 
        _GAME.CONFIG.SDK.SDK_TWITTERANDFACEBOOK, 
    	_GAME.CONFIG.SDK.SDK_APPSTORE_JP, 
    	_GAME.CONFIG.SDK.SDK_UCUBE) then
        	DIAMONDPLATFORM = require("sdk.AppleIapPlatform") --appstore购买类
        	DIAMONDPLATFORM.appstoreIAP() --初始化
        	DIAMONDPLATFORM.loadIapS() --初始化
    end
end

_GAME.CONFIG.GOODS = require("config.config_goods")

-- 这里用来切换帐号
if not RELEASE_MODE then
    device.getOpenUDID_ = device.getOpenUDID_ or device.getOpenUDID
    device.getOpenUDID = function ( ... )
        local udid = device.getOpenUDID_()
        return udid .. "1"
    end
end

-- 当为无sdk时直接无效化掉该功能
if device.platform == "mac" then --isSDK(_GAME.CONFIG.SDK.SDK_NONE) or not RELEASE_MODE then 
    function util.removeUnusedTextures() 
    end
end

function game.getServerSDKTag()
    if io.exists(PTAH_STORAGE_RES .. FILE_SERVER) then
        local server = io.readfile(PTAH_STORAGE_RES .. FILE_SERVER)
        if "_tmp" == server then 
            DEBUG = 2
            LOG_LEVEL = 3
        end
        return server
    elseif RELEASE_MODE == false or require(PH_MDL_UTIL .. "TDHookUtils").existsServer() then 
        -- 是测试环境,全拿
        return _GAME.CONFIG.SDK_PLATFORM[2]
        -- return _GAME.CONFIG.SDK_PLATFORM[2]
    else
        return _GAME.CONFIG.SDK_PLATFORM[2]
    end 
end

-- 初始全局变量
PATH_CARD_DB = UpdateHelper.fetchDBPath(FILE_CARD_DB)
ISDIAMONDALERTRELOAD    = false  --消费弹窗是否重置
require("db.carddb")
require("db.backpackdb")
require("db.equipdb")
require("db.equipsetdb")
require("db.equipskilldb")
TalentDB             = require("db.talentdb") --天赋数据库
ArmyTalentDB             = require("db.armytalentdb") --军团天赋数据库

NetHelper            = require("net.NetHelper")
Player               = require("cardui.Player")
PlayerHead           = require("cardui.PlayerHead")

GameRateEntity       = require("model.main.GameRateEntity")
SOUND                = require("GameSoundManager")
Player                = require("cardui.Player")
DiamondAlertEntity  = require("model.alert.DiamondAlertEntity")

-- 玩家信息
ME = require("model.player.PlayerEntity").new{}

GAMERATE = {}
-- ==============游戏状态控制===============
GSVALUES = nil
GS =  require("framework.client.api.GameState")
try(function ()
    -- 定义存储器
    GS.init(function( t )
                return t
            end,"userInfo","user")
end,nil,nil)

--or not GSVALUES.username or not GSVALUES.pass 
function ISGSVALUES()
    -- if not table.isnotempty(GSVALUES) or #GSVALUES == 1 or
    -- (GSVALUES and (not GSVALUES.username or not GSVALUES.pass))  then  -- GSVALUES == nil or #
    if not GSVALUES then    
        try(function ()
            if GS.load().values then 
                GSVALUES = GS.load().values.values 
            end
        end, nil, nil)
    elseif _GAME.CONFIG.LANGUAGE_SET.LOGIN_STYLE == "ja" and not GSVALUES.udid and (not GSVALUES.username or GSVALUES.username == "" or not GSVALUES.pass or GSVALUES.pass == "") then
        try(function ()
            local flag = nil
            if GSVALUES.moviePlayed then
                flag = GSVALUES.moviePlayed
            end

            if GS.load().values then 
                GSVALUES = GS.load().values.values 
                GSVALUES.moviePlayed = flag
            end
        end, nil, nil)
    elseif not GSVALUES.username or GSVALUES.username == "" or not GSVALUES.pass or GSVALUES.pass == "" then
        try(function ()
            local flag = nil
            if GSVALUES.moviePlayed then
                flag = GSVALUES.moviePlayed
            end

            if GS.load().values then 
                GSVALUES = GS.load().values.values 
                GSVALUES.moviePlayed = flag
            end
        end, nil, nil)
    end
    return GSVALUES     
end

function game.initGameState_()
    return ISGSVALUES()
end

function game.resetGameState( newValues )
    GSVALUES = newValues or {}
    GS.save(GSVALUES)
end

function game.addGameState( key, value, autosave )
    if game.initGameState_() then
        GSVALUES[key] = value
        if autosave then 
            GS.save(GSVALUES)
        end
        return true
    end
    return false
end

function game.getGameState( key )
    if game.initGameState_() then
        if key  then 
            return GSVALUES[key]
        else
            return GSVALUES
        end
    end
    return nil
end

function game.saveGameState()
    if game.initGameState_() then
        GS.save(GSVALUES)
        return true
    end
    return false
end

function game.hasGameState()
    return game.initGameState_() ~= nil
end

-- ==============数据缓存处理===============
_GAME.CACHE_IDS = nil
-- 暂时先关闭吧
if NetHelper then 
    _GAME.CACHE_IDS = {
        -- MainNewLayer     首页配置信息
        -- [ACTID.MSG_GET_CONFIG] = true,
        -- CardGroupLayer   卡组信息
        -- [ACTID.MSG_INTO_CARD] = true,
        -- MapContainerLayerView    地图关卡信息
        [ACTID.MSG_GET_LEVELLIST] = true,
        -- FriendLayer  好友界面,取得好友列表
        -- [ACTID.MSG_GET_FRIENDLIST] = true,
        -- ShopLayer  商店界面,取得列表
        [ACTID.MSG_GET_SHOPLIST] = true,
        -- StarShopLayer  星星商店界面,取得列表
        [ACTID.MSG_GET_STAR_SHOPLIST] = true,
        -- EquipmentLayer 装备界面,取得装备列表
        [ACTID.MSG_GET_EQUIPMENT] = true,
        -- EquipmentLVLayer 装备强化界面
        -- [ACTID.MES_GET_EQUIPMENTLV] = true,
    }
else
    _GAME.CACHE_IDS = {}
end

function game.addCache(actionId, properties)
    if isnumber(actionId) and properties then 
        -- 看看有没有在要进行缓存的table,不在就给屎边去,在就缓存下
        if _GAME.CACHE_IDS and nil ~= _GAME.CACHE_IDS[actionId] then
            _GAME.CACHE[actionId] = properties
        end
    end
    return true
end

function game.callbackWithCache(actionId, callback)
    if _GAME.CACHE_IDS and isnumber(actionId) and isfunction(callback) then 
        local properties = _GAME.CACHE[actionId]
        if properties then 
            callback(properties)
        end
    end
end

function game.skipCacheLoading( actionId )
    if isnumber(actionId) then 
        return nil ~= _GAME.CACHE[actionId]
    else
        return false
    end
end
-- =====================================

HelperView2 = require("cardui.HelperView2")
TopButtons = require("cardui.TopButtons")
ScrollLabel = require("cardui.ScrollLabel")
ScrollTalkPanel = require("cardui.ScrollTalkPanel")
ISOPENALERT = false

-- 聊天消息集合
function game.clearCacheData( )
    chat_messages = {}
    chat_messages2 = {}
    globalMessagesFW = {}
    globalMessageMeFW = {}
    _GAME.CACHE_IDS = nil
end

game.clearCacheData( )

-- 准备下字体
-- local defaultFont
if isandroid() then
    -- defaultFont = "RTWS YueGothic G0v1.ttf"
    FONT_1      = "VerdanaBold.otf"
else
    -- defaultFont = "RTWS YueGothic G0v1"
    FONT_1      = "Verdana-Bold"
end
-- 越南要换字体
if _GAME.CONFIG.SDK_PLATFORM.lan == "vi" then
    ui.DEFAULT_TTF_FONT = isandroid() and "Times New Roman.ttf" or "Times New Roman"
end
-- ui.DEFAULT_TTF_FONT      = defaultFont

--[=====[方法重载 START]=====]--
function string.urlencodeChar(char)
    return "%" .. string.format("%02X", string.byte(char))
end
-- 重载CCBCenter.replaceScene,加入监听
CCBCenter.replaceScene_ = CCBCenter.replaceScene_ or CCBCenter.replaceScene
function CCBCenter.replaceScene( msgId, controllerPath, data, callback, transitionType, time, more )
    GameHelper.cleanCurScene()
    local scene = CCBCenter.replaceScene_( msgId, controllerPath, data, callback, transitionType, time, more )
    game.bindScene( scene )
    return scene
end

-- 重载CCBCenter.readCCBFromFile, 加入对路径的自动判断
CCBCenter.readCCBFromFile_ = CCBCenter.readCCBFromFile_ or CCBCenter.readCCBFromFile
function CCBCenter.readCCBFromFile( ccbiFileName )
    local path = string.gsub(PTAH_STORAGE_RES .. ccbiFileName , "/res/", "/")
    if not io.exists(path) then 
        path = ccbiFileName
    end
    return CCBCenter.readCCBFromFile_(path)
end

-- 重载json.encode, 对无值情况进行处理
json.encode_ = json.encode_ or json.encode
function json.encode(var, isDebug)
    local result = json.encode_(var, isDebug)
    return ifnil(result, "")
end

local _node_show_topbuttons = nil
CCBCenter.pushCCBNode_ = CCBCenter.pushCCBNode_ or CCBCenter.pushCCBNode
function CCBCenter.pushCCBNode( level, node, cleanUp, animType, args )
    if CCBCenter.pushCCBNode_( level, node, cleanUp, animType, args ) then
        _node_show_topbuttons = node.SHOW_TOPBUTTONS
        if isnumber(node.SHOW_TOPBUTTONS) then
            if node.SHOW_TOPBUTTONS == TopButtons.INDEX_NONE then 
                game.hideTopButtons()
                game.removeTopButtons()
            else
                game.showTopButtons(node.SHOW_TOPBUTTONS)
            end
        else
            game.hideTopButtons()
        end
    end
end

CCBCenter.popCCBNode_ = CCBCenter.popCCBNode_ or CCBCenter.popCCBNode
function CCBCenter.popCCBNode( level, callDisplayCCBContainer, animType, args )
    local result, node, delay = CCBCenter.popCCBNode_( level, callDisplayCCBContainer, animType, args )
    if result and node then 
        local function onPopCCBNodeEnd_( ... )
            _node_show_topbuttons = node.SHOW_TOPBUTTONS
            if isnumber(node.SHOW_TOPBUTTONS) then
                if node.SHOW_TOPBUTTONS == TopButtons.INDEX_NONE then
                    game.hideTopButtons()
                    game.removeTopButtons()
                else
                    game.showTopButtons(node.SHOW_TOPBUTTONS, 0)
                end
            else
                game.hideTopButtons()
            end
        end
        if isnumber(delay) then 
            util.delayCall(onPopCCBNodeEnd_, delay)
        else
            onPopCCBNodeEnd_()
        end
    end
    return result, node
end

--[=====[方法重载 END]=====]--


function game.bindScene( scene )
    GameHelper.bindSceneEvent(scene)
    --[[ 针对Scene绑定的事件 ]]
    -- 返回到Login界面
    scene:addEventListener(GameHelper.EVENT_NAME_RETURN_TO_LOGIN, game.enterLoginScene)
    -- 退出游戏
    scene:addEventListener(GameHelper.EVENT_NAME_BACK_TO_EXIT, game.pressBackExitListener)
end



function game.pressBackExitListener(  )
    local ccbNode = CCBCenter.getLastCCBNode()
    if ccbNode then 
        local function showDialog4Exit_(  )
            if isSDK(_GAME.CONFIG.SDK.SDK_360) then 
                _sdkPlatform.logout(function ( ... )
                    util.delayCall(function ( ... )
                        GameHelper.exit()
                    end, 0.5)
                end)
            -- android 91后退，退出游戏
            elseif isSDK(_GAME.CONFIG.SDK.SDK_91) and isandroid() then 
                _sdkPlatform.exitgame(function ( ... )
                    GameHelper.exit()
                end)
            else
                if display.getRunningScene():getChildByTag(100000) then return end 
                AlertLayer.new({
                        content     = _s("QUIT_CONFIRM"), -- "进入专家模式，独战开始! :)",
                        ccbLevel    = 12,
                        btnStr      = { _s("COMMON_OK"), _s("COMMON_CANCEL"), },
                        btnPriority    = -2000,
                        btnClickEvent = {       -- 按钮事件数组
                            function()
                                util.delayCall(function ( ... )
                                    GameHelper.exit()
                                end, 0.3)
                            end,
                            function()
                                NetHelper.QuickToast(_s("QUIT_CANCEL"))
                                local node = display.getRunningScene():getChildByTag(100000)
                                node:removeFromParentAndCleanup(true)
                            end,
                        },
                        callback = function ( ccbNode )
                            local mask = display.newLayer()
                            widget.bindTouchEvent{
                                layer = mask,
                                priority = -1999,
                                swallowsTouches = true,
                            }
                            ccbNode:addChild(mask)
                            display.getRunningScene():addChild(ccbNode, 100000, 100000)
                        end 
                    })
            end
        end
        if ccbNode._IGNOR_PRESS_BACK then return end
        if not HelperView2.isShowing() and ccbNode.closeLayer then 
            local result = ccbNode.closeLayer()
            if result == true then
                showDialog4Exit_()
            elseif isnumber(result) then 
                if result == 99 then 
                    -- gun gun,什么都不要处理
                else
                    -- 以现有方式,这里都用不上...
                    ccbNode = CCBCenter.getLastCCBNode()
                    if ccbNode then 
                        if ccbNode.requestDataForLoad then 
                            util.delayCall(ccbNode.requestDataForLoad, result)
                        end
                    end
                end
            end 
        else 
            if not HelperView2.isShowing() then 
                showDialog4Exit_()
            end
        end
    end
    
end

--[=====[项部菜单控制相关 START]=====]--
local _showHandle, _topButtons
-- 移除顶部菜单
function game.removeTopButtons( containear )
    -- if 1 == 1 then return end
    if _showHandle then 
        scheduler.unscheduleGlobal(_showHandle)
        _showHandle = nil
    end
    if not containear then containear = display.getRunningScene() end
    local topButtons = containear:getChildByTag(9900)
    if topButtons then 
        topButtons:removeFromParentAndCleanup(true)
        _topButtons = nil
    end
end

function game.enableTopButtonTouch( enable )
    if _topButtons then
        util.enableCustomTouch(_topButtons, enable)
    end
end
--[[ 

显示顶部菜单

**Parameters:**

-   select:         int         选择哪一个菜单项
-   delay:          float       延迟调用,默认为0.5,不想延迟要设置为0
-   containearNode: CCNode      具体的显示菜单容器,通常不写啦,就是当前的scene

--]]
function game.showTopButtons( select, delay, containearNode )
    -- if 1 == 1 then return end
    local containear = containearNode
    if not containear then containear = display.getRunningScene() end
    local topButtons = containear:getChildByTag(9900)
    if not topButtons then
        local function createTopButtons_()
            if isnumber(_node_show_topbuttons) then 
                if _node_show_topbuttons == TopButtons.INDEX_NONE then 
                    game.removeTopButtons()
                else
                    if not containearNode then containear = display.getRunningScene() end
                    topButtons = containear:getChildByTag(9900)
                    if not topButtons then
                        topButtons = TopButtons.new{select = select}
                        _topButtons = topButtons
                        containear:addChild(topButtons, 9900, 9900)
                        util.alignNode(topButtons, CCSize(CONFIG_SCREEN_WIDTH, CONFIG_SCREEN_HEIGHT), display.TOP_CENTER)
                    end
                end
            else
                game.hideTopButtons()
            end
        end
        if _showHandle then 
            scheduler.unscheduleGlobal(_showHandle)
            _showHandle = nil
        end
        _showHandle = util.delayCall(function ()
                -- 不存在就新建个吧
            _showHandle = nil
            createTopButtons_()
            if _topButtons then _topButtons.setSelect(select) end
        end, delay or LOAD_DELAY_TIME) 
        
    else
        -- 存在就判断下是不是显示的,不是就显示出来
        if not topButtons:isVisible() then
            topButtons:setVisible(true)
        end
        if _topButtons then _topButtons.setSelect(select) end
    end
end

-- 隐藏顶部菜单
function game.hideTopButtons( containear )
    -- if 1 == 1 then return end
    if not containear then containear = display.getRunningScene() end
    if not containear then return end
    if _showHandle then 
        scheduler.unscheduleGlobal(_showHandle)
        _showHandle = nil
    end
    local topButtons = containear:getChildByTag(9900)
    if topButtons then 
        topButtons:setVisible(false)
    end
end

-- 请求Load界面后的数据
function game.requestDateForLoad( node, showDelay )
    showDelay = showDelay or 0

    if node.requestDataForLoad then 
        node.requestDataForLoad()
    end
    if node.SHOW_TOPBUTTONS and isnumber(node.SHOW_TOPBUTTONS) then
        if node.SHOW_TOPBUTTONS == TopButtons.INDEX_NONE then
            game.hideTopButtons()
        else
            game.showTopButtons(node.SHOW_TOPBUTTONS, showDelay)
        end
    end
end

--[=====[项部菜单控制相关 END]=====]--





--[=====[登录/登出控制相关 START]=====]--

-- 进入主界面
function game.enterMainLayer( anim )
    CCBCenter.replaceLayer(
        MAINLAYER_ID
        , MAINLAYER_PATH
        , {fromMain = false, }
        , function (node)
                game.requestDateForLoad( node )
            end
        , anim or CCBCenter.ANIM_LAYER_LEFT_TO_RIGHT)
end

-- 返回登录界面
function game.returnToLogin()
    SOUND.setMusicOff()
    Log.d(TAG,"注销~")
    _GAME.CACHE = {}
    sendParams = {}
    GSVALUES = {}
    GSVALUES.serverid = SERVERID 
    GSVALUES.moviePlayed = true
    GS.save(GSVALUES)
    ME.isHelpLoaded = false
    LOGINBASEINFO = DEFLOGINBASEINFO
    -- Log.d(TAG, "注销+++++++%s",json.encode(GSVALUES))

    -- 清除全局缓存文件
    game.clearCacheData( )
    -- 重新加载新手引导
    package.loaded["cardui.HelperView2"]  = nil
    package.loaded["static.helper_info2"]  = nil
    HelperView2 = require("cardui.HelperView2")

    GameHelper.returnToLogin()
end

--[=====[登录/登出控制相关 END]=====]--





--[=====[更新相关 START]=====]--
local function hotupdate()
    local hotupdates = require("static.hotupdate")
    for i,v in ipairs(hotupdates) do
        package.loaded[v] = nil
    end
    require("init")
    require("game")
end
-- 显示更新版本提示框（点击后退出游戏）
function game.dialogVersionUpdateCompleted(version, desc)
    -- 显示[更新完成]对话框
    -- AlertLayer.new{
    --     titleStr    = _s("LOGIN_UPDATE_TITLE"),               -- 标题
    --     content     = _s("LOGIN_UPDATE_SUCCESS"),      -- 内容
    --     btnStr      = { _s("COMMON_OK") },           -- 按钮文字数组
    --     btnColor    = { AlertLayer.BTN_COLOR_YELLOW },
    --     -- 按钮颜色数组
    --     btnClickEvent = {                       -- 按钮事件数组
    --         function()                          -- 按钮[确定]回调
    --             -- print("按钮[确认]回调")
    --             -- GameHelper.exit()
    --             hotupdate()
    --             game.startup()
    --         end,
    --     }, 
    -- }
    hotupdate()
    game.startup()
end

-- 显示更新版本提示框（点击后退出游戏）
function game.dialogVersionFound(version, desc)
    -- 显示[更新]对话框
    AlertLayer.new{
        titleStr    = _s("LOGIN_UPDATE_TITLE"),               -- 标题
        content     = _s("LOGIN_UPDATE_CONTENT", version, desc),      -- 内容
        btnStr      = { _s("LOGIN_UPDATE_OK_BUTTON") },           -- 按钮文字数组
        btnColor    = { AlertLayer.BTN_COLOR_YELLOW },
        -- 按钮颜色数组
        btnClickEvent = {                       -- 按钮事件数组
            function()                          -- 按钮[确定]回调
                -- print("按钮[更新版本]回调")
                GameHelper.returnToLogin()
                -- game.judgeVersionUpdate(nil, nil)
            end,
        }, 
    }
end

function game.judgeVersionUpdate(callbackCancelUpdate, callbackCompleteUpdate,hideBtn)
    local downloadProgressBar
    local totalSize, curSize = 0, 0
    local lastPercentage = 0
    local packageSizes = {}
    -- 等到强更时把这个打开吧,因为要c++方面的配合,只能等强更了 
    local USE_PROGRESS = true

    local function createDownloadingProgressBar_( ... )
        local layer = widget.newMaskLayer(nil, nil, ccc4(0,0,0,0))
        local top = 3.0
        if isandroid() then top = 2.0 end
        local downloadProgressBar = widget.newProgressBar{
            frontSprintParams = {file = "login/login_downloading_pb_front.png",},
            bgSprintParams = {file = "login/login_downloading_pb_bg.png"},
            -- percentageLabelString = "%d%%",
            ttfLabelParams = { size = 18, padding = { top = top } }
        }
        display.getRunningScene():addChild(layer)
        layer:addChild(downloadProgressBar)
        util.alignNode(downloadProgressBar, layer:getContentSize(), display.BOTTOM_CENTER, 58)

        local label = ui.newTTFLabel{
            text = _s("LOGIN_UPDATE_DOWNLOADING"),
            size = 36
        }
        layer:addChild(label)
        util.alignNode(label, layer:getContentSize(), display.BOTTOM_CENTER, 50)

        label:runAction(CCRepeatForever:create(transition.sequence{
            CCFadeOut:create(1),
            CCFadeIn:create(1),
            CCDelayTime:create(0.3),
            }))
        if not USE_PROGRESS then 
            downloadProgressBar:runProgressFromTo(30, 0, 95)
        end

        layer.removeOnSuccess = function ( onComplete )
            -- downloadProgressBar:stopAllActions()
            local lastTime = 0.5
            if not USE_PROGRESS then lastTime = 2.0 end
            downloadProgressBar:runProgressTo(lastTime, 100, false, function ( progressBar,percentage )
                layer:removeSelf() 
                if onComplete then onComplete() end
            end)
        end

        layer.removeOnFail = function ( onComplete )
            layer:removeSelf() 
            if onComplete then onComplete() end
        end

        layer.runProgressTo = function ( self, time, per )
            downloadProgressBar:runProgressTo(time, per)
        end

        return layer
    end

    local function updateVersion_(updates)
        if updates and #updates > 0 then 
            -- print("::::::::::::::::::开始完毕")
            --[[
              BASESER_URL
              updates :[ ver 版本号, desc 更新描述, url, adddate, md5 ]
            ]]--
            local function progressCallback_( event )
                if USE_PROGRESS and downloadProgressBar then 
                    local size = curSize + event.length
                    if size > totalSize then size = totalSize end
                    if size < 0 then size = 0 end
                    local per = math.floor(size / totalSize * 100)
                    if per ~= lastPercentage then 
                        lastPercentage = per
                        downloadProgressBar:runProgressTo(0.1, per)
                    end
                end
            end
            local function downloadFail_( )
                if downloadProgressBar then 
                    downloadProgressBar.removeOnFail()
                    downloadProgressBar = nil
                end
                AlertLayer.new{
                    content     = _s("LOGIN_UPDATE_FAILED"),
                    btnStr      = { _s("COMMON_RETRY"), _s("COMMON_CANCEL") },
                    btnColor    = { AlertLayer.BTN_COLOR_YELLOW, AlertLayer.BTN_COLOR_YELLOW },
                    btnClickEvent = {
                        function()
                            downloadProgressBar = createDownloadingProgressBar_()
                            updateVersion_(updates)
                        end,
                        function()
                            if callbackCancelUpdate then 
                                callbackCancelUpdate()
                            end
                        end
                    },
                }
            end
            local i = 1
            
            local function download_( index )
                local updateInfo = updates[index]
                if updateInfo then 
                    local updateUrl = updateInfo.url
                    if not string.startwith(string.lower(updateUrl), "http") then 
                        updateUrl = BASESER_URL .. updateUrl
                    end
                    UpdateHelper.downloadAndUnzipFile(
                        updateUrl, 
                        function(fileName, successful)
                            if successful then
                                Log.d(TAG, "@@@@@@@@@@@@@@@@@@@@@successful")
                                curSize = curSize + packageSizes[i]
                                i = i + 1
                                if i > #updates then 
                                    --结束更新
                                    local TDVersionUtils = require(PH_MDL_UTIL .. "TDVersionUtils")
                                    local version        = updateInfo.ver
                                    local desc           = updateInfo.desc

                                    if type(version) == "number" then 
                                        version = TDVersionUtils.convert(version)
                                    end

                                    -- 判断版本
                                    if type(version) == "string" and TDVersionUtils.isOutdate(version) then 
                                        -- 显示[更新]对话框
                                        if downloadProgressBar then
                                            downloadProgressBar.removeOnSuccess(function ( ... )
                                                game.dialogVersionUpdateCompleted(version, desc)
                                                downloadProgressBar = nil
                                            end)
                                        else
                                            game.dialogVersionUpdateCompleted(version, desc)
                                        end
                                        return
                                    end
                                else
                                    download_(i)
                                end
                            else
                                Log.d(TAG, "@@@@@@@@@@@@@@@@@@@@@fail")
                                -- NetHelper.hideLoading(true)
                                updateUrl = updateInfo.url2
                                if updateUrl and updateUrl ~= "" then
                                    if not string.startwith(string.lower(updateUrl), "http") then 
                                        updateUrl = BASESER_URL .. updateUrl
                                    end
                                    UpdateHelper.downloadAndUnzipFile(
                                        updateUrl, 
                                        function(fileName, successful)
                                            if successful then
                                                Log.d(TAG, "@@@@@@@@@@@@@@@@@@@@@successful")
                                                curSize = curSize + packageSizes[i]
                                                i = i + 1
                                                if i > #updates then 
                                                    --结束更新
                                                    local TDVersionUtils = require(PH_MDL_UTIL .. "TDVersionUtils")
                                                    local version        = updateInfo.ver
                                                    local desc           = updateInfo.desc

                                                    if type(version) == "number" then 
                                                        version = TDVersionUtils.convert(version)
                                                    end

                                                    -- 判断版本
                                                    if type(version) == "string" and TDVersionUtils.isOutdate(version) then 
                                                        -- 显示[更新]对话框
                                                        if downloadProgressBar then
                                                            downloadProgressBar.removeOnSuccess(function ( ... )
                                                                game.dialogVersionUpdateCompleted(version, desc)
                                                                downloadProgressBar = nil
                                                            end)
                                                        else
                                                            game.dialogVersionUpdateCompleted(version, desc)
                                                        end
                                                        return
                                                    end
                                                else
                                                    download_(i)
                                                end
                                            else
                                                Log.d(TAG, "@@@@@@@@@@@@@@@@@@@@@fail")
                                                -- NetHelper.hideLoading(true)
                                                downloadFail_( )
                                            end
                                        end,
                                        progressCallback_,
                                        30
                                    )
                                else
                                    downloadFail_( )
                                end
                            end
                        end,
                        progressCallback_,
                        30
                    )
                end
            end
            download_(i)
        else
            if callbackCompleteUpdate then 
                callbackCompleteUpdate()
            end
        end
    end

    local function callback_(properties)
        if properties and properties.updates then 
            local updates = properties.updates
            local tsize = 0
            if updates and #updates > 0 then 
                for i=1,#updates do
                    if updates[i] and updates[i].size then 
                        tsize = tsize + updates[i].size
                        packageSizes[i] = updates[i].size
                    end
                end
            end
            if tsize > 0 then 
                totalSize = tsize
                local rsize = (tsize * 1.0 /1024)/1024
                if rsize < 0.1 then rsize = 0.1 end
                rsize = math.round(rsize * 10)/10
                AlertLayer.new4OK{
                    content   = string.format(_s("LOGIN_UPDATE_NOT_WIFI"), rsize),
                    ignorPressBack = true,
                    btnClickEvent = {
                        function()
                            -- NetHelper.showLoading(true,true,false, _s("LOGIN_UPDATES"))
                            if isfunction(hideBtn) then
                                hideBtn()
                            end
                            -- 创建进度条显示
                            downloadProgressBar = createDownloadingProgressBar_()
                            lastPercentage = 0
                            updateVersion_(updates)
                        end,
                    },
                }
            else 
                if callbackCompleteUpdate then 
                    callbackCompleteUpdate()
                end
            end
        else 
            if callbackCompleteUpdate then 
                callbackCompleteUpdate()
            end
        end 
    end

    NetHelper.sendGetUpdateInfo(callback_)
end
--[=====[更新相关 END]=====]--


--[=====[全局事件处理 START]=====]--
function game_global_event_handle( eventName )
    -- print("@@@@@@@@@@@@@@ game_global_event_handle" .. eventName)
    if eventName == "GLOBAL_EVENT_ONRESUME" then 
        if display.getRunningScene():getChildByTag(100000) then return end
        if not HelperView2.isShowing() then  
            util.delayCall(function()
                AlertLayer.new4OK{
                    content = _s("GAME_PAUSE_HINT"),
                    noanim = true,
                }
            end, 0.1)
        end
    -- 91全局事件，回到登录页面
    elseif eventName == "game_global_event_relogin" and not isSDK(_GAME.CONFIG.SDK.SDK_91) then 
        util.delayCall(function ()
            game.returnToLogin()
        end)
    end
end

local notificationCenter = CCNotificationCenter:sharedNotificationCenter()
notificationCenter:registerScriptObserver(nil, function ( ... )
    -- print("##############################")
    -- print("####APP_ENTER_BACKGROUND####")
    -- print("##############################")
    if isios() then 
        local luaoc = require("framework.client.luaoc")

        luaoc.callStaticMethod( "AppController", "localPush", {key = "pushC12" ,push = _s("LOCAL_PUSH_CONTEXT12"), time = "12:00:00"})
        luaoc.callStaticMethod( "AppController", "localPush", {key = "pushC20" ,push = _s("LOCAL_PUSH_CONTEXT20"), time = "20:15:00"})
    end
end, "APP_ENTER_BACKGROUND")
notificationCenter:registerScriptObserver(nil, function ( ... )
    -- print("##############################")
    -- print("####APP_ENTER_FOREGROUND####")
    -- print("##############################")
    if isios() then 
        local luaoc = require("framework.client.luaoc")

        luaoc.callStaticMethod( "AppController", "delLocalPush", {key = "pushC12"})
        luaoc.callStaticMethod( "AppController", "delLocalPush", {key = "pushC20"})
    end
    game.getGameState()
    if game.getGameState("GAME_MUSIC") == false then 
        SOUND.setMusicOff()
        SimpleAudioEngine:sharedEngine():pauseBackgroundMusic()
    end
    if game.getGameState("GAME_SOUND") == false then 
        SOUND.setEffectsOff()
        SimpleAudioEngine:sharedEngine():pauseAllEffects()
    end
end, "APP_ENTER_FOREGROUND")
--[=====[全局事件处理 END]=====]--
--[==========[
--[=====[新更新 START]=====]--
--[[
{
    [1] = {
            ver = "20140206135955",
            over = false,
            desc = "更新描述1",
            files = {
                        {
                            url = "xxxx1",
                            downloaded = false,
                            size = 20983,
                        },
                        {
                            url = "xxxx2",
                            downloaded = false,
                            size = 20983,
                        },
                        {
                            url = "xxxx3",
                            downloaded = false,
                            size = 20983,
                        },
                    },
        },
    [2] = {
            ver = "20140206142955",
            over = false,
            desc = "更新描述2",
            files = {
                        {
                            url = "xxxx1",
                            downloaded = false,
                            size = 20983,
                        },
                        {
                            url = "xxxx2",
                            downloaded = false,
                            size = 20983,
                        },
                    },
        },
}
--]]
function game.checkUpdate( callbackCancelUpdate, callbackCompleteUpdate )
    -- 1.准备公用的一些参数,我们要先从文件中读取出来想要的信息吧
    local _downloadProgressBar
    local _totalSize, _curSize = 0, 0
    local _lastPercentage = 0
    local UpdateState = require("tandyfw.UpdateState").new(device.documentsPath .. "update.dat")
    -- 5.开始进行更新操作吧,过程只是下载和解压
    local function updateVersion_()
        -- 看看有没有下一个要操作的
        local file = UpdateState:getNext()
        if file then 
            -- 开始一个一个的操作
            local url = file.url
            if not string.startwith(url, "http", true) then 
                url = BASESER_URL .. url
            end
            local function progressCallback( event )
                if _downloadProgressBar then 
                    local size = _curSize + event.length
                    if size > _totalSize then size = _totalSize end
                    if size < 0 then size = 0 end
                    local per = math.floor(size / _totalSize * 100)
                    if per ~= _lastPercentage then 
                        _lastPercentage = per
                        _downloadProgressBar:runProgressTo(0.1, per)
                    end
                end
            end
            local function successCallback( url, filePath, fileName )
                Log.d(TAG, "game.checkUpdate, download successCallback %s", url)
                _curSize = _curSize + file.size
                -- 下载成功了,要改变下标识位
                UpdateState:update()
                updateVersion_()
            end
            local function failCallback( url, filePath, fileName )
                Log.d(TAG, "game.checkUpdate, download failCallback %s", url)
                if _downloadProgressBar then 
                    _downloadProgressBar.removeOnFail()
                    _downloadProgressBar = nil
                end
                AlertLayer.new{
                    content     = _s("LOGIN_UPDATE_FAILED"),
                    btnStr      = { _s("COMMON_RETRY"), _s("COMMON_CANCEL") },
                    btnColor    = { AlertLayer.BTN_COLOR_YELLOW, AlertLayer.BTN_COLOR_YELLOW },
                    btnClickEvent = {
                        function()
                            _downloadProgressBar = createDownloadingProgressBar_()
                            updateVersion_()
                        end,
                        function()
                            if callbackCancelUpdate then 
                                callbackCancelUpdate()
                            end
                        end
                    },
                }
            end
            net.download(url, successCallback, failCallback, {
                unzip = true,
                deleteZip = true,
                timeout = 1800,
                progressCallback = progressCallback
                })
        else
            -- 没东东下了,可以直接gun了
            if downloadProgressBar then
                downloadProgressBar.removeOnSuccess(function ()
                    downloadProgressBar = nil
                    game.dialogVersionUpdateCompleted()
                end)
            else
                game.dialogVersionUpdateCompleted()
            end
        end
                
    end
    -- 4.创建下载进度条
    local function createDownloadingProgressBar_( )
        local layer = widget.newMaskLayer(nil, nil, ccc4(0,0,0,0))
        local top = 3.0
        if isandroid() then top = 2.0 end
        local downloadProgressBar = widget.newProgressBar{
            frontSprintParams = {file = "login/login_downloading_pb_front.png",},
            bgSprintParams = {file = "login/login_downloading_pb_bg.png"},
            ttfLabelParams = { size = 18, padding = { top = top } }
        }
        display.getRunningScene():addChild(layer)
        layer:addChild(downloadProgressBar)
        util.alignNode(downloadProgressBar, layer, display.BOTTOM_CENTER, 58)

        local label = ui.newTTFLabel{
            text = _s("LOGIN_UPDATE_DOWNLOADING"),
            size = 36
        }
        layer:addChild(label)
        util.alignNode(label, layer, display.BOTTOM_CENTER, 120)

        label:runAction(CCRepeatForever:create(transition.sequence{
            CCFadeOut:create(1),
            CCFadeIn:create(1),
            CCDelayTime:create(0.3),
            }))

        layer.removeOnSuccess = function ( onComplete )
            -- downloadProgressBar:stopAllActions()
            local lastTime = 0.5
            downloadProgressBar:runProgressTo(lastTime, 100, false, function ( progressBar,percentage )
                layer:removeSelf() 
                if onComplete then onComplete() end
            end)
        end

        layer.removeOnFail = function ( onComplete )
            layer:removeSelf() 
            if onComplete then onComplete() end
        end

        layer.runProgressTo = function ( self, time, per )
            downloadProgressBar:runProgressTo(time, per)
        end

        return layer
    end
    -- 3.查看服务器有无新的更新信息返回
    local function updateInfoCallback_( updateInfos )
        if updateInfos then 
            -- 有新的更新进来,先加进管理中
            local infos = updateInfos
            UpdateState:add(infos)
        end
        local size = UpdateState:calcTotalSize()
        if size > 0 then 
            -- 有要更新的,cow,小弹出个框来确认下
            _totalSize = size
            AlertLayer.new4OK{
                content   = _s("LOGIN_UPDATE_NOT_WIFI", _totalSize),
                ignorPressBack = true,
                btnClickEvent = {
                    function()
                        -- 创建进度条显示
                        _downloadProgressBar = createDownloadingProgressBar_()
                        _lastPercentage = 0
                        updateVersion_()
                    end,
                },
            }
        else
            -- 没有需要更新有,继续流程吧
            if callbackCompleteUpdate then callbackCompleteUpdate() end
        end              
    end
    -- 2.把本地记录的最新版本号作为参数传给服务器,看有没有需要更新的
    local ver = UpdateState:getVersion()
    NetHelper.sendGetUpdateInfo(updateInfoCallback_)
end
--[=====[新更新 END]=====]--

--]==========]






















