﻿--Create Class CardEntity
local CardEntity = class( "CardEntity", BaseEntity )

local PlayerTalentsEntity = require("model.player.PlayerTalentsEntity")
local PlayerEquipEffectEntity = require("model.player.PlayerEquipEffectEntity")

function CardEntity:ctor( properties )
	self:setProperties( properties )
	-- 经验超过上限时候, 处理
	if self:getLv() >= CARD_MAX_LEVEL then
		self:setCurexp(self:getLevelMaxExp( ))
	end
	if self:getId() and 5000 < self:getId() and self:getId() < 10000 then
		local id = self:getId() - 5000
		self:setId(id)
	end
end

CardEntity.schema = {

	
	-- 编号
	id 			= { "number", },
	-- 等级
	lv 			= { "number", 1},
	-- 上场卡组
	onwar		= { "number", 0},
	-- 主卡牌
	main 		= { "number", 0},
	-- 受保护卡牌
	locked		= { "number", },
	-- 卡牌角色关系
	cpid 		= {"number", },
	-- 星级
	star 		= { "number", 1},
	-- 进阶
	evolv 		= { "number", 1},
	-- 卡牌序号，由服务端绑定
	sn      	= { "string", },
	-- 名字
	name 		= { "string", },
	-- 图标
    icon        = { "string", },
    -- 图片
    image       = { "string", },
    -- 图片（加大版）
    imageEX     = { "string", },
    -- 当前经验
    curexp      = { "number", 0},
    -- 本级总经验
    totalexp 	= { "number", 0},
    -- COST
	cost 		= { "number", 5},
	-- 种族
	ethnicity 	= { "number", 1},
	-- 等待回合
	waitRounds 	= { "number", },
	-- 生命值
	hp 			= { "number", 100},
	-- 攻击力
	attack 		= { "number", 100},
    -- 已经收集数量
    cur 		= { "number", },
    -- 需要收集数量
    total 		= { "number", },
    -- 碎片状态
    status 		= { "number", 0},
    -- 天赋
	talent 		= { "string", "0,0,0,0,0,0,0,0,0|0,0,0,0,0,0,0,0,0"},
	-- 数量		
	count  		= { "number", 0},
	-- 装备效果
	equipEffect = { "string", ""},
	-- 主卡
	ismain 		= { "number", 0}, 
	-- 附加值
	addvalue 	= { "string", ""},
	-- 队伍卡
	teamcard	= { "number", 0},
	-- 卡牌类型
	kind		= { "number", 0},
	-- 是否超过阶级
	isover		= {"number", 0},
	-- 是否稀有
	isRare		= {"number", 0},
}

CardEntity.fields = {
	"id", 			-- 编号
	"lv", 			-- 等级
	"onwar",		-- 上场卡组
	"star", 		-- 星级
	"evolv", 		-- 进阶
	"sn", 			-- 卡牌序号
	"main", 		-- 主卡牌
	"locked",		-- 受保护卡牌
	"cpid",			-- 卡牌角色关系
	"name", 		-- 名字
	"icon", 		-- 图标
	"image",		-- 图片
	"imageEX", 		-- 大图
	"curexp",		-- 当前经验
	"totalexp",		-- 本级总经验
	"cost", 		-- COST
	"ethnicity", 	-- 种族
	"waitRounds", 	-- 等待回合
	"hp", 			-- 生命值
	"attack", 		-- 攻击力
	"cur",			-- 已经收集数量
	"total", 		-- 需要收集数量
	"status",		-- 碎片状态
	"talent",		-- 天赋
	"count", 		-- 数量
	"equipEffect",  -- 装备效果
	"ismain", 		-- 主卡
	"addvalue", 	-- 附加值
	"teamcard", 	-- 队伍卡
	"kind", 		-- 卡牌类型
	"isover",		-- 是否超过阶级
	"isRare",		-- 是否稀有
}


function CardEntity:getIsmain()
    return self.ismain_
end
function CardEntity:setIsmain(newKey)
    self.ismain_ = newKey
end

function CardEntity:getCardIcon( )
	return string.format(CARD_ICON, self:getId())
end

function CardEntity:getCardImage( )
	return string.format(CARD_IMAGE, self:getId())
end

function CardEntity:getCardImageEx( )
	return string.format(CARD_IMAGEEX, self:getId())
end

function CardEntity:getCardImageMax( )
	return string.format(CARD_IMAGEMAX, self:getId())
end

function CardEntity:getCardName( )
	local cardDB = CardDB.getByID(self:getId())
	if cardDB then
		return cardDB.name
	end
end

-- 星级
function CardEntity:getCardStar( )
	local cardDB = CardDB.getByID(self:getId())
	if cardDB then
		local starlevel = cardDB.starlevel
		if starlevel > 90 then
			starlevel = starlevel - 89
		elseif starlevel > 80 then
			starlevel = starlevel - 79
		end
		return starlevel
	end
end

-- 种族
function CardEntity:getCardRaceId( )
	local cardDB = CardDB.getByID(self:getId())
	if cardDB then
		return cardDB.race
	end
end

-- 种族
function CardEntity:getCardRace( )
	local cardDB = CardDB.getByID(self:getId())
	if cardDB then
		return RACE_NAMES[cardDB.race]
	end
end

-- 获取卡牌Cost
function CardEntity:getCardCost( )
	local cardDB = CardDB.getByID(self:getId())
	if cardDB then
		return cardDB.cost
	end
end
-- 获取卡牌Cost
function CardEntity:getCardWaitRounds( )
	local cardDB = CardDB.getByID(self:getId())
	if cardDB then
		return cardDB.wt
	end
end
-- 获取当前等级经验上限
function CardEntity:getLevelMaxExp( )
	if self:getId() > 1000 then
		return 100
	elseif self:getLv() >= CARD_MAX_LEVEL then
		return CardDB.getExp(self:getId(), CARD_MAX_LEVEL - 1)
	else
		return CardDB.getExp(self:getId(), self:getLv())
	end
end

-- 获取种族
function CardEntity:getRace( )
	local cardDB = CardDB.getByID(self:getId())
	if cardDB then
		return RACE_NAMES[cardDB.race]
	end
end

-- 技能
function CardEntity:getSkills( )
	return CardDB.getSkills(self:getId(), self.evolv_)
end

-- 当前等级攻击力 = 基础攻击 + 等级 * 攻击系数
function CardEntity:getLvAttack( )
	local cardDb = CardDB.getByID(self:getId())
	if cardDb then
		-- local evoRate = 1
		-- if self:getEvolv() == 2 then
		-- 	evoRate = GAMERATE:getEvoRate()
		-- end
		local evoRate = GAMERATE[string.format("getEvoRate_%d", self:getEvolv())](GAMERATE)
		return math.floor(cardDb.attack + (self:getLv() - 1) * cardDb.atrate * evoRate)
	end
end

-- 当前等级生命 = 基础生命 + 等级 * 生命系数
function CardEntity:getLvHp( )
	local cardDb = CardDB.getByID(self:getId())
	if cardDb then
		-- local evoRate = 1
		-- if self:getEvolv() == 2 then
		-- 	evoRate = GAMERATE:getEvoRate()
		-- end
		local evoRate = GAMERATE[string.format("getEvoRate_%d", self:getEvolv())](GAMERATE)
		return math.floor(cardDb.hp + (self:getLv() - 1) * cardDb.hprate * evoRate)
	end
end

-- 强化获取经验 = 强化星级基础经验 + 卡牌原有累计经验 * 强化经验系数
function CardEntity:getEnhancedExp( )
	return CardDB.getEnhancedExp(self:getId()) + self:getTotalexp() * GAMERATE:getEnhancedExpRate()
end

-- 强化消耗金币 = 强化获取经验 * 强化价格系数
function CardEntity:getEnhancedGlod( )
	return (CardDB.getEnhancedExp(self:getId()) + self:getTotalexp() * GAMERATE:getEnhancedExpRate()) * GAMERATE:getEnhancedGlodRate()
end

-- 出售金币 = 销售基础金币 + 卡牌原有累计经验 * 卡牌销售经验系数
function CardEntity:getSellGlod( )
	return CardDB.getSellGlod(self:getId()) + self:getTotalexp() * GAMERATE:getSellStartLevelRate()
end

-- 天赋获取经验 = 强化获取经验 / 天赋需求卡牌经验降低值
function CardEntity:getTalentExp( )
	return self:getEnhancedExp( ) / GAMERATE:getTalentExpRate()
end

-- 玩家天赋entity
function CardEntity:getTalentEntity( )
	return PlayerTalentsEntity.new{ talent = self:getTalent()}
end
-- 获取军团天赋经验值
function CardEntity:getArmyTalentExp( )
	return CardDB.getAExp(self:getId())
end
-- 获取军团天赋金币值
function CardEntity:getArmyTalentGold( )
	return CardDB.getAGold(self:getId())
end

-- 天赋增加的攻击力
function CardEntity:getTalentAttack( )
	local atk = 0
	local raceAtk = 0
	local race = self:getCardRaceId()
	local talentEntity = self:getTalentEntity( )

	if race == RACE_ALLIANCE then
		raceAtk = talentEntity:getAllianceAttack()
	elseif race == RACE_TRIBE then
		raceAtk = talentEntity:getTribeAttack()
	elseif race == RACE_UNDEAD then
		raceAtk = talentEntity:getUndeadAttack()
	end
	atk = math.floor(talentEntity:getAllRaceAtk() + raceAtk)
	return atk
end

-- 天赋增加的生命力
function CardEntity:getTalentHp( )
	local hp = 0
	local raceHp = 0
	local race = self:getCardRaceId()
	local talentEntity = self:getTalentEntity( )
	
	if race == RACE_ALLIANCE then
		raceHp = talentEntity:getAllianceHp()
	elseif race == RACE_TRIBE then
		raceHp = talentEntity:getTribeHp()
	elseif race == RACE_UNDEAD then
		raceHp = talentEntity:getUndeadHp()
	end

	hp = math.floor(talentEntity:getAllRaceHp() + raceHp)
	return hp
end

-- 玩家装备效果entity
function CardEntity:getEquipEffectEntity( )
	return PlayerEquipEffectEntity.new{ equipEffect = self.equipEffect_ }
end

-- 装备效果增加的攻击力
function CardEntity:getEquipEffectAttack( )
	local atk = 0
	local raceAtk = 0
	local race = self:getCardRaceId()
	local equipEffectEntity = self:getEquipEffectEntity( )

	if race == RACE_ALLIANCE then
		raceAtk = equipEffectEntity:getAllianceAttack()
	elseif race == RACE_TRIBE then
		raceAtk = equipEffectEntity:getTribeAttack()
	elseif race == RACE_UNDEAD then
		raceAtk = equipEffectEntity:getUndeadAttack()
	end
	if self.main_ == TRUE then
		raceAtk = raceAtk + equipEffectEntity:getMainAtk()
	end
	
	atk = math.floor(equipEffectEntity:getAllRaceAtk() + raceAtk)
	return atk
end

-- 装备效果增加的生命力
function CardEntity:getEquipEffectHp( )
	local hp = 0
	local raceHp = 0
	local race = self:getCardRaceId()
	local equipEffectEntity = self:getEquipEffectEntity( )
	
	if race == RACE_ALLIANCE then
		raceHp = equipEffectEntity:getAllianceHp()
	elseif race == RACE_TRIBE then
		raceHp = equipEffectEntity:getTribeHp()
	elseif race == RACE_UNDEAD then
		raceHp = equipEffectEntity:getUndeadHp()
	end

	if self.main_ == TRUE then
		raceHp = raceHp + equipEffectEntity:getMainHp()
	end
	hp = math.floor(equipEffectEntity:getAllRaceHp() + raceHp)
	return hp
end

-- 获得附加值
function CardEntity:getAddvalue( )
	return self.addvalue_
end
function CardEntity:setAddvalue( value )
	self.addvalue_ = value
end
-- 获得附加攻击值
function CardEntity:getAddvalueAtk( )
	local addvalue = self.addvalue_
	local atk = tonumber(string.split(addvalue, ":")[1])
	if not atk then
		atk = 0
	end
	return atk
end
-- 获得附加生命值
function CardEntity:getAddvalueHp( )
	local addvalue = self.addvalue_
	local hp = tonumber(string.split(addvalue, ":")[2])
	if not hp then
		hp = 0
	end
	return hp
end
-- 卡牌最终攻击力 包括 天赋影响, 装备影响, 附加值
function CardEntity:getFinalAttack( )
	return  math.floor(self:getLvAttack( ) + self:getTalentAttack( ) + self:getEquipEffectAttack() + self:getAddvalueAtk())
end

-- 卡牌最终生命力 包括 天赋影响, 装备影响, 附加值
function CardEntity:getFinalHp( )
	return  math.floor(self:getLvHp( ) + self:getTalentHp( ) + self:getEquipEffectHp() + self:getAddvalueHp())
end


-- [Gettor/Settor方法]
-- 编号
function CardEntity:getId()
	return self.id_
end
function CardEntity:setId(newKey)
	if newKey > 5000 then
		newKey = newKey - 5000
	end
	self.id_ = newKey
end

-- 等级
function CardEntity:getLv()
	return self.lv_
end
function CardEntity:setLv(newKey)
	self.lv_ = newKey
end

-- 进阶
function CardEntity:getEvolv()
	return self.evolv_
end
function CardEntity:setEvolv(newKey)
	self.evolv_ = newKey
end

-- 上场卡组
function CardEntity:getOnwar()
	return self.onwar_
end
function CardEntity:setOnwar(newKey)
	self.onwar_ = newKey
end

-- 主卡牌
function CardEntity:getMain()
	return self.main_
end
function CardEntity:setMain(newKey)
	self.main_ = newKey
end

-- 受保护卡牌
function CardEntity:getLocked()
	return self.locked_
end
function CardEntity:setLocked(newKey)
	self.locked_ = newKey
end

-- 卡牌角色关系
function CardEntity:getCpid()
	return self.cpid_
end
function CardEntity:setCpid(newKey)
	self.cpid_ = newKey
end

-- 星级
function CardEntity:getStar()
	return self.star_
end
function CardEntity:setStar(newKey)
	self.star_ = newKey
end

-- 卡牌序号，由服务端绑定
function CardEntity:getSn()
	return self.sn_
end
function CardEntity:setSn(newKey)
	self.sn_ = newKey
end

-- 名字
function CardEntity:getName()
	return self.name_
end
function CardEntity:setName(newKey)
	self.name_ = newKey
end

-- 图标
function CardEntity:getIcon()
	return self.icon_
end
function CardEntity:setIcon(newKey)
	self.icon_ = newKey
end

-- 图片
function CardEntity:getImage()
	return self.image_
end
function CardEntity:setImage(newKey)
	self.image_ = newKey
end

-- 图片（加大版）
function CardEntity:getImageEX()
	return self.imageEX_
end
function CardEntity:setImageEX(newKey)
	self.imageEX_ = newKey
end

-- 当前经验
function CardEntity:getCurexp()
	return self.curexp_
end
function CardEntity:setCurexp(newKey)
	self.curexp_ = newKey
end

    -- 本级总经验
function CardEntity:getTotalexp()
	return self.totalexp_
end
function CardEntity:setTotalexp(newKey)
	self.totalexp_ = newKey
end

    -- COST
function CardEntity:getCost()
	return self.cost_
end
function CardEntity:setCost(newKey)
	self.cost_ = newKey
end

-- 种族
function CardEntity:getEthnicity()
	return self.ethnicity_
end
function CardEntity:setEthnicity(newKey)
	self.ethnicity_ = newKey
end

-- 等待回合
function CardEntity:getWaitRounds()
	return self.waitRounds_
end
function CardEntity:setWaitRounds(newKey)
	self.waitRounds_ = newKey
end

-- 生命值
function CardEntity:getHp()
	return self.hp_
end
function CardEntity:setHp(newKey)
	self.hp_ = newKey
end

-- 攻击力
function CardEntity:getAttack()
	return self.attack_
end
function CardEntity:setAttack(newKey)
	self.attack_ = newKey
end

    -- 已经收集数量
function CardEntity:getCur()
	return self.cur_
end
function CardEntity:setCur(newKey)
	self.cur_ = newKey
end

    -- 需要收集数量
function CardEntity:getTotal()
	return self.total_
end
function CardEntity:setTotal(newKey)
	self.total_ = newKey
end

    -- 碎片状态
function CardEntity:getStatus()
	return self.status_
end
function CardEntity:setStatus(newKey)
	self.status_ = newKey
end

-- 天赋
function CardEntity:getTalent()
	return self.talent_
end
function CardEntity:setTalent(newKey)
	self.talent_ = newKey
end

-- 装备效果
function CardEntity:getEquipEffect()
	return self.equipEffect_
end
function CardEntity:setEquipEffect(newKey)
	self.equipEffect_ = newKey
end

-- 数量
function CardEntity:getCount()
	return self.count_
end
function CardEntity:setCount(newKey)
	self.count_ = newKey
end
-- 战队卡
function CardEntity:getTeamcard()
	return self.teamcard_
end
function CardEntity:setTeamcard(newKey)
	self.teamcard_ = newKey
end
-- 卡牌类型
function CardEntity:getKind()
	return self.kind_
end
function CardEntity:setKind(newKey)
	self.kind_ = newKey
end
-- 是否超过阶级
function CardEntity:getIsover()
	return self.isover_
end
function CardEntity:setIsover(newKey)
	self.isover_ = newKey
end
-- 是否稀有
function CardEntity:getIsRare()
	local cardDB = CardDB.getByID(self:getId())
	if cardDB and cardDB.isxy then
		return TRUE == cardDB.isxy
	else
		return self.isRare_
	end
end
function CardEntity:setIsRare(newKey)
	self.isRare_ = newKey
end
return CardEntity