--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

    战斗掉落物品集合


]]

local M = class("EntityBrList", BaseEntity)



-- [包含文件]
-- 模型常量
local I = require(PH_EN_I .. NAM_CONSTANT)

 


-- [构造方法]
-- 构造方法，定义实体初始化赋值操作。
function M:ctor(properties)
    self:setProperties(properties)
end



-- [辅助方法]
-- 属性定义，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.fields = {
    "id",
    "no",                        -- 类型：无              物品编号
    "count",                    -- 类型：无。            数量
    "typ", 		                -- 类型：无。	  	        类型
    "styp",                     -- 类型：无。            2级类型
    "price",                    -- 类型：无。            售价
    "quality",                  -- 类型：无。            品质 白绿蓝紫橙
    "usetype",                  -- 类型：无。            使用类型  0 不能使用 1 使用 2 装备
    "ext",                      -- 类型：无。            扩展

    "cansell",                  -- 无   是否可以出售
    "sex",                      -- 无   性别
    "needlv",                   -- 类型：无。            需求人物等级
    "strlv",                    -- 类型：无。            强化等级 （装备才有）
    "evolv",                    -- 类型：无。            阶数     (装备才有)
    "part",                     -- 类型：无。            部位    （装备才有）
    "pros",                     -- 类型: EntityPros     属性数组 （装备才有）
    "skills",                   -- 类型: EntitySkills   技能数组 （装备才有）

    "add",
    "setid",
    "eqid",
    "pic",
    "isOn",
    "setids",       -- 对应套装id数组

}
 

-- 属性细则，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.schema = {
    -- ID
    id          = { "number", I.UNSET_NUMBER },

    -- 物品编号
    no          = { "number", I.UNSET_NUMBER },
    -- 数量
    count  	    = { "number", M.UNSET_NUMBER },
    -- 类型
    typ  	    = { "number", M.UNSET_NUMBER },
    -- 2级类型
    styp        = { "number", M.UNSET_NUMBER },
    -- 售价
    price            = { "number", M.UNSET_NUMBER },
    -- 品质 白绿蓝紫橙
    quality          = { "number", M.UNSET_NUMBER },
    -- 使用类型  0 不能使用 1 使用 2 装备
    usetype          = { "number", M.UNSET_NUMBER },
    -- 是否可以出售
    cansell          = { "number", M.UNSET_NUMBER },
    -- 性别
    sex              = { "number", I.UNSET_NUMBER},
    -- 需求人物等级
    needlv           = { "number", I.UNSET_NUMBER},
    -- 强化等级 （装备才有）
    strlv            = { "number", I.UNSET_NUMBER},
    -- 阶数     (装备才有)
    evolv            = { "number", I.UNSET_NUMBER},
    -- 部位    （装备才有）
    part             = { "number", I.UNSET_NUMBER},
    -- 属性数组 （装备才有）
    pros             = { "table", M.UNSET_TABLE, PH_EN_FIGHT .. "EntityPros"},
    -- 技能数组 （装备才有）
    skills           = { "table", M.UNSET_TABLE, PH_EN_FIGHT .. "EntitySkills"},

    ext              = { "string", ""},
     

    add            = { "number", 0},
    -- 套装id （装备才有）
    setid           = { "number", 0},
    -- 装备关系id（装备才有）
    eqid            = { "number", 0},
    -- 传递的图片
    pic             = { "string", ""},
    -- 是否穿在身上（装备才有）
    isOn            = { "number", 0},
    -- 对应套装id数组
    setids          = { "table", {}},
}



-- [附加方法]
-- 注入根据ID注入属性
function M:inject(params)
    -- 参数赋值
    local model  = nil
    

    -- 卡牌id
    self["id_"]         = model:getId()       
    -- 卡牌等级
    -- self["lv_"]         = model:getLv() 
    -- 卡牌星级
    self["star_"]       = model:getCardStar()  
    -- 名称
    self["name_"]       = model:getCardName()
    -- 图标
    self["icon_"]       = model:getCardIcon()
    -- 图片
    self["image_"]      = model:getCardImage()
    -- 图片（加大版）
    self["imageEx_"]    = model:getCardImageEx()
    -- 经验值
    self["exp_"]        = model:getCurexp()
    -- COST值（上场值）
    self["cost_"]       = model:getCost()
    -- 种族
    self["ethnicity_"]  = model:getCardRaceId()
    -- 种族名称
    self["ethnicityName_"] = model:getCardRace()
    -- 种族
    self["waitRounds_"] = model:getCardWaitRounds()
    -- 等待回合数
    self["cd_"]         = self["waitRounds_"] 
    -- 生命值HP
    self["hp_"]         = model:getFinalHp()
    -- 攻击力
    self["attack_"]     = model:getFinalAttack()
    -- 技能集
    self.getSkills      = model.getSkills

    -- TODO: TEST
    -- self["icon_"]       = PHI_FIGHT_CARD .. "f_c_ico1_b.png"
    -- self["image_"]      = PHI_FIGHT_CARD .. "f_c_image_b.png"
end
 


-- [Gettor/Settor方法]
-- ID
function M:getId()
    return self.id_
end
function M:setId(newKey)
    self.id_ = newKey
end
  
function M:getNo()
    return self.no_
end
function M:setNo(newKey)
    -- print("...............setno...............")
    self.no_ = newKey
end

  
function M:getCount()
    return self.count_
end
function M:setCount(newKey)
    self.count_ = newKey
end 
function M:getTyp()
    return self.typ_
end
function M:setTyp(newKey)
    self.typ_ = newKey
end 
function M:getStyp()
    return self.styp_
end
function M:setStyp(newKey)
    self.styp_ = newKey
end 
function M:getPrice()
    return self.price_
end
function M:setPrice(newKey)
    self.price_ = newKey
end 
function M:getQuality()
    return self.quality_
end
function M:setQuality(newKey)
    self.quality_ = newKey
end
 
function M:getUsetype()
    return self.usetype_
end
function M:setUsetype(newKey)
    self.usetype_ = newKey
end 
function M:getCansell()
    return self.cansell_
end
function M:setCansell(newKey)
    self.cansell_ = newKey
end

function M:getSex()
    return self.sex_
end
function M:setSex(newKey)
    self.sex_ = newKey
end 
function M:getNeedlv()
    return self.needlv_
end
function M:setNeedlv(newKey)
    self.needlv_ = newKey
end
 
function M:getStrlv()
    return self.strlv_
end
function M:setStrlv(newKey)
    self.strlv_ = newKey
end 
function M:getEvolv()
    return self.evolv_
end
function M:setEvolv(newKey)
    self.evolv_ = newKey
end
 
function M:getPart()
    return self.part_
end
function M:setPart(newKey)
    self.part_ = newKey
end 
function M:getPros()
    return self.pros_
end
function M:setPros(newKey)
    self.pros_ = newKey
end 
function M:getSkills()
    return self.skills_
end
function M:setSkills(newKey)
    self.skills_ = newKey
end
function M:getExt()
    return self.ext_
end
function M:setExt(newKey)
    self.ext_ = newKey
end
-- 追加等级（装备才有）
function M:getAdd()
    return self.add_
end
function M:setAdd(newKey)
    self.add_ = newKey
end

-- 套装id （装备才有）
function M:getSetid()
    return self.setid_
end
function M:setSetid(newKey)
    self.setid_ = newKey
end

-- 装备关系id（装备才有）
function M:getEqid()
    return self.eqid_
end
function M:setEqid(newKey)
    self.eqid_ = newKey
end

-- 传递的图片
function M:getPic()
    return self.pic_
end
function M:setPic(newKey)
    self.pic_ = newKey
end

-- 是否穿在身上（装备才有）
function M:getIsOn()
    return self.isOn_
end
function M:setIsOn(newKey)
    self.isOn_ = newKey
end

-- 对应套装id数组
function M:getSetids()
    return self.setids_
end
function M:setSetids(newKey)
    self.setids_ = newKey
end


return M