--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

卡牌信息实体类，定义卡牌基础信息实体Entity相关操作方法及逻辑实现。

-   存储卡牌基础信息相关信息。


]]

local M = class("EntityCard", BaseEntity)



-- [包含文件]
-- 模型常量
local I                 = require(PH_EN_I .. NAM_CONSTANT)
-- 断言
local TDAssertUtils     = require(PH_MDL_UTIL .. "TDAssertUtils")
-- 实体[卡牌]
local CardEntity        = require("model.card.CardEntity")










-- [构造方法]
-- 构造方法，定义实体初始化赋值操作。
function M:ctor(properties)
    self:setProperties(properties)
end











-- [辅助方法]
-- 属性定义，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.fields = {
    "id",                   -- 类型：无。        卡牌id。
    "lv",                   -- 类型：无。        卡牌等级。
    "star",                 -- 类型：无。        卡牌星级。
    "sn",                   -- 类型：无。        卡牌序号，由服务端绑定。
    "po",                   -- 类型：无。        卡牌位置索引
    "cd",                   -- 类型：无。        等待回合数
    "evolv",
    "ismain",               -- 是否主卡
    "addvalue",             -- 附加值
    -- "name",                  -- 类型：无。      名称
    -- "icon",                  -- 类型：无。      图标
    -- "image",                 -- 类型：无。      图片
    -- "imageEx",               -- 类型：无。      图片（加大版）
    -- "exp",                   -- 类型：无。      经验值
    -- "cost",                  -- 类型：无。      COST值（上场值）
    -- "ethnicity",             -- 类型：无。      种族
    -- "waitRounds",            -- 类型：无。      等待回合数
    -- "hp",                    -- 类型：无。      生命值HP
    -- "attack",                -- 类型：无。      攻击力
    -- "defense",               -- 类型：无。      防御力
    -- "skills",                -- 类型：Table。   技能集合
}

-- 属性细则，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.schema = {
    -- 卡牌id
    id          = { "number", I.UNSET_NUMBER },
    -- 卡牌等级
    lv          = { "number", I.UNSET_NUMBER },
    -- 卡牌星级
    star        = { "number", I.UNSET_NUMBER },
    -- 卡牌序号，由服务端绑定
    sn          = { "string", I.UNSET_STRING },
    -- 卡牌位置索引
    po          = { "number", I.UNSET_NUMBER },

    -- 名称
    name        = { "string", I.UNSET_STRING },
    -- 图标
    icon        = { "string", I.UNSET_STRING },
    -- 图片
    image       = { "string", I.UNSET_STRING },
    -- 图片（加大版）
    imageEx     = { "string", I.UNSET_STRING },
    -- 经验值
    exp         = { "number", I.UNSET_NUMBER },
    -- COST值（上场值）
    cost        = { "number", I.UNSET_NUMBER },
    -- 种族
    ethnicity   = { "number", I.UNSET_NUMBER },
    -- 等待回合数
    waitRounds  = { "number", I.UNSET_NUMBER },
    -- 等待回合数
    cd          = { "number", I.UNSET_NUMBER },
    -- 生命值HP
    hp          = { "number", I.UNSET_NUMBER },
    -- 攻击力
    attack      = { "number", I.UNSET_NUMBER },
    -- 防御力
    defense     = { "number", I.UNSET_NUMBER },
    -- 技能集合
    skills      = { "table", I.UNSET_TABLE, PH_EN_FIGHT .. "EntitySkill" },

    evolv       = { "number", I.UNSET_NUMBER }, 

    -- 主卡 
    ismain      = { "number", FALSE }, 
    -- 附加值
    addvalue    = { "string", ""},
}


-- [附加方法]
-- 注入根据ID注入属性
function M:inject(params)
    -- 参数赋值
    local model  = nil
    local talent = nil
    local equipEffect = nil
    if params ~= nil then 
        talent = params.talent
        equipEffect = params.equipEffect
    end
    if self:getLv() == -1 then 
        self:setLv(1)
    end
    -- 获得模型
    if talent ~= nil or equipEffect ~= nil then 
        model = CardEntity.new({ id = self:getId(), main = self:getIsmain(), lv = self:getLv(), talent = talent, equipEffect = equipEffect, evolv = self:getEvolv(), addvalue = self:getAddvalue()})
    else 
        model = CardEntity.new({ id = self:getId(), main = self:getIsmain(), lv = self:getLv(), evolv = self:getEvolv(), addvalue = self:getAddvalue()})
    end

    -- 卡牌id
    self["id_"]         = model:getId()     

    self["ismain_"]     = model:getMain()  
    -- 卡牌等级
    -- self["lv_"]         = model:getLv() 
    -- 卡牌星级
    self["star_"]       = model:getCardStar()  
    -- 名称
    self["name_"]       = model:getCardName()
    -- 图标
    self["icon_"]       = model:getCardIcon()
    -- 图片
    self["image_"]      = model:getCardImage()
    -- 图片（加大版）
    self["imageEx_"]    = model:getCardImageEx()
    -- 经验值
    self["exp_"]        = model:getCurexp()
    -- COST值（上场值）
    self["cost_"]       = model:getCost()
    -- 种族
    self["ethnicity_"]  = model:getCardRaceId()
    -- 种族名称
    self["ethnicityName_"] = model:getCardRace()
    -- 种族
    self["waitRounds_"] = model:getCardWaitRounds()
    -- 等待回合数
    self["cd_"]         = self["waitRounds_"] 
    -- 生命值HP
    self["hp_"]         = model:getFinalHp()
    -- 攻击力
    self["attack_"]     = model:getFinalAttack()
    -- 技能集
    self.getSkills      = model.getSkills

    -- TODO: TEST
    -- self["icon_"]       = PHI_FIGHT_CARD .. "f_c_ico1_b.png"
    -- self["image_"]      = PHI_FIGHT_CARD .. "f_c_image_b.png"
end

-- 设置CD
function M:setWaitRounds(newCd)
    self["cd_"]         = newCd
    self["waitRounds_"] = newCd
end
-- 设置CD
M.setCd = M.setWaitRounds

function M:getEthnicityName()
    return self["ethnicityName_"]
end

function M:getFrameNameBorder()
    local ethnicity = self["ethnicity_"]
    return format("%sf_c_img_b%d%d.png", PHI_FIGHT_CARD, ethnicity, self:getEvolv() > 1 and 2 or 1)
end








function M:getIsmain()
    return self.ismain_
end
function M:setIsmain(newKey)
    self.ismain_ = newKey
end

function M:getEvolv()
    return self.evolv_
end
function M:setEvolv(newKey)
    self.evolv_ = newKey
end
-- [Gettor/Settor方法]
-- 卡牌id
function M:getId()
    return self.id_
end
function M:setId(newKey)
    self.id_ = newKey
end

-- 卡牌等级
function M:getLv()
    return self.lv_
end
function M:setLv(newKey)
    self.lv_ = newKey
end

-- 卡牌星级
function M:getStar()
    return self.star_
end
function M:setStar(newKey)
    self.star_ = newKey
end

-- 卡牌序号，由服务端绑定
function M:getSn()
    return self.sn_
end
function M:setSn(newKey)
    self.sn_ = newKey
end

-- 卡牌位置索引
function M:getPo()
    return self.po_
end
function M:setPo(newKey)
    self.po_ = newKey
end


-- 名称
function M:getName()
    return self.name_
end
function M:setName(newKey)
    self.name_ = newKey
end

-- 图标
function M:getIcon()
    return self.icon_
end
function M:setIcon(newKey)
    self.icon_ = newKey
end

-- 图片
function M:getImage()
    return self.image_
end
function M:setImage(newKey)
    self.image_ = newKey
end

-- 图片（加大版）
function M:getImageEx()
    return self.imageEx_
end
function M:setImageEx(newKey)
    self.imageEx_ = newKey
end

-- 经验值
function M:getExp()
    return self.exp_
end
function M:setExp(newKey)
    self.exp_ = newKey
end

-- COST值（上场值）
function M:getCost()
    return self.cost_
end
function M:setCost(newKey)
    self.cost_ = newKey
end

-- 种族
function M:getEthnicity()
    return self.ethnicity_
end
function M:setEthnicity(newKey)
    self.ethnicity_ = newKey
end

-- 等待回合数
function M:getWaitRounds()
    return self.waitRounds_
end
function M:setWaitRounds(newKey)
    self.waitRounds_ = newKey
end

-- 等待回合数
function M:getCd()
    return self.cd_
end
function M:setCd(newKey)
    self.cd_ = newKey
end

-- 生命值HP
function M:getHp()
    return self.hp_
end
function M:setHp(newKey)
    self.hp_ = newKey
end

-- 攻击力
function M:getAttack()
    return self.attack_
end
function M:setAttack(newKey)
    self.attack_ = newKey
end

-- 防御力
function M:getDefense()
    return self.defense_
end
function M:setDefense(newKey)
    self.defense_ = newKey
end

-- 技能集合
function M:getSkills()
    return self.skills_
end
function M:setSkills(newKey)
    self.skills_ = newKey
end

-- 是否主卡
function M:getIsmain()
    return self.ismain_
end
function M:setIsmain(newKey)
    self.ismain_ = newKey
end

-- 附加值
function M:getAddvalue()
    return self.addvalue_
end
function M:setAddvalue(newKey)
    self.addvalue_ = newKey
end

return M