
--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作信息实体类，定义战斗动作信息实体Entity相关操作方法及逻辑实现。

-   存储卡牌基础信息相关信息。

]]

local M = class("EntityFightAction", BaseEntity)



-- [包含文件]
-- 模型常量
local I = require(PH_EN_I .. NAM_CONSTANT)










-- [构造方法]
-- 构造方法，定义实体初始化赋值操作。
function M:ctor(properties)
    self:setProperties(properties)
end










-- [辅助方法]
-- 属性定义，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.fields = {
    "id",                   -- 类型：无。           ID
    "sn", 		            -- 类型：无。	  	       卡牌序号
    "actid",                -- 类型：无。            动作ID
    "varhp",                -- 类型：无。            变化值[hp]
    "varatk",               -- 类型：无。            变化值[攻击力]
    "curhp",                -- 类型：无。            当前值[hp]
    "curatk",               -- 类型：无。            当前值[攻击力]
    "po",                   -- 类型：无。            移动到的位置（action是上场，死亡等的时候移动到的位置）
    "nb",                   -- 类型：无。            动作范围及种族标识（正数为种族，负数为动作标识，相对应的枚举分别为ETHNICITY_*, MODE_ATK_*，参考config_fight.lua）
    "syn",                  -- 类型：无。            同步标识
    "tag",                  -- 类型：无。            其他标识
}

-- 属性细则，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.schema = {
    -- ID
    id          = { "number", I.UNSET_NUMBER },
    -- 战斗结果（成功、失败）
    sn  	    = { "string", "" },
    -- 动作ID
    actid  	    = { "number", I.UNSET_NUMBER },
    -- 变化值[hp]
    varhp       = { "number", I.UNSET_NUMBER },
    -- 变化值[攻击力]
    varatk      = { "number", I.UNSET_NUMBER },
    -- 当前值[hp]
    curhp       = { "number", I.UNSET_NUMBER },
    -- 当前值[攻击力]
    curatk      = { "number", I.UNSET_NUMBER },
    -- 移动到的位置（action是上场，死亡等的时候移动到的位置）
    po          = { "number", I.UNSET_NUMBER },
    -- 动作范围及种族标识（正数为种族，负数为动作标识，相对应的枚举分别为ETHNICITY_*, MODE_ATK_*，参考config_fight.lua）
    nb          = { "number", I.UNSET_NUMBER },
    -- 同步标识
    syn         = { "number", I.UNSET_NUMBER },
    -- 其他标识
    tag         = { "number", I.UNSET_NUMBER },
}










-- [Gettor/Settor方法]
-- ID
function M:getId()
    return self.id_
end
function M:setId(newKey)
    self.id_ = newKey
end

-- 战斗结果（成功、失败）
function M:getSn()
    return self.sn_
end
function M:setSn(newKey)
    self.sn_ = newKey
end

-- 动作ID
function M:getActid()
    return self.actid_
end
function M:setActid(newKey)
    self.actid_ = newKey
end

-- 变化值[hp]
function M:getVarhp()
    return self.varhp_
end
function M:setVarhp(newKey)
    self.varhp_ = newKey
end

-- 变化值[攻击力]
function M:getVaratk()
    return self.varatk_
end
function M:setVaratk(newKey)
    self.varatk_ = newKey
end

-- 当前值[hp]
function M:getCurhp()
    return self.curhp_
end
function M:setCurhp(newKey)
    self.curhp_ = newKey
end

-- 当前值[攻击力]
function M:getCuratk()
    return self.curatk_
end
function M:setCuratk(newKey)
    self.curatk_ = newKey
end

-- 移动到的位置（action是上场，死亡等的时候移动到的位置）
function M:getPo()
    return self.po_
end
function M:setPo(newKey)
    self.po_ = newKey
end

-- 动作范围及种族标识（正数为种族，负数为动作标识，相对应的枚举分别为ETHNICITY_*, MODE_ATK_*，参考config_fight.lua）
function M:getNb()
    return self.nb_
end
function M:setNb(newKey)
    self.nb_ = newKey
end

-- 同步标识
function M:getSyn()
    return self.syn_
end
function M:setSyn(newKey)
    self.syn_ = newKey
end

-- 其他标识
function M:getTag()
    return self.tag_
end
function M:setTag(newKey)
    self.tag_ = newKey
end





return M