--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗结果信息实体类，定义战斗结果基础信息实体Entity相关操作方法及逻辑实现。

-   存储卡牌基础信息相关信息。


]]

local M = class("EntityFightResult", BaseEntity)



-- [包含文件]
-- 模型常量
local I = require(PH_EN_I .. NAM_CONSTANT)










-- [构造方法]
-- 构造方法，定义实体初始化赋值操作。
function M:ctor(properties)
    self:setProperties(properties)
end



-- [辅助方法]
-- 属性定义，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.fields = {
    "id",                   -- 类型：无。           ID
    "res", 		            -- 类型：无。	  	       战斗结果（成功、失败）
    "diamond",              -- 类型：无。            钻石
    "gold",                 -- 类型：无。            金币
    "exp",                  -- 类型：无。            经验
    "fp",                   -- 类型：无。            友情点
    -- "cards",                -- 类型：EntityCard。   获得卡牌数组，不超过5张
    -- "chips",                -- 类型：EntityChip。   获得碎片数组，不超过5张
    "star",                 -- 类型：无。            获得星星
    "jjcScore",             -- 类型：无。            获得竞技场分数
    "rank",                 -- 类型：无。            排名
    "rankvar",              -- 类型：无。            排名变化值
    "brlist",               -- 类型: EntityBrList   掉落物数组
    "levels",               -- 类型: EntityLevel    地图等级数据
    "flag",
}


-- 属性细则，标识实体属性具体的类型、默认值、子元素类型、验证操作等属性相关规则
M.schema = {
    -- ID
    id          = { "number", I.UNSET_NUMBER },
    -- 战斗结果（成功、失败）
    res  	    = { "number", M.UNSET_NUMBER },
    -- 钻石
    diamond  	= { "number", M.UNSET_NUMBER },
    -- 金币
    gold        = { "number", M.UNSET_NUMBER },
    -- 经验值
    exp         = { "number", M.UNSET_NUMBER },
    -- 友情点
    fp          = { "number", M.UNSET_NUMBER },
    -- 卡牌数组，不超过5张
    -- cards       = { "table", M.UNSET_TABLE, PH_EN_FIGHT .. "EntityCard" },
    -- -- 碎片数组，不超过5张
    -- chips       = { "table", M.UNSET_TABLE, PH_EN_FIGHT .. "EntityChip" },
    
    -- 等级
    lv          = { "number", },
    -- 生命
    hp          = { "number", },
    -- 总召唤值
    totalcost   = { "number", },
    -- 卡牌上限
    ccount      = { "number", },
    -- 卡组上限
    gcount      = { "number", },
    -- 好友上限
    tcount      = { "number", },
    -- 当前经验
    curexp      = { "number", },
    -- 总经验值
    totalexp    = { "number", },

    -- 获得星星
    star        = { "number", },
    -- 获得竞技场分数
    jjcScore    = { "number", },
    -- 排名
    rank        = { "number", },
    -- 排名变化值
    rankvar     = { "number", },
    flag        = { "number", },
    -- 掉落物数组
    brlist      = { "table", M.UNSET_TABLE, PH_EN_FIGHT .. "EntityBrList" },
    -- 关卡集
    levels          = { "table", I.UNSET_TABLE, PH_EN_MAP .. "EntityLevel" },
}








-- [Gettor/Settor方法]
-- ID

function M:getId()
    return self.id_
end
function M:setId(newKey)
    self.id_ = newKey
end

function M:getFlag()
    return self.flag_
end
function M:setFlag(newKey)
    self.flag_ = newKey
end

-- 战斗结果（成功、失败）
function M:getRes()
    return self.res_
end
function M:setRes(newKey)
    self.res_ = newKey
end

-- 钻石
function M:getDiamond()
    return self.diamond_
end
function M:setDiamond(newKey)
    self.diamond_ = newKey
end

-- 金币
function M:getGold()
    return self.gold_
end
function M:setGold(newKey)
    self.gold_ = newKey
end

-- 经验值
function M:getExp()
    return self.exp_
end
function M:setExp(newKey)
    self.exp_ = newKey
end

-- 友情点
function M:getFp()
    return self.fp_
end
function M:setFp(newKey)
    self.fp_ = newKey
end

-- 卡牌数组，不超过5张
function M:getCards()
    return self.cards_
end
function M:setCards(newKey)
    self.cards_ = newKey
end

-- 碎片数组，不超过5张
function M:getChips()
    return self.chips_
end
function M:setChips(newKey)
    self.chips_ = newKey
end

    
-- 等级
function M:getLv()
    return self.lv_
end
function M:setLv(newKey)
    self.lv_ = newKey
end

-- 生命
function M:getHp()
    return self.hp_
end
function M:setHp(newKey)
    self.hp_ = newKey
end

-- 总召唤值
function M:getTotalcost()
    return self.totalcost_
end
function M:setTotalcost(newKey)
    self.totalcost_ = newKey
end

-- 卡牌上限
function M:getCcount()
    return self.ccount_
end
function M:setCcount(newKey)
    self.ccount_ = newKey
end

-- 卡组上限
function M:getGcount()
    return self.gcount_
end
function M:setGcount(newKey)
    self.gcount_ = newKey
end

-- 好友上限
function M:getTcount()
    return self.tcount_
end
function M:setTcount(newKey)
    self.tcount_ = newKey
end

-- 当前经验
function M:getCurexp()
    return self.curexp_
end
function M:setCurexp(newKey)
    self.curexp_ = newKey
end

-- 总经验值
function M:getTotalexp()
    return self.totalexp_
end
function M:setTotalexp(newKey)
    self.totalexp_ = newKey
end


-- 获得星星
function M:getStar()
    return self.star_
end
function M:setStar(newKey)
    self.star_ = newKey
end

-- 获得竞技场分数
function M:getJjcScore()
    return self.jjcScore_
end
function M:setJjcScore(newKey)
    self.jjcScore_ = newKey
end

-- 排名
function M:getRank()
    return self.rank_
end
function M:setRank(newKey)
    self.rank_ = newKey
end

-- 排名变化值
function M:getRankvar()
    return self.rankvar_
end
function M:setRankvar(newKey)
    self.rankvar_ = newKey
end


-- 掉落物数组
function M:getBrlist()
    return self.brlist_
end
function M:setBrlist(newKey)
    -- print("setbrlist........")
    self.brlist_ = newKey
end

-- 关卡集
function M:getLevels()
    return self.levels_
end
function M:setLevels(newKey)
    self.levels_ = newKey
end



return M