--Create Class RewardEntity
local RewardEntity 					= class( "RewardEntity", BaseEntity )
local GoodsMaster 					= require("cardui.GoodsMaster")
local CardMaster 					= require("cardui.CardMaster")
local GoodsEntity 					= require("model.goods.GoodsEntity")
local CardEntity 					= require("model.card.CardEntity")

-- local REWARD_TYPE_CARD  		= 0
-- local REWARD_TYPE_CHIP  		= 1
-- local REWARD_TYPE_GOODS 		= 2
-- local REWARD_TYPE_SP			= 3
-- local REWARD_TYPE_GOLD  		= 4
-- local REWARD_TYPE_FP    		= 5
-- local REWARD_TYPE_DIAMOND  		= 6
-- local REWARD_TYPE_FREEBUY  		= 7
-- local REWARD_TYPE_STAR  		= 8
-- local REWARD_TYPE_EXP   		= 9

RewardEntity.REWARD_TYPE = {
	CARD    = 0,
	CHIP    = 1,
	GOODS   = 2,
	SP      = 3,
	GOLD    = 4,
	FP      = 5,
	DIAMOND = 6,
	FREEBUY = 7,
	STAR    = 8,
	EXP     = 9,
	APOINT  = 10,
}

local BG_IMAGE = "goods/goodBack.png"
function RewardEntity:ctor( properties )
	self:setProperties( properties )
	if properties.diamonds then
		self:setDiamond(properties.diamonds)
	end
end

RewardEntity.schema = {
	-- 卡牌
	cards 			= { "table", {}, "model.card.CardEntity"},
	-- 卡牌碎片
	chips 			= { "table", {}, "model.card.CardEntity"},
	-- 体力
	sp 		 		= { "number", 0},
	-- 金币
	gold      		= { "number", 0},
	-- 友情点
	fp      		= { "number", 0},
	-- 钻石
	diamond      	= { "number", 0},
	-- 免费抽卡次数
	freebuy      	= { "number", 0},
	-- 经验
	exp      		= { "number", 0},
	-- 星星
	star 			= { "number", 0},
	-- 掉落物数组
	brlist 			= { "table", {}},
	--军团个人贡献
	apoint  		= {"number",0},
}

RewardEntity.fields = {

	"cards" ,		-- 卡牌
	"chips" ,		-- 卡牌碎片
	"sp",			-- 体力
	"gold",			-- 金币
	"fp",			-- 友情点
	"diamond",		-- 钻石
	"freebuy",		-- 免费抽卡次数
	"exp",			-- 经验
	"star",			-- 星星
	"brlist", 		-- 掉落物数组
	"apoint",		-- 军团个人贡献
}


-- [Gettor/Settor方法]
function RewardEntity:notEmpty()
	return #self.cards_   > 0 
		or #self.chips_   > 0 
		or #self.brlist_  > 0 
		or self.sp_ 	  > 0 
		or self.gold_     > 0 
		or self.fp_       > 0 
		or self.diamond_  > 0 
		or self.freebuy_  > 0 
		or self.exp_      > 0 
		or self.star_     > 0 
		or self.apoint_	  > 0
end
-- 加入数量
local function addCountLabel( count, parentNode )
	if count > 1 then
		local countLabel = ui.newTTFLabel{text = _s("X%d", count), size = 22,}
		parentNode:addChild(countLabel, 2)
		util.alignNode(countLabel, parentNode, display.BOTTOM_RIGHT, 5)
	end
end
-- 加入获得类型方法
local function addGetTypeFun( view )
	view["getType"] = function ( )
		return view._TYPE
	end
end
-- 加入获得类型方法
local function addGetCountFun( view )
	view["getCount"] = function ( )
		return view._COUNT
	end
end
-- 加入获得实体类方法
local function addGetEntityFun( view )
	view["getEntity"] = function ( )
		return view._ENTITY
	end
end
function RewardEntity:getSpView( )
	-- 物品背景
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local spView = widget.newNode{contentSize = bgSprite:getContentSize()}
	spView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, spView)
	local rewardSprite = widget.newSprite{file = "mission/img_sign_burette.png"}
	spView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, spView)
	addCountLabel(self.sp_, spView)
	spView._TYPE = RewardEntity.REWARD_TYPE.SP
	spView._COUNT = self.sp_
	addGetTypeFun(spView)
	addGetCountFun(spView)
	return spView
end
function RewardEntity:getGoldView( )
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local goldView = widget.newNode{contentSize = bgSprite:getContentSize()}
	goldView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, goldView)
	local rewardSprite = widget.newSprite{file = "reward/icon/icon_gold.png"}
	goldView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, goldView)
	addCountLabel(self.gold_, goldView)
	goldView._TYPE = RewardEntity.REWARD_TYPE.GOLD
	goldView._COUNT = self.gold_
	addGetTypeFun(goldView)
	addGetCountFun(goldView)
	return goldView
end
function RewardEntity:getFpView( )
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local fpView = widget.newNode{contentSize = bgSprite:getContentSize()}
	fpView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, fpView)
	local rewardSprite = widget.newSprite{file = "main/img_sign_friendpoint.png"}
	fpView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, fpView)
	addCountLabel(self.fp_, fpView)
	fpView._TYPE = RewardEntity.REWARD_TYPE.FP
	fpView._COUNT = self.fp_
	addGetTypeFun(fpView)
	addGetCountFun(fpView)
	return fpView
end
function RewardEntity:getDiamondView( )
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local diamondView = widget.newNode{contentSize = bgSprite:getContentSize()}
	diamondView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, diamondView)
	local rewardSprite = widget.newSprite{file = "main/img_sign_diamond.png"}
	diamondView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, diamondView)
	addCountLabel(self.diamond_, diamondView)
	diamondView._TYPE = RewardEntity.REWARD_TYPE.DIAMOND
	diamondView._COUNT = self.diamond_
	addGetTypeFun(diamondView)
	addGetCountFun(diamondView)
	return diamondView
end
function RewardEntity:getFreebuyView( )
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local freebuyView = widget.newNode{contentSize = bgSprite:getContentSize()}
	freebuyView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, freebuyView)
	local rewardSprite = widget.newSprite{file = "mission/img_sign_freebuy.png"}
	freebuyView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, freebuyView)
	addCountLabel(self.freebuy_, freebuyView)
	freebuyView._TYPE = RewardEntity.REWARD_TYPE.FREEBUY
	freebuyView._COUNT = self.freebuy_
	addGetTypeFun(freebuyView)
	addGetCountFun(freebuyView)
	return freebuyView
end
function RewardEntity:getExpView( )
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local expView = widget.newNode{contentSize = bgSprite:getContentSize()}
	expView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, expView)
	local rewardSprite = widget.newSprite{file = "main/img_sign_exp.png"}
	expView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, expView)
	addCountLabel(self.exp_, expView)
	expView._TYPE = RewardEntity.REWARD_TYPE.EXP
	expView._COUNT = self.exp_
	addGetTypeFun(expView)
	addGetCountFun(expView)
	return expView
end
function RewardEntity:getStarView( )
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local starView = widget.newNode{contentSize = bgSprite:getContentSize()}
	starView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, starView)
	local rewardSprite = widget.newSprite{file = "main/img_main_star.png"}
	starView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, starView)
	addCountLabel(self.star_, starView)
	starView._TYPE = RewardEntity.REWARD_TYPE.STAR
	starView._COUNT = self.star_
	addGetTypeFun(starView)
	addGetCountFun(starView)
	return starView
end
function RewardEntity:getApointView( ... )
	local bgSprite = widget.newSprite{file = BG_IMAGE}
	local apointView = widget.newNode{contentSize = bgSprite:getContentSize()}
	apointView:addChild(bgSprite, 0)
	util.alignNode(bgSprite, apointView)
	local rewardSprite = widget.newSprite{file = "army/img_army_my_contribute.png"}
	apointView:addChild(rewardSprite, 1)
	util.alignNode(rewardSprite, apointView)
	addCountLabel(self.apoint_, apointView)
	apointView._TYPE = RewardEntity.REWARD_TYPE.APOINT
	apointView._COUNT = self.apoint_ 
	addGetTypeFun(apointView)
	addGetCountFun(apointView)
	return apointView
end
-- 获得所有奖励Views
function RewardEntity:getRewardViews( params )
	local rewardViews = {}
	-- 物品或卡牌碎片等
	if #self.brlist_ > 0 then
        local _cache = {}
		for i,v in ipairs(self.brlist_) do
			local bgSprite = widget.newSprite{file = BG_IMAGE}
			local view = widget.newNode{contentSize = bgSprite:getContentSize()}
			view:addChild(bgSprite, 0)
			util.alignNode(bgSprite, view)
			view._COUNT = 0
			local rewardSprite
            local entity
            if v.no < 5000 then
                -- 绑成卡牌
                entity = CardEntity.new{id = v.no, lv = v.strlv, evolv = v.evolv}
                entity.type = CardMaster.MINISTARCARD
                rewardSprite = CardMaster.new(entity)
                rewardSprite:setScale(0.76)
                view:addChild(rewardSprite, 1)
                util.alignNode(rewardSprite, view)
                view._TYPE = RewardEntity.REWARD_TYPE.CARD
            elseif v.no < 10000 then
                -- 碎片
                entity = CardEntity.new{id = v.no}
				entity.type = CardMaster.MINISTARCARD
                rewardSprite = CardMaster.new(entity)
                rewardSprite:setScale(0.76)
                view:addChild(rewardSprite, 1)
                util.alignNode(rewardSprite, view)
                local chipSprite = widget.newSprite{file = "fight/result3/f_r3_cardchip.png"}
                view:addChild(chipSprite, 2)
				util.alignNode(chipSprite, view)
				view._TYPE = RewardEntity.REWARD_TYPE.CHIP
            else
                -- 物品
                entity = GoodsEntity.new(v)
                entity.type = GoodsMaster.MATERIAL
                entity.showFrame = true
                rewardSprite = GoodsMaster.new(entity)
                view:addChild(rewardSprite, 1)
                util.alignNode(rewardSprite, view)
                view._TYPE = RewardEntity.REWARD_TYPE.GOODS
            end
            if _cache[v.no] then
                view = _cache[v.no]
                view._COUNT = view._COUNT + v.count
                view._COUNT = view._COUNT == 0 and 1 or view._COUNT
                view._ENTITY:setCount(view._COUNT)
            else
                view._COUNT = v.count
                view._ENTITY = entity
                view._ENTITY:setCount(view._COUNT)
                rewardViews[#rewardViews + 1] = view
                _cache[v.no] = view
            end
		end
		-- 加上数量
		for i, view in ipairs(rewardViews) do
			addCountLabel(view._COUNT, view)
			addGetTypeFun(view)
			addGetCountFun(view)
			addGetEntityFun(view)
		end
	end
	-- 体力
	if self.sp_ > 0 then
		rewardViews[#rewardViews + 1] = self:getSpView()
	end
	-- 金币
	if self.gold_ > 0 then
		rewardViews[#rewardViews + 1] = self:getGoldView( )
	end
	-- 友情点
	if self.fp_ > 0 then
		rewardViews[#rewardViews + 1] = self:getFpView( )
	end
	-- 钻石
	if self.diamond_ > 0 then
		rewardViews[#rewardViews + 1] = self:getDiamondView( )
	end
	-- 免费抽奖
	if self.freebuy_ > 0 then
		rewardViews[#rewardViews + 1] = self:getFreebuyView( )
	end
	-- 经验
	if self.exp_ > 0 then
		rewardViews[#rewardViews + 1] = self:getExpView( )
	end
	-- 星星
	if self.star_ > 0 then
		rewardViews[#rewardViews + 1] = self:getStarView( )
	end
	-- 军团个人贡献值
	if self.apoint_ > 0 then
		rewardViews[#rewardViews + 1] = self:getApointView( )
	end
	return rewardViews
end

-- 卡牌
function RewardEntity:getCards()
	return self.cards_
end
function RewardEntity:setCards(newKey)
	self.cards_ = newKey
end

-- 卡牌碎片
function RewardEntity:getChips()
	return self.chips_
end
function RewardEntity:setChips(newKey)
	self.chips_ = newKey
end

-- 体力
function RewardEntity:getSp()
	return self.sp_
end
function RewardEntity:setSp(newKey)
	self.sp_ = newKey
end

-- 金币
function RewardEntity:getGold()
	return self.gold_
end
function RewardEntity:setGold(newKey)
	self.gold_ = newKey
end

-- 友情点
function RewardEntity:getFp()
	return self.fp_
end
function RewardEntity:setFp(newKey)
	self.fp_ = newKey
end

-- 钻石
function RewardEntity:getDiamond()
	return self.diamond_
end
function RewardEntity:setDiamond(newKey)
	self.diamond_ = newKey
end
function RewardEntity:getDiamonds()
	return self.diamond_
end
function RewardEntity:setDiamonds(newKey)
	self.diamond_ = newKey
end

-- 免费抽卡次数
function RewardEntity:getFreebuy()
	return self.freebuy_
end
function RewardEntity:setFreebuy(newKey)
	self.freebuy_ = newKey
end

-- 经验
function RewardEntity:getExp()
	return self.exp_
end
function RewardEntity:setExp(newKey)
	self.exp_ = newKey
end

-- 星星
function RewardEntity:getStar()
	return self.star_
end
function RewardEntity:setStar(newKey)
	self.star_ = newKey
end

-- 掉落物数组
function RewardEntity:getBrlist()
	return self.brlist_
end
function RewardEntity:setBrlist(newKey)
	self.brlist_ = newKey
end
-- 军团个人贡献值
function RewardEntity:getApoint()
	return self.apoint_
end
function RewardEntity:setApoint(newKey)
	self.apoint_ = newKey
end

return RewardEntity