--Create Class PlayerEquipEffectEntity
local PlayerEquipEffectEntity = class( "PlayerEquipEffectEntity", BaseEntity )

function PlayerEquipEffectEntity:ctor( properties )
	self:setProperties( properties )
	self:setEquipEffect(properties.equipEffect)
end

PlayerEquipEffectEntity.schema = {
	equipEffect 		= { "string", },
	-- 全种族加攻 (泰坦之力)
	allRaceAtk	 		= { "number", 0},
	-- 全阵营加生命 (圣光祝福)
	allRaceHp 			= { "number", 0},
	-- 联盟加攻 (荣耀圣印)
	allianceAttack 		= { "number", 0},
	-- 联盟加生命 (希望圣歌)
	allianceHp      	= { "number", 0},
	-- 部落加攻 (战争怒吼)
	tribeAttack      	= { "number", 0},
	-- 部落生命 (野性守护)
	tribeHp      		= { "number", 0},
	-- 亡灵加攻 (邪灵之握)
	undeadAttack      	= { "number", 0},
	-- 亡灵生命 (死亡契约)
	undeadHp      		= { "number", 0},
	-- 英雄生命 (天使庇护)
	heroHp 				= { "number", 0},
	-- 主卡攻击
	mainAtk				= { "number", 0},
	-- 主卡生命
	mainHp 				= { "number", 0},
}

PlayerEquipEffectEntity.fields = {
	"equipEffect",
	"allRaceAtk" ,		-- 全种族加攻 (泰坦之力)
	"allRaceHp" ,		-- 全阵营加生命 (圣光祝福)				
	"allianceAttack",	-- 联盟加攻 (荣耀圣印)
	"allianceHp",		-- 联盟加生命 (希望圣歌)
	"tribeAttack",		-- 部落加攻 (战争怒吼)
	"tribeHp",			-- 部落生命 (野性守护)
	"undeadAttack",		-- 亡灵加攻 (邪灵之握)
	"undeadHp",			-- 亡灵生命 (死亡契约)
	"heroHp",			-- 英雄生命 (天使庇护)
	"mainAtk",			-- 主卡攻击
	"mainHp",			-- 主卡生命
}

-- [Gettor/Settor方法]
function PlayerEquipEffectEntity:getEquipEffect()
	return self.equipEffect_
end
function PlayerEquipEffectEntity:setEquipEffect(newKey)
	self.equipEffect_ = newKey
	-- 计算加成
	local equipEffectTable = string.split(self.equipEffect_, "|")
	local effect, race, value
	local equipEffect
	for i, v in ipairs(equipEffectTable) do
		equipEffect = string.split(v, ":")
		effect = tonumber(equipEffect[1])
		race = tonumber(equipEffect[2])
		value = tonumber(equipEffect[3])
		if effect and race and value then
			if TALENT_EFFECT_ATK == effect then
				if RACE_ALL == race then
					self.allRaceAtk_ = self.allRaceAtk_ + value
				elseif RACE_ALLIANCE == race then
					self.allianceAttack_ = self.allianceAttack_ + value
				elseif RACE_TRIBE == race then
					self.tribeAttack_ = self.tribeAttack_ + value
				elseif RACE_UNDEAD == race then
					self.undeadAttack_ = self.undeadAttack_ + value
				elseif RACE_MAIN == race then
					self.mainAtk_ = self.mainAtk_ + value
				end
			elseif TALENT_EFFECT_HP == effect then
				if RACE_ALL == race then
					self.allRaceHp_ = self.allRaceHp_ + value
				elseif RACE_ALLIANCE == race then
					self.allianceHp_ = self.allianceHp_ + value
				elseif RACE_TRIBE == race then
					self.tribeHp_ = self.tribeHp_ + value
				elseif RACE_UNDEAD == race then
					self.undeadHp_ = self.undeadHp_ + value
				elseif RACE_HERO == race then
					self.heroHp_ = self.heroHp_ + value
				elseif RACE_MAIN == race then
					self.mainHp_ = self.mainHp_ + value
				end
			end
		end
	end
	-- print("allRaceAtk, allianceAttack, tribeAttack, undeadAttack, allRaceHp, allianceHp, tribeHp, undeadHp, heroHp", 
	-- 	self.allRaceAtk_, self.allianceAttack_, self.tribeAttack_, self.undeadAttack_, self.allRaceHp_, self.allianceHp_, self.tribeHp_, self.undeadHp_, self.heroHp_
	-- )
end

-- 全种族加攻 (泰坦之力)1
function PlayerEquipEffectEntity:getAllRaceAtk()
	return self.allRaceAtk_
end
function PlayerEquipEffectEntity:setAllRaceAtk(newKey)
	self.allRaceAtk_ = newKey
end

-- 全阵营加生命 (圣光祝福)2
function PlayerEquipEffectEntity:getAllRaceHp()
	return self.allRaceHp_
end
function PlayerEquipEffectEntity:setAllRaceHp(newKey)
	self.allRaceHp_ = newKey
end

-- 联盟加攻 (荣耀圣印)3
function PlayerEquipEffectEntity:getAllianceAttack()
	return self.allianceAttack_
end
function PlayerEquipEffectEntity:setAllianceAttack(newKey)
	self.allianceAttack_ = newKey
end

-- 联盟加生命 (希望圣歌)4
function PlayerEquipEffectEntity:getAllianceHp()
	return self.allianceHp_
end
function PlayerEquipEffectEntity:setAllianceHp(newKey)
	self.allianceHp_ = newKey
end

-- 部落加攻 (战争怒吼)5
function PlayerEquipEffectEntity:getTribeAttack()
	return self.tribeAttack_
end
function PlayerEquipEffectEntity:setTribeAttack(newKey)
	self.tribeAttack_ = newKey
end

-- 部落生命 (野性守护)6
function PlayerEquipEffectEntity:getTribeHp()
	return self.tribeHp_
end
function PlayerEquipEffectEntity:setTribeHp(newKey)
	self.tribeHp_ = newKey
end

-- 亡灵加攻 (邪灵之握)7
function PlayerEquipEffectEntity:getUndeadAttack()
	return self.undeadAttack_
end
function PlayerEquipEffectEntity:setUndeadAttack(newKey)
	self.undeadAttack_ = newKey
end

-- 亡灵生命 (死亡契约)8
function PlayerEquipEffectEntity:getUndeadHp()
	return self.undeadHp_
end
function PlayerEquipEffectEntity:setUndeadHp(newKey)
	self.undeadHp_ = newKey
end

-- 英雄生命 (天使庇护)9
function PlayerEquipEffectEntity:getHeroHp()
	return self.heroHp_
end
function PlayerEquipEffectEntity:setHeroHp(newKey)
	self.heroHp_ = newKey
end

-- 主卡攻击
function PlayerEquipEffectEntity:getMainAtk()
	return self.mainAtk_
end
function PlayerEquipEffectEntity:setMainAtk(newKey)
	self.mainAtk_ = newKey
end

-- 主卡生命
function PlayerEquipEffectEntity:getMainHp()
	return self.mainHp_
end
function PlayerEquipEffectEntity:setMainHp(newKey)
	self.mainHp_ = newKey
end

return PlayerEquipEffectEntity