--Create Class PlayerTalentsEntity
local PlayerTalentsEntity = class( "PlayerTalentsEntity", BaseEntity )
local EntityTalent = require("ccb.m.talent.EntityTalent")
local EntityArmyTalent = require("ccb.m.talent.EntityArmyTalent")

function PlayerTalentsEntity:ctor( properties )
	self:setProperties( properties )
	self:setTalent(properties.talent)
end

PlayerTalentsEntity.schema = {
	talent 				= { "string", },
	-- 全种族加攻 (泰坦之力)
	allRaceAtkLv	 	= { "number", },
	-- 全阵营加生命 (圣光祝福)
	allRaceHpLv 		= { "number", },
	-- 联盟加攻 (荣耀圣印)
	allianceAttackLv 	= { "number", },
	-- 联盟加生命 (希望圣歌)
	allianceHpLv      	= { "number", },
	-- 部落加攻 (战争怒吼)
	tribeAttackLv      	= { "number", },
	-- 部落生命 (野性守护)
	tribeHpLv      		= { "number", },
	-- 亡灵加攻 (邪灵之握)
	undeadAttackLv      = { "number", },
	-- 亡灵生命 (死亡契约)
	undeadHpLv      	= { "number", },
	-- 英雄生命 (天使庇护)
	heroHpLv 			= { "number", },
	-- 军团全种族加攻 (泰坦之力)
	armyAllRaceAtkLv	 	= { "number", },
	-- 军团全阵营加生命 (圣光祝福)
	armyAllRaceHpLv 		= { "number", },
	-- 军团联盟加攻 (荣耀圣印)
	armyAllianceAttackLv 	= { "number", },
	-- 军团联盟加生命 (希望圣歌)
	armyAllianceHpLv      	= { "number", },
	-- 军团部落加攻 (战争怒吼)
	armyTribeAttackLv      	= { "number", },
	-- 军团部落生命 (野性守护)
	armyTribeHpLv      		= { "number", },
	-- 军团亡灵加攻 (邪灵之握)
	armyUndeadAttackLv      = { "number", },
	-- 军团亡灵生命 (死亡契约)
	armyUndeadHpLv      	= { "number", },
	-- 军团英雄生命 (天使庇护)
	armyHeroHpLv 			= { "number", },
}

PlayerTalentsEntity.fields = {
	"talent",
	"allRaceAtk" ,		-- 全种族加攻 (泰坦之力)
	"allRaceHp" ,		-- 全阵营加生命 (圣光祝福)				
	"allianceAttack",	-- 联盟加攻 (荣耀圣印)
	"allianceHp",		-- 联盟加生命 (希望圣歌)
	"tribeAttack",		-- 部落加攻 (战争怒吼)
	"tribeHp",			-- 部落生命 (野性守护)
	"undeadAttack",		-- 亡灵加攻 (邪灵之握)
	"undeadHp",			-- 亡灵生命 (死亡契约)
	"heroHp",			-- 英雄生命 (天使庇护)
	"armyAllRaceAtkLv" ,	-- 军团全种族加攻 (泰坦之力)
	"armyAllRaceHpLv" ,		-- 军团全阵营加生命 (圣光祝福)				
	"armyAllianceAttackLv",	-- 军团联盟加攻 (荣耀圣印)
	"armyAllianceHpLv",		-- 军团联盟加生命 (希望圣歌)
	"armyTribeAttackLv",	-- 军团部落加攻 (战争怒吼)
	"armyTribeHpLv",		-- 军团部落生命 (野性守护)
	"armyUndeadAttackLv",	-- 军团亡灵加攻 (邪灵之握)
	"armyUndeadHpLv",		-- 军团亡灵生命 (死亡契约)
	"armyHeroHpLv",			-- 军团英雄生命 (天使庇护)
}


-- [Gettor/Settor方法]
function PlayerTalentsEntity:getTalent()
	return self.talent_
end
function PlayerTalentsEntity:setTalent(newKey)
	
	self.talent_ = newKey
	local talentTable = string.split(newKey, "|")
	local talentArr = string.split(talentTable[1], ",")
	self:setAllRaceAtkLv(talentArr[1])
	self:setAllRaceHpLv(talentArr[2])
	self:setAllianceAttackLv(talentArr[3])
	self:setAllianceHpLv(talentArr[4])
	self:setTribeAttackLv(talentArr[5])
	self:setTribeHpLv(talentArr[6])
	self:setUndeadAttackLv(talentArr[7])
	self:setUndeadHpLv(talentArr[8])
	self:setHeroHpLv(talentArr[9])
	if not talentTable[2] then
		talentTable[2] = "0,0,0,0,0,0,0,0,0"
	end
	talentArr = string.split(talentTable[2], ",")
	self:setArmyAllRaceAtkLv(talentArr[1])
	self:setArmyAllRaceHpLv(talentArr[2])
	self:setArmyAllianceAttackLv(talentArr[3])
	self:setArmyAllianceHpLv(talentArr[4])
	self:setArmyTribeAttackLv(talentArr[5])
	self:setArmyTribeHpLv(talentArr[6])
	self:setArmyUndeadAttackLv(talentArr[7])
	self:setArmyUndeadHpLv(talentArr[8])
	self:setArmyHeroHpLv(talentArr[9])
end



-- 全种族加攻 (泰坦之力)1
function PlayerTalentsEntity:getAllRaceAtk() 
	return EntityTalent.new{lv = self:getAllRaceAtkLv(), typ = TALENT_EFFECT_ATK, camp = RACE_ALL}:getValue() 
		+ EntityArmyTalent.new{lv = self:getArmyAllRaceAtkLv(), typ = TALENT_EFFECT_ATK, camp = RACE_ALL}:getTalentval()
end

function PlayerTalentsEntity:getAllRaceAtkLv()
	return self.allRaceAtkLv_
end
function PlayerTalentsEntity:setAllRaceAtkLv(newKey)
	self.allRaceAtkLv_ = newKey
end

function PlayerTalentsEntity:getArmyAllRaceAtkLv()
	return self.armyAllRaceAtkLv_
end
function PlayerTalentsEntity:setArmyAllRaceAtkLv(newKey)
	self.armyAllRaceAtkLv_ = newKey
end

-- 全阵营加生命 (圣光祝福)2
function PlayerTalentsEntity:getAllRaceHp()
	return EntityTalent.new{lv = self:getAllRaceHpLv(), typ = TALENT_EFFECT_HP, camp = RACE_ALL}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyAllRaceHpLv(), typ = TALENT_EFFECT_HP, camp = RACE_ALL}:getTalentval()
end
function PlayerTalentsEntity:getAllRaceHpLv()
	return self.allRaceHpLv_
end
function PlayerTalentsEntity:setAllRaceHpLv(newKey)
	self.allRaceHpLv_ = newKey
end
function PlayerTalentsEntity:getArmyAllRaceHpLv()
	return self.armyAllRaceHpLv_
end
function PlayerTalentsEntity:setArmyAllRaceHpLv(newKey)
	self.armyAllRaceHpLv_ = newKey
end

-- 联盟加攻 (荣耀圣印)3
function PlayerTalentsEntity:getAllianceAttack()
	return EntityTalent.new{lv = self:getAllianceAttackLv(), camp = RACE_ALLIANCE, typ = TALENT_EFFECT_ATK}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyAllianceAttackLv(), camp = RACE_ALLIANCE, typ = TALENT_EFFECT_ATK}:getTalentval()
end
function PlayerTalentsEntity:getAllianceAttackLv()
	return self.allianceAttackLv_
end
function PlayerTalentsEntity:setAllianceAttackLv(newKey)
	self.allianceAttackLv_ = newKey
end
function PlayerTalentsEntity:getArmyAllianceAttackLv()
	return self.armyAllianceAttackLv_
end
function PlayerTalentsEntity:setArmyAllianceAttackLv(newKey)
	self.armyAllianceAttackLv_ = newKey
end

-- 联盟加生命 (希望圣歌)4
function PlayerTalentsEntity:getAllianceHp()
	return EntityTalent.new{lv = self:getAllianceHpLv(), camp = RACE_ALLIANCE, typ = TALENT_EFFECT_HP}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyAllianceHpLv(), camp = RACE_ALLIANCE, typ = TALENT_EFFECT_HP}:getTalentval()
end
function PlayerTalentsEntity:getAllianceHpLv()
	return self.allianceHpLv_
end
function PlayerTalentsEntity:setAllianceHpLv(newKey)
	self.allianceHpLv_ = newKey
end
function PlayerTalentsEntity:getArmyAllianceHpLv()
	return self.armyAllianceHpLv_
end
function PlayerTalentsEntity:setArmyAllianceHpLv(newKey)
	self.armyAllianceHpLv_ = newKey
end

-- 部落加攻 (战争怒吼)5
function PlayerTalentsEntity:getTribeAttack()
	return EntityTalent.new{lv = self:getTribeAttackLv(), camp = RACE_TRIBE, typ = TALENT_EFFECT_ATK}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyTribeAttackLv(), camp = RACE_TRIBE, typ = TALENT_EFFECT_ATK}:getTalentval()
end
function PlayerTalentsEntity:getTribeAttackLv()
	return self.tribeAttackLv_
end
function PlayerTalentsEntity:setTribeAttackLv(newKey)
	self.tribeAttackLv_ = newKey
end
function PlayerTalentsEntity:getArmyTribeAttackLv()
	return self.armyTribeAttackLv_
end
function PlayerTalentsEntity:setArmyTribeAttackLv(newKey)
	self.armyTribeAttackLv_ = newKey
end
-- 部落生命 (野性守护)6
function PlayerTalentsEntity:getTribeHp()
	return EntityTalent.new{lv = self:getTribeHpLv(), camp = RACE_TRIBE, typ = TALENT_EFFECT_HP}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyTribeHpLv(), camp = RACE_TRIBE, typ = TALENT_EFFECT_HP}:getTalentval()
end
function PlayerTalentsEntity:getTribeHpLv()
	return self.tribeHpLv_
end
function PlayerTalentsEntity:setTribeHpLv(newKey)
	self.tribeHpLv_ = newKey
end
function PlayerTalentsEntity:getArmyTribeHpLv()
	return self.armyTribeHpLv_
end
function PlayerTalentsEntity:setArmyTribeHpLv(newKey)
	self.armyTribeHpLv_ = newKey
end

-- 亡灵加攻 (邪灵之握)7
function PlayerTalentsEntity:getUndeadAttack()
	return EntityTalent.new{lv = self:getUndeadAttackLv(), camp = RACE_UNDEAD, typ = TALENT_EFFECT_ATK}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyUndeadAttackLv(), camp = RACE_UNDEAD, typ = TALENT_EFFECT_ATK}:getTalentval()
end
function PlayerTalentsEntity:getUndeadAttackLv()
	return self.undeadAttackLv_
end
function PlayerTalentsEntity:setUndeadAttackLv(newKey)
	self.undeadAttackLv_ = newKey
end
function PlayerTalentsEntity:getArmyUndeadAttackLv()
	return self.armyUndeadAttackLv_
end
function PlayerTalentsEntity:setArmyUndeadAttackLv(newKey)
	self.armyUndeadAttackLv_ = newKey
end

-- 亡灵生命 (死亡契约)8
function PlayerTalentsEntity:getUndeadHp()
	return EntityTalent.new{lv = self:getUndeadHpLv(), camp = RACE_UNDEAD, typ = TALENT_EFFECT_HP}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyUndeadHpLv(), camp = RACE_UNDEAD, typ = TALENT_EFFECT_HP}:getTalentval()
end
function PlayerTalentsEntity:getUndeadHpLv()
	return self.undeadHpLv_
end
function PlayerTalentsEntity:setUndeadHpLv(newKey)
	self.undeadHpLv_ = newKey
end
function PlayerTalentsEntity:getArmyUndeadHpLv()
	return self.armyUndeadHpLv_
end
function PlayerTalentsEntity:setArmyUndeadHpLv(newKey)
	self.armyUndeadHpLv_ = newKey
end

-- 英雄生命 (天使庇护)9
function PlayerTalentsEntity:getHeroHp()
	return EntityTalent.new{lv = self:getHeroHpLv(), camp = RACE_HERO, typ = TALENT_EFFECT_HP}:getValue()
		+ EntityArmyTalent.new{lv = self:getArmyHeroHpLv(), camp = RACE_HERO, typ = TALENT_EFFECT_HP}:getTalentval()
end
function PlayerTalentsEntity:getHeroHpLv()
	return self.heroHpLv_
end
function PlayerTalentsEntity:setHeroHpLv(newKey)
	self.heroHpLv_ = newKey
end
function PlayerTalentsEntity:getArmyHeroHpLv()
	return self.armyHeroHpLv_
end
function PlayerTalentsEntity:setArmyHeroHpLv(newKey)
	self.armyHeroHpLv_ = newKey
end

function PlayerTalentsEntity:getTalentImg( index )
	
	local lvarg
	local typarg
	local camparg  
	if index == 1 then
		lvarg = self:getAllRaceAtkLv()
		typarg = TALENT_EFFECT_ATK
		camparg = RACE_ALL
		 
	elseif index == 2 then

		lvarg = self:getAllRaceHpLv()
		typarg = TALENT_EFFECT_HP
		camparg = RACE_ALL 
	elseif index == 3 then 
		lvarg = self:getAllianceAttackLv()
		typarg = TALENT_EFFECT_ATK
		camparg = RACE_ALLIANCE 
	elseif index == 4 then 
		lvarg = self:getAllianceHpLv()
		typarg = TALENT_EFFECT_HP
		camparg = RACE_ALLIANCE 
	elseif index == 5 then 
		lvarg = self:getTribeAttackLv()
		typarg = TALENT_EFFECT_ATK
		camparg = RACE_TRIBE 
	elseif index == 6 then 
		lvarg = self:getTribeHpLv()
		typarg = TALENT_EFFECT_HP
		camparg = RACE_TRIBE
		
	elseif index == 7 then 
		lvarg = self:getUndeadAttackLv()
		typarg = TALENT_EFFECT_ATK
		camparg = RACE_UNDEAD
		
	elseif index == 8 then 
		lvarg = self:getUndeadHpLv()
		typarg = TALENT_EFFECT_HP
		camparg = RACE_UNDEAD
		
	elseif index == 9 then	 
		lvarg = self:getHeroHpLv()
		typarg = TALENT_EFFECT_HP
		camparg = RACE_HERO
		
	end
	
	return format("talent_%s_%s.png",
	 EntityTalent.new{lv = lvarg, typ = typarg, camp = camparg}:getCamp(), 
	 EntityTalent.new{lv = lvarg, typ = typarg, camp = camparg}:getTyp())
end

function PlayerTalentsEntity:getTalentValue( index )
	
	local value
	if index == 1 then
		-- 全种族加攻 (泰坦之力)1
 		value = self:getAllRaceAtk() 
	elseif index == 2 then
		-- 全阵营加生命 (圣光祝福)2
 		value = self:getAllRaceHp() 
	elseif index == 3 then 
		-- 联盟加攻 (荣耀圣印)3
 		value = self:getAllianceAttack() 
	elseif index == 4 then 
		-- 联盟加生命 (希望圣歌)4
 		value = self:getAllianceHp() 
	elseif index == 5 then 
	-- 部落加攻 (战争怒吼)5
 		value = self:getTribeAttack() 
	elseif index == 6 then 
		-- 部落生命 (野性守护)6
 		value = self:getTribeHp() 
	elseif index == 7 then 
		-- 亡灵加攻 (邪灵之握)7
 		value = self:getUndeadAttack() 
	elseif index == 8 then 
		-- 亡灵生命 (死亡契约)8
 		value = self:getUndeadHp() 
	elseif index == 9 then	 
		-- 英雄生命 (天使庇护)9
 		value = self:getHeroHp() 
	end 
	return value
end




return PlayerTalentsEntity