--Create Class TriggerMsgEntity
local TriggerMsgEntity = class( "TriggerMsgEntity", BaseEntity )
local TAG = "TriggerMsgEntity"
local UNOPEN 		= 0 -- 未开启
local POPSHOW		= 1 -- 触发 弹窗
local UNPOPSHOW		= 2 -- 触发 不弹窗
local OPEN   		= 3 -- 已开启
local FIRST_GUIDE_END = 9 -- 第一轮新手引导结束标志

TriggerMsgEntity.fields = {
	"reward",					-- 1 奖励
	"unlockCard",				-- 2 卡组解锁
	"unlockCardGroup",			-- 3 卡槽解锁	
	"unlockEnhanced",			-- 4 强化解锁	
	"unlockEvolution",			-- 5 进阶解锁	
	"unlockTalent",				-- 6 天赋解锁
	"unlockStar",				-- 7 星星收集开启
	"unlockChip",				-- 8 碎片开启
	"unlockArena",				-- 9 竞技场开启
	"unlockFriend",				-- 10 好友开启
	"helpInfoStep",				-- 11 新手帮助
	"unlockExplore",			-- 12 探索解锁
	"unlockArenaWeek",			-- 13 周赛解锁
	"unlockEquip",				-- 14 探索装备
	"unlockEquipEnhanced",		-- 15 装备强化
	"unlockMining",				-- 16 挖矿
	"unlockEquipAdd",			-- 17 装备追加
	"unlockEquipEvo",			-- 18 装备进阶
	"unlockLevelStarReward",	-- 19 关卡5星奖励
	"unlockTower1",			    -- 20 藏宝海湾
	"unlockTower2",			    -- 21 幽冥魔塔
}

function TriggerMsgEntity:ctor( properties )
	local triggerMsgs = string.split(properties, "|")
	local paramsList = {"",}
	local msg = {}
	local resultProperties = {}
	for i,v in ipairs(triggerMsgs) do
		local triggerMsg = string.split(v, ":")
		local triggerId = tonumber(triggerMsg[1])
		if 2 == triggerId or 3 == triggerId then
			local temp = string.split(triggerMsg[2], "-")
			msg[triggerId] = {id = tonumber(temp[1]), isShow = tonumber(temp[2])}
		else
			msg[triggerId] = tonumber(triggerMsg[2])
		end
	end
	-- 重新构建参数
	for i = 1, #TriggerMsgEntity.fields do
		if msg[i] then
			resultProperties[TriggerMsgEntity.fields[i]] = msg[i]
		end
	end
	self:setProperties( resultProperties )
end

TriggerMsgEntity.schema = {
	-- 奖励 (这里先用int,等到确认后需要修改成字符串)
	reward 					= { "number", 0},
	-- 卡组解锁	
	unlockCard	 			= { "table", {id = 0, isShow = 2}},
	-- 卡槽解锁	
	unlockCardGroup			= { "table", {id = 0, isShow = 2}},
	-- 强化解锁	
	unlockEnhanced			= { "number", 0},
	-- 进阶解锁	
	unlockEvolution			= { "number", 0},
	-- 天赋解锁	
	unlockTalent			= { "number", 0},
	-- 星星收集开启	
	unlockStar				= { "number", 0},
	-- 碎片开启	
	unlockChip				= { "number", 0},
 	-- 竞技场开启	
 	unlockArena				= { "number", 0},
 	-- 好友开启	
 	unlockFriend			= { "number", 0},
 	-- 新手帮助	
 	helpInfoStep 			= { "number", 3},
 	-- 探索解锁	
 	unlockExplore 			= { "number", 0},
 	-- 周赛解锁	
 	unlockArenaWeek 		= { "number", 0},
 	-- 探索装备	
 	unlockEquip 			= { "number", 0},
 	-- 装备强化	
 	unlockEquipEnhanced		= { "number", 0},
 	-- 挖矿	
	unlockMining			= { "number", 0},
	-- 17 装备追加	
	unlockEquipAdd			= { "number", 0},
	-- 18 装备进阶	
	unlockEquipEvo			= { "number", 0},
	-- 19 关卡5星奖励
	unlockLevelStarReward	= { "number", 0},
	-- 20 藏宝海湾
	unlockTower1			= { "number", 0},
	-- 21 幽冥魔塔
	unlockTower2			= { "number", 0},    
}

function TriggerMsgEntity:printProperties()
	Log.d(TAG, "unlockCard", self.unlockCard_.id, self.unlockCard_.isShow)
	Log.d(TAG, "unlockCardGroup", self.unlockCardGroup_.id, self.unlockCardGroup_.isShow)
	Log.d(TAG, "otherProperties", self.reward_, self.unlockEnhanced_, self.unlockEvolution_,
			self.unlockTalent_, self.unlockStar_, self.unlockChip_, self.unlockArena_, self.unlockFriend_, self.helpInfoStep_, self.unlockExplore_)
end


-- [Gettor/Settor方法]

-- 奖励
function TriggerMsgEntity:getReward()
	return self.reward_
end
function TriggerMsgEntity:setReward(newKey)
	self.reward_ = newKey
end

-- 卡槽解锁
function TriggerMsgEntity:getUnlockCard()
	return self.unlockCard_
end
function TriggerMsgEntity:setUnlockCard(newKey)
	self.unlockCard_ = newKey
end
-- (解锁卡槽位)
function TriggerMsgEntity:getUnlockCardIndex()
	return self.unlockCard_.id
end
-- (卡槽解锁动画)
function TriggerMsgEntity:isShowUnlockCard()
	return self.unlockCard_.id ~= UNOPEN
end
-- (弹出窗跳到卡组)
function TriggerMsgEntity:isJumpToUnlockCard()
	return self.unlockCard_.id ~= UNOPEN and self.unlockCard_.isShow == POPSHOW
end

-- 卡组解锁	
function TriggerMsgEntity:getUnlockCardGroup()
	return self.unlockCardGroup_
end
function TriggerMsgEntity:setUnlockCardGroup(newKey)
	self.unlockCardGroup_ = newKey
end
-- (解锁卡组位)
function TriggerMsgEntity:getUnlockCardGroupIndex()
	return self.unlockCardGroup_.id
end
-- (卡组解锁动画)
function TriggerMsgEntity:isShowUnlockCardGroup()
	return self.unlockCardGroup_.id ~= UNOPEN
end
-- (弹出窗跳到卡组)
function TriggerMsgEntity:isJumpToCardGroup()
	return self.unlockCardGroup_.id ~= UNOPEN and self.unlockCardGroup_.isShow == POPSHOW
end

-- 强化解锁	
function TriggerMsgEntity:getUnlockEnhanced()
	return self.unlockEnhanced_
end
function TriggerMsgEntity:setUnlockEnhanced(newKey)
	self.unlockEnhanced_ = newKey
end
-- (强化开启没)
function TriggerMsgEntity:isUnlockEnhanced()
	return self.unlockEnhanced_ ~= UNOPEN 
end
-- (强化帮助引导)
function TriggerMsgEntity:isShowEnhancedHelper()
	return self.unlockEnhanced_ == POPSHOW or self.unlockEnhanced_ == UNPOPSHOW
end
-- (弹出窗跳到强化帮助引导)
function TriggerMsgEntity:isJumpToEnhancedHelper()
	return self.unlockEnhanced_ == POPSHOW
end

-- 进阶解锁	
function TriggerMsgEntity:getUnlockEvolution()
	return self.unlockEvolution_
end
function TriggerMsgEntity:setUnlockEvolution(newKey)
	self.unlockEvolution_ = newKey
end
-- (进阶开启没)
function TriggerMsgEntity:isUnlockEvolution()
	return self.unlockEvolution_ ~= UNOPEN
end
-- (进阶帮助引导)
function TriggerMsgEntity:isShowEvolutionHelper()
	return self.unlockEvolution_ == POPSHOW or self.unlockEvolution_ == UNPOPSHOW
end
-- (弹出窗跳到进阶帮助引导)
function TriggerMsgEntity:isJumpToEvolutionHelper()
	return self.unlockEvolution_ == POPSHOW
end

-- 天赋解锁
function TriggerMsgEntity:getUnlockTalent()
	return self.unlockTalent_
end
function TriggerMsgEntity:setUnlockTalent(newKey)
	self.unlockTalent_ = newKey
end
-- (天赋开启没)
function TriggerMsgEntity:isUnlockTalent()
	return self.unlockTalent_ ~= UNOPEN
end
-- (天赋帮助引导)
function TriggerMsgEntity:isShowTalentHelper()
	return self.unlockTalent_ == POPSHOW or self.unlockTalent_ == UNPOPSHOW
end
-- (弹出窗跳到天赋帮助引导)
function TriggerMsgEntity:isJumpToTalentHelper()
	return self.unlockTalent_ == POPSHOW
end

-- 星星收集开启
function TriggerMsgEntity:getUnlockStar()
	return self.unlockStar_
end
function TriggerMsgEntity:setUnlockStar(newKey)
	self.unlockStar_ = newKey
end
-- (星星开启没)
function TriggerMsgEntity:isUnlockStar()
	return self.unlockStar_ ~= UNOPEN
end
-- (星星帮助引导)
function TriggerMsgEntity:isShowStarHelper()
	return self.unlockStar_ == POPSHOW or self.unlockStar_ == UNPOPSHOW
end
-- (弹出窗跳到星星帮助引导)
function TriggerMsgEntity:isJumpToStarHelper()
	return self.unlockStar_ == POPSHOW
end

-- 碎片开启
function TriggerMsgEntity:getUnlockChip()
	return self.unlockChip_
end
function TriggerMsgEntity:setUnlockChip(newKey)
	self.unlockChip_ = newKey
end
-- (碎片开启没)
function TriggerMsgEntity:isUnlockChip()
	return self.unlockChip_ ~= UNOPEN
end
-- (碎片帮助引导)
function TriggerMsgEntity:isShowChipHelper()
	return self.unlockChip_ == POPSHOW or self.unlockChip_ == UNPOPSHOW
end
-- (弹出窗跳到碎片帮助引导)
function TriggerMsgEntity:isJumpToChipHelper()
	return self.unlockChip_ == POPSHOW
end

-- 竞技场开启
function TriggerMsgEntity:getUnlockArena()
	return self.unlockArena_
end
function TriggerMsgEntity:setUnlockArena(newKey)
	self.unlockArena_ = newKey
end
-- (竞技场开启没)
function TriggerMsgEntity:isUnlockArena()
	return self.unlockArena_ ~= UNOPEN
end
-- (竞技场帮助引导)
function TriggerMsgEntity:isShowArenaHelper()
	return self.unlockArena_ == POPSHOW or self.unlockArena_ == UNPOPSHOW
end
-- (弹出窗跳到竞技场帮助引导)
function TriggerMsgEntity:isJumpToArenaHelper()
	return self.unlockArena_ == POPSHOW
end

-- 好友开启
function TriggerMsgEntity:getUnlockFriend()
	return self.unlockFriend_
end
function TriggerMsgEntity:setUnlockFriend(newKey)
	self.unlockFriend_ = newKey
end
-- (好友开启没)
function TriggerMsgEntity:isUnlockFriend()
	return self.unlockFriend_ ~= UNOPEN
end
-- (好友帮助引导)
function TriggerMsgEntity:isShowFriendHelper()
	return self.unlockFriend_ == POPSHOW or self.unlockFriend_ == UNPOPSHOW
end
-- (弹出窗跳到好友帮助引导)
function TriggerMsgEntity:isJumpToFriendHelper()
	return self.unlockFriend_ == POPSHOW
end

-- 新手帮助
function TriggerMsgEntity:getHelpInfoStep()
	return self.helpInfoStep_
end
function TriggerMsgEntity:setHelpInfoStep(newKey)
	self.helpInfoStep_ = newKey
end
-- (显示新手引导)
function TriggerMsgEntity:isShowHelpInfo()
	return self.helpInfoStep_ < FIRST_GUIDE_END
end

-- 探索开启
function TriggerMsgEntity:getUnlockExplore()
	return self.unlockExplore_
end
function TriggerMsgEntity:setUnlockExplore(newKey)
	self.unlockExplore_ = newKey
end
-- (探索开启没)
function TriggerMsgEntity:isUnlockExplore()
	return self.unlockExplore_ ~= UNOPEN
end
-- (探索帮助引导)
function TriggerMsgEntity:isShowExploreHelper()
	return self.unlockExplore_ == POPSHOW or self.unlockExplore_ == UNPOPSHOW
end
-- (弹出窗跳到探索帮助引导)
function TriggerMsgEntity:isJumpToExploreHelper()
	return self.unlockExplore_ == POPSHOW
end

-- 装备开启
function TriggerMsgEntity:getUnlockEquip()
	return self.unlockEquip_
end
function TriggerMsgEntity:setUnlockEquip(newKey)
	self.unlockEquip_ = newKey
end
-- (装备开启没)
function TriggerMsgEntity:isUnlockEquip()
	return self.unlockEquip_ ~= UNOPEN
end
-- (装备帮助引导)
function TriggerMsgEntity:isShowEquipHelper()
	return self.unlockEquip_ == POPSHOW or self.unlockEquip_ == UNPOPSHOW
end
-- (弹出窗跳到装备帮助引导)
function TriggerMsgEntity:isJumpToEquipHelper()
	return self.unlockEquip_ == POPSHOW
end

-- 周赛开启
function TriggerMsgEntity:getUnlockArenaWeek()
	return self.unlockArenaWeek_
end
function TriggerMsgEntity:setUnlockArenaWeek(newKey)
	self.unlockArenaWeek_ = newKey
end
-- (周赛开启没)
function TriggerMsgEntity:isUnlockArenaWeek()
	return self.unlockArenaWeek_ ~= UNOPEN
end
-- (周赛帮助引导)
function TriggerMsgEntity:isShowArenaWeekHelper()
	return self.unlockArenaWeek_ == POPSHOW or self.unlockArenaWeek_ == UNPOPSHOW
end
-- (弹出窗跳到周赛帮助引导)
function TriggerMsgEntity:isJumpToArenaWeekHelper()
	return self.unlockArenaWeek_ == POPSHOW
end

-- 装备强化开启
function TriggerMsgEntity:getUnlockEquipEnhanced()
	return self.unlockEquipEnhanced_
end
function TriggerMsgEntity:setUnlockEquipEnhanced(newKey)
	self.unlockEquipEnhanced_ = newKey
end
-- (装备强化开启没)
function TriggerMsgEntity:isUnlockEquipEnhanced()
	return self.unlockEquipEnhanced_ ~= UNOPEN
end
-- (装备强化帮助引导)
function TriggerMsgEntity:isShowEquipEnhancedHelper()
	return self.unlockEquipEnhanced_ == POPSHOW or self.unlockEquipEnhanced_ == UNPOPSHOW
end
-- (弹出窗跳到装备强化帮助引导)
function TriggerMsgEntity:isJumpToEquipEnhancedHelper()
	return self.unlockEquipEnhanced_ == POPSHOW
end

-- 挖矿开启
function TriggerMsgEntity:getUnlockMining()
	return self.unlockMining_
end
function TriggerMsgEntity:setUnlockMining(newKey)
	self.unlockMining_ = newKey
end
-- (挖矿开启没)
function TriggerMsgEntity:isUnlockMining()
	return self.unlockMining_ ~= UNOPEN
end
-- (挖矿帮助引导)
function TriggerMsgEntity:isShowMiningHelper()
	return self.unlockMining_ == POPSHOW or self.unlockMining_ == UNPOPSHOW
end
-- (弹出窗跳到挖矿帮助引导)
function TriggerMsgEntity:isJumpToMiningHelper()
	return self.unlockMining_ == POPSHOW
end

-- 装备追加开启
function TriggerMsgEntity:getUnlockEquipAdd()
	return self.unlockEquipAdd_
end
function TriggerMsgEntity:setUnlockEquipAdd(newKey)
	self.unlockEquipAdd_ = newKey
end
-- (装备追加开启没)
function TriggerMsgEntity:isUnlockEquipAdd()
	return self.unlockEquipAdd_ ~= UNOPEN
end
-- (装备追加帮助引导)
function TriggerMsgEntity:isShowEquipAddHelper()
	return self.unlockEquipAdd_ == POPSHOW or self.unlockEquipAdd_ == UNPOPSHOW
end
-- (弹出窗跳到装备追加帮助引导)
function TriggerMsgEntity:isJumpToEquipAddHelper()
	return self.unlockEquipAdd_ == POPSHOW
end

-- 装备进阶开启
function TriggerMsgEntity:getUnlockEquipEvo()
	return self.unlockEquipEvo_
end
function TriggerMsgEntity:setUnlockEquipEvo(newKey)
	self.unlockEquipEvo_ = newKey
end
-- (装备进阶开启没)
function TriggerMsgEntity:isUnlockEquipEvo()
	return self.unlockEquipEvo_ ~= UNOPEN
end
-- (装备进阶帮助引导)
function TriggerMsgEntity:isShowEquipEvoHelper()
	return self.unlockEquipEvo_ == POPSHOW or self.unlockEquipEvo_ == UNPOPSHOW
end
-- (弹出窗跳到装备进阶帮助引导)
function TriggerMsgEntity:isJumpToEquipEvoHelper()
	return self.unlockEquipEvo_ == POPSHOW
end

-- 关卡5星奖励开启
function TriggerMsgEntity:getUnlockLevelStarReward()
	return self.unlockLevelStarReward_
end
function TriggerMsgEntity:setUnlockLevelStarReward(newKey)
	self.unlockLevelStarReward_ = newKey
end
-- (关卡5星奖励开启没)
function TriggerMsgEntity:isUnlockLevelStarReward()
	return self.unlockLevelStarReward_ ~= UNOPEN
end
-- (关卡5星奖励帮助引导)
function TriggerMsgEntity:isShowLevelStarRewardHelper()
	return self.unlockLevelStarReward_ == POPSHOW or self.unlockLevelStarReward_ == UNPOPSHOW
end
-- (弹出窗跳到关卡5星奖励帮助引导)
function TriggerMsgEntity:isJumpToLevelStarRewardHelper()
	return self.unlockLevelStarReward_ == POPSHOW
end

-- 藏宝海湾开启
function TriggerMsgEntity:getUnlockTower1()
	return self.unlockTower1_
end
function TriggerMsgEntity:setUnlockTower1(newKey)
	self.unlockTower1_ = newKey
end
-- (藏宝海湾开启没)
function TriggerMsgEntity:isUnlockTower1()
	return self.unlockTower1_ ~= UNOPEN
end
-- (藏宝海湾帮助引导)
function TriggerMsgEntity:isShowTower1Helper()
	return self.unlockTower1_ == POPSHOW or self.unlockTower1_ == UNPOPSHOW
end
-- (弹出窗跳到藏宝海湾帮助引导)
function TriggerMsgEntity:isJumpToTower1Helper()
	return self.unlockTower1_ == POPSHOW
end

-- 幽冥魔塔开启
function TriggerMsgEntity:getUnlockTower1()
	return self.unlockTower2_
end
function TriggerMsgEntity:setUnlockTower2(newKey)
	self.unlockTower2_ = newKey
end
-- (幽冥魔塔开启没)
function TriggerMsgEntity:isUnlockTower2()
	return self.unlockTower2_ ~= UNOPEN
end
-- (幽冥魔塔帮助引导)
function TriggerMsgEntity:isShowTower2Helper()
	return self.unlockTower2_ == POPSHOW or self.unlockTower2_ == UNPOPSHOW
end
-- (弹出窗跳到幽冥魔塔帮助引导)
function TriggerMsgEntity:isJumpToTower2Helper()
	return self.unlockTower2_ == POPSHOW
end

return TriggerMsgEntity