﻿ module("CommonTable", package.seeall)

--适合索引从1开始的连续Table                                                                 \/]]
function reverse(self)
    local copy = table.copy(self);
    local len = #self;

    for i = 1, len do
        self[i] = copy[len];
        len = (len - 1);
    end;

    return self;
end;

--------------------------------------------------------------------------------
--适合索引从1开始的连续Table
function slice(self, from, len)
    if (from + 0) < 0 then
        from = (#self + 1 + from);
    elseif from == 0 then
        from = (from + 1);
    end;

    if not len then
        len = #self;
    elseif len <= 0 then
        len = (#self + len + 1);
    end;

    for i, v in ipairs(self) do
        if i < from or i > len then
            self[i] = nil;
        end;
    end;

    return self;
end;

--------------------------------------------------------------------------------
--适合索引从1开始的连续Table
function merge(self, ...)
    for i = 1, arg.n do
    	local count = #self
        for k, v in pairs(arg[i]) do
            self[type(k) == 'number' and (k + count) or k] = v;
        end;
    end;

    return self;
end;

--------------------------------------------------------------------------------

function dump(self)
    local result = tostring(self)..' {\n';
    local count, scope = 1, {};
    local map = {[true] = 'true', [false] = 'false'};
    local function _dump(t)
        local id = tostring(t);
        local tab = ('    '):rep(count);

        if scope[id] then
            result = ('%s%s*RECURSION*\n'):format(result, tab);
            return;
        else
            scope[id] = true;
        end;

        for k, v in pairs(t) do
            if type(v) ~= 'table' then
                result = ('%s%s[%s] => (%s)%s\n'):format(result, tab, k, type(v), tostring(map[v] or v));
            else
                result = ('%s%s[%s] => %s {\n'):format(result, tab, k, id);
                count = (count + 1);
                _dump(v);
                result = ('%s%s}\n\n'):format(result, tab);
                count = (count - 1);
            end;
        end;
    end;

    _dump(self);
    return result..'}';
end;

--------------------------------------------------------------------------------

function keys(self)
    local result = {};

    for k, v in pairs(self) do
        table.insert(result, k);
    end;

    return result;
end;

--------------------------------------------------------------------------------

function values(self)
    local result = {};

    for k, v in pairs(self) do
        table.insert(result, v);
    end;

    return result;
end;

--------------------------------------------------------------------------------

function copy(self)
    local result = {};

    for k, v in pairs(self) do
        if type(v) == 'table' then
            result[k] = table.copy(v);
        else
            result[k] = v;
        end;
    end;

    return result;
end;

--------------------------------------------------------------------------------

function each(self, callback)
    for k, v in pairs(self) do
        if type(v) == 'table' then
            self[k] = table.each(v, callback);
        else
            self[k] = callback(k, v);
        end;
    end;

    if #self > 0 then
        return self;
    end;
end;

--------------------------------------------------------------------------------

function search(self, value)
    for k, v in pairs(self) do
        if v == value then
            return k;
        end;
    end;
end;

--------------------------------------------------------------------------------

function first(self)
    local key = pairs(self);
    return self[key(self)];
end;

--------------------------------------------------------------------------------

function last(self)
	return self[#self]
end;

--------------------------------------------------------------------------------

function size(self)
    local count = 0;

    for k in pairs(self) do
        count = (count + 1);
    end;

    return count;
end;


--------------------------------------------------------------------------------

function insert(self, pos, value)
    table.insert(self, pos, value)
end

--------------------------------------------------------------------------------

--针对Key是数组的下标的情况
function push_back(self,value)
	if __DEBUG__ then
		if self == nil then
			LogFile("table.txt", "a", "Got nil table %s", debugInfo())
		end
	end
    table.insert(self, value)
end

--------------------------------------------------------------------------------

function remove(self, pos)
    table.remove(self, pos)
end

--------------------------------------------------------------------------------

function pop_back(self)
    table.remove(self)
end

--------------------------------------------------------------------------------

function clear(self)
	for k,v in pairs(self) do
	    table.remove(self, k)
	end
end

--为table t设置默认值
local key = {}
local mt = {__index = function(t) return t[key] end}
function setDefault(t, d)
	t[key] = d
	setmetatable(t, mt)
end

-- 跟踪table的访问
local index = {}
mt = {
	__index = function(t, k)
		print("*access to element " .. tostring(k))
		return t[index][k]
	end,

	__newindex = function(t, k, v)
		print("*update of element " .. tostring(k) .. " to " .. tostring(v))
		t[index][k] = v
	end
}

function track(t)
	local proxy = {}
	proxy[index] = t
	setmetatable(proxy, mt)
	return proxy
end

-- 只读的table
function readOnly(t)
	local proxy = {}
	local mt = {
		__index = t,
		__newindex = function(t, k, v)
			error("attempt to update a read-only table", 2)
		end
	}
	setmetatable(proxy, t)
	return proxy
end

-- 连接嵌套的字符串数组
function rconcat(t)
	if type(t) ~= "table" then return t end
	local res = {}
	for i = 1, #t do
		res[i] = rconcat(t[i])
	end
	return table.concat(res)
end

-- 克隆表格
function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end  -- if
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end  -- for
        return setmetatable(new_table, getmetatable(object))
	end  -- function _copy
    return _copy(object)
end  -- function deepcopy

