--Message ID表,全局
MSG_ID = {}
--Message 处理函数表,全局
MSG_HANDLER = {}
--Message 公共传递数据,全局,请用完就删...
MSG_DATA = {}

local M = {}

M.MIN_PRIORITY_FIRST	 	= -1000

M.TAG_TAB_SUB_LAYER 		= 9999
M.TAG_TAB_CHILD				= 9998
M.TAG_FIRST_LAYER 			= 9990
M.TAG_SECOND_LAYER 			= 9991


-- local _mainContainer
local _firstLayerContainer
local _levelNodes = {}
local _lastTouchPriority = 0

local TAG = "[tandyfw.CCBCenter]"

local _secondLayerContainer


--=================================================
-- CCBProxy 函数
--=================================================
--[[ 

读取CCB文件
PS.注意看里面添加的快捷方法:
proxy:bindButton(btnName, fn)
proxy:bindMenuItem(menuItemName, fn)
proxy:bindAnimationCompletedEvent( fn )
proxy:getAnimationManager( )
proxy:playCCBAnimation( animName, duration )

**Parameters:**

-   ccbiFileName:        string     ccb文件路径,相对于Resources

**Returns:**

-   return  proxy, ccbNode  代理对象, 对应的ccbNode

--]]
function M.readCCBFromFile( ccbiFileName )
	local proxy = CCBProxy:create()
	proxy:retain()

	local ccbNode = proxy:readCCBFromFile( ccbiFileName )
	ccbNode._PROXY = proxy
	CCNodeExtend.extend(ccbNode)
	ccbNode:registerNodeEvent()

	function ccbNode:onCleanup()
		if self:getProxy() then 
			self:getProxy():release()
		end
	end

	function ccbNode:getProxy()
		return ccbNode._PROXY
	end

	function ccbNode:getNode( nodeName, nodeType )
		return self:getProxy():getNodeWithType( nodeName, nodeType )
	end
	function ccbNode:getTTFLabel( nodeName, text)
		local ttfLabel = self:getProxy():getNodeWithType( nodeName, "CCLabelTTF" )
		if isstring(text) then
			ttfLabel:setString(text)
		end
		return ttfLabel
	end
	--[[ 
	
	绑定按键
	
	**Parameters:**
	
	-   btnName:        string 	     	按键名
	- 	fn: 			fun(btn, CCControlEvent int)		绑定事件
	-	title: 			string 			标题
	-	ccc3: 			ccc3 			标题颜色
	- 	state: 			enum 			状态
										CCControlStateNormal (默认)
										CCControlStateHighlighted
										CCControlStateDisabled
										CCControlStateSelected
	
	--]]
	function ccbNode:bindButton(btnName, fn, title, ccc3, state)
		local btn = self:getProxy():getNodeWithType(btnName, "CCControlButton")
		if btn then
			btn.NODE_TYPE = "CCControlButton"
			local function touchBeganHandler_( x, y )
				return util.isCustomTouchEnabled(btn)
			end
			if btn.registerScriptTouchBeganHandler then
				btn:registerScriptTouchBeganHandler(touchBeganHandler_)
			end
			self:getProxy():handleButtonEvent( btn, fn, 0 )
			if title then 
				btn:setTitleForState(CCString:create(title), state or CCControlStateNormal)
			end
			if ccc3 then
				btn:setTitleColorForState(ccc3, state or CCControlStateNormal)
			end
		end
		return btn
	end


	--[[ 
	
	设置CCControlButton文字
	
	**Parameters:**
	
	-   btnName:        string 	     	按键名
	-	title: 			string 			标题
	-	ccc3: 			ccc3 			标题颜色
	- 	state: 			enum 			状态
										CCControlStateNormal (默认)
										CCControlStateHighlighted
										CCControlStateDisabled
										CCControlStateSelected
	
	--]]
	function ccbNode:setButtonTitle( btnName, title, ccc3, state )
		local btn = self:getNode( btnName, "CCControlButton" )
		if title then 
			btn:setTitleForState(CCString:create(title), state or CCControlStateNormal)
		end
		if ccc3 then
			btn:setTitleColorForState(ccc3, state or CCControlStateNormal)
		end
	end 

	--[[ 
	
	绑定菜单项
	
	**Parameters:**
	
	-   btnName:        string 	     	菜单项名
	- 	fn: 			fun(menuItem)	绑定事件
	
	--]]
	function ccbNode:bindMenuItem( menuItemName, fn )
		self:getProxy():handleMenuEvent( self:getProxy():getNodeWithType(menuItemName, "CCMenuItemImage"), fn )
	end
	--[[ 
	
	绑定动画完成事件
	
	**Parameters:**
	
	- 	fn:   fun(CCBAnimationManager, animName, LuaEventHandler)	绑定事件
	
	--]]
	function ccbNode:bindAnimationCompletedEvent( fn )
		Log.a(isfunction(fn), TAG, "readCCBFromFile() -> proxy:bindAnimationCompletedEvent() fn must be a function ...")
		local animationManager = self:getAnimationManager( )
		if animationManager then 
			self:getProxy():handleAnimationComplate(animationManager, fn)
		else
			Log.d(TAG, "readCCBFromFile() -> proxy:bindAnimationCompletedEvent() can't get CCBAnimationManager")
		end
	end
	--[[ 
	
	取得CCBAnimationManager
	
	**Returns:**
	
	-   return CCBAnimationManager

	-- Play another timeline (sequence).
	-- 播放另一个时间线动画。
	-- "n" is returned from LuaProxy:readCCBFromFile()
	-- "n" 是前面从文件读取的节点。
	local am = CCBAnimationManager:pickFromNode(n)
	am:runAnimations"2nd Timeline"
	-- Handle timeline complate event.
	-- "proxy" must be retained.
	-- 监听时间线完成事件，"proxy" 必须执行retain函数。
	-- Callback: CCBAnimationManager *am, string timelineName, LuaEventHandler *hnd
	proxy:handleAnimationComplate(am, function(am, timelineName, hnd)
		-- do something after timeline complate
	end)
	
	--]]
	function ccbNode:getAnimationManager( )
		local animationManager = nil
		if ccbNode then
			animationManager = self:getProxy():nodeToType(ccbNode:getUserObject(), "CCBAnimationManager")
		end
		return animationManager
	end
	--[[ 
	
	播放CCB动画
	
	**Parameters:**
	
	-   animName:        string 	     动画名称
	- 	duration: 		 float 			 动画播放时长,默认为0
	
	**Returns:**
	
	-   return return
	
	--]]
	function ccbNode:playCCBAnimation( animName, duration )
		Log.a(animName, TAG, "readCCBFromFile() -> proxy:playAnimation() animName is nil...")
		local animationManager = self:getAnimationManager( )
		if animationManager then 
			animationManager:runAnimationsForSequenceNamedTweenDuration(animName, duration or 0)
		else
			Log.d(TAG, "readCCBFromFile() -> proxy:playAnimation() can't get CCBAnimationManager")
		end
	end

	return ccbNode
end

--=================================================
-- Message 函数
--=================================================

--[[ 

处理传递的信息

**Parameters:**

-   msg:        GameMessage     传递信息

-	luaValue:	CCLuaValue 		附带信息

--]]
local function handleMessage(msg, luaValue)
	local fn = MSG_HANDLER[msg:getMsgId()]
	--判断是否存在指定msgid的方法,有则调用
	if fn then
		fn(msg, luaValue)
	end
	--自动回收吧
	if msg:isAutoReturn() then
		if msg.userdata then msg.userdata = nil end
		GameMessageManager:returnMessage(msg)
	end
end
--[[ 

发送Lua信息(从Lua发送到Lua,属调用内部)

**Parameters:**

-   msgId:        int     	信息id
- 	node: 		  CCNode 	所需的node对象,无则为nil
- 	userdata: 	  随便 		用户数据,接收到时可以直接使用,类型随意

--]]
function M.sendLuaMessage(msgId, node, userdata)
	if not msgId then
		return
	end
	local msg = GameMessageManager:borrowMessage(msgId, node)
	if userdata then
		msg.userdata = userdata
	end
	handleMessage(msg, userdata)
end
--[[ 

发送CPP信息(从Lua发送到CPP,属调用外部)

**Parameters:**

-   msgId:        int     	信息id
- 	node: 		  CCNode 	所需的node对象,无则为nil

--]]
function M.sendCppMessage( msgId, node )
	GameMessageManager:sendMessage(msgId, node)
end

--=================================================
-- CCB加载 函数
--=================================================
local function createFirstLayerContainer( tag )
	local layer = widget.newColorLayer()
	layer:setTouchEnabled(false)
	layer:setVisible(false)
	layer:setTag(tag or M.TAG_FIRST_LAYER)
	return layer
end

local function clearLevelNodes_()
	-- local levelNodes = _levelNodes
	-- local function delayClearLevelNode( )
	-- 	if istable(levelNodes) and #levelNodes > 0 then
	-- 		for k,v in pairs(levelNodes) do
	-- 			try(
	-- 				function ()
	-- 					local node = tolua.cast(v.node, "CCNode")
	-- 					if node:retainCount() > 0 then 
	-- 						node:release()
	-- 					end
	-- 				end,
	-- 				function (  )
	-- 				end, nil
	-- 			)
	-- 		end
	-- 	end
	-- end
	-- util.delayCall(delayClearLevelNode, 1)

	_levelNodes = {}
end
--[[ 

更换Scene
记得要更换的scene要在onCCBLoadBack方法最后调用回调方法

**Parameters:**

-   msgId:        int/string     信息id,int时使用MSG_ID.XXX;string时使用"MSG_XXX_MODEL"
- 	controllerPath: string 		 require的路径
- 	data:		  随意 			 生成ccb时使用的参数数据,任意类型,只要你接收时能处理
- 	callback: 	  function 		 回调方法,某些场合需要,无则不写或nil
- 	, transitionType, time, more 参看display.replaceScene

--]]
function M.replaceScene( msgId, controllerPath, data, callback, transitionType, time, more )
    local sceneGame = display.newScene()
    local function _layerCallback( layer )
        -- sceneGame:addChild(layer)
        M.clearFirstLayerContainer()
        M.getFirstLayerContainer( sceneGame )
        M.pushFirstCCBNode(layer)
        display.replaceScene(sceneGame, transitionType, time, more)
        if callback then 
        	callback(layer)
        end
    end
    clearLevelNodes_()
    if isstring(msgId) or isnumber(msgId) then 
		M.loadCCB( msgId, controllerPath, data, _layerCallback )
	else
		_layerCallback(msgId)
	end
	return sceneGame
end

M.ANIM_LAYER_LEFT_TO_RIGHT = 1
M.ANIM_LAYER_RIGHT_TO_LEFT = 2

local function playerAnimationForReplaceLayer( srcLayer, destLayer, anim, node, callback )
	if srcLayer and destLayer then 
		local function destAnimCallback_()
			if _firstLayerContainer then
				_firstLayerContainer:removeFromParentAndCleanup(true)
				_firstLayerContainer = nil
			end
			_firstLayerContainer = destLayer
			destLayer:setTag(M.TAG_FIRST_LAYER)
			destLayer:setPosition(0, 0)
			_secondLayerContainer = nil
			if callback then callback(node) end
		end
		if anim == M.ANIM_LAYER_LEFT_TO_RIGHT then 
			-- 从左向右移动
			destLayer:setPosition(-CONFIG_SCREEN_WIDTH, 0)
			destLayer:setVisible(true)
			transition.moveBy(srcLayer, {
				time = LOAD_DELAY_TIME, x = CONFIG_SCREEN_WIDTH, y = 0,
				})
			transition.moveBy(destLayer, {
				time = LOAD_DELAY_TIME, x = CONFIG_SCREEN_WIDTH, y = 0,
				onComplete = destAnimCallback_
				})
		elseif anim == M.ANIM_LAYER_RIGHT_TO_LEFT then 
			-- 从右向左移动
			destLayer:setPosition(CONFIG_SCREEN_WIDTH, 0)
			destLayer:setVisible(true)
			transition.moveBy(srcLayer, {
				time = LOAD_DELAY_TIME, x = -CONFIG_SCREEN_WIDTH, y = 0,
				})
			transition.moveBy(destLayer, {
				time = LOAD_DELAY_TIME, x = -CONFIG_SCREEN_WIDTH, y = 0,
				onComplete = destAnimCallback_
				})
		else
			-- 直接点放进去就行了
			destAnimCallback_()
		end
	end
end

function M.replaceLayer( msgId, controllerPath, data, callback, anim )
	local curScene = display.getRunningScene()
	if curScene and curScene:getChildByTag(M.TAG_FIRST_LAYER) then
		-- 存在第一层容器,可以正式进行切换
		local function _layerCallback( layer )
			local secondLayer = createFirstLayerContainer(M.TAG_SECOND_LAYER)
			_secondLayerContainer = secondLayer
			curScene:addChild(secondLayer)
			clearLevelNodes_()
			M.pushLevelCCBNode(1, layer)
			playerAnimationForReplaceLayer(M.getFirstLayerContainer(), secondLayer, anim, layer, callback)
		end
		M.loadCCB( msgId, controllerPath, data, _layerCallback )
	else
		-- nnd,根本没有第一导容器,你tmd是CCBCenter.replaceScene出来的吗,不是你就调用啊,cow
		M.replaceScene(msgId, controllerPath, data, callback)
	end
end

--[[ 

加载CCB文件

**Parameters:**

-   msgId:        int/string     信息id,int时使用MSG_ID.XXX;string时使用"MSG_XXX_MODEL"
- 	controllerPath: string 		 require的路径
- 	data:		  随意 			 生成ccb时使用的参数数据,任意类型,只要你接收时能处理
- 	callback: 	  function(node) 回调方法,某些场合需要,无则不写或nil

--]]
function M.loadCCB( msgId, controllerPath, data, callback )
	if controllerPath then
		require(controllerPath)
	end
	local fn
	if isstring(msgId) then
		if MSG_ID[msgId] then
			fn = MSG_HANDLER[MSG_ID[msgId]]
		end
	else
		fn = MSG_HANDLER[msgId]
	end
	if fn then
		fn(data, callback)
	else
		Log.e(TAG, "404: Not find msgId(%s)", tostring(msgId) or "0")
	end
end

--=================================================
-- CCB层显示控制 函数
--=================================================

--[[ 

显示/隐藏CCB Node

**Parameters:**

-   visible: 			bool 			是否显示
-	levelNode:			table 			node堆栈单条信息
							level 			int 		层级,1或2
							visible 		bool 		此node的显示状态
							node 			CCNode 		node
- 	cleanUp:			bool 			是否彻底清除,默认为false
- 	firstPush: 			bool 			是否是第一次放入,默认为false,

--]]
local function displayCCBNode( visible, levelNode, cleanUp, firstPush )
	local node = levelNode.node
	if not node then return end 
	if cleanUp == true then
		--这个node即将消亡了,阿框~~~
		if node.getParent and node:getParent() then
			node:removeFromParentAndCleanup(true)
		end
		table.remove(_levelNodes)
	else
		levelNode.visible = visible
		--需要狗言残喘的node,根据其visible状态来决定是否显示出来
		if levelNode.visible then
			local container = _secondLayerContainer or M.getFirstLayerContainer() 
			container:addChild(node, levelNode.level + 100)
			if not ifnil(firstPush, false) then
				node:release()
			end
		else
			if node.getParent and node:getParent()  then
				node:retain()
				-- cow,这里一定要是false才行,要不会屎得很惨
				node:removeFromParentAndCleanup(false)
			end
		end
	end
end
--[[ 

显示/隐藏CCB容器

--]]
local function displayCCBContainer(  )
	local function _displayContainer( index, visible )
		local container = _secondLayerContainer or M.getFirstLayerContainer() 
		if container:isVisible() ~= visible then container:setVisible(visible) end
		if container:isTouchEnabled() ~= visible then container:setTouchEnabled(visible) end
	end

	local count = #_levelNodes
	if count == 0 then
		--没东东啊,全部隐藏掉
		_displayContainer(1, false)
	else
		--有东东要显示,看看是第一层还是第二层,分开处理
		local lastLevel = _levelNodes[#_levelNodes]
		_displayContainer(1, true)
		for i = 1, #_levelNodes - 1 do
			if _levelNodes[i].visible then
				util.enableCustomTouch(_levelNodes[i].node, false)
				-- util.enableWidgetInContainer(_levelNodes[i].node, false)
			end
		end
		util.enableCustomTouch(lastLevel.node)
	end
end

--[[ 

创建带遮罩功能的Layer容器

**Returns:**

-   return 带遮罩功能的Layer容器

--]]
local function createMaskContainer(  )
	-- local lastTouchPriority = M.MIN_PRIORITY_FIRST --* (#_levelNodes + 1)
	-- _lastTouchPriority = lastTouchPriority
	local function onTouch(eventType, x, y)
        if eventType == widget.TOUCHBEGAN then
            return false
        end
    end

	return widget.newMaskLayer(0, onTouch, ccc4(0, 0, 0, 0))--, lastTouchPriority
end

function M.getLastCCBNode(  )
	if _levelNodes then 
		local lastLevel = _levelNodes[#_levelNodes]
		if lastLevel and lastLevel.node then 
			return lastLevel.node
		end
	end
	return nil
end

local TAG_CCBNODE_MASK_LAYER = 9990
local OPACITY_MASK_LAYER = 220
local function createMaskLayer_( node )
	local maskLayer = node:getChildByTag(TAG_CCBNODE_MASK_LAYER)
	if not maskLayer then 
		maskLayer = widget.newLayer{ color = COLOR_MASK }
		node:addChild(maskLayer, TAG_CCBNODE_MASK_LAYER, TAG_CCBNODE_MASK_LAYER)
	else
		maskLayer = tolua.cast(maskLayer, "CCLayerColor")
	end
	return maskLayer
end

local function pushMaskLayer_()
	local layer 
	-- 给我滚去把上层要显示mask的node给翻出来!!!
	for i = #_levelNodes - 1, 1, -1 do
		if _levelNodes[i].visible == true then 
			layer = _levelNodes[i].node
			break
		end
	end
	if layer then 
		-- 有翻到就继续加吧
		local maskLayer = createMaskLayer_(layer)
		if maskLayer then 
			-- maskLayer:stopAllActions()
			maskLayer:setOpacity(0)
			transition.fadeTo(maskLayer, {time = 0.5, opacity = OPACITY_MASK_LAYER})
		end
	end
end

local function popMaskLayer_(level)
	local layer 
	-- 给我滚去把上层要显示mask的node给翻出来!!!
	for i = #_levelNodes, 1, -1 do
		if _levelNodes[i].visible == true and  _levelNodes[i].level < level then 
			layer = _levelNodes[i].node
			break
		end
	end
	if layer then 
		local maskLayer = createMaskLayer_(layer)
		if maskLayer then 
			-- maskLayer:stopAllActions()
			maskLayer:setOpacity(OPACITY_MASK_LAYER)
			transition.fadeTo(maskLayer, {time = 0.5, opacity = 0})
		end
	end
end

--[[ 

压入CCB node,这个是最通用的方法,在超出三层后只能使用这东东了

**Parameters:**

- 	index: 		int 		层下标/层数
- 	node: 		CCNode 		待压入的node
- 	cleanUp: 	bool 		是否彻底清除,默认为false

**Returns:**

-   return 是否成功压入

--]]
function M.pushCCBNode( level, node, cleanUp, animType, args )
	if isnumber(level) and node then
		if #_levelNodes > 0 then
			--取得此时最后一个
			local lastLevel = _levelNodes[#_levelNodes]
			if lastLevel.level == level then
				--没想要新来的是同级的,花客啊,要么删了俺,要么埋了俺,都是屎路一条啊
				displayCCBNode(false, lastLevel, cleanUp)
			elseif lastLevel.level > level then
				--新来的比我小,惨了,只有他能显示,我们这些都得埋起来
				--找到第一个与level同级的并显示的,从这个开始到当前最后一个,显示的都给埋起来
				displayCCBNode(false, lastLevel, cleanUp)
				for i = #_levelNodes - 1, 1, -1 do
					findedLevel = _levelNodes[i]
					if findedLevel.visible then
						displayCCBNode(false, findedLevel)
					end
					if findedLevel.level == level then
						break
					end
				end
			--else
				--新来的比我大,越大越先显示,还好我还能留着
				--怎么觉得好像什么也不用做...
			end
		end
		table.insert(_levelNodes, {level = level, visible = true, node = node} )
		--加入进来并显示出来
		displayCCBNode(true, _levelNodes[#_levelNodes], false, true)
		
		displayCCBContainer()

		-- if level > 1 then 
			pushMaskLayer_()
		-- end

		if animType then
			util.enableCustomTouch(node, false)
			args = args or {}
			local onComplete = args.onComplete
			args.onComplete = function ( ... )
				util.enableCustomTouch(node, true)
				if onComplete then onComplete() end
			end
			anim.playNodeAnimation(node, animType, args)
		end

		return true
	end
	return false
end
--[[ 

弹出/去除 CCB node

**Parameters:**

- 	level: 						int 		层数
- 	callDisplayCCBContainer: 	bool 		是否使用displayCCBContainer方法,默认为true
											当要处理瞬间多个pop时,可设置为false,最后再手动调用displayCCBContainer方法
**Returns:**

-   return 是否成功弹出

--]]
local function popCCBNode_src( level, callDisplayCCBContainer, noanim )
	local result = false
	local willShowNode 
	if #_levelNodes > 0 then
		if isnumber(level) then
			--cow,还规定了以级数删除,这个灰常麻烦啊
			while true do
				local nodeLevel = _levelNodes[#_levelNodes]
				if nodeLevel and nodeLevel.level > level then
					--在level上面的都哈哈掉
					displayCCBNode(false, nodeLevel, true)
				else
					break
				end
			end
		end
		local lastLevel = _levelNodes[#_levelNodes] 
		if level == nil then level = lastLevel.level end
		if lastLevel and lastLevel.level == level then
			--这才是真正删除想要删的
			displayCCBNode(false, lastLevel, true)
			result = true
			--开始考虑在删除后要怎么显示的了
			if #_levelNodes > 0 then
				--把本级及比本级小的第一个都显示出来吧~~~
				lastLevel = _levelNodes[#_levelNodes]
				willShowNode = lastLevel.node
				local curLevel = lastLevel.level
				for i = #_levelNodes, 1, -1 do
					lastLevel = _levelNodes[i]
					if lastLevel.level == curLevel then
						if not lastLevel.visible then
							displayCCBNode(true, lastLevel)
						end
						curLevel = curLevel - 1
					end
				end
			end
		end
		--控制是否进行层容器的显示
		if ifnil(callDisplayCCBContainer, true) then
			displayCCBContainer()
		end
		if not noanim then 
			popMaskLayer_(level)
		end
	end
	return result, willShowNode
end

function M.popCCBNode( level, callDisplayCCBContainer, animType, args )
	if not animType then 
		return popCCBNode_src( level, callDisplayCCBContainer )
	else
		local node, willShowNode 
		for i = #_levelNodes, 1, -1 do
			if _levelNodes[i].visible == true and _levelNodes[i].level == level then 
				node = _levelNodes[i].node
				willShowNode = _levelNodes[i - 1].node
				break
			end
		end
		if node then
			util.enableCustomTouch(node, false)
			-- if level > 1 then 
				popMaskLayer_(level)
			-- end
			args = args or {}
			local onComplete = args.onComplete
			local isOver = false
			args.onComplete = function ( ... )
				isOver = true
				popCCBNode_src( level, callDisplayCCBContainer, true )
				if onComplete then onComplete() end
			end
			anim.playNodeAnimation(node, animType, args)
			-- 没办法,只好再次确认是不是有完全结束动画了,没结束也要结束掉
			node:performWithDelay(function ()
				if not isOver then 
					popCCBNode_src( level, callDisplayCCBContainer, true )
					if onComplete then onComplete() end
				end
			end, (args.playTime or 0.2) + 0.05)

			return true, willShowNode, args.playTime or 0.2
		else
			return popCCBNode_src( level, callDisplayCCBContainer )
		end
	end
end

--[[ 

替换指定层弹出框

**Parameters:**

- 	level:		 int 		 指定的层数
-   node:        CCNode      弹出框主体node,若为nil则表示清除

**Returns:**

-   return 是否成功

--]]
function M.replaceLevelCCBNode( level, node, animType, args )
	if node then
		return M.pushCCBNode( level, node, true, animType, args )
	else
		return M.popCCBNode( level, nil, animType, args )
	end
end
--[[ 

压入指定层弹出框

**Parameters:**

- 	level:		 int 		 指定的层数
-   node:        CCNode      弹出框主体node

**Returns:**

-   return 是否成功

--]]
function M.pushLevelCCBNode( level, node, animType, args )
	level = level or _levelNodes[#_levelNodes].level + 1
	return M.pushCCBNode( level, node, nil, animType, args )
end
--[[ 

弹出指定层弹出框,若有指定层之上的层的弹出框也将一并弹出

**Parameters:**

- 	level:		 int 		 指定的层数

**Returns:**

-   return 是否成功

--]]
function M.popLevelCCBNode( level, animType, args )
	level = level or _levelNodes[#_levelNodes].level
	return M.popCCBNode(level, nil, animType, args)
end
--[[ 

弹出指定层弹出框,若有指定层之上的层的弹出框也将一并弹出,直至出现第一个下层层

**Parameters:**

- 	level:		 int 		 指定的层数

**Returns:**

-   return 是否成功

--]]
function M.popLevelAllCCBNode( level )
	local result = false
	while true do
		if M.popCCBNode(level, false) then 
			result = true
		else
			break 
		end
	end
	if result then
		displayCCBContainer()
	end
	return result
end
--[[ 

替换第一层弹出框

**Parameters:**

-   node:        CCNode      弹出框主体node,若为nil则表示清除

**Returns:**

-   return 是否成功

--]]
function M.replaceFirstCCBNode( node, animType, args )
	M.replaceLevelCCBNode( 1, node, animType, args )
end
--[[ 

压入第一层弹出框

**Parameters:**

-   node:        CCNode      弹出框主体node

**Returns:**

-   return 是否成功

--]]
function M.pushFirstCCBNode( node, animType, args )
	return M.pushLevelCCBNode( 1, node, animType, args )
end
--[[ 

弹出第一层弹出框,若有第二,三,...层的弹出框也将一并弹出

**Returns:**

-   return 是否成功

--]]
function M.popFirstCCBNode( animType, args )
	return M.popLevelCCBNode(1, animType, args)
end
--[[ 

弹出所有CCBNode

**Returns:**

-   return 是否成功

--]]
function M.popAllCCBNode(  )
	local result = false
	for i = #_levelNodes, 1, -1 do
		M.popCCBNode(nil, false)
		result = true
	end
	if result then
		displayCCBContainer()
	end
	return result
end
--[[ 

弹出第一层所有CCBNode

**Returns:**

-   return 是否成功

--]]
M.popAllFirstCCBNode = M.popAllCCBNode
--[[ 

压入第二层弹出框

**Parameters:**

-   node:        CCNode      弹出框主体node,若为nil则表示清除

**Returns:**

-   return 是否成功

--]]
function M.replaceSecondCCBNode( node, animType, args )
	return M.replaceLevelCCBNode( 2, node, animType, args )
end
--[[ 

压入第二层弹出框

**Parameters:**

-   node:        CCNode      弹出框主体node

**Returns:**

-   return 是否成功

--]]
function M.pushSecondCCBNode( node, animType, args )
	return M.pushLevelCCBNode( 2, node, animType, args )
end
--[[ 

弹出第二层弹出框,若有第三,四,...层的弹出框也将一并弹出

**Returns:**

-   return 是否成功

--]]
function M.popSecondCCBNode( animType, args )
	return M.popLevelCCBNode(2, animType, args)
end
--[[ 

弹出第二层所有CCBNode,若有第三,四,...层的弹出框也将一并弹出,直至出现第一个第一层

**Returns:**

-   return 是否成功

--]]
function M.popAllSecondCCBNode(  )
	return M.popLevelAllCCBNode(2)
end
--[[ 

压入第三层弹出框

**Parameters:**

-   node:        CCNode      弹出框主体node,若为nil则表示清除

**Returns:**

-   return 是否成功

--]]
function M.replaceThirdCCBNode( node, animType, args )
	return M.replaceLevelCCBNode( 3, node, animType, args )
end
--[[ 

压入第三层弹出框

**Parameters:**

-   node:        CCNode      弹出框主体node

**Returns:**

-   return 是否成功

--]]
function M.pushThirdCCBNode( node, animType, args )
	return M.pushLevelCCBNode( 3, node, animType, args )
end
--[[ 

弹出第三层弹出框,若有第四,五,...层的弹出框也将一并弹出

**Returns:**

-   return 是否成功

--]]
function M.popThirdCCBNode( animType, args )
	return M.popLevelCCBNode(3, animType, args)
end
--[[ 

弹出第三层所有CCBNode,若有第四,五,...层的弹出框也将一并弹出,直至出现第一个第二层

**Returns:**

-   return 是否成功

--]]
function M.popAllThirdCCBNode(  )
	return M.popLevelAllCCBNode( 3 )
end

function M.pushSameLevelCCBNode( node, animType, args )
	local lastLevel = _levelNodes[#_levelNodes]
	if lastLevel then 
		M.pushLevelCCBNode( lastLevel.level, node, animType, args )
		return lastLevel.level
	end
end

function M.popSameLevelCCBNode( animType, args )
	local lastLevel = _levelNodes[#_levelNodes]
	if lastLevel then 
		M.popLevelCCBNode( lastLevel.level, animType, args )
		return lastLevel.level
	end
end

function M.pushLastLevelCCBNode( node, animType, args )
	local lastLevel = _levelNodes[#_levelNodes]
	if lastLevel then 
		M.pushLevelCCBNode( lastLevel.level + 1, node, animType, args )
		return lastLevel.level + 1
	end
end

function M.popLastLevelCCBNode( animType, args )
	local lastLevel = _levelNodes[#_levelNodes]
	if lastLevel then 
		M.popLevelCCBNode( lastLevel.level, animType, args )
		return lastLevel.level
	end
end

function M.popLastSecondLevelCCBNode( animType, args )
	local lastLevel = _levelNodes[#_levelNodes]
	local level = lastLevel.level
	for i = #_levelNodes, 1, -1 do
		if _levelNodes[i].level < lastLevel.level then 
			level = _levelNodes[i].level
			break
		end
	end
	if lastLevel then 
		M.popLevelCCBNode( level, animType, args )
		return level
	end
end


--=================================================
-- 绑定view,tab 函数
--=================================================

--[[ 

绑定Tab外窗,绑定成功后触发回调函数	

**Parameters:**

-   tabViewPath:        string	     对应tabView的路径,用于require
- 	callback: 			function 	 相应的回调函数

--]]
function M.bindTabView( tabViewPath, callback )
	--先看看有没有绑定过了,未绑定则先进行绑定,已绑定就直接触发回调
	local tab = M.getFirstLayerContainer():getChildByTag(M.TAG_TAB_SUB_LAYER)
	if not tab then
		local tabView = require(tabViewPath)
		tabView.bindTabView(callback)
	else
		callback(tab)
	end
end
--[[ 

绑定视图

**Parameters:**

-   viewPath:        string     view路径
-	node: 			 CCNode 	绑定的ccb node
- 	nodeTable: 		 table 		绑定过来的绑定变量表

--]]
function M.bindView( viewPath, node, nodeTable , ...)
	local view = require(viewPath)
	view.bindView(node, nodeTable, ...)
end
--[[ 

更换Tab窗口中的内容(即点击下去后的对应内容区域,非tab的buttons也)

**Parameters:**

-   node:        CCNode     新的内容,用于更换以展示,未定义则只是清空原有内容

--]]
function M.replaceTabContainer(node, tab)
	local tabContainer = tab
	--这里是灰常麻烦的地方,进来的tab可能是纯tab的,也有可能是tab带其父容器进来的,麻烦
	if tabContainer:getTag() == -1 then
		tabContainer:setTag(M.TAG_TAB_SUB_LAYER)
	end
	if tabContainer:getParent():getTag() == -1 then
		tabContainer:getParent():setTag(M.TAG_TAB_SUB_LAYER)
	end
	--这里进行保证拿到的tabContainer一定是纯tab,不带其父容器的,你肯定看不懂,不要在那装了,花客
	if tabContainer:getChildByTag(M.TAG_TAB_SUB_LAYER) then
		tabContainer = tabContainer:getChildByTag(M.TAG_TAB_SUB_LAYER)
	end
	local lastNode = tabContainer:getChildByTag(M.TAG_TAB_CHILD)
	if lastNode then
		lastNode:removeFromParentAndCleanup(true)
	end
	if node then
		node:setTag(M.TAG_TAB_CHILD)
		tabContainer:addChild(node)
		-- util.delayCall(CCBCenter.changeAllLayerTouchPriorityInTab)
	end
end

--=================================================
-- 遮罩层 函数
--=================================================

-- function M.getLastTouchPriority(  )
-- 	return _lastTouchPriority
-- end
--[[ 

搜索node下所有有触摸事件的node,并修改触摸优先级

**Parameters:**

-   node:        CCNode     父容器
- 	priority: 	 int 		优先级
- 	force: 		 bool 		是否强制要求改变优先级

--]]
function M.changeAllLayerTouchPriority( node, priority, force )
	if node then
		if priority ~= 0 then priority = priority + M.MIN_PRIORITY_FIRST / 2 end
		GameMessageManager:changeAllLayerTouchPriority(priority, node, force or false)
	end
end
--[[ 

修改/提高 第一层内子元素的触摸优先级

--]]
function M.changeAllLayerTouchPriorityInTab()
	local tabContainer = M.getFirstLayerContainer():getChildByTag(M.TAG_TAB_SUB_LAYER)
	M.changeAllLayerTouchPriority(tabContainer, tabContainer.PRIORITY)
end
--[[ 

修改/提高 第一层顶层显示子元素的触摸优先级

--]]
function M.changeAllLayerTouchPriorityForTop(  )
	local lastLevel = _levelNodes[#_levelNodes]
	if lastLevel then
		M.changeAllLayerTouchPriority( lastLevel.node, lastLevel.node.PRIORITY )
	end
end

function M.changeAllLayerTouchPriorityToLast( node, time )
	local function callChangeAllLayerTouchPriority()
		local lastTouchPriority = _lastTouchPriority
		if node.PRIORITY then 
			lastTouchPriority = node.PRIORITY 
		end
		M.changeAllLayerTouchPriority(node, lastTouchPriority)
	end
	util.delayCall(callChangeAllLayerTouchPriority, time or 0.5)
end

--=================================================
-- MSG_DATA 函数
--=================================================

--[[ 

存放全局数据	

**Parameters:**
	
-   key:        int/string     			key,通常使用MSG_ID来表达
-	value: 		int/string/table/...	value,随意类型,接收者自己清楚就好

**Returns:**

-   return boolean 是否成功

--]]
function M.pushData( key, value )
	if key and value then
		MSG_DATA[key] = value
		return true
	else
		Log.e(TAG, "pushData() miss key or value")
		return false
	end
end
--[[ 

取出全局数据(默认取出后就从全局中删除)

**Parameters:**

-   key:        int/string     			key,通常使用MSG_ID来表达
-	hold: 		boolean					是否不删除数据,只有在需要保留数据时才设置为true,否则不填写

**Returns:**

-   return value,无则为nil

--]]
function M.popData( key, hold )
	if key then
		local result = MSG_DATA[key]
		if result and hold ~= true then
			MSG_DATA[key] = nil
		end
		return result
	else
		Log.e(TAG, "popData() miss key")
		return nil
	end
end

--=================================================
--[[ 

重置显示/隐藏CCB容器

**Parameters:**
- 	isdelay: 		boolean 		是否延迟调用,默认为true

--]]
function M.resetDisplayCCBContainer( isdelay )
	displayCCBContainer( isdelay )
end


--[[ 

取得第一层容器

**Returns:**

-   return 第一层容器

--]]
function M.getFirstLayerContainer( scene )
	if not _firstLayerContainer then 
		_firstLayerContainer = createFirstLayerContainer()
		if scene then 
			scene:addChild(_firstLayerContainer)
		else
			display.getRunningScene():addChild(_firstLayerContainer)
		end
	end
	return _firstLayerContainer
end
--[[ 

清除第一层容器

--]]
function M.clearFirstLayerContainer()
	if _firstLayerContainer then
		-- _firstLayerContainer:removeSelf()
		_firstLayerContainer = nil
	end
end

--[[ 

通过tag从指定node中取得对应的子node

**Parameters:**

-   node:        CCNode     父容器
-   tag:		 int 		元素的tag值,格式:父tag+子tag,e.g. 402,则父tag是4,子在父中是第2个
- 	nodeType:    string 	将要转换成的类型,默认nil时为CCNode,可选值如下:
								CCNode
								CCLayer
								CCSprite
								CCLabel
								CCLabelAtlas
								CCLabelBMFont
								CCMenu
								CCMenuItem
								CCControlButton
								CCTableView
								CCScrollView
								CCScale9Sprite
								CCEditBox
	
**Returns:**

-   return 取得的元素

--]]
function M.getNodeByTag( node, tag, nodeType )
	local result = node
	if tag and type(tag) == "number" then
		local tagStr = tostring(tag)
		if #tagStr % 2 == 1 then
			tagStr = "0" .. tagStr
		end
		local subTag = ""
		for i=1, #tagStr, 2 do
			if not result then return nil end
			subTag = subTag .. string.sub(tagStr, i, i + 1)
			result = result:getChildByTag(tonumber(subTag))
		end
	end
	if result == node then result = nil end
	if result and nodeType then 
		if nodeType == "CCScrollView" then 
			result = util.autoFixScrollView(result)
		else
			result = util.toXNode(result, nodeType)
		end
	end
	return result
end

function M.setScaleByTags( scale, node, tags )
	if isnumber(scale) and istable(tags) then 
		for i, tag in ipairs(tags) do
			local child = M.getNodeByTag( node, tag )
			if child then 
				child:setScale(scale)
			end
		end
	end
end

--[[ 

初始化

**Parameters:**

-	mainContainer:				CCNode 		主容器

-   firstLayerContainer:        CCNode     第一层容器

--]]
-- function M.init( mainContainer, firstLayerContainer )
-- 	_mainContainer = mainContainer
--     --添加第一层容器
-- 	if firstLayerContainer then
-- 		_firstLayerContainer = firstLayerContainer
-- 	elseif _mainContainer then
-- 		_firstLayerContainer = widget.newColorLayer()
-- 	end
-- 	_firstLayerContainer.PRIORITY = M.MIN_PRIORITY_FIRST / 2
-- 	_firstLayerContainer.INDEX = 1
-- 	_firstLayerContainer:setTouchEnabled(false)
-- 	_firstLayerContainer:setVisible(false)
-- 	_firstLayerContainer:setTag(1)
-- 	if _firstLayerContainer:getParent() == nil then	
-- 		if _mainContainer:getParent() then
-- 			_mainContainer:getParent():addChild(_firstLayerContainer)
-- 		else
-- 			_mainContainer:addChild(_firstLayerContainer)
-- 		end
-- 	end
-- end

--[[ 

注册监听

--]]
function M.registerHandler( )
	GameMessageManager:getInstance():registerGameMessageHandler(handleMessage)
end


--=================================================

return M