local GameHelper = {}

GameHelper.EVENT_NAME_BACK_TO_EXIT = "EVENT_NAME_BACK_TO_EXIT"
GameHelper.EVENT_NAME_RETURN_TO_LOGIN = "EVENT_NAME_RETURN_TO_LOGIN"
--[[ 

绑定Scene事件

**Parameters:**

-   scene:        CCScene     待绑定的scene

**Returns:**

-   return return

--]]
function GameHelper.bindSceneEvent( scene )
    if not scene then scene = display.getRunningScene() end
    if not scene.addAutoCleanImage then CCSceneExtend.extend(scene) end
    if device.platform == "android" then
        scene.layer = display.newLayer()
        scene.layer:addKeypadEventListener(function(event)
            if event == "back" then
                GameHelper.dispatchSceneEvent( GameHelper.EVENT_NAME_BACK_TO_EXIT )
            end
        end)
        scene:addChild(scene.layer)
        scene.onEnter = function(self)
            -- avoid unmeant back
            self:performWithDelay(function()
                self.layer:setKeypadEnabled(true)
            end, 0.5)
        end
    end

    EventProtocol.extend(scene)
    GameHelper.CUR_SCENE = scene
end
--[[ 

取得当前运行的scene

**Returns:**

-   return 当前运行的scene

--]]
function GameHelper.getCurScene()
    return GameHelper.CUR_SCENE
end
--[[ 

清除当前运行的scene

--]]
function GameHelper.cleanCurScene()
    GameHelper.CUR_SCENE = nil
end
--[[ 

返回到登陆界面

--]]
function GameHelper.returnToLogin()
    GameHelper.dispatchSceneEvent( GameHelper.EVENT_NAME_RETURN_TO_LOGIN )
end
--[[ 

分发Scene事件

**Parameters:**

-   eventName:        string         事件名称
-   eventParams:      table          事件传递的参数

--]]
function GameHelper.dispatchSceneEvent( eventName, eventParams )
    if GameHelper.CUR_SCENE and isstring(eventName) then 
        if not eventParams then eventParams = {} end
        eventParams.name = eventName
        GameHelper.CUR_SCENE:dispatchEvent(eventParams)
    end
end
--[[ 

结束游戏

--]]
function GameHelper.exit()
    CCDirector:sharedDirector():endToLua()
    if isios() then 
        os.exit()  -- 这个看来不是很有用,在android上时会导致关闭很慢很慢的很慢,cow
    end
end

return GameHelper