local M = class( "UpdateState" )

function M:ctor( file, pw )
	self.file = file
	self.pw = pw
	self.state = nil
	self:load()
end

function M:load()
	local jsonContent = io.readfile(self.file)
	if jsonContent then 
		self.state = json.decode(jsonContent)
	end
	if table.isnotempty(self.state) then 
		return true
	else
		self.state = {}
		return false
	end
end

function M:save()
	if table.isnotempty(self.state) then 
		local jsonContent = json.encode(self.state)
		if jsonContent then 
			return io.writefile(self.file, jsonContent, "w")
		end
	end
	return false
end

function M:add( states )
	util.walk(function ( t )
		table.insert(self.state, t)
	end, states)
	self:save()
end

function M:update()
	if table.isnotempty(self.state) then 
		for k, v in pairs(self.state) do
			if not v.over then 
				for i, file in pairs(v.files) do
					if not file.downloaded then 
						file.downloaded = true
						if #v.files == i then 
							v.over = true
							while #self.state > 1 and self.state[1].over do 
								table.remove(self.state, 1)
							end
						end
						self.save()
						return true
					end
				end
			end
		end
	end
	return false
end

function M:getVersion()
	if table.isnotempty(self.state) then 
		return self.state[#self.state].ver
	else
		return nil
	end
end

function M:getNext()
	if table.isnotempty(self.state) then 
		for k, v in pairs(self.state) do
			if not v.over then 
				for i, file in pairs(v.files) do
					if not file.downloaded then 
						return clone(file)
					end
				end
			end
		end
	end
	return nil
end

function M:calcTotalSize()
	local size = 0
	util.walk(function ( verTable )
		if verTable and not verTable.over then 
			util.walk(function ( file )
				if not file.downloaded then 
					size = size + file.size
				end
			end, verTable.files)
		end
	end, self.state)
	return size
end

return M