--NetworkUtil

local M = {}

M.TAG = "[tandyfw.net]"
--[==========[ HTTP REQUEST ]==========]
--[[

HTTP网络请求,基本不用这个方法的,用下面的get和post吧
实在不行再用这个,还不行就用network.createHTTPRequest

** Parameters: **

- 	url: 				string 					请求url
- 	method: 			string 					请示方式, 有"GET"和"POST"
- 	successCallback: 	fun(event,response) 	成功回调
- 	failCallback: 		fun(event,code) 		失败回调,code有值则是有返回结果,但不成功,无值则是请求错误
- 	params: 			table 					其它参数,如下:
							return_data 		bool 			是否返回data而非string,默认为false
							timeout     		float 			超时时间,单位为s,默认为10s
							header 				string 		 	请求头部
							progressCallback 	fun  			进度请求回调

** Returns: **

- return request http请求对象

--]]
function M.request( url, method, successCallback, failCallback, params )
	params = params or {}
	local function callback_( event )
		if not failCallback then failCallback = successCallback end
		local ok = (event.name == "completed")
		if ok then 
			local request = event.request
			local code = request:getResponseStatusCode()
			if code == 200 then
				local response 
				if params.return_data == true then 
					response = request:getResponseDataLua()
				else
					response = request:getResponseString()
				end
				if isfunction(successCallback) then 
					successCallback(event, response)
				end
			else
				Log.d(M.TAG, "net error, error code = %d, error msg = %s", 
					request:getErrorCode(), request:getErrorMessage())
				if isfunction(failCallback) then 
					failCallback(event, code)
				end
			end
		else
			Log.w(M.TAG, "net status error " .. event.name)
			if isfunction(failCallback) then 
				failCallback(event)
			end
		end
	end
	local request = network.createHTTPRequest(callback_, url, method)
	if isnumber(params.timeout) then 
		request:setTimeout(params.timeout)
	end
	if isstring(params.header) then 
		request:addRequestHeader(params.header)
	end
	if isfunction(params.progressCallback) and request.setProgressCallback then 
		request:setProgressCallback(params.progressCallback)
	end
	return request
end
--[[

HTTP GET 方式请求

** Parameters: **

- 	url: 				string 					请求url
- 	successCallback: 	fun(event,response) 	成功回调
- 	failCallback: 		fun(event,code) 		失败回调,code有值则是有返回结果,但不成功,无值则是请求错误
- 	params: 			table 					其它参数,如下:
							return_data 		bool 			是否返回data而非string,默认为false
							timeout     		float 			超时时间,单位为s,默认为10s
							header 				string 		 	请求头部
							progressCallback 	fun  			进度请求回调

** Returns: **

- return request http请求对象

--]]
function M.get( url, successCallback, failCallback, params )
	local request = M.request(url, "GET", successCallback, failCallback, params)
	request:start()
	return request
end
--[[

HTTP POST 方式请求

** Parameters: **

- 	url: 				string 					请求url
- 	posts: 				table(kv) 				post参数,key value 对
- 	successCallback: 	fun(event,response) 	成功回调
- 	failCallback: 		fun(event,code) 		失败回调,code有值则是有返回结果,但不成功,无值则是请求错误
- 	params: 			table 					其它参数,如下:
							return_data 		bool 			是否返回data而非string,默认为false
							timeout     		float 			超时时间,单位为s,默认为10s
							header 				string 		 	请求头部
							progressCallback 	fun  			进度请求回调

** Returns: **

- return request http请求对象

--]]
function M.post( url, posts, successCallback, failCallback, params )
	local request = M.request(url, "POST", successCallback, failCallback, params)
	if istable(posts) then 
		for key, value in pairs(posts) do
			request:addPOSTValue(key, value)
		end
	elseif isstring(posts) then 
		request:setPOSTData(posts)
	end
	request:start()
	return request
end
--[[

HTTP GET 方式请求下载

** Parameters: **

- 	url: 				string 							请求url
- 	successCallback: 	fun(url, filePath, fileName) 	成功回调
- 	failCallback: 		fun(url, filePath, fileName) 	失败回调,code有值则是有返回结果,但不成功,无值则是请求错误
- 	params: 			table 							其它参数,如下:
							relativePath 		string 			下载的documentsPath下相对路径
							unzip 				bool 			是否进行解压操作,只针对以.zip结果的,默认为false
							unzipPath 			string 			解压路径,只有unzip为true时启效,默认为documentsPath
							deleteZip 			bool 			是否删除zip文件,只有unzip为true时启效
							timeout     		float 			超时时间,单位为s,默认为10s
							header 				string 		 	请求头部
							progressCallback 	fun  			进度请求回调

** Returns: **

- return request http请求对象

--]]
function M.download( url, successCallback, failCallback, params )
	params = params or {}
	params.return_data = true
	-- 失败回调
	local function failCallback_( event, code )
		Log.w(M.TAG, "download fail:" .. url)
		if failCallback then failCallback(url) end
	end
	-- 成功回调
	local function successCallback_( event, response )
		-- 成功下载啦,response就是request:getResponseDataLua()
		local fileName = io.pathinfo(url).filename
		local relativePath = params.relativePath or ""
		local filePath = device.documentsPath .. relativePath .. fileName
		-- 创建下文件夹,来写入最后的文件
		io.mkdir(filePath)
		if io.writefile(filePath, response, "wb") then
			Log.i(M.TAG, "download successfully:" .. filePath)
			-- 写入成功后,看看要不要解压,以及要不要自动删除
			if params.unzip == true and string.endwith(url, "%.zip", true) then 
				local unzipRet = util.unzip(filePath, params.unzipPath or device.documentsPath)
				if unzipRet then 
					-- 解压成功
					Log.i(M.TAG, "unzip successfully")
					if params.deleteZip == true then 
						-- 真要删除了
						os.remove(filePath)
						Log.i(M.TAG, "delete zip successfully")
					end
				else
					-- 解压失败
					Log.w(M.TAG, "unzip fail")
					os.remove(filePath)
					if failCallback then failCallback(url, filePath, fileName) end
					return
				end
			end
			if successCallback then successCallback(url, filePath, fileName) end
		else 
			Log.w(M.TAG, "download writefile fail:" .. url)
			os.remove(filePath)
			if failCallback then failCallback(url, filePath, fileName) end
		end
	end
	-- 从这里开始啊
	local request = M.get( url, successCallback_, failCallback_, params)
	return request
end

--[==========[ SOCKET REQUEST ]==========]
--TO BE CONTINUE...


return M




















