local M = {}
local TAG = "[ViewFactory]"

M.viewMatchTable = {}

M.viewMatchTable[WIDGET_TYPE.TTFLabel]                = ui.newTTFLabel
M.viewMatchTable[WIDGET_TYPE.BMFontLabel]             = ui.newBMFontLabel
M.viewMatchTable[WIDGET_TYPE.TTFLabelWithShadow]      = ui.newTTFLabelWithShadow
M.viewMatchTable[WIDGET_TYPE.TTFLabelWithOutline]     = ui.newTTFLabelWithOutline

M.viewMatchTable[WIDGET_TYPE.Button]                  = widget.newButton
M.viewMatchTable[WIDGET_TYPE.Tab]                     = widget.newTab
M.viewMatchTable[WIDGET_TYPE.TabContainer]            = widget.newTabContainer
M.viewMatchTable[WIDGET_TYPE.Table]                   = widget.newTable
M.viewMatchTable[WIDGET_TYPE.Grid]                    = widget.newGrid
M.viewMatchTable[WIDGET_TYPE.Toast]                   = widget.newToast
M.viewMatchTable[WIDGET_TYPE.Scale9]                  = widget.newScale9
M.viewMatchTable[WIDGET_TYPE.DropDownBox]             = widget.newDropDownBox
M.viewMatchTable[WIDGET_TYPE.ScrollLayer]             = widget.newScrollLayer
M.viewMatchTable[WIDGET_TYPE.ScrollView]              = widget.newScrollView
M.viewMatchTable[WIDGET_TYPE.ScrollLinear]            = widget.newScrollLinear
M.viewMatchTable[WIDGET_TYPE.ScrollGrid]           	  = widget.newScrollGrid
M.viewMatchTable[WIDGET_TYPE.PageGrid]           	  = widget.newPageGrid
--M.viewMatchTable[WIDGET_TYPE.ExpandLayer]             = widget.newExpandLayer
M.viewMatchTable[WIDGET_TYPE.ExpandCell]              = widget.newExpandcell
M.viewMatchTable[WIDGET_TYPE.LinearLayer]             = widget.newLinearLayer
M.viewMatchTable[WIDGET_TYPE.Layer]                   = widget.newLayer
M.viewMatchTable[WIDGET_TYPE.Node]                    = widget.newNode
M.viewMatchTable[WIDGET_TYPE.Sprite]                  = widget.newSprite
-- M.viewMatchTable[WIDGET_TYPE.TileSprite]              = widget.newTileSprite
M.viewMatchTable[WIDGET_TYPE.EditBox] 				  = widget.newEditBox
M.viewMatchTable[WIDGET_TYPE.ProgressBar] 			  = widget.newProgressBar
M.viewMatchTable[WIDGET_TYPE.HelpMaskLayer] 		  = widget.newHelpMaskLayer

M.viewMatchTable[WIDGET_TYPE.CountDownView]           = views.newCountDownView
M.viewMatchTable[WIDGET_TYPE.PageTurningView]         = views.newPageTurningView
M.viewMatchTable[WIDGET_TYPE.PageContainer]           = views.newPageContainer
-- M.viewMatchTable[WIDGET_TYPE.ShopView]                = views.newShop
-- M.viewMatchTable[WIDGET_TYPE.MapPageStateView]        = views.newPageState
M.viewMatchTable[WIDGET_TYPE.TableView]               = views.newTableView
M.viewMatchTable[WIDGET_TYPE.ExpandScrollLinearLayer] = views.newExpandScrollLinearLayer
-- M.viewMatchTable[WIDGET_TYPE.TopLeftButtons] 		  = views.newTopLeftButtons
-- M.viewMatchTable[WIDGET_TYPE.ShopsMapView] 		 	  = views.newShopsMap

M.viewMatchTable[WIDGET_TYPE.QuickButton]             = quickui.createButton
M.viewMatchTable[WIDGET_TYPE.QuickCheckBox]           = quickui.createCheckBox
M.viewMatchTable[WIDGET_TYPE.QuickDropDownBox]        = quickui.createDropDownBox
M.viewMatchTable[WIDGET_TYPE.QuickItemSprite]         = quickui.createItemSprite
M.viewMatchTable[WIDGET_TYPE.QuickTableButton]        = quickui.createTableButton


--[[ 

## name

**Parameters:**

-   params: 数组包含
            [必填项]
            string 			viewType			view的类型名称,参看ViewFactory
			table 			viewParams 			具体实例view时用的参数,请参看对应view的参数说明
            
--]]
function M.create( params )
	local _view
--[=[ ## local params --]=]
	--#必填项#
	local _viewType 		= params.viewType 
	local _viewParams 		= params
	--#选填项#
	
	--#内部参数#
--[=[ ## local fuction --]=]
	local function createView( viewType, viewParams, ... )
		local view
		--display类中定义的控件
		if M.viewMatchTable[viewType] then
			local fun = M.viewMatchTable[viewType]
			view = fun(viewParams, ...)
		end
		return view
	end

	local function init( )
		--检查必填项
		Log.a(_viewType, TAG, "create() invalid params.viewType")
		Log.a(_viewParams, TAG, "create() invalid params.viewParams")
		--使用定义的参数
		_view = createView(_viewType, _viewParams)
		Log.a(_view, TAG, "create(): 404... What is " .. _viewType)
	end
--[=[ ## public fuction --]=]
	
	init()
	return _view
end

return M