--[[ 

## 创建带多状态的Button,包括9宫图及一般Button,可带文字
##      1.传入的参数params中不包含rect和capInsets时为一般Button,否则为9宫图

**Parameters:**

-   params: 数组包含
            [必填项]
             String             normalImage     常态图片
             [选填项]
             string             sound           点击事件触发时播放的声音
             CCSize             contentSize     Button尺寸,当为9宫图时此为必填项
             int                x                   x坐标
             int                y                   y坐标
             bool               ignoreAnchor        是否忽略锚点(默认false)
             String             selectImage     选中图片
             String             disableImage    不可选图片
             function(button)   listener        事件方法
             enum               state           按钮的状态:   
                                        Botton.STATE_BUTTON_NORMAL (default)
                                        Botton.STATE_BUTTON_SELECT
                                        Botton.STATE_BUTTON_DISABLE  
             CCRect             rect            9宫图区域        其与capInsets构成是否是9宫的判定
             CCRect             capInsets       9宫图拉伸区域
             CCNode             container       所属容器,不必是直接父容器,只是作为位置判定看是否启用,若位置不在指定容器内则无效,一般情况下不进行设置
             CCNode             triggerContainer 触发按键的容器,若指定,点击在指定容器范围内都会触发按键的点击,要求是按键本身也在指定容器范围内
             bool               cancelOnMove    是否启用移动后取消其可点击模式,默认为false,在TableView这样的控件中通常设置为true,并同时设置swallowsTouches为false
             bool               swallowsTouches 是否启用touch拦截,默认为true
             table              labelParams     字体属性,若无需文字则不用设置,数组包含
                                    [必填项]
                                    string      text                显示的文字
                                    [选填项]
                                    string      font                字体, 默认为Arial
                                    int         size                字体大小, 默认为24
                                    ccc3/ccc4   color               文字颜色, 默认为白色
                                    enum        textAlign           文字水平对齐方式
                                                        ui.TEXT_ALIGN_LEFT (default)
                                                        ui.TEXT_ALIGN_CENTER
                                                        ui.TEXT_ALIGN_RIGHT
                                    enum        textValign          文字竖直对齐方式
                                                        ui.TEXT_VALIGN_TOP
                                                        ui.TEXT_VALIGN_CENTER (default)
                                                        ui.TEXT_VALIGN_BOTTOM
                                    int         x                   x坐标
                                    int         y                   y坐标
                                    CCSize      dimensions          尺寸                                           

--]]
local Botton = class( "Botton", function ()
    return display.newLayer()
end )

Botton.STATE_BUTTON_NORMAL  = 0
Botton.STATE_BUTTON_SELECT  = 1
Botton.STATE_BUTTON_DISABLE = 2

Botton.TAG = "widget.Botton"

function Botton:ctor( params )
    -- 初始参数
    self.params = params
    self.params.state = params.state or Botton.STATE_BUTTON_NORMAL
    self.params.enable = not params.locked
    self.params.scale = params.scale or 1
    self.params.isSelected = params.isSelected or false
    self.params.cancelOnMove = params.cancelOnMove or false
    self.params.swallowsTouches = ifnil( params.swallowsTouches, true )
    self.params.priority = params.priority or -128
    self._type_ = WIDGET_TYPE.Button 
    self.params_ = {}
    self.views_ = {}
    -- 验证参数
    Log.a( self.params.normalImage, Botton.TAG, "ctor() invalid params.normalImage" )
    if self.params.rect or self.params.capInsets then 
        Log.a( self.params.contentSize, Botton.TAG, "ctor() invalid params.contentSize" )
        if not self.params.rect then self.params.rect = CCRectZero end
        if not self.params.capInsets then self.params.capInsets = CCRectZero end
    end
    -- 使用定义的参数
    widget.initView( self, self.params )

    self:setState_( self.params.state, true )
    if self.params.checkMode then
        if self.params.isSelected then
            self:setState_( Botton.STATE_BUTTON_SELECT )
        else
            self:setState_( Botton.STATE_BUTTON_NORMAL )
        end
    end
    -- 加下文本
    if self.params.labelParams then 
        if self.params.labelParams.shadow then
            self.views_.label  = widget.newTTFLabelWithShadow( self.params.labelParams )
        else
            self.views_.label  = ui.newTTFLabel( self.params.labelParams )
        end
        util.alignNodeWithPadding( self.views_.label, self:getContentSize(), display.CENTER, self.params.labelParams.padding )
        self:addChild( self.views_.label, 5 )
    end

    -- 处理下touch事件
    --[[ 
    
    执行缩放
    只有设置过scale才会有效

    **Parameters:**
    
    -   scale:        float     数值

    --]]
    local function doScale( scale )
        if self.params.scale ~= 1 then 
            self:setScale( scale )
        end
    end
    --[[ 

    取得Layer尺寸

    **Returns:**

    -   包含Layer尺寸的矩形

    --]]

    local function getRect()
        if not self.params_.buttonRect then
            local layer = self.params.triggerContainer or self
            self.params_.buttonRect =  CCRectMake( 0, 0,
                                layer:getContentSize().width, 
                                layer:getContentSize().height )
        end
        return self.params_.buttonRect 
    end
    --[[ 

    判断内部点击

    **Parameters:** 

    -   x: 点击坐标x

    -   y: 点击坐标x

    **Returns:**

    -   判断结果bool

    --]]
    local function isTouchInside( x, y )
        local touchLocation = ccp( x, y )
        if self.params.container then
            local loc = self.params.container:convertToNodeSpace( touchLocation )
            local rect = self.params.container:boundingBox()
            if not rect:containsPoint( loc ) then
                return false
            end
        end
        if self.params.triggerContainer then
            touchLocation = self.params.triggerContainer:convertToNodeSpace( touchLocation )
        else
            touchLocation = self:convertToNodeSpace( touchLocation )
        end
        local r = getRect()
        return r:containsPoint( touchLocation )
    end
    local function onTouchCancelled( x, y )
        self.params_.beganX, self.params_.beganY = 0, 0
        self.params_.ismoved = nil
    end
    
    local function onTouchBegan( x, y )
        if not util.isCustomTouchEnabled( self ) then
            return false
        end
        if not isTouchInside( x, y ) then return false end
        if self.params.cancelOnMove then
            self.params_.beganX, self.params_.beganY = x, y
        end
        doScale( self.params.scale )
        if not self.params.checkMode then
            self:setState_( Botton.STATE_BUTTON_SELECT )
        end
        return true
    end

    local function onTouchMoved( x, y )
        if isTouchInside( x, y ) and not self.params_.ismoved then 
            doScale( self.params.scale )
            if not self.params.checkMode then
                self:setState_( Botton.STATE_BUTTON_SELECT )
            end
        else
            doScale( 1 )
            if not self.params.checkMode then
                self:setState_( Botton.STATE_BUTTON_NORMAL )
            end
        end
        if self.params.cancelOnMove then
            if self.params_.ismoved ~= true and ( math.abs( x - self.params_.beganX ) > 5 
                or math.abs( y - self.params_.beganY ) > 5 ) then
                self.params_.ismoved = true
            end
        end
    end

    local function onTouchEnded( x, y )
        doScale( 1 )
        if isTouchInside( x, y ) and not self.params_.ismoved then 
            if self.params.checkMode then
                if self.params.isSelected then
                    self:setState_( Botton.STATE_BUTTON_NORMAL )
                else
                    self:setState_( Botton.STATE_BUTTON_SELECT )
                end
                self.params.isSelected = not self.params.isSelected
            else
                self:setState_( Botton.STATE_BUTTON_NORMAL )
            end
            if self.params.listener then
                if self.params.sound then 
                    -- audio.stopAllEffects()
                    audio.playSound( self.params.sound )
                end
                self.params.listener( self )
            end
        else
            if not self.params.checkMode then
                self:setState_( Botton.STATE_BUTTON_NORMAL )
            end
        end
        onTouchCancelled( x, y )
    end

    function onTouch( eventType, x, y )
        if self.params.enable and self:isVisible() then
            if eventType == widget.TOUCHBEGAN then
                return onTouchBegan( x, y )
            elseif eventType == widget.TOUCHMOVED then
                onTouchMoved( x, y )
            elseif eventType == widget.TOUCHENDED then
                onTouchEnded( x, y )
            else 
                onTouchCancelled( x, y )
            end
        end    
    end

    self:setTouchEnabled( true )
    self:addTouchEventListener( onTouch, false, self.params.priority, self.params.swallowsTouches )
    self:ignoreAnchorPointForPosition( false )
end
--[[ 

设置图片，会刷新一次

--]]
function Botton:setImage_( fileName )
    if self.views_.image then 
        self.views_.image:removeFromParentAndCleanup( true )
    end
    if not self.params.rect and not self.params.capInsets then 
        -- sprite
        self.views_.image = display.newSprite( fileName )
        self.views_.image:ignoreAnchorPointForPosition( true )
    else
        -- 9宫
        self.views_.image = M.newScale9{
                            file = fileName,
                            rect = self.params.rect,
                            capInsets = self.params.capInsets,                                     
                            contentSize = self.params.contentSize,
                        }
        self.views_.image:ignoreAnchorPointForPosition( true )
    end
    if not self.params.contentSize then 
        self.params.contentSize = self.views_.image:getContentSize() 
        self:setContentSize( self.params.contentSize )
    end
    self:addChild( self.views_.image, 0 )
end
--[[ 

更改Button状态

**Parameters:**

-   state: 状态

-   force: 强制刷新仅init()使用

--]]
function Botton:setState_( state, force )
    if self.params.state ~= state or force or not self.params.state then 
        if self.params.state then
            self.params.state = state
        end
        if state == Botton.STATE_BUTTON_DISABLE and self.params.disableImage then
            self:setImage_( self.params.disableImage ) 
        elseif state == Botton.STATE_BUTTON_SELECT and self.params.selectImage then
            self:setImage_( self.params.selectImage )
        elseif state == Botton.STATE_BUTTON_NORMAL or force then
            self:setImage_( self.params.normalImage )
        end
    end
end

--[[ 

更改文字

**Parameters:**

-   text: 文本信息

--]]

function Botton:setTextString( string )
    if string and self.views_.label then 
        self.views_.label:setString( string )
        -- 擦不顶用.
        util.alignNodeWithPadding( self.views_.label, self:getContentSize(), self.params.labelParams.padding )
    end
end

--[[ 

更改文字

**Parameters:**

-   text: 文本信息

--]]
function Botton:getTextString()
    if self.views_.label then 
        return self.views_.label:getString()
    end
end

--[[ 

设置不可选状态

**Parameters:** 

-   isDisable: bool

--]]
function Botton:setStateDisabled( isDisable )
    if self.params.enable == isDisable then
        self.params.enable = not isDisable
        self:setState_( isDisable and Botton.STATE_BUTTON_DISABLE or Botton.STATE_BUTTON_NORMAL )
    end
end

--[[ 

锁定按钮

**Parameters:** 

-   islocked: bool

--]]
function Botton:locked( islocked )
    self.params.enable = not islocked
end

--[[ 
    
设置缩放尺寸

**Parameters:**

-   scale:        float     比例

--]]
function Botton.setChangeScale( scale )
    self.params.scale = scale
end
--[[ 

设置响应事件

**Parameters:** 

-   listener: funtion事件

--]]

function Botton:setListener( listener )
    self.params.listener = listener
end
--[[ 

取得响应事件

**Returns:**

-   return 响应事件

--]]
function Botton:getListener()
    return self.params.listener
end

--[[ 

设置文字位置

**Parameters:** 

-   x: position x

-   y: position y

--]]
function Botton:setTextPosition( x, y )
    if self.views_.label then
        self.views_.label:setPosition( x, y )
    end
end

--[[ 

取得按钮状态

**Returns:**

-   返回按钮状态三种: Botton.STATE_BUTTON_NORMAL, Botton.STATE_BUTTON_SELECT, Botton.STATE_BUTTON_DISABLE 

--]]
function Botton:getState( )
    return self.params.state
end
function Botton:isSelected( )
    return self.params.isSelected
end
function Botton:setSelected( select )
    self.params.isSelected = select
    if self.params.isSelected then
        self:setState_( Botton.STATE_BUTTON_SELECT )
    else
        self:setState_( Botton.STATE_BUTTON_NORMAL )
    end
    if not self.params.checkMode then
        self.params.enable = not select
    end
end
--[[ 

设置点击触发的容器,即点击此容器即可触发button

**Parameters:**

-   triggerContainer:        CCNode      触发容器

--]]
function Botton:setTriggerContainer( triggerContainer )
    self.params.triggerContainer = triggerContainer
end

return Botton





























