local M = {}

local TAG = "[ui.views.CountDownView]"

M.STATE_RUNNING		= 1
M.STATE_STOP		= 2
M.STATE_PAUSE		= 3

--[[ 

## 创建倒计时器

**Parameters:**

-   params: 数组包含
			[必填项]
			[选填项]
			CCSize		contentSize 		控件的尺寸
			ini 		time 	 			倒计时的时间,单位为s
			string 		format 				显示时间的格式,可用的如下:[hh],[mm],[ss],[h],[m],[s]
			table 		labelParams 		文本参数
				[必填项]
				string 		text				显示的文字
				[选填项]
				string 		font 				字体, 默认为Arial
				int 		size 				字体大小, 默认为24
				ccc3/ccc4	color				文字颜色, 默认为白色
				enum		textAlign			文字水平对齐方式
									ui.TEXT_ALIGN_LEFT (default)
									ui.TEXT_ALIGN_CENTER
									ui.TEXT_ALIGN_RIGHT
				enum		textValign			文字竖直对齐方式
									ui.TEXT_VALIGN_TOP
									ui.TEXT_VALIGN_CENTER (default)
									ui.TEXT_VALIGN_BOTTOM
				int 		x 					x坐标
				int 		y 					y坐标
				CCSize		dimensions 			尺寸
			int 		x 					x坐标
			int 		y 					y坐标
			bool        ignoreAnchor        是否忽略锚点(默认false)
			function 	listener 	 		时间到回调事件
			

--]]
function M.new( params )
--[[ ## local params --]]
	local _layer 		= display.newLayer()
	--#必填项#
	--#选填项#
	local _time			= params.time
	local _contentSize		= params.contentSize
	local _labelParams	= params.labelParams or {}
	local _x			= params.x
	local _y 			= params.y
	local _listener		= params.listener
	local _format 		= params.format or "[hh]:[mm]:[ss]"
	--#内部参数#
	local _state		= M.STATE_STOP
	local _curTime		= _time
	local _label
	local _ostime
--[[ ## local fuction --]]

    --[[ 

    创建时间字符串

    **Parameters:**

    -   time:    int 	倒计时的时间,单位为s

    **Returns:**

    -   时间对应的字符串

    --]]	
	local  function createTimeString(time)
		local str = _format
		time = time or 0
		local d, h, m, s = 0, 0, 0, 0

		if string.find(str, "%[d%]") then
			s = time % 60
			m = math.floor(time / 60) % 60
			h = math.floor(time / 3600) % 24
			d = math.floor(time / 3600 / 24)
			str = string.gsub(str, "%[hh%]", string.format("%02d", h))
			str = string.gsub(str, "%[mm%]", string.format("%02d", m))
			str = string.gsub(str, "%[ss%]", string.format("%02d", s))
			-- str = string.gsub(str, "%[d%]", string.format("%d", d))

			str = string.gsub(str, "%[h%]", h)
			str = string.gsub(str, "%[m%]", m)
			str = string.gsub(str, "%[s%]", s)
			str = string.gsub(str, "%[d%]", d)
		elseif string.find(str, "%[hh%]") or string.find(str, "%[h%]") then 
			s = time % 60
			m = math.floor(time / 60) % 60
			h = math.floor(time / 3600)
			str = string.gsub(str, "%[hh%]", string.format("%02d", h))
			str = string.gsub(str, "%[mm%]", string.format("%02d", m))
			str = string.gsub(str, "%[ss%]", string.format("%02d", s))

			str = string.gsub(str, "%[h%]", h)
			str = string.gsub(str, "%[m%]", m)
			str = string.gsub(str, "%[s%]", s)
		elseif string.find(str, "%[mm%]") or string.find(str, "%[m%]") then
			s = time % 60
			m = math.floor(time / 60)
			str = string.gsub(str, "%[mm%]", string.format("%02d", m))
			str = string.gsub(str, "%[ss%]", string.format("%02d", s))

			str = string.gsub(str, "%[m%]", m)
			str = string.gsub(str, "%[s%]", s)
		else
			s = time
			str = string.gsub(str, "%[ss%]", string.format("%02d", s))
			str = string.gsub(str, "%[s%]", s)
		end

		return str
	end

    --[[ 

    显示时间字符串

    **Parameters:**

    -   time:    int 	倒计时的时间,单位为s

    --]]
	local function showTime(time)
		local str = createTimeString(time)
		_label:setString(str)
	end

    --[[ 

    单步更新

    **Parameters:**

    -   dt:    int 		间隔时间,单位为s

    --]]
	local function step(dt)
		if not _ostime then return end 
		local curOstime = os.time()
		_curTime = _curTime - (curOstime - _ostime)
		_ostime = curOstime
		-- _curTime = _curTime - 1
		if _curTime <= 0 then 
			_curTime = 0 
			_layer:stop()
			if _listener then
				_listener(_layer)
			end
		end
		showTime(_curTime)
	end

	local function init()
		-- util.checkParams(TAG, "init()", {
		-- 		{ _contentSize, "contentSize" },
		-- 		{ _time, "time" },
		-- 		{ _labelParams, "labelParams" },
		-- 	})
        widget.initView(_layer, params)
		_layer:ignoreAnchorPointForPosition(false)
		--_layer:setContentSize(_contentSize)
		if not _labelParams.text then 
			_labelParams.text = createTimeString(0)
		end
		_label = ui.newTTFLabel(_labelParams)
		if not _contentSize then 
			--print(_label:getContentSize().width, _label:getContentSize().height)
			_layer:setContentSize(_label:getContentSize())
		end
		_label:setPosition(_layer:getContentSize().width / 2, _layer:getContentSize().height / 2)
        _layer:addChild(_label, 10)
		showTime(_curTime)
	end
--[[ ## public fuction --]]

    --[[ 

    开启倒计时

    **Parameters:**

    -   time:    int 	倒计时的时间,单位为s

    **Returns:**

    -   是否正常开启

    --]]
	function _layer:start(time)
		Log.a(time or _time, TAG, "start() must be set a time")
		if _state == M.STATE_STOP then
			if time then
				_curTime = time
			else
				_curTime = _time
			end
			_ostime = os.time()
			self:scheduleUpdate(step, 1)
			_state = M.STATE_RUNNING
			return true
		end
		return false
	end

    --[[ 

    停止倒计时

    **Returns:**

    -   是否正常停止

    --]]
	function _layer:stop( )
		if _state == M.STATE_RUNNING or _state == M.STATE_PAUSE then
			if _state == M.STATE_RUNNING then
				self:unscheduleUpdate()
				_ostime = nil
			end
			_state = M.STATE_STOP
			showTime(0)
			return true
		end
		return false
	end

    --[[ 

    暂停倒计时

    **Returns:**

    -   是否正常暂停

    --]]
	function _layer:pause()
		if _state == M.STATE_RUNNING then
			self:unscheduleUpdate()
			_state = M.STATE_PAUSE
			_ostime = nil
			return true
		end
		return false
	end

    --[[ 

    恢复倒计时

    **Returns:**

    -   是否正常恢复

    --]]
	function _layer:resume()
		if _state == M.STATE_PAUSE then
			_ostime = os.time()
			self:scheduleUpdate(step, 1)
			_state = M.STATE_RUNNING
		end
	end

	function _layer:modifyTime( time )
		_curTime = time
	end
	--[[ 

    恢复倒计时

    **Returns:**

    -   是否正常恢复

    --]]
    function _layer:setFormat( string )
		_format = string
	end

	init()
	_layer._type_ = WIDGET_TYPE.CountDownView
	return _layer
end

return M