local M = {}
local TAG = "[ui.views.ExpandScrollLinearLayer]"

--[[ 

## 创建ScrollLinear

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize                              contentSize             		尺寸
            fun(expandCell, cellTitle)      	titleNodeCreateListener         标题node创建监听
            fun(expandCell, cellItem)       	itemNodeCreateListener          内容项node创建监听
            [选填项]
            fun(scrollLinear, index, item)      tapListener             		点击cell回调事件(这是属于ScrollLinear的)
            fun(node, index, item)          	cellClickListener               点击监听(这是属于ExpandCell的)
            fun(scrollLinear)                   onBeforeRefresh         		刷新前处理函数
            fun(scrollLinear)                   onAfterRefresh          		刷新后处理函数
            table                               cellSelectedParams          	选中效果(同9宫图参数)
                                                    string      file        背景图片名
                                                    CCRect      rect        图片尺寸矩形
                                                    int         offset      与其容器的间隔值
            table                               items                   		这还要解释!!!
            bool                                bounceable              		是否启用边界阻尼效果,默认true
            bool                                clippingToBounds        		是否启用界面截取效果,默认true
            enum                                direction               		方向
                                                     widget.TABLEVIEW_DIRECTION_NONE
                                                     widget.TABLEVIEW_DIRECTION_HORIZONTAL
                                                     widget.TABLEVIEW_DIRECTION_VERTICAL 
                                                     widget.TABLEVIEW_DIRECTION_BOTH(default)
            int                                 x                       		x坐标
            int                                 y                       		y坐标
            int                                 tag                     		node的tag值
            bool                                ignoreAnchor            		是否忽略锚点(默认false)
--]]
function M.newExpandScrollLinearLayer ( params )
--[=[ ## local params --]=]
	--#必填项#
	local _titleNodeCreateListener	= params.titleNodeCreateListener
	local _itemNodeCreateListener 	= params.itemNodeCreateListener
	local _items 					= params.items
	--#选填项#
	local _cellClickListener		= params.cellClickListener
	local _cellSelectedParams 		= params.cellSelectedParams
	
	--#内部参数#
	local _layer
	local selectNoneEffect_

	local function cellAtIndexListener_( scrollLinear, index, item )
		local function expandListener_( expandCell )
			expandCell:getParentNode():refreshContentSizeWithHoldContentOffset()
		end

		local function cellClickListener_( node, index, item )
			--若有用背景,则全部设置为无选择中
			if _cellSelectedParams then 
				selectNoneEffect_()
			end
			--若有设置点击事件,则进行调用
			if _cellClickListener then 
				_cellClickListener( node, index, item )
			end
		end

		local cell = widget.newExpandCell{
					contentSize             = scrollLinear:getContentSize(),
					expandListener          = expandListener_,
					parentNode              = scrollLinear,
					title                   = item:getTitle(),
					items                   = item:getItems(),
					clickListener           = cellClickListener_,
					titleNodeCreateListener = _titleNodeCreateListener,
					itemNodeCreateListener  = _itemNodeCreateListener,
					cellSelectedParams		= _cellSelectedParams,
				}
		return cell	
	end

	params.cellAtIndexListener = cellAtIndexListener_
	_layer = widget.newScrollLinear(params)

	function selectNoneEffect_()
		local ccarray = _layer:getLinearLayer():getChildren()
		for i = 0, ccarray:count() - 1 do
			local obj = ccarray:objectAtIndex(i)
			obj:setSelectEffect()
		end
	end

	if _items then 
		_layer:setItems(_items)
	end

--[=[ ## public fuction --]=]

	_layer._type_ = WIDGET_TYPE.ExpandScrollLinearLayer 
	return _layer
end

return M