local M = {}

M.ACTION_FIRST		= 1
M.ACTION_PREV 		= 2
M.ACTION_NEXT 		= 3
M.ACTION_LAST 		= 4
M.ACTION_JUMP 		= 5
M.ACTION_CURRENT 	= 6

--[[ 

## 翻页控件

**Parameters:**

-   params: 数组包含
            [必填项]
            int 			totalPage			总页数
            function		listener 			翻页监听,参数为
            									1:self
            									2:动作类型
            										M.ACTION_FIRST
													M.ACTION_PREV
													M.ACTION_NEXT
													M.ACTION_LAST
													M.ACTION_JUMP
            [选填项]
            bool 			simpleAction 		简单动作,只有上下页,默认为false
            int 			padding 			大小跳之间的间隔,默认为0
            table 			labelParams 		字体属性,默认有一套配置
								以下是TTFLabel的字体属性说明
								[必填项]
								string 		text				显示的文字
								[选填项]
								string 		font 				字体, 默认为Arial
								int 		size 				字体大小, 默认为24
								ccc3/ccc4	color				文字颜色, 默认为白色
								enum		textAlign			文字水平对齐方式
												ui.TEXT_ALIGN_LEFT (default)
												ui.TEXT_ALIGN_CENTER
												ui.TEXT_ALIGN_RIGHT
								enum		textValign			文字竖直对齐方式
												ui.TEXT_VALIGN_TOP
												ui.TEXT_VALIGN_CENTER (default)
												ui.TEXT_VALIGN_BOTTOM
								int 		x 					x坐标
								int 		y 					y坐标
								CCSize		dimensions 			尺寸
            int         	x                   x坐标
            int         	y                   y坐标
            bool			ignoreAnchor		是否忽略锚点(默认false)
--]]
function M.new( params )
	local _layer					= LinearLayer:create()
--[=[ ## local params --]=]
	--#必填项#
	local _totalPage				= params.totalPage
	local _listener 				= params.listener
	--#选填项#
	local _simpleAction 			= params.simpleAction or false
	local _padding 					= params.padding or 0
	local _labelParams 				= params.labelParams
	local _x 						= params.x
	local _y 						= params.y
	local _ignoreAnchor				= params.ignoreAnchor or false
	
	--#内部参数#
	local _curPage 					= 1
	local _pageLabel

--[=[ ## local fuction --]=]
	--[[ 
	
	组合图片名
	
	**Parameters:**
	
	-   image:		string 		图片类型:first,prev,next,last

	- 	state:		string 		图片状态:n,s,d	
	
	**Returns:**
	
	-   图片名
	
	--]]
	local function ipath(image, state)
		return string.format("arrow_%s_%s.png", image, state)
	end
	--[[ 
	
	启用按键
	
	**Parameters:**
	
	-   enable:         bool    是否启用

	-	...:			table	控件的按键的tag值数组
	
	--]]
	local function enableButton( enabled, ...  )
		for i,tag in ipairs(...) do
			_layer:getChildByTag(tag).setStateDisabled(not enabled)
		end
	end
	--[[ 
	
	改变按键状态
	
	--]]
	local function changeState( )
		if _simpleAction then
			enableButton(_curPage ~= 1, { M.ACTION_PREV})
			enableButton(_curPage ~= _totalPage, { M.ACTION_NEXT})
		else
			enableButton(_curPage ~= 1, { M.ACTION_FIRST, M.ACTION_PREV})
			enableButton(_curPage ~= _totalPage, { M.ACTION_NEXT, M.ACTION_LAST})
		end
		_pageLabel:setString(_curPage .. "/" .. _totalPage)
	end
	--[[ 
	
	点击事件
	
	**Parameters:**
	
	-   button:        button     点击的按键

	--]]
	local function clickbutton( button )
		local action = button and button:getTag() or 0
		if action == M.ACTION_FIRST then
			--到首页
			_curPage = 1
		elseif action == M.ACTION_PREV then
			--上一页
			_curPage = _curPage - 1
		elseif action == M.ACTION_NEXT then
			--下一页
			_curPage = _curPage + 1
		elseif action == M.ACTION_LAST then
			--尾页
			_curPage = _totalPage
		else
			echo("[PageTurningView]clickbutton : invalid param button's tag")
			return
		end
		changeState()
		_listener(_layer, action)
	end
	--[[ 
	
	创建按键
	
	**Parameters:**
	
	-   buttonType:        string     按键类型:first,prev,next,last
	
	**Returns:**
	
	-   return
	
	--]]
	local function createButton( buttonType )
		local button = widget.newButton{
					normalImage		= ipath(buttonType, "n"),
					selectImage 	= ipath(buttonType, "s"),
					disableImage 	= ipath(buttonType, "d"),
					listener		= clickbutton,
					}
		return button
	end

	local function init( )
		--检查必填项
		assert(_totalPage and _totalPage > 0, "[richer.ui.views.PageTurningView] init() invalid params.totalPage")
		assert(_listener, "[richer.ui.views.PageTurningView] init() invalid params.listener")
		--使用定义的参数
		_layer:ignoreAnchorPointForPosition(_ignoreAnchor)
		_layer:alignItemsHorizontally(0, HORIZONTAL_CENTER, false)
		--添加所有元素
		local button
		if not _simpleAction then
			--首页
			button = createButton("first")
			button:setTag(M.ACTION_FIRST)
			_layer:addChild(button)
			--占位
			if _padding > 0 then
				local empty = display.newGroup()
				empty:setContentSize(CCSize(_padding, button:getContentSize().height))
				_layer:addChild(empty)
			end	
		end
		--上一页
		button = createButton("prev")
		button:setTag(M.ACTION_PREV)
		_layer:addChild(button)
		--中间显示区
		local textSprite = display.newSprite("arrow_text.png")
		_layer:addChild(textSprite)
		local labelX, labelY = centerNode(textSprite)
		if not _labelParams then
			_labelParams = {
							text 		= 1 .. "/" .. _totalPage,
							font 		= 18,
							textAlign 	= ui.TEXT_ALIGN_CENTER,
							x 			= labelX,
							y 			= labelY,
							}
		end
		_pageLabel = ui.newTTFLabel(_labelParams)
		_pageLabel:align(display.CENTER, labelX, labelY)
		textSprite:addChild(_pageLabel)
		--下一页
		button = createButton("next")
		button:setTag(M.ACTION_NEXT)
		_layer:addChild(button)
		if not _simpleAction then
			--尾页
			button = createButton("last")
			button:setTag(M.ACTION_LAST)
			--占位
			if _padding > 0 then
				local empty = display.newGroup()
				empty:setContentSize(CCSize(_padding, button:getContentSize().height))
				_layer:addChild(empty)
			end

			_layer:addChild(button)
		end
		--初始显示状态
		changeState()

		if _x and _y then
			_layer:setPosition(_x, _y)
		end
	end
--[=[ ## public fuction --]=]
	--[[ 
	
	设置总页数	
	
	**Parameters:**
	
	-   page:        int     总页数
	
	**Returns:**
	
	-   return
	
	--]]
	function _layer:setTotalPage( page )
		_totalPage = page
	end

	function _layer:getTotalPage(  )
		return _totalPage
	end

	function _layer:getCurPage( )
		return _curPage
	end

	function _layer:jump2Page( page )
		assert(0 < page and page <= _totalPage
			, "[richer.ui.views.PageTurningView] jump2Page(page) invalid params page")
		_curPage = page
		changeState()
		_listener(_layer, M.ACTION_JUMP)
	end

	init()
	_layer._type_ = WIDGET_TYPE.PageTurningView
	return _layer
end

return M







