local  M = {}

local TAG = "[ui.views.PageView]"
--[[ 

## 创建带padding上下块容器

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize			尺寸
			[选填项]
            int 		bottomViewHeight 	下部容器高度(有了这个可以自动计算,就不要mainViewParams和bottomViewParams了)
            table 		mainViewParams 		主容器属性
            				具体包括:
							[必填项]
							CCSize 		contentSize 		尺寸
							[选填项]
							ccc4 		color 				颜色,用于debug
							enum 		anchorPoint 		锚点位置,参看display使用说明
							int 		x 					x坐标
							int 		y 					y坐标
							bool 		ignoreAnchor 		是否忽略锚点(默认true)
			table 		bottomViewParams 	下部容器属性, 同上述mainViewParams
            talbe 		padding 			与容器的内部间隔,属性如下
            				left 		int 		与左边的间隔(default:10)
            				bottom 		int 		与底边的间隔(default:10)
            				right 		int 		与右边的间隔(default:10)
            				top 		int 		与顶边的间隔(default:10)
            				middle 		int 		中间间隔(default:5)
			table 		bgParams 			背景属性
							具体包括:[必填项]
				            CCSize      contentSize            尺寸
				            CCRect      rect                   9宫图区域
				            [选填项]
				            int         x                   x坐标
				            int         y                   y坐标
				            CCRect      capInsets           9宫图拉伸区域
				            bool        ignoreAnchor        是否忽略锚点(默认false)
			ccc4 		color 				颜色,用于debug
            enum 		anchorPoint 		锚点,参看display说明
            int         x                   x坐标
            int         y                   y坐标
            bool		ignoreAnchor		是否忽略锚点(默认true)
--]]
function M.newContainer( params )
--[=[ ## local params --]=]
	--#必填项#
	local _contentSize				= params.contentSize
	--#选填项#
	local _bottomViewHeight 		= params.bottomViewHeight or 0
	local _mainViewParams 			= params.mainViewParams or {}
	local _bottomViewParams 		= params.bottomViewParams or {}
	local _padding 					= params.padding or {}
	local _bgParams 				= params.bgParams
	local _color 					= params.color
	local _anchorPoint 				= params.anchorPoint or display.CENTER
	local _x 						= params.x
	local _y 						= params.y
	local _ignoreAnchor				= params.ignoreAnchor or false
	
	--#内部参数#
	local _layer					= _color and CCLayerColor:create(_color) or display.newLayer()
	local _containerLayer
	local _mainView
	local _bottomView
--[=[ ## local fuction --]=]
	--[[ 
	
	创建容器内部view	
	
	**Parameters:**
	
	-   viewParams:        table 			view的属性,具体包括:
								[必填项]
								CCSize 		contentSize 		尺寸
								[选填项]
								ccc4 		color 				颜色,用于debug
								enum 		anchorPoint 		锚点位置,参看display使用说明
								int 		x 					x坐标
								int 		y 					y坐标
								bool 		ignoreAnchor 		是否忽略锚点(默认true)
	
	**Returns:**
	
	-   return 容器内部view
	
	--]]
	local function createView( viewParams )
		local view 
		if viewParams.color then
			view = display.newColorLayer(viewParams.color) --CCLayerColor:create(viewParams.color)
		else
			view = display.newLayer()
		end
		view:ignoreAnchorPointForPosition(viewParams.ignoreAnchor == nil and true or viewParams.ignoreAnchor)
		if viewParams.contentSize then
			view:setContentSize(viewParams.contentSize)
		end
		if viewParams.x and viewParams.y then
			display.align(view, viewParams.anchorPoint or display.BOTTOM_LEFT, viewParams.x, viewParams.y)
		end
		return view
	end

	local function init( )
		--检查必填项
		Log.a(_contentSize, TAG, " newContainer() invalid params.contentSize")
		--使用定义的参数
		_layer:ignoreAnchorPointForPosition(_ignoreAnchor)
		_layer:setContentSize(_contentSize)
		--容器初始
		_containerLayer = display.newLayer()
		_containerLayer:ignoreAnchorPointForPosition(false)
		local w = _contentSize.width - (_padding.left or 10) - (_padding.right or 10)
		local h = _contentSize.height - (_padding.top or 10) - (_padding.bottom or 10)
		_containerLayer:setContentSize(CCSize(w, h))
		_containerLayer:setPosition(util.centerNode(_layer))
		_layer:addChild(_containerLayer)
		--创建MainView
		if not _bottomViewHeight or _bottomViewHeight <= 0 then
			_bottomViewHeight = 0
			_padding.middle = 0
		end
		if not _mainViewParams.contentSize then
			_mainViewParams.contentSize = CCSize(w, h - _bottomViewHeight - (_padding.middle or 5))
		end
		_mainViewParams.x = _mainViewParams.x or 0
		_mainViewParams.y = _mainViewParams.y or (_bottomViewHeight + (_padding.middle or 5))
		_mainView = createView(_mainViewParams)
		_containerLayer:addChild(_mainView)
		--创建BottomView
		if _bottomViewHeight > 0 then 
			if not _bottomViewParams.contentSize then
				_bottomViewParams.contentSize = CCSize(w, _bottomViewHeight)
			end
			-- _bottomViewParams.x = _bottomViewParams.x or 0
			-- _bottomViewParams.y = _bottomViewParams.y or 0
			_bottomView = createView(_bottomViewParams)
			_containerLayer:addChild(_bottomView)
		end
		--创建背景
		if _bgParams then
			local bg = widget.newScale9(_bgParams)
			bg:setContentSize(_contentSize)
			bg:setPosition(util.centerNode(_layer))
			_layer:addChild(bg, -1);
		end
		if _anchorPoint then
			display.align(_layer, _anchorPoint, _x or 0, _y or 0)
		end
	end
--[=[ ## public fuction --]=]
	
	function _layer:getMainView( )
		return _mainView
	end

	function _layer:getBottomView(  )
		return _bottomView
	end

	init()
	return _layer
end

--[[ 

## 带分页滚动容器

**Parameters:**

-   params: 数组包含
            [必填项]
            table 		containerParams 		容器属性
							[必填项]
				            CCSize      contentSize			尺寸
				            int 		bottomViewHeight 	下部容器高度(有了这个可以自动计算,就不要mainViewParams和bottomViewParams了)
							[选填项]
				            table 		mainViewParams 		主容器属性
				            				具体包括:
											[必填项]
											CCSize 		contentSize 		尺寸
											[选填项]
											ccc4 		color 				颜色,用于debug
											enum 		anchorPoint 		锚点位置,参看display使用说明
											int 		x 					x坐标
											int 		y 					y坐标
											bool 		ignoreAnchor 		是否忽略锚点(默认true)
							table 		bottomViewParams 	下部容器属性, 同上述mainViewParams
				            talbe 		padding 			与容器的内部间隔,属性如下
				            				left 		int 		与左边的间隔(default:10)
				            				bottom 		int 		与底边的间隔(default:10)
				            				right 		int 		与右边的间隔(default:10)
				            				top 		int 		与顶边的间隔(default:10)
				            				middle 		int 		中间间隔(default:5)
							table 		bgParams 			背景属性
											具体包括:[必填项]
								            CCSize      contentSize            尺寸
								            CCRect      rect                   9宫图区域
								            [选填项]
								            int         x                   x坐标
								            int         y                   y坐标
								            CCRect      capInsets           9宫图拉伸区域
								            bool        ignoreAnchor        是否忽略锚点(默认false)
							ccc4 		color 				颜色,用于debug
				            enum 		anchorPoint 		锚点,参看display说明
				            int         x                   x坐标
				            int         y                   y坐标
				            bool		ignoreAnchor		是否忽略锚点(默认true)
            table 		contentViewParams 			table属性
            				[必填项]
				            string 			viewType			view的类型名称,参看ViewFactory
				            其它同对应view的属性信息,具体实例view时用的参数,请参看对应view的参数说明,无contentSize时默认为main容器大小 
			[选填项]       
            table 		pageParams 				分页控件属性
							[必填项]
				            int 			totalPage			总页数
				            function		listener 			翻页监听,参数为
				            									1:self
				            									2:动作类型
				            										PageTurningView.ACTION_FIRST
																	PageTurningView.ACTION_PREV
																	PageTurningView.ACTION_NEXT
																	PageTurningView.ACTION_LAST
																	PageTurningView.ACTION_JUMP
				            [选填项]
				            bool 			simpleAction 		简单动作,只有上下页,默认为false
				            int 			padding 			大小跳之间的间隔,默认为0
				            table 			labelParams 		字体属性,默认有一套配置
												以下是TTFLabel的字体属性说明
												[必填项]
												string 		text				显示的文字
												[选填项]
												string 		font 				字体, 默认为Arial
												int 		size 				字体大小, 默认为24
												ccc3/ccc4	color				文字颜色, 默认为白色
												enum		textAlign			文字水平对齐方式
																ui.TEXT_ALIGN_LEFT (default)
																ui.TEXT_ALIGN_CENTER
																ui.TEXT_ALIGN_RIGHT
												enum		textValign			文字竖直对齐方式
																ui.TEXT_VALIGN_TOP
																ui.TEXT_VALIGN_CENTER (default)
																ui.TEXT_VALIGN_BOTTOM
												int 		x 					x坐标
												int 		y 					y坐标
												CCSize		dimensions 			尺寸
							enum 			anchor 				分页控件在容器中的布局方式,定义了这个后x,y,ignoreAnchor三个无效
												display.CENTER 
												display.CENTER_LEFT
												display.CENTER_RIGHT
				            int         	x                   x坐标
				            int         	y                   y坐标
				            bool			ignoreAnchor		是否忽略锚点(默认false)
							
			function 	pageTurnCallback 		翻页回调事件,返回参数如下
												1.pageView
												2.tableView
												3.action 包括
													PageTurningView.ACTION_FIRST
													PageTurningView.ACTION_PREV
													PageTurningView.ACTION_NEXT
													PageTurningView.ACTION_LAST
													PageTurningView.ACTION_JUMP
--]]
function M.newPageContainer( params )
--[=[ ## local params --]=]
	--#必填项#
	local _containerParams 			= params.containerParams
	local _contentViewParams 		= params.contentViewParams
	--#选填项#
	local _pageParams 				= params.pageParams
	local _pageTurnCallback 		= params.pageTurnCallback
	
	--#内部参数#
	local _contentView 
	local _pageView
	Log.a(_containerParams, TAG, "newPageContainer() invalid params.containerParams")
	if not _containerParams.ignoreAnchor then
		_containerParams.ignoreAnchor = true
	end
	local _layer					= M.newContainer(_containerParams)
--[=[ ## local fuction --]=]
	local function pageTurnListener( pageView, action )
		if _pageParams.listenerOld then
			_pageParams.listenerOld(pageView, action)
		end
		if _pageTurnCallback then
			_pageTurnCallback(pageView, _contentView, action)
		end
	end
	local function init( )
		--检查必填项
		-- assert(_contentViewParams, "[richer.ui.views.PageView] newPageContainer() invalid params.contentViewParams")
		-- assert(_pageParams, "[richer.ui.views.PageView] newPageContainer() invalid params.pageParams")
		-- assert(_pageTurnCallback, "[richer.ui.views.PageView] newPageContainer() invalid params.pageTurnCallback")
		util.checkParams(TAG, "newPageContainer()",{
				{ _contentViewParams, "contentViewParams" },
			})
		--使用定义的参数
		--创建View
		if not _contentViewParams.contentSize then
			_contentViewParams.contentSize = _layer:getMainView():getContentSize()--_containerParams.mainViewParams.contentSize
		end
		if not _contentViewParams.ignoreAnchor then
			_contentViewParams.ignoreAnchor = true
		end
		_contentView = ViewFactory.create(_contentViewParams)
		_contentView:setTag(1)
		_layer:getMainView():addChild(_contentView)
		if _pageParams then
			--创建翻页控件
			if _pageParams.listener then _pageParams.listenerOld = _pageParams.listener end
			_pageParams.listener = pageTurnListener
			_pageView = views.newPageTurningView(_pageParams) --PageTurningView.new(_pageParams)
			_pageView:setTag(2)
			_layer:getBottomView():addChild(_pageView)
			--设置翻页控件位置
			if _pageParams.anchor then
				local cx, cy = util.centerNode(_layer:getBottomView())
				if _pageParams.anchor == display.CENTER then
					display.align(_pageView, display.CENTER, cx, cy)
				elseif _pageParams.anchor == display.CENTER_LEFT then
					display.align(_pageView, display.CENTER_LEFT, 0, cy)
				elseif _pageParams.anchor == display.CENTER_RIGHT then
					display.align(_pageView, display.CENTER_RIGHT, cx * 2, cy)
				end
			end
			--为pageview追加取得view的方法
			_pageView.getContentView =  _layer:getContentView()
			--初始请求第一批数据
			if _pageTurnCallback and not _contentViewParams.items then 
				_pageTurnCallback( _pageView, _contentView, PageTurningView.ACTION_FIRST )
			end
		end
	end
--[=[ ## public fuction --]=]
	
	function _layer:getContentView( )
		return _contentView
	end

	function _layer:getPageView(  )
		return _pageView
	end
	--[[ 
	
	刷新当前页
	
	--]]
	function _layer:refreshCurrentContentView()
		_pageTurnCallback( _pageView, _contentView, PageTurningView.ACTION_CURRENT )
	end

	function _layer:setTotalPage( page )
		_pageView:setTotalPage(page)
	end

	function _layer:getTotalPage(  )
		return _pageView:getTotalPage()
	end

	function _layer:getCurPage( )
		return _pageView:getCurPage()
	end

	init()
	_layer._type_ = WIDGET_TYPE.PageContainer
	return _layer
end


return M