local M = {}
--[[ 

## name

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize			尺寸
            int 		numOfCells 			cell个数
            table 		thParams 			标题行th的相关参数,具体如下
            				int 		height 			行高
            				table 		bgColor 		背景色
            				table 		ttfParams 		标题字体样式
            				table 		columns 		列信息参数,具体如下
            								float 		width 		列宽,小于1当百分比处理,大于1当px处理
            								string 		title 		标题文字,无则不用定义
            table 		trParams 			信息行tr的相关参数,具体如下
            				int 		height 			行高
            				table 		bgColors 		背景色数组,将循环显示指定色
            				table 		columns 		列显示参数,具体如下
            								enum 		anchor 			锚点,默认为display.CENTER,其它值参看display
            								table 		padding 		间距,分别有top,bottom,left,right,默认均为0
            								table 		params 			要放置的view的基本参数,采用ViewFactory进行创建
            												string 		viewType 		创建的view的类型
            												bool 		useCheckBox 	单条记录是否使用checkbox,默认为true
            												bool 		useContainer 	是否使用容器限制,button和checkbox在滚动情况下要设置此为true,默认为false
				            								CCNode 		node 			无敌乱入,具体放置的node,有最高的放置优先级,一旦设定,其它的都不再理会,
				            															这个通常放在linear情况下由的itemCreateListener进行生成时使用
            												...			...				具体要创建的view的参数
            [选填项]
            fun(checkBox)			checkBoxClickListener 	checkBox点击事件
            fun(tableView, cell)	tabListener 			点击cell事件
            table       selectedParams      选中效果(同9宫图参数)
                            string      file        背景图片名
                            CCRect      rect        图片尺寸矩形
                            int         offset      与其容器的间隔值
            table 		items 				行属性数组,两种形态:
            									1.具体的元素属性table,将与trParams.columns.params进行整合;
            									2.entity,将做为参数调用itemCreateListener从而生成1中的属性table.
            fun(item, index)	itemCreateListener 	行属性创建监听,将由items中的项为参数创建相应的属性table,若items为形态1则不用设置
            bool 		useCheckBox 		是否使用checkbox,默认为false
            bool 		useLinearLayer 		是否使用linear,默认为false,有按键定义时设置为true
            bool        bounceable          是否启用边界阻尼效果,默认true
            int         x                   x坐标
            int         y                   y坐标
            bool		ignoreAnchor		是否忽略锚点(默认false)
--]]
function M.newTableView ( params )
	local TAG 						= "[ui.views.TableView]"
	local _layer					= display.newLayer()
--[=[ ## local params --]=]
	--#必填项#
	local _contentSize				= params.contentSize
	local _numOfCells 				= params.numOfCells
	local _useLinearLayer 			= params.useLinearLayer or false
	local _thParams					= params.thParams  --标题栏参数,包括显示的文字,宽度
										--height
										--ttfParams
										--bgColor
										--columns = {width, title/ttfParams, }
	local _trParams 				= params.trParams  --值对应的参数,包括第一列存放的类型,位置,但其宽度由thParams决定
										--height  这个是每个tr的统一高度,当然可以被下面的items里的给换掉
										--bgColors = {...}
										--columns = {params,padding,anchor}
	local _items 					= params.items  --具体的值就在这里面了
										--bgColor
	--#选填项#
    local _bounceable               = ifnil(params.bounceable, true)
	local _checkBoxClickListener 	= params.checkBoxClickListener
	local _tapListener 				= params.tapListener
	local _selectedParams 			= params.selectedParams 
	local _itemCreateListener 		= params.itemCreateListener
	local _useCheckBox 				= params.useCheckBox or false
	local _x 						= params.x
	local _y 						= params.y
	local _tag 						= params.tag
	-- local _ignoreAnchor				= params.ignoreAnchor or false
	local _checkBoxWidth 			= iff(_useCheckBox, params.checkBoxWidth or 30 , nil)
	
	--#内部参数#
	local _checkBoxTable
	local _table
	local _topCheckBox

--[=[ ## local fuction --]=]
	local init
	local createLineLayer
	local createTR
	local createTH
	local onCheckBoxChanged
	local tapListener_
	local onBeforeRefresh_
	local foreachCheckBox
	local onCheckBoxAllChanged
	local cellAtIndexListener_

	-- local function isClickCheckbox( index, point )
	-- 	if _useCheckBox and index > 0 and point then
	-- 		local checkBox = _checkBoxTable[index]
	-- 		if checkBox then
	-- 			if checkBox:isVisible() then
	-- 				local touchPoint = checkBox:convertToNodeSpace(point)
	-- 				local rect = CCRectMake(0, 0, checkBox:boundingBox().size.width, checkBox:boundingBox().size.height)
	-- 				return rectContainsPoint( rect, touchPoint ) 
	-- 			end
	-- 		end
	-- 	end
	-- 	return false
	-- end

	function onCheckBoxChanged( checkBox )
		_table:setSelectEffect(checkBox.item_index, true)
		if _checkBoxClickListener then
			_checkBoxClickListener(checkBox)
		end
	end

	function tapListener_( tableView, cell, index )
		if _tapListener then
			_tapListener(tableView, cell, index)
		end
	end
	function onBeforeRefresh_( tableView )
		if _useCheckBox then
			_checkBoxTable = {}
		end
	end

	function onCheckBoxAllChanged( checkBox )
		foreachCheckBox(checkBox.isSelected())
	end

	function cellAtIndexListener_( tableView, idx )
		local cell = tableView:dequeueCell()
        if not cell then 
            cell = tableView.newCell() 
        else
            tableView.removeCellAll(cell)
        end
        
        local layer = createTR(tableView, idx, tableView:itemAt(idx))
        layer:ignoreAnchorPointForPosition(true)
        layer:setTag(idx)
        cell:addChild(layer)
        return cell
	end

	function foreachCheckBox( checked )
		local result = iff(isboolean(checked), nil, {})
		for i, checkBox in ipairs(_checkBoxTable) do
			if checkBox:isVisible() then
				if isboolean(checked) then
					if checked ~= checkBox.isSelected() then
						checkBox.setSelected( checked )
					end
				elseif isnil(checked) then
					if checkBox.isSelected() then
						table.insert(result, checkBox.item_index)
					end
				end
			end
		end
		if isnil(checked) then
			return result
		end
	end

	function createTR( tableView, index, item )
		local thParams, trParams = _thParams, _trParams
		local itemParams
		if not item then
			item = tableView:itemAt(index)
		end
		if _itemCreateListener then
			itemParams = _itemCreateListener(item, index)
		else
			itemParams = item
		end
		--背景色切换
		local color = itemParams.bgColor
		if not color and trParams.bgColors and istable(trParams.bgColors) and #trParams.bgColors > 0 then
			color = trParams.bgColors[index % #trParams.bgColors] or trParams.bgColors[#trParams.bgColors]
		end

		local layer = display.newColorLayer(color)
        layer:ignoreAnchorPointForPosition(false)
		local w, h = _contentSize.width, trParams.height
		local nextX = 0
		layer:setContentSize(CCSize(w, h))
		--有没用checkbox啊,有用的话还得要处理下
		local checkBox
		if _useCheckBox then
			nextX = _checkBoxWidth
			w = w - nextX
			local node = display.newGroup() --ColorLayer(ccc4(255,0,0,255))
			node:setContentSize(CCSize(nextX, h))
			node:align(display.BOTTOM_LEFT, 0, 0)
			node:setTag(0)
			layer:addChild(node)
			checkBox = ViewFactory.create{--tableView:borrowWidget{
							viewType = WIDGET_TYPE.QuickCheckBox,
							-- viewName = "line_checkbox",
							container = tableView,
							listener = onCheckBoxChanged,
							triggerContainer = node,
						}
			checkBox:setPosition(util.centerNode(node))
			checkBox:setTag(1)
			node:addChild(checkBox)
			util.alignNodeWithPadding(checkBox, node:getContentSize())
			checkBox.item_index = index
			table.insert(_checkBoxTable, checkBox)
		end
		--开始遍历吧
		local singleUseCheckBox = true
		for i,column in ipairs(trParams.columns) do
			local node = display.newGroup()
			local nodeW = thParams.columns[i].width
			--都小于1了,就当是你百分比了
			if nodeW < 1 then nodeW = w * nodeW end
			node:setContentSize(CCSize(nodeW, h))
			node:align(display.BOTTOM_LEFT, nextX, 0)
			--node:setTag( (i - 1) * #trParams.columns + index )
			node:setTag(i)
			layer:addChild(node)--, 0, 100 * (index + 1) + i)
			--该准备文字了,只要有title或ttfParams就算是要显示文字
			local params = {}
			table.merge(params, column.params)
			table.merge(params, itemParams[i])
			local view 
			if params.node then
				view = node
			-- elseif params.borrow then
			-- 	if not params.viewName then params.viewName = params.borrow end
			-- 	if not params.container then params.container = tableView end
			-- 	view = tableView:borrowWidget(params)
			elseif params.viewType then 
				if params.useContainer then
					params.container = tableView
				end
				view = ViewFactory.create(params)
				if params.useContainer then
					view.params = params
				end
			end
			if view then
				node:addChild(view, 0, node:getTag() * 100 + 1)
				util.alignNodeWithPadding(view, node:getContentSize(), column.anchor, column.padding)
			end
			--每一项都可以做为是否使用本条checkbox的依据,但只要有一次是说不使用则就不使用
			if singleUseCheckBox and params.useCheckBox == false then
				singleUseCheckBox = false
			end

			nextX = nextX + nodeW
		end
		--设置是否显示checkBox
		if _useCheckBox and singleUseCheckBox == false then
			checkBox:setVisible(false)
		end
		-- layer:setTag(index)
		return layer
	end

	function createTH(  )
		local thParams = _thParams
		local layer = display.newColorLayer(thParams.bgColor)
		local w, h = _contentSize.width, thParams.height
		local nextX = 0
		layer:setContentSize(CCSize(w, h))
		layer:ignoreAnchorPointForPosition(false)
		--有没用checkbox啊,有用的话还得要处理下
		if _useCheckBox then
			nextX = _checkBoxWidth 
			local node = display.newGroup() --ColorLayer(ccc4(255,0,0,255))
			node:setContentSize(CCSize(nextX, h))
			node:align(display.BOTTOM_LEFT, 0, 0)
			node:setTag(0)
			layer:addChild(node)
			w = w - nextX
			local checkBox = quickui.createCheckBox{
				listener = onCheckBoxAllChanged,
				triggerContainer = node,
			}
			checkBox:setPosition(util.centerNode(node))
			checkBox:setTag(1)
			node:addChild(checkBox)
			util.alignNodeWithPadding(checkBox, node:getContentSize())
			_topCheckBox = checkBox
		end
		--开始遍历吧
		for i,column in ipairs(thParams.columns) do
			local node = display.newGroup()
			local nodeW = column.width
			--都小于1了,就当是你百分比了
			if nodeW < 1 then nodeW = w * nodeW end
			node:setContentSize(CCSize(nodeW, h))
			node:align(display.BOTTOM_LEFT, nextX, 0)
			node:setTag(i)
			layer:addChild(node)
			--该准备文字了,只要有title或ttfParams就算是要显示文字
			if column.title or column.ttfParams then
				local ttfParams = column.ttfParams
				if not ttfParams then
					--貌似没定义ttf样式的话就要用公用的了,要是连公用的都没有就默认吧
					if thParams.ttfParams then
						ttfParams = thParams.ttfParams
						ttfParams.text = column.title
					else
						ttfParams = {text = column.title}
					end
				end
				local title = ui.newTTFLabel(ttfParams)
				local nodeCX, nodeCY = util.centerNode(node)
				display.align(title, display.CENTER, nodeCX, nodeCY)
				node:addChild(title, 0, node:getTag() * 100 + 1)
			end
			nextX = nextX + nodeW
		end
		layer:setTag(0)
		return layer
	end

	function init( )
		--检查必填项
		-- Log.a(_contentSize, "[richer.ui.views.TableView]", "newTableView() invalid params.contentSize")
		-- Log.a(_numOfCells, "[richer.ui.views.TableView]", "newTableView() invalid params.numOfCells")
		util.checkParams(TAG, "newTableView()", {
				{ _contentSize, "contentSize" },
				{ _numOfCells, "numOfCells" },
			})
		--使用定义的参数
		params.ignoreAnchor = true
		initView(_layer, params)
		--开始构建
		--放入标题栏
		local titleLayer = createTH()
		util.alignNode(titleLayer, _layer:getContentSize(), display.TOP_LEFT, 0)
		_layer:addChild(titleLayer, 0, 1)
		--放入Table
		_table = widget.newTable{
						contentSize = CCSizeAdd(_contentSize, 0, -titleLayer:getContentSize().height),
						cellSize = CCSize(_contentSize.width, _trParams.height),
						numOfCells = _numOfCells,
						cellAtIndexListener = iff(_useLinearLayer, nil, cellAtIndexListener_),
						useLinearLayer = _useLinearLayer,
						linearCellAtIndexListener = iff(_useLinearLayer, createTR, nil),
						ignoreAnchor = true,
						items = _items,
						tapListener = tapListener_,
						selectedParams = _selectedParams,
						onBeforeRefresh = onBeforeRefresh_,
						bounceable = _bounceable,
					}
		_layer:addChild(_table, 0, 2)
		-- if _items then _layer:setTableItems( _items ) end
	end
--[=[ ## public fuction --]=]
	--[[ 
	
	取得所有选中的项的item
	
	-   return 所有选中的项的item
	
	--]]
	function _layer:getSelectedItems(  )
		local result 
		if _useCheckBox then
			local selectedIndexs = foreachCheckBox()
			if selectedIndexs and #selectedIndexs > 0 then
				result = {}
				for i,index in ipairs(selectedIndexs) do
					table.insert(result, _table:itemAt(index))
				end
			end
		end
		return result
	end
	--[[ 
	
	取得具体Table,很多的widget.Table中的方法调用要通过这个
	
	**Returns:**
	
	-   return 具体Table
	
	--]]
	function _layer:getTable(  )
		return _table
	end

	function _layer:setItems( items )
		_table:setItems( items )
	end
	--[[ 
	
	设置选中项效果
	
	**Parameters:**
	
	-   index:        int     选中项,若无可设置为nil或0
	- 	force: 		  bool 	  是否强制设置

	--]]
	function _layer:setSelectEffect( index, force )
		_layer:getTable():setSelectEffect( index, force )
	end
	--[[ 
	
	设置顶部全选checkbox
	
	**Parameters:**
	
	-   selected:       bool     	是否选中
	-	checkAll:		bool 		是否同时改变所有项的checkbok,默认为false
	
	--]]
	function _layer:setTopCheckBoxSelected( selected, checkAll )
		if _useCheckBox then
			_topCheckBox.setSelected(selected)
			if isboolean(checkAll) then
				foreachCheckBox(checkAll)
			end
		end
	end
	--[[ 
	
	取得指定行列的node,可以修改里面的元素,前提是必须采用linear,直接table无法使用
	
	**Parameters:**
	
	-   row:        int     行,从1开始
	-	column:		int 	列,从1开始,若使用了checkbox,则0表示是checkbox
		
	**Returns:**
	
	-   return 指定行列的node
	
	--]]
	function _layer:getLinearTableNode( row, column )
		if _useLinearLayer then
			local cellLinear = _table:getCurCellLinear()
			if cellLinear then
				local layer = cellLinear:getChildByTag(row)
				if layer then
					return layer:getChildByTag(column)
				end
			end
		end
		return nil
	end
	--[[ 
	
	启用/停用指定行上的CheckBox
	
	**Parameters:**
	
	-   row:        int     行
	- 	enable: 	bool 	是否启用,默认为true
	
	--]]
	function _layer:enableCheckBox( row, enable )
		if _useLinearLayer and _useCheckBox then
			local checkBoxNode = self:getLinearTableNode(row, 0)
			if checkBoxNode then
				local checkBox = checkBoxNode:getChildByTag(1)
				if checkBox then
					checkBox:setVisible( ifnil( enable, true ) )
				end
			end
		end
	end

	init()
	_layer._type_ = WIDGET_TYPE.TableView
	return _layer
end


return M











