local M = {}
local TAG = "[ui.views.quickui]"
--[[ 

创建项目中的tab按键

**Parameters:**

-   params:        table     tab button参数,参看widget.newButton参数
					[必填项]
					string 			text 			button上的文字
					[可选项]
					table		 	labelParams 	文字样式
					int  			width 			button宽度
		            enum               state           按钮的状态:   
		                                        widget.STATE_BUTTON_NORMAL (default)
		                                        widget.STATE_BUTTON_SELECT
		                                        widget.STATE_BUTTON_DISABLE  

					table 			target 			目标信息
										int 	id 		目标id
										...随意信息
**Returns:**

-   retrun tab按键

--]]
function M.createFirstLayerTabMenuButton( params )
	local btnParams = params
	btnParams.imageNamePre = "arrowRightBtn"
	btnParams.imageWidth = 123
	btnParams.imageHeight = 39
	return M.createButton(btnParams)
end
--[[ 

创建button

**Parameters:**

-   params:        table     button参数
						[必填项]
						string 		imageNamePre  	图片名前缀
						int 		imageWidth 		图片宽度
						int 		imageHeight 	图片高度	
						[选填项]
						int 		width 			生成的宽度
						strint 		text 			button上的文字
						int 		fontSize 		文字大小
						table 		labelParams 	文字样式
						table 		target 			目标信息
										int 	id 		目标id
										...随意信息

**Returns:**

-   return button

--]]	
function M.createButton( params )
	-- assert(params, "[richer.ui.views.quickui] createButton() invalid params")
	-- assert(params.imageNamePre, "[richer.ui.views.quickui] createButton() invalid params.imageNamePre")
	-- assert(params.imageWidth, "[richer.ui.views.quickui] createButton() invalid params.imageWidth")
	-- assert(params.imageHeight, "[richer.ui.views.quickui] createButton() invalid params.imageHeight")
	util.checkParams(TAG, "createButton()", {
			{params, ""},
			{params.imageNamePre, "imageNamePre"},
			{params.imageWidth, "imageWidth"},
			{params.imageHeight, "imageHeight"},
		})
	local btnParams = params
	if isnil(btnParams.postfix) then
		btnParams.postfix = true
	end
	if not btnParams.normalImage then
		btnParams.normalImage = btnParams.imageNamePre .. iff( btnParams.postfix, "_normal.png", "")
	end
	if not btnParams.selectImage then
		btnParams.selectImage = btnParams.imageNamePre .. iff(btnParams.postfix, "_select.png", "")
	end
	if not btnParams.disableImage then
		btnParams.disableImage = btnParams.imageNamePre .. iff(btnParams.postfix, "_disable.png", "")
	end
	if not btnParams.contentSize then
		btnParams.contentSize = CCSize(params.width or params.imageWidth, params.imageHeight)
	end
	if btnParams.contentSize.width ~= params.imageWidth then
		if not btnParams.rect then
			btnParams.rect = CCRectMake(0, 0, params.imageWidth, params.imageHeight)
		end
		if not btnParams.capInsets then
			btnParams.capInsets = CCRectMake(params.imageWidth / 2 - 15, 0, 30, params.imageHeight)
		end
	end
	if not btnParams.labelParams and params.text then
		btnParams.labelParams = {
			text 	= params.text,
			size 	= params.fontSize or 20,
		}
	end
	local result = widget.newButton(btnParams)
	if params.target then
		result.target = params.target
	end
	return result
end
--[[ 

快速创建CheckBox

**Parameters:**

-   params:        table      参数可参看widget.newCheckBox

**Returns:**

-   return CheckBox

--]]
function M.createCheckBox( params )
	local btnParams = params or {}
	if not btnParams.normalImage then btnParams.normalImage = "btn_check_off.png" end
	if not btnParams.selectImage then btnParams.selectImage = "btn_check_on.png" end
	if not btnParams.contentSize then btnParams.contentSize = CCSize(19, 19) end
	local result = widget.newCheckBox(btnParams)
	if params and params.target then
		result.target = params.target
	end
	return result
end
--[[ 

创建项目Tab

**Parameters:**

-   params:        table     tab参数
					参看widget.newTab
					fn 			buttonCreateFun  	创建button的方法,默认quickui.createButton
					table 		buttons  	按键参数(多个)


**Returns:**

-   return Tab

--]]
function M.createTab( params )
	params.ignoreAnchor = ifnil(params.ignoreAnchor, true)
	if not params.x then params.x = 0 end
	if not params.y then params.y = 0 end
	local tab = widget.newTab(params)
	local fn = params.buttonCreateFun or M.createButton
	for i,v in ipairs(params.buttons) do
		v.tag = ifnil(v.tag, i)
		tab.addButton(fn(v))
	end
	return tab
end


--[[

创建下接框

**Parameters:**

-   params:        table     下接框参数
						[必填项]
			            CCSize      contentSize         尺寸
			            [选填项]
			            table       items               item必须含有text属性,若定义则忽略 itemParams 和 itemTexts
			            table 		itemParams 			item的文字样式,仅在无 items 定义时有效,不定义则默认为{}
			            table 		itemTexts 			item的文字表,仅在无 items 定义时有效且必须定义
			            int         x                   x坐标
			            int         y                   y坐标
			            bool        ignoreAnchor        是否忽略锚点(默认true)
			            string      defaulTitle         默认提示
			            int         selected            选中
			            function    listener            方法包含item
			            string      font                字体
			            int         fontSize            字号
			            int         line                最多显示行数
						fun(button,isOpen)  btnClickListener 按键点击事件
			            table       buttonParams        按钮参数,若下面的 button 未定义则此项必须定义
			            Button      button              按钮,若定义,上面的 buttonParams 将无效
			            最后两项都不设定有默认值

**Returns:**

-   return 下接框

--]]
function M.createDropDownBox( params )
	-- assert(params, "[richer.ui.views.quickui] createDropDownBox() invalid params")
	-- assert(params.contentSize, "[richer.ui.views.quickui] createDropDownBox() invalid params.contentSize")
	util.checkParams(TAG, "createDropDownBox()", {
			{ istable(params), "params is not a table" },
			{ params.contentSize, "contentSize" },
		})
	params.ignoreAnchor = ifnil(params.ignoreAnchor, true)
	if not params.items then
		Log.a(params.itemTexts, TAG, "createDropDownBox() invalid params.itemTexts")
		if not params.itemParams then params.itemParams = {} end
		params.items = {}
		for i,text in ipairs(params.itemTexts) do
			local item  = {}
			table.merge(item, params.itemParams)
			item.text = text
			item.tag = i
			table.insert(params.items, item)
		end
	end
	if not params.button and not params.buttonParams then
		params.button  = quickui.createButton{
	    			imageNamePre = "dropdownbox",
	    			imageWidth   = 28,
	    			imageHeight  = 25,
	    		}
    end
    if params.btnClickListener then
    	params.button.setListener(params.btnClickListener)
    end
    if not params.titleBgParams then
    	params.titleBgParams = {
    				file = "dropdownbox_title_bg.png",
    			}
    end
    if not params.itemsBgParams then
    	params.itemsBgParams = {
    				file = "dropdownbox_content_bg.png",
    			}
    end
    return widget.newDropDownBox(params)
end
--[[ 

快速添加下拉框

**Parameters:**

-   params:        table     参数表
						[必填项]
						CCNode 							node 			父容器,存入下拉框的直接容器
						table 							itemTexts 		项目文字
						[选填项]
						int 							zorder 			zorder
						fun(cell,itemParams)	 		listener 		切换监听
						fun(button,isOpen)				btnClickListener 按键点击事件
						table 							itemParams 		item的文字样式,用于生成TTFLabel
--]]
function M.addDropDownBox(params)
	util.checkParams(TAG, "addDropDownBox()", {
			{ istable(params), "is not a table" },
			{ params.node, "node" },
			{ istable(params.itemTexts), "itemTexts is not a table" },
		})

	local _node = params.node
	local _changedListener = params.listener
	local _itemTexts = params.itemTexts
	local _itemParams = params.itemParams or {size = 20}
	local _btnClickListener = params.btnClickListener
	
	local box = quickui.createDropDownBox{
							contentSize 	= _node:getContentSize(),
							ignoreAnchor 	= true,
							listener 		= _changedListener,
							itemParams 		= _itemParams,
							itemTexts 		= _itemTexts,
							btnClickListener= _btnClickListener
						}
	_node:addChild(box, params.zorder or 0)
	return box
end
--[[ 

添加背景

**Parameters:**

-   node:        CCNode     待添加背景的node
- 	bgParams: 	 table 		9宫背景参数,不设置则使用默认值
- 	padding: 	 table 		背景的间距,默认都为0,right未设定时同left,bottom未设定时同top

--]]
function M.addBackground( node, bgParams, padding )
	if node then
		bgParams = bgParams or {}
		if not bgParams.file then
			bgParams.file =  "subview_content_bg.png"
		end
		local bg = widget.newScale9(bgParams)
		if padding then
			local w, h = node:getContentSize().width, node:getContentSize().height
			bg:setContentSize(CCSize(
				w - (padding.left or 0) - (padding.right or padding.left or 0),
				h - (padding.top or 0) - (padding.bottom or padding.top or 0)
				))
		else
			bg:setContentSize(node:getContentSize())
		end
		
		bg:setPosition(centerNode(node))
		node:addChild(bg, -100, -100);
	end
end
--[[ 

快速添加内容半透明灰色背景

**Parameters:**

-   node:        CCNode     待添加背景的node
- 	padding: 	 table 		背景的间距,默认都为0,right未设定时同left,bottom未设定时同top

--]]
-- function M.addItemBackground( node, padding )
-- 	if node then
-- 		local bgParams = {
-- 				file = "subview_item_bg.png",
-- 			}
-- 		M.addBackground(node, bgParams, padding)
-- 	end
-- end
--[[ 

创建item精灵,有边框,可选文字显示

**Parameters:**

-   params:        table     参数
						[必填项]
						string 		file 			精灵图片名
						[选填项]
						table 		bgParams 		背景参数,无则默认用蓝框
						table 		labelParams 	文字样式,参看TTFLabel
						int/string 	text 			显示的文字
						enum 		anchor 			锚点,参看display,默认为display.TOP_RIGHT 			

**Returns:**

-   return item

--]]
function M.createItemSprite( params )
	util.checkParams(TAG, "createItemSprite", {
			{ istable(params), "is not a table" },
			{ params.file, "file" },
		})
	local _file        = params.file
	local _bgParams    = params.bgParams or {}
	local _labelParams = params.labelParams or {}
	local _text        = params.text
	local _anchor      = params.anchor or display.TOP_RIGHT
	
	local _layer       = widget.newColorLayer()
	local _label

	local function init(  )
		local sprite = display.newSprite(_file)
		_layer:setContentSize(CCSizeAdd(sprite:getContentSize(), 6, 6))
		initView(_layer, params)
		if not _bgParams.file then _bgParams.file = "item_blue_bg.png" end
		M.addBackground(_layer, _bgParams)

		sprite:setPosition(util.centerNode(_layer))
		_layer:addChild(sprite)

		if not _labelParams.text then
			_labelParams.text = ""
		end
		if not _labelParams.color then
			_labelParams.color = ccc3(255, 255, 0)
		end
		if _text then
			_layer:setText(_text, _anchor)
		end
	end 

	function _layer:setText( text, anchor )
		if not _label then
			_label = ui.newTTFLabel(_labelParams)
			util.alignNode(_label, _layer:getContentSize(), anchor)
			_layer:addChild(_label)
		elseif anchor then
			util.alignNode(_label, _layer:getContentSize(), anchor)
		end
		_label:setString(tostring(text))
	end

	init()

	return _layer
end


--[[ 

创建可滚动可翻页的Tab

**Parameters:**

-   params:        table     参数
						[必填项]
						cclayer 		layer 			容器
						function 		createItemView  view填充
						ccsize 			cellSize 		cell尺寸
						table 			createItems 	Items集合
						int 			numOfCells 		cell数量
						bool 			showPage 		是否显示翻页
						[选填项]
						bool 			bounceable 		边缘反弹
						int 			select 			初始选中(默认第一个)		

**Returns:**

-   return table

--]]
function M.addScrollTab( params )
	Log.a(istable(params), TAG, "createScrollTab() params is not a table")
	local _layer 			= params.layer
	local _createItemView 	= params.createItemView
	local _onItemSelect 	= params.onItemSelect
	local _createItems		= params.createItems
	local _showPage 		= params.showPage or false
	local _totalPage 		= params.totalPage
	local _numOfCells 		= params.numOfCells
	local _cellSize 		= params.cellSize
	local _select 			= params.select or 1
	local _bounceable 		= ifnil(params.bounceable, true)

	Log.a(_layer, TAG, "addScrollTab() invalid params.layer")
	Log.a(_createItemView, TAG, "addScrollTab() invalid params.createItemView")
    Log.a(_cellSize, TAG, "addScrollTab() invalid params.cellSize")
    Log.a(_createItems, TAG, "addScrollTab() invalid params.createItems")
    Log.a(_numOfCells, TAG, "addScrollTab() invalid params.numOfCells")
    Log.a(iff(_showPage, isnotnil(_totalPage), true), TAG, "addScrollTab() invalid params.totalPage")
   
    --点击item事件
    local function onItemClick( tableView, cell, index )
    	-- print("index = "..index)
    	Log.d(TAG, "onItemClick%d", index)
    	if _onItemSelect then _onItemSelect( tableView, index ) end
    end

	--翻页回调
    local function itemTablePageTurnCallback( pageView, contentView, action )
        --printf("pageTurnCallback action:%d, curPage:%d, %d", action, pageView:getCurPage(), contentView:getNumberOfCells())
        contentView:setItems(_createItems(pageView:getCurPage()))
        contentView:setSelectEffect(1)
        onItemClick(contentView, 1)
    end

	local table = views.newPageContainer{
		containerParams = {
					contentSize = _layer:getContentSize(),
     				bottomViewHeight = iff(_showPage, 30, 0) ,
				},
		contentViewParams = {
					viewType 	= WIDGET_TYPE.Table,
					cellSize 	= _cellSize, 
					numOfCells 	= _numOfCells,
					tapListener = onItemClick,
					items 		= _createItems(_select),
					useLinearLayer = true,
					linearCellAtIndexListener = _createItemView,
					bounceable = _bounceable,
					selectedParams = {
									file = "item_select.png",
								},
				},
		-- 怪异吧
		pageParams 		= iff(_showPage, {
					totalPage 	= _totalPage,
					simpleAction = true,
					anchor      = display.CENTER,
				}, nil ),
		pageTurnCallback = itemTablePageTurnCallback,
	}
	table:getContentView():setSelectEffect(_select)
	_layer:addChild(table)
	return table
end
--[[ 

创建提示窗口

**Parameters:**

-   params:             table               参数
                            [必填项]
                            text            string  将要提示的文本
                            [选填项]
                            level           int     弹出时处于的层,CBCCenter中的层概念,nil时默认自动加1层
                            okListener      fun     点击ok时的事件
                            cancelListener  fun     点击取消时的事件,nil时只会关闭容器

--]]
-- function M.createHintLayer( params )
--     Log.a(istable(params), TAG, "createHintLayer() params is not a table")
--     Log.a(isstring(params.text), TAG, "createHintLayer() invalid params.text, it must be a string")
--     local CommonHintLayer = require("richer.controller.common.CommonHintLayer")
--     CCBCenter.pushData(MSG_ID.MSG_COMMONHINTLAYER_MODEL, params)
--     CCBCenter.loadCCB(MSG_ID.MSG_COMMONHINTLAYER_MODEL)
-- end
--[[ 

创建输入框
优势:1.默认了背景; 
	2.若layer为全界面容器的直接子项,增加layer值直接设置contentSize,以及直接设置其坐标

**Parameters:**

-   params:        table      参数,参看widget.newEditBox

**Returns:**

-   return 输入框

--]]
-- function M.createEditBox( params )
-- 	if params.layer then 
-- 		params.contentSize = params.layer:getContentSize()
-- 	end
-- 	if not params.normalBgImage then 
-- 		params.normalBgImage = { file = "dropdownbox_title_bg.png", }
-- 	end
-- 	local editbox = widget.newEditBox( params )
-- 	if params.layer then 
-- 		editbox:setPosition(params.layer:getPositionLua()) 
-- 		params.layer:getParent():addChild(editbox)
-- 	end
-- 	return editbox
-- end

-- function M.toast( params )
-- 	if not params.bgParams then 
-- 		params.bgParams = {
-- 			file = "subview_content_bg.png",
-- 		}
-- 	end
-- 	params.padding = params.padding or 20
-- 	params.width = params.width or 600
-- 	params.delayTime = params.delayTime or 3
-- 	widget.newToast( params )
-- end

local _loadingHandler
--[[ 

显示载入界面

**Parameters:**

-   params:        table     参数
						float 			delay 			出现的延迟时间,默认为0
						int 			size 			显示文字的大小,默认为36
						string 			text 			显示的文字内容,默认为"LOADING"

--]]
function M.showLoadingLayer( params )
	params = params or {}
	local text = params.text or "Loading..."
	local function getLoadingView()
		local loading = display.newLayer()
		local loadingSprite = display.newSprite("loading.png")
		loading:addChild(loadingSprite)
		loadingSprite:setScale(0.8)
		local offset = loadingSprite:getContentSize().width * 0.1
		util.alignNodeWithPadding(loadingSprite, loading:getContentSize(), display.CENTER, { top = -25, left = offset})
		local label = ui.newTTFLabel{
				size = params.size or (72 * SCALE_FOR_SPECIAL_LABEL),
				text = text,
			}
		loading:addChild(label)
		util.alignNodeWithPadding(label, loading:getContentSize(), display.CENTER, { top = 50})
		loadingSprite:runAction(CCRepeatForever:create(CCRotateBy:create(0.7, 360)))
		return loading
	end
	local function show()
		_loadingHandler = nil
		--M.hideLoadingLayer(  )
		local maskLayer = display.getRunningScene():getChildByTag(9999) 
		if maskLayer then 
			if maskLayer.LOADING_TEXT == text then
				--cow,连文字都一样,可以滚了
				return 
			else
				--cow,出来都出来了,还改个P字
				maskLayer:removeFromParentAndCleanup(true)
			end
		end
		maskLayer = widget.newMaskLayer()
		local loadingNode = getLoadingView()
		util.alignNode(loadingNode, maskLayer:getContentSize(), display.CENTER, 0)
		maskLayer:addChild(loadingNode)
		maskLayer:setTag(9999)
		maskLayer.LOADING_TEXT = text
		display.getRunningScene():addChild(maskLayer, 9999)
	end
	if _loadingHandler then 
		scheduler.unscheduleGlobal(_loadingHandler) 
		_loadingHandler = nil
	end
	if params.delay and isnumber(params.delay) then 
		_loadingHandler = util.delayCall(show, params.delay)
	else
		show()
	end
end
--[[ 

隐藏载入界面

--]]
function M.hideLoadingLayer(  )
	if _loadingHandler then 
		scheduler.unscheduleGlobal(_loadingHandler) 
		_loadingHandler = nil
	end
	local maskLayer = display.getRunningScene():getChildByTag(9999)
	if maskLayer then 
		maskLayer:removeFromParentAndCleanup(true)
	end
end

--[[ 

## name

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize			尺寸
            
            [选填项]
            int         x                   x坐标
            int         y                   y坐标
            int 		tag 				node的tag值
            bool		ignoreAnchor		是否忽略锚点(默认false)
--]]
function M.toast ( params )
	--具体再说
end

--[[ 

创建Table中使用的Button,允许其可在移动后取消点击功能,默认为透明背景

**Parameters:**

-   params:        table     	参看M.createButton

**Returns:**

-   return return

--]]
function M.createTableButton( params )
	Log.a(istable(params), TAG, "createTableButton() params must be a table")
	params.useContainer = true
	params.cancelOnMove = true
	if isnil(params.swallowsTouches) then params.swallowsTouches = false end
	if not params.imageNamePre then 
		params.imageNamePre = "transparent_bg.png" 
		params.imageWidth = 36
		params.imageHeight = 36
		params.postfix = false
		if not params.width then params.width = 80 end
	end
	return M.createButton(params)
end

return M