--[[

widget控件

button: M.newButton(params)

tab: M.newTab(params)

--]]

local M = {}
M.TAG = "[ui.widget]"

--[[ 

Button状态: STATE_BUTTON_NORMAL   普通
            STATE_BUTTON_SELECT   选中
            STATE_BUTTON_DISABLE  不可选

--]]

M.STATE_BUTTON_NORMAL  = 0
M.STATE_BUTTON_SELECT  = 1
M.STATE_BUTTON_DISABLE = 2

--[[CCTableViewVerticalFillOrder]]
M.TABLEVIEW_FILL_TOP_DOWN           = kCCTableViewFillTopDown
M.TABLEVIEW_FILL_BOTTOM_UP          = kCCTableViewFillBottomUp

--[[CCScrollViewDirection]]
M.TABLEVIEW_DIRECTION_NONE          = kCCScrollViewDirectionNone
M.TABLEVIEW_DIRECTION_HORIZONTAL    = kCCScrollViewDirectionHorizontal
M.TABLEVIEW_DIRECTION_VERTICAL      = kCCScrollViewDirectionVertical
M.TABLEVIEW_DIRECTION_BOTH          = kCCScrollViewDirectionBoth

--[[TOUCH类型]]
M.TOUCHBEGAN = "began" --CCTOUCHBEGAN
M.TOUCHMOVED = "moved" --CCTOUCHMOVED
M.TOUCHENDED = "ended" --CCTOUCHENDED
M.TOUCHCANCELLED = "cancelled" --CCTOUCHCANCELLED

--[[ 

初始view

**Parameters:**

-   view:        CCNode     待初始的node
-   params:     table       初始参数
                contentSize     CCSize
                ignoreAnchor    bool
                x               int
                y               int
                tag             int
                anchorPoint     CCPoint
--]]
function M.initView( view, params )
    if view and params then
        view:ignoreAnchorPointForPosition(params.ignoreAnchor or false)
        if params.contentSize then
            view:setContentSize(params.contentSize)
        end
        if params.x and params.y then
            view:setPosition(params.x, params.y)
        end
        if params.tag then
            view:setTag(params.tag)
        end
        if params.anchorPoint then
            view:setAnchorPoint(params.anchorPoint)
        end
        if params.parentNode then 
            params.parentNode:addChild(view)
        end
    end
end

function M.createBatchNode( batch )
    display.addSpriteFramesWithFile(batch .. ".plist", batch .. ".png")
    return display.newBatchNode( batch .. ".png" )
end
--[[ 

## 创建带多状态的Button,包括9宫图及一般Button,可带文字
##      1.传入的参数params中不包含rect和capInsets时为一般Button,否则为9宫图

**Parameters:**

-   params: 数组包含
            [必填项]
             String             normalImage     常态图片
             [选填项]
             string             sound           点击事件触发时播放的声音
             CCSize             contentSize     Button尺寸,当为9宫图时此为必填项
             int                x                   x坐标
             int                y                   y坐标
             bool               ignoreAnchor        是否忽略锚点(默认false)
             String             selectImage     选中图片
             String             disableImage    不可选图片
             function(button)   listener        事件方法
             enum               state           按钮的状态:   
                                        widget.STATE_BUTTON_NORMAL (default)
                                        widget.STATE_BUTTON_SELECT
                                        widget.STATE_BUTTON_DISABLE  
             CCRect             rect            9宫图区域        其与capInsets构成是否是9宫的判定
             CCRect             capInsets       9宫图拉伸区域
             CCNode             container       所属容器,不必是直接父容器,只是作为位置判定看是否启用,若位置不在指定容器内则无效,一般情况下不进行设置
             CCNode             triggerContainer 触发按键的容器,若指定,点击在指定容器范围内都会触发按键的点击,要求是按键本身也在指定容器范围内
             bool               cancelOnMove    是否启用移动后取消其可点击模式,默认为false,在TableView这样的控件中通常设置为true,并同时设置swallowsTouches为false
             bool               swallowsTouches 是否启用touch拦截,默认为true
             table              labelParams     字体属性,若无需文字则不用设置,数组包含
                                    [必填项]
                                    string      text                显示的文字
                                    [选填项]
                                    string      font                字体, 默认为Arial
                                    int         size                字体大小, 默认为24
                                    ccc3/ccc4   color               文字颜色, 默认为白色
                                    enum        textAlign           文字水平对齐方式
                                                        ui.TEXT_ALIGN_LEFT (default)
                                                        ui.TEXT_ALIGN_CENTER
                                                        ui.TEXT_ALIGN_RIGHT
                                    enum        textValign          文字竖直对齐方式
                                                        ui.TEXT_VALIGN_TOP
                                                        ui.TEXT_VALIGN_CENTER (default)
                                                        ui.TEXT_VALIGN_BOTTOM
                                    int         x                   x坐标
                                    int         y                   y坐标
                                    CCSize      dimensions          尺寸  
                                    table       padding             文字位置相对于display.CENTER的偏移,有效值为+left, -top                                         

--]]

function M.newButton( params )

--[[ ## local params --]]
    local _layer            = display.newLayer()
    --#必填项#
    local _contentSize      = params.contentSize
    local _normalImage      = params.normalImage
    --#选填项#
    local _sound            = params.sound
    local _selectImage      = params.selectImage 
    local _disableImage     = params.disableImage 
    local _listener         = params.listener 
    local _rect             = params.rect
    local _capInsets        = params.capInsets
    local _state            = params.state or M.STATE_BUTTON_NORMAL
    local _checkMode        = params.checkMode or false
    local _enable           = not params.locked
    local _initScale        = params.initScale or 1
    local _scale            = params.scale or _initScale
    local _discolor         = params.discolor
    local _labelParams      = params.labelParams
    local _isSelected       = params.isSelected or false
    local _container        = params.container
    local _triggerContainer = params.triggerContainer
    local _cancelOnMove     = params.cancelOnMove or false
    local _swallowsTouches  = ifnil(params.swallowsTouches, true)
    local _priority         = params.priority or -128
    --#内部参数#   
    local _tTFLabel
    local _image
    local _buttonRect

--[[ ## local fuction --]]

    local getRect
    local setState
    local setImage
    local doScale
    local onTouch
    local onTouchBegan
    local onTouchMoved
    local onTouchEnded
    local isTouchInside
    local init

--[[ ## public fuction --]]

    --[[ 

    更改文字

    **Parameters:**

    -   text: 文本信息

    --]]

    function _layer.setTextString( string )
        if _tTFLabel and string then 
            _tTFLabel:removeSelf(true)
            _tTFLabel = nil
            -- _tTFLabel:setString(string)
            -- 擦不顶用.
            -- util.alignNodeWithPadding(_tTFLabel, _layer:getContentSize(), display.CENTER, _labelParams.padding)
        end
        if not _tTFLabel then 
            -- 文字
            if _labelParams then
                if string then 
                    _labelParams.text = string
                end
                if not _labelParams.text then return end 
                if _labelParams.shadow then
                    _tTFLabel  = M.newTTFLabelWithShadow(_labelParams)
                else
                    _tTFLabel  = ui.newTTFLabel(_labelParams)
                end
                util.alignNodeWithPadding(_tTFLabel, _layer:getContentSize(), display.CENTER, _labelParams.padding)
                _layer:addChild(_tTFLabel, 5)
            end
        end
    end

    --[[ 

    更改文字

    **Parameters:**

    -   text: 文本信息

    --]]

    function _layer.getTextString()
        if _tTFLabel then 
            return _tTFLabel:getString()
        end
    end

    --[[ 

    设置选中状态

    **Parameters:** 

    -   isSelect: bool

    --]]

    -- function _layer.setSelected( isSelect )
    --     if _enable == isSelect then
    --         _enable = not isSelect
    --         setState(isSelect and M.STATE_BUTTON_SELECT or M.STATE_BUTTON_NORMAL)
    --     end
    -- end

    --[[ 

    设置不可选状态

    **Parameters:** 
ne
    -   isDisable: bool

    --]]

    function _layer.setStateDisabled( isDisable )
        if _enable == isDisable then
            _enable = not isDisable
            setState(isDisable and M.STATE_BUTTON_DISABLE or M.STATE_BUTTON_NORMAL)
        end
    end

    --[[ 

    锁定按钮

    **Parameters:** 

    -   islocked: bool

    --]]

    function _layer.locked( islocked )
        _enable = not islocked
    end
    --[[ 
    
    设置缩放尺寸
    
    **Parameters:**
    
    -   scale:        float     比例
    
    --]]
    function _layer.setChangeScale( scale )
        _scale = scale
    end
    --[[ 

    设置响应事件

    **Parameters:** 

    -   listener: funtion事件

    --]]

    function _layer.setListener( listener )
        _listener = listener
    end
    --[[ 
    
    取得响应事件

    **Returns:**
    
    -   return 响应事件
    
    --]]
    function _layer.getListener()
        return _listener
    end

    --[[ 

    设置文字位置

    **Parameters:** 

    -   x: position x

    -   y: position y

    --]]

    function _layer.setTextPosition( x, y )
        if _tTFLabel then
            _tTFLabel:setPosition(x, y)
        end
    end

    --[[ 

    设置文字位置

    **Parameters:** 

    -   images:  设置状态图片集

    -   isForceReflesh: 是否强制刷新，若强制刷新，则会立即更新当前按钮图片

    --]]

    function _layer.setStateImages(images, isForceReflesh)
        _normalImage  = images[1]
        _selectImage  = images[2]
        _disableImage = images[3]

        if isForceReflesh then 
            setState(_state, true)
        end
    end

    --[[ 

    取得按钮状态

    **Returns:**

    -   返回按钮状态三种: M.STATE_BUTTON_NORMAL, M.STATE_BUTTON_SELECT, M.STATE_BUTTON_DISABLE 

    --]]

    function _layer.getState( )
        return _state
    end
    function _layer.isSelected( )
        return _isSelected
    end
    function _layer.setSelected( select )
        _isSelected = select
        if _isSelected then
            setState(M.STATE_BUTTON_SELECT)
        else
            setState(M.STATE_BUTTON_NORMAL)
        end
        if not _checkMode then
            _enable = not select
        end
    end
    --[[ 
    
    设置点击触发的容器,即点击此容器即可触发button
    
    **Parameters:**
    
    -   triggerContainer:        CCNode      触发容器
    
    --]]
    function _layer.setTriggerContainer( triggerContainer )
        _triggerContainer = triggerContainer
    end
--[[ ## local fuction --]]

    --[[ 

    取得Layer尺寸

    **Returns:**

    -   包含Layer尺寸的矩形

    --]]

    function getRect()
        if not _buttonRect then
            local layer = _triggerContainer or _layer
            _buttonRect =  CCRectMake(  0, 0,
                                layer:getContentSize().width, 
                                layer:getContentSize().height)
        end
        return _buttonRect 
    end


    local _beganX, _beganY = 0, 0
    local _ismoved
    local function resetTouchStatus(  )
        _beganX, _beganY = 0, 0
        _ismoved = nil
    end

    function onTouchBegan(x, y)
        if not util.isCustomTouchEnabled(_layer) then
            return false
        end
        -- printf("onTouchBegan:x = " .. x .. "y=" .. y)
        if not isTouchInside(x, y) then return false end
        if _cancelOnMove then
            _beganX, _beganY = x, y
        end
        doScale(_scale)
        if not _checkMode then
            setState(M.STATE_BUTTON_SELECT)
        end
        return true
    end

    function onTouchMoved(x, y)
        -- printf("onTouchMoved:x = " .. x .. "y=" .. y)
        if isTouchInside(x, y) and not _ismoved then 
            doScale(_scale)
            if not _checkMode then
                setState(M.STATE_BUTTON_SELECT)
            end
        else
            doScale(_initScale)
            if not _checkMode then
                setState(M.STATE_BUTTON_NORMAL)
            end
        end
        if _cancelOnMove then
            if _ismoved ~= true and (math.abs(x - _beganX) > 5 or math.abs(y - _beganY) > 5) then
                _ismoved = true
            end
        end
    end

    function onTouchEnded(x, y)
        -- printf("onTouchEnded:x = " .. x .. "y=" .. y)
        doScale(_initScale)
        if isTouchInside(x, y) and not _ismoved then 
            if _checkMode then
                if _isSelected then
                    setState(M.STATE_BUTTON_NORMAL)
                else
                    setState(M.STATE_BUTTON_SELECT)
                end
                _isSelected = not _isSelected
            else
                setState(M.STATE_BUTTON_NORMAL)
            end
            if _listener then
                if _sound then 
                    -- audio.stopAllEffects()
                    audio.playSound(_sound)
                end
                _listener(_layer, x, y)
            end
        else
            if not _checkMode then
                setState(M.STATE_BUTTON_NORMAL)
            end
        end
        resetTouchStatus(  )
    end
    --[[ 
    
    执行缩放
    只有设置过scale才会有效

    **Parameters:**
    
    -   scale:        float     数值

    --]]
    function doScale( scale )
        if _scale ~= _initScale then
            -- _layer:setScale(scale)
            transition.scaleTo(_layer, {time = 0.05, scale = scale})
        end
    end
    --[[ 

    判断内部点击

    **Parameters:** 

    -   x: 点击坐标x

    -   y: 点击坐标x

    **Returns:**

    -   判断结果bool

    --]]

    function isTouchInside( x, y)
        -- printf("enter isTouchInside")
        local touchLocation = CCPoint(x, y)
        if _container then
            local loc = _container:convertToNodeSpace(touchLocation)
            local rect = _container:boundingBox()
            if not rect:containsPoint(loc) then
                return false
            end
        end
        if _triggerContainer then
            touchLocation = _triggerContainer:convertToNodeSpace(touchLocation)
        else
            touchLocation = _layer:convertToNodeSpace(touchLocation)
        end
        local r = getRect()
        --r.origin = CCPointZero
        return r:containsPoint(touchLocation)
    end

    function onTouch(eventType, x, y)
        -- printf("eventType:" .. eventType .. ":x = " .. x .. "y=" .. y)
        if _enable and _layer:isVisible() then
            -- printf("yes isTouchInside")
            if eventType == M.TOUCHBEGAN then
                return onTouchBegan(x, y)
            elseif eventType == M.TOUCHMOVED then
                return onTouchMoved(x, y)
            elseif eventType == M.TOUCHENDED then
                return onTouchEnded(x, y)
            else 
                resetTouchStatus(  )
            end
        end    
    end

    function init( )
        Log.a(_normalImage, M.TAG, "newButton() invalid params.normalImage")
        if _rect or _capInsets then 
            Log.a(_contentSize, M.TAG, "newButton() invalid params.contentSize")
            if not _rect then _rect = CCRectZero end
            if not _capInsets then _capInsets = CCRectZero end
        end

        --使用定义的参数
        M.initView(_layer, params)

        -- 背景
    	setState(_state, true)
        if _checkMode then
            if _isSelected then
                setState(M.STATE_BUTTON_SELECT)
            else
                setState(M.STATE_BUTTON_NORMAL)
            end
        end
        -- 文字
        _layer.setTextString()
        -- if _labelParams then
        --     if _labelParams.shadow then
        --         _tTFLabel  = M.newTTFLabelWithShadow(_labelParams)
        --     else
        --         _tTFLabel  = ui.newTTFLabel(_labelParams)
        --     end
        --     -- _tTFLabel  = ui.newTTFLabel(_labelParams)
        --     -- _tTFLabel:setPosition(_layer:getContentSize().width/2, _layer:getContentSize().height/2)
        --     util.alignNodeWithPadding(_tTFLabel, _layer:getContentSize(), display.CENTER, _labelParams.padding)
        --     _layer:addChild(_tTFLabel, 5)
        -- end
        _layer:setScale(_initScale)
        -- 添加Touch事件
        _layer:setTouchEnabled(true)
        _layer:addTouchEventListener(onTouch, false, _priority, _swallowsTouches)
        _layer:ignoreAnchorPointForPosition(false)
    end

    --[[ 

    设置图片，会刷新一次

    --]]
    function setImage( fileName, force )
        local function setImage_( fileName )
            if _image then
                _image:removeFromParentAndCleanup(true)
            end
            if not _rect and not _capInsets then
                --sprite
                _image = display.newSprite(fileName)
                _image:ignoreAnchorPointForPosition(true)
            else 
                --9宫
                _image = M.newScale9{
                                    file = fileName,
                                    rect = _rect,
                                    capInsets = _capInsets,                                     
                                    contentSize = _contentSize,
                                }
                _image:ignoreAnchorPointForPosition(true)
            end
            if not _contentSize then 
                _contentSize = _image:getContentSize() 
                _layer:setContentSize(_contentSize)
            end
            _layer:addChild(_image, 0)
        end

        if _normalImage and not _selectImage and not _disableImage and not force then 
            if not _image then 
                setImage_(fileName)
            end
        else
            setImage_(fileName)
        end
    	
    end

    --[[ 

    更改Button状态

    **Parameters:**

    -   state: 状态

    -   force: 强制刷新仅init()使用

    --]]

    function setState( state , force)
        if _state ~= state or force or not _state then
            if _state then
                _state = state
            end

            if(state == M.STATE_BUTTON_DISABLE and _disableImage) then
                setImage(_disableImage) 
                if _discolor then
                    local colorCache = _labelParams.color
                    _labelParams.color = _discolor
                    _layer.setTextString( _labelParams.text )
                    _labelParams.color = colorCache
                end
            elseif(state == M.STATE_BUTTON_SELECT and _selectImage)  then
                setImage(_selectImage)
                if _discolor then
                    local colorCache = _labelParams.color
                    _labelParams.color = _discolor
                    _layer.setTextString( _labelParams.text )
                    _labelParams.color = colorCache
                end
            elseif(state == M.STATE_BUTTON_NORMAL or force)  then
                setImage(_normalImage, force)
                if _discolor then
                    _layer.setTextString( _labelParams.text )
                end
            end
        end
    end

    function _layer.playAnimation(animation, forever, args)
        if _image then 
            _image:stopAllActions()
            args = args or {}
            if forever then 
                transition.playAnimationForever(_image, animation, args.delay)
            else
                transition.playAnimationOnce(_image, animation
                    , args.removeWhenFinished
                    , args.onComplete
                    , args.delay)
            end
        end
    end

    function _layer.stopAnimation()
        if _image then 
            _image:stopAllActions()
            setState(_state, true)
        end
    end

--[[ ## running --]]

    init()
    _layer._type_ = WIDGET_TYPE.Button
    return _layer
end
--[[ 

## checkBox

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize         尺寸
            String      normalImage         常态图片
            String      selectImage         选中图片
            [选填项]
            int         x                   x坐标
            int         y                   y坐标
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newCheckBox( params )
    util.checkParams(M.TAG, "newCheckBox()", {
            { istable(params), "is not a table" },
            { params.normalImage, "normalImage"},
            { params.selectImage, "selectImage" },
        })
    params.checkMode = true
    local result = widget.newButton(params)
    result._type_ = WIDGET_TYPE.CheckBox
    return result
end
--[[ 

## 切换用Tab, 需要增加按钮时调用addButton, 禁使用addChild不响应事件

**Parameters:**

-   params: 数组包含
            [必填项]
            
            [选填项]
             function(index, button) listener 事件方法
             bool enable           允许响应事件
             direction        排列方向:DIRECTION_HORIZONTAL
                                       DIRECTION_VERTICAL 
             verticalAlign    垂直对齐方式:VERTICAL_CENTER
                                           VERTICAL_LETF
                                           VERTICAL_RIGHT
             horizontalAlign  水平对齐方式:HORIZONTAL_CENTER
                                           HORIZONTAL_TOP
                                           HORIZONTAL_BOTTOM
                                           HORIZONTAL_NONE
             bool showGone         隐藏的占位
             int  padding          不解释
             bool autoRefresh      自动刷新
             items{button1, button2 ...} 初始按钮
            int         x                   x坐标
            int         y                   y坐标
            bool        ignoreAnchor        是否忽略锚点(默认false)

--]]

function M.newTab( params )

--[[ ## local params --]]
    --#必填项#
    --local _contentSize      = params.contentSize
    --#可选参数#
    local _items           =   params.items 
    local _listener        =   params.listener
    local _padding         =   params.padding
    local _direction       =   params.direction
    local _verticalAlign   =   params.verticalAlign
    local _horizontalAlign =   params.horizontalAlign
    local _showGone        =   params.showGone
    local _autoRefresh     =   params.autoRefresh
    local _selectIndex     =   params.selectIndex
    local _layer           =   LinearLayer:create()  
    local _x                = params.x
    local _y                = params.y
    local _ignoreAnchor     = params.ignoreAnchor or false 

--[[ ## local fuction --]]

    local onTouch
    local init

--[[ ## public fuction --]]

    --[[

    增加按钮

    **Parameters:**

    -   button: 按钮

    --]]
    function _layer.addButton( button, zorder)
        button.setListener(onTouch)
        if zorder then 
            _layer:addChild(button, zorder)
        else
            _layer:addChild(button)
        end
    end

    --[[ 

    自动刷新
    
    **Parameters:** 

    -   isAuto:     bool        刷新

    --]]

    function _layer.autoRefresh( isAuto )
        _autoRefresh = isAuto
        _layer:setAutoRefresh(isAuto)
    end

    --[[

    排列方式

    **Parameters:**

    -   state: 状态, 参看LinearLayer.h
                    DIRECTION_HORIZONTAL 
                    DIRECTION_VERTICAL

    --]]

    function _layer.direction( state )
        _direction = state
        _layer:setDirection(state)
    end

    --[[

    垂直对齐方式
    
    **Parameters:**

    -   state: 状态, 参看LinearLayer.h
                    VERTICAL_CENTER
                    VERTICAL_LETF  
                    VERTICAL_RIGHT 

    --]]

    function _layer.verticalAlign( state )
        _verticalAlign = state
        _layer:setVerticalAlign(state)
    end

    --[[ 

    水平对齐方式
        
    **Parameters:**

    -   state: 状态, 参看LinearLayer.h
                    HORIZONTAL_CENTER
                    HORIZONTAL_TOP   
                    HORIZONTAL_BOTTOM

    --]]

    function _layer.horizontalAlign( state )
        _horizontalAlign = state
        _layer:setHorizontalAlign(state)
    end

    --[[ 

    设置响应事件
        
    **Parameters:**

    -   listener: funtion(index, button)事件包含参数, index 位置, button 按钮

    --]]

    function _layer.setListener( listener )
        _listener = listener
    end

    --[[ 

    设置Padding
        
    **Parameters:**

    -   padding: 不解释

    --]]

    function _layer.padding( padding )
        _padding = padding
        _layer:setPadding(padding)
    end

    --[[ 

    选中button
    
    **Parameters:**

    -   index: 位置 
    
    -   first: 仅内部调用初始化时为true, 其他一律不要设置

    --]]

    function _layer.selectButton( index, first)
        if(_selectIndex ~= index or first) then
            local children = _layer:getChildren()
            children:objectAtIndex(index - 1).setSelected(true)
            if (_selectIndex and not first )then
                children:objectAtIndex(_selectIndex - 1).setSelected(false)
            end
            _selectIndex = index
        end      
    end
    function _layer.noSelected( force )
        if _selectIndex or force then
            local children = _layer:getChildren()
            children:objectAtIndex(_selectIndex - 1).setSelected(false)
            _selectIndex = nil
        end
    end
    function _layer.getSelected( )
        return _selectIndex
    end
--[[ ## local fuction --]]

    function init( )
        if _items then
            local ccarray = CCArray:create()
            for i=1, #_items do
                _items[i].setListener(onTouch)
                ccarray:addObject(_items[i])
            end
            _layer:initWithArray(ccarray)
        end
        if _padding then _layer:setPadding(_padding) end
        if _showGone then _layer:setDirection(_showGone) end
        if _direction then _layer:setDirection(_direction) end      
        if _autoRefresh then _layer:setAutoRefresh(_autoRefresh) end
        if _selectIndex then _layer.selectButton(_selectIndex, true) end
        if _verticalAlign then _layer:setVerticalAlign(_verticalAlign) end
        if _horizontalAlign then _layer:setHorizontalAlign(_horizontalAlign) end
        --如果初始化前已经有内容，则需要刷新下应用上配置
        if _items then _layer:refresh() end
        --使用定义的参数
        M.initView(_layer, params)
    end

    function onTouch(button)
        if not util.isCustomTouchEnabled(_layer) then return false end
        local children = _layer:getChildren()
        local index = children:indexOfObject(button) + 1
        if(_selectIndex ~= index) then
            if _selectIndex then
                _layer.selectButton(index)
            else
                _layer.selectButton(index, true)
            end
            
            if _listener then
                -- printf("index:" .. index)
                _listener(index, button)
            end
        end

    end

--[[ ## running --]]

    init()
    _layer._type_ = WIDGET_TYPE.Tab
    return _layer
end

--[[ 

## 包含Tab的容器

**Parameters:**

-   params: 数组包含
            [必填项]
             CCSize             contentSize     控件的尺寸
             [选填项]
             function(index, button) listener 事件方法
             items{button1, button2 ...} 初始按钮
             button closeButton    关闭按钮
             CCSize contentSize    尺寸
             CCNode content        内容
            int         x                   x坐标
            int         y                   y坐标
            bool        ignoreAnchor        是否忽略锚点(默认false)


--]]

function M.newTabContainer( params )

--[[ ## local params --]]

    --#必填项#
    local _contentSize      = params.contentSize
    --#选填项#
    local _tabItems        =   params.tabItems 
    Log.a(_tabItems, M.TAG, "newTabContainer() invalid params.tabItems")
    local _listener        =   params.listener
    local _close           =   params.closeButton
    local _padding         =   params.padding or 5
    local _contentSize     =   params.contentSize or CCSize(display.width, display.height)
    local _tab             =   M.newTab({items = _tabItems, listener = _listener})
    local _content         =   display.newLayer()
    local _layer           =   display.newLayer()

    local _x                = params.x
    local _y                = params.y
    local _ignoreAnchor     = params.ignoreAnchor or false
--[[ ## local fuction --]]
    local init
    local onTouchClose    

--[[ ## public fuction --]]

    --[[

    替换内容

    **Parameters:**

    -   content: 需要替换的内容

    --]]
    function _layer.replaceContent( content )
         _content:removeAllChildrenWithCleanup(true)
         -- content:setPosition(_layer:getPositionX() + _layer:getContentSize().width/2 , _layer:getPositionY() + _layer:getContentSize().height/2)
         _content:addChild(content)
    end 

    --[[

    增加Tab按钮

    **Parameters:**

    -   button: Tab按钮

    --]]

    function _layer.addTabButton( button )
        _tab.addButton(button)
    end

    --[[

    增加关闭按钮

    **Parameters:**

    -   button: 关闭按钮

    --]]

    function _layer.setCloseButton( button )
        _close = button
    end

    --[[

    关闭自己

    --]]

    function _layer.close( )
        _layer:removeFromParentAndCleanup(true)
    end

--[[ ## local fuction --]]

    function init( )
        _content:setContentSize(_contentSize)
        _layer:setContentSize(_contentSize)
        _layer:addChild(_content, 0)
        _layer:addChild(_tab, 1)
        _content:setPosition(_content:getContentSize().width/2, _content:getContentSize().height/2)
        _tab:setPosition(_tab:getContentSize().width/2, _content:getContentSize().height - _tab:getContentSize().height/2)
        if _close then
            _close.setListener(onTouchClose)
            _close:setPosition( _content:getContentSize().width - _close:getContentSize().width/2,
                                _content:getContentSize().height - _close:getContentSize().height/2)
            _layer:addChild(_close, 2)
        end
        --使用定义的参数
        M.initView(_layer, params)
    end

    function onTouchClose(button)
        _layer.close( )
    end

--[[ ## running --]]

    init()
    _layer._type_ = WIDGET_TYPE.TabContainer
    return _layer
end

--[[ 
 
 ## 创建TableView
 
 **Parameters:**
 
 -   params: 数组包含
             [必填项]
             CCSize                     contentSize 			    TableView的尺寸
             CCSize                     cellSize 			        TableCell的尺寸
             [人妖项]
             fun(tableView, index)      cellAtIndexListener         cell取得事件,用于创建或取得对应的cell,当useLinearLayer为true时,不用设置此项,否则一定要设置
             fun(tableView,index,item,cell) createItemCellListener  创建Cell事件,在useLinearLayer为false时,且cellAtIndexListener未定义时有效,
                                                                    不必返回值,主要是根据item在cell中拼界面,要区分item存在不存在的问题
             fun(tableView,index,item,cell) modifyItemCellListener  修改cell事件,使用前提同createItemCellListener,若未定义则会清空所有子项后触发createItemCellListener
             [选填项]
             int                        numOfCells                  cell个数
             bool                       bounceable                  是否启用边界阻尼效果,默认true
             boolean                    useLinearLayer              使用线性布局,启用后将一次性生成所有项放于LinearLayer中,解决其中的按键点击问题,默认为false
             fun(tableView, idx, item)  linearCellAtIndexListener   使用线性布局时每个cell的创建监听,只有在useLinearLayer为true时有效
             enum		                direction 			        方向
                                             widget.TABLEVIEW_DIRECTION_NONE
                                             widget.TABLEVIEW_DIRECTION_HORIZONTAL
                                             widget.TABLEVIEW_DIRECTION_VERTICAL (default)
                                             widget.TABLEVIEW_DIRECTION_BOTH
             enum		                verticalFillOrder 	        垂直方向时的填充顺序
                                             widget.TABLEVIEW_FILL_TOP_DOWN (default)
                                             widget.TABLEVIEW_FILL_BOTTOM_UP
             table                      items                       显示元素对应的对象entity
             int 		                x 					        x坐标
             int                        y 					        y坐标
             bool                       ignoreAnchor                是否忽略锚点,没有这个参数啊~~~一定要为true的
             fun(tableView, cell, index)       tapListener 		        点击cell回调事件
             fun(tableView)             onBeforeRefresh             刷新前处理函数
             fun(tableView)             onAfterRefresh              刷新后处理函数
             table                      selectedParams              选中效果(同9宫图参数)
                                            string      file        背景图片名
                                            CCRect      rect        图片尺寸矩形
                                            int         offset      与其容器的间隔值
 --]]
function M.newTable( params )
	local _tableView, _cell
    local cellAtIndexListenerForLinear = function ( ... ) end
	--选填项
    local _bounceable               = ifnil(params.bounceable, true)
    local _useLinearLayer           = params.useLinearLayer or false
    local _linearCellAtIndexListener= params.linearCellAtIndexListener
    local _createItemCellListener   = params.createItemCellListener
    local _modifyItemCellListener   = params.modifyItemCellListener
    --必填项
	local _contentSize 				= params.contentSize
	local _cellSize 				= iff(_useLinearLayer, CCSizeMul(params.cellSize, 1, params.numOfCells or 1), params.cellSize)
    --乱入项...
    if _useLinearLayer then
        cellAtIndexListenerForLinear = function ( tableView, idx )
            local cell = tableView:dequeueCell()
            if not cell then 
                cell = tableView.newCell() 
            else
                tableView.removeCellAll(cell)
                -- cell:removeFromParentAndCleanup(true)
                -- cell = tableView.newCell()
            end
            
            local items = tableView:getItems()
            if items then
                for i,item in ipairs(items) do
                    local view = _linearCellAtIndexListener(tableView, i, item)
                    --设置下其tag为i,讨论会日后找到
                    view:setTag(i)
                    cell:addItemView(view, view:getZOrder(), view:getTag())
                end
            end
            _cell = cell
            return cell
        end
    end
    --人妖项...我都不能确定你是不是必要的了
    local _cellAtIndexListener      = params.cellAtIndexListener or iff(_useLinearLayer, cellAtIndexListenerForLinear, nil)
    if not _cellAtIndexListener then 
        -- 未定义_cellAtIndexListener但定义了_createItemCellListener时,自动封装_cellAtIndexListener
        if _createItemCellListener then 
            _cellAtIndexListener = function ( tableView, index )
                local cell = tableView:dequeueCell()
                local item = tableView:itemAt(index)
                if not cell then
                    -- 进行创建 
                    cell = tableView.newCell()
                    cell:setContentSize(_cellSize)
                    _createItemCellListener( tableView, index, item, cell )
                else
                    -- 进行修改
                    if _modifyItemCellListener then 
                        -- 有修改方法则进行修改操作
                        _modifyItemCellListener( tableView, index, item, cell )
                    else
                        -- 无修改方法,则清空cell中所有元素后再次进行创建操作
                        cell:removeAllChildrenWithCleanup(true)
                        _createItemCellListener( tableView, index, item, cell )
                    end
                end
                return cell
            end
        end
    end
	--选填项
	local _direction 				= params.direction or M.TABLEVIEW_DIRECTION_VERTICAL
    local _items                    = params.items or {}
    local _numOfCells               = iff(_useLinearLayer, 1, params.numOfCells or #_items)
	local _verticalFillOrder 		= params.verticalFillOrder or M.TABLEVIEW_FILL_TOP_DOWN
	local _x 						= params.x
	local _y 						= params.y
	local _tapListener 				= params.tapListener
    local _selectedParams           = params.selectedParams
    local _onBeforeRefresh          = params.onBeforeRefresh
    local _onAfterRefresh           = params.onAfterRefresh

    --内部参数
    local _lastSelectedItemIndex    = -1
    local _selectedBg               = nil
    local _linearCellSize           = iff(_useLinearLayer, params.cellSize, nil)
    local _linearNumOfCells         = params.numOfCells

	--创建对象
	Log.a(_contentSize, M.TAG, "newTable() invalid params.contentSize")
    Log.a(_cellSize, M.TAG, "newTable() invalid params.cellSize")
    Log.a(_numOfCells, M.TAG, "newTable() invalid params.numOfCells")
    Log.a(_cellAtIndexListener, M.TAG, "newTable() invalid params.cellAtIndexListener")
    Log.a(iff(_useLinearLayer, isnotnil(_linearCellAtIndexListener), true), M.TAG, "newTable() invalid params.linearCellAtIndexListener")
	
    _tableView = LuaTableView:create(_contentSize, _cellSize, _numOfCells, _cellAtIndexListener)
	
	--[[ ## local fuction --]]
    local function onItemClickListener( tableView, cell )
        if not util.isCustomTouchEnabled(_tableView) then
            return 
        end
        local index = cell:getIdx() + 1
        local temp_lastSelectedItemIndex = _lastSelectedItemIndex
        local i = _tableView:getClickedIndex(index)
        if 0 < i and i <= _linearNumOfCells then
            _tableView:setSelectEffect(i, true)
            if _tapListener then -- and i ~= temp_lastSelectedItemIndex then
                -- _tapListener(tableView, cell, i)
                _tapListener(tableView, i, tableView:itemAt(i), cell)
            end
        end
    end

    local function onTouchBeganHandler_( x, y )
        _tableView.clickedPoint = ccp(x, y)
        return util.isCustomTouchEnabled(_tableView) 
            -- and (_tableView:getContentSize().width > _tableView:getViewSize().width 
            -- or _tableView:getContentSize().height > _tableView:getViewSize().height)
    end

	local function init()
        if _selectedParams and not _selectedParams.offset then
            _selectedParams.offset = 6
        end
		_tableView:setDirection(_direction)
		if _verticalFillOrder then
			_tableView:setFillOrder(_verticalFillOrder)
		end
        params.ignoreAnchor = true
        M.initView(_tableView, params)
		_tableView:registerScriptTapHandler(onItemClickListener)
        if _tableView:isBounceable() ~= _bounceable then 
            _tableView:setBounceable(_bounceable) 
        end
        _tableView:registerScriptTouchBeganHandler(onTouchBeganHandler_)
        if _items then 
		  _tableView:refresh()
        end
	end

    --[[ 
    
    停止停下后的位置调整
    
    **Parameters:**
    
    -   x:        float      停下时的x坐标值
    -   y:        float      停下时的y坐标值
    
    --]]
    local function onRelocateHandler_( x, y )
        if _tableView:getDirection() == M.TABLEVIEW_DIRECTION_HORIZONTAL then 
            local cellWidth = _cellSize.width
            local index = math.ceil(math.abs(x) / cellWidth)
            local isleft = index - 0.5 < math.abs(x) / cellWidth
            local offset = math.abs(x) - (index - 1) * cellWidth
            if isleft then offset = offset - cellWidth end
            _tableView:setContentOffsetInDuration(ccp(x + offset, y), 0.15)
        elseif _tableView:getDirection() == M.TABLEVIEW_DIRECTION_VERTICAL then
            local cellHeight = _cellSize.height
            local yy = _tableView:getViewSize().height + math.abs(y) 
            local index = math.ceil(yy / cellHeight)
            local isup = index - 0.5 > yy / cellHeight
            local offset = yy - index * cellHeight
            if isup then offset = offset + cellHeight end
            _tableView:setContentOffsetInDuration(ccp(x, y + offset ), 0.15)
        end
    end
    _tableView:registerScriptRelocateHandler(onRelocateHandler_)
--[[ ## public fuction --]]
	
	--[[ 
	 
	 设置垂直填充方式
	 
	 **Parameters:**
	 
	 -   verticalFillOrder: 垂直填充方式
                        	 TABLEVIEW_FILL_TOP_DOWN
                        	 TABLEVIEW_FILL_BOTTOM_UP
	 
	 --]]
	function _tableView:setFillOrder( verticalFillOrder )
		self:setVerticalFillOrder(verticalFillOrder)
	end
	
    --[[ 
	 
	 刷新table
	 
	 --]]
	function _tableView:refresh()
        self:setSelectEffect(0)
        -- returnAllWidgets(  )
        if _onBeforeRefresh then _onBeforeRefresh(_tableView) end
		self:reloadData()
        if _onAfterRefresh then _onAfterRefresh(_tableView) end
        --切换页面时重新将顶部显示在tableView的顶部
        _tableView:setContentOffset(ccp(0, 
            _tableView:getViewSize().height - _tableView:getContainer():getContentSize().height
            ))
        -- if _useLinearLayer then 
            --对新建的项要设置其优先级,要不还是不能用,就因为这个卡了不知道多久啊,花客
            -- CCBCenter.changeAllLayerTouchPriorityToLast(_tableView)
        -- end
	end
	
	--[[ 
	 
	 创建空cell
	 
	 --]]
	function _tableView.newCell()
		local cell = LuaTableView:createCell()
        if _useLinearLayer then
            local layer = _tableView:createLinearLayer()
            cell:addChild(layer, 0, 1000)

            function cell:getLinearLayer(  )
                return self:getChildByTag(1000)
            end
            function cell:addItemView( view, zorder, tag )
                self:getLinearLayer(  ):addChild(view, zorder or 0, tag or -1)
            end
        end

        return cell
	end
    --[[ 
    
    移除指定cell中的所有项
    
    **Parameters:**
    
    -   cell:        CCNode     table的cell
    -   cleanup:     bool       是否彻底清除,默认为true
        
    --]]
    function _tableView.removeCellAll( cell, cleanup )
        if cell then
            if _useLinearLayer then
                cell:getChildByTag(1000):removeAllChildrenWithCleanup( ifnil(cleanup , true) )
            else
                cell:removeAllChildrenWithCleanup( ifnil(cleanup , true) )
            end
        else
            Log.w(M.TAG, "newTable() -->removeCellAll() cell is nil" )
        end
    end
	
    --[[ 
	 
	 插入cell
	 
	 **Parameters:** 
	 
	 -   idx: int 下标
	 
	 --]]
	function _tableView:insertCell( idx )
		self:insertCellAtIndex(idx - 1)
	end
	
    --[[ 
	 
	 移除cell
	 
	 **Parameters:** 
	 
	 -   idx: int 下标
	 
	 --]]
	function _tableView:removeCell( idx )
		self:removeCellAtIndex(idx - 1)
	end
	
    --[[ 
	 
	 取得cell
	 
	 **Parameters:** 
	 
	 -   idx: int 下标
	 
	 --]]
	function _tableView:cellAt( idx )
		return self:cellAtIndex(idx - 1)
	end

    --[[ 
    
    取得指定下标的item
    
    **Parameters:**
    
    -   idx:        int     下标
    
    **Returns:**
    
    -   return
    
    --]]
    function _tableView:itemAt( idx )
        --idx = idx - 1
        if _items and 0 < idx and idx <= #_items then
            return _items[idx]
        else
            return nil
        end
    end

    --[[ 
    
    设置item的entity
    
    **Parameters:**
    
    -   items:        table     item应对元素的entity
    
    --]]
    function _tableView:setItems( items )
        _items = nil
        _items = items
        self:resetNumberOfCells(#items)
        _linearNumOfCells = #items
        -- self:setViewSize(_contentSize)
        self:refresh()
        -- if self:getContentSize().width < self:getViewSize().width 
        --     or self:getContentSize().height < self:getViewSize().height then
        --     self:setViewSize(CCSize(self:getContentSize().width, self:getContentSize().height))
        -- -- else
        -- --     self:setViewSize(_contentSize)
        -- end
    end
    function _tableView:getItems(  )
        return _items
    end
	--[[ 
   
    重置cell尺寸
    
    **Parameters:**
    
    -   cellSize:       CCSize      cell尺寸

    -   update:         bool        是否进行更新(默认为true)
    
    --]]
    function _tableView:resetCellSize( cellSize,  update)
        _tableView:setCellSize(cellSize, update == nil and true or update)
    end
    --[[ 
   
    重置cell个数
    
    **Parameters:**
    
    -   numberOfCells:      int      cell个数

    -   update:             bool        是否进行更新(默认为true)
    
    --]]
    function _tableView:resetNumberOfCells( numberOfCells, update )
        _tableView:setNumberOfCells(numberOfCells, update == nil and true or update)
    end
	--[[ 
    
    取得选择中的背景
    
    **Returns:**
    
    -   return 选择中的背景
    
    --]]
    local function getSelectedBg()
        if not _selectedBg then
            if _selectedParams then
                --设置下选中背景
                _selectedBg = M.newScale9(_selectedParams)
                _tableView:addChild(_selectedBg, 100)
                _selectedBg:setVisible(false)
                _selectedBg:ignoreAnchorPointForPosition(false)
            end
        end
        return _selectedBg
    end

    function _tableView:getClickedIndex( index )
        local result = index
        if _useLinearLayer then
            local cellSize = _linearCellSize or _cellSize
            local cell = _tableView:cellAt(index)
            if cell and _tableView.clickedPoint then
                local touchPoint = cell:convertToNodeSpace(_tableView.clickedPoint)
                if _direction == M.TABLEVIEW_DIRECTION_VERTICAL then
                    if _verticalFillOrder == widget.TABLEVIEW_FILL_TOP_DOWN then
                        result = #_items - math.ceil(touchPoint.y / cellSize.height) + 1
                    else
                        result = math.ceil(touchPoint.y / cellSize.height)
                    end
                else
                    result = math.ceil(touchPoint.x / cellSize.width)
                end
            end
        end
        if result < 1 or result > _linearNumOfCells then result = _lastSelectedItemIndex end
        return result
    end
    --[[ 
    
    设置选中项效果
    
    **Parameters:**
    
    -   index:        int     选中项,若无可设置为nil或0
    -   force:        bool    是否强制设置

    --]]
    function _tableView:setSelectEffect(index, force)
        if _selectedParams then
            local bg = getSelectedBg()
            if bg then
                if not index or index <= 0 then
                    bg:setVisible(false)
                elseif _useLinearLayer or _lastSelectedItemIndex ~= index then
                    local cellSize = _linearCellSize or _cellSize
                    local cell = _tableView:cellAt(index)
                    if not force then
                        index = _tableView:getClickedIndex(index)
                    end
                    if 0 < index and index <= _linearNumOfCells then 
                        bg:setContentSize(CCSize(
                            cellSize.width + _selectedParams.offset,
                            cellSize.height + _selectedParams.offset 
                            ))
                        if _direction == M.TABLEVIEW_DIRECTION_VERTICAL then
                            --垂直方向时
                            local cellY 
                            if _useLinearLayer then
                                cellY = (#_items - index) * cellSize.height
                            else
                                cellY = cell:getPositionY()
                            end
                            display.align(bg, display.CENTER, cellSize.width / 2, cellY + cellSize.height / 2)
                        else
                            --水平方向时
                            local cellX 
                            if _useLinearLayer then
                                cellX = (index - 1) * cellSize.width
                            else
                                cellX = cell:getPositionX()
                            end
                            display.align(bg, display.CENTER, cellX + cellSize.width / 2, cellSize.height / 2)
                        end
                        
                        bg:setVisible(true)
                    else
                        bg:setVisible(false)
                    end
                end
            end
        end
        _lastSelectedItemIndex = index or -1
    end

    if _useLinearLayer then
        --[[ 
        
        取得Linear模式下的cell尺寸
        
        **Returns:**
        
        -   return Linear模式下的cell尺寸
        
        --]]
        function _tableView:getLinearCellSize(  )
            return _linearCellSize
        end
        --[[ 
        
        取得Linear模式下的cell个数
        
        **Returns:**
        
        -   return Linear模式下的cell个数
        
        --]]
        function _tableView:getLinearNumOfCells(  )
            return _linearNumOfCells
        end
        --[[ 
        
        创建LinearLayer

        **Returns:**
        
        -   return 创建的LinearLayer
        
        --]]
        function _tableView:createLinearLayer(  )   
            local layer = widget.newLinearLayer{
                            contentSize = _cellSize,
                            direction = iff(_direction == M.TABLEVIEW_DIRECTION_VERTICAL, 
                                DIRECTION_VERTICAL, DIRECTION_HORIZONTAL)
                        }
            display.align(layer, display.BOTTOM_LEFT, 0, 0)
            return layer
        end
        --[[ 
        
        取得当前cell中的LinearLayer,仅在linear模式下能使用
        
        **Returns:**
        
        -   return 当前cell中的LinearLayer
        
        --]]
        function _tableView:getCurCellLinear(  )
            if _cell then
                return _cell:getLinearLayer()
            end
            return nil
        end
    end
    
    --[[ 
    
    取得最后一次选中的项的下标
    
    **Returns:**
    
    -   return 最后一次选中的项的下标
    
    --]]
    function _tableView:getLastSelectedItemIndex(  )
        return _lastSelectedItemIndex
    end

    --[[ 
    
    滚动到指定下标的cell
    
    **Parameters:**
    
    -   index:        int      cell下标
    
    --]]
    function _tableView:scrollToItemIndex( index, time )
        local durationTime = time or 1
        -- print("durationTime", durationTime)
        if index < 1 then 
            Log.w(TAG, "newTable()->scrollToCell(index = %d), index out of range", index)
            index = 1 
        end
        
        local pos = self:getContentOffset()
        local lastPoint
        if self:getDirection() == M.TABLEVIEW_DIRECTION_HORIZONTAL then 
            if self:getContentSize().width < self:getViewSize().width then return end
            local cellWidth = _cellSize.width
            local offset = math.abs(pos.x) - (index - 1) * cellWidth
            if math.abs(pos.x + offset) > self:getContentSize().width -  self:getViewSize().width then 
                offset = -(self:getContentSize().width - self:getViewSize().width - math.abs(pos.x))
            end
            lastPoint = ccp(pos.x + offset, pos.y)
            self:setContentOffsetInDuration(lastPoint, durationTime)
        elseif self:getDirection() == M.TABLEVIEW_DIRECTION_VERTICAL then 
            if self:getContentSize().height < self:getViewSize().height then return end
            local cellHeight = _cellSize.height
            local offset = self:getViewSize().height + math.abs(pos.y) - (#_items - index + 1) * cellHeight
            if pos.y + offset > 0 then 
                offset = -pos.y
            end
            lastPoint = ccp(pos.x, pos.y + offset)
            self:setContentOffsetInDuration(lastPoint, durationTime)
        end

        if lastPoint then
            local function scrollToItemIndexCallback_( )
                _tableView:setContentOffset(lastPoint)
            end 
            util.delayCall(scrollToItemIndexCallback_, 1.5)
        end
    end

	init()
    _tableView._type_ = WIDGET_TYPE.Table
	return _tableView
end
--[[ 
 
 ## 创建GridView
 通常不设定x,y,ignoreAnchor,并且将Grid放在一Layer左下角,用Layer定位
 
 **Parameters:**
 
 -   params: 数组包含
             [必填项]
             CCSize                             contentSize             Grid的尺寸
             CCSize                             cellSize                每个cell的尺寸,有了这个和item的确数,上面的contentSize就可以不要了
             [选填项]
             table                              items                   显示用数据/entity
             fun(grid,index,item)               cellAtIndexListener     cell取得事件,用于创建或取得对应的cell,若定义了此项,则下面两项不用定义
             fun(grid,index,item)               createItemCellListener  cell创建监听,若cellAtIndexListener未定义则必须定义
             fun(grid,index,item,itemCell)      modifyItemCellListener  cell修改监听 
             boolean                            disableAutoCalcContentSize 禁用自动计算尺寸,默认为false,为true时一定要设置row和column
             enum                               direction               方向
                                                     widget.TABLEVIEW_DIRECTION_HORIZONTAL
                                                     widget.TABLEVIEW_DIRECTION_VERTICAL (default)
             int                                column                  列数,当direction为widget.TABLEVIEW_DIRECTION_VERTICAL时必须设定
             int                                row                     行数,当direction为widget.TABLEVIEW_DIRECTION_HORIZONTAL时必须设定
             fun(grid,index,item,x,y)           tapListener             点击cell回调事件
             table                              padding                 间隔
                                                    int         middleH     水平方向的cell间间距
                                                    int         middleV     垂直方向的cell间间距
                                                    int         top         与顶部的间距
                                                    int         bottom      与顶部的间距
                                                    int         left        与顶部的间距
                                                    int         right       与顶部的间距
             table                              selectedParams          选中效果(同9宫图参数)
                                                    string      file        背景图片名
                                                    CCRect      rect        图片尺寸矩形
                                                    int         offset      与其容器的间隔值
             CCNode                             container               判断点击有效的容器,非本grid,为容纳grid的容器
             bool                               isMultiTouches          是否允许多点触摸,默认为false
             float                              maxScale                最大缩放尺寸,通常isMultiTouches为true时才有用,默认为1
             float                              minScale                最小缩放尺寸,通常isMultiTouches为true时才有用,不设置会自动计算
             int                                x                       x坐标
             int                                y                       y坐标
             bool                               ignoreAnchor            是否忽略锚点(默认true)
             fun(grid, index, item, itemCell)   onCellEnter             cell进入时的处理事件
             fun(grid, index, item, itemCell)   onCellExit              cell离开时的处理事件
 --]]
function M.newGrid( params )
	--[[ ## local params --]]
    -- local _grid = CCLayerColor:create(ccc4(255,255,255,255))
	local _grid                     = display.newLayer()
	--#必填项#
	local _contentSize 				= params.contentSize
    local _cellSize                 = params.cellSize 
	--#选填项#
    local _items                    = nil
    local _direction                = params.direction or widget.TABLEVIEW_DIRECTION_VERTICAL
    local _column                   = params.column 
    local _row                      = params.row
    local _cellAtIndexListener      = params.cellAtIndexListener
    local _createItemCellListener   = params.createItemCellListener
    local _modifyItemCellListener   = params.modifyItemCellListener
    local _disableAutoCalcContentSize = params.disableAutoCalcContentSize
	-- local _x 						= params.x
	-- local _y 						= params.y
    -- local _ignoreAnchor             = ifnil(params.ignoreAnchor, true)
	local _tapListener 				= params.tapListener
    local _padding                  = params.padding or {}
    local _selectedParams           = params.selectedParams
    local _container                = params.container
    local _isMultiTouches           = params.isMultiTouches
    local _MAX_SCALE                = params.maxScale or 1
    local _MIN_SCALE                = params.minScale
    local _onCellEnter              = params.onCellEnter
    local _onCellExit               = params.onCellExit
	--#内部变量#
    local _paddingH                 = _padding.middleH or 0
    local _paddingV                 = _padding.middleV or 0
    local _paddingLeft              = _padding.left or 0
    local _paddingTop               = _padding.top or 0
    local _paddingRight             = _padding.right or 0
    local _paddingBottom            = _padding.bottom or 0
    --实际显示区域,扣除了padding
	local _realcontentSize
    --item对应的界面元素
	local _itemCells = {}
    --点击的item下标
    local _touchedItemIndex = -1
    local _lastSelectedItemIndex = -1
    local _selectedBg = nil

    local _oldOffset
    local _lastDistance 
    local _multiTouchesing = false
	
--[[ ## local fuction --]]
    local cellAtIndexListener_
    local isAvailableClick
    local onTouchBegan
    local onTouchMoved
    local onTouchEnded
    local onTouch
    local calcCellPosition
    local calcCellIndexInPoint
    local calcIndex
    local init
    local getSelectedBg
    local calcContentSize

    --[[ 
    
    取得选择中的背景
    
    **Returns:**
    
    -   return 选择中的背景
    
    --]]
    function getSelectedBg()
        if not _selectedBg then
            if _selectedParams then
                --设置下选中背景
                _selectedBg = M.newScale9(_selectedParams)
                _grid:addChild(_selectedBg, -100)
                _selectedBg:setVisible(false)
            end
        end
        return _selectedBg
    end
    --[[ 
    
    是否是有效的点击
    
    **Parameters:**
    
    -   x:        int     x
    -   y:        int     y
    
    **Returns:**
    
    -   return [bool, int], 是否有效, 点击的下标
    
    --]]
    function isAvailableClick(x, y)
        if _container then 
            if not util.isPointInNode(_container, x, y) then 
                return false, -1
            end
        end
        if util.isPointInNode(_grid, x, y) then 
            local row, column = calcCellIndexInPoint(x, y)
            local item = _grid:getItemCell(row, column)
            if item then
                touchedItemIndex = calcIndex(row, column)
                return util.isPointInNode(item, x, y), touchedItemIndex
            end
        end
        return false, -1
    end

    
    local function createTouches( touches )
        local result = {}
        if istable(touches) and #touches > 3 then 
            local i = 0
            while true do 
                if touches[i * 3 + 3] then
                    result[touches[i * 3 + 3] + 1] = ccp(touches[i * 3 + 1], touches[i * 3 + 2])
                    i = i + 1
                else
                    break
                end
            end
        end
        return result
    end
    local function calcDistance(touches)
        local touches_ = createTouches(touches)
        local distance = nil
        local newDistance = nil
        if #touches_ >= 2 then 
            if touches_[1] and touches_[2] then
                newDistance = ccpDistance(touches_[1], touches_[2])
            end 
            if newDistance then 
                if _lastDistance then
                    distance = math.round((newDistance - _lastDistance) * 100) / 100
                end
                _lastDistance = newDistance
            end
        end
        return distance, touches_[1], touches_[2]
    end
    -- local _mpoints
    function onTouchBegan(x, y, touches)
        if not util.isCustomTouchEnabled(_grid) or not _items or #_items == 0 then
            return false
        end
        if _multiTouchesing == true then _multiTouchesing = false end
        -- _mpoints = {}
        if not touches or #touches <= 3 then 
            -- 单点
            local clicked, touchedItemIndex = isAvailableClick(x, y)
            if clicked then
                _touchedItemIndex = touchedItemIndex
            end
            return clicked
        else
            -- 多点 根本到不了这
            return true
        end
    end
    local _touchWillEnable = nil
    function onTouchMoved(x, y, touches)
        -- dump(touches)
        if touches then 
            if #touches > 3 then
                _multiTouchesing = true
                if _grid.getScrollView and _grid:getScrollView():isTouchEnabled() then
                    _touchWillEnable = true
                    _grid:getScrollView():setTouchEnabled(false)
                end
                local distance, touch1, touch2 = calcDistance(touches)

                ---- 另一种缩放比例算法,在缩小时感觉有些慢,先放着
                -- if _mpoints.start1 == nil then _mpoints.start1 = touch1 end
                -- if _mpoints.start2 == nil then _mpoints.start2 = touch2 end
                -- if _mpoints.pre1 == nil then 
                --     _mpoints.pre1 = touch1 
                -- else
                --      _mpoints.pre1 = _mpoints.cur1
                -- end
                -- if _mpoints.pre2 == nil then 
                --     _mpoints.pre2 = touch2 
                -- else
                --     _mpoints.pre2 = _mpoints.cur2
                -- end
                -- _mpoints.cur1, _mpoints.cur2 = touch1, touch2
                -- local curD = ccpDistance(_mpoints.cur1, _mpoints.cur2)
                -- local preD = ccpDistance(_mpoints.pre1, _mpoints.pre2)
                -- local startD = ccpDistance(_mpoints.start1, _mpoints.start2)
                -- local factor = 0
                -- if curD > startD then 
                --     factor = math.exp(-0.25 * (curD / startD -1)) * (1 - preD /curD) + preD / curD
                -- else
                --     factor = math.exp(4 * (curD / startD - 1)) * (1 - preD / curD) + preD / curD
                -- end

                if distance and distance ~= 0 then
                    local scale = _grid:getScale()
                    local lastScale = scale
                    scale = scale + distance / 1000 -- scale * curD / preD * factor -- 
                    if distance > 0 then
                        -- 放大
                        if scale > _MAX_SCALE then
                            scale = _MAX_SCALE
                        end
                    else
                        -- 缩小
                        if not _MIN_SCALE and _grid.getScrollView then 
                            local gridSize = _grid:getContentSize()
                            local scrollViewSize = _grid:getScrollView():getViewSize()
                            _MIN_SCALE = math.max(scrollViewSize.width / gridSize.width, scrollViewSize.height / gridSize.height )
                        end
                        if scale < _MIN_SCALE then
                            scale = _MIN_SCALE
                        end
                    end
                    if scale == lastScale then return end
                    if _grid.getScrollView then
                        local oldOffset = _oldOffset or _grid:getScrollView():getContentOffset()
                        local point = ccpMidpoint(touch1, touch2) --touch1

                        local x0, y0 = point.x, point.y
                        local w1, h1 = _grid:getContentSize().width * lastScale, _grid:getContentSize().height * lastScale
                        local w2, h2 = _grid:getContentSize().width * scale, _grid:getContentSize().height * scale
                        local x1, y1 = math.abs(oldOffset.x) + x0, math.abs(oldOffset.y - h1) + y0 --point.x, point.y
                        local x2 = w2 * x1 / w1
                        local y2 = h2 * y1 / h1
                        local rx, ry = x0 - x2, -1 * math.abs(y0 - y2 + h2)
                        local newOffset = ccp(rx, ry)
                        _grid:setScale(scale)
                        _grid:getScrollView():updateInset()

                        _grid:getScrollView():setContentOffset(newOffset)
                        _oldOffset = newOffset
                        -- print(string.format("OLD OFFSET: [%f, %f]\nNEW OFFSET: [%f, %f]\nOLD SIZE: [%d, %d]\nNEW SIZE:[%d, %d]\nx,y: [%d, %d]"
                        --     , oldOffset.x, oldOffset.y
                        --     , _grid:getScrollView():getContentOffset().x, _grid:getScrollView():getContentOffset().y
                        --     , w1, h1
                        --     , w2, h2
                        --     , rx, ry
                        --     ))
                    end
                end
            else
                _multiTouchesing = false
                if _grid.getScrollView then
                    _oldOffset = _grid:getScrollView():getContentOffset()
                end
            end
        end
    end

    function onTouchEnded(x, y, touches)
        if not touches or #touches <= 3 then 
            -- 单点
            if _grid:isMoved() ~= true and not _multiTouchesing then 
                local clicked, touchedItemIndex = isAvailableClick(x, y)
                if clicked and touchedItemIndex > 0 and touchedItemIndex == _touchedItemIndex then
                    --和按下时是同一个item,这下算是点击了
                    _grid:setSelectEffect(touchedItemIndex)
                    if _tapListener then
                        _tapListener(_grid, touchedItemIndex, _grid:itemAt(touchedItemIndex), x, y)
                    end
                    _touchedItemIndex = -1
                end
            end
        end
        if _grid.getScrollView and _touchWillEnable then 
            _grid:getScrollView():setTouchEnabled(true)
            _lastDistance = nil
            _touchWillEnable = nil
        end
    end

    function cellAtIndexListener_( grid, index, item )
        local itemCell = grid:cellAt(index)
        local item = item or grid:itemAt(index)
        if item then
            if not itemCell then
                --printf("%s itemGridCellAtIndex CREATE ITEMCELL %d", TAG, index)
                --创建一个itemCell
                itemCell = _createItemCellListener(grid, index, item)
                grid:addItemCell(itemCell, index)
            else
                if _modifyItemCellListener then
                    --修改对应的itemCell
                    _modifyItemCellListener(grid, index, item, itemCell)
                else
                    -- 实在没有...只好重新再创建次了,贱人啊
                    itemCell:removeSelf()
                    itemCell = _createItemCellListener(grid, index, item)
                    grid:addItemCell(itemCell, index)
                end
            end
        end
    end

	function init( )
        Log.a(_contentSize or _cellSize, M.TAG, "newGrid() invalid params.contentSize ")
        if _disableAutoCalcContentSize then 
            Log.a(isnumber(_column) and isnumber(_row) and _column > 0 and _row > 0, M.TAG, "newGrid() invalid params.column or params.row, must > 0")
        end
        if _direction == widget.TABLEVIEW_DIRECTION_VERTICAL then 
            Log.a(isnumber(_column) and _column > 0, M.TAG, "newGrid() invalid params.column")
        else
            Log.a(isnumber(_row) and _row > 0, M.TAG, "newGrid() invalid params.row")
        end
        Log.a(_cellAtIndexListener or _createItemCellListener, M.TAG, 
            "newGrid() must be set one of params.cellAtIndexListener and params.cellAtIndexListener")

        if not _cellSize and _row > 0 and _column > 0 and _contentSize then 
            -- _cellSize = CCSize((_contentSize.width - _paddingLeft - _paddingRight) / _column, _contentSize.height / _row)
            local cellW = (_contentSize.width - (_column - 1) * _paddingH - _paddingLeft - _paddingRight) / _column
            local cellH = (_contentSize.height - (_row - 1) * _paddingV - _paddingTop - _paddingBottom) / _row
            _cellSize = CCSize(cellW, cellH)
        end

        if not params.ignoreAnchor then params.ignoreAnchor = true end
		M.initView(_grid, params)
        if not _cellAtIndexListener then _cellAtIndexListener = cellAtIndexListener_ end
        _grid:enableTouch(true)

        if params.items then 
            _grid:setItems(params.items)
        end
	end
    function calcContentSize(  )
        if not _disableAutoCalcContentSize then 
            -- 启用自动计算size时进行相应计算
            if _direction == widget.TABLEVIEW_DIRECTION_VERTICAL then 
                _row = math.ceil(#_items / _column)
            else 
                _column = math.ceil(#_items / _row)
            end
            if _cellSize then 
                local w = _cellSize.width * _column + (_column - 1) * _paddingH + _paddingLeft + _paddingRight
                local h = _cellSize.height * _row + (_row - 1) * _paddingV + _paddingTop + _paddingBottom
                if not _contentSize or not(_contentSize.width == w and _contentSize.height == h) then
                    _contentSize = CCSize( w, h )  
                    _grid:setContentSize(_contentSize)
                end
            end
            --实际存放元素的空间
            _realcontentSize = CCSize(
                                   _contentSize.width - _paddingLeft - _paddingRight,
                                   _contentSize.height - _paddingTop - _paddingBottom
                                   )
        else
            -- 不允许自动计算,即row和column被固定下来
            if not _realcontentSize then 
                if not _contentSize and _cellSize then 
                    local w = _cellSize.width * _column + (_column - 1) * _paddingH + _paddingLeft + _paddingRight
                    local h = _cellSize.height * _row + (_row - 1) * _paddingV + _paddingTop + _paddingBottom
                    -- if not(_contentSize.width == w and _contentSize.height == h) then
                        _contentSize = CCSize( w, h )  
                        _grid:setContentSize(_contentSize)
                    -- end
                end
                --实际存放元素的空间
                _realcontentSize = CCSize(
                                       _contentSize.width - _paddingLeft - _paddingRight,
                                       _contentSize.height - _paddingTop - _paddingBottom
                                       )
            end
        end
        
    end
    --[[ 
	 
	 计算指定位置的坐标
	 
	 **Parameters:**
	 
	 -   row:        int     行号,若column没设置则表示index
	 
	 -   column:     int     列号
	 
	 **Returns:**
	 
	 -   指定位置对应的坐标
	 
	 --]]
    function calcCellPosition( row, column )
        if column then
            --定义了row和column则直接使用
            local w, h = _realcontentSize.width, _realcontentSize.height
            local cellW, cellH = w / _column, h / _row
            local x, y = column * cellW - cellW / 2, h - row * cellH + cellH / 2
            --做下相应的偏移
            -- if _column == 1 and _direction == widget.TABLEVIEW_DIRECTION_VERTICAL then 
            --     return x + _paddingLeft + (params.contentSize.width - _contentSize.width), y + _paddingBottom + (params.contentSize.height - _contentSize.height)
            -- else
                return x + _paddingLeft, y + _paddingBottom
            -- end
            
		else
            --未定义column,把row当index使用,由其计算row和column
            return calcCellPosition(math.ceil(row / _column), (row - 1) % _column + 1)
        end
    end
    --[[ 
    
    根据指定的点计算出点所在的行列
    
    **Parameters:**
    
    -   point:        Point     指定的点
    
    **Returns:**
    
    -   return 点所在的行列(row, column)
    
    --]]
    function calcCellIndexInPoint( x, y )
        -- local x, y = point.x, point.y
        local touchPoint = _grid:convertToNodeSpace(ccp(x, y))
        local px, py = touchPoint.x - _paddingLeft, touchPoint.y - _paddingBottom
        local w, h = _realcontentSize.width, _realcontentSize.height
        local cellW, cellH = w / _column, h / _row
        local row = _row - math.ceil(py / cellH) + 1
        local column = math.ceil(px / cellW)
        return row, column
    end
	
    --[[ 
	 
	 计算指定位置的下标
	 
	 **Parameters:**
	 
	 -   row:        int     行号,若column没设置则表示index
	 
	 -   column:     int     列号
	 
	 **Returns:**
	 
	 -   指定位置对应的下标
	 
	 --]]
    function calcIndex( row, column )
        local index = -1
        if column then
            --定义了row和column则计算出index
            index = (row - 1) * _column + column
		else
            --未定义column,把row当index使用
            index = row
        end
        return index
    end
--[[ ## public fuction --]]
    --[[ 
    
    在多点环境下初始scale
    
    **Parameters:**
    
    -   scale:        float      初始scale
    
    --]]
    function _grid:initScale4Multi( scale )
        if _isMultiTouches == true then 
            _grid:setScale(scale)
            _scale = scale
            if _grid.getScrollView then 
                -- _grid:getScrollView():setContentOffset(CCPointZero)
                _grid:getScrollView():updateInset()
                util.autoFixScrollView( _grid:getScrollView())
            end
        end
    end
    -- 获取cell坐标位置
	function _grid:getCellPosition( row, column )
        return calcCellPosition( row, column )
    end
    --[[ 
	 
	 插入item entity到指定位置
	 
	 **Parameters:**
	 
	 -   item:   table       item的entity对象
	 
	 -   row:    int         行号,若column没设置则表示index
	 
	 -   column: int         列号
	 
	 --]]
    function _grid:insertItem( item, row, column )
        if item then
            local index = calcIndex(row, column)
            if index > 0 then
                --TODO:看看要不要把所有的往后推...
                _items[index] = item
            end
        end
    end
    --[[ 
    
    设置item的entity
    
    **Parameters:**
    
    -   items:        table     item应对元素的entity
    
    --]]
    function _grid:setItems( items )
        _items = nil
        _items = items
        _itemCells = {}
        calcContentSize(  )
        self:removeAllChildrenWithCleanup(true)
        self:refresh()
    end	
    --[[ 
	 
	 取得指定位置的item entity
	 
	 **Parameters:**
	 
	 -   row:    int         行号,若column没设置则表示index
	 
	 -   column: int         列号
	 
	 **Returns:**
	 
	 -   指定位置的item entity,无则为nil
	 
	 --]]

	function _grid:itemAt( row, column )
		local index = calcIndex(row, column)
		if _items and 0 < index and index <= #_items then
			--存在了,取出
			return _items[index]
		end
		return nil
	end
    _grid.getItem = _grid.itemAt
	
    --[[ 
	 
	 添加item cell到指定位置
	 
	 **Parameters:**
	 
	 -   item:   table       item的entity对象
	 
	 -   row:    int         行号,若column没设置则表示index
	 
	 -   column: int         列号
	 
	 **Returns:**
	 
	 -   是否添加成功
	 
	 --]]
	function _grid:addItemCell( itemCell, row,  column )
        if itemCell then
            local index = calcIndex(row, column)
            if index > 0 then
        		_itemCells[index] = itemCell
        		itemCell:setTag(index)
                itemCell:ignoreAnchorPointForPosition(false)
        		itemCell:setPosition(calcCellPosition(index))
                if _onCellEnter or _onCellExit then 
                    itemCell:registerNodeEvent()
                    if _onCellEnter then 
                        local onEnter = itemCell.onEnter
                        itemCell.onEnter = function ( self )
                            onEnter(self)
                            _onCellEnter(_grid, index, _grid:itemAt(index), itemCell)
                        end
                    end
                    if _onCellExit then 
                        local onExit = itemCell.onExit
                        itemCell.onExit = function ( self )
                            onExit(self)
                            _onCellExit(_grid, index, _grid:itemAt(index), itemCell)
                        end
                    end
                end
                self:addChild(itemCell)
                return true
            end
        end
        return false
	end
	
    --[[ 
	 
	 取得指定位置的item cell
	 
	 **Parameters:**
	 
	 -   row:    int         行号,若column没设置则表示index
	 
	 -   column: int         列号
	 
	 **Returns:**
	 
	 -   指定位置的item cell,无则为nil
	 
	 --]]
	function _grid:cellAt( row,  column )
        local index = calcIndex(row, column)
		if _itemCells and 0 < index and index <= #_itemCells then
			--存在了,取出
			return _itemCells[index]
		end
		return nil
	end
    _grid.getItemCell = _grid.cellAt

    function _grid:getSelectedItemIndex()
        return _lastSelectedItemIndex;
    end
	
    --[[ 
	 
	 更新指定位置的item cell
	 
	 **Parameters:**
	 
	 -   row:    int         行号,若column没设置则表示index
	 
	 -   column: int         列号
	 
	 --]]
	function _grid:updateItem(row, column )
		local index = calcIndex(row, column)
		if 0 < index and index <= (_row * _column) then
			_cellAtIndexListener(self, index, self:itemAt(index))
		end
	end
	
    --[[ 
	 
	 刷新所有cell
	 
	 --]]
	function _grid:refresh()
		for row = 1, _row do
			for column = 1, _column do
				self:updateItem(row, column)
			end
		end
        -- CCBCenter.changeAllLayerTouchPriorityToLast(_grid)
	end
    --[[ 
    
    为指定下标设置选中效果
    
    **Parameters:**
    
    -   index:        int     选中项的下标
    
    --]]
    function _grid:setSelectEffect(index)
        if _selectedParams then
            local bg = getSelectedBg()
            if bg then
                if not index or index <= 0 then
                    bg:setVisible(false)
                elseif _lastSelectedItemIndex ~= index then
                    local size = _grid:getItemCell(index):getContentSize()
                    bg:setContentSize(CCSize(size.width + (_selectedParams.offset or 20), size.height + (_selectedParams.offset or 20)))
                    bg:setPosition(calcCellPosition(index))
                    bg:setVisible(true)
                end
            end
        end
        _lastSelectedItemIndex = index or -1
    end
    --[[ 
    
    设置是否启用touch
    
    **Parameters:**
    
    -   enable:        bool     是否启用touch
    
    --]]
    function _grid:enableTouch( enable )
        if enable then
            -- _grid:addTouchEventListener(onTouch, false, 0, false)
            -- _grid:setTouchEnabled(true)
            widget.bindTouchEvent{
                layer = _grid,
                onTouchBegan = onTouchBegan,
                onTouchMoved = onTouchMoved, 
                onTouchEnded = onTouchEnded,
                -- onTouchCancelled = onTouchCancelled,
                clickListener = clickListener,
                isMultiTouches = _isMultiTouches,
            }
        else
            _grid:setTouchEnabled(false)
        end
    end
    --[[ 
    
    取得单元尺寸

    **Returns:**
    
    -   return 单元尺寸
    
    --]]
    function _grid:getCellSize(  )
        return _cellSize
    end
    --[[ 
    
    设置点击判定用容器node,通常是收纳grid的容器
    
    **Parameters:**
    
    -   node :        CCNode       点击判定用容器node,通常是收纳grid的容器
    
    --]]
    function _grid:setContainerNode( node )
        _container = node
    end

    function _grid:getRealcontentSize(  )
        return _realcontentSize
    end

    function _grid:getRows(  )
        return _row
    end

    function _grid:getColumns( )
        return _column
    end

    function _grid:calcRowColumnByIndex( index )
        index = index - 1
        local row = math.floor(index / _column)
        return row + 1, index - row * _column + 1
    end
	
	init()
    _grid._type_ = WIDGET_TYPE.Grid
	return _grid
end
--[[ 
 
 ## 创建Toast
 
 **Parameters:**
 
 -   params: 数组包含
             [必填项]
             string      text                    内容文本

             [选填项]
             int         width                   宽度
             int         padding                 与框的四周间距, 默认为20
             int         middlePadding           若使用图片,表示图片与文字的间距, 默认为5
             table       labelParams             文字样式参数
             int         delayTime               延迟时间
             string      image                   图标文件
             table       bgParams                背景参数
             bool        enableTouchClose        是否允许点击后关闭,默认为false

 --]]
function M.newToast ( params )
--[=[ ## local params --]=]
    local _layer                    = display.newLayer()
    --#必填项#
    local _text                     = params.text
    --#选填项#
    local _width                    = params.width or display.width
    local _padding                  = params.padding or 20
    local _middlePadding            = params.middlePadding or 5
    local _labelParams              = params.labelParams or {}
    local _image                    = params.image
    local _bgParams                 = params.bgParams
    local _delayTime                = params.delayTime or 5
    local _enableTouchClose         = params.enableTouchClose
    
    --#内部参数#
    local closeToast
    local calcContainerSize

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        Log.a(_text, M.TAG, "newToast() invalid params.text")
        --使用定义的参数
        M.initView(_layer, params)
        _layer:setCascadeOpacityEnabled(true)
        --图片
        local image
        if _image then 
            image = M.newSprite{ 
                file = _image, 
            }
        end
        --文字
        _labelParams.text = _text
        local label = ui.newTTFLabel(_labelParams)
        -- 准备下容器
        local contentSize, labelSize = calcContainerSize(label, image)
        _labelParams.dimensions = labelSize
        _labelParams.textValign = _labelParams.textValign or ui.TEXT_ALIGN_LEFT
        label = ui.newTTFLabel(_labelParams)
        _layer.label = label
        _layer:setContentSize(contentSize)
        local labelX = _padding
        if image then 
            local imageNode = M.newLayer{
                    -- color = ccc4(255,0,0,255),
                    contentSize = CCSize(image:getContentSize().width, contentSize.height - 2 * _padding),
                    x = _padding,
                    y = _padding
                }
            util.alignNode(image, imageNode:getContentSize(), display.TOP_CENTER)
            imageNode:addChild(image)
            _layer:addChild(imageNode)
            labelX = _middlePadding + _padding + image:getContentSize().width
        end
        local labelNode = M.newLayer{
            -- color = ccc4(0,255,0,255),
            contentSize = CCSize(labelSize.width, contentSize.height - 2 * _padding),
            x = labelX,
            y = _padding,
        }
        util.alignNode(label, labelNode:getContentSize(), display.TOP_LEFT)
        labelNode:addChild(label)
        _layer:addChild(labelNode)
        --背景
        if _bgParams then 
            _bgParams.contentSize = contentSize
            _bgParams.ignoreAnchor = true
            local bg = M.newScale9(_bgParams)
            _layer:addChild(bg, -999 )
            _layer.bg = bg
        end
        --事件绑定
        M.bindTouchEvent{
            layer = _layer,
            clickListener = iff( _enableTouchClose, closeToast, nil) ,
            priority = -9999999,
            swallowsTouches = true,
        }
        _layer:performWithDelay(closeToast, _delayTime)
        --自动添加到当前scene
        local scene = display.getRunningScene()
        if scene then 
            util.alignNode(_layer, scene:getContentSize(), display.CENTER)
            scene:addChild(_layer, 99999)   
        end 
    end

    function calcContainerSize( label, image )
        local labelSize = label:getContentSize()
        local labelWidth, labelHeight, line, layerWidth, layerHeight
        labelWidth = _width - 2 * _padding
        if image then 
            -- 有图片时要加上其内容
            labelWidth = labelWidth - image:getContentSize().width - _middlePadding
        end
        if labelSize.width > labelWidth then 
            line = math.ceil(labelSize.width / labelWidth)
        else
            labelWidth = labelSize.width
            line = 1
        end
        labelHeight = line * (labelSize.height - 0)
        if image then 
            -- 有图时
            if image:getContentSize().height < labelHeight then 
                -- 图片高度比文字的还要小,整体以文字高度为准
                layerHeight = 2 * _padding + labelHeight
            else
                -- 图片高度比文字的来得大,当然是以图片的为准
                layerHeight = 2 * _padding + image:getContentSize().height
            end
            layerWidth = labelWidth + image:getContentSize().width + 2 * _padding + _middlePadding
        else
            layerHeight = labelHeight + 2 * _padding
            layerWidth = labelWidth + 2 * _padding
        end
        return CCSize(layerWidth, layerHeight), CCSize(labelWidth, labelHeight)
    end

    function closeToast()
        transition.execute(_layer, CCMoveBy:create(0.5, ccp(0, 50)), {
            easing = "CCEaseInOut",
            onComplete = function ( ... )
                _layer:removeSelf(true)
            end })
        transition.fadeOut(_layer.bg, {time = 0.5})
        transition.fadeOut(_layer.label, {time = 0.5})
    end
--[=[ ## public fuction --]=]
    
    init()
    _layer._type_ = WIDGET_TYPE.Toast 
    return _layer
end

--[[ 

## 九宫图

**Parameters:**

-   params: 数组包含
            [必填项]
            string      file                图片名
            [选填项]
            CCRect      rect                9宫图区域
            CCSize      contentSize         尺寸
            CCRect      capInsets           9宫图拉伸区域
            int         x                   x坐标
            int         y                   y坐标
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newScale9 ( params )
    local _layer
--[=[ ## local params --]=]
    --#必填项#
    local _file               = params.file
    --#选填项#
    local _rect               = params.rect or CCRectZero 
    local _capInsets          = params.capInsets or CCRectZero
    local _contentSize        = params.contentSize or _rect.size
    local _x                  = params.x
    local _y                  = params.y
    local _ignoreAnchor       = params.ignoreAnchor or false
    --#内部参数#

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        Log.a(_file, M.TAG, "newScale9() invalid params.file")
        --Log.a(_rect, M.TAG, "newScale9() invalid params.rect")
        --使用定义的参数
        if string.byte(_file) == 35 then -- first char is #
            _layer = CCScale9Sprite:createWithSpriteFrameName(string.sub(_file, 2), _capInsets)
        else
            _layer = CCScale9Sprite:create(_file, _rect, _capInsets)
        end
        M.initView(_layer, params)
    end
--[=[ ## public fuction --]=]
    
    init()
    _layer._type_ = WIDGET_TYPE.Scale9
    return _layer
end

--[[ 

## 下拉框

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize         尺寸
            table       items               item必须含有text属性
            [选填项]
            int         x                   x坐标
            int         y                   y坐标
            bool        ignoreAnchor        是否忽略锚点(默认false)
            table       defaulTitle         默认提示的文字样式
            int         selected            选中
            function    listener            方法包含item
            int         line                最多显示行数
            table       buttonParams        按钮参数,若下面的button没定义则此项必须定义
            Button      button              按钮,若定义,上面的buttonParams将无效

--]]

function M.newDropDownBox ( params )
    local _layer = display.newLayer()
--[=[ ## local params --]=]
    --#必填项#
    local _contentSize        = params.contentSize
    local _items              = params.items
    --#选填项#
    local _defaulTitleParams         = params.defaultTitleParams --or {text = "Choose one"}
    local _selected                  = params.selected 
    local _listener                  = params.listener
    local _line                      = params.line or 3
    local _buttonParams              = params.buttonParams
    local _button                    = params.button
    local _x                         = params.x
    local _y                         = params.y
    local _ignoreAnchor              = params.ignoreAnchor or false
    local _titleBgParams             = params.titleBgParams
    local _itemsBgParams             = params.itemsBgParams
    
    --#内部参数#
    local onTouch
    local onItemTouch
    local tableItemAtIndex
    local openBox
    local closeBox
    local _isOpen = false
    local _tableView
    local _title 
    local _itemLayer
    local _buttonOldListener
    local _maskLayer
    local _isFirst = true
    local _isChoosed
--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        Log.a(_contentSize, M.TAG, "newDropDownBox() invalid params.contentSize")
        Log.a(_items, M.TAG, "newDropDownBox() invalid params.items")
        --使用定义的参数
        if _defaulTitleParams then
            _title = ui.newTTFLabel(_defaulTitleParams)
        else
            _title = ui.newTTFLabel(_items[1])
            _selected = _selected or 1
        end
        if _selected then
            _title:setString(_items[_selected].text)
        end
        if not _button then
            Log.a(_buttonParams, M.TAG, "newDropDownBox() invalid params.buttonParams")
            _button = M.newButton(_buttonParams)
        end
        _buttonOldListener = _button.getListener()
        _button.setListener(onTouch)
        _button.setTriggerContainer(_layer)
        local offsetHeight = _line == #_items and 0 or _contentSize.height / 2
        local itemsLayerSize = CCSize(_contentSize.width, _contentSize.height * _line + offsetHeight)
        _tableView = M.newTable{
                                contentSize         = itemsLayerSize,
                                cellSize            = _contentSize,
                                numOfCells          = #_items,
                                cellAtIndexListener = tableItemAtIndex,
                                tapListener         = onItemTouch,
                                verticalFillOrder   = M.TABLEVIEW_FILL_TOP_DOWN,
        }
        _button:setPosition(_contentSize.width - _button:getContentSize().width  ,
                            _contentSize.height - _button:getContentSize().height)
        --制作item容纳层
        local function onItemLayerTouchEvent(eventType, x, y)
            if eventType == M.TOUCHBEGAN then
                return _isChoosed or util.isPointInNode(_itemLayer, x, y) 
            end
        end
        _itemLayer = M.newMaskLayer( nil, onItemLayerTouchEvent )
        -- _itemLayer = display.newColorLayer()
        _itemLayer:setContentSize(itemsLayerSize)
        _itemLayer:setPosition(0, -_contentSize.height * _line - offsetHeight)
        _itemLayer:addChild(_tableView)

        M.initView(_layer, params)

        local cx, cy = util.centerNode(_layer)
        display.align(_title, display.CENTER, cx - _button:getContentSize().width, cy)
        _layer:addChild(_button, 4)
        _layer:addChild(_itemLayer, 3)
        _layer:addChild(_title, 2)
        --布标题框背景
        if _titleBgParams then
            local titleBg = widget.newScale9(_titleBgParams)
            titleBg:setContentSize(_layer:getContentSize())
            titleBg:setPosition(util.centerNode(_layer))
            _layer:addChild(titleBg, -100, -100)
        end
        --布下拉框背景
        if _itemsBgParams then
            local itemsBg = widget.newScale9(_itemsBgParams)
            itemsBg:setContentSize(itemsLayerSize)
            itemsBg:setPosition(util.centerNode(_itemLayer))
            _itemLayer:addChild(itemsBg, -100, -100)
        end
        --制作touch阻挡层
        local function onTouchEvent(eventType, x, y)
            if eventType == M.TOUCHBEGAN then
                return not ( util.isPointInNode(_layer, x, y) or util.isPointInNode(_itemLayer, x, y) )
            elseif eventType == M.TOUCHENDED then
                _layer.closeBox()
            end
        end
        _maskLayer = M.newMaskLayer( nil, onTouchEvent )
        _layer:addChild(_maskLayer, -2)

        --初始操作
        _layer.closeBox(true)
    end

    function tableItemAtIndex( tableView, idx )
        local cell = tableView:dequeueCell()
        if  not cell then
            cell = tableView.newCell()
            local label = ui.newTTFLabel(_items[idx])
            label:setTag(111)
            display.align(label, display.CENTER, _contentSize.width / 2 - _button:getContentSize().width, _contentSize.height / 2)
            cell:addChild(label)
        else
            local label = cell:getChildByTag(111)
            label = util.toCCLabel(label)
            label:setString(_items[idx].text)
        end
        

        return cell
    end

    function onItemTouch( tableView, cell )
        if _isChoosed then return end
        _isChoosed = true
        local index = cell:getIdx() + 1
        if _selected ~= index then
            _title:setString(_items[index].text)
            _selected = index
            if _listener then
                _listener(cell, _items[index])
            end
        end
        --延迟下第二次打开才能用
        util.delayCall( _layer.closeBox, 0.01 )
    end

    function onTouch( button )
        if _isOpen then
            _layer.closeBox()
        else
            _layer.openBox()
        end
    end

--[=[ ## public fuction --]=]

    --[[ 
    
    打开Box
    
    --]]

    function _layer.openBox(force)
        if not _isOpen or force then
            _itemLayer:setVisible(true)
            _itemLayer:setTouchEnabled(true)
            _tableView:setVisible(true)
            _maskLayer:setVisible(true)
            _maskLayer:setTouchEnabled(true)
            _isOpen = true
            _isChoosed = nil
            if _isFirst then
                _isFirst = nil
                -- CCBCenter.changeAllLayerTouchPriority( _itemLayer, _itemLayer.PRIORITY, true )
            end
            if _buttonOldListener then
                _buttonOldListener(_button, _isOpen)
            end
        end
    end

    --[[ 
    
    关闭Box
    
    --]]

    function _layer.closeBox(force)
        if _isOpen or force then
            _itemLayer:setVisible(false)
            _itemLayer:setTouchEnabled(false)
            _tableView:setVisible(false)
            _maskLayer:setVisible(false)
            _maskLayer:setTouchEnabled(false)
            _isOpen = false

            if _buttonOldListener then
                _buttonOldListener(_button, _isOpen)
            end
        end
    end
    --[[ 
    
    取得打开状态
    
    **Returns:**
    
    -   return 打开状态
    
    --]]
    function _layer.isOpen(  )
        return _isOpen
    end
    
    --[[ 
    
    取得选中item
    
    **Returns:**
    
    -   选中的item,没有选中的时候返回nil
    
    --]]

    function _layer.getSelectItem( )
        if _selected then
            return _items[_selected]
        end
    end

    init()
    _layer._type_ = WIDGET_TYPE.DropDownBox

        
    return _layer
end

M.LAYER_POI_PRE = 0
M.LAYER_POI_CUR = 1
M.LAYER_POI_NEX = 2
--[[ 

## 旋转的滑动的地图背景

**Parameters:**

-   params: 数组包含
            [必填项]
            int         totalPages          总页数
            fun(curId,fillId,layerPoi,layer)   listener    用于填充layer,curId是当前中间页面,fillId,
                                                    是返回的layer需要填充的id,
                                                    layerPoi是用于填充的layer的相对位置包含三种,
                                                    1. M.LAYER_POI_PRE 前一页
                                                    2. M.LAYER_POI_CUR 中间页
                                                    3. M.LAYER_POI_NEX 后一页
            [选填项]
            CCRect      rect                显示尺寸,默认为全屏
            int         curPage             初始页面
            bool        loop                是否开启循环模式,即到首尾相连
            bool        enable              一个锁暂时还用不到
            int         x                   x坐标
            int         y                   y坐标
            bool        ignoreAnchor        是否忽略锚点(默认false)
            table       indicatorParams     页数指示器参数
                                            string      normal          普通图片
                                            string      selected        选中时的图片
                                            bool        anim            是否使用动画,默认为true
                                            float       time            动画时间,单位为s,默认为0.2
                                            float       height          指示器所占用的高度,默认为图片的2倍
                                            int         padding         间距,默认为0
--]]
function M.newScrollLayer ( params )
    local _rect = params.rect
    if not _rect then _rect = CCRect(0, 0, 0, 0) end
    local _layer              = display.newClippingRegionNode(_rect) -- display.newLayer()
    local _mapContainer       = widget.newLayer()
--[=[ ## local params --]=]
    --#必填项#
    local _totalPages         = params.totalPages
    local _listener           = params.listener
    --#选填项#
    local _contentSize        = _rect.size --params.contentSize or CCDirector:sharedDirector():getWinSize()
    local _curPage            = params.curPage or 1
    local _clickListener      = params.clickListener
    local _enable             = ifnil(params.enable, true)
    local _x                  = params.x
    local _y                  = params.y
    local _ignoreAnchor       = params.ignoreAnchor or false
    local _indicatorParams    = params.indicatorParams
    local _width              = _contentSize.width
    local _height             = _contentSize.height
    local _loop               = ifnil(params.loop, true)
    --#内部参数#
    -- local _map                = CCLayerColor:create(ccc4(0,255,255,255))
    -- local _preMap             = CCLayerColor:create(ccc4(255,0,255,255))
    -- local _nextMap            = CCLayerColor:create(ccc4(255,255,0,255))
    local _map                = display.newLayer()
    local _preMap             = display.newLayer()
    local _nextMap            = display.newLayer()
    local _touchDelta         = 30
    local _layerIndex
    local _indicatorHeight    = 0
    local _indicatorLayer
    local _animTime           = 0.2
    local onTouchBegan
    local onTouchMoved
    local onTouchEnded
    local onTouch
    local moveToPage
    local createIndicators
    local _TouchCurPoint
    local _TouchDownPoint
    local _TouchUpPoint
--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        Log.a(_listener, M.TAG, "newScrollLayer() invalid params.listener")
        Log.a(_totalPages, M.TAG, "newScrollLayer() invalid params.totalPages")
        --使用定义的参数
        local pageContentSize = _contentSize
        if _indicatorParams then 
            if _indicatorParams.time then  _animTime = _indicatorParams.time end
            _indicatorHeight = _indicatorParams.height
            _indicatorParams.anim = ifnil(_indicatorParams.anim, true)
            pageContentSize = CCSizeAdd( pageContentSize, 0, -_indicatorHeight )
            createIndicators()
        end
        _preMap:setContentSize(pageContentSize)
        _map:setContentSize(pageContentSize)
        _nextMap:setContentSize(pageContentSize)
        -- 容器定位
        display.align(_preMap, display.BOTTOM_LEFT, 0 , 0)
        display.align(_map, display.BOTTOM_LEFT, _map:getContentSize().width, 0)
        display.align(_nextMap, display.BOTTOM_LEFT, _map:getContentSize().width * 2, 0)

        _layer:addChild(_mapContainer)
        _mapContainer:addChild(_preMap)
        _mapContainer:addChild(_map)
        _mapContainer:addChild(_nextMap)

        -- _layer.setTotalPages(_curPage)

        _mapContainer:addTouchEventListener(onTouch)
        _mapContainer:setTouchEnabled(true)
        
        -- M.initView(_layer, params)
        _mapContainer:setContentSize(CCSize(_map:getContentSize().width * 3, _map:getContentSize().height))
        -- 初始化容器
        _layer:jumpToPage( _curPage )
    end
    function createIndicators( ... )
        if _indicatorParams then 
            _indicatorLayer = display.newLayer()
            _indicatorLayer.indicators_ = {}
            _indicatorLayer:setContentSize(CCSize(_width, _indicatorParams.height))
            _layer:addChild(_indicatorLayer)
            local linearLayer = widget.newLinearLayer{ padding = _indicatorParams.padding }
            linearLayer:setAutoRefresh(false)
            for i = 1, _totalPages do
                local normalIndicator = display.newSprite(_indicatorParams.normal)
                linearLayer:addChild(normalIndicator)
                _indicatorLayer.indicators_[i] = normalIndicator
            end
            linearLayer:refresh()
            local indicator = display.newSprite(_indicatorParams.selected)
            linearLayer:addChild(indicator)
            indicator:setPosition( _indicatorLayer.indicators_[1]:getPosition() )
            _indicatorLayer.indicator_ = indicator
            _indicatorLayer:addChild(linearLayer)
            util.alignNode(linearLayer, _indicatorLayer:getContentSize(), display.CENTER)
        end
    end
    function onTouchBegan(x, y)
        if not util.isCustomTouchEnabled(_layer) or not util.isPointInNode( _layer, x, y ) then
            -- print("cow, 被返回了!")
            return false
        end
        -- print("onTouchBegan:x = " .. x .. "y=" .. y)
        _TouchDownPoint = _mapContainer:convertToNodeSpace(ccp(x, y)) -- CCDirector:sharedDirector():convertToGL(ccp(x, y))
        _TouchCurPoint = _TouchDownPoint
        return true
    end

    function onTouchMoved(x, y)
        -- print("onTouchMoved:x = " .. x .. "y=" .. y)
        local touchPoint = _mapContainer:convertToNodeSpace(ccp(x, y)) -- CCDirector:sharedDirector():convertToGL(ccp(x, y))
        if _totalPages > 1 then
            if _loop or ((_mapContainer:getPositionX() + touchPoint.x - _TouchCurPoint.x) >= (-_width * 2) 
                and (_mapContainer:getPositionX() + touchPoint.x - _TouchCurPoint.x) < 0) then
                _mapContainer:setPosition(_mapContainer:getPositionX() + touchPoint.x - _TouchCurPoint.x, _indicatorHeight)
            end
        else
            -- if (_mapContainer:getPositionX() + touchPoint.x - _TouchCurPoint.x) >= 0 
            --     and (_mapContainer:getPositionX() + touchPoint.x - _TouchCurPoint.x) < 0 then
            --     _mapContainer:setPosition(_mapContainer:getPositionX() + touchPoint.x - _TouchCurPoint.x, _indicatorHeight)
            -- end
        end
        _TouchCurPoint = touchPoint
    end

    function onTouchEnded(x, y)
        -- print("onTouchEnded:x = " .. x .. "y=" .. y)
        local isChange, changePoi = false, M.LAYER_POI_CUR
         _TouchUpPoint = _mapContainer:convertToNodeSpace(ccp(x, y)) -- CCDirector:sharedDirector():convertToGL(ccp(x, y))
        -- 计算按下和抬起的偏移量
        local offset = (_TouchUpPoint.x - _TouchDownPoint.x) * (_TouchUpPoint.x - _TouchDownPoint.x) 
                     + (_TouchUpPoint.y - _TouchDownPoint.y) * (_TouchUpPoint.y - _TouchDownPoint.y)
    
        if offset < _touchDelta then
            -- print("当前为点击")
            -- 点击
            -- 发送Click消息
            if _clickListener then
                _clickListener(_TouchUpPoint.x, _TouchUpPoint.y)
            end
        else 
            -- 滑动结束
            -- local scrollOffset = _mapContainer:getPositionX() - _layerIndex * (- _width)
            local scrollOffset = _TouchUpPoint.x - _TouchDownPoint.x
            local DIS_MOVE = _width / 4
            if DIS_MOVE > 80 then DIS_MOVE = 80 end
            -- if scrollOffset > _width / 3 then
            if scrollOffset > DIS_MOVE then 
                -- 上一页
                if _loop then 
                    _curPage = _curPage - 1
                    if _curPage < 1 then _curPage = _totalPages end
                    isChange = true
                    _layerIndex = 0
                else
                    if _curPage > 1 and _curPage < _totalPages then
                        _curPage = _curPage - 1
                        _layerIndex = 0
                        -- printf("上一页")
                        isChange = true
                    elseif _curPage > 1 then
                        _curPage = _curPage - 1
                        _layerIndex = 1
                        -- printf("上一页")
                        isChange = true
                    end
                end
               
            -- elseif scrollOffset < -_width / 3 then
            elseif scrollOffset < -DIS_MOVE then
                -- 下一页
                if _loop then
                    _curPage = _curPage + 1
                    if _curPage > _totalPages then _curPage = 1 end
                    isChange = true
                    _layerIndex = 2
                else
                    if _curPage < _totalPages and _curPage > 1 then
                        _curPage = _curPage + 1
                        _layerIndex = 2
                        -- printf("下一页")
                        isChange = true
                    elseif _curPage < _totalPages then
                        _curPage = _curPage + 1
                        _layerIndex = 1
                        -- printf("下一页")
                        isChange = true
                    end
                end
            end
        end
        _TouchDownPoint = nil
        _TouchUpPoint = nil
        _TouchCurPoint = nil
        -- 执行跳转动画
        if _totalPages > 1 then 
            moveToPage(isChange, changePoi)
        end
    end

    local function prepareDataForLoop( ... )
        local prePage, nextPage = _curPage - 1, _curPage + 1
        if prePage < 1 then prePage = _totalPages end
        if nextPage > _totalPages then nextPage = 1 end
        _listener(_curPage, prePage, M.LAYER_POI_PRE, _preMap)
        _listener(_curPage, _curPage, M.LAYER_POI_CUR, _map)
        _listener(_curPage, nextPage, M.LAYER_POI_NEX, _nextMap)
        _layerIndex = 1
        _layer.refresh()
    end

    function moveToPage( isChange, changePoi)
        -- print("@@@@@@@@@@@@@@@@ moveToPage", isChange, _curPage)
        local function prepareData( ... )
            if isChange then
                if not _loop then 
                    if _curPage == 1 then
                        _layerIndex = 0
                        if _totalPages == 2 then 
                            -- print("类型", 1)
                            -- print("当前第", _curPage)
                            -- print("刷新第", _curPage)
                            -- print("刷新第", _curPage + 1)
                            _listener(_curPage, _curPage, M.LAYER_POI_CUR, _preMap)
                            _listener(_curPage, _curPage + 1, M.LAYER_POI_NEX, _map)
                        end
                    elseif _curPage == _totalPages then
                        _layerIndex = 2
                        if _totalPages == 2 then 
                            -- print("类型", 2)
                            -- print("当前第", _curPage)
                            -- print("刷新第", _curPage)
                            -- print("刷新第", _curPage - 1)
                            _listener(_curPage, _curPage, M.LAYER_POI_CUR, _nextMap)
                            _listener(_curPage, _curPage - 1, M.LAYER_POI_PRE, _map)
                        end
                    else
                        _layerIndex = 1
                        if _curPage == 2 and changePoi == M.LAYER_POI_NEX then 
                            -- print("类型", 3)
                            -- print("当前第", _curPage)
                            -- print("刷新第", _curPage + 1)
                            -- _listener(_curPage, _curPage + 1, M.LAYER_POI_NEX, _nextMap)
                        elseif _curPage == _totalPages - 1 and changePoi == M.LAYER_POI_PRE then 
                            -- print("类型", 4)
                            -- print("当前第", _curPage)
                            -- print("刷新第", _curPage - 1)
                            -- _listener(_curPage, _curPage - 1, M.LAYER_POI_PRE, _preMap)
                        else
                            -- print("类型", 5)
                            -- print("当前第", _curPage)
                            -- print("刷新第", _curPage - 1)
                            -- print("刷新第", _curPage)
                            -- print("刷新第", _curPage + 1)
                            _listener(_curPage, _curPage - 1, M.LAYER_POI_PRE, _preMap)
                            _listener(_curPage, _curPage, M.LAYER_POI_CUR, _map)
                            _listener(_curPage, _curPage + 1, M.LAYER_POI_NEX, _nextMap)
                        end
                    end
                    _layer.refresh()
                else
                    prepareDataForLoop()
                end
            end
        end
        -- print("-_layerIndex * _width,", -_layerIndex * _width)
        -- print("_layerIndex", _layerIndex)
        -- print("_width", _width)
        transition.moveTo(_mapContainer, { 
            time = _animTime, 
            x = -_layerIndex * _width,
            onComplete = function ( ... )
                _layer:performWithDelay(function ( ... )
                    prepareData()
                end, 0.01)
            end,
            })
        if isChange then 
            if _indicatorParams then 
                transition.stopTarget( _indicatorLayer.indicator_ )
                if _indicatorParams.anim then 
                    transition.moveTo(_indicatorLayer.indicator_, {
                        x = _indicatorLayer.indicators_[_curPage]:getPositionX(), 
                        -- y = _indicatorLayer.indicators_[_curPage]:getPositionY(),
                        time = _animTime })
                else
                    _indicatorLayer.indicator_:setPosition(_indicatorLayer.indicators_[_curPage]:getPosition())
                end
            end
        end
        
        -- printf("当前页:".._curPage.."实际页面:".._layerIndex)
    end



    function onTouch(eventType, x, y)
        -- print("eventType:" .. eventType .. ":x = " .. x .. "y=" .. y)
        if _enable then
            if eventType == M.TOUCHBEGAN then
                return onTouchBegan(x, y)
            elseif eventType == M.TOUCHMOVED then
                return onTouchMoved(x, y)
            else
                return onTouchEnded(x, y)
            end
        end    
    end
--[=[ ## public fuction --]=]
    function _layer.refresh( )
        if _layerIndex == 0 then
            _mapContainer:setPosition( 0, _indicatorHeight)
        elseif _layerIndex == 1 then
            _mapContainer:setPosition( - _width, _indicatorHeight)
        elseif _layerIndex == 2 then
            _mapContainer:setPosition( - _width * 2, _indicatorHeight)
        end
    end

    function _layer:jumpToPage( pageIndex )
        -- print("jumpToPage..pageIndex", pageIndex)
        -- print("jumpToPage..pageIndex", pageIndex)
        -- print("jumpToPage..pageIndex", pageIndex)
        _curPage = pageIndex or 1
        if _curPage < 1 then 
            _curPage = 1 
        elseif _curPage > _totalPages then 
            _curPage = _totalPages
        end
        if not _loop then 
            if _curPage == 1 then
                _layerIndex = 0
                _listener(_curPage, _curPage, M.LAYER_POI_CUR, _preMap)
                _listener(_curPage, _curPage + 1, M.LAYER_POI_NEX, _map)
                if _totalPages > 2 then 
                    _listener(_curPage, _curPage + 2, M.LAYER_POI_NEX, _nextMap)
                elseif _totalPages == 2 then
                    _listener(_curPage, _curPage + 1, M.LAYER_POI_NEX, _nextMap)
                end
                _mapContainer:setPosition(0, _indicatorHeight)
            elseif _curPage == _totalPages then
                _layerIndex = 2
                if _totalPages > 2 then 
                    _listener(_curPage, _curPage - 2, M.LAYER_POI_PRE, _preMap)
                elseif _totalPages == 2 then
                    _listener(_curPage, _curPage - 1, M.LAYER_POI_PRE, _preMap)
                end
                _listener(_curPage, _curPage, M.LAYER_POI_CUR, _nextMap)
                _listener(_curPage, _curPage - 1, M.LAYER_POI_PRE, _map)
                _mapContainer:setPosition( - _width * 2, _indicatorHeight)
            else
                _layerIndex = 1
                _listener(_curPage, _curPage - 1, M.LAYER_POI_PRE, _preMap)
                _listener(_curPage, _curPage, M.LAYER_POI_CUR, _map)
                _listener(_curPage, _curPage + 1, M.LAYER_POI_NEX, _nextMap)
                _mapContainer:setPosition( - _width, _indicatorHeight)
            end
        else
            prepareDataForLoop()
        end
        if _indicatorParams then
            _indicatorLayer.indicator_:setPosition(_indicatorLayer.indicators_[_curPage]:getPosition())
        end
    end
    --[[ 
    
    设置总页数
    
    **Parameters:**
    
    -   total:        int     总页数

    --]]
    function _layer.setTotalPages( total )
        if total < 1 then total = 1 end
        if _totalPages ~= total then 
            _totalPages = total
            if _indicatorLayer then
                _indicatorLayer:removeSelf()
                _indicatorLayer = nil
            end
            createIndicators()
        end
    end
    function _layer.getTotalPages( )
        return _totalPages
    end
    -- 取当前页node
    function _layer.getCurPage( )
        if _loop then 
            return _map
        else
            if _curPage == 1 then
                return _preMap
            elseif _curPage == _totalPages then
                return _nextMap 
            else
                return _map
            end
        end
    end
    -- 取前页node
    function _layer.getPrePage( )
        if 1 == _totalPages then
            return nil
        end
        if _loop then 
            return _preMap
        else
            if _curPage == 1 then
                return nil
            elseif _curPage == _totalPages then
                return _map 
            else
                return _preMap
            end
        end
    end
    -- 取下页node
    function _layer.getNextPage( )
        if 1 == _totalPages then
            return nil
        end
        if _loop then 
            return _nextMap
        else
            if _curPage == 1 then
                return _map
            elseif _curPage == _totalPages then
                return nil 
            else
                return _nextMap
            end
        end
    end
    -- 取当前页码
    function _layer.getCurPageIndex(  )
        return _curPage
    end
    -- 取上页页码
    function _layer.getPrePageIndex( )
        if 1 == _totalPages and _loop then
            return nil
        end
        if 1 == _curPage then
            if _loop then 
                return _totalPages
            else
                return nil
            end
        else
            return _curPage - 1
        end
    end
    -- 取下页页码
    function _layer.getNextPageIndex( )
        if 1 == _totalPages and _loop then
            return nil
        end
        if _totalPages == _curPage then
            if _loop then 
                return 1
            else
                return nil
            end
        else
            return _curPage + 1
        end
    end
    init()
    _layer._type_ = WIDGET_TYPE.ScrollLayer
    return _layer
end

--[[ 

## name

**Parameters:**

-   params: 数组包含
            [必填项]
            table       pageParams          页面的参数,见newScrollLayer,不需要其中的
            table       gridParams          每页grid的参数,见newGrid,不需要其中的contentSize, items
            table       items               数据
            [选填项]
            int         x                   x坐标
            int         y                   y坐标
            int         tag                 node的tag值
            bool        ignoreAnchor        是否忽略锚点(默认false)
e.g.

local function createItemCellListener( grid,index,item )
    local gridCell = widget.newLayer{ contentSize = grid:getCellSize() }

    gridCell:ignoreAnchorPointForPosition(false)
    return gridCell
end

local function modifyItemCellListener( grid,index,item,itemCell )
    ...
end

local function tapListener( grid,index,item )
    ...
end

local scrollLayer = widget.newPageGrid{
    pageParams = {
            rect = CCRect(0, 0, layer:getContentSize().width, layer:getContentSize().height),
            totalPages = 4,
            contentSize = layer:getContentSize(),
            ignoreAnchor = true,
            indicatorParams = {
                normal = "goods/goods_Indicator.png",
                selected = "goods/goods_Indicator_selected.png",
                height = 30,
                padding = 10,
            },
    },
    gridParams = {
            disableAutoCalcContentSize = true,
            row = 3, 
            column = 6, 
            createItemCellListener = createItemCellListener,
            modifyItemCellListener = modifyItemCellListener,
            tapListener = tapListener,
    },
}
--]]
function M.newPageGrid ( params )
--[=[ ## local params --]=]
    --#必填项#
    local _pageParams               = params.pageParams
    local _gridParams               = params.gridParams
    local _items                    = params.items or {}
    -- local _contentSize                = params.contentSize
    --#选填项#
    local _items                    = params.items
    local _x                         = params.x
    local _y                         = params.y
    local _tag                         = params.tag
    local _ignoreAnchor                = params.ignoreAnchor or false
    
    --#内部参数#
    local _layer                     = display.newLayer()
    local _scrollLayer 
    local _highLightPage
    local _highLightIndex
    local _hightLighItem

--[=[ ## local fuction --]=]
    local function createPageGrid_( curId, fillId, layerPoi, layer )
        _gridParams.contentSize = layer:getContentSize()
        _gridParams.container = _layer
        local grid = M.newGrid(_gridParams)
        layer:removeAllChildrenWithCleanup(true)
        layer:addChild(grid)
        layer.GRID_ = grid
        grid.getPageGrid = function (  )
            return _layer
        end
    end
    local function refreshPageItems_( page, pageIndex )
        if page and page.GRID_ and 0 < pageIndex then 
            local pageItemsCount = _gridParams.row * _gridParams.column 
            local items = table.slice( _items, 1 + pageItemsCount * (pageIndex - 1), pageItemsCount )
            if #items < pageItemsCount then 
                for i = #items + 1, pageItemsCount do
                    items[i] = {}
                    -- items[i] = { index = (pageIndex - 1) * pageItemsCount + i}
                    -- print(items[i].index)
                end
            end
            page.GRID_:setItems(items)
        end
    end
    local function pageTurnListener_( curId, fillId, layerPoi, layer )
        -- print("@@@@@@@@@@@@ pageTurnListener_", curId, fillId, layerPoi, layer )
        if not layer.GRID_ then 
            createPageGrid_(curId, fillId, layerPoi, layer)
        end
        refreshPageItems_( layer, fillId )
    end

    local function init( )
        _layer:setContentSize(_pageParams.rect.size)
        _pageParams.listener = pageTurnListener_
        _scrollLayer = M.newScrollLayer(_pageParams)
        _layer:addChild(_scrollLayer)
    end
    --[[ 
    
    高亮控制
    
    **Parameters:**
    
    -   pageIndex:        int     要高亮的cell所处页数
    -   index:            int     所处页数中的下标
    -   highlight:        bool    是否高亮,默认为不高亮
    
    **Returns:**
    
    -   return 高亮设置是否成功
    
    --]]
    local function highLight_( pageIndex, index, highlight )
        local pageGrid = nil
        if pageIndex == _scrollLayer.getPrePageIndex() then 
            pageGrid = _scrollLayer.getPrePage().GRID_
        elseif pageIndex == _scrollLayer.getCurPageIndex() then 
            pageGrid = _scrollLayer.getCurPage().GRID_
        elseif pageIndex == _scrollLayer.getNextPageIndex() then 
            pageGrid = _scrollLayer.getNextPage().GRID_
        end
        
        local pageItemsCount = _gridParams.row * _gridParams.column 
        local item = _items[pageItemsCount * (pageIndex - 1) + index]
        if item then 
            _hightLighItem = iff(highlight, item, nil)
            if pageGrid then 
                pageGrid:updateItem(index)
            end
            return true
        end
        return false
    end

    
--[=[ ## public fuction --]=]
    function _layer:refreshPage( pagePoi )
        local page, pageIndex
        if pagePoi == M.LAYER_POI_PRE then 
            page, pageIndex = _scrollLayer.getPrePage(), _scrollLayer.getPrePageIndex()
        elseif pagePoi == M.LAYER_POI_CUR then
            page, pageIndex = _scrollLayer.getCurPage(), _scrollLayer.getCurPageIndex()
        else
            page, pageIndex = _scrollLayer.getNextPage(), _scrollLayer.getNextPageIndex()
        end
        refreshPageItems_( page, pageIndex )
    end
    --[[ 
    
    刷新页面(只刷新三页)
    
    **Parameters:**
    
    **Returns:**
    
    -   return return
    
    --]]
    function _layer:refresh()
        self:refreshPage( M.LAYER_POI_PRE )
        self:refreshPage( M.LAYER_POI_CUR )
        self:refreshPage( M.LAYER_POI_NEX )
    end
    --[[ 
    
    设置数据并刷新
    
    **Parameters:**
    
    -   items:        			table    数据
    -   isChangeTotalPage:      bool     是否根据数据改变页数,默认为false
    -   isHoldPageIndex:        bool     是否保持当前页数,不保持则直接回到第一页,默认为false
    
    --]]
    function _layer:setItems( items, isChangeTotalPage, isHoldPageIndex )
        _items = items
        _highLightPage, _highLightIndex, _hightLighItem = nil, nil, nil
        if isChangeTotalPage then 
            _scrollLayer.setTotalPages(math.ceil(#_items / (_gridParams.row * _gridParams.column)))
        end
        if isHoldPageIndex then 
            self:jumpToPage(_scrollLayer.getCurPageIndex())
        else
            self:jumpToPage(1)
        end
        self:highLight()
    end
    --[[ 
    
    取得滚动页控件
    
    **Parameters:**
    
    **Returns:**
    
    -   return return
    
    --]]
    function _layer:getScrollLayer(  )
        return _scrollLayer
    end
    --[[ 
    
    跳转到指定页面
    
    **Parameters:**
    
    -   int:        pageIndex     指定页面下标
    
    **Returns:**
    
    -   return return
    
    --]]
    function _layer:jumpToPage( pageIndex )
        _scrollLayer:jumpToPage( pageIndex )
    end

    --[[ 
    
    高亮控制
    
    **Parameters:**
    
    -   pageIndex:        int     要高亮的cell所处页数,当index未nil时候表示下标
    -   index:            int     所处页数中的下标
    
    **Returns:**
    
    -   return return
    
    --]]
    function _layer:highLight( pageIndex, index )
        if pageIndex and not index then 
            local pageItemsCount = _gridParams.row * _gridParams.column 
            local totalIndex = pageIndex
            pageIndex = math.ceil(totalIndex / pageItemsCount)
            index = totalIndex - math.floor(totalIndex / pageItemsCount) * pageItemsCount
            if index == 0 then 
                index = pageItemsCount
            end
        end
        -- 实际页数 getCurPageIndex
        if _highLightPage and _highLightIndex then 
            -- cow,有高亮过,先灭了你
            if _highLightPage == pageIndex and _highLightIndex == index then 
                -- nnnd不要这时候才跟我说是一样的好不,gun
                return 
            end
            -- 灭~~~
            highLight_(_highLightPage, _highLightIndex, nil)
        end
        if pageIndex and index then 
            -- 小进行下高亮操作
            if highLight_(pageIndex, index, true) then 
                _highLightPage, _highLightIndex = pageIndex, index
            end
        else
            _highLightPage, _highLightIndex, _hightLighItem = nil, nil, nil
        end
    end
    --[[ 
    
    判断是否高亮
    
    **Parameters:**
    
    -   item:        table[entity]       待判定是否高亮的item
    
    **Returns:**
    
    -   return 是否高亮
    
    --]]
    function _layer:isHighLight( item )
        return item and _hightLighItem == item
    end

    init()
    _layer._type_ = WIDGET_TYPE.PageGrid
    return _layer
end

--[[ 

## 创建ScrollView
添加进ScrollView中的Layer默认ignoreAnchor=false, anchorPoint=ccp(0, 0),且不可修改
ScrollView的定位都以左下角定义
!!!如果改变了内部尺寸,一定要调用updateInset(),花客,查了这么久
!!!util.enableCustomTouch(layer, false),加入无效,began还未进行处理
**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize            尺寸
            CCNode      contentLayer           置于ScrollView中的内容Layer,若此项不定义则必须定义contentParams
            table       contentParams          置于ScrollView中的内容的生成属性,若contentLayer定义了则此项无效
            [选填项]
            bool        bounceable             是否启用边界阻尼效果,默认true
            bool        clippingToBounds       是否启用界面截取效果,默认true
            enum        direction                   方向
                             widget.TABLEVIEW_DIRECTION_NONE
                             widget.TABLEVIEW_DIRECTION_HORIZONTAL
                             widget.TABLEVIEW_DIRECTION_VERTICAL 
                             widget.TABLEVIEW_DIRECTION_BOTH(default)
            int         x                   x坐标
            int         y                   y坐标
            int         tag                 node的tag值
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newScrollView ( params )
    
--[=[ ## local params --]=]
    --#必填项#
    local _contentSize              = params.contentSize
    local _contentLayer             = params.contentLayer
    local _contentParams            = params.contentParams
    --#选填项#
    local _bounceable               = ifnil(params.bounceable, true)
    local _clippingToBounds         = ifnil(params.clippingToBounds, true)
    local _direction                = params.direction or widget.TABLEVIEW_DIRECTION_BOTH
    local _x                        = params.x
    local _y                        = params.y
    local _tag                      = params.tag
    --local _ignoreAnchor             = ifnil(params.ignoreAnchor, true)
    
    --#内部参数#
    local _layer                    = CCLayerExtend.extend(LuaScrollView:create(_contentSize, _contentLayer))

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        Log.a(_contentSize, M.TAG, "newScrollView() invalid params.contentSize")
        -- Log.a(_contentLayer or istable(_contentParams), M.TAG, 
        --     "newScrollView() invalid params.contentLayer or params.contentParams")
        --使用定义的参数
        if not params.ignoreAnchor then params.ignoreAnchor = true end
        M.initView(_layer, params)
        if isnil(_contentLayer) and istable(_contentParams) then
            --居然是要拼的内容,好吧,拼~~~
            if not _contentParams.contentSize then
                _contentParams.contentSize = _contentSize
            end
            if _contentParams.contentSize.height == 0 then
                _contentParams.contentSize = CCSizeAdd(_contentParams.contentSize, 0, _contentSize.height)
            end
            if not _contentParams.ignoreAnchor then
                _contentParams.ignoreAnchor = true
            end
            _contentLayer = ViewFactory.create(_contentParams)
            if _contentLayer then
                _layer:setContainer(_contentLayer)
            end
        end
        if _contentLayer then
            _contentLayer:ignoreAnchorPointForPosition(false)
            _contentLayer:setAnchorPoint( CCPointZero )
        end
        _layer:setDirection(_direction)
        if _layer:isBounceable() ~= _bounceable then 
            _layer:setBounceable(_bounceable) 
        end
        if _layer:isClippingToBounds() ~= _clippingToBounds then 
            _layer:setClippingToBounds(_clippingToBounds) 
        end
        _layer:updateInset()

        local function onTouchBeganHandler_( x, y )
            _layer.clickedPoint = ccp(x, y)
            return util.isCustomTouchEnabled(_layer) and (
                _layer:getContentSize().width > _layer:getViewSize().width 
                or _layer:getContentSize().height > _layer:getViewSize().height)
        end
        _layer:registerScriptTouchBeganHandler(onTouchBeganHandler_)

        
        -- _layer:registerScriptRelocateHandler(_layer.onRelocateHandler)
    end
--[=[ ## public fuction --]=]
    function _layer:getContentView(  )
        return _contentLayer
    end

    init()
    _layer._type_ = WIDGET_TYPE.ScrollView
    return _layer
end
--[[ 

## 创建ScrollGrid
**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize            尺寸
            table       contentParams          grid属性
                    [必填项]
                     CCSize                             contentSize             Grid的尺寸
                     [选填项]
                     table                              items                   显示用数据/entity
                     fun(grid,index,item)               cellAtIndexListener     cell取得事件,用于创建或取得对应的cell,若定义了此项,则下面两项不用定义
                     fun(grid,index,item)               createItemCellListener  cell创建监听,若cellAtIndexListener未定义则必须定义
                     fun(grid,index,item,itemCell)      modifyItemCellListener  cell修改监听 
                     boolean                            disableAutoCalcContentSize 禁用自动计算尺寸,默认为false,为true时一定要设置row和column
                     int                                column                  列数,当direction为widget.TABLEVIEW_DIRECTION_VERTICAL时必须设定
                     int                                row                     行数,当direction为widget.TABLEVIEW_DIRECTION_HORIZONTAL时必须设定
                     fun(grid,index,item)               tapListener             点击cell回调事件
                     table                              padding                 间隔
                                                            int         middleH     水平方向的cell间间距
                                                            int         middleV     垂直方向的cell间间距
                                                            int         top         与顶部的间距
                                                            int         bottom      与顶部的间距
                                                            int         left        与顶部的间距
                                                            int         right       与顶部的间距
                     table                              selectedParams          选中效果(同9宫图参数)
                                                            string      file        背景图片名
                                                            CCRect      rect        图片尺寸矩形
                                                            int         offset      与其容器的间隔值
            [选填项]
            bool        needRelocate           停止时是否进行定位调整,默认为true
            bool        bounceable             是否启用边界阻尼效果,默认true
            bool        clippingToBounds       是否启用界面截取效果,默认true
            enum        direction                   方向
                             widget.TABLEVIEW_DIRECTION_HORIZONTAL
                             widget.TABLEVIEW_DIRECTION_VERTICAL (default)
                             widget.TABLEVIEW_DIRECTION_BOTH
            int         x                   x坐标
            int         y                   y坐标
            int         tag                 node的tag值
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newScrollGrid ( params )
    if not params.direction then params.direction = widget.TABLEVIEW_DIRECTION_VERTICAL end
    params.contentParams.direction = params.direction
    params.contentParams.x = nil 
    params.contentParams.y = nil
    params.contentParams.ignoreAnchor = nil
    params.contentParams.viewType = WIDGET_TYPE.Grid 

    params.contentParams.padding = params.contentParams.padding or {}
    local _middlePadding = params.contentParams.padding.middleH or params.contentParams.padding.middleV or 0

    local _layer = widget.newScrollView(params)

    local _grid = _layer:getContentView()

    _grid:setContainerNode(_layer)
    _grid.getScrollView = function ( ... )
        return _layer
    end
    --[[ 
    
    取得grid
    
    **Returns:**
    
    -   return grid
    
    --]]
    function _layer:getGrid(  )
        return _grid
    end
    --[[ 
    
    设置items数据
    
    **Parameters:**
    
    -   items:        待填充的数据      desc
    
    
    --]]
    function _layer:setItems( items , offset)
        self:getGrid():setItems(items)
        self:updateInset()
        util.autoFixScrollView(self)
        if isnumber(offset) then 
            self:setContentOffset(ccp(0, offset))
        end
    end
    --[[ 
    
    滚动到指定下标的cell
    
    **Parameters:**
    
    -   index:        int      cell下标
    
    --]]
    local function scrollToCell( index, duration )
        duration = duration or 1
        if index < 1 then 
            Log.w(TAG, "newScrollGrid()->scrollToCell(index = %d), index out of range", index)
            index = 1 
        end
        local function moveToCell_( position )
            if duration > 0 then
                _layer:setContentOffsetInDuration(position, duration)
            else
                _layer:setContentOffset(position)
            end
        end
        local pos = _layer:getContentOffset()
        if _layer:getDirection() == M.TABLEVIEW_DIRECTION_HORIZONTAL then 
            if _layer:getContentSize().width < _layer:getViewSize().width then return end
            local cellWidth = _layer:getGrid():getCellSize().width + _middlePadding
            local offset = math.abs(pos.x) - (index - 1) * cellWidth
            if math.abs(pos.x + offset) > _layer:getContentSize().width -  _layer:getViewSize().width then 
                offset = -(_layer:getContentSize().width - _layer:getViewSize().width - math.abs(pos.x))
            end
            moveToCell_(ccp(pos.x + offset, pos.y), duration)
        elseif _layer:getDirection() == M.TABLEVIEW_DIRECTION_VERTICAL then 
            if _layer:getContentSize().height < _layer:getViewSize().height then return end
            local cellHeight = _layer:getGrid():getCellSize().height + _middlePadding
            local offset = _layer:getViewSize().height + math.abs(pos.y) - (_layer:getGrid():getRows() - index + 1) * cellHeight
            if pos.y + offset > 0 then 
                offset = -pos.y
            end
            moveToCell_(ccp(pos.x, pos.y + offset))
        else
            if _layer:getContentSize().width < _layer:getViewSize().width 
                and _layer:getContentSize().height < _layer:getViewSize().height then 
                return 
            end

            local cellWidth = _layer:getGrid():getCellSize().width + _middlePadding
            local offset = math.abs(pos.x) - (index - 1) * cellWidth
            if math.abs(pos.x + offset) > _layer:getContentSize().width -  _layer:getViewSize().width then 
                offset = -(_layer:getContentSize().width - _layer:getViewSize().width - math.abs(pos.x))
            end
            local resultX = pos.x + offset

            local cellHeight = _layer:getGrid():getCellSize().height + _middlePadding
            local offset = _layer:getViewSize().height + math.abs(pos.y) - (_layer:getGrid():getRows() - index + 1) * cellHeight
            if pos.y + offset > 0 then 
                offset = -pos.y
            end
            local resultY = pos.y + offset

            moveToCell_(ccp(resultX, resultY ))
        end
    end
    if params.needRelocate ~= false then
        --[[ 
        
        停止停下后的位置调整
        
        **Parameters:**
        
        -   x:        float      停下时的x坐标值
        -   y:        float      停下时的y坐标值
        
        --]]
        local function onRelocateHandler_( x, y )
            if _layer:getDirection() == M.TABLEVIEW_DIRECTION_HORIZONTAL then 
                local cellWidth = _layer:getGrid():getCellSize().width + _middlePadding
                local index = math.ceil(math.abs(x) / cellWidth)
                local isleft = index - 0.5 < math.abs(x) / cellWidth
                local offset = math.abs(x) - (index - 1) * cellWidth
                if isleft then offset = offset - cellWidth end
                _layer:setContentOffsetInDuration(ccp(x + offset, y), 0.15)
            elseif _layer:getDirection() == M.TABLEVIEW_DIRECTION_VERTICAL then
                local cellHeight = _layer:getGrid():getCellSize().height + _middlePadding
                local yy = _layer:getViewSize().height + math.abs(y) 
                local index = math.ceil(yy / cellHeight)
                local isup = index - 0.5 > yy / cellHeight
                local offset = yy - index * cellHeight - (params.contentParams.padding.top or 0)
                if isup then offset = offset + cellHeight end
                _layer:setContentOffsetInDuration(ccp(x, y + offset ), 0.15)
            else
                local cellWidth = _layer:getGrid():getCellSize().width + _middlePadding
                local index = math.ceil(math.abs(x) / cellWidth)
                local isleft = index - 0.5 < math.abs(x) / cellWidth
                local offset = math.abs(x) - (index - 1) * cellWidth
                if isleft then offset = offset - cellWidth end
                local resultX = x + offset

                local cellHeight = _layer:getGrid():getCellSize().height + _middlePadding
                local yy = _layer:getViewSize().height + math.abs(y) 
                local index = math.ceil(yy / cellHeight)
                local isup = index - 0.5 > yy / cellHeight
                local offset = yy - index * cellHeight - (params.contentParams.padding.top or 0)
                if isup then offset = offset + cellHeight end
                local resultY = y + offset

                _layer:setContentOffsetInDuration(ccp(resultX, resultY ), 0.15)
            end
        end
        _layer:registerScriptRelocateHandler(onRelocateHandler_)
    end
    --[[ 
    
    滚动到指定行,用于仅一列的gird时
    
    **Parameters:**
    
    -   row:        int     指定行
    
    --]]
    function _layer:scrollToRow( row, duration )
        scrollToCell(row, duration)
    end
    --[[ 
    
    滚动到指定列,用于仅一行的gird时
    
    **Parameters:**
    
    -   column:        int     指定列
    
    --]]
    function _layer:scrollToColumn( column, duration )
        scrollToCell(column, duration)
    end
    --[[ 
    
    滚动到指定items下标的位置,根据滚动方向决定滚动方式
    
    **Parameters:**
    
    -   itemIndex:        int           item下标
    
    --]]
    function _layer:scrollToItemIndex( itemIndex, duration )
        local row, column = _layer:getGrid():calcRowColumnByIndex(itemIndex)
        if _layer:getDirection() == M.TABLEVIEW_DIRECTION_HORIZONTAL then 
            scrollToCell(column, duration)
        else
            scrollToCell(row, duration)
        end
    end

    _layer._type_ = WIDGET_TYPE.ScrollGrid 
    return _layer
end
--[[ 

创建带颜色的Layer

**Parameters:**

-   color:        ccc4     颜色,nil时则完全透明,等同于CCLayer

**Returns:**

-   return 带颜色的Layer

--]]
function M.newColorLayer(color)
    --return display.extendLayer(display.extendNode(CCLayerColor:create(color or ccc4(0,0,0,0))))
    return CCLayerExtend.extend(CCLayerColor:create(color or ccc4(0,0,0,0)))
end

--[[ 

## 线性布局容器

**Parameters:**

-   params: 数组包含
            [必填项]
            
            [选填项]
            enum        direction           方向
                                                DIRECTION_HORIZONTAL (default)
                                                DIRECTION_VERTICAL
            enum        verticalAlign       垂直排列
                                                VERTICAL_CENTER (default)
                                                VERTICAL_LETF
                                                VERTICAL_RIGHT
            enum        horizontalAlign     水平排列
                                                HORIZONTAL_CENTER (default)
                                                HORIZONTAL_TOP
                                                HORIZONTAL_BOTTOM
            enum        showGone            隐藏时的处理方式,是否占位
                                                SHOW_GONE (default)
                                                SHOW_INVISIBLE
            int         padding             间距,默认为0
            bool        autoRefresh         是否自动刷新,默认为true
            int         x                   x坐标
            int         y                   y坐标
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newLinearLayer ( params )
    local _layer              = CCLayerExtend.extend(LinearLayer:create())--display.extendLayer(display.extendNode(LinearLayer:create()))
--[=[ ## local params --]=]
    params = params or {}
    --#必填项#
    --#选填项#
    -- local _contentSize        = params.contentSize
    -- local _direction          = params.direction or DIRECTION_HORIZONTAL
    -- local _verticalAlign      = params.verticalAlign or VERTICAL_CENTER
    -- local _horizontalAlign    = params.horizontalAlign or HORIZONTAL_CENTER
    -- local _showGone           = params.showGone or SHOW_GONE
    -- local _padding            = params.padding or 0
    -- local _autoRefresh        = ifnil(params.autoRefresh, true)

    local _x                  = params.x
    local _y                  = params.y
    local _ignoreAnchor       = ifnil(params.ignoreAnchor, true)
    
    --#内部参数#

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        --Log.a(_contentSize, M.TAG, "newLinearLayer() invalid params.contentSize")
        --使用定义的参数
        M.initView(_layer, params)
        if _ignoreAnchor then _layer:ignoreAnchorPointForPosition(_ignoreAnchor) end

        if params.direction and params.direction ~= DIRECTION_HORIZONTAL then
            -- print("params.direction", params.direction)
            -- print("params.direction", DIRECTION_HORIZONTAL)
            _layer:setDirection(params.direction)
        end
        
        if params.verticalAlign and params.verticalAlign ~= VERTICAL_CENTER then
            _layer:setVerticalAlign(params.verticalAlign)
        end

        if params.horizontalAlign and params.horizontalAlign ~= HORIZONTAL_CENTER then
            _layer:setHorizontalAlign(params.horizontalAlign)
        end

        if params.showGone ~= nil and params.showGone ~= SHOW_GONE then
            _layer:setIsShowGone(params.showGone)
        end

        if params.padding and params.padding ~= 0 then
            _layer:setPadding(params.padding)
        end

        if params.autoRefresh ~= nil and params.autoRefresh ~= true then
            _layer:setAutoRefresh(params.autoRefresh)
        end
    end
--[=[ ## public fuction --]=]
    function _layer:alignV( padding, align, isShowGone )
        _layer:alignItemsVertically( padding, align, isShowGone )
    end

    function _layer:alignH( padding, align, isShowGone )
        _layer:alignItemsHorizontally( padding, align, isShowGone )
    end

    --function refresh()
    --function get/set Direction()
    --function get/set VerticalAlign()
    --function get/set HorizontalAlign()
    --function get/set IsShowGone()
    --function get/set Padding()
    --function get/set AutoRefresh()
    --function addChild( ... )

    init()
    _layer._type_ = WIDGET_TYPE.LinearLayer
    return _layer
end

--[[ 

## Layer

**Parameters:**

-   params: 数组包含
            [必填项]
            
            [选填项]
            CCSize      contentSize            尺寸
            ccc4        color               填充颜色
            int         x                   x坐标
            int         y                   y坐标
            int         tag                 node的tag值
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newLayer ( params )
--[=[ ## local params --]=]
    params = params or {}
    --#必填项#
    --#选填项#
    -- local _contentSize  = params.contentSize
    local _color        = params.color
    -- local _x            = params.x
    -- local _y            = params.y
    -- local _tag          = params.tag
    -- local _ignoreAnchor = params.ignoreAnchor
    
    --#内部参数#
    
    local _layer        
    if _color then 
        _layer = widget.newColorLayer(_color) 
    else
        _layer = display.newLayer()
    end

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        --使用定义的参数
        params.ignoreAnchor = ifnil(params.ignoreAnchor, true)
        M.initView(_layer, params)
    end
--[=[ ## public fuction --]=]
    
    init()
    _layer._type_ = WIDGET_TYPE.Layer
    return _layer
end

--[[ 

## Node

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize      contentSize            尺寸
            
            [选填项]
            int         x                   x坐标
            int         y                   y坐标
            int         tag                 node的tag值
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newNode ( params )
    local _layer        = display.newNode()
--[=[ ## local params --]=]
    --#必填项#
    local _contentSize  = params.contentSize
    --#选填项#
    local _x            = params.x
    local _y            = params.y
    local _tag          = params.tag
    local _ignoreAnchor = params.ignoreAnchor or false
    
    --#内部参数#

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        --使用定义的参数
        M.initView(_layer, params)
    end
--[=[ ## public fuction --]=]
    
    init()
    _layer._type_ = WIDGET_TYPE.Node
    return _layer
end

--[[ 

## Sprite

**Parameters:**

-   params: 数组包含
            [必填项]
            string      file                图片名称
            
            [选填项]
            int         x                   x坐标
            int         y                   y坐标
            int         tag                 node的tag值
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newSprite ( params )
--[=[ ## local params --]=]
    --#必填项#
    local _file         = params.file
    --#选填项#
    local _x            = params.x
    local _y            = params.y
    local _tag          = params.tag
    local _ignoreAnchor = params.ignoreAnchor or false
    
    --#内部参数#
    Log.a(_file, M.TAG, "newSprite() invalid params.file")
    local _layer        = display.newSprite(_file)
    
--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项

        --使用定义的参数
        M.initView(_layer, params)
    end
--[=[ ## public fuction --]=]
    
    init()
    _layer._type_ = WIDGET_TYPE.Sprite
    return _layer
end

--[[ 

## TileSprite

**Parameters:**

-   params: 数组包含
            [必填项]
            string      file                图片名称
            
            [选填项]
            CGRect      rect                平铺面积
            CCPoint     offset              位置偏移
            int         x                   x坐标
            int         y                   y坐标
            int         tag                 node的tag值
            bool        ignoreAnchor        是否忽略锚点(默认false)
--]]
function M.newTileSprite ( params )
--[=[ ## local params --]=]
    --#必填项#
    local _file         = params.file
    --#选填项#
    local _rect         = params.rect         or CCRectMake(0, 0, display.width, display.height)
    local _offset       = params.offset       or ccp(0, 0)
    local _ignoreAnchor = params.ignoreAnchor or false
    
    --#内部参数#
    Log.a(_file, M.TAG, "newTileSprite() invalid params.file")
    local _sprite       = CCSprite:create(_file, _rect)

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        if not _sprite then
            echoError("widget.newTileSprite() - create sprite failure, filename %s", tostring(_file))
            return
        end

        --使用定义的参数
        local tp = ccTexParams()
        tp.minFilter = 9729
        tp.magFilter = 9729
        tp.wrapS = 10497
        tp.wrapT = 10497
        _sprite:getTexture():setTexParameters(tp)

        display.align(_sprite, display.LEFT_BOTTOM, _offset.x, _offset.y)
        M.initView(_sprite, params)
    end
--[=[ ## public fuction --]=]
    
    init()
    -- _sprite._type_ = WIDGET_TYPE.TileSprite
    return _sprite
end
--[[ 

创建遮罩层

**Parameters:**

-   priority:           int                     优先级,默认为-990000,相当高啊
-   onTouchEvent:       fun(eventType,x,y)      touch事件,默认为全拦截
-   color:              ccc4/ccc3               层的背景色
-   clickListener:      fun(layer)              点击事件

**Returns:**

-   return 遮罩层

--]]
function M.newMaskLayer( priority, onTouchEvent, color, clickListener )
    local layer = widget.newColorLayer(color or COLOR_MASK)

    local function onTouch(eventType, x, y)
        if eventType == M.TOUCHBEGAN and util.isPointInNode(layer, x, y) then
            return true
        else
            return false
        end
    end
    
    -- M.bindTouchEvent{
    --     layer = layer,
    --     priority = priority or -990000,
    --     swallowsTouches = true,
    --     clickListener = clickListener,
    -- }
    layer.PRIORITY = priority or -990000
    layer:addTouchEventListener(onTouchEvent or onTouch, false, layer.PRIORITY, true)
    layer:setTouchEnabled(true)
    return layer
end



--[[ 

## 创建ScrollLinear,本类进行了bindTouchEvent和bindSelectEffect
切记:cell要保证ignoreAnchorPointForPosition(false)

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize                              contentSize             尺寸
            LinearLayer                         contentLayer            置于ScrollView中的内容LinearLayer,若此项不定义则必须定义contentParams
            table                               contentParams           置于ScrollView中的内容的生成属性,若contentLayer定义了则此项无效
            [选填项]
            fun(scrollLinear, index, item)      cellAtIndexListener     cell取得事件
            fun(scrollLinear, index, item)      tapListener             点击cell回调事件
            fun(scrollLinear)                   onBeforeRefresh         刷新前处理函数
            fun(scrollLinear)                   onAfterRefresh          刷新后处理函数
            table                               selectedParams          选中效果(同9宫图参数)
                                                    string      file        背景图片名
                                                    CCRect      rect        图片尺寸矩形
                                                    int         offset      与其容器的间隔值
            table                               items                   这还要解释!!!
            bool                                bounceable              是否启用边界阻尼效果,默认true
            bool                                clippingToBounds        是否启用界面截取效果,默认true
            enum                                direction               方向
                                                     widget.TABLEVIEW_DIRECTION_NONE
                                                     widget.TABLEVIEW_DIRECTION_HORIZONTAL
                                                     widget.TABLEVIEW_DIRECTION_VERTICAL 
                                                     widget.TABLEVIEW_DIRECTION_BOTH(default)
            int                                 x                       x坐标
            int                                 y                       y坐标
            int                                 tag                     node的tag值
            bool                                ignoreAnchor            是否忽略锚点(默认false)
--]]
function M.newScrollLinear ( params )
    local _layer                    = widget.newLayer()
--[=[ ## local params --]=]
    --#必填项#
    local _contentSize              = params.contentSize
    -- local _contentLayer             = params.contentLayer
    -- local _contentParams            = params.contentParams
    --#选填项#
    local _items                    = params.items
    local _cellAtIndexListener      = params.cellAtIndexListener
    local _tapListener              = params.tapListener
    local _selectedParams           = params.selectedParams
    local _onBeforeRefresh          = params.onBeforeRefresh
    local _onAfterRefresh           = params.onAfterRefresh
    local _swallowsTouches          = params.swallowsTouches
    -- local _bounceable               = ifnil(params.bounceable, true)
    -- local _clippingToBounds         = ifnil(params.clippingToBounds, true)
    local _direction                = params.direction or widget.TABLEVIEW_DIRECTION_BOTH
    local _x                        = params.x
    local _y                        = params.y
    local _tag                      = params.tag
    local _ignoreAnchor             = params.ignoreAnchor or false
    
    --#内部参数#
    local _linearLayer
    local _scrollView

--[=[ ## local fuction --]=]  

    local function init( )
        --检查必填项
        Log.a(_contentSize, M.TAG, "newScrollLinear() invalid params.contentSize")
        --使用定义的参数
        M.initView(_layer, params)
        --初始化Linear
        if not params.contentLayer then
            if not params.contentParams then params.contentParams = {} end
            if not params.contentParams.direction then 
                if _direction == widget.TABLEVIEW_DIRECTION_HORIZONTAL then 
                    params.contentParams.direction = DIRECTION_HORIZONTAL
                else
                    params.contentParams.direction = DIRECTION_VERTICAL
                end
            end
            _linearLayer = widget.newLinearLayer(params.contentParams)
        else
            _linearLayer = params.contentLayer
        end
        --初始化ScrollView
        params.contentLayer = _linearLayer
        params.x, params.y = 0, 0
        _scrollView = widget.newScrollView(params)
        _scrollView:ignoreAnchorPointForPosition(true)
        _layer:ignoreAnchorPointForPosition(true)
        _layer:addChild(_scrollView)

        --处理touch事件
        if _selectedParams or _tapListener then 
            if _selectedParams then
                M.bindSelectEffect( _layer, _linearLayer, _selectedParams )
            end
            M.bindTouchEvent{
                layer = _layer,
                swallowsTouches = _swallowsTouches,
                clickListener = function(layer, x, y)
                    if istable(_items) then
                        --可以算是点击了,看看点的是谁
                        local index
                        for i = 1, #_items do
                            local node = _linearLayer:getChildByTag(i)
                            if node and util.isPointInNode(node, x, y) then 
                                index = i
                                break
                            end
                        end
                        if index then
                            --设置高亮
                            if _selectedParams then 
                                _layer:setSelectEffect(index)
                            end
                            --反馈点击事件
                            if _tapListener then 
                                _tapListener(_layer, index, _layer:itemAt(index))
                            end
                        end
                    end
                end,
            }
        end
        if _items then 
            _layer:setItems(_items)
        end
    end
--[=[ ## public fuction --]=]
    --[[ 
    
    重置内容尺寸
    
    **Parameters:**
    
    -   size:        CCSize     新尺寸,不设置则为_linearLayer的尺寸

    --]]
    function _layer:resetContentSize( size )
        _scrollView:setContentSize(size or _linearLayer:getContentSize())
    end
    --[[ 
    
    重新刷新,保持现场的偏移量,用于展开和收缩后的调整
    
    --]]
    function _layer:refreshContentSizeWithHoldContentOffset()
        local contentOffset = _scrollView:getContentOffset()
        local oldHeight = _linearLayer:getContentSize().height
        _linearLayer:refresh()
        local height = _linearLayer:getContentSize().height
        _scrollView:setContentOffset(ccp(
            0, 
            _scrollView:getContentOffset().y + oldHeight - height
            ))
    end
    --[[ 
    
    取得内容的ScrollView

    **Returns:**
    
    -   return 
    
    --]]
    function _layer:getScrollView(  )
        return _scrollView
    end
    --[[ 
    
    取得内容的ScrollView的内部Linear

    **Returns:**
    
    -   return 
    
    --]]
    function _layer:getLinearLayer(  )
        return _linearLayer
    end
    --[[ 
    
    刷新

    --]]
    function _layer:refresh(  )
        if _onBeforeRefresh then _onBeforeRefresh(_layer) end
        _linearLayer:removeAllChildrenWithCleanup(true)
        if _items and _cellAtIndexListener then
            _linearLayer:setAutoRefresh(false)
            for i, item in ipairs(_items) do
                local cellNode = _cellAtIndexListener(_layer, i, item)
                if cellNode then 
                    cellNode:setTag(i)
                    _linearLayer:addChild(cellNode)
                end
            end
            _linearLayer:refresh()
        end
        if _onAfterRefresh then _onAfterRefresh(_layer) end
        _layer:resetContentSize()
        util.autoFixScrollView(_scrollView)
        -- CCBCenter.changeAllLayerTouchPriorityToLast(_layer)
    end
    --[[ 
    
    取得item
    
    **Parameters:**
    
    -   idx:        int         下标
    
    **Returns:**
    
    -   return item
    
    --]]
    function _layer:itemAt( idx )
        if _items and 0 < idx and idx <= #_items then
            return _items[idx]
        else
            return nil
        end
    end
    --[[ 
    
    设置items
    
    **Parameters:**
    
    -   items:        table     item表
    
    --]]
    function _layer:setItems( items )
        _items = nil
        _items = items
        self:refresh()
    end
    --[[ 
    
    取得items

    **Returns:**
    
    -   return items
    
    --]]
    function _layer:getItems(  )
        return _items
    end

--[=[ ## running --]=]
    init()
    _layer._type_ = WIDGET_TYPE.ScrollLinear 
    return _layer
end

--[[ 

## 创建可展开收缩单元控件,本类进行了bindTouchEvent和bindSelectEffect

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize                          contentSize                     尺寸,其实我只要宽度...
            string                          title                           标题
            table                           items                           具体的第一项,当然,第一项还是个table,这里比较麻烦,格式上没有完全的确定
            fun(expandCell, cellTitle)      titleNodeCreateListener         标题node创建监听
            fun(expandCell, cellItem)       itemNodeCreateListener          内容项node创建监听
            
            [选填项]
            boolean                         expandStatus                    初始的展开状态,默认为true
            enum                            direction                       方向
                                                DIRECTION_HORIZONTAL 
                                                DIRECTION_VERTICAL(default)
            CCNode                          parentNode                      所属父容器
            fun(expandCell)                 expandListener                  展开事件监听
            fun(node, index, item)          clickListener                   点击监听
            int                             x                               x坐标
            int                             y                               y坐标
            int                             tag                             node的tag值
            bool                            ignoreAnchor                    是否忽略锚点(默认false)
--]]
function M.newExpandCell ( params )
--[=[ ## local params --]=]
    --#必填项#
    local _contentSize             = params.contentSize
    local _title                   = params.title
    local _items                   = params.items
    local _titleNodeCreateListener = params.titleNodeCreateListener
    local _itemNodeCreateListener  = params.itemNodeCreateListener
    --#选填项#
    local _expandStatus            = ifnil(params.expandStatus, true)
    local _direction               = params.direction or DIRECTION_VERTICAL
    local _parentNode              = params.parentNode
    local _expandListener          = params.expandListener
    local _clickListener           = params.clickListener
    local _cellSelectedParams      = params.cellSelectedParams
    local _x                       = params.x
    local _y                       = params.y
    local _tag                     = params.tag
    local _ignoreAnchor            = params.ignoreAnchor or false
    
    --#内部参数#
    local _layer                   = widget.newLinearLayer()

--[=[ ## local fuction --]=]
    local function init( )
        --检查必填项
        Log.a(_contentSize, M.TAG, "newExpandcell() invalid params.contentSize")
        Log.a(_title, M.TAG, "newExpandcell() invalid params.title")
        Log.a(_items, M.TAG, "newExpandcell() invalid params.items")
        Log.a(isfunction(_titleNodeCreateListener), M.TAG, "newExpandcell() invalid params.titleNodeCreateListener")
        Log.a(isfunction(_itemNodeCreateListener), M.TAG, "newExpandcell() invalid params.itemNodeCreateListener")
        --使用定义的参数
        initView(_layer, params)
        if _cellSelectedParams then
            M.bindSelectEffect( _layer, nil, _cellSelectedParams )
        end
        _layer:setDirection(_direction)
        _layer:setAutoRefresh(false)
        --初始化标题
        local titleNode = _titleNodeCreateListener(_layer, _title)
        titleNode:setTag(0)
        _layer:addChild(titleNode)
        --初始化items
        for i,item in ipairs(_items) do
            local itemNode = _itemNodeCreateListener(_layer, item)
            itemNode:setTag(i)
            _layer:addChild(itemNode)
        end
        _layer:refresh()
        --处理点击事件
        if _clickListener then
            M.bindTouchEvent{
                layer = _layer,
                clickListener = function(layer, x, y)
                    if not _expandStatus then return end
                    for i = 1, #_items do
                        local node = _layer:getChildByTag(i)
                        if util.isPointInNode(node, x, y) then
                            _clickListener(node, i, _items[i])
                            _layer:setSelectEffect(i)
                            break
                        end
                    end
                end,
            }
        end
        
        --如果默认收缩的话,还是要小处理下的.麻烦的东东
        if not _expandStatus then _layer:toggle( _expandStatus ) end
    end

--[=[ ## public fuction --]=]

    function _layer:getWidth(  )
        return _contentSize.width
    end

    function _layer:getParentNode()
        return _parentNode
    end

    function _layer:toggle( expandStatus )
        expandStatus = iff(isboolean(expandStatus), expandStatus, not _expandStatus)
        for i = 1, #_items do
            _layer:getChildByTag(i):setVisible(expandStatus)
        end
        if _layer:getLastSelectedItemIndex() > 0 then 
            local bg = _layer:getSelectedBg()
            if bg then
                bg:setVisible(expandStatus)
            end
        end
        _layer:refresh()
        _expandStatus = expandStatus
        if _expandListener then _expandListener( _layer ) end
    end

    init()
    _layer._type_ = nil
    return _layer
end

--[[ 

## name

**Parameters:**

-   params: 数组包含
            [必填项]
            CCSize                          contentSize                         尺寸
            table                           normalBgImage                       普通背景
            
            [选填项]
            table                           pressedBgImage                      点击后的背景
            table                           disabledBgImage                     无效时的背景
            CCNode                          positionContainer                   用于定位的容器
            int                             offsetX                             定位时的X轴偏移
            int                             offsetY                             定位时的Y轴偏移
            fun(editBox)                    editingBeginListener                开始编辑监听
            fun(editBox)                    editingEndListener                  结束编辑监听
            fun(editBox, text)              textChangedListener                 文本改变监听
            fun(editBox)                    returnListener                      文本返回监听
            ccc3                            fontColor                           文本颜色(默认为白色)
            string                          text                                文本
            ccc3                            placeholderFontColor                提示文字颜色(默认为灰色)
            string                          placeHolderText                     提示文字
            int                             textMaxLength                       文本最大长度(默认为0)
            enum                            inputMode                           输入模式
                                                    kEditBoxInputModeAny
                                                    kEditBoxInputModeEmailAddr
                                                    kEditBoxInputModeNumeric
                                                    kEditBoxInputModePhoneNumber
                                                    kEditBoxInputModeUrl
                                                    kEditBoxInputModeDecimal
                                                    kEditBoxInputModeSingleLine (default)
            enum                            inputFlag                           输入标识
                                                    kEditBoxInputFlagPassword
                                                    kEditBoxInputFlagSensitive
                                                    kEditBoxInputFlagInitialCapsWord
                                                    kEditBoxInputFlagInitialCapsSentence
                                                    kEditBoxInputFlagInitialCapsAllCharacters (default)
            enum                            returnType                          键盘返回类型
                                                    kKeyboardReturnTypeDefault (default)
                                                    kKeyboardReturnTypeDone
                                                    kKeyboardReturnTypeSend
                                                    kKeyboardReturnTypeSearch
                                                    kKeyboardReturnTypeGo
            int                             x                                   x坐标
            int                             y                                   y坐标
            int                             tag                                 node的tag值
            bool                            ignoreAnchor                        是否忽略锚点(默认false)
--]]
function M.newEditBox ( params )
--[=[ ## local params --]=]
    --#必填项#
    local _contentSize          = params.contentSize
    local _normalBgImage        = params.normalBgImage
    --#选填项#
    local _pressedBgImage       = params.pressedBgImage
    local _disabledBgImage      = params.disabledBgImage

    local _positionContainer    = params.positionContainer
    local _offsetX              = params.offsetX or 0
    local _offsetY              = params.offsetY or 0

    local _editingBeginListener = params.editingBeginListener
    local _editingEndListener   = params.editingEndListener
    local _textChangedListener  = params.textChangedListener
    local _returnListener       = params.returnListener
    
    local _fontColor            = params.fontColor --COLOR3.WHITE 
    local _text                 = params.text
    local _placeholderFontColor = params.placeholderFontColor --COLOR3.GREE 
    local _placeHolderText      = params.placeHolderText
    local _textMaxLength        = params.textMaxLength  --0
    local _inputMode            = params.inputMode --kEditBoxInputModeSingleLine
    local _inputFlag            = params.inputFlag --kEditBoxInputFlagInitialCapsAllCharacters
    local _returnType           = params.returnType --kKeyboardReturnTypeDefault
    
    local _x                    = params.x
    local _y                    = params.y
    local _tag                  = params.tag
    local _ignoreAnchor         = params.ignoreAnchor or false
    
    --#内部参数#
    --检查必填项
    Log.a(_contentSize, M.TAG, "newEditBox() invalid params.contentSize")
    Log.a(_normalBgImage, M.TAG, "newEditBox() invalid params.normalBgImage")
    local _layer                = CCEditBox:create(_contentSize, widget.newScale9(_normalBgImage))

--[=[ ## local fuction --]=]
    local function init( )
        _layer.NODE_TYPE = "CCEditBox"
        --初始化条件
        if _pressedBgImage then
            _layer:setBackgroundSpriteForState(widget.newScale9(_pressedBgImage), CCControlStateHighlighted)
        end
        if _disabledBgImage then
            _layer:setBackgroundSpriteForState(widget.newScale9(_disabledBgImage), CCControlStateDisabled)
        end
        local function editBoxHandler(eventType)
            if eventType == "began" then
                -- triggered when an edit box gains focus after keyboard is shown
                if _editingBeginListener then _editingBeginListener(_layer) end
            elseif eventType == "ended" then
                -- triggered when an edit box loses focus after keyboard is hidden.
                if _editingEndListener then _editingEndListener(_layer) end
            elseif eventType == "changed" then
                -- triggered when the edit box text was changed.
                if _textChangedListener then _textChangedListener(_layer, _layer:getText()) end
            elseif eventType == "return" then
                -- triggered when the return button was pressed or the outside area of keyboard was touched.
                if _returnListener then _returnListener(_layer) end
            end
        end
        _layer:registerScriptEditBoxHandler(editBoxHandler)
        -- if _editingBeginListener then _layer:registerScriptBeginHandle(_editingBeginListener) end
        -- if _editingEndListener then _layer:registerScriptEndHandle(_editingEndListener) end
        -- if _textChangedListener then _layer:registerScriptTextChangedHandle(_textChangedListener) end
        -- if _returnListener then _layer:registerScriptReturnHandle(_returnListener) end

        if _fontColor then _layer:setFontColor(_fontColor) end
        if _text then _layer:setText(_text) end
        if _placeholderFontColor then _layer:setPlaceholderFontColor(_placeholderFontColor) end
        if _placeHolderText then _layer:setPlaceHolder(_placeHolderText) end
        if _textMaxLength then _layer:setMaxLength(_textMaxLength) end
        if _inputMode and _inputMode ~= kEditBoxInputModeSingleLine then _layer:setInputMode(_inputMode) end
        if _inputFlag and _inputFlag ~= kEditBoxInputFlagInitialCapsAllCharacters then _layer:setInputFlag(_inputFlag) end
        if _returnType and _returnType ~= kKeyboardReturnTypeDefault then _layer:setReturnType(_returnType) end
        --使用定义的参数
        M.initView(_layer, params)
        if _positionContainer then
            local pos = _positionContainer:convertToWorldSpace(_positionContainer:getPositionLua())
            _layer:setPosition(ccp(pos.x + _offsetX, pos.y + _offsetY))
        end
        local function touchBeganHandler_( x, y )
            return util.isCustomTouchEnabled(_layer)
        end
        if _layer.registerScriptTouchBeganHandler then
            _layer:registerScriptTouchBeganHandler(touchBeganHandler_)
        end
    end
--[=[ ## public fuction --]=]
    -- setTitleTTFSizeForState(float size, CCControlState state); ?????
    -- registerScriptBeginHandle(fun)
    -- registerScriptEndHandle(fun)
    -- registerScriptTextChangedHandle(fun)
    -- registerScriptReturnHandle(fun)
    -- unregisterScriptBeginHandle()
    -- unregisterScriptEndHandle()
    -- unregisterScriptTextChangedHandle()
    -- unregisterScriptReturnHandle()
    -- setText(string)
    -- getText()
    -- setFontColor(ccc3: ccWHITE) 
    -- setPlaceholderFontColor(ccc3: ccGRAY)
    -- setPlaceHolder(string)
    -- getPlaceHolder()
    -- setMaxLength(int: 0)
    -- getMaxLength()
    -- [[
    -- enum EditBoxInputMode
    --     kEditBoxInputModeAny,
    --     kEditBoxInputModeEmailAddr,
    --     kEditBoxInputModeNumeric,
    --     kEditBoxInputModePhoneNumber,
    --     kEditBoxInputModeUrl,
    --     kEditBoxInputModeDecimal,
    --     kEditBoxInputModeSingleLine (default)
    -- ]]
    -- setInputMode(EditBoxInputMode)
    -- [[
    -- enum EditBoxInputFlag
    --     kEditBoxInputFlagPassword,
    --     kEditBoxInputFlagSensitive,
    --     kEditBoxInputFlagInitialCapsWord,
    --     kEditBoxInputFlagInitialCapsSentence,
    --     kEditBoxInputFlagInitialCapsAllCharacters (default)
    -- ]]
    -- setInputFlag(EditBoxInputFlag)
    -- [[
    -- enum KeyboardReturnType
    --     kKeyboardReturnTypeDefault, (default)
    --     kKeyboardReturnTypeDone,
    --     kKeyboardReturnTypeSend,
    --     kKeyboardReturnTypeSearch,
    --     kKeyboardReturnTypeGo
    -- ]]
    -- setReturnType(KeyboardReturnType)

    init()
    _layer._type_ = WIDGET_TYPE.EditBox 
    return _layer
end

--[[ 

## 灰常遗憾的告诉你,想要让这个控件正常使用,请找到-x的libs/cocos2dx/misc_nodes/CCProgressTimer.h文件
## 将其中的updateProgress方法前加上virtual,nnd这里要给它追加功能,不加不用就是,花客

创建进度条

**Parameters:**

-   params: 数组包含
            [必填项]
            table                       frontSprintParams                       前景用sprite属性
            
            [选填项]
            CCSize                      contentSize                             尺寸
            table                       bgSprintParams                          背景用sprite属性
            enum                        barType                                 进度条类型
                                                kCCProgressTimerTypeBar (default)
                                                kCCProgressTimerTypeRadial
            ccp                         midpoint                                进度条中点
                                                barType为kCCProgressTimerTypeBar时默认为左下角ccp(0, 0)
                                                barType为kCCProgressTimerTypeRadial时默认为中点ccp(0.5, 0.5)
            boolean                     reverseProgress                         是否反转,默认为false
            boolean                     useBarChangeRateH                       是否使用水平进度条,默认为true
            boolean                     useBarChangeRateV                       是否使用垂直进度条,默认为false
            int                         percentage                              初始时的百分比(0-100), 默认为0
            boolean                     showPercentage                          是否显示百分比,默认为true
            string                      percentageLabelString                   显示百分比格式,为string.format使用,默认只显示百分比
            table                       ttfLabelParams                          显示文本的属性
            fun(progressBar)            clickListener                           点击事件
            fun(progressBar, percentage, totalPercentage) updateProgressListener                          更新监听,当百分比改变后就触发
            int                         x                                       x坐标
            int                         y                                       y坐标
            int                         tag                                     node的tag值
            bool                        ignoreAnchor                            是否忽略锚点(默认false)
--]]
function M.newProgressBar ( params )
--[=[ ## local params --]=]
    --#必填项#
    local _frontSprintParams      = params.frontSprintParams
    --#选填项#
    local _frontSprintsParams     = params.frontSprintsParams
    local _contentSize            = params.contentSize
    local _bgSprintParams         = params.bgSprintParams
    -- local _barType             = params.barType or kCCProgressTimerTypeBar --kCCProgressTimerTypeRadial
    -- local _midpoint            = params.midpoint
    -- local _reverseProgress     = params.reverseProgress
    local _useBarChangeRateH      = ifnil(params.useBarChangeRateH, true)
    local _useBarChangeRateV      = params.useBarChangeRateV or false
    --local _percentage           = params.percentage or 0
    local _showPercentage         = ifnil(params.showPercentage, true)
    local _percentageLabelString  = params.percentageLabelString or "%d%%"
    local _ttfLabelParams         = params.ttfLabelParams or { text = ""}
    local _clickListener          = params.clickListener
    local _updateProgressListener = params.updateProgressListener
    
    local _x                      = params.x
    local _y                      = params.y
    local _tag                    = params.tag
    local _ignoreAnchor           = params.ignoreAnchor or false
    
    --#内部参数#
    local _layer                  = display.newLayer()
    local _progressTimers         = {}
    local _progressLabel
    local _progressHandle
    local _singleCounter          = 0

--[=[ ## local fuction --]=]
    local function isSingleMode()
        return #_progressTimers == 1
    end
    local function createProgressTimer( sprintsParams, zorder )
        local progressTimer = LuaCCProgressTimer:create(widget.newSprite(sprintsParams))
        progressTimer:registerScriptUpdateHandle(_layer.updatePercentageLabel)
        util.alignNode(progressTimer, _layer:getContentSize(), display.CENTER_LEFT, 0)
        _layer:addChild(progressTimer, zorder)
        progressTimer:setType(params.barType)
        if params.midpoint then progressTimer:setMidpoint(params.midpoint) end
        if params.reverseProgress then progressTimer:setReverseProgress(params.reverseProgress) end
        return progressTimer
    end

    local function setPercentageByIndex( index, percentage )
        local progressTimer = _layer:getProgressTimer(index)
        if progressTimer then 
            progressTimer:setPercentage(percentage)
        end
    end

    local function createAllProgressTimer()
        if _frontSprintsParams then 
            for i,v in ipairs(_frontSprintsParams) do
                _progressTimers[i] = createProgressTimer( v, i )
            end
        end
    end

    local function init( )
        --检查必填项
        Log.a(_frontSprintParams or _frontSprintsParams , M.TAG, "newProgressBar() invalid params.frontSprintParams")
        --准备前后背景
        if not _frontSprintParams then 
            _frontSprintParams = _frontSprintsParams[1]
        end
        if not _frontSprintsParams then 
            _frontSprintsParams = {_frontSprintParams}
        end
        local frontSprite = widget.newSprite(_frontSprintParams)
        local bgSprite = iff(_bgSprintParams, widget.newSprite(_bgSprintParams), nil)
        if not _contentSize then
            if bgSprite then
                --以背景为layer尺寸
                _contentSize = bgSprite:getContentSize()
            else 
                --只能以前景了
                _contentSize = frontSprite:getContentSize()
            end
        end
        --使用定义的参数
        M.initView(_layer, params)
        _layer:setContentSize(_contentSize)
        --放置背景
        if bgSprite then
            util.alignNode(bgSprite, _layer:getContentSize(), display.CENTER_LEFT, 0)
            _layer:addChild(bgSprite, -1)
        end
        --相关参数初始化
        if not params.barType then params.barType = kCCProgressTimerTypeBar end
        if not params.midpoint then
            if params.barType == kCCProgressTimerTypeBar then 
                params.midpoint = CCPointZero
            end
        end
        --放置前景
        createAllProgressTimer()
        --放置文本
        if _ttfLabelParams.shadow then
            _progressLabel = ui.newTTFLabelWithShadow(_ttfLabelParams)
        else
            _progressLabel = ui.newTTFLabel(_ttfLabelParams)
        end
        util.alignNodeWithPadding(_progressLabel, _layer:getContentSize(), display.CENTER, _ttfLabelParams.padding)
        _layer:addChild(_progressLabel, 200)
        _layer:setBarChangeRate(_useBarChangeRateH, _useBarChangeRateV)
        _layer:setPercentage(params.percentage or 0, true)
        --绑定点击事件
        if _clickListener then
            M.bindTouchEvent{
                layer = _layer,
                clickListener = _clickListener,
            }
        end
    end

--[=[ ## public fuction --]=]
    --[[ 
    
    取得进度条主体
    
    **Returns:**
    
    -   return 进度条主体
    
    --]]
    function _layer:getProgressTimer( index )
        --[[可调用的方法
        get/setPercentage(float fPercentage)
        get/setSprite(CCSprite *pSprite)
        get/setType(CCProgressTimerType type)
        setReverseProgress(bool reverse)
        get/setColor(const ccColor3B& color)
        get/setOpacity(GLubyte opacity)
        is/setOpacityModifyRGB(bool bValue)
        is/setReverseDirection(bool value)
        get/setMidpoint(ccp)
        get/setBarChangeRate(ccp)
        --]]
        return _progressTimers[ index or 1 ]
    end
    --[[ 
    
    更新进度条字符串

    **Parameters:**

    -   percentage:         int           当前百分比
    
    --]]
    function _layer.updatePercentageLabel(percentage)
        local totalPercentage = percentage
        -- if not isSingleMode() then 
            totalPercentage = _layer:getPercentage() 
        -- end
        if _showPercentage then
            _progressLabel:setString(string.format(_percentageLabelString, math.round(totalPercentage)))
        end
        if _updateProgressListener then 
            _updateProgressListener(_layer, percentage, totalPercentage)
        end
    end
    --[[ 
    
    设置进度条改变模式
    
    **Parameters:**
    
    -   useHorizontal:              boolean                 是否启用水平进度条
    -   useVertical:                boolean                 是否启用垂直进度条
    
    --]]
    function _layer:setBarChangeRate( useHorizontal, useVertical )
        --ccp第一个值为横方向,第二个为竖方向,值为1为启用
        local barChangeRate = ccp(iff(useHorizontal, 1, 0), iff(useVertical, 1, 0))
        for i, progressTimer in ipairs(_progressTimers) do
            progressTimer:setBarChangeRate(barChangeRate)
        end 
    end
    --[[ 
    
    设置百分比,同时可修改显示文本
    
    **Parameters:**
    
    -   percentage:             int               百分比
    -   force:                  boolean             是否强制设置,默认为false
    
    --]]
    function _layer:setPercentage( percentage, force )
        if force then 
            if _showPercentage then
                _progressLabel:setString(string.format(_percentageLabelString, math.round(percentage or 0)))
            end
        end
        if isSingleMode() then 
            if math.floor(percentage / 100) > 0 and percentage % 100 == 0 then
                setPercentageByIndex(1, 100)
                _singleCounter = math.floor(percentage / 100) - 1
            else
                setPercentageByIndex(1, percentage % 100)
                _singleCounter = math.floor(percentage / 100)
            end
            
        else
            local showCount = math.floor(percentage / 100)
            local lastPercentage = percentage % 100
            --找到需要显示百分百的有多少个,设置过去
            for i = 1, showCount do
                setPercentageByIndex(i, 100)
            end
            --把零头的设置一下
            setPercentageByIndex(showCount + 1, lastPercentage)
            --把剩下为0的也都设置下,真是超麻烦啊cow
            if showCount + 1 < #_progressTimers then
                for i = showCount + 2, #_progressTimers do
                    setPercentageByIndex(i, 0)
                end
            end
        end
    end

    --[[ 
    
    取得百分比

    **Returns:**
    
    -   return 百分比
    
    --]]
    function _layer:getPercentage()
        if isSingleMode() then
            return self:getProgressTimer():getPercentage() + _singleCounter * 100
        else
            local percentage = 0
            for i, progressTimer in ipairs(_progressTimers) do
                percentage = percentage + progressTimer:getPercentage()
            end
            return percentage
        end
    end
    --[[ 
    
    启用/停用显示百分比
    
    **Parameters:**
    
    -   enable:                 boolean                 是否启用
    -   percentageLabelString:  string                  百分比显示格式文本,为string.format使用
    
    --]]
    function _layer:enableShowPercentage( enable, percentageLabelString )
        _showPercentage = enable
        if percentageLabelString then
            _percentageLabelString = percentageLabelString
        end
    end
    --[[ 
    
    设置显示的文本,同时取消显示百分比
    
    **Parameters:**
    
    -   text:        string       显示的文本

    --]]
    function _layer:setString(text)
        _showPercentage = false
        _progressLabel:setString(text)
    end
    --[[ 
    
    停止所有进度条的动作
    
    --]]
    function _layer:stopAllActions()
        for i, progressTimer in ipairs(_progressTimers) do
            progressTimer:stopAllActions()
        end
    end
    --[[ 
    
    执行进度条调整到指定值动作
    
    **Parameters:**
    
    -   duration:               float               经过时间,单位为s
    -   percentage:             int                 百分比
    -   forever:                boolean             是否永久执行,默认为false,仅在单条滚动条且percentage为100时设置为true才会启用
    -   onComplete:             fun(progressBar,percentage) 完成回调事件
    -   onSingleComplete:       fun(progressBar,percentage,totalPercentage) 单次完成回调事件,即到100%

    --]]
    function _layer:runProgressTo( duration, percentage, forever, onComplete, onSingleComplete )
        if isSingleMode() then
            local progressTimer = _layer:getProgressTimer()
            if progressTimer and duration and percentage then
                progressTimer:stopAllActions()
                if percentage > 100 then forever = false end
                if forever then
                    progressTimer:runAction(CCRepeatForever:create(CCProgressTo:create(duration, percentage)))
                else
                    local totalPercentage --  = percentage - _singleCounter * 100
                    local issub = math.floor(percentage / 100) < math.floor(self:getPercentage() / 100)
                    if issub then 
                        totalPercentage = self:getPercentage() - percentage
                    else
                        totalPercentage = percentage - _singleCounter * 100
                    end
                    local function runSingleProgress_( isfirst )
                        -- 想多了,继续gun去
                        if issub then
                            -- 减少
                            if not isfirst then 
                                -- 判断是否结束
                                if totalPercentage > 0 then
                                    setPercentageByIndex( 1, 100 )
                                    _singleCounter = _singleCounter - 1
                                    if onSingleComplete then onSingleComplete(self, self:getPercentage(), percentage) end
                                else
                                    local curPer = self:getPercentage()
                                    if curPer % 100 == 0 and curPer / 100 > 0 then 
                                        if onSingleComplete then onSingleComplete(self, curPer, percentage) end
                                    end 
                                    if onComplete then onComplete(self, percentage) end
                                    return
                                end
                            end

                            local curPer = self:getPercentage()
                            local fromper = iff( math.floor(curPer / 100) > 0 and curPer % 100 == 0, 100, curPer % 100 ) 
                            local toper = 0
                            if math.floor(percentage / 100) == math.floor(curPer / 100) or math.ceil(percentage / 100) * 100 == curPer then 
                                toper = percentage % 100
                            end
                            local dur = duration / 100 * (fromper - toper)
                            totalPercentage = totalPercentage - (fromper - toper)
                            local actions = {
                                --滚动动画
                                CCProgressFromTo:create(dur, fromper, toper), 
                                --触发事件
                                CCCallFunc:create(runSingleProgress_),
                            }
                            progressTimer:runAction(transition.sequence(actions))
                        else
                            -- 增长
                            if not isfirst then
                                -- 判断是否结束 
                                totalPercentage = totalPercentage - 100
                                if totalPercentage > 0 then
                                    if onSingleComplete then onSingleComplete(self, self:getPercentage(), percentage) end
                                    _singleCounter = _singleCounter + 1
                                else
                                    local curPer = self:getPercentage()
                                    if curPer % 100 == 0 and curPer / 100 > 0 then 
                                        if onSingleComplete then onSingleComplete(self, curPer, percentage) end
                                    end
                                    if onComplete then onComplete(self, percentage) end
                                    return
                                end
                            end
                            local per, dur = 100, duration
                            if totalPercentage < 100 then 
                                per = totalPercentage
                                dur = duration / 100 * per
                            elseif isfirst then 
                                dur = duration / 100 * (per - self:getPercentage() % 100)
                            end
                            local actions = {
                                --滚动动画
                                CCProgressTo:create(dur, per), 
                                --触发事件
                                CCCallFunc:create(runSingleProgress_),
                            }
                            progressTimer:runAction(transition.sequence(actions))
                        end
                    end
                    runSingleProgress_(true)
                end
            end
        else
            if percentage > #_progressTimers * 100 then percentage = #_progressTimers * 100 end
            self:stopAllActions()
            local count = math.ceil( percentage / 100 )
            local index = math.floor(self:getPercentage() / 100)
            local lastPercentage = percentage % 100
            local function runProgress_()
                local per = 100
                index = index + 1
                if index > count then 
                    if onComplete then onComplete(self, percentage) end
                    return
                else
                    if onSingleComplete then onSingleComplete(self, (index - 1) * 100, percentage) end
                end
                local progressTimer = _layer:getProgressTimer(index)
                if progressTimer then
                    if index == count and lastPercentage ~= 0 then 
                        per = lastPercentage
                    end
                    local dur = duration / percentage * (per - progressTimer:getPercentage())
                
                    local action = transition.sequence{
                            --滚动动画
                            CCProgressTo:create(dur, per),
                            --触发事件
                            CCCallFunc:create(runProgress_),
                        }
                    progressTimer:runAction(action)
                end
            end
            runProgress_()
        end
    end
    --[[ 
    
    执行进度条调整从指定开始值到指定结束值动作
    
    **Parameters:**
    
    -   duration:               float               经过时间,单位为s
    -   fromPercentage:         int                 开始百分比
    -   toPercentage:           int                 结束百分比
    -   onComplete:             fun(progressBar,percentage) 完成回调事件
    
    --]]
    function _layer:runProgressFromTo( duration, fromPercentage, toPercentage, onComplete )
        self:setPercentage(fromPercentage)
        self:runProgressTo(duration, toPercentage, nil, onComplete )
    end

    init()
    _layer._type_ = WIDGET_TYPE.ProgressBar
    return _layer
end

--[[ 

## 创建帮助用遮罩层,即局部可点击的遮罩层

**Parameters:**

-   params: 数组包含
            [必填项]
            CCRect                  rect                显示出来,并可点击的范围
            
            [选填项]
            fun(layer, rectLayer) 	clickListener 		中间区域的点击事件
            ccc4                    color               遮罩层显示的颜色,默认为 COLOR_MASK
            bool                    allLayerClickable   允许全Layer点击,默认为false
            bool                    swallowsTouches     是否拦截,默认为false
            int                     x                   x坐标
            int                     y                   y坐标
            int                     tag                 node的tag值
--]]
function M.newHelpMaskLayer ( params )
--[=[ ## local params --]=]
    --#必填项#
    local _rect                     = params.rect
    --#选填项#
    local _clickListener            = params.clickListener
    local _color                    = params.color or COLOR_MASK
    local _allLayerClickable        = params.allLayerClickable
    local _tag                      = params.tag
    
    --#内部参数#
    local _layer                    = display.newLayer() 

--[=[ ## local fuction --]=]
    local function createSubLayer( rect, color )
        local layer = display.newColorLayer(color or _color)
        layer:setContentSize(rect.size)
        layer:setPosition(rect.origin)
        return layer
    end
    local function createSubLayers( rect )
        local w, h = CONFIG_SCREEN_WIDTH, CONFIG_SCREEN_HEIGHT
        -- left
        _layer:addChild(createSubLayer(CCRectMake(0, 0, rect.origin.x, h)))
        -- top
        _layer:addChild(createSubLayer(CCRectMake(rect.origin.x, rect.origin.y + rect.size.height, rect.size.width, h - rect.origin.y - rect.size.height)))
        -- right
        _layer:addChild(createSubLayer(CCRectMake(rect.origin.x + rect.size.width, 0, w - rect.origin.x - rect.size.width, h)))
        -- bottom
        _layer:addChild(createSubLayer(CCRectMake(rect.origin.x, 0, rect.size.width, rect.origin.y)))
    end
    local function rectLayerClickListener( rectLayer)
        if _clickListener then 
            _clickListener(_layer, rectLayer)
        end
    end
    local function init( )
        --检查必填项
        Log.a(_rect, TAG, "newHelpMaskLayer() invalid params.rect")
        --使用定义的参数
        params.ignoreAnchor = ifnil(ignoreAnchor, true)
        M.initView(_layer, params)
        -- 放上可以点的
        local rectLayer = createSubLayer(_rect, ccc4(0,0,0,0))
        if not _allLayerClickable then 
            M.bindTouchEvent{
                    layer = rectLayer,
                    priority = -1280,
                    swallowsTouches = params.swallowsTouches,
                    clickListener = rectLayerClickListener,
                }
        end
        _layer:addChild(rectLayer)
        -- 放上不让点的
        local function onTouch(eventType, x, y)
            if eventType == M.TOUCHBEGAN then
                if _allLayerClickable then
                    return true
                else
                    return not util.isPointInNode(rectLayer, x, y)
                end
            elseif eventType == M.TOUCHENDED then
                if _allLayerClickable then
                    rectLayerClickListener(_layer)
                end
            end
        end
        local maskLayer = M.newMaskLayer( nil, onTouch, ccc4(0, 0, 0, 0) )
        _layer:addChild(maskLayer)
        --开始花傻
        createSubLayers(_rect)
    end
--[=[ ## public fuction --]=]
    
    init()
    _layer._type_ = WIDGET_TYPE.HelpMaskLayer
    return _layer
end

--******************************************************************************************************
--[[ 

绑定Touch事件

**Parameters:**

-   params:             table               参数
                           --#必填项#
                           CCLayer                  layer                   待绑定的layer
                           --#选填项# 
                           string                   sound                   点击事件触发时播放的声音
                           boolean                  isMultiTouches          是否是多点触摸,默认为false
                           int                      priority                优先级,默认为0
                           boolean                  swallowsTouches         是否是截断处理,默认为false
                           fun(x, y)                onTouchBegan
                           fun(x, y)                onTouchMoved
                           fun(x, y)                onTouchEnded
                           fun(x, y)                onTouchCancelled
                           fun(layer)               clickListener           点击处理
                           int                      moveDistence            点击允许移动的距离,超过这个距离就不管是点击,默认为10
                           bool                     clickInner              采用内部点击式判定,只要down和up都在容器内就算是点击,默认为false先
                
--]]
function M.bindTouchEvent( params )
    Log.a(params.layer, M.TAG, "bindTouchEvent() invalid params.layer")
    --#必填项#
    local _layer = params.layer 
    --#选填项#
    local _sound            = params.sound
    local _isMultiTouches   = params.isMultiTouches or false
    local _priority         = params.priority or 0
    local _swallowsTouches  = params.swallowsTouches or false
    local _onTouchBegan     = params.onTouchBegan
    local _onTouchMoved     = params.onTouchMoved
    local _onTouchEnded     = params.onTouchEnded
    local _onTouchCancelled = params.onTouchCancelled
    local _clickListener    = params.clickListener
    local _moveDistence     = params.moveDistence or 10
    local _container        = params.container
    local _clickInner       = params.clickInner
    --#内部参数#
    local _beganX, _beganY
    local _ismoved
    local _rect
    local onTouch
    local onTouchBegan
    local onTouchMoved
    local onTouchEnded
    local onTouchCancelled

    function onTouchBegan( x, y, touches )
        if not util.isCustomTouchEnabled(_layer) then
            return false
        end
        if _rect and not util.rectContainsPoint( _rect, ccp(x, y) ) then
            return false
        end
        _beganX, _beganY = x, y
        if _onTouchBegan then 
            return util.isPointInNode(_layer, x, y) and _onTouchBegan(x, y, touches)
        else
            return util.isPointInNode(_layer, x, y)
        end
    end

    function onTouchMoved( x, y, touches )
        if not _clickInner and _ismoved ~= true and (math.abs(x - _beganX) > _moveDistence or math.abs(y - _beganY) > _moveDistence) then
            _ismoved = true
        end
        if _onTouchMoved then 
            _onTouchMoved(x, y, touches)
        end
    end

    function onTouchEnded( x, y, touches )
        if (_clickInner and util.isPointInNode(_layer, x, y) ) or _ismoved ~= true then
            --可以算是点击了,看看点的是谁
            if _clickListener then
                if _sound then 
                    audio.playSound(_sound)
                end
                _clickListener(_layer, x, y)
            end
        end
        if _onTouchEnded then 
            _onTouchEnded(x, y, touches)
        end
        _layer:resetTouchStatus(  )
    end

    function onTouchCancelled( x, y, touches )
        if _onTouchCancelled then 
            _onTouchCancelled(x, y, touches)
        end
        _layer:resetTouchStatus(  )
    end

    function onTouch(eventType, x, y)
        local touches
        
        if istable(x) then 
            touches = x
            x, y = touches[1], touches[2]
            -- Log.d_table("bindTouchEvent", touches, nil, eventType)
        end
        if eventType == M.TOUCHBEGAN then
            return onTouchBegan(x, y, touches)
        elseif eventType == M.TOUCHMOVED then 
            onTouchMoved(x, y, touches)
        elseif eventType == M.TOUCHENDED then
            onTouchEnded(x, y, touches)
        elseif eventType == M.TOUCHCANCELLED then
            onTouchCancelled(x, y, touches)
        end
    end

--[[ ## public fuction --]]
    function _layer:getBeganX(  )
        return _beganX
    end

    function _layer:setBeganX( x )
        _beganX = x
    end

    function _layer:getBeganY(  )
        return _beganY
    end

    function _layer:setBeganY( y )
        _beganY = y
    end

    function _layer:isMoved(  )
        return _ismoved
    end

    function _layer:setIsMoved( ismoved )
        _ismoved = ismoved
    end

    function _layer:resetTouchStatus(  )
        _beganX, _beganY = 0, 0
        _ismoved = nil
    end

    if _container then
        _rect = _container:boundingBox()
    end
    -- display.extendLayer(_layer)
    CCLayerExtend.extend(_layer)
    _layer:addTouchEventListener(onTouch, _isMultiTouches, _priority, _swallowsTouches)
    _layer:setTouchEnabled(true)
end
--[[ 

绑定选中效果功能

**Parameters:**

-   layer:                  CCLayer                 待绑定的node
-   bgContainer:            CCNode                  放置选中背景node的容器,若不设置则为上面的layer
-   selectedParams:         table                   选中前景的效果,为9宫图,具体有
                                string              file                图片名称
                                offset              int                 
                                ...                 ...                 参看9宫参数
    
--]]
function M.bindSelectEffect( layer, bgContainer, selectedParams )
    local _lastSelectedItemIndex   = -1
    bgContainer = ifnil(bgContainer, layer)

    --[[ 
    
    取得最后选中项的下标
    
    **Returns:**
    
    -   return 最后选中项的下标
    
    --]]
    function layer:getLastSelectedItemIndex(  )
        return _lastSelectedItemIndex
    end
    --[[ 
    
    取得选择中的背景
    
    **Returns:**
    
    -   return 选择中的背景
    
    --]]
    function layer:getSelectedBg()
        local selectedBg
        if selectedParams then
            selectedBg = bgContainer:getChildByTag(-10010)
            if not selectedBg then
                --设置下选中背景
                selectedBg = M.newScale9(selectedParams)
                selectedBg:setTag(-10010)
                bgContainer:addChild(selectedBg)
                selectedBg:setVisible(false)
            end
        end
        return selectedBg
    end
    --[[ 
    
    为指定下标设置选中效果
    
    **Parameters:**
    
    -   index:        int     选中项的下标
    
    --]]
    function layer:setSelectEffect(index)
        if _lastSelectedItemIndex == -1 and (not index or index <= 0) then
            return 
        end
        if selectedParams then
            local bg = layer:getSelectedBg()
            if bg then
                if not index or index <= 0 or index > bgContainer:getChildrenCount() then
                    bg:setVisible(false)
                elseif _lastSelectedItemIndex ~= index then
                    local node = bgContainer:getChildByTag(index)
                    if node then
                        local size = node:getContentSize()
                        bg:setContentSize(CCSize(size.width + (selectedParams.offset or 5), size.height + (selectedParams.offset or 5)))
                        bg:setPosition(node:getPositionX(), node:getPositionY())
                        bg:setVisible(true)
                    end
                end
            end
        end
        _lastSelectedItemIndex = index or -1
    end
end
--[[ 

绑定遮罩层
特点:1.绑定的node变成不可见状态,要用showWithMask函数进行展示
    2.点击node以外的的部分,node将自动隐藏起来
    3.点击了node内部,node也不会隐藏,要隐藏的话就得自行调用hideWithMask函数
    4.如果是固定出现的菜单,要进行反复使用,则autoRemove要为false,然后用一变量保持下来就好

**Parameters:**

-   layer:        CCNode     待绑定的node
-   defaultShow:  boolean    是否默认显示,默认为false
-   autoRemove:   boolean    在点击范围外的区域后隐藏时是否自动移除,默认为true
-   color4:       ccc4       遮罩背景色

--]]
function M.bindMaskLayer( layer, defaultShow, autoRemove, color4,listener )
    local _maskLayer
--[[ ## private fuction --]]
    local function init(  )
        local function onTouchEvent(eventType, x, y)
            if eventType == M.TOUCHBEGAN then
                return not util.isPointInNode(layer, x, y)
            elseif eventType == M.TOUCHENDED then
                if not util.isPointInNode(layer, x, y) then 
                    layer:hideWithMask(ifnil(autoRemove, true))
                    if isfunction(listener) then
                        listener()
                    end
                end
            end
        end
        _maskLayer = M.newMaskLayer( 0, onTouchEvent, color4 )
        _maskLayer:ignoreAnchorPointForPosition(false)
        _maskLayer:setPosition(util.centerNode(layer))
        layer:addChild(_maskLayer, -9999)
        layer.PRIORITY = -1001
        CCBCenter.changeAllLayerTouchPriorityToLast( layer, 0.2 )
        if defaultShow == true then
            layer:showWithMask()
        else
            layer:hideWithMask()
        end
    end
--[[ ## public fuction --]]
    --[[ 
    
    以带遮罩层的方式显示出来
    
    --]]
    function layer:showWithMask()
        layer:setVisible(true)
        _maskLayer:setTouchEnabled(true)
    end
    --[[ 
    
    连同遮罩层进行隐藏/移除
    
    **Parameters:**
    
    -   remove:        boolean        是否移除
    
    --]]
    function layer:hideWithMask(remove)
        layer:setVisible(false)
        _maskLayer:setTouchEnabled(false)
        if remove == true then 
            layer:removeFromParentAndCleanup(true)
        end
    end

    init()
end

--[[ 

重写带阴影文字, 可自定义偏移

--]]
function M.newTTFLabelWithShadow(params)
    assert(type(params) == "table",
           "[framework.client.ui] newTTFLabelWithShadow() invalid params")

    local color       = params.color or display.COLOR_WHITE
    local shadowColor = params.shadowColor or display.COLOR_BLACK
    local offset      = params.offset or 1 / (display.widthInPixels / display.width)
    local x, y        = params.x, params.y
    -- local g = M.newColorLayer(ccc4(255,255,255,255))
    local g = display.newNode()
    params.size = params.size
    params.color = shadowColor
    params.x, params.y = 0, 0
    g.shadow1 = ui.newTTFLabel(params)
    -- local offset = 1 / (display.widthInPixels / display.width)
    g.shadow1:realign(offset, -offset)
    g.shadow1:setAnchorPoint(ccp(0.5, 0))
    g:addChild(g.shadow1)

    params.color = color
    g.label = ui.newTTFLabel(params)
    g.label:realign(0, 0)
    g.label:setAnchorPoint(ccp(0.5, 0))
    g:addChild(g.label)
    g:setContentSize(g.label:getContentSize())

    function g:setString(text)
        g.shadow1:setString(text)
        g.label:setString(text)
        g:setContentSize(g.label:getContentSize())
    end

    function g:realign(x, y)
        g:setPosition(x, y)
    end

    function g:getContentSize()
        return g.label:getContentSize()
    end

    function g:setColor(...)
        g.label:setColor(...)
    end

    function g:setShadowColor(...)
        g.shadow1:setColor(...)
    end

    function g:setOpacity(opacity)
        g.label:setOpacity(opacity)
        g.shadow1:setOpacity(opacity)
    end

    if x and y then
        g:setPosition(x, y)
        g:pixels()
    end

    return g
end

--[[ 

重写带边框文字, 可自定义偏移

--]]
function M.newTTFLabelWithOutline(params)
    assert(type(params) == "table",
           "[framework.client.ui] newTTFLabelWithShadow() invalid params")

    local color        = params.color or display.COLOR_WHITE
    local outlineColor = params.outlineColor or display.COLOR_BLACK
    local outlineWidth = params.outlineWidth or 1
    local x, y         = params.x, params.y

    local g = display.newNode()
    -- local g = M.newColorLayer(ccc4(255,255,255,255))
    
    params.size  = params.size
    params.color = outlineColor
    params.x, params.y = 0, 0
    g.shadow1 = ui.newTTFLabel(params)
    g.shadow1:realign(outlineWidth, 0)
    g.shadow1:setAnchorPoint(ccp(0.5, 0))
    g:addChild(g.shadow1)
    g.shadow2 = ui.newTTFLabel(params)
    g.shadow2:realign(-outlineWidth, 0)
    g.shadow2:setAnchorPoint(ccp(0.5, 0))
    g:addChild(g.shadow2)
    g.shadow3 = ui.newTTFLabel(params)
    g.shadow3:realign(0, -outlineWidth)
    g.shadow3:setAnchorPoint(ccp(0.5, 0))
    g:addChild(g.shadow3)
    g.shadow4 = ui.newTTFLabel(params)
    g.shadow4:realign(0, outlineWidth)
    g.shadow4:setAnchorPoint(ccp(0.5, 0))
    g:addChild(g.shadow4)

    params.color = color
    g.label = ui.newTTFLabel(params)
    g.label:realign(0, 0)
    g.label:setAnchorPoint(ccp(0.5, 0))
    g:addChild(g.label)

    g:setContentSize(g.label:getContentSize())

    function g:setString(text)
        g.shadow1:setString(text)
        g.shadow2:setString(text)
        g.shadow3:setString(text)
        g.shadow4:setString(text)
        g.label:setString(text)
    end

    function g:getContentSize()
        return g.label:getContentSize()
    end

    function g:setColor(...)
        g.label:setColor(...)
    end

    function g:setOutlineColor(...)
        g.shadow1:setColor(...)
        g.shadow2:setColor(...)
        g.shadow3:setColor(...)
        g.shadow4:setColor(...)
    end

    function g:setOpacity(opacity)
        g.label:setOpacity(opacity)
        g.shadow1:setOpacity(opacity)
        g.shadow2:setOpacity(opacity)
        g.shadow3:setOpacity(opacity)
        g.shadow4:setOpacity(opacity)
    end

    if x and y then
        g:setPosition(x, y)
        g:pixels()
    end

    return g
end
--[[ 

##线性字符布局
    1.大体参数参照LinearLayer

**Parameters:**

-   params:     table       参数
                    table       paramTable  
                    enum        direction   方向
                                    widget.TABLEVIEW_DIRECTION_HORIZONTAL (default)
                                    widget.TABLEVIEW_DIRECTION_VERTICAL 
                    enum        verticalAlign       垂直排列
                                    VERTICAL_CENTER (default)
                                    VERTICAL_LETF
                                    VERTICAL_RIGHT
                    enum        horizontalAlign     水平排列
                                    HORIZONTAL_CENTER (default)
                                    HORIZONTAL_TOP
                                    HORIZONTAL_BOTTOM 
--]]
function M.newLinearLabel( params )
    local _direction = params.direction or widget.TABLEVIEW_DIRECTION_HORIZONTAL
    local _paramTable = params.paramTable
    local _padding            = params.padding or 0

    local _layer = M.newLinearLayer{
                    direction = _direction,
                    padding = _padding
                }

    --私有方法
    local init

    function init(  )

        if params.verticalAlign and params.verticalAlign ~= VERTICAL_CENTER then
            _layer:setVerticalAlign(params.verticalAlign)
        end

        if params.horizontalAlign and params.horizontalAlign ~= HORIZONTAL_CENTER then
            _layer:setHorizontalAlign(params.horizontalAlign)
        end

        local _cell

        for i,v in ipairs(_paramTable) do
            if istable(v) then
                _cell = ui.newTTFLabel(v)
                _layer:addChild(_cell, i)
            elseif isstring(v) then
                _cell = widget.newSprite{file = v}
                _layer:addChild(_cell, i)
            elseif tolua.type(v) == "CCNode" then
                _cell = v
                _layer:addChild(_cell, i)
            else
                _cell = v
                _layer:addChild(_cell, i)

            end
            
        end
    end

    init()
    return _layer
end

return M