--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[基础]动作]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionBaseOperate", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                  = self._nodeUp                                          -- 目标结点            
    self._aTime1                = params.aTime1             or FA__BASE_OPERATE__T1       -- [动画]时间1
    self._aTime2                = params.aTime2             or FA__BASE_OPERATE__T2       -- [动画]时间2
    self._aTime3                = params.aTime3             or FA__BASE_OPERATE__T3       -- [动画]时间3
    self._aTime4                = params.aTime4             or FA__BASE_OPERATE__T4       -- [动画]时间4
    self._aOffset1              = params.aOffset1           or FA__BASE_OPERATE__O1       -- [动画]位移1

    self._e1Resource            = FA__BASE_OPERATE__E1_RES               -- [效果1]资源
    self.e1Z                    = FA__BASE_OPERATE__E1_Z                 -- [效果1]Z值
    self.e1Prefix               = FA__BASE_OPERATE__E1_PFX               -- [效果1]帧前缀
    self.e1Num                  = FA__BASE_OPERATE__E1_NUM               -- [效果1]帧数量
    self.e1Interval             = FA__BASE_OPERATE__E1_ITV               -- [效果1]帧间隔



    -- 具体操作
    -- 加载资源文件
    TDAnimationUtils.addSpriteFramesWithResource(PHI_FIGHT_EFFECT_BASE .. self._e1Resource)
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- [效果]
    local effect = nil
    -- 定义配置参数
    local params = {
        fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(self.e1Prefix),        -- 文件名
        parentNode  = self:getNode(),                                   -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        z           = self.e1Z,                                         -- 层深度
    }
    -- 生成精灵
    effect = TDUIUtils.loadSprite(params)

    local params            = {
        prefix      = self.e1Prefix,
        frameCount  = self.e1Num,
        interval    = self.e1Interval,
    }
    effect:runAction(TDAnimationUtils.animateWithFrameRepeat(params))





    -- [结点]
    -- 辅助方法
    -- 格式化并返回位置偏移
    local function formatOffset1(offset, owner)
        return (owner == OWNER_ME) and offset or ccp(offset.x, -offset.y)
    end

    -- 格式化并返回位置偏移
    local function cleanupEffect()
        effect:removeFromParentAndCleanup(true)
    end



    -- 辅助参数
    local target            = self:getNode()                            -- 目标对象
    local positionOrigin    = ccp(target:getPosition())                 -- 当前位置
    local owner             = target:getOwner()                         -- 获得拥有者标识
    local offset1           = formatOffset1(self:getAOffset1(), owner)  -- 获得位置偏移1



    -- 动画参数
    local time1             = self:T(self._aTime1)                      -- [动画]时间1
    local time2             = self:T(self._aTime2)                      -- [动画]时间2
    local time3             = self:T(self._aTime3)                      -- [动画]时间3
    local time4             = self:T(self._aTime4)                      -- [动画]时间4
    local position1         = self:O(positionOrigin, offset1)           -- [动画]位置1
    local position3         = self:P(positionOrigin)                    -- [动画]位置3



    -- 位置重排
    local isChange = self:handleViewSwitch()
    -- 构建动作对象
    action = TDAnimationUtils.sequence({
        CCDelayTime:create(isChange and FA__BASE_CORRECT__T or 0),
        CCMoveTo:create(time1, position1),              -- moving up or down
        CCDelayTime:create(time2),                      -- delay time
        CCMoveTo:create(time3, position3),              -- moving origin
        CCCallFunc:create(cleanupEffect),               -- cleanup effect
        CCDelayTime:create(time4),                      -- delay time
    })



    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end

-- 获得[动画]时间3
function M:getATime3()
    return self._aTime3
end
-- 设置[动画]时间3
function M:setATime3(time)
    self._aTime3 = time
end

-- 获得[动画]时间4
function M:getATime4()
    return self._aTime4
end
-- 设置[动画]时间3
function M:setATime4(time)
    self._aTime4 = time
end




-- 获得[动画]位移1
function M:getAOffset1()
    return self._aOffset1
end
-- 设置[动画]位移1
function M:setAOffset1(offset)
    self._aOffset1 = offset
end

-- 获得[动画]旋转角度2[攻击旋转－目标]
function M:getARotation2()
    return self._aRotation2
end
-- 设置[动画]旋转角度2[攻击旋转－目标]
function M:setTime2(rotation)
    self._aRotation2 = rotation
end

-- 获得[动画]旋转角度3[攻击旋转－起始]
function M:getARotation3()
    return self._aRotation3
end
-- 设置[动画]旋转角度3[攻击旋转－起始]
function M:setTime3(rotation)
    self._aRotation3 = rotation
end





return M