--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[基础]文字]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionBaseText", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                  = self._nodeUp                                          -- 目标结点
    self._txtFontName           = self.txtFontName          or FA__BASE_TEXT__FNT_FN    -- [标签]字体名称
    self._txtFontSize           = self.txtFontSize          or FA__BASE_TEXT__FNT_SZ    -- [标签]字体尺寸
    self._txtOffset             = params.txtOffset          or FA__BASE_TEXT__O         -- [标签]偏移量
    self._aTime1                = params.aTime1             or FA__BASE_TEXT__T1        -- [动画]时间1
    self._aTime2                = params.aTime1             or FA__BASE_TEXT__T2        -- [动画]时间2
    self._aRotation1            = params.aRotation1         or FA__BASE_TEXT__R1        -- [动画]旋转角度1
    self._aScale2               = params.aScale2            or FA__BASE_TEXT__S2        -- [动画]缩放比例2
    self._aAlpha2               = params.aAlpha2            or FA__BASE_TEXT__A2        -- [动画]透明度2



    -- 具体操作
    -- ..
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- 添加结点
    -- 标签
    local lblParams = {
        text        = self:getActionText(),         -- 标签文字
        fontSize    = self._txtFontName,            -- 字体大小
        color       = self:getActionColor(),        -- 颜色
        parentNode  = self._node,                   -- 父结点
        align       = display.CENTER,               -- 对齐方式
        offset      = self._txtOffset,              -- 偏移量
    }
    -- 生成标签
    local label = TDUIUtils.loadLabelTTF(lblParams)



    -- 辅助方法
    -- 格式化并返回位置偏移
    local function onComplete()
        label:removeFromParentAndCleanup(true)
    end



    -- 动画参数
    local time1             = self:T(self._aTime1)                      -- [动画]时间1
    local time2             = self:T(self._aTime2)                      -- [动画]时间2
    local rotation1         = self:R(self._aRotation1)                  -- [动画]旋转角度1
    local scale2            = self:S(self._aScale2)                     -- [动画]缩放比例2
    local alpha2            = self:A(self._aAlpha2)                     -- [动画]透明度2



    -- 构建动作对象
    -- [标签]
    local actionForLabel = transition.sequence({
        -- CCRotateBy:create(time1, rotation1),            -- rotate circle
        CCDelayTime:create(time1),                      -- delay
        TDAnimationUtils.spawn({
            CCScaleTo:create(time2, scale2),            -- scale expand
            CCFadeTo:create(time2, alpha2),             -- fade out
        }),
        CCCallFunc:create(onComplete),                  -- cleanup callback
    })
    label:runAction(actionForLabel)
    -- [卡片]
    action = transition.sequence({
        CCDelayTime:create(time1 + time2),              -- delay for label animation
    })



    return action
end




    





-- [实例方法]
-- [实例方法][内部调用]
-- 获得动作文本
function M:getActionText()
    local text

    -- 获得动作类名
    local className = self:getClassName()
    -- 获得映射文本
    text = ActionClassNamingMapper[className]

    return text
end

-- 获得动作颜色 
function M:getActionColor()
    local color

    -- 获得动作类名
    local className = self:getClassName()
    -- 获得映射颜色
    -- color = ActionClassColorMapper[className]
    color = ccc3(255, 0, 0)

    return color
end

-- 获得动作类名
function M:getClassName()
    return self.__cname
end


-- [实例方法][属性Getter/Setter]
-- 获得[标签]字体名称
function M:getTxtFontName()
    return self._txtFontName
end
-- 设置[标签]字体名称
function M:setTxtFontName(text)
    self._txtFontName = text
end

-- 获得[标签]字体尺寸
function M:getTxtFontName()
    return self._txtFontSize
end
-- 设置[标签]字体尺寸
function M:setTxtFontName(size)
    self._txtFontSize = size
end

-- 获得[标签]偏移量
function M:getTxtFontName()
    return self._txtOffset
end
-- 设置[标签]偏移量
function M:setTxtFontName(offset)
    self._txtOffset = size
end

-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end

-- 获得[动画]旋转角度1
function M:getARotation1()
    return self._aRotation1
end
-- 设置[动画]旋转角度1
function M:setARotation1(rotation)
    self._aRotation1 = rotation
end

-- 获得[动画]缩放比例2
function M:getAScale2()
    return self._aScale2
end
-- 设置[动画]缩放比例2
function M:setAScale2(scale)
    self._aScale2 = scale
end

-- 获得[动画]透明度2
function M:getAAlpha2()
    return self._aAlpha2
end
-- 设置[动画]透明度2
function M:setAAlpha2(alpha)
    self._aAlpha2 = alpha
end





return M