--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[持续]封印]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionCtnSeal", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                      = self._nodeUp                                                      -- 目标结点
    self._shieldTime                = params.shieldTime             or FA__CTN_SEAL__S_T                -- [盾]时间
    self._shieldTimeCleanup         = params.shieldTimeCleanup      or FA__CTN_SEAL__S_T_CLEAN          -- [盾]时间清除
    self._lightTime                 = params.lightTime              or FA__CTN_SEAL__S_L_T              -- [光]时间
    self._lightTimeCleanup          = params.lightTimeCleanup       or FA__CTN_SEAL__S_L_T_CLEAN        -- [光]时间清除


    -- 具体操作
    -- ..
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT_EFFECT_CTN .. "e_f_ctn_seal1_1.png",     -- 文件名
        parentNode  = self:getNode(),                                   -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        z           = self._eZ,                                         -- 层深度
    }
    -- 生成精灵
    local light = TDUIUtils.loadSprite(params)

    local params = {
        fileName    = PHI_FIGHT_EFFECT_CTN .. "e_f_ctn_seal2_1.png",     -- 文件名
        parentNode  = self:getNode(),                                   -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        z           = self._eZ,                                         -- 层深度
    }
    -- 生成精灵
    local shield = TDUIUtils.loadSprite(params)

    -- 存储状态
    local actions = self._node:getActions()
    if actions[ActionEnum.CTN_SEAL]  ~= nil then
        local effects = actions[ActionEnum.CTN_SEAL]
        for k in pairs(effects) do
            effects[k]:removeFromParentAndCleanup(true)
        end
    end
    actions[ActionEnum.CTN_SEAL] = { [1] = shield, [2] = light, }



    -- 进入第二阶段
    local function onStep2()
        -- [参数配置]
        local time1         = self:T(.5)                                -- 时间1
        local time2         = self:T(.5)                                -- 时间2

        -- [初始化]
        light:setVisible(true)

        local time1 = self:T(self._lightTime)                           -- [光]时间
        local time2 = self:T(self._lightTimeCleanup)                    -- [光]时间清除
        local actionLight = TDAnimationUtils.sequenceRepeat({
            CCFadeIn:create(time1),
            CCFadeOut:create(time2),
        })
        light:runAction(actionLight)
    end



    -- [参数配置]
    local time          = self:T(.3)                                    -- 时间
    local scaleOrigin   = shield:getScale()                             -- 缩放比例[原]
    local scaleStart    = 0                                             -- 缩放比例[起始]
    local scaleEnd      = scaleOrigin                                   -- 缩放比例[结束]

    -- [初始化]
    light:setVisible(false)
    shield:setScale(scaleStart)

    -- [执行动作]
    local actionShield = TDAnimationUtils.sequence({
        CCEaseElasticInOut:create(CCScaleTo:create(time, scaleEnd), .3),
        CCCallFunc:create(onStep2),
    })
    shield:runAction(actionShield)




    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..


-- [实例方法][属性Getter/Setter]
-- 获得[盾]时间
function M:getShieldTime()
    return self._shieldTime
end
-- 设置[盾]时间
function M:setShieldTime(time)
    self._shieldTime = time
end

-- 获得[盾]时间清除
function M:getShieldTimeCleanup()
    return self._shieldTimeCleanup
end
-- 设置[盾]时间清除
function M:setShieldTimeCleanup(time)
    self._shieldTimeCleanup = time
end

-- 获得[光]时间
function M:getLightTime()
    return self._lightTime
end
-- 设置[光]时间
function M:setLightTime(time)
    self.lightTime = time
end

-- 获得[光]时间清除
function M:getLightTimeCleanup()
    return self.lightTimeCleanup
end
-- 设置[光]时间清除
function M:setLightTimeCleanup(time)
    self.lightTimeCleanup = time
end





return M