--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[英雄]扣血]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionHHurtBlood", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._e1Sound               = params._e1Sound           or FA__H_HURT_BLOOD__S1     -- [声音]
    self._isPlayValueChange     = false                                                 -- 播放数值变化 
    self._aPosition2            = params.aPosition2         or FA__H_HURT_BLOOD__P2     -- [动画]位置2
    self._aEffectTime1          = params.aEffectTime1       or FA__H_HURT_BLOOD__E_T1   -- [动画]效果－时间1
    self._aEffectTime2          = params.aEffectTime2       or FA__H_HURT_BLOOD__E_T2   -- [动画]效果－时间2
    self._aEffectOffset         = params.aEffectOffset      or FA__H_HURT_BLOOD__E_O    -- [动画]效果－偏移量


    -- 具体操作
    TDAnimationUtils.addSpriteFramesWithResource(self:pathResource(FA__HERO_HURT_BLOOD__E1_RES))
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil


    -- 辅助方法
    local function onEffect()
        -- [生成对象]
        -- 定义配置参数
        local params = {
            fileName    = PHI_FIGHT .. "x.png",                         -- 文件名
            parentNode  = self:getNode(),                               -- 父结点
            align       = display.CENTER,                               -- 对齐方式
            anchor      = display.CENTER,                               -- 锚点
            offset      = FA_E_BLOOD_O,                                 -- 效果偏移量
        }
        -- 生成精灵
        local effect = TDUIUtils.loadSprite(params)



        -- [声音]
        self:playSound()

        -- [动画]
        local time1     = self:T(self._aEffectTime1)                               -- [动画]效果－时间1
        local time2     = self:T(self._aEffectTime2)                               -- [动画]效果－时间2
        local position1 = self:O(ccp(effect:getPosition()), self._aEffectOffset)   -- [动画]效果－偏移量
        local scale0    = self:S(0)                                                -- [动画]效果－缩短比例0
        local scale1    = self:S(effect:getScale())                                -- [动画]效果－缩短比例1

        -- 初始设置
        effect:setScale(scale0)
        -- 动画逻辑
        local sequence = transition.sequence({
            CCScaleTo:create(time1, scale1),
            CCMoveTo:create(time2, position1),
            CCCallFunc:create(function() effect:removeFromParentAndCleanup(true) end),
        })
        effect:runAction(sequence)

        self:getNode():animateHp(self._curHp)
    end
    local function onEffect2()
        -- [常量]
        local resource = FA__HERO_HURT_BLOOD__E1_RES
        local prefix   = FA__HERO_HURT_BLOOD__E1_PFX
        local num      = FA__HERO_HURT_BLOOD__E1_NUM
        local interval = self:T(FA__HERO_HURT_BLOOD__E1_ITV)
        local delay    = self:T(FA__HERO_HURT_BLOOD__T1)
        local z        = FA__HERO_HURT_BLOOD__E1_Z

        -- [生成对象]
        -- 定义配置参数
        local params = {
            fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(prefix),              -- 文件名
            parentNode  = self:getNode(),                               -- 父结点
            align       = display.CENTER,                               -- 对齐方式
            anchor      = display.CENTER,                               -- 锚点
        }
        -- 生成精灵
        local effect = TDUIUtils.loadSprite(params)


        -- 定义配置参数
        -- [效果][起始动画]
        local p            = {
            prefix      = prefix,
            frameCount  = num,
            interval    = interval,
        }
        local action = TDAnimationUtils.animateWithFrameRepeat(p)



        -- [声音]
        self:playSound()

        -- [动画]
        effect:runAction(action)
        -- 动画逻辑
        local sequence = transition.sequence({
            CCDelayTime:create(delay),
            CCCallFunc:create(function() effect:removeFromParentAndCleanup(true) end),
        })
        effect:runAction(sequence)
        self:getNode():animateHp(self._curHp)
    end



    -- [结点]
    -- 动画参数
    local time1             = self:T(self._aEffectTime1)                -- [动画]时间1
    local time2             = self:T(self._aEffectTime2)                -- [动画]时间2

    -- 构建动作对象
    action = transition.sequence({
        CCCallFunc:create(onEffect2),                    -- 效果显示
        CCDelayTime:create(time1 + time2),              -- 延时同样时间
    })



    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end

-- 获得路径[资源文件]
function M:pathSound(resourceName)
    return PHS_FIGHT_EFFECT .. resourceName
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end

-- 获得[动画]时间3
function M:getATime3()
    return self._aTime3
end
-- 设置[动画]时间3
function M:setATime3(time)
    self._aTime3 = time
end

-- 获得[动画]时间4
function M:getATime4()
    return self._aTime4
end
-- 设置[动画]时间3
function M:setATime4(time)
    self._aTime4 = time
end




-- 获得[动画]位移1
function M:getAOffset1()
    return self._aOffset1
end
-- 设置[动画]位移1
function M:setAOffset1(offset)
    self._aOffset1 = offset
end

-- 获得[动画]旋转角度2[攻击旋转－目标]
function M:getARotation2()
    return self._aRotation2
end
-- 设置[动画]旋转角度2[攻击旋转－目标]
function M:setTime2(rotation)
    self._aRotation2 = rotation
end

-- 获得[动画]旋转角度3[攻击旋转－起始]
function M:getARotation3()
    return self._aRotation3
end
-- 设置[动画]旋转角度3[攻击旋转－起始]
function M:setTime3(rotation)
    self._aRotation3 = rotation
end





return M