--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[英雄]渐射血]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionHTrapBlood", EffectActionStep1)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)

    -- [加载超类构造函数]
    M.super.ctor(self, params)


    -- [参数赋值]
    self._e1Z                   = params.e1Z            or FA__HERO_TRAP_BLOOD__E1_Z             -- [效果]层深度 
    self._e1Resource            = params._e1Resource    or FA__HERO_TRAP_BLOOD__E1_RES           -- [资源]
    self._e1Sound               = params._e1Sound       or FA__HERO_TRAP_BLOOD__S1               -- [声音]
    self._e1Delay               = params.e1Delay        or 0                                    -- [效果]延迟
    self._e1Prefix              = params.e1Prefix       or FA__HERO_TRAP_BLOOD__E1_PFX           -- [效果]帧前缀
    self._e1Num                 = params.e1Num          or FA__HERO_TRAP_BLOOD__E1_NUM           -- [效果]帧数量
    self._e1Interval            = params.e1Interval     or FA__HERO_TRAP_BLOOD__E1_ITV           -- [效果]帧间隔
    self._e1OffsetStart         = params.e1OffsetStart  or FA__HERO_TRAP_BLOOD__E1_OS            -- [效果]位移起始
    self._e1OffsetEnd           = params.e1OffsetEnd    or FA__HERO_TRAP_BLOOD__E1_OE            -- [效果]位移结束
    self._isSustained           = params.isSustained    or false                                -- [效果]持续标识（如果持续，则播放至目标位置后动画不进行消除）
    self._aTime1                = params.aTime1         or FA__HERO_TRAP_BLOOD__T1               -- [动画]时间


    -- 具体操作
    -- ..
    TDAnimationUtils.addSpriteFramesWithResource(self:pathResource(self._e1Resource))
end





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- [效果]
    if true then 
        -- [目标对象]（血量）
        local target = self:getBlood()

        -- [构建实体]
        -- [效果][起始动画]
        local prefix            = self._e1Prefix                                -- 帧前缀
        local frameCount        = self._e1Num                                   -- 动画帧数
        local interval          = self._e1Interval                              -- 帧间隔
        local z                 = self._e1Z                                     -- 层深度
        local params            = {
            prefix      = prefix,
            frameCount  = frameCount,
            interval    = interval,
        }
        local action = TDAnimationUtils.animateWithFrameRepeat(params)


        -- 定义配置参数
        local paramsCtor = {
            fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(prefix),     -- 文件名
            parentNode  = target,                                               -- 父结点
            align       = display.CENTER,                                       -- 对齐方式
            anchor      = display.CENTER,                                       -- 锚点
            z           = z,                                                    -- 层深度
            offset      = FA_E_BLOOD_O,                                         -- 效果偏移量
        }
        -- 生成精灵
        local effect = TDUIUtils.loadSprite(paramsCtor)

        

        -- [动画定义]
        local delay             = self:T(self._e1Delay)                             -- 延迟时间
        local time              = self._e1Interval * self._e1Num                    -- 时间
        local positionOrigin    = ccp(effect:getPosition())                         -- 位置[原]
        local positionStart     = ccpAdd(positionOrigin, self._e1OffsetStart)       -- 位置[起始]
        local positionEnd       = ccpAdd(positionOrigin, self._e1OffsetEnd)         -- 位置[结束]
        effect:setVisible(true)
        effect:setPosition(positionStart)

        local function onComplete()
            target:animateHp(self._curHp)
            effect:removeFromParentAndCleanup(true)
        end

        if delay == 0 then 
            effect:runAction(action)
            effect:runAction(TDAnimationUtils.sequence({
                CCMoveTo:create(time, positionEnd),             -- move 
                CCCallFunc:create(onComplete),
            }))
        else
            effect:runAction(TDAnimationUtils.sequence({ action, CCDelayTime:create(delay) }))
            effect:runAction(TDAnimationUtils.sequence({
                CCDelayTime:create(delay),
                CCMoveTo:create(time, positionEnd),             -- move 
                CCCallFunc:create(onComplete),
            }))
        end
    end



    -- [卡片]
    local action

    -- 动画参数
    local time1             = self:T(self._aTime1)                  -- [动画]时间1


    -- 构建动作对象
    action = transition.sequence({
        CCCallFunc:create(function() self:playSound() end),         -- play effect
        CCDelayTime:create(time1),                  -- delay
    })

    return action
end




    





-- [实例方法]
-- [实例方法][内部调用]
-- 获得血量
function M:getBlood()
    -- 获得血量
    local node    = self:getNode()
    local topView = node:getTopView()
    local owner   = (node:getOwner() == OWNER_ME) and OWNER_OPPONENT or OWNER_ME
    return topView:getPlayerBlood(owner)
end


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end


-- [实例方法]
-- [实例方法][外部调用]
-- ..


-- [实例方法][属性Getter/Setter]
-- ..





return M