--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[伤害]普通]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionHurtNormal", EffectActionStep1)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [构建参数]
    local ethnicity             = params.nb - 100                             -- 种族
    if     ethnicity == RACE_ALLIANCE then
        params._e1Sound             = FA__HURT_NORMAL__S                      -- [声音]
        params._e1Resource          = FA__HURT_NORMAL__1_E1_RES               -- [效果1]资源
        params.e1Z                  = FA__HURT_NORMAL__1_E1_Z                 -- [效果1]Z值
        params.e1Prefix             = FA__HURT_NORMAL__1_E1_PFX               -- [效果1]帧前缀
        params.e1Num                = FA__HURT_NORMAL__1_E1_NUM               -- [效果1]帧数量
        params.e1Interval           = FA__HURT_NORMAL__1_E1_ITV               -- [效果1]帧间隔
        params.e1OffsetStart        = FA__HURT_NORMAL__1_E1_OS                -- [效果1]位移起始
        params.e1OffsetEnd          = FA__HURT_NORMAL__1_E1_OE                -- [效果1]位移结束
        params.aTime1               = FA__HURT_NORMAL__1_T1                   -- [动画]时间1
    elseif ethnicity == RACE_TRIBE    then
        params._e1Sound             = FA__HURT_NORMAL__S                      -- [声音]
        params._e1Resource          = FA__HURT_NORMAL__2_E1_RES               -- [效果1]资源
        params.e1Z                  = FA__HURT_NORMAL__2_E1_Z                 -- [效果1]Z值
        params.e1Prefix             = FA__HURT_NORMAL__2_E1_PFX               -- [效果1]帧前缀
        params.e1Num                = FA__HURT_NORMAL__2_E1_NUM               -- [效果1]帧数量
        params.e1Interval           = FA__HURT_NORMAL__2_E1_ITV               -- [效果1]帧间隔
        params.e1OffsetStart        = FA__HURT_NORMAL__2_E1_OS                -- [效果1]位移起始
        params.e1OffsetEnd          = FA__HURT_NORMAL__2_E1_OE                -- [效果1]位移结束
        params.aTime1               = FA__HURT_NORMAL__2_T1                   -- [动画]时间1
    elseif ethnicity == RACE_UNDEAD   then
        params._e1Sound             = FA__HURT_NORMAL__S                      -- [声音]
        params._e1Resource          = FA__HURT_NORMAL__3_E1_RES               -- [效果1]资源
        params.e1Z                  = FA__HURT_NORMAL__3_E1_Z                 -- [效果1]Z值
        params.e1Prefix             = FA__HURT_NORMAL__3_E1_PFX               -- [效果1]帧前缀
        params.e1Num                = FA__HURT_NORMAL__3_E1_NUM               -- [效果1]帧数量
        params.e1Interval           = FA__HURT_NORMAL__3_E1_ITV               -- [效果1]帧间隔
        params.e1OffsetStart        = FA__HURT_NORMAL__3_E1_OS                -- [效果1]位移起始
        params.e1OffsetEnd          = FA__HURT_NORMAL__3_E1_OE                -- [效果1]位移结束
        params.aTime1               = FA__HURT_NORMAL__3_T1                   -- [动画]时间1
    end




    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                  = self._nodeUp                                          -- 目标结点
    self._aTime1                = params.aTime1             or FA__HURT_NORMAL__T1      -- [动画]时间1
    self._aTime2                = params.aTime2             or FA__HURT_NORMAL__T2      -- [动画]时间2
    self._aScale1               = params.aScale1            or FA__HURT_NORMAL__S1      -- [动画]缩放比例1
    self._aEffectTime           = params.aEffectTime        or FA__HURT_NORMAL__E_T     -- [动画]效果－时间
    self._aEffectOffset         = params.aEffectOffset      or FA__HURT_NORMAL__E_O     -- [动画]效果－偏移量
    self._eZ                    = params.eZ                 or FA__HURT_NORMAL__E_Z     -- [动画]效果－层深度


    -- 具体操作
    -- ..
end

    

    





-- [覆写方法]
-- 运行动作
-- [Version1]
--[[
function M:buildAction(params)
    -- 定义动作对象
    local action = nil


    -- 辅助方法
    -- 显示效果x
    local function onEffect()
        -- [生成对象]
        -- 定义配置参数
        local params = {
            fileName    = PHI_FIGHT .. "x.png",                          -- 文件名
            parentNode  = self:getNode(),                               -- 父结点
            align       = display.CENTER,                               -- 对齐方式
            anchor      = display.CENTER,                               -- 锚点
            z           = self._eZ,                                     -- 层深度
        }
        -- 生成精灵
        local effect = TDUIUtils.loadSprite(params)


        -- [动画]
        local time   = self:T(self._aEffectTime)                                -- [动画]效果－时间
        local offset = self:O(ccp(effect:getPosition()), self._aEffectOffset)   -- [动画]效果－偏移量
        local sequence = transition.sequence({
            CCMoveTo:create(time, offset),
            CCCallFunc:create(function() effect:removeSelf() end),
        })
        effect:runAction(sequence)
    end

    -- 显示伤害红色
    local function onHurtColor() 
        local target = self:getNode()

        target:animateHp(self._curHp)

        local action = transition.sequence({
            CCCallFunc:create(function()
                target:setColor(ccc3(255, 0, 0))
                onEffect()
            end),
            -- CCCallFunc:create(onEffect),                     -- 效果x显示
            CCFadeOut:create(.2),                               -- scale -
            CCFadeIn:create(.2),                                -- scale -
            CCCallFunc:create(function()
                target:setColor(ccc3(255, 255, 255))
            end),
        })

    end



    -- 辅助参数
    local originScale       = self:getNode():getScale()                 -- 原始缩放比例



    -- 动画参数
    local time1             = self:T(self._aTime1)                      -- [动画]时间1
    local time2             = self:T(self._aTime2)                      -- [动画]时间2
    local scale1            = self:S(originScale * self._aScale1)       -- [动画]缩放比例1
    local scale2            = self:S(originScale)                       -- [动画]缩放比例2



    -- 构建动作对象
    action = transition.sequence({
        -- CCCallFunc:create(onEffect),                    -- 效果x显示
        CCCallFunc:create(onHurtColor),                 -- 伤害颜色
        CCScaleTo:create(time1, scale1),                -- scale -
        CCScaleTo:create(time2, scale2),                -- scale +
    })

    -- [效果数字]
    self:playFightNum(self._varHp * -1, 0.0)
    




    return action
end
--]]

-- 构建卡片动作对象
function M:buildActionCard(params) 
    -- 定义动作对象
    local action = nil


    -- 辅助参数
    local originScale       = self:getNode():getScale()                 -- 原始缩放比例
    local positionOrigin    = ccp(self:getNode():getPosition()) 
    -- 动画参数
    local time1             = self:T(self._aTime1)                      -- [动画]时间1
    local time2             = self:T(self._aTime2)                      -- [动画]时间2
    local scale1            = self:S(originScale * self._aScale1)       -- [动画]缩放比例1
    local scale2            = self:S(originScale)                       -- [动画]缩放比例2
    local position4         = self:P(positionOrigin)                    -- [动画]位置4



    -- 辅助方法
    -- 显示伤害红色
    local function onHurtColor() 
        local target = self:getNode()

        local action = TDAnimationUtils.sequence({
            CCCallFunc:create(function()
                target:setColor(ccc3(255, 0, 0))
            end),
            CCFadeOut:create(time1 / 2),                               -- scale -
            CCFadeIn:create(time2 / 2),                                -- scale -
            CCCallFunc:create(function()
                target:setColor(ccc3(255, 255, 255))
            end),
        })

        target:runAction(action)
    end

    local pointL = ccp(positionOrigin.x - 3, positionOrigin.y)
    local pointR = ccp(positionOrigin.x + 3, positionOrigin.y)

    local pointU = ccp(positionOrigin.x, positionOrigin.y + 4)
    local pointD = ccp(positionOrigin.x, positionOrigin.y - 4)


    -- 位置重排
    local isChange = self:handleViewSwitch()
    -- 构建动作对象
    action = TDAnimationUtils.sequence({
        CCDelayTime:create(isChange and FA__BASE_CORRECT__T or 0),
        CCCallFunc:create(function() self:playSound() end),     -- 播放音效
        -- CCCallFunc:create(onHurtColor),                         -- 伤害颜色
        CCScaleTo:create(0.05, 0.95),                        -- scale -
        CCMoveTo:create(0.05, pointL), --左
        CCMoveTo:create(0.02, pointU), --上
        CCMoveTo:create(0.02, pointD), --下
        CCMoveTo:create(0.05, pointR),  --右
        CCMoveTo:create(0.02, position4), 
        CCScaleTo:create(time2, scale2),                        -- scale +
        -- 
       
    })
    




    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end

-- 获得[动画]时间3
function M:getATime3()
    return self._aTime3
end
-- 设置[动画]时间3
function M:setATime3(time)
    self._aTime3 = time
end

-- 获得[动画]时间4
function M:getATime4()
    return self._aTime4
end
-- 设置[动画]时间3
function M:setATime4(time)
    self._aTime4 = time
end




-- 获得[动画]位移1
function M:getAOffset1()
    return self._aOffset1
end
-- 设置[动画]位移1
function M:setAOffset1(offset)
    self._aOffset1 = offset
end

-- 获得[动画]旋转角度2[攻击旋转－目标]
function M:getARotation2()
    return self._aRotation2
end
-- 设置[动画]旋转角度2[攻击旋转－目标]
function M:setTime2(rotation)
    self._aRotation2 = rotation
end

-- 获得[动画]旋转角度3[攻击旋转－起始]
function M:getARotation3()
    return self._aRotation3
end
-- 设置[动画]旋转角度3[攻击旋转－起始]
function M:setTime3(rotation)
    self._aRotation3 = rotation
end





return M