--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[伤害]幻之刃]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionHurtRegionEclipse", EffectActionStep1)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [构建参数]
    params._e1Resource          = FA__HURT_REGION__E1_RES               -- [效果1]资源
    params._e1Sound             = FA__HURT_REGION__S1                   -- [声音1]
    params.e1Z                  = FA__HURT_REGION__E1_Z                 -- [效果1]Z值
    params.e1Prefix             = FA__HURT_REGION__E1_PFX               -- [效果1]帧前缀
    params.e1Num                = FA__HURT_REGION__E1_NUM               -- [效果1]帧数量
    params.e1Interval           = FA__HURT_REGION__E1_ITV               -- [效果1]帧间隔
    params.e1OffsetStart        = FA__HURT_REGION__E1_OS                -- [效果1]位移起始
    params.e1OffsetEnd          = FA__HURT_REGION__E1_OE                -- [效果1]位移结束
    params.aTime1               = FA__HURT_REGION__T1                    -- [动画]时间1



    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                  = self._nodeUp                                                  -- 目标结点      
    self._aTime                 = params.aTime              or FA__HURT_LUNAR_ECLIPSE__T        -- [动画]时间(总)
    self._aTime1                = params.aTime1             or FA__HURT_LUNAR_ECLIPSE__T1       -- [动画]时间1
    self._aTime2                = params.aTime2             or FA__HURT_LUNAR_ECLIPSE__T2       -- [动画]时间2
    self._aOffset1              = params.aOffset1           or FA__HURT_LUNAR_ECLIPSE__O1       -- [动画]位移1
end




    





-- [实例方法]
-- [实例方法][覆盖调用]
-- 构建卡片动作对象
function M:buildActionCard(params) 
    -- 定义动作对象
    local action = nil



    -- 辅助方法
    -- 格式化并返回位置偏移
    local function formatOffset1(offset, owner)
        return (owner == OWNER_ME) and ccp(offset.x, -offset.y) or offset
    end



    -- 辅助参数
    local positionOrigin    = ccp(self:getNode():getPosition())         -- 当前位置
    local owner             = self:getNode():getOwner()                 -- 获得拥有者标识
    local offset1           = formatOffset1(self:getAOffset1(), owner)  -- 获得位置偏移1



    -- 动画参数
    local time1             = self:T(self._aTime1)                      -- [动画]时间1
    local time2             = self:T(self._aTime2)                      -- [动画]时间2
    local position1         = self:O(positionOrigin, offset1)           -- [动画]位置1
    local position2         = self:P(positionOrigin)                    -- [动画]位置4
    local rate              = .3                                        -- [动画]比率



    -- 构建动作对象
    action = transition.sequence({
        CCCallFunc:create(function() self:playSound() end),
        CCEaseExponentialIn:create(CCMoveTo:create(time1, position1)),  -- moving up or down
        CCEaseOut:create(CCMoveTo:create(time2, position2), rate),      -- moving origin
    })




    return action
end


-- [实例方法][外部调用]
-- ..


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end

-- 获得[动画]位移1
function M:getAOffset1()
    return self._aOffset1
end
-- 设置[动画]位移1
function M:setAOffset1(offset)
    self._aOffset1 = offset
end





return M