--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[操作]数值]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionOptValue", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                      = self._nodeUp                                          -- 目标结点
    self._aTime                     = params.aTime              or FA__OPT_VALUE__T         -- [动画]时间
    self._isPlayValueChange         = params.isPlayValueChange  or true                     -- 播放数值变化 
    self._isViewZeroEH              = params.isViewZeroEH       or false                    -- 是否显示补伤数字
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- [卡片]
    -- 构建卡片动作对象
    action = self:buildActionCard(params)
    -- [数值]
    -- 播放数值变化 
    self:playValueChange()



    return action
end




    





-- [实例方法]
-- [实例方法][卡片]
-- 构建卡片动作对象
function M:buildActionCard(params)
    local action

    -- 动画参数
    local time1             = self:T(self._aTime)                  -- [动画]时间1

    -- 构建动作对象
    action = transition.sequence({
        CCDelayTime:create(time1),                  -- delay
    })

    return action
end


---------


-- [实例方法][数值]
-- 播放数值变化 
function M:playValueChange()
    -- [过渡判断]
    if not self._isPlayValueChange then 
        return
    end

    -- [逻辑处理]
    -- 补伤数字
    if true then 
        local value         = self._varHp
        local isViewZero    = self._isViewZeroEH
        if value ~= 0 or (value == 0 and isViewZero) then 
            self:playFightNum(value)
        end
    end
    -- 攻击力
    if true then 
        local node          = self:getNodeUp()
        local value         = self._curAtk
        local origin        = node:getPo():getAttack()

        local var           = self._varAtk
        local color0        = display.COLOR_WHITE
        local color1        = display.COLOR_WHITE
        if var ~= 0 then 
            -- Version1:
            --[[
            if var > 0 then 
                color0 = FA__OPT_VALUE__C_ATK_U
                color1 = FA__OPT_VALUE__C_ATK_UE
            else 
                color0 = FA__OPT_VALUE__C_ATK_D
                color1 = FA__OPT_VALUE__C_ATK_DE
            end
            ]]

            -- Version2:
            if value > origin then 
                color0 = FA__OPT_VALUE__C_ATK_U
                color1 = FA__OPT_VALUE__C_ATK_UE
            else
                color0 = FA__OPT_VALUE__C_ATK_D
                color1 = FA__OPT_VALUE__C_ATK_DE
            end

            node:animteAttack(value, color0, color1)
        end
    end
    -- 血量
    if true then 
        local node          = self:getNodeUp()
        local value         = self._curHp
        local origin        = node:getPo():getHp()

        local var           = self._varHp
        local color0        = display.COLOR_WHITE
        local color1        = display.COLOR_WHITE
        if var ~= 0 then 
            -- Version1:
            --[[
            ]]
            if var > 0 then 
                color0 = FA__OPT_VALUE__C_HP_U
                color1 = FA__OPT_VALUE__C_HP_UE
            else 
                color0 = FA__OPT_VALUE__C_HP_D
                color1 = FA__OPT_VALUE__C_HP_DE
            end

            -- Version2:
            --[[
            if value > origin then 
                color0 = FA__OPT_VALUE__C_HP_U
                color1 = FA__OPT_VALUE__C_HP_UE
            elseif value < origin then
                color0 = FA__OPT_VALUE__C_HP_D
                color1 = FA__OPT_VALUE__C_HP_DE
            else
                color0 = display.WHITE
                color1 = display.WHITE
            end
            ]]

            node:animateHp(value, color0, color1)
        end
    end
end


---------


-- [实例方法][属性Getter/Setter]
-- 设置动画[时间]
function M:setATime(aTime)
    self._aTime = aTime
end
-- 获得动画[时间]
function M:getATime()
    return self._aTime
end





return M