--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[其他]召唤]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionOthSummon2", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                  = self._nodeDown                        -- 目标结点
    -- [构建参数]
    self._e1Resource            = FA__OTH_SUMMON2__E1_RES               -- [效果1]资源
    self._e1Sound               = FA__OTH_SUMMON2__S1                   -- [声音]
    self._e1Z                   = FA__OTH_SUMMON2__E1_Z                 -- [效果1]Z值
    self._e1Prefix              = FA__OTH_SUMMON2__E1_PFX               -- [效果1]帧前缀
    self._e1Num                 = FA__OTH_SUMMON2__E1_NUM               -- [效果1]帧数量
    self._e1Interval            = FA__OTH_SUMMON2__E1_ITV               -- [效果1]帧间隔
    self._e1OffsetStart         = FA__OTH_SUMMON2__E1_OS                -- [效果1]位移起始
    self._e1OffsetEnd           = FA__OTH_SUMMON2__E1_OE                -- [效果1]位移结束
    self._aTime1                = FA__OTH_SUMMON2__T1                   -- [动画]时间1



    -- [其他操作]
    -- 加载资源文件[图片]
    TDAnimationUtils.addSpriteFramesWithResource(PHI_FIGHT_EFFECT_OTH .. self._e1Resource)
    -- 加载资源文件[声音]
    SOUND.preloadSound(self:pathSound(self._e1Sound))
end




    







-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil


    local function runEffect()
        -- print("========================================")
        -- print("runEffect()")
        -- print("========================================")
        -- [参数定义]
        local effect    = nil                                                   -- 效果
        local action    = nil                                                   -- 动作

        -- [辅助参数定义]
        local prefix        = self._e1Prefix                                    -- [效果]帧前缀
        local frameCount    = self._e1Num                                       -- [效果]帧数量
        local interval      = self._e1Interval                                  -- [效果]帧间隔
        local z             = self._e1Z                                         -- [效果]Z值
        local parentNode    = self:getNode():getParent()                        -- [效果]父结点
        local owner         = self:getNode():getOwner()                         -- [效果]所属者


        -- [效果]
        -- 定义配置参数
        local paramsCtor = {
            fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(prefix),     -- 文件名
            parentNode  = parentNode,                                           -- 父结点
            align       = display.CENTER,                                       -- 对齐方式
            anchor      = display.CENTER,                                       -- 锚点
            z           = z,                                                    -- 层深度
        }
        -- 生成精灵
        effect = TDUIUtils.loadSprite(paramsCtor)
        effect:setPosition(self:getNode():getPosition())
        effect:setZOrder(100000000)
        effect:setFlipY((owner == OWNER_ME) and false or true)


        -- [动画]
        local params            = {
            prefix      = prefix,
            frameCount  = frameCount,
            interval    = interval,
        }
        action = TDAnimationUtils.animateWithFrame(params)


        -- [执行动画]
        effect:runAction(TDAnimationUtils.sequence({
            action,
            CCCallFunc:create(function()
                effect:removeFromParentAndCleanup(true)
            end),
        }))
    end

    local function resetNode()
        self:getNode():reset()
    end

    local function createGlitter()
        -- print("========================================")
        -- print("createGlitter()")
        -- print("========================================")
        -- 定义配置参数
        local paramsCtor = {
            fileName    = "#e_f_oth_summon2_box.png",                           -- 文件名
            parentNode  = self:getNode(),                                       -- 父结点
            align       = display.CENTER,                                       -- 对齐方式
            anchor      = display.CENTER,                                       -- 锚点
            z           = -1,                                                   -- 层深度
        }
        -- 生成精灵
        self._glitter = TDUIUtils.loadSprite(paramsCtor)
    end
    local function removeGlitter()
        -- print("========================================")
        -- print("removeGlitter()")
        -- print("========================================")
        self._glitter:removeFromParentAndCleanup(true)
        self._glitter = nil
    end




    -- [动画]
    local node              = self:getNode()
    local owner             = node:getOwner()
    local delay             = self:T(self._aTime1)
    local time1             = 1.0
    local time2             = .3
    local offset1           = ccp(0, self:D((owner == OWNER_ME) and 150 or -150))
    local scaleOrigin       = node:getScale()
    local scale1_1          = 0
    local scale1_2          = 1
    local scale1_3          = 0
    local scale1_4          = 1
    local scaleRevival      = scaleOrigin * .3
    local prototype         = self._prototypes:getPrototype(self._index)    -- 原型对象
    local position          = prototype:getPosition()                       -- [动画]目标位置
    local rotation          = prototype:getRotation()                       -- [动画]目标旋转角度
    local scale             = prototype:getScale()                          -- [动画]目标缩放比例


    action = TDAnimationUtils.sequence({
        CCCallFunc:create(function() self:playSound() end), 
        CCCallFunc:create(runEffect), 
        CCDelayTime:create(delay),                          -- 延迟时间
        CCCallFunc:create(resetNode),

        CCShow:create(),
        CCCallFunc:create(createGlitter), 
        CCCallFunc:create(function()
            node:setScale(scaleRevival)
        end),

        TDAnimationUtils.spawn({
            CCMoveBy:create(time1, offset1),
            CCScaleTo:create(time1, scaleOrigin),
            TDAnimationUtils.sequence({
                CCScaleTo:create(time1 * .25, scale1_1, 1),
                CCScaleTo:create(time1 * .25, scale1_2, 1),
                CCScaleTo:create(time1 * .25, scale1_3, 1),
                CCScaleTo:create(time1 * .25, scale1_4, 1),
            })
        }),

        TDAnimationUtils.spawn({
            -- CCMoveTo:create(time2, position),                -- 位移
            CCJumpTo:create(time2, position, 100, 1),        -- 位移
            CCRotateTo:create(time2, rotation),              -- 旋转角度
            CCScaleTo:create(time2, scale),                  -- 缩放比例
        }),
        CCCallFunc:create(removeGlitter), 
    })



    return action
end


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end


-- [实例方法][属性Getter/Setter]
-- ..





return M