--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[舞台]死亡]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionStageDeath", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]IDX_CEMETERY
    self._node                  = self._nodeUp                                              -- 目标结点
    self._e1Sound               = params.e1Sound            or FA__STAGE_DEATH__S           -- [效果]音效
    self._prototypes            = params.prototypes         or {}                           -- 原型集
    self._aTime1                = params.aTime1             or FA__STAGE_DEATH__T1          -- [动画]时间1
    self._aTime2                = params.aTime2             or FA__STAGE_DEATH__T2          -- [动画]时间2
    self._aTime3                = params.aTime3             or FA__STAGE_DEATH__T3          -- [动画]时间3
    self._aTime4                = params.aTime4             or FA__STAGE_DEATH__T4          -- [动画]时间4
    self._aAlpha1               = params.aAlpha1            or FA__STAGE_DEATH__A1          -- [动画]透明度1
    self._aAlpha2               = params.aAlpha2            or FA__STAGE_DEATH__A2          -- [动画]透明度2
    self._aAlpha3               = params.aAlpha3            or FA__STAGE_DEATH__A3          -- [动画]透明度3
    self._aEffectTime           = params.aEffectTime        or FA__STAGE_DEATH_E__T         -- [动画]效果－时间
    self._aEffectOffset         = params.aEffectOffset      or FA__STAGE_DEATH_E__O         -- [动画]效果－偏移量


    -- 具体操作
    -- ..
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- 动画参数
    local time1             = self:T(self._aTime1)                      -- [动画]时间1
    local time2             = self:T(self._aTime2)                      -- [动画]时间2
    local time3             = self:T(self._aTime3)                      -- [动画]时间3
    local time4             = self:T(self._aTime4)                      -- [动画]时间4
    local alpha1            = self:A(self._aAlpha1)                     -- [动画]透明度1
    local alpha2            = self:A(self._aAlpha2)                     -- [动画]透明度2
    local alpha3            = self:A(self._aAlpha3)                     -- [动画]透明度3



    -- 辅助方法
    -- 卡牌斩断
    local function onCutOff()
        -- [Version1]
        --[[
        -- 获得拷贝模板
        local node = self:getNode()

        -- [生成对象]
        -- 定义配置参数
        local time      = self:T(self._aEffectTime)                     -- 时间
        local offset    = self:P(self._aEffectOffset)                   -- 偏移量

        -- 生成精灵[左]
        local params = {
            fileName    = node:getFileName(),                           -- 文件名
            parentNode  = node,                                         -- 父结点
            align       = display.CENTER,                               -- 对齐方式
            anchor      = display.CENTER_LEFT,                          -- 锚点
        }
        local left = TDUIUtils.loadSprite(params)
        -- 生成精灵[右]
        local params = {
            fileName    = node:getFileName(),                           -- 文件名
            parentNode  = node,                                         -- 父结点
            align       = display.CENTER,                               -- 对齐方式
            anchor      = display.CENTER_RIGHT,                         -- 锚点
        }
        local right = TDUIUtils.loadSprite(params)

        -- 设置显示区域
        local width, height = node:getContentSize().width, node:getContentSize().height
        left:setTextureRect(CCRect(0, 0, width / 2, height))
        right:setTextureRect(CCRect(width / 2, 0, width / 2, height))
        -- node:setOpacity(0)                                              -- HOCK

        left:runAction(transition.sequence({
            CCMoveBy:create(time, ccp(-offset.x, offset.y)),
            CCCallFunc:create(function() left:removeSelf() end),
        }))

        right:runAction(transition.sequence({
            CCMoveBy:create(time, offset),
            CCCallFunc:create(function() right:removeSelf() end),
        }))

        self:playSound()
        ]]


        -- [Version2]

        --[[
        -- 获得拷贝模板
        local node = self:getNode()

        -- [生成对象]
        -- 定义配置参数
        local prefix1   = "e_f_oth_death0"
        local count1    = 8
        local interval  = self:T(1.0 / 12)                              -- 帧间隔
        local prefix2   = "e_f_oth_death1"
        local count2    = 8

        -- 生成精灵[左]
        local params = {
            fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(prefix1),   -- 文件名
            parentNode  = node,                                         -- 父结点
            align       = display.CENTER,                               -- 对齐方式
            anchor      = display.CENTER,                               -- 锚点
        }
        local node1 = TDUIUtils.loadSprite(params)
        node1:runAction(TDAnimationUtils.sequence({
            TDAnimationUtils.animateWithFrame({
                prefix      = prefix1,
                frameCount  = count1,
                interval    = interval,
            }),
            CCCallFunc:create(function()
                node1:runAction(TDAnimationUtils.animateWithFrameRepeat({
                    prefix      = prefix2,
                    frameCount  = count2,
                    interval    = interva,
                }))

                local prototypeCemetery = self._prototypes:getPrototype(IDX_CEMETERY)
                local time1      =  self:T(.5)                             -- [动画]时间1
                local time2      =  self:T(.3)                             -- [动画]时间2
                node1:runAction(TDAnimationUtils.sequence({
                    TDAnimationUtils.spawn({
                        CCMoveTo:create(time1, prototypeCemetery:getPosition()),     -- 移动到目标位置
                        CCRotateTo:create(time1, prototypeCemetery:getRotation()),   -- 旋转到目标旋转角度
                        CCScaleTo:create(time1, prototypeCemetery:getScale()),       -- 缩放到目标缩放比例
                    }),
                    CCFadeOut:create(time2),
                    CCCallFunc:create(function()
                        node1:removeFromParentAndCleanup(true)
                    end),
                }))
            end),
        }))
        ]]
    end

    -- 移动至墓地
    local function onMoveToCemetery()
        if true then 
            local nodeUp            = self:getNodeUp()
            local nodeDown          = self:getNodeDown()
            local prototypeCemetery = self._prototypes:getPrototype(IDX_CEMETERY)

            -- 初始设置
            self:playSound()
            nodeUp:setVisible(false) 
            nodeUp:setOpacity(255)
            nodeDown:setVisible(true) 
            nodeDown:setColor(nodeDown:getColorDie())                           -- 设置颜色 
            nodeDown:setPosition(nodeUp:getPosition())                          -- 设置位置
            -- 动画实现
            local time      = time4                                             -- [动画]时间4
            local action    = TDAnimationUtils.sequence({
                TDAnimationUtils.spawn({
                    CCMoveTo:create(time, prototypeCemetery:getPosition()),     -- 移动到目标位置
                    CCRotateTo:create(time, prototypeCemetery:getRotation()),   -- 旋转到目标旋转角度
                    CCScaleTo:create(time, prototypeCemetery:getScale()),       -- 缩放到目标缩放比例
                }),
                CCCallFunc:create(function() 
                    nodeDown:setVisible(false) 
                    nodeDown:setIndex(IDX_CEMETERY)  
                    nodeDown:setIndexActual(IDX_CEMETERY)  
                    nodeDown:getParent():glitterCemetery()
                end),
            })
            nodeDown:runAction(action)
        end
    end



    -- 构建动作对象
    action = transition.sequence({
        -- CCCallFunc:create(onCutOff),                    -- cut half
        -- CCFadeTo:create(time1, alpha1),                 -- alpha -
        -- CCFadeTo:create(time2, alpha2),                 -- alpha +
        -- CCFadeTo:create(time3, alpha3),                 -- alpha -
        CCCallFunc:create(onMoveToCemetery),            -- moving position
        CCDelayTime:create(time4),                      -- delay
    })



    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end

-- 获得[动画]时间3
function M:getATime3()
    return self._aTime3
end
-- 设置[动画]时间3
function M:setATime3(time)
    self._aTime3 = time
end

-- 获得[动画]时间4
function M:getATime4()
    return self._aTime4
end
-- 设置[动画]时间3
function M:setATime4(time)
    self._aTime4 = time
end




-- 获得[动画]位移1
function M:getAOffset1()
    return self._aOffset1
end
-- 设置[动画]位移1
function M:setAOffset1(offset)
    self._aOffset1 = offset
end

-- 获得[动画]旋转角度2[攻击旋转－目标]
function M:getARotation2()
    return self._aRotation2
end
-- 设置[动画]旋转角度2[攻击旋转－目标]
function M:setTime2(rotation)
    self._aRotation2 = rotation
end

-- 获得[动画]旋转角度3[攻击旋转－起始]
function M:getARotation3()
    return self._aRotation3
end
-- 设置[动画]旋转角度3[攻击旋转－起始]
function M:setTime3(rotation)
    self._aRotation3 = rotation
end





return M