--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[舞台]下场]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionStageDown", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)


    -- [验证参数合法性]
    self:assertParameters(params)



    -- [参数赋值]
    self._node                  = self._nodeUp                                              -- 目标结点
    self._prototypes            = params.prototypes         or {}                           -- 原型集
    self._toIndex               = params.index                                              -- 索引[目标位置]
    self._delayTimeEnd          = params.delayTimeEnd       or FA__STAGE_UP__D_E            -- 延迟时间[结束]
    self._aTime                 = params.aTime              or FA__STAGE_UP__T              -- [动画]时间
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil
    local isChange = false


    -- 辅助参数
    local originScale       = self:getNode():getScale()                     -- 原始缩放比例


    -- 动画参数
    local topView           = self:getNode():getTopView()                   -- 顶层视图
    local owner             = self:getNode():getOwner()                     -- 拥有者标识
    local indexBase         = topView:getIndexBase()                        -- 索引[基础]



    local nodeUp            = self:getNodeUp()                              -- 上场结点
    local nodeDown          = self:getNodeDown()                            -- 下场结点
    local toIndex           = self._toIndex                                 -- 目标位置
    local toIndexActual     = self._toIndex + indexBase 
    
-- 
    local prototype         = self._prototypes:getPrototype(toIndexActual)  -- 原型对象
    local time              = self:T(self._aTime)                           -- [动画]时间
    local position          = prototype:getPosition()                       -- [动画]目标位置
    local rotation          = prototype:getRotation()                       -- [动画]目标旋转角度
    local scale             = prototype:getScale()                          -- [动画]目标缩放比例



    -- 初始设置
    function onCompleted()
        -- 卡牌[上]浮动
        nodeUp:float()
        -- 卡牌[上]浮动
        nodeDown:float()
        
        -- 隐藏就绪盒子
        nodeDown:showBoxReady()
        -- 隐藏下场卡牌
        nodeDown:show()
        -- 设置下场卡牌颜色
        nodeDown:setColor(self:getNodeDownColor(nodeDown))
        -- 设置索引
        nodeDown:setIndex(toIndex)
        nodeDown:setIndexActual(toIndexActual)
        -- 移除当前卡牌从卡牌数组
        topView:removeFromCardsUp(owner, nodeDown)
        topView:addToCardsDown(owner, nodeDown)
        -- 显示上场卡牌
        nodeUp:hide()
        
        -- 位置重排
        local isChange = self:handleViewSwitch()
    end
    -- -- 校正原型(位置、旋转角度、缩放比例等)
    -- nodeUp:setPosition(ccp(nodeDown:getPosition()))
    -- nodeUp:setRotation(nodeDown:getRotation())
    -- nodeUp:setScale(nodeDown:getScale())


    -- 构建动作对象
    action = TDAnimationUtils.sequence({
        CCDelayTime:create(isChange and FA__BASE_CORRECT__T or 0),
        TDAnimationUtils.spawn({
            CCMoveTo:create(time, position),                -- 位移
            CCRotateTo:create(time, rotation),              -- 旋转角度
            CCScaleTo:create(time, scale),                  -- 缩放比例
        }),
        CCCallFunc:create(onCompleted),
    })


    -- 处理场上卡牌超过显示张数的情况
    



    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]


function M:getNodeDownColor(node)
    return node:getColorAlive() 
end



-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    TDAssertUtils.assertParameters(params)
    -- 验证位置
    self:assertLocation(params)
end

-- 验证合法性[位置]
function M:assertLocation(params)
    -- 如果开启位置自动校正
    if self:isPositionAutoCorrect() then 
        -- 默认赋值
        local args = params or { positions = self._positions, indexBegin = self._indexBegin, indexEnd = self._indexEnd, }

        -- 验证合法性
        TDAssertUtils.assertNotNil(args.positions)
        TDAssertUtils.assert(args.indexBegin >= 0, "Card index [Begin] must be greater than or equal to zero!")
        TDAssertUtils.assert(args.indexEnd   >= 0, "Card index  [End]  must be greater than or equal to zero!")
    end
end



-- [实例方法][属性Getter/Setter]
-- 获得索引[起始位置]
function M:getIndexBegin()
    return self._indexBegin
end

-- 获得索引[结束位置]
function M:getIndexEnd()
    return self._indexEnd
end

-- 获得是否开启位置自动校正（如果开启，则根据索引自动指定位置）
function M:isPositionAutoCorrect()
    return self._isPositionAutoCorrect
end
-- 设置是否开启位置自动校正（如果开启，则根据索引自动指定位置）
function M:setPositionAutoCorrect(isAutoCorrect)
    self._isPositionAutoCorrect = isAutoCorrect
end





return M