--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[舞台]上手牌]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionStageHand", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._e1Sound               = params.e1Sound            or FA__STAGE_HAND__S        -- [声音]
    self._aTime1                = params.aTime1             or FA__STAGE_HAND__T1       -- [动画]时间1
    self._aTime2                = params.aTime2             or FA__STAGE_HAND__T2       -- [动画]时间2
    self._prototypes            = params.prototypes                                     -- 原型集
    self._index                 = params.index                                          -- 目标位置索引


    -- [具体操作]
    -- ..
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil


    -- 辅助参数
    local target           = self._node                                 -- 目标结点



    -- 方法定义
    local function onComplete()
        target:setIndex(self._index)
        target:setIndexActual(self._index)
    end



    -- 动画参数
    local prototype         = self._prototypes:getPrototype(self._index)    -- 原型对象
    local time1             = self:T(self._aTime1)                          -- [动画]时间1
    local time2             = self:T(self._aTime2)                          -- [动画]时间2
    local position2         = prototype:getPosition()                       -- [动画]位置2
    local rotation2         = prototype:getRotation()                       -- [动画]旋转角度2
    local scale2            = prototype:getScale()                          -- [动画]缩放比例2



    -- 初始设置
    target:setVisible(true)                             -- 设置可见性
    target:setScaleX(0)                                 -- 设置缩放比例X
    -- 构建动作对象
    action = TDAnimationUtils.sequence({
        CCScaleTo:create(time1, 1, 1),                  -- 还原缩放比例
        CCCallFunc:create(function() if self._index < IDX_STAGEUP_INC then self:playSound() end end),
        TDAnimationUtils.spawn({
            CCMoveTo:create(time2, position2),              -- 移动到目标位置
            CCRotateTo:create(time2, rotation2),            -- 旋转到目标旋转角度
            CCScaleTo:create(time2, scale2),                -- 缩放到目标缩放比例
        }),
        CCCallFunc:create(onComplete),
    })


    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end





return M