--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[舞台]移动]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionStageMovement", FightActionPosition)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)


    -- [验证参数合法性]
    self:assertParameters(params)



    -- [参数赋值]
    self._positions 				= params.positions                 	or {}               -- 位置集
    self._indexBegin 				= params.indexBegin              	or 0                -- 索引[起始位置]
    self._indexEnd 					= params.indexEnd                 	or 0                -- 索引[结束位置]
    self._isPositionAutoCorrect 	= params.isPositionAutoCorrect 		or true          	-- 是否开启位置自动校正（如果开启，则根据索引自动指定位置）


    -- 设置起始、结束位置
    -- 起始
    self:setIndexBegin(self._indexBegin)
    -- 结束
    self:setIndexEnd(self._indexEnd)
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- 设置索引[起始位置]
function M:setIndexBegin(index)
    self._indexBegin = index

    -- 是否进行位置自动校正
    if self:isPositionAutoCorrect() then
        self:setPositionBegin(self:getPositionBeginExpect())
    end
end

-- 设置索引[结束位置]
function M:setIndexEnd(index)
    self._indexEnd = index

    if self:isPositionAutoCorrect() then
        self:setPositionEnd(self:getPositionEndExpect())
    end
end

-- 获得期待位置[起始位置]
function M:getPositionBeginExpect()
    -- 验证合法性
    self:assertLocation()

    -- 获得位置
    return self._positions[self._indexBegin]
end
-- 获得期待位置[结束位置]
function M:getPositionEndExpect()
    -- 验证合法性
    self:assertLocation()

    -- 获得位置
    return self._positions[self._indexEnd]
end



-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    TDAssertUtils.assertParameters(params)
    -- 验证位置
    self:assertLocation(params)
end

-- 验证合法性[位置]
function M:assertLocation(params)
    -- 如果开启位置自动校正
    if self:isPositionAutoCorrect() then 
        -- 默认赋值
        local args = params or { positions = self._positions, indexBegin = self._indexBegin, indexEnd = self._indexEnd, }

        -- 验证合法性
        TDAssertUtils.assertNotNil(args.positions)
        TDAssertUtils.assert(args.indexBegin >= 0, "Card index [Begin] must be greater than or equal to zero!")
        TDAssertUtils.assert(args.indexEnd   >= 0, "Card index  [End]  must be greater than or equal to zero!")
    end
end



-- [实例方法][属性Getter/Setter]
-- 获得索引[起始位置]
function M:getIndexBegin()
    return self._indexBegin
end

-- 获得索引[结束位置]
function M:getIndexEnd()
    return self._indexEnd
end

-- 获得是否开启位置自动校正（如果开启，则根据索引自动指定位置）
function M:isPositionAutoCorrect()
    return self._isPositionAutoCorrect
end
-- 设置是否开启位置自动校正（如果开启，则根据索引自动指定位置）
function M:setPositionAutoCorrect(isAutoCorrect)
    self._isPositionAutoCorrect = isAutoCorrect
end





return M