--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[舞台]上场]，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("ActionStageUp", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                  = self._nodeUp                                              -- 目标结点
    self._prototypes 			= params.prototypes         or {}                           -- 原型集
    self._isKeepNodeDown        = params.isKeepNodeDown     or false                        -- 是否保持下场结点
    self._toIndex               = params.index                                              -- 索引[目标位置]
    self._delayTimeEnd          = params.delayTimeEnd       or FA__STAGE_UP__D_E            -- 延迟时间[结束]
    self._aTime                 = params.aTime              or FA__STAGE_UP__T              -- [动画]时间
end

    

    





-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil
    -- 位置重排
    local isChange = self:handleViewSwitch()

    -- 辅助参数
    local originScale       = self:getNode():getScale()                     -- 原始缩放比例


    -- 动画参数
    local topView           = self:getNode():getTopView()                   -- 顶层视图
    local owner             = self:getNode():getOwner()                     -- 拥有者标识
    local indexBase         = topView:getIndexBase()                        -- 索引[基础]



    local nodeUp            = self:getNodeUp()                              -- 上场结点
    local nodeDown          = self:getNodeDown()                            -- 下场结点
    local isKeepNodeDown    = self._isKeepNodeDown                          -- 是否保持下场结点
    local toIndex           = self._toIndex                                 -- 目标位置
    local toIndexActual     = self._toIndex + indexBase                     -- 目标位置

    --如果超过15要回到15，上场最多只能显示5张， 而FightCardSetLayer的197行的index是11到15得下标
    if toIndexActual > 15 then
        toIndexActual = 15
    end
--      print("self._toIndex "..self._toIndex.."+ indexBase:"..indexBase)
-- print("ActionStageDown----toIndexActual:"..toIndexActual)
    local prototype         = self._prototypes:getPrototype(toIndexActual)  -- 原型对象
    local time              = self:T(self._aTime)                           -- [动画]时间
    local position          = prototype:getPosition()                       -- [动画]目标位置
    local rotation          = prototype:getRotation()                       -- [动画]目标旋转角度
    local scale             = prototype:getScale()                          -- [动画]目标缩放比例




    -- 初始设置
    function aaa()
        -- 卡牌[上]浮动
        nodeUp:float()
        -- 卡牌[上]浮动
        nodeDown:float()
        
        -- 隐藏就绪盒子
        nodeDown:hideBoxReady()
        -- 隐藏下场卡牌
        nodeDown:setVisible(isKeepNodeDown)
        -- 设置下场卡牌颜色
        nodeDown:setColor(self:getNodeDownColor(nodeDown, isKeepNodeDown))
        -- 显示上场卡牌
        nodeUp:setVisible(true)
        -- 校正原型(位置、旋转角度、缩放比例等)
        nodeUp:setPosition(ccp(nodeDown:getPosition()))
        nodeUp:setRotation(nodeDown:getRotation())
        nodeUp:setScale(nodeDown:getScale())
    end


    -- 构建动作对象
    action = TDAnimationUtils.sequence({
        CCCallFunc:create(aaa),
        CCShow:create(),
        CCDelayTime:create(isChange and FA__BASE_CORRECT__T or 0),
        TDAnimationUtils.spawn({
            CCMoveTo:create(time, position),                -- 位移
            CCRotateTo:create(time, rotation),              -- 旋转角度
            CCScaleTo:create(time, scale),                  -- 缩放比例
        }),
        CCCallFunc:create(function()
            nodeUp:setIndex(toIndex)
            nodeUp:setIndexActual(toIndexActual)

            topView:addToCardsUp(owner, nodeDown)          --添加到上场卡牌
            topView:removeFromCardsDown(owner, nodeDown)

               
        end)
    })


    -- 处理场上卡牌超过显示张数的情况
    



    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]



-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- [加载超类函数]
    M.super.assertParameters(self, params)
end


-- [侦听事件]
-- 处理点击事件处理函数
function M:onTouch() 
    if self:isVisible() == false then
        -- print("onStageDownTouched: visible hidden!")
        return 
    end

    -- 显示卡片详细信息
    self:showCardDetail()
end

-- 显示卡片详细信息
function M:showCardDetail()
    local topView = self:getParent():getParent():getParent()

    local layer = CardDetailLayer.new({poCard = self._po})
    topView:addChild(layer)
end

function M:getNodeDownColor(node, isKeep)
    return isKeep and node:getColorStateUp() or node:getColorAlive() 
end



-- [实例方法][属性Getter/Setter]
-- 设置动画[时间]
function M:setATime(aTime)
    self._aTime = aTime
end
-- 获得动画[时间]
function M:getATime()
    return self._aTime
end





return M