--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[系统]校正冷却时间]，定义战斗动作相关操作方法及逻辑实现。

-   定义切换回合动作方法及成员变量。

]]

local M = class("ActionSystemCorrectCD", AbstractAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._node                      = params.node                                           -- 目标结点[任意]
    self._cardsMe                   = params.cardsMe                                        -- 我方卡片集
    self._cardsMeEnd                = params.cardsMeEnd                                     -- 我方卡片集[结束]
    self._cardsOpp                  = params.cardsOpp                                       -- 敌方卡片集
    self._cardsOppEnd               = params.cardsOppEnd                                    -- 敌方卡片集[结束]
    self._turn                      = params.turn                                           -- 轮换标识
    self._isAnimate                 = params.isAnimate      or FA__S_CORRECT_CD__A_IS       -- 动画切换效果标识(注：当该值为false时，则直接更改CD; 为true时，则动画切换)
    self._aTime1                    = params.aTime1         or FA__S_CORRECT_CD__A_T1       -- 动画时间1
    self._aTime2                    = params.aTime2         or FA__S_CORRECT_CD__A_T2       -- 动画时间2
    self._roundNo                   = params.roundNo                                        -- 回合数
end



    





-- [抽象方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil


    -- 动画参数
    local isAutoFight       = self:getTopView():isAutoFight()                   -- 自动战斗标识
    local isDataReceiveAll  = self:getTopView():isDataReceiveAll()              -- 获得所有数据标识
    local isLastRound       = self:getTopView():isLastRound(self._roundNo)      -- 最后一轮标识
    local isShowBoxReady    = (not isAutoFight) and (not isDataReceiveAll) and (self._turn == TURN_ME) and isLastRound  -- 是否显示就绪盒子
    local isAnimate         = self._isAnimate                                   -- 是否动画显示
    local time1             = self:T(self._aTime1)                              -- 时间1
    local time2             = self:T(self._aTime2)                              -- 时间2
    local cdsMe             = self:getCDsMe()                                   -- 我方CD时间
    local cdsOpp            = self:getCDsOpp()                                  -- 敌方CD时间


    -- 辅助方法
    -- 非动画显示
    local function onNonAnimate()
        -- 我方卡牌
        local cards = self._cardsMe
        local cds   = cdsMe
        if TDArrayUtils.isNotEmpty(cards) then
            for sn in pairs(cards) do
                local card = cards[sn]
                local cd   = cds[sn]

                card:render({ isShowBoxReady = isShowBoxReady, waitRounds = cd })
            end
        end

        -- 敌方卡牌
        local cards = self._cardsOpp
        local cds   = cdsOpp
        if TDArrayUtils.isNotEmpty(cards) then
            for sn in pairs(cards) do
                local card = cards[sn]
                local cd   = cds[sn]

                card:render({ isShowBoxReady = false, waitRounds = cd })
            end
        end
    end

    -- 动画显示
    local function onAnimate()
        onNonAnimate()
    end



    -- 构建动作对象
    if isAnimate then 
        action = transition.sequence({
            CCCallFunc:create(onAnimate),                               -- 动画
        })
    else
        action = transition.sequence({
            CCCallFunc:create(onNonAnimate),                            -- 非动画
        })
    end

    return action
end




    





-- [实例方法]
-- [实例方法][内部调用]
-- 获得顶层视图
function M:getTopView()
    return self._node:getParent():getParent()
end



-- [实例方法][动画]
-- 获得CD时间集[我方卡牌]
function M:getCDsMe()
    return self:getCDs(OWNER_ME)
end

-- 获得CD时间集[敌方卡牌]
function M:getCDsOpp()
    return self:getCDs(OWNER_OPPONENT)
end

-- 获得CD时间集[我方卡牌]
function M:getCDs(owner)
    local cds = {}

    local cards = (owner == OWNER_ME) and self._cardsMeEnd or self._cardsOppEnd
    if TDArrayUtils.isNotEmpty(cards) then 
        for _, card in pairs(cards) do
            local sn   = card:getSn()
            local cd   = card:getCd()

            cds[sn] = cd
        end
    end

    return cds
end



-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- [加载超类函数]
    M.super.assertParameters(self, params)
end



-- [实例方法][属性Getter/Setter]
-- 获得我方卡片集
function M:getCardsMe()
    return self._cardsMe
end
-- 设置我方卡片集
function M:setCardsMe(cards)
    self._cardsMe = cards
end

function M:getCardsMeEnd()
    return self._cardsMeEnd
end
-- 设置我方卡片集[结束]
function M:setCardsMeEnd(cards)
    self._cardsMeEnd = cards
end

-- 获得敌方卡片集
function M:getCardsOpp()
    return self._cardsOpp
end
-- 设置敌方卡片集
function M:setCardsOpp(cards)
    self._cardsOpp = cards
end

-- 获得敌方卡片集[结束]
function M:getCardsOppEnd()
    return self._cardsOppEnd
end
-- 设置敌方卡片集[结束]
function M:setCardsOppEnd(cards)
    self._cardsOppEnd = cards
end

-- 获得是否动画显示
function M:isAnimate()
    return self._isReassignPositionEnd
end
-- 设置是否动画显示
function M:setAnimate(isAnimate)
    self._isAnimate = isAnimate
end

-- 获得动画时间1
function M:getATime1()
    return self._aTime1
end
-- 设置动画时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得动画时间2
function M:getATime2()
    return self._aTime2
end
-- 设置动画时间2
function M:setATime2(time)
    self._aTime2 = time
end






return M