--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[系统]重新排列位置]，定义战斗动作相关操作方法及逻辑实现。

-   定义动作方法及成员变量。

]]

local M = class("ActionSystemRearrangePositions", AbstractAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- 动画时间
M.T_CARDS_DELAY                             = .1
M.T_MOVE                                    = .1


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._turn              = params.turn                                       -- 回合标识
    self._prototypesMe      = params.prototypesMe                               -- 我方卡片原型集
    self._prototypesOpp     = params.prototypesOpp                              -- 敌方卡片原型集
    self._cardsMe           = params.cardsMe                                    -- 我方卡片集
    self._cardsMeEnd        = params.cardsMeEnd                                 -- 我方卡片集[结束]
    self._cardsOpp          = params.cardsOpp                                   -- 敌方卡片集
    self._cardsOppEnd       = params.cardsOppEnd                                -- 敌方卡片集[结束]
end

    

    





-- [抽象方法]
-- 运行动作
function M:buildAction(params)
    --[[Version1:]]
    --[[
    -- 定义动作对象
    local action = nil



    -- 动画参数
    -- ..



    -- 辅助变量
    local turn              = self._turn                                 -- 回合标识
    local delayMe           = 0
    local delayOpp          = 0

    -- 定义延迟时间
    if turn == TURN_ME then 
        delayMe   = 0
        delayOpp  = M.T_CARDS_DELAY
    elseif turn == TURN_OPPONENT then 
        delayMe   = M.T_CARDS_DELAY
        delayOpp  = 0
    end


    local cardsMe       = self._cardsMe
    local cardsMeEnd    = self._cardsMeEnd
    local cardsOpp      = self._cardsOpp
    local cardsOppEnd   = self._cardsOppEnd
    local prototypesMe  = self._prototypesMe
    local prototypesOpp = self._prototypesOpp


    local cardsActual = cardsMe
    local cardsExpect = cardsMeEnd
    local prototypes  = prototypesMe
    local delay       = delayMe
    if TDArrayUtils.isNotEmpty(cardsActual) and TDArrayUtils.isNotEmpty(cardsExpect) then 
        for i,v in ipairs(cardsExpect) do
            local sn = cardsExpect[i]:getSn()
            local index = tonumber(string.sub(sn, 2))

            local card = cardsActual[sn]

            local cardIndexActual = card:getIndex()
            local cardIndexExpect = cardsExpect[i]:getPo()

            if cardIndexExpect ~= cardIndexActual then 
                local prototype      = prototypes:getPrototype(cardIndexExpect)
                local positionExpect = prototype:getPosition()
                local rotationExpect = prototype:getRotation()
                local scaleExpect    = prototype:getScale()
                local action = transition.sequence({ 
                    CCDelayTime:create(delay),
                    TDAnimationUtils.sequence({
                        CCMoveTo:create(M.T_MOVE, positionExpect),              -- 移动到目标位置
                        CCRotateTo:create(M.T_MOVE, rotationExpect),            -- 旋转到目标旋转角度
                        CCScaleTo:create(M.T_MOVE, scaleExpect),                -- 缩放到目标缩放比例
                    }),
                })

                if cardIndexExpect > IDX_CEMETERY then 
                    card:getAssociatedCard():setIndex(cardIndexExpect)
                    card:getAssociatedCard():runAction(action)
                else 
                    card:setIndex(cardIndexExpect)
                    card:runAction(action)
                end
            end
        end
    end

    local cardsActual = cardsOpp
    local cardsExpect = cardsOppEnd
    local prototypes  = prototypesOpp
    local delay       = delayOpp
    if TDArrayUtils.isNotEmpty(cardsOpp) and TDArrayUtils.isNotEmpty(cardsOppEnd) then 
        for i,v in ipairs(cardsExpect) do
            local sn = cardsExpect[i]:getSn()
            local index = tonumber(string.sub(sn, 2))

            local card = cardsActual[sn]

            local cardIndexActual = card:getIndex()
            local cardIndexExpect = cardsExpect[i]:getPo()

            if cardIndexExpect ~= cardIndexActual then 
                local prototype      = prototypes:getPrototype(cardIndexExpect)
                local positionExpect = prototype:getPosition()
                local rotationExpect = prototype:getRotation()
                local scaleExpect    = prototype:getScale()
                local action = transition.sequence({ 
                    CCDelayTime:create(delay),
                    TDAnimationUtils.sequence({
                        CCMoveTo:create(M.T_MOVE, positionExpect),              -- 移动到目标位置
                        CCRotateTo:create(M.T_MOVE, rotationExpect),            -- 旋转到目标旋转角度
                        CCScaleTo:create(M.T_MOVE, scaleExpect),                -- 缩放到目标缩放比例
                    }),
                })
                
                if cardIndexExpect > IDX_CEMETERY then 
                    card:getAssociatedCard():setIndex(cardIndexExpect)
                    card:getAssociatedCard():runAction(action)
                else 
                    card:setIndex(cardIndexExpect)
                    card:runAction(action)
                end
            end
        end
    end

    -- 构建动作对象
    action = CCDelayTime:create(M.T_CARDS_DELAY + M.T_MOVE)

    return action
    ]]



    
    --[[Version2:]]
    --[[
    --]]
    -- 全局设置
    local topView       = self._cardsMe["m1"]:getTopView()
    local indexBase     = 0
    topView:setIndexBase(indexBase)


    -- 定义动作对象
    local action = nil



    -- 动画参数
    -- ..



    -- 辅助变量
    local turn              = self._turn                                 -- 回合标识
    local delayMe           = 0
    local delayOpp          = 0
    local time              = self:T(M.T_MOVE)

    -- 定义延迟时间
    if turn == TURN_ME then 
        delayMe   = 0
        delayOpp  = M.T_CARDS_DELAY
    elseif turn == TURN_OPPONENT then 
        delayMe   = M.T_CARDS_DELAY
        delayOpp  = 0
    end


    local cardsMe       = self._cardsMe
    local cardsMeEnd    = self._cardsMeEnd
    local cardsOpp      = self._cardsOpp
    local cardsOppEnd   = self._cardsOppEnd
    local prototypesMe  = self._prototypesMe
    local prototypesOpp = self._prototypesOpp


    local cardsActual = cardsMe
    local cardsExpect = cardsMeEnd
    local prototypes  = prototypesMe
    local delay       = delayMe
    if TDArrayUtils.isNotEmpty(cardsActual) and TDArrayUtils.isNotEmpty(cardsExpect) then 
        for i,v in ipairs(cardsExpect) do
            local sn = cardsExpect[i]:getSn()
            local index = tonumber(string.sub(sn, 2))

            local card = cardsActual[sn]

            local cardIndexActual = card:getIndex()
            local cardIndexExpect = cardsExpect[i]:getPo()

            -- print("indexExcept: ", cardIndexExpect + indexBase)
            -- 设置索引[实际]
            card:setIndexActual(cardIndexExpect + indexBase)
-- print("设置索引2"..cardIndexExpect + indexBase)
            if cardIndexExpect ~= cardIndexActual then 
                local action
                if cardIndexExpect % IDX_CEMETERY <= NUM_FIGHT_VIEW then 
                    local prototype      = prototypes:getPrototype(cardIndexExpect)
                    local positionExpect = prototype:getPosition()
                    local rotationExpect = prototype:getRotation()
                    local scaleExpect    = prototype:getScale()
                    action = transition.sequence({ 
                        CCShow:create(),
                        CCDelayTime:create(delay),
                        TDAnimationUtils.sequence({
                            CCMoveTo:create(time, positionExpect),              -- 移动到目标位置
                            CCRotateTo:create(time, rotationExpect),            -- 旋转到目标旋转角度
                            CCScaleTo:create(time, scaleExpect),                -- 缩放到目标缩放比例
                        }),
                    })
                else
                    action = transition.sequence({ 
                        CCHide:create(),
                    })
                end

                if cardIndexExpect > IDX_CEMETERY then 
                    card:getAssociatedCard():setIndex(cardIndexExpect)
                    card:getAssociatedCard():runAction(action)
                else 
                    card:setIndex(cardIndexExpect)
                    card:runAction(action)
                end
            end
        end
    end

    local cardsActual = cardsOpp
    local cardsExpect = cardsOppEnd
    local prototypes  = prototypesOpp
    local delay       = delayOpp
    if TDArrayUtils.isNotEmpty(cardsOpp) and TDArrayUtils.isNotEmpty(cardsOppEnd) then 
        for i,v in ipairs(cardsExpect) do
            local sn = cardsExpect[i]:getSn()
            local index = tonumber(string.sub(sn, 2))

            local card = cardsActual[sn]

            local cardIndexActual = card:getIndex()
            local cardIndexExpect = cardsExpect[i]:getPo()

            -- 设置索引[实际]
            card:setIndexActual(cardIndexExpect + indexBase)

            if cardIndexExpect ~= cardIndexActual then 
                local action
                if cardIndexExpect % IDX_CEMETERY <= NUM_FIGHT_VIEW then 
                    local prototype      = prototypes:getPrototype(cardIndexExpect)
                    local positionExpect = prototype:getPosition()
                    local rotationExpect = prototype:getRotation()
                    local scaleExpect    = prototype:getScale()
                    action = transition.sequence({ 
                        CCShow:create(),
                        CCDelayTime:create(delay),
                        TDAnimationUtils.sequence({
                            CCMoveTo:create(time, positionExpect),              -- 移动到目标位置
                            CCRotateTo:create(time, rotationExpect),            -- 旋转到目标旋转角度
                            CCScaleTo:create(time, scaleExpect),                -- 缩放到目标缩放比例
                        }),
                    })
                else
                    action = transition.sequence({ 
                        CCHide:create(),
                    })
                end
                
                if cardIndexExpect > IDX_CEMETERY then 
                    card:getAssociatedCard():setIndex(cardIndexExpect)
                    card:getAssociatedCard():runAction(action)
                else 
                    card:setIndex(cardIndexExpect)
                    card:runAction(action)
                end
            end
        end
    end

    -- 构建动作对象
    action = CCDelayTime:create(M.T_CARDS_DELAY + M.T_MOVE)

    return action



    --[[Version3:]]
    --[[
    -- 全局设置
    local topView       = self._cardsMe["m1"]:getTopView()
    local indexBase     = 0
    topView:setIndexBase(indexBase)


    -- 定义动作对象
    local action = nil



    -- 动画参数
    -- ..



    -- 辅助变量
    local turn              = self._turn                                 -- 回合标识
    local delayMe           = 0
    local delayOpp          = 0
    local time              = M.T_MOVE

    -- 定义延迟时间
    if turn == TURN_ME then 
        delayMe   = 0
        delayOpp  = M.T_CARDS_DELAY
    elseif turn == TURN_OPPONENT then 
        delayMe   = M.T_CARDS_DELAY
        delayOpp  = 0
    end


    local cardsMe       = self._cardsMe
    local cardsMeEnd    = self._cardsMeEnd
    local cardsOpp      = self._cardsOpp
    local cardsOppEnd   = self._cardsOppEnd
    local prototypesMe  = self._prototypesMe
    local prototypesOpp = self._prototypesOpp


    local cardsActual = cardsMe
    local cardsExpect = cardsMeEnd
    local prototypes  = prototypesMe
    local delay       = delayMe
    if TDArrayUtils.isNotEmpty(cardsActual) and TDArrayUtils.isNotEmpty(cardsExpect) then 
        for i,v in ipairs(cardsExpect) do
            local sn = cardsExpect[i]:getSn()
            local index = tonumber(string.sub(sn, 2))

            local card = cardsActual[sn]

            local cardIndexActual = card:getIndex()
            local cardIndexExpect = cardsExpect[i]:getPo()

            if cardIndexExpect ~= cardIndexActual then 

                if cardIndexExpect > IDX_CEMETERY then 
                    card:getAssociatedCard():setIndex(cardIndexExpect)
                    -- card:getAssociatedCard():runAction(action)
                else 
                    local prototype      = prototypes:getPrototype(cardIndexExpect)
                    local positionExpect = prototype:getPosition()
                    local rotationExpect = prototype:getRotation()
                    local scaleExpect    = prototype:getScale()
                    local action = transition.sequence({ 
                        CCDelayTime:create(delay),
                        TDAnimationUtils.sequence({
                            CCMoveTo:create(M.T_MOVE, positionExpect),              -- 移动到目标位置
                            CCRotateTo:create(M.T_MOVE, rotationExpect),            -- 旋转到目标旋转角度
                            CCScaleTo:create(M.T_MOVE, scaleExpect),                -- 缩放到目标缩放比例
                        }),
                    })

                    card:setIndex(cardIndexExpect)
                    card:runAction(action)
                end
            end
        end

        local function rerange(owner)
            local cardsUp     = topView:getCardsUp(owner)
            local prototypes  = topView:getPlayerPrototypes(owner)
            -- 移动逻辑处理
            for i, card in ipairs(cardsUp) do
                local nodeUpCurrent     = card:getAssociatedCard()
                local index             = nodeUpCurrent:getIndexActual()                         -- 索引[当前] nodeUpCurrent:getIndex()
                local iActual           = nodeUpCurrent:getIndex() + indexBase             -- 索引[实际]

                local action
                if i <= NUM_FIGHT_VIEW then 
                    -- if index ~= iActual then 
                        local prototype         = prototypes:getPrototype(iActual)
                        local position          = prototype:getPosition()                       -- [动画]目标位置
                        local rotation          = prototype:getRotation()                       -- [动画]目标旋转角度
                        local scale             = prototype:getScale()                          -- [动画]目标缩放比例
                        -- 构建动作对象
                        action = TDAnimationUtils.sequence({
                            TDAnimationUtils.spawn({
                                CCShow:create(),                                -- 显示
                                CCMoveTo:create(time, position),                -- 位移
                                CCRotateTo:create(time, rotation),              -- 旋转角度
                                CCScaleTo:create(time, scale),                  -- 缩放比例
                            }),
                            CCCallFunc:create(function()
                                nodeUpCurrent:setIndexActual(iActual)
                            end)
                        })
                        nodeUpCurrent:runAction(action)
                    -- end
                else
                    action = TDAnimationUtils.sequence({
                        CCHide:create(),
                        CCCallFunc:create(function()
                            nodeUpCurrent:setIndexActual(iActual)
                        end)
                    })
                    nodeUpCurrent:runAction(action)
                end

            end
        end

        -- 我方卡牌
        rerange(OWNER_ME)
        -- 敌方卡牌
        rerange(OWNER_OPPONENT)
    end

    local cardsActual = cardsOpp
    local cardsExpect = cardsOppEnd
    local prototypes  = prototypesOpp
    local delay       = delayOpp
    if TDArrayUtils.isNotEmpty(cardsOpp) and TDArrayUtils.isNotEmpty(cardsOppEnd) then 
        for i,v in ipairs(cardsExpect) do
            local sn = cardsExpect[i]:getSn()
            local index = tonumber(string.sub(sn, 2))

            local card = cardsActual[sn]

            local cardIndexActual = card:getIndex()
            local cardIndexExpect = cardsExpect[i]:getPo()

            if cardIndexExpect ~= cardIndexActual then 
                if cardIndexExpect > IDX_CEMETERY then 
                    card:getAssociatedCard():setIndex(cardIndexExpect)
                    -- card:getAssociatedCard():runAction(action)
                else 

                    local prototype      = prototypes:getPrototype(cardIndexExpect)
                    local positionExpect = prototype:getPosition()
                    local rotationExpect = prototype:getRotation()
                    local scaleExpect    = prototype:getScale()
                    local action = transition.sequence({ 
                        CCDelayTime:create(delay),
                        TDAnimationUtils.sequence({
                            CCMoveTo:create(M.T_MOVE, positionExpect),              -- 移动到目标位置
                            CCRotateTo:create(M.T_MOVE, rotationExpect),            -- 旋转到目标旋转角度
                            CCScaleTo:create(M.T_MOVE, scaleExpect),                -- 缩放到目标缩放比例
                        }),
                    })
                    card:setIndex(cardIndexExpect)
                    card:runAction(action)
                end
            end
        end

        local function rerange(owner)
            local cardsUp     = topView:getCardsUp(owner)
            local prototypes  = topView:getPlayerPrototypes(owner)
            -- 移动逻辑处理
            for i, card in ipairs(cardsUp) do
                local nodeUpCurrent     = card:getAssociatedCard()
                local index             = nodeUpCurrent:getIndexActual()                         -- 索引[当前] nodeUpCurrent:getIndex()
                local iActual           = nodeUpCurrent:getIndex() + indexBase             -- 索引[实际]

                local action
                if i <= NUM_FIGHT_VIEW then 
                    -- if index ~= iActual then 
                        local prototype         = prototypes:getPrototype(iActual)
                        local position          = prototype:getPosition()                       -- [动画]目标位置
                        local rotation          = prototype:getRotation()                       -- [动画]目标旋转角度
                        local scale             = prototype:getScale()                          -- [动画]目标缩放比例
                        -- 构建动作对象
                        action = TDAnimationUtils.sequence({
                            TDAnimationUtils.spawn({
                                CCShow:create(),                                -- 显示
                                CCMoveTo:create(time, position),                -- 位移
                                CCRotateTo:create(time, rotation),              -- 旋转角度
                                CCScaleTo:create(time, scale),                  -- 缩放比例
                            }),
                            CCCallFunc:create(function()
                                nodeUpCurrent:setIndexActual(iActual)
                            end)
                        })
                        nodeUpCurrent:runAction(action)
                    -- end
                else
                    action = TDAnimationUtils.sequence({
                        CCHide:create(),
                        CCCallFunc:create(function()
                            nodeUpCurrent:setIndexActual(iActual)
                        end)
                    })
                    nodeUpCurrent:runAction(action)
                end
            end
        end

        -- 我方卡牌
        rerange(OWNER_ME)
        -- 敌方卡牌
        rerange(OWNER_OPPONENT)
    end

    -- 构建动作对象
    action = CCDelayTime:create(M.T_CARDS_DELAY + M.T_MOVE)

    return action
    ]]
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..



-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- [加载超类函数]
    M.super.assertParameters(self, params)
end



-- [实例方法][属性Getter/Setter]
-- ..





return M