--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[系统]状态刷新]，定义战斗动作相关操作方法及逻辑实现。

-   定义动作方法及成员变量。

]]

local M = class("ActionSystemStateRenew", AbstractAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._cardsMe           = params.cardsMe                                    -- 我方卡牌集
    self._cardsMeEnd        = params.cardsMeEnd                                 -- 我方卡牌集[结束]
    self._cardsOpp          = params.cardsOpp                                   -- 敌方卡牌集
    self._cardsOppEnd       = params.cardsOppEnd                                -- 敌方卡牌集[结束]
    self._result            = params.result                                     -- 战斗结果
    self._turn              = params.turn                                       -- 回合数
end

    







-- [抽象方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- 定义函数
    local function onRenewCards(owner)
        -- 辅助变量
        local view          = self._cardsMe["m1"]:getTopView()  -- 视图
        local cardsAll
        local cardsEnd
        if owner == OWNER_ME then
            cardsAll      = self._cardsMe                     -- 卡牌集[所有]
            cardsEnd      = self._cardsMeEnd                  -- 卡牌集[结束]
        else
            cardsAll      = self._cardsOpp                    -- 卡牌集[所有]
            cardsEnd      = self._cardsOppEnd                 -- 卡牌集[结束]
        end

        -- 重置数据
        local cardsUp = {}
        local cardsDown = {}
        -- 刷新数据
        for _, card in ipairs(cardsEnd) do
            local sn        = card:getSn()
            local index     = card:getPo()

            if index > IDX_STAGEUP_INC then 
                cardsUp[#cardsUp + 1] = cardsAll[sn]
            else
                cardsDown[#cardsDown + 1] = cardsAll[sn]
            end
        end

        -- 设置卡牌集[场上]
        view:setCardsUp(owner, cardsUp)
        -- 设置卡牌集[场下]
        view:setCardsDown(owner, cardsDown)

        -- print("up=======>")
        -- for i,v in ipairs(cardsUp) do
        --     print(i, v:getAssociatedCard():getIndex())
        -- end
        -- print("downp=======>")
        -- for i,v in ipairs(cardsDown) do
        --     print(i, v:getIndex())
        -- end
    end

    -- 处理新手引导
    local function onHelpJudge()
        -- 辅助变量
        local view          = self._cardsMe["m1"]:getTopView()  -- 视图
        view:helpJudgeEndRound()
    end

    -- 刷新
    local function onRenew(owner)
        -- 处理卡牌更新[我方]
        onRenewCards(OWNER_ME)
        -- 处理卡牌更新[敌方]
        onRenewCards(OWNER_OPPONENT)
        -- 处理新手引导
        onHelpJudge()
    end

    -- 定义动作
    action = CCCallFunc:create(onRenew)

    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..



-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- [加载超类函数]
    M.super.assertParameters(self, params)
end



-- [实例方法][属性Getter/Setter]
-- ..





return M