--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[[系统]切换回合]，定义战斗动作相关操作方法及逻辑实现。

-   定义切换回合动作方法及成员变量。

]]

local M = class("ActionSystemTurnRound", AbstractAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._round                     = params.node                                           -- 回合数
    -- self._node                      = params.node:getLabel()                                -- 回合数标签（仅标签翻转，如果注释此行，则整个标签所在窗口一并翻转）
    self._roundNo                   = params.roundNo                                        -- 回合编号
    self._cards                     = params.cards                                          -- 回合卡片集
    self._aTime1                    = params.aTime1         or FA__S_TURN_ROUND__A_T1       -- 动画时间1
    self._aTime2                    = params.aTime2         or FA__S_TURN_ROUND__A_T2       -- 动画时间2
end

    

    





-- [抽象方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- 辅助方法
    local function onHandleTurn()
        -- 更新回合数
        self._round:setNo(self._roundNo)


        ----------------------------------------------------
        -- 判断如果是自动战斗，则递减冷却时间，不显示就结盟盒子
        ----------------------------------------------------
        --[[
        -- 处理自动战斗
        if self:getTopView():isAutoFight() then 
            local cards = self._cards

            if TDArrayUtils.isNotEmpty(cards) then
                for _, card in pairs(cards) do
                    card:decreaseWaitRounds()
                    card:render({ isShowBoxReady = false })
                end
            end
        end
        ]]
    end



    -- 动画参数
    local time1             = self:T(self._aTime1)                      -- 时间1
    local time2             = self:T(self._aTime2)                      -- 时间2
    local scaleOrigin       = self._node:getScale()                     -- 缩放比例[原]
    local scaleBegin        = 0                                         -- 缩放比例[起始]
    local scaleEnd          = scaleOrigin                               -- 缩放比例[结束]



    -- 构建动作对象
    action = transition.sequence({
        CCScaleTo:create(time1,  scaleOrigin, scaleBegin),              -- 缩放到0
        CCCallFunc:create(onHandleTurn),                                -- 重新设置回合数
        CCScaleTo:create(time2,  scaleEnd),                             -- 缩放到原始比例
    })

    return action
end




    





-- [实例方法]
-- [实例方法][内部调用]
-- 获得顶层视图
function M:getTopView()
    return self._round:getParent():getParent()
end



-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- [加载超类函数]
    M.super.assertParameters(self, params)
end



-- [实例方法][属性Getter/Setter]
-- 获得回合编号
function M:getRoundNo()
    return self._roundNo
end
-- 设置回合编号
function M:setRoundNo(roundNo)
    self._roundNo = roundNo
end

-- 获得自动战斗标识
function M:isAutoFight()
    return self._isAutoFight
end
-- 设置自动战斗标识
function M:setAutoFight(isAuto)
    self._isAutoFight = iaAuto
end

-- 获得卡牌集
function M:getCards()
    return self._cards
end
-- 设置卡牌集
function M:setCards(cards)
    self._cards = iaAuto
end

-- 获得动画时间1
function M:getATime1()
    return self._aTime1
end
-- 设置动画时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得动画时间2
function M:getATime2()
    return self._aTime2
end
-- 设置动画时间2
function M:setATime2(time)
    self._aTime2 = time
end




return M