--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

效果卡片动作[单阶段]类，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("EffectActionStep1", AbstractEffectFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._aTime1                = params.aTime1                     -- [动画]时间
    self._e1Z                   = params.e1Z                        -- [效果]层深度 
    self._e1Sound               = params._e1Sound                   -- [声音]
    self._e1Delay               = params.e1Delay        or 0        -- [效果]延迟
    self._e1Prefix              = params.e1Prefix                   -- [效果]帧前缀
    self._e1Num                 = params.e1Num                      -- [效果]帧数量
    self._e1Interval            = params.e1Interval                 -- [效果]帧间隔
    self._e1OffsetStart         = params.e1OffsetStart              -- [效果]位移起始
    self._e1OffsetEnd           = params.e1OffsetEnd                -- [效果]位移结束
    self._isSustained           = params.isSustained    or false    -- [效果]持续标识（如果持续，则播放至目标位置后动画不进行消除）
    self._isRecord              = params.isRecord       or false    -- [效果]是否记录
    self._recordTag             = params.recordTag                  -- [效果]动作记录标识
end




    





-- [实例方法]
-- [实例方法][资源]
-- 获得预加载的资源[图片]
function M:getResourcesImage(params)
    return {
        params._e1Resource,
    }
end

-- 获得预加载的资源[音乐]
function M:getResourcesSound(params)
    return {
        params._e1Sound,
    }
end


---------


-- [实例方法][卡片]
-- 构建卡片动作对象
function M:buildActionCard(params)
    local action

    -- 动画参数
    local time1             = self:T(self._aTime1)                  -- [动画]时间1


    -- 构建动作对象
    action = transition.sequence({
        CCCallFunc:create(function() self:playSound() end),         -- play effect
        CCDelayTime:create(time1),                  -- delay
    })

    return action
end


---------


-- [实例方法][效果]
-- 播放效果
function M:playEffect()
    -- [配置定义]
    local IDX_EFFECT  = 1
    local IDX_ACTION  = 1

-- print("播放效果啦。。。。。。。。。。。。。。。"..self._e1Prefix)
    -- [构建实体]
    if self:getEffect(IDX_EFFECT) == nil then 
        -- [效果][起始动画]
        local prefix            = self._e1Prefix                                -- 帧前缀
        local frameCount        = self._e1Num                                   -- 动画帧数
        local interval          = self._e1Interval                              -- 帧间隔
        local z                 = self._e1Z                                     -- 层深度
        local params            = {
            prefix      = prefix,
            frameCount  = frameCount,
            interval    = interval,
        }
        -- print("播放效果")
        self:setEffectAction(IDX_EFFECT, IDX_ACTION, TDAnimationUtils.animateWithFrameRepeat(params))


        -- 定义配置参数
        local paramsCtor = {
            fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(prefix),     -- 文件名
            parentNode  = self:getNode(),                                       -- 父结点
            align       = display.CENTER,                                       -- 对齐方式
            anchor      = display.CENTER,                                       -- 锚点
            z           = z,                                                    -- 层深度
        }
        -- 生成精灵
        self:setEffect(IDX_EFFECT, TDUIUtils.loadSprite(paramsCtor))
    end

    

    -- [动画定义]
    local effect            = self:getEffect(IDX_EFFECT)                        -- 效果
    local action            = self:getEffectAction(IDX_EFFECT, IDX_ACTION)      -- 动作
    local delay             = self:T(self._e1Delay)                             -- 延迟时间
    local time              = self._e1Interval * self._e1Num                    -- 时间
    local positionOrigin    = ccp(effect:getPosition())                         -- 位置[原]
    local positionStart     = ccpAdd(positionOrigin, self._e1OffsetStart)       -- 位置[起始]
    local positionEnd       = ccpAdd(positionOrigin, self._e1OffsetEnd)         -- 位置[结束]
    local function onCompleted() 
        -- 是否持续判断
        effect:setVisible(self._isSustained)
        -- 是否记录判断
        if self._isRecord then 
            local actions = self:getNode():getActions()
            if actions[self._recordTag]  ~= nil then
                local effects = actions[self._recordTag]
                for k in pairs(effects) do
                    effects[k]:removeFromParentAndCleanup(true)
                end
            end
            actions[self._recordTag] = { [1] = effect, }
        end

        self:onCompletedEffect() 
    end
    effect:setVisible(true)
    effect:setPosition(positionStart)

    if delay == 0 then 
        effect:runAction(action)
        effect:runAction(TDAnimationUtils.sequence({
            CCMoveTo:create(time, positionEnd),             -- move 
            CCCallFunc:create(onCompleted),                 -- cleanup
        }))
    else
        effect:runAction(TDAnimationUtils.sequence({ action, CCDelayTime:create(delay) }))
        effect:runAction(TDAnimationUtils.sequence({
            CCDelayTime:create(delay),
            CCMoveTo:create(time, positionEnd),             -- move 
            CCCallFunc:create(onCompleted),                 -- cleanup
        }))
    end
end

-- 处理效果结束后的逻辑
function M:onCompletedEffect()
    -- Implemented by sub-class!
end


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[效果]层深度
function M:getE1Z()
    return self._e1Z
end
-- 设置[效果]层深度
function M:setE1Z(z)
    self._e1Z = z
end

-- 获得[效果]资源
function M:getE1Resource()
    return self._e1Resource
end
-- 设置[效果]资源
function M:setE1Resource(resource)
    self._e1Resource = resource
end

-- 获得[效果]帧前缀
function M:getE1Prefix()
    return self._e1Prefix
end
-- 设置[效果]帧前缀
function M:setE1Prefix(prefix)
    self._e1Prefix = prefix
end

-- 获得[效果]帧数量
function M:getE1Num()
    return self._e1Num
end
-- 设置[效果]帧数量
function M:setE1Num(num)
    self._e1Num = num
end

-- 获得[效果]帧间隔
function M:getE1Interval()
    return self._e1Interval
end
-- 设置[效果]帧间隔
function M:setE1Interval(interval)
    self._e1Interval = interval
end

-- 获得[效果]位移起始
function M:getE1OffsetStart()
    return self._e1OffsetStart
end
-- 设置[效果]位移起始
function M:setE1OffsetStart(offset)
    self._e1OffsetStart = offset
end

-- 获得[效果]位移结束
function M:getE1OffsetEnd()
    return self._e1OffsetEnd
end
-- 设置[效果]位移结束
function M:setE1OffsetEnd(offset)
    self._e1OffsetEnd = offset
end

-- 获得[效果]持续标识（如果持续，则播放至目标位置后动画不进行消除）
function M:isSustained()
    return self._isSustained
end
-- 设置[效果]持续标识（如果持续，则播放至目标位置后动画不进行消除）
function M:setIsSustained(isSustained)
    self._isSustained = isSustained
end





return M