--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

效果卡片动作[两阶段]类，定义战斗动作相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("EffectActionStep2", AbstractEffectFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- ..


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._aTime1                = params.aTime1                     or FA__EFFECT_2__T1     -- [动画]时间1
    self._aTime2                = params.aTime2                     or FA__EFFECT_2__T2     -- [动画]时间2
    self._aTime3                = params.aTime3                     or FA__EFFECT_2__T3     -- [动画]时间3
    self._aOffset2              = params.aOffset2                   or FA__EFFECT_2__O2     -- [动画]位移2

    self._e1Sound               = params._e1Sound                   -- [声音1]
    self._e1Z                   = params.e1Z                        -- [效果1]层深度
    self._e1Prefix              = params.e1Prefix                   -- [效果1]帧前缀
    self._e1Num                 = params.e1Num                      -- [效果1]帧数量
    self._e1Interval            = params.e1Interval                 -- [效果1]帧间隔
    self._e1OffsetStart         = params.e1OffsetStart              -- [效果1]位移起始
    self._e1OffsetEnd           = params.e1OffsetEnd                -- [效果1]位移结束

    self._e2Sound               = params._e2Sound                   -- [声音2]
    self._e2Z                   = params.e2Z                        -- [效果2]层深度
    self._e2Prefix              = params.e2Prefix                   -- [效果2]帧前缀
    self._e2Num                 = params.e2Num                      -- [效果2]帧数量
    self._e2Interval            = params.e2Interval                 -- [效果2]帧间隔
    self._e2OffsetStart         = params.e2OffsetStart              -- [效果2]位移起始
    self._e2OffsetEnd           = params.e2OffsetEnd                -- [效果2]位移结束
    self._isSustained           = params.isSustained    or false    -- [效果]持续标识（如果持续，则播放至目标位置后动画不进行消除）
    self._isRecord              = params.isRecord       or false    -- [效果]是否记录
    self._recordTag             = params.recordTag                  -- [效果]动作记录标识
end




    





-- [实例方法]
-- [实例方法][资源]
-- 获得预加载的资源[图片]
function M:getResourcesImage(params)
    return {
        params._e1Resource,
        params._e2Resource,
    }
end

-- 获得预加载的资源[图片]
function M:getResourcesSound(params)
    return {
        params._e1Sound,
    }
end


---------


-- [实例方法][卡片]
-- 构建卡片动作对象
function M:buildActionCard(params)
    -- 定义动作对象
    local action = nil



    -- 辅助方法
    -- 格式化并返回位置偏移
    local function formatOffset2(offset, owner)
        return (owner == OWNER_ME) and offset or ccp(offset.x, -offset.y)
    end



    -- 辅助参数
    local target            = self:getNode()                            -- 目标对象
    local positionOrigin    = ccp(target:getPosition())                 -- 当前位置
    local owner             = target:getOwner()                         -- 获得拥有者标识
    local offset2           = formatOffset2(self:getAOffset2(), owner)  -- 获得位置偏移2



    -- 动画参数
    local target            = self._node                                -- 目标结点
    local time1             = self:T(self._aTime1)                      -- [动画]时间1
    local time2             = self:T(self._aTime2)                      -- [动画]时间2
    local time3             = self:T(self._aTime3)                      -- [动画]时间3
    local position2         = self:O(positionOrigin, offset2)           -- [动画]位置2
    local position3         = self:P(positionOrigin)                    -- [动画]位置3



    -- 构建动作对象
    -- [卡片]
    action = transition.sequence({
        CCDelayTime:create(time1),                      -- delay
        TDAnimationUtils.spawn({
            CCCallFunc:create(function() self:playSound1() end),         -- play effect
            CCMoveTo:create(time2, position2),
            -- CCShaky3D:create(time2, CCSize(15,10), 5, false),
        }),
        CCMoveTo:create(time3, position3),              -- moving up or down
    })



    return action
end


---------


-- [实例方法][效果]
-- 播放效果
function M:playEffect()
    -- [配置定义]
    local IDX_EFFECT1 = 1
    local IDX_EFFECT2 = 2
    local IDX_ACTION1 = 1
    local IDX_ACTION2 = 2


    -- [构建实体]
    -- [构建实体][效果1]
    if self:getEffect(IDX_EFFECT1) == nil then 
        -- [效果][起始动画]
        local target            = self._node                                    -- 目标结点
        local z1                = self._e1Z                                     -- 层深度
        local prefix1           = self._e1Prefix                                -- 帧前缀
        local frameCount1       = self._e1Num                                   -- 动画帧数
        local interval1         = self._e1Interval                              -- 帧间隔
        local params1           = {
            prefix      = prefix1,
            frameCount  = frameCount1,
            interval    = interval1,
        }
        self:setEffectAction(IDX_EFFECT1, IDX_ACTION1, TDAnimationUtils.animateWithFrameRepeat(params1))


        -- 定义配置参数
        local paramsCtor1 = {
            fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(prefix1),    -- 文件名
            parentNode  = target,                                               -- 父结点
            align       = display.CENTER,                                       -- 对齐方式
            anchor      = display.CENTER,                                       -- 锚点
            z           = z1,                                                   -- 层深度
        }
        -- 生成精灵
        self:setEffect(IDX_EFFECT1, TDUIUtils.loadSprite(paramsCtor1))
    end

    

    -- [动画定义]
    local effect1           = self:getEffect(IDX_EFFECT1)                       -- [效果1]对象
    local action1           = self:getEffectAction(IDX_EFFECT1, IDX_ACTION1)    -- [效果1]动作
    local eTime1            = self._e1Interval * self._e1Num                    -- [效果1]时间
    local e1PositionOrigin  = ccp(effect1:getPosition())                        -- [效果1]位置[原]
    local e1PositionStart   = ccpAdd(e1PositionOrigin, self._e1OffsetStart)     -- [效果1]位置[起始]
    local e1PositionEnd     = ccpAdd(e1PositionOrigin, self._e1OffsetEnd)       -- [效果1]位置[结束]
    local function onSwitch()
        -- [构建实体][效果2]
        if self:getEffect(IDX_EFFECT2) == nil then 
            -- [效果][起始动画]
            local target            = self._node                                    -- 目标结点
            local z2                = self._e2Z                                     -- 层深度
            local prefix2           = self._e2Prefix                                -- 帧前缀
            local frameCount2       = self._e2Num                                   -- 动画帧数
            local interval2         = self._e2Interval                              -- 帧间隔
            local params2           = {
                prefix      = prefix2,
                frameCount  = frameCount2,
                interval    = interval2,
            }
            self:setEffectAction(IDX_EFFECT2, IDX_ACTION2, TDAnimationUtils.animateWithFrameRepeat(params2))


            -- 定义配置参数
            local paramsCtor2 = {
                fileName    = "#" .. TDAnimationUtils.frameNameAtFirst(prefix2),    -- 文件名
                parentNode  = target,                                               -- 父结点
                align       = display.CENTER,                                       -- 对齐方式
                anchor      = display.CENTER,                                       -- 锚点
                z           = z2,                                                   -- 层深度
            }
            -- 生成精灵
            self:setEffect(IDX_EFFECT2, TDUIUtils.loadSprite(paramsCtor2))
        end


        -- [动画定义]
        local effect2           = self:getEffect(IDX_EFFECT2)                       -- [效果2]对象
        local action2           = self:getEffectAction(IDX_EFFECT2, IDX_ACTION2)    -- [效果2]动作
        local eTime2            = self._e2Interval * self._e2Num                    -- [效果2]时间
        local e2PositionOrigin  = ccp(effect2:getPosition())                        -- [效果2]位置[原]
        local e2PositionStart   = ccpAdd(e2PositionOrigin, self._e2OffsetStart)     -- [效果2]位置[起始]
        local e2PositionEnd     = ccpAdd(e2PositionOrigin, self._e2OffsetEnd)       -- [效果2]位置[结束]
        effect1:stopAllActions()
        effect1:setVisible(false)
        effect2:setPosition(e2PositionStart)
        effect2:runAction(action2)



        local function onCompleted() 
            if not self._isSustained then 
                effect2:setVisible(false) 
                effect2:stopAllActions() 
            end
            
            -- 是否记录判断
            if self._isRecord then 
                local actions = self:getNode():getActions()
                if actions[self._recordTag]  ~= nil then
                    local effects = actions[self._recordTag]
                    for k in pairs(effects) do
                        effects[k]:removeFromParentAndCleanup(true)
                    end
                end
                actions[self._recordTag] = { [1] = effect2, }
            end

            self:onCompletedEffect() 
        end

        effect2:runAction(TDAnimationUtils.sequence({
            CCDelayTime:create(eTime2),                 -- delay
            CCCallFunc:create(onCompleted),             -- completed
        }))
    end
    

    effect1:setVisible(true)
    effect1:setPosition(e1PositionStart)
    effect1:runAction(action1)
    effect1:runAction(TDAnimationUtils.sequence({
        CCMoveTo:create(eTime1, e1PositionEnd),         -- move 
        CCCallFunc:create(onSwitch),                    -- switch action
    }))
end

-- 处理效果结束后的逻辑
function M:onCompletedEffect()
    -- Implemented by sub-class!
end


---------


-- [实例方法][音效]
-- 播放效果
function M:playSound1()
    local sound = self._e1Sound

    if sound ~= nil then 
        SOUND.playEffect(self:pathSound(sound))
    end
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得[动画]时间1
function M:getATime1()
    return self._aTime1
end
-- 设置[动画]时间1
function M:setATime1(time)
    self._aTime1 = time
end

-- 获得[动画]时间2
function M:getATime2()
    return self._aTime2
end
-- 设置[动画]时间2
function M:setATime2(time)
    self._aTime2 = time
end

-- 获得[动画]时间3
function M:getATime3()
    return self._aTime3
end
-- 设置[动画]时间3
function M:setATime3(time)
    self._aTime3 = time
end

-- 获得[动画]位移2
function M:getAOffset2()
    return self._aOffset2
end
-- 设置[动画]位移2
function M:setAOffset2(offset)
    self._aOffset2 = offset
end

-- 获得[效果]层深度
function M:getE1Z()
    return self._e1Z
end
-- 设置[效果]层深度
function M:setE1Z(z)
    self._e1Z = z
end

-- 获得[效果]资源1
function M:getE1Resource()
    return self._e1Resource
end
-- 设置[效果]资源1
function M:setE1Resource(resource)
    self._e1Resource = resource
end

-- 获得[效果]帧前缀1
function M:getE1Prefix()
    return self._e1Prefix
end
-- 设置[效果]帧前缀1
function M:setE1Prefix(prefix)
    self._e1Prefix = prefix
end

-- 获得[效果]帧数量1
function M:getE1Num()
    return self._e1Num
end
-- 设置[效果]帧数量1
function M:setE1Num(num)
    self._e1Num = num
end

-- 获得[效果]帧间隔1
function M:getE1Interval()
    return self._e1Interval
end
-- 设置[效果]帧间隔1
function M:setE1Interval(interval)
    self._e1Interval = interval
end

-- 获得[效果]位移1起始
function M:getE1OffsetStart()
    return self._e1OffsetStart
end
-- 设置[效果]位移1起始
function M:setE1OffsetStart(offset)
    self._e1OffsetStart = offset
end

-- 获得[效果]位移1结束
function M:getE1OffsetEnd()
    return self._e1OffsetEnd
end
-- 设置[效果]位移1结束
function M:setE1OffsetEnd(offset)
    self._e1OffsetEnd = offset
end

-- 获得[效果]层深度
function M:getE2Z()
    return self._e2Z
end
-- 设置[效果]层深度
function M:setE2Z(z)
    self._e2Z = z
end

-- 获得[效果]资源2
function M:getE2Resource()
    return self._e2Resource
end
-- 设置[效果]资源2
function M:setE2Resource(resource)
    self._e2Resource = resource
end

-- 获得[效果]帧前缀2
function M:getE2Prefix()
    return self._e2Prefix
end
-- 设置[效果]帧前缀2
function M:setE2Prefix(prefix)
    self._e2Prefix = prefix
end

-- 获得[效果]帧数量2
function M:getE2Num()
    return self._e2Num
end
-- 设置[效果]帧数量2
function M:setE2Num(num)
    self._e2Num = num
end

-- 获得[效果]帧间隔2
function M:getE2Interval()
    return self._e2Interval
end
-- 设置[效果]帧间隔2
function M:setE2Interval(interval)
    self._e2Interval = interval
end

-- 获得[效果]位移2起始
function M:getE2OffsetStart()
    return self._e2OffsetStart
end
-- 设置[效果]位移2起始
function M:setE2OffsetStart(offset)
    self._e2OffsetStart = offset
end

-- 获得[效果]位移2结束
function M:getE2OffsetEnd()
    return self._e2OffsetEnd
end
-- 设置[效果]位移2结束
function M:setE2OffsetEnd(offset)
    self._e2OffsetEnd = offset
end




return M