--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作类[位移]，定义战斗动作[位移]相关操作方法及逻辑实现。

-   定义战斗动作方法及成员变量。

]]

local M = class("FightActionPosition", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- 移动模式[绝对移动－调用CCMoveTo]
M.MODE_ABSOLUTE             = 1
-- 移动模式[相对移动－调用CCMoveBy]
M.MODE_RELATIVE             = 2


-- [成员变量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._mode                  = params.mode               or M.MODE_ABSOLUTE          -- 移动模式(绝对/相对)
    self._aTime                 = params.aTime              or FA__POSITION__T          -- 动画[时间]



    -- [具体操作]
    -- 设置是否重新指派[起始位置]
    self:setReassignPositionBegin(true)
    -- 设置是否重新指派[结束位置]
    self:setReassignPositionEnd(true)
end

    

    





-- [抽象方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil



    -- 动画参数
    local mode              = self._mode                                -- 移动模式(绝对/相对)
    local timeAction        = self:T(self._aTime)                       -- 时间[动作]
    local positionBegin     = self:P(self._positionBegin)               -- 位置[起始]
    local positionEnd       = self:P(self._positionEnd)                 -- 位置[结束]



    -- 构建动作对象
    if mode == M.MODE_ABSOLUTE then
        action = CCMoveTo:create(timeAction, positionEnd)
    elseif mode == M.MODE_RELATIVE then
        action = CCMoveBy:create(timeAction, positionEnd)
    else 
        TDExceptionUtils.raiseUnRunHereException()
    end

    return action
end




    





-- [实例方法]
-- [实例方法][外部调用]
-- ..


-- [实例方法][属性Getter/Setter]
-- 设置结点
function M:setNode(node)
    self._node = node
end
-- 获得结点
function M:getNode()
    return self._node
end

-- 设置动画[时间]
function M:setATime(aTime)
    self._aTime = aTime
end
-- 获得动画[时间]
function M:getATime()
    return self._aTime
end





return M