--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗动作解释器类，定义公共战斗动作解释相关操作方法及逻辑实现。

-   定义基础动作方法及成员变量。

该处主要用到以下几种设计模式
    解释器，职责链，工厂，组合，代理，命令(command)。。。。

--------------------------------------------------------------------------------------------------------------------
                                                    添加新技能流程
示例：添加技能[月蚀]，步骤流程如下：
    (1)在FightActionEnum类中定义对应新增的ActionId及Action类名，操作如下：
        -- ActionId定义：
            -- [攻击]月蚀
            M.ATK_LUNAR_ECLIPSE  = 26;M.LUNAR_ECLIPSE_ATK  = 26
            -- [伤害]月蚀
            M.HURT_LUNAR_ECLIPSE = 46;M.LUNAR_ECLIPSE_HURT = 46
        -- Action类名定义：
            -- [攻击]月蚀
            [26] = "ActionAtkLunarEclipse",
            -- [伤害]月蚀
            [46] = "ActionHurtLunarEclipse",
    (2)新建类：
        在view.Fight.Action下新建ActionAtkLunarEclipse.lua及ActionHurt
        LunarEclipse.lua文件
    (3)复制参考文件内容：
        攻击参考：           ActionAtkSniper
        伤害[1阶]参考：      ActionHurtSniper
        伤害[2阶]参考：      ActionHurtIce
        持续参考：           ActionCtnIce
        解除参考：           ActionCclIce
    (4)定义Action参数：
        在view.Fight.Constant.IncludeAction.lua定义参数，可复制参考Action
        对应的参数
        如：定义ActionHurtLunarEclipse参数
            ActionHurtLunarEclipse参考文件为:
                "伤害[1阶]参考：ActionHurtSniper"
        则复制：
            -- [伤害]狙击       Class: ActionHurtSniper
            FA__HURT_SNIPER__S1             = "e_f_hurt_sniper.mp3"         -- [声音]
            FA__HURT_SNIPER__E1_RES         = "e_f_hurt_sniper"             -- [效果]资源文件
            FA__HURT_SNIPER__E1_Z           = 0                             -- [效果]Z值
            FA__HURT_SNIPER__E1_PFX         = "e_f_hurt_sniper"             -- [效果][1阶]帧前缀
            FA__HURT_SNIPER__E1_NUM         = 6                             -- [效果][1阶]帧数量
            FA__HURT_SNIPER__E1_ITV         = FA_A_INTERVAL                 -- [效果][1阶]帧间隔
            FA__HURT_SNIPER__E1_OS          = acp(0, 0)                     -- [效果][1阶]位移[起始]
            FA__HURT_SNIPER__E1_OE          = acp(0, 0)                     -- [效果][1阶]位移[结束]
            FA__HURT_SNIPER__T1             = FA__HURT_SNIPER__E1_NUM * FA__HURT_SNIPER__E1_ITV     -- 时间1
        修改为：
            -- [伤害]狙击       Class: ActionHurtLunarEclipse
            FA__HURT_LUNAR_ECLIPSE__S1          = "e_f_hurt_lunar_eclipse.mp3"      -- [声音]
            FA__HURT_LUNAR_ECLIPSE__E1_RES      = "e_f_hurt_lunar_eclipse"          -- [效果]资源文件
            FA__HURT_LUNAR_ECLIPSE__E1_Z        = 0                                 -- [效果]Z值
            FA__HURT_LUNAR_ECLIPSE__E1_PFX      = "e_f_hurt_lunar_eclipse"          -- [效果][1阶]帧前缀
            FA__HURT_LUNAR_ECLIPSE__E1_NUM      = 6                                 -- [效果][1阶]帧数量
            FA__HURT_LUNAR_ECLIPSE__E1_ITV      = FA_A_INTERVAL                     -- [效果][1阶]帧间隔
            FA__HURT_LUNAR_ECLIPSE__E1_OS       = acp(0, 0)                         -- [效果][1阶]位移[起始]
            FA__HURT_LUNAR_ECLIPSE__E1_OE       = acp(0, 0)                         -- [效果][1阶]位移[结束]
            FA__HURT_LUNAR_ECLIPSE__T1          = FA__HURT_LUNAR_ECLIPSE__E1_NUM * FA__HURT_LUNAR_ECLIPSE__E1_ITV       -- 时间1
    (5)修改Action.lua文件内容：
        修改动作定义为第(4)定义的常量名称
--------------------------------------------------------------------------------------------------------------------

]]

local M = class("FightInterpreter", AbstractFightInterpreter)



-- [包含文件]
-- ..


-- [操作变量]
-- [系统]切换回合
local ActionSystemTurnRound             = requireA("ActionSystemTurnRound")
-- [系统]延迟时间
local ActionSystemDelay                 = requireA("ActionSystemDelay")
-- [系统]重新排列卡牌位置
local ActionSystemRearrangePositions    = requireA("ActionSystemRearrangePositions")
-- [系统]校正冷却时间
local ActionSystemCorrectCD             = requireA("ActionSystemCorrectCD")
-- [系统]状态刷新
local ActionSystemStateRenew            = requireA("ActionSystemStateRenew")
-- [系统]游戏结束
local ActionSystemGameOver              = requireA("ActionSystemGameOver")


-- [公共参数]
-- [常量]
-- ..

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- 加载父类的构造方法
    M.super.ctor(self, params)
end

    

    





-- [覆写方法]
-- 运行动作
function M:interpret(params)
    TDPerformanceTestingUtils.clockBegin("FightInterpreter:interpret(params)")


    -- [参数赋值]
	local _data             	= params.data                            	-- 数据
	local _playerMe          	= params.playerMe                        	-- 我方玩家[玩家]
	local _playerOpponent      	= params.playerOpponent              		-- 敌方玩家[玩家]
	local _round             	= params.round                            	-- 回合
    local _delegate             = params.delegate                           -- 代理对象
    local _callfront            = params.callfront                          -- 每次动作前处理
    local _callback             = params.callback                           -- 每次动作后处理


    -- [操作变量]
    local _meHero               = _playerMe:getHeroHp()                     -- 我方玩家[英雄血量]
    local _meCards 				= _playerMe:getCards() 						-- 我方玩家[卡片集]
    local _mePrototypes 		= _playerMe:getPrototypes() 				-- 我方玩家[原型集]
    local _opponentHero         = _playerOpponent:getHeroHp()               -- 敌方玩家[英雄血量]
    local _opponentCards 		= _playerOpponent:getCards() 				-- 敌方玩家[卡片集]
    local _opponentPrototypes 	= _playerOpponent:getPrototypes() 			-- 敌方玩家[原型集]
    local _dataFights           = _data:getFight()                          -- 战斗数据集
    local _dataResult           = _data:getResult()                         -- 战斗结果

 

    -- [解析动作]
    local sequence = {}
    for _, dataFight in pairs(_dataFights) do
        -- 属性
        local roundNo       = dataFight:getRound()
        local turn          = dataFight:getTurn()
        local dataActions   = dataFight:getActions()
        local endCardsMe    = dataFight:getFme()
        local endCardsOpp   = dataFight:getFopp()

        local player        = self:getTurnPlayer(turn, _playerMe, _playerOpponent)
        local playerHp      = player:getHeroHp()
        local cardsInRound  = self:getTurnCards(turn, _meCards, _opponentCards)



        -- 逻辑
        -- 1、切换当前回合数 
        local actTurnRound  = ActionSystemTurnRound.new({ node = _round, roundNo = roundNo, cards = cardsInRound, delegate = _delegate, })
        sequence[#sequence + 1] = actTurnRound
        
        -- local actDelayTime  = ActionSystemDelay.new({ node = _round, delegate = _delegate, aTime = 0.5, str = "校正冷却时间延迟"})
        -- sequence[#sequence + 1] = actDelayTime
        
        -- 2、校正冷却时间
        local actCorrectCD  = ActionSystemCorrectCD.new({ node = _round, turn = turn, roundNo = roundNo, cardsMe = _meCards, cardsMeEnd = endCardsMe, cardsOpp = _opponentCards, cardsOppEnd = endCardsOpp, delegate = _delegate, })
        sequence[#sequence + 1] = actCorrectCD
 

        -- local actDelayTime  = ActionSystemDelay.new({ node = _round, delegate = _delegate,  aTime = 0.5, str = "解析动作序列延迟"})
        -- sequence[#sequence + 1] = actDelayTime

        -- 3、解析动作序列
        for i, dataAction in ipairs(dataActions) do
            local sn            = dataAction:getSn()
            local actionId      = dataAction:getActid()
            local varHp         = dataAction:getVarhp()
            local varAtk        = dataAction:getVaratk()
            local curHp         = dataAction:getCurhp()
            local curAtk        = dataAction:getCuratk()
            local nb            = dataAction:getNb()
            local index         = dataAction:getPo()
            local sync          = dataAction:getSyn()
            local entity        = self:getEntity(sn, _meHero, _meCards, _opponentHero, _opponentCards)
            local prototypes    = self:getPrototypes(sn, _mePrototypes, _opponentPrototypes)
            local action        = self:getAction(actionId)

            local params = {
                node            = entity,
                prototypes      = prototypes,
                index           = index,
                sn              = sn,
                action          = action,
                varHp           = varHp,
                varAtk          = varAtk,
                curHp           = curHp,
                curAtk          = curAtk,
                nb              = nb,
                sync            = sync,
                delegate        = _delegate,
                callfront       = _callfront,
                callback        = _callback,
            }
            if _callfront or _callback then 
                local p = {
                    roundNo         = roundNo,
                    turn            = turn,
                    actionId        = actionId,
                    node            = entity,
                    index           = index,
                    sn              = sn,
                    i               = i,
                }
                if _callfront then params.callfrontParams = p end
                if _callback then params.callbackParams = p end
            end
            sequence[#sequence + 1] = action.new(params)
        end
        
        -- 4、重新排列位置
        local actRearrangePositions = ActionSystemRearrangePositions.new({ node = _round, turn = turn, cardsMe = _meCards, cardsMeEnd = endCardsMe, cardsOpp = _opponentCards, cardsOppEnd = endCardsOpp, prototypesMe = _mePrototypes, prototypesOpp = _opponentPrototypes, delegate = _delegate, })
        sequence[#sequence + 1] = actRearrangePositions
        
        -- 5、更新设定
        local actStateRenew = ActionSystemStateRenew.new({ node = _round, turn = turn, cardsMe = _meCards, cardsMeEnd = endCardsMe, cardsOpp = _opponentCards, cardsOppEnd = endCardsOpp, delegate = _delegate, })
        sequence[#sequence + 1] = actStateRenew



        -- z、延迟时间
        local actDelayTime  = ActionSystemDelay.new({ node = _round, delegate = _delegate, })
        sequence[#sequence + 1] = actDelayTime
    end

    -- z、结束判定
    local actGameOver = ActionSystemGameOver.new({ node = _round, result = _dataResult, delegate = _delegate, })
    sequence[#sequence + 1] = actGameOver


    -- [运行动作]
    if #sequence == 1 then
        self:runSequenceOne(sequence)
    else 
        self:runSequenceMany(sequence)
    end


    TDPerformanceTestingUtils.clockEnd("FightInterpreter:interpret(params)")
end

-- 执行动作序列[单]
function M:runSequenceOne(sequence)
    sequence[1]:run()
end

-- 执行动作序列[多]
function M:runSequenceMany(sequence)
    -------------------------------------------------------------------
    -- 解析意图：
    --      依次遍历动作序列中的每个序列对象，将动作对象中sync相同的动作绑定到同
    --      一个后继者动作对象中，之后再将每一个动作对象链接，再执行播放
    -------------------------------------------------------------------
    -- 设置解析动作索引
    local iNext = 1
    -- 设置关联性
    for i = 1, #sequence - 1 do
        if i == iNext then 
            -- 获得动作
            local actionCurrent         = sequence[i]
            local syncCurrent           = actionCurrent:getSync()

            -- 定义后继者序列
            local successors = {}
            local syncNext   = nil
            for j = i + 1, #sequence do
                -- 获得同步标识
                local action = sequence[j]
                local sync   = action:getSync()

                if syncNext == nil then 
                    if sync == SYNC_IGNORE then 
                        successors[#successors + 1] = action
                        iNext = j
                        break
                    else 
                        syncNext = sync
                        successors[#successors + 1] = action
                    end
                else
                    if sync == syncNext then 
                        successors[#successors + 1] = action
                    else
                        iNext = j - 1
                        break
                    end
                end
            end

            actionCurrent:setSuccessors(successors)
        end
    end

    sequence[1]:run()
end

-- 获得当前回合玩家
function M:getTurnPlayer(turn, playerMe, playerOpponent)
    local player = nil

    if turn == TURN_ME then 
        player = playerMe
    elseif turn == TURN_OPPONENT then 
        player = playerOpponent
    end

    TDAssertUtils.assertNotNil(player)
    return player
end

-- 获得当前回合卡片集
function M:getTurnCards(turn, cardsMe, cardsOpponent)
    local cards = nil

    if turn == TURN_ME then 
        cards = cardsMe
    elseif turn == TURN_OPPONENT then 
        cards = cardsOpponent
    end

    TDAssertUtils.assertNotNil(cards)
    return cards
end

-- 获得卡片
function M:getEntity(sn, meHero, meCards, opponentHero, opponentCards)
    local entity = nil

    -- 玩家标识
    local flag  = string.sub(sn, 1, 1)
    -- 实体索引
    local index = tonumber(string.sub(sn, 2))

    if index == IDX_HERO then
        entity = (flag == FLAG_ME) and meHero or opponentHero
    else 
        if flag == FLAG_ME then 
            entity = meCards[sn]
        elseif flag == FLAG_OPPONENT then 
            entity = opponentCards[sn]
        end
    end

    TDAssertUtils.assertNotNil(entity)
    return entity
end

-- 获得位置集合
function M:getPrototypes(sn, mePrototypes, opponentPrototypes)
    local prototypes = nil

    -- 玩家标识
    local flag  = string.sub(sn, 1, 1)
    -- 实体索引
    local index = tonumber(string.sub(sn, 2))


    if index == IDX_HERO then
        -- Nothing to do, be nil!
    else 
        if flag == FLAG_ME then 
            prototypes = mePrototypes
        elseif flag == FLAG_OPPONENT then 
            prototypes = opponentPrototypes
        end
    end

    return prototypes
end

-- 获得动作类
function M:getAction(actionId)
    local actionName = ActionEnum.ACTIONS[actionId]
-- print("actionID"..actionId)
-- print("actionName"..actionName)
    -------------------------------------------------------------------
    -- 意图：
    --      开启以下代码能够修改一些actionId指向的类为我们自己定义的测试类
    --      此功能多用于在新技能添加时，开启测试使用
    -------------------------------------------------------------------
    -- if string.find(actionName, "ActionHurt") ~= nil or string.find(actionName, "ActionAtk") ~= nil  then
    -- if string.find(actionName, "ActionHurt") ~= nil then
    --     actionName = "ActionHurtLunarEclipse"
    -- end

   return requireA(actionName)
end





return M