--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

抽象效果战斗动作类，定义公共效果战斗动作相关操作方法及逻辑实现。

-   定义效果动作方法及成员变量。

]]

local M = class("AbstractEffectFightAction", AbstractFightAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- [常量][效果]
M.TAG_E                 = "E"
M.TAG_EA                = "EA"

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._isPlayValueChange         = params.isPlayValueChange  or true                 -- 播放数值变化 
    self._isViewZeroEH              = params.isViewZeroEH       or false                -- 是否显示补伤数字
end




    





-- [抽象方法]
-- [抽象方法][效果]
function M:playEffect()
    TDExceptionUtils.raiseUnRunHereException("AbstractEffectFightAction:playEffect()")
end

-- [覆写方法]
-- 运行动作
function M:buildAction(params)
    -- 定义动作对象
    local action = nil

-- print("-- 运行动作 -- function M:buildAction(params)")

    -- [卡片]
    action = TDAnimationUtils.sequence({
        self:buildActionCard(params),
    })
    -- [效果]
    -- 播放效果
    self:playEffect()



    return action
end




    





-- [实例方法]
-- [实例方法][卡片]
-- 构建卡片动作对象
function M:buildActionCard()
    return nil
end


---------


-- [实例方法][数值]
-- 播放数值变化 
function M:playValueChange()
    -- [过渡判断]
    if not self._isPlayValueChange then 
        return
    end

    -- [逻辑处理]
    -- 补伤数字
    if true then 
        local value         = self._varHp
        local isViewZero    = self._isViewZeroEH
        if value ~= 0 or (value == 0 and isViewZero) then 
            self:playFightNum(value)
        end
    end
    -- 攻击力
    if true then 
        local node          = self:getNodeUp()
        local value         = self._curAtk
        node:animteAttack(value)
    end
    -- 血量
    if true then 
        local node          = self:getNodeUp()
        local value         = self._curHp
        node:animateHp(value)
    end
end


---------


-- [实例方法][效果]
-- 获得效果对象
function M:getEffect(index)
    local className = self:getClassName()
    return self[className .. M.TAG_E .. index]
end
-- 设置效果对象
function M:setEffect(index, effect)
    local className = self:getClassName()
    self[className .. M.TAG_E .. index] = effect
end

-- 获得效果对象
function M:getEffectAction(indexEffect, indexAction)
    local className = self:getClassName()
    return self[className .. M.TAG_EA .. indexEffect .. "_" .. indexAction]
end
-- 设置效果对象
function M:setEffectAction(indexEffect, indexAction, action)
    local className = self:getClassName()
    self[className .. M.TAG_EA .. indexEffect .. "_" .. indexAction] = action
end





return M