--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

抽象战斗动作类，定义公共战斗动作相关操作方法及逻辑实现。

-   定义基础动作方法及成员变量。

]]

local M = class("AbstractFightAction", AbstractAction)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- [常量][效果]
M.TAG_E                 = "E"
M.TAG_EA                = "EA"

-- [常量][标签]
local TAG_BASE          = "Base"
local TAG_ATK           = "Atk"
local TAG_HURT          = "Hurt"
local TAG_INC           = "Inc"
local TAG_DEC           = "Dec"
local TAG_CTN           = "Ctn"
local TAG_REB           = "Reb"
local TAG_OTH           = "Oth"
local TAG_HHURT         = "HHurt"
local TAG_HINC          = "HInc"
local TAG_HDEC          = "HDec"
local TAG_STAGE         = "Stage"

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    -- [结点]
    if self:isTypeOfCard() then
        self._nodeDown          = params.node                                               -- 下场结点
        self._nodeUp            = params.node:getAssociatedCard()                           -- 上场结点

        self._nodeDown:float()
        self._nodeUp:float()
    end
    -- [数值]
    self._varHp                 = params.varHp
    self._varAtk                = params.varAtk
    self._curHp                 = params.curHp
    self._curAtk                = params.curAtk
    self._prototypes            = params.prototypes
    self._index                 = params.index
    self._nb                    = params.nb
end




    





-- [实例方法]
-- [实例方法][数字]
-- 播放战斗
function M:playFightNum(num, delay, position)
    -- [具体操作]
    local LinearNum = require("cardui.LinearNum")
    local z         = 10000
    local target    = self._node

    -- 构建实体
    local node = LinearNum.new({
        resource               = PHI_FIGHT_NUM .. "f_num",               -- 资源名(不包含后缀名的前缀，如要缓存要引入num.plist和num.png，则resource为num)
        prefix                 = num < 0 and "f_num_h" or "f_num_a",    -- 数字及符号前缀
        value                  = num,                                   -- 值
        isShowPlus             = true,                                  -- 显示＋标识
        isShowMinus            = true,                                  -- 显示－标识
        zeroSymbol             = true,                                  -- 数字0显示符号 可选值：+或-
        margin                 = ccp(-1, 0),                            -- 项目间隔
        marginSymbol           = ccp(0, 0),
        align                  = LinearNum.ALIGN_CENTER,                -- 项目对齐方式
    })
    target:addChild(node, z)
    if position then
        node:setPosition(position)
    else
        util.alignNode(node, target:getContentSize(), display.CENTER)
    end

    -- 创建动画
    local actions = {}
    local time          = self:T(FA__BASE_NUM__T)
    local timeCleanup   = self:T(FA__BASE_NUM__T_CLEAN)
    local originScale   = node:getScale()
    -- 0.初始设置
    node:setScale(0)
    -- 1.延迟时间
    if delay then
        actions[#actions + 1] = CCDelayTime:create(delay)
    end
    -- 2.显示
    actions[#actions + 1] = TDAnimationUtils.spawn({
        CCEaseElasticInOut:create(CCFadeIn:create(time), .3),
        CCScaleTo:create(time, originScale),
    })
    -- 3.延迟时间
    actions[#actions + 1] = CCDelayTime:create(timeCleanup)
    -- 4.清除
    actions[#actions + 1] = CCCallFunc:create(function() node:removeFromParentAndCleanup(true) end)
    node:runAction(TDAnimationUtils.sequence(actions))
end


---------


-- [实例方法][排列]
-- 处理视图切换
function M:handleViewSwitch()
    local isChange = false
   -- [过滤判断]
    -- 过滤非卡牌
    if not self:isTypeOfCard() then
        return isChange
    end
    -- print("通过")
    -- 过滤非上场卡牌
    
    if self._index <= IDX_STAGEUP_INC then 
        return isChange
    end
    -- print("通过2")
    -- 过滤非上场卡牌
    if self._sync ~= SYNC_IGNORE then 
        return isChange
    end
    -- print("通过3")


    -- 索引[实际]
    local nodeUp      = self:getNodeUp()
    local indexActual = self._index
    local topView     = nodeUp:getTopView()
    local time        = self:T(FA__COMMON_MOVE__T)

    -- 判断当前校正索引如果大于上场最大卡牌索引
    if indexActual > IDX_STAGEUP_INC + NUM_FIGHT_VIEW then 
        -- print(" -- 超时5张情况")
        -----------------------
        -- 超时5张情况
        -----------------------
        -- 设置基础索引
        local indexBase = IDX_STAGEUP_INC + NUM_FIGHT_VIEW - indexActual

        topView:setIndexBase(0)
         
        -- print("IDX_STAGEUP_INC + NUM_FIGHT_VIEW - indexActual", IDX_STAGEUP_INC + NUM_FIGHT_VIEW - indexActual)

        local function rerange(owner)
            local cardsUp     = topView:getCardsUp(owner)
            local prototypes  = topView:getPlayerPrototypes(owner)
            for i, card in ipairs(cardsUp) do
                local nodeUpCurrent     = card:getAssociatedCard()
                local index             = nodeUpCurrent:getIndexActual()                      -- 索引[当前]
                local iActual           = nodeUpCurrent:getIndex() + indexBase                                 -- 索引[实际]
               
                local action
                -- if index ~= iActual then 
                    -- print("超时5张重排")
                     -- print("位置:"..iActual)
                    if (IDX_STAGEUP_INC + 1 <= iActual and iActual <= IDX_STAGEUP_INC + NUM_FIGHT_VIEW) then 

                        local prototype         = prototypes:getPrototype(iActual)
                        local position          = prototype:getPosition()                       -- [动画]目标位置
                        local rotation          = prototype:getRotation()                       -- [动画]目标旋转角度
                        local scale             = prototype:getScale()                          -- [动画]目标缩放比例

                        -- 构建动作对象
                        action = TDAnimationUtils.sequence({
                            TDAnimationUtils.spawn({
                                CCShow:create(),                                -- 显示
                                CCMoveTo:create(time, position),                -- 位移
                                CCRotateTo:create(time, rotation),              -- 旋转角度
                                CCScaleTo:create(time, scale),                  -- 缩放比例
                            }),
                            CCCallFunc:create(function()
                               
                                nodeUpCurrent:setIndexActual(iActual)
                            end)
                        })
                        nodeUpCurrent:runAction(action)
                    else
                        action = TDAnimationUtils.sequence({
                            CCHide:create(),
                            CCCallFunc:create(function()
                                
                                nodeUpCurrent:setIndexActual(iActual)
                            end)
                        })
                        nodeUpCurrent:runAction(action)
                    end
                --     isChange = true
                 -- end
            end
        end

        -- 我方卡牌
        rerange(OWNER_ME)
        -- 敌方卡牌
        rerange(OWNER_OPPONENT)

    else
        -- print(" -- 低于5张情况")

        -----------------------
        -- 低于5张情况
        -----------------------
        local indexBase = 0
        -- 设置基础索引
        topView:setIndexBase(indexBase)

        local function rerange(owner)
            local cardsUp     = topView:getCardsUp(owner)
            local prototypes  = topView:getPlayerPrototypes(owner)
            -- 移动逻辑处理
            for i, card in ipairs(cardsUp) do
                local nodeUpCurrent     = card:getAssociatedCard()
                local index             = nodeUpCurrent:getIndexActual()                         -- 索引[当前] nodeUpCurrent:getIndex()
                local iActual           = nodeUpCurrent:getIndex() + indexBase             -- 索引[实际]
                -- print("owner=", owner, "i=", i, index, iActual)

                local action
                if i <= NUM_FIGHT_VIEW then 
                    if index ~= iActual then 
                        local prototype         = prototypes:getPrototype(iActual)
                        local position          = prototype:getPosition()                       -- [动画]目标位置
                        local rotation          = prototype:getRotation()                       -- [动画]目标旋转角度
                        local scale             = prototype:getScale()                          -- [动画]目标缩放比例
                        -- 构建动作对象
                        action = TDAnimationUtils.sequence({
                            TDAnimationUtils.spawn({
                                CCShow:create(),                                -- 显示
                                CCMoveTo:create(time, position),                -- 位移
                                CCRotateTo:create(time, rotation),              -- 旋转角度
                                CCScaleTo:create(time, scale),                  -- 缩放比例
                            }),
                            CCCallFunc:create(function()
                                nodeUpCurrent:setIndexActual(iActual)
                            end)
                        })
                        nodeUpCurrent:runAction(action)
                    --     isChange = true
                    end
                else
                    action = TDAnimationUtils.sequence({
                        CCHide:create(),
                        CCCallFunc:create(function()
                            nodeUpCurrent:setIndexActual(iActual)
                        end)
                    })
                    nodeUpCurrent:runAction(action)
                end

            end
        end

        -- 我方卡牌
        rerange(OWNER_ME)
        -- 敌方卡牌
        rerange(OWNER_OPPONENT)
    end

    return false
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- [加载超类函数]
    M.super.assertParameters(self, params)
    -- 验证目标结点
    TDAssertUtils.assertInherit(params.node, AbstractCard)
end


---------


-- [实例方法][便捷方法]
-- 获得路径[资源文件]
function M:pathResource(resourceName)
    local path

    local className = self:getClassName()
    if string.find(className, TAG_BASE) ~= nil then 
        path = PHI_FIGHT_EFFECT_BASE
    elseif string.find(className, "HTrap") ~= nil or string.find(className, TAG_HHURT) ~= nil or string.find(className, TAG_HINC) ~= nil or string.find(className, TAG_HDEC) ~= nil then 
        
        path = PHI_FIGHT_EFFECT_HERO
    elseif string.find(className, TAG_STAGE) ~= nil then 
        path = PHI_FIGHT_EFFECT_STAGE
    elseif string.find(className, TAG_ATK) ~= nil then 
        path = PHI_FIGHT_EFFECT_ATK
    elseif string.find(className, TAG_HURT) ~= nil then 
        path = PHI_FIGHT_EFFECT_HURT
    elseif string.find(className, TAG_INC) ~= nil then 
        path = PHI_FIGHT_EFFECT_INC
    elseif string.find(className, TAG_DEC) ~= nil then 
        path = PHI_FIGHT_EFFECT_DEC
    elseif string.find(className, TAG_CTN) ~= nil then 
        path = PHI_FIGHT_EFFECT_CTN
    elseif string.find(className, TAG_REB) ~= nil then 
        path = PHI_FIGHT_EFFECT_REB
    elseif string.find(className, TAG_OTH) ~= nil then 
        path = PHI_FIGHT_EFFECT_OTH
    end

    return path .. resourceName
end

-- 获得路径[资源文件]
function M:pathSound(resourceName)
    return PHS_FIGHT_EFFECT .. resourceName
end


---------


-- [实例方法][判断]
-- 判断操作对象是否是英雄
function M:isTypeOfHero()
    local className = self:getClassName()
    return (string.find(className, TAG_HHURT) ~= nil or string.find(className, TAG_HINC) ~= nil or string.find(className, TAG_HDEC) ~= nil)
end

-- 判断操作对象是否是卡片
function M:isTypeOfCard()
    return not self:isTypeOfHero()
end


---------


-- [实例方法][属性Getter/Setter]
-- 设置结点[上场结点]
function M:setNodeUp(node)
    self._nodeUp = node
end
-- 获得结点
function M:getNodeUp()
    return self._nodeUp
end

-- 设置结点
function M:setNodeDown(node)
    self._nodeDown = node
end
-- 获得结点[下场结点]
function M:getNodeDown()
    return self._nodeDown
end





return M