--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

卡片工厂类，定义游戏卡片实例化等相关操作方法及逻辑实现。

-   不同类型卡片的实例化操作。
-   提供一些卡片快捷工具方法。

]]

local M = {}



-- [包含文件]





-- [公共参数]
-- 常量
-- ...

-- 局部变量
local OF_CARDSTAGE_NAME                 = ccp(0, 0)
local OF_CARDSTAGE_LEVEL                = ccp(0, 0)
local OF_CARDSTAGE_ATTACK               = ccp(-5, 15)
local OF_CARDSTAGE_HP                   = ccp(-5, 0)
local OF_CARDSTAGE_WAITROUNDS           = ccp(5, 5)
local OF_FIGHT                          = ccp(0, 10)
local OF_HURT                           = ccp(0, -10)
local FZ_CARDSTAGE_LABEL                = (13.5 * display.contentScaleFactor)
local FZ_CARDSTAGE_LABEL_ROUNDWAITS     = (15 * display.contentScaleFactor)
local FZ_CARDSTAGE_LABEL_NAME           = (16 * display.contentScaleFactor)
local FZ_CARDSTAGE_LABEL_LEVEL          = (18 * display.contentScaleFactor)
local A_READY_TIME                      = .3
local A_FIGHT_TIME                      = .15
local A_HURT_TIME                       = .15
local A_OPERATE_ROTATE                  = 20
local A_DIE_TIME                        = .3


-- 操作变量
-- 断言
local TDAssertUtils     = require(PH_MDL_UTIL .. "TDAssertUtils")
-- 用户PO
local poUser            = require(PH_EN_FIGHT .. "EntityUser")
-- 卡片PO
local poCard            = require(PH_EN_FIGHT .. "EntityCard")
-- 卡片详请层
local CardDetailLayer   = require(PH_MDL_LAYER .. "CardDetailLayer")










-- [实例方法]
--[[--
    创建英雄卡片
]]
function M.newHero(params)
    -- [验证参数合法性]
    -- 验证参数集
    TDAssertUtils.assertParameters(params)

    -- [参数赋值]
    local po            = params.po                         -- PO对象
    local owner         = params.owner                      -- 拥有者标识
    local mode          = params.mode                       -- 战斗模式
    local fileName      = PHI_FIGHT .. "f_hero_box.png"     -- 图像文件名

    return HeroCard.new({ fileName = fileName, po = po, owner = owner, mode = mode })
end

--[[--
    创建图标卡片
]]
function M.newIcon(params)
    -- [验证参数合法性]
    -- 验证参数集
    TDAssertUtils.assertParameters(params)

    -- [参数赋值]
    local po            = params.po                         -- PO对象
    local owner         = params.owner                      -- 拥有者标识
    local fileName      = po:getIcon()                      -- 图像文件名
    local prototypes    = params.prototypes                 -- 原型集
    local index         = params.index      or 0            -- 原型索引
    local isPrototypeAutoCorrect = params.isPrototypeAutoCorrect      -- 自动校正原型(如果自动校正，则设置_index时会自动将位置、旋转角度、缩放比例等定位到指定索引位置)

    return IconCard.new({ fileName = fileName, po = po, owner = owner, prototypes = prototypes, index = index, isPrototypeAutoCorrect = isPrototypeAutoCorrect })
end

--[[--
    创建图片卡片
]]
function M.newImage(params)
    -- [验证参数合法性]
    -- 验证参数集
    TDAssertUtils.assertParameters(params)

    -- [参数赋值]
    local po            = params.po                         -- PO对象
    local owner         = params.owner                      -- 拥有者标识
    local fileName      = po:getImage()                     -- 图像文件名
    local prototypes    = params.prototypes                 -- 原型集
    local index         = params.index      or 0            -- 原型索引
    local isPrototypeAutoCorrect = params.isPrototypeAutoCorrect      -- 自动校正原型(如果自动校正，则设置_index时会自动将位置、旋转角度、缩放比例等定位到指定索引位置)

    return ImageCard.new({ fileName = fileName, po = po, owner = owner, prototypes = prototypes, index = index, isPrototypeAutoCorrect = isPrototypeAutoCorrect })
end

--[[--
    创建下场卡片
]]
function M.newStageDown(params)
    -- [验证参数合法性]
    -- 验证参数集
    TDAssertUtils.assertParameters(params)

    -- [参数赋值]
    local po            = params.po                         -- PO对象
    local owner         = params.owner                      -- 拥有者标识
    local fileName      = po:getIcon()                      -- 图像文件名
    local prototypes    = params.prototypes                 -- 原型集
    local index         = params.index      or 0            -- 原型索引
    local isPrototypeAutoCorrect = params.isPrototypeAutoCorrect      -- 自动校正原型(如果自动校正，则设置_index时会自动将位置、旋转角度、缩放比例等定位到指定索引位置)
    local handler       = params.handler    or nil          -- 点击事件处理函数
    local filter        = params.filter     or nil          -- 点击事件过滤函数(当其返回值为true时才允许handler被调用)

    return StageDownCard.new({ fileName = fileName, po = po, owner = owner, prototypes = prototypes, index = index, isPrototypeAutoCorrect = isPrototypeAutoCorrect, handler = handler, filter = filter, })
end

--[[--
    创建上场卡片
]]
function M.newStageUp(params)
    -- [验证参数合法性]
    -- 验证参数集
    TDAssertUtils.assertParameters(params)

    -- [参数赋值]
    local po            = params.po                         -- PO对象
    local owner         = params.owner                      -- 拥有者标识
    local fileName      = po:getFrameNameBorder()           -- 图像文件名
    local prototypes    = params.prototypes                 -- 原型集
    local index         = params.index          or 0        -- 原型索引
    local offsetShadow  = params.offsetShadow   or P_ZERO   -- 阴影偏移量
    local isPrototypeAutoCorrect = params.isPrototypeAutoCorrect      -- 自动校正原型(如果自动校正，则设置_index时会自动将位置、旋转角度、缩放比例等定位到指定索引位置)
    local handler       = params.handler        or nil      -- 点击事件处理函数
    local filter        = params.filter         or nil      -- 点击事件过滤函数(当其返回值为true时才允许handler被调用)

    return StageUpCard.new({ fileName = fileName, po = po, owner = owner, prototypes = prototypes, index = index, isPrototypeAutoCorrect = isPrototypeAutoCorrect, offsetShadow = offsetShadow, handler = handler, filter = filter, })
end






return M