--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

效果工厂类，定义游戏效果实例化等相关操作方法及逻辑实现。

-   不同类型效果的实例化操作。
-   提供一些效果快捷工具方法。

]]

local M = {}



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- ...

-- 局部变量
local OF_HURT_NORMAL                    = ccp(10, 15)
local FZ_HURT_NORMAL                    = (15 * display.contentScaleFactor)
local C_HURT_NORMAL                     = ccc3(200, 0, 0)
local A_HURT_NORMAL_TIME                = .3
local A_HURT_NORMAL_OFFSET              = ccp(0, 30)
local A_HURT_NORMAL_SCALE               = 2
local A_HURT_NORMAL_CALLBACK_DELAY      = 0
local A_HURT_NORMAL_S_EXP_MIN           = 1000
local A_HURT_NORMAL_S_EXP_MAX           = 3000
local A_HURT_NORMAL_S_GOLD_MIN          = 500
local A_HURT_NORMAL_S_GOLD_MAX          = 1000
local A_HURT_NORMAL_F_EXP_MIN           = 10
local A_HURT_NORMAL_F_EXP_MAX           = 100
local A_HURT_NORMAL_F_GOLD_MIN          = 50
local A_HURT_NORMAL_F_GOLD_MAX          = 100

-- 枚举

-- 操作变量
-- 断言
local TDAssertUtils     = require(PH_MDL_UTIL .. "TDAssertUtils")
-- 数字操作
local TDNumberUtils     = require(PH_MDL_UTIL .. "TDNumberUtils")
-- 随机
local TDRandomUtils     = require(PH_MDL_UTIL .. "TDRandomUtils")
-- 触碰事件
local TDTouchUtils      = require(PH_MDL_UTIL .. "TDTouchUtils")

-- 提示视图
local FightResultLayer  = require(PH_MDL_LAYER .. "FightResultLayer")










-- [辅助方法]
-- ...










-- 加载提示层
local function loadFightResultLayer(owner, topView)

    -- 根据攻击者标识获得指定的金币
    local function goldsByAttackOwner(owner)
        return (attackerOwner == OWNER_ME) and TDRandomUtils.num(A_HURT_NORMAL_S_GOLD_MIN, A_HURT_NORMAL_S_GOLD_MAX) or TDRandomUtils.num(A_HURT_NORMAL_F_GOLD_MIN, A_HURT_NORMAL_F_GOLD_MAX)
    end
    -- 根据攻击者标识获得指定的经验值
    local function expsByAttackOwner(owner)
        return (attackerOwner == OWNER_ME) and TDRandomUtils.num(A_HURT_NORMAL_S_EXP_MIN, A_HURT_NORMAL_S_EXP_MAX) or TDRandomUtils.num(A_HURT_NORMAL_F_EXP_MIN, A_HURT_NORMAL_F_EXP_MAX)
    end


    -- 定义配置参数
    local params = {
        title                 = (attackerOwner == OWNER_ME) and "战斗胜利" or "战斗失败",           -- 标题
        gold                  = goldsByAttackOwner(owner),                                      -- 金币
        experience            = expsByAttackOwner(owner),                                       -- 经验值
        viewToJump            = nil                                                             -- 跳转视图
    }

    local layer = FightResultLayer.new(params)
    topView:addChild(layer)
end










-- [实例方法]
-- 处理受伤[基础攻击]
function M.handleHurtNormal(params)
    -- 验证参数合法性
    -- TDAssertUtils.assertClassname(po, UserPo);

    -- 提取配置参数        
    local delay             = params.delay                      -- 延迟时间
    local image             = params.image or "x.png"           -- 图片
    local num               = params.num                        -- 伤害数值
    local attackerOwner     = params.attackerOwner              -- 攻击者拥有标识(我/敌方标识)
    local injuryer          = params.injuryer                   -- 受伤对象(父结点)
    local injuryerType      = params.injuryerType               -- 受伤对象角色类型
    local offset            = params.offset or OF_HURT_NORMAL   -- 偏移量
    local callback          = params.callback                   -- 回调函数 

    -- 验证参数合法性（文件名）
    TDAssertUtils.assertTypeString(image);





    -- 定义局部变量
    local parentNode = injuryer





    -- 定义方法声明
    local animte





    -- 实例化一个空对象
    local node
    node = display.newNode()                                    -- 创建容器
    node:setContentSize(parentNode:getContentSize())


    -- 配置对象
    if node then
        -- (1)加载X
        local sprite = display.newSprite(image)
        -- 添加到精灵中
        node:addChild(sprite)
        -- 定位结点
        util.alignNode(sprite, node:getContentSize(), display.CENTER)
        sprite:setPosition(ccp(sprite:getPositionX() - sprite:getContentSize().width / 2 - offset.x, sprite:getPositionY() + offset.y))


        -- (2)加载伤害数字
        local label = ui.newTTFLabel({
            text       = tostring(num),                         -- 标签文字
            font       = ui.DEFAULT_TTF_FONT,                   -- 字体名称
            size       = FZ_HURT_NORMAL,                        -- 字体大小
            color      = C_HURT_NORMAL,                         -- 颜色
        })
        -- 添加到精灵中
        node:addChild(label)
        -- 定位结点
        util.alignNode(label, node:getContentSize(), display.CENTER)
        label:setPosition(ccp(label:getPositionX() + label:getContentSize().width / 2 + offset.x, label:getPositionY() + offset.y))



        -- 方法定义
        function animte()
            -- 参数定义
            local time1 = A_HURT_NORMAL_TIME
            local time2 = A_HURT_NORMAL_TIME
            local time3 = A_HURT_NORMAL_TIME
            local time4 = A_HURT_NORMAL_TIME


            -- [x]
            local positionTarget = ccp(sprite:getPositionX() + A_HURT_NORMAL_OFFSET.x, sprite:getPositionY() + A_HURT_NORMAL_OFFSET.y)
            -- 动画显示
            local action = transition.sequence({
                CCDelayTime:create(delay),                              -- 延迟操作
                CCScaleTo:create(time2*.5, A_HURT_NORMAL_SCALE),        -- 缩放
                CCScaleTo:create(time2*.5, 1),                          -- 缩放
                CCMoveTo:create(time3, positionTarget),                 -- 移动位置
                CCFadeOut:create(time4),                                -- 透明度渐变
                CCCallFunc:create(function()
                    sprite:removeSelf()
                end)
            })
            -- 执行动作
            sprite:runAction(action)


            -- [伤害数字]
            positionTarget = ccp(label:getPositionX() + A_HURT_NORMAL_OFFSET.x, label:getPositionY() + A_HURT_NORMAL_OFFSET.y)
            -- 初始设置
            label:setScaleX(0)
            -- 动画显示
            action = transition.sequence({
                CCDelayTime:create(delay),                              -- 延迟操作
                CCScaleTo:create(time2, 1),                             -- 缩放
                CCMoveTo:create(time3, positionTarget),                 -- 移动位置
                CCFadeOut:create(time4),                                -- 透明度渐变
                CCCallFunc:create(function()                            -- 回调函数 
                    label:removeSelf()
                end)
            })
            -- 执行动作
            label:runAction(action)


            -- [处理实体受伤]
            if injuryerType == TYPE_ROLE_HERO then 
                -- TODO: 断言判断类型
                -- ..
                
                -- 计算HP
                local hp = TDNumberUtils.max(0, injuryer:getHp() - num)
                -- 设置HP显示
                injuryer:setHp(hp)

                -- 处理死亡逻辑
                if (hp == 0) then 
                    -- 获得顶级视图
                    local topView = injuryer:getParent():getParent()

                    if topView:isGameOver() ~= true then
                        -- 设定游戏结束
                        topView:gameOver()
                        -- 加载结果层
                        loadFightResultLayer(attackerOwner, topView)
                    end
                end

            elseif injuryerType == TYPE_ROLE_CARD then 
                -- TODO: 断言判断类型
                -- ..
                
                -- 计算HP
                local hp = TDNumberUtils.max(0, injuryer:getHp() - num)
                -- 设置HP显示
                injuryer:setHp(hp)

                -- 处理死亡逻辑
                if (hp == 0) then 
                    injuryer.delegate:handleCardDie(injuryer)
                end
            end


            -- [结点]
            local timeTotal = time1 + time2 + time3 + time4 + A_HURT_NORMAL_CALLBACK_DELAY
            action = transition.sequence({
                CCDelayTime:create(timeTotal),                          -- 延迟操作
                CCCallFunc:create(callback),                            -- 回调函数 
            })
            -- 执行动作
            node:runAction(action)
        end

        parentNode:addChild(node)
        animte()
    end
end







return M