--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗视图工厂类，定义游戏战斗视图实例化等相关操作方法及逻辑实现。

-   不同战斗视图的实例化操作。
-   提供一些战斗视图快捷工具方法。

]]

local M = {}



-- [包含文件]





-- [公共参数]
-- 常量
-- ..

-- 局部变量
-- ..


-- 操作变量
-- 断言
local TDAssertUtils     = require(PH_MDL_UTIL .. "TDAssertUtils")
-- 战斗场景
local FightScene        = require(PH_MDL_SCENE .. "FightScene")










-- [实例方法]
--[[--
    跳转到视图PK－好友PK
]]
function M.viewPkLearn(params)
    -- [参数赋值]
    params.mode                     = { battle = MODE_BATTLE_PK, pk = MODE_PK_LEARN }               -- 战斗模式
    -- [视图跳转]
    M.viewPk(params)
end

-- [实例方法]
--[[--
    跳转到视图BOSS
]]
function M.viewBOSSLearn(params)
    -- [参数赋值]
    params.mode                     = { battle = MODE_BATTLE_BOSS, pk = MODE_PK_LEARN }               -- 战斗模式
    -- [视图跳转]
    M.viewPk(params)
end

--[[--
    跳转到视图PK－团队赛
]]
function M.viewPkTeamWar(params)
    -- [参数赋值]
    params.mode                     = { battle = MODE_BATTLE_PK, pk = MODE_PK_TEAM_FIGHT }            -- 战斗模式
    -- [视图跳转]
    M.viewPk(params)
end

--[[--
    跳转到视图PK－排名赛
]]
function M.viewPkRankWar(params)
    -- [参数赋值]
    params.mode                     = { battle = MODE_BATTLE_PK, pk = MODE_PK_RANK_WAR }            -- 战斗模式
    -- [视图跳转]
    M.viewPk(params)
end

--[[--
    跳转到视图PK－竞争赛（周、月、季）
]]
function M.viewPkCompetitiveRace(params)
    -- [参数赋值]
    params.mode                     = { battle = MODE_BATTLE_PK, pk = MODE_PK_COMPETITIVE_RACE }    -- 战斗模式
    -- [视图跳转]
    M.viewPk(params)
end

--[[--
    跳转到视图PK
]]
function M.viewPk(params)
    -- [验证参数合法性]
    -- 验证参数集
    TDAssertUtils.assertParameters(params)

    -- 参数赋值
    local tid                       = params.tid                                            -- 战队ID
    local serid                     = params.serid                                          -- 战队ID
    local friendId                  = params.friendId                                       -- 好友ID
    local friendName                = params.friendName                                     -- 好友名称
    local friendPoint               = 0                                                     -- 友情点
    local levelId                   = 0                                                     -- 关卡ID
    local difficulty                = 0                                                     -- 关卡难度
    local mode                      = params.mode                                           -- 战斗模式
    local fnNextView                = params.fnNextView                                     -- 下一跳转视图方法

    -- print("friendId", friendId)
    -- print("friendName", friendName)
    -- print("friendPoint", friendPoint)
    -- print("levelId", levelId)
    -- print("difficulty", difficulty)
    -- print("mode", mode.battle, mode.pk)

    -- 场景转换
    local scene = FightScene.new({ serid = serid, tid = tid, friendId = friendId, friendName = friendName, friendPoint = friendPoint, levelId = levelId, difficulty = difficulty, mode = mode, fnNextView = fnNextView, })
    CCBCenter.replaceScene(scene)
end

--[[--
    跳转到视图PK－竞争赛（周、月、季）
]]
function M.viewRecordRankWar(params)
    -- [参数赋值]
    params.mode                     = { battle = MODE_BATTLE_RECORD, record = MODE_RECODE_RANK_WAR }            -- 战斗模式
    -- [视图跳转]
    M.viewRecord(params)
end

--[[--
    跳转到视图PK－竞争赛（周、月、季）
]]
function M.viewRecordCompetitiveRace(params)
    -- [参数赋值]
    params.mode                      = { battle = MODE_BATTLE_RECORD, record = MODE_RECODE_COMPETITIVE_RACE }    -- 战斗模式
    -- [视图跳转]
    M.viewRecord(params)
end

--[[--
    跳转到视图PK
]]
function M.viewRecord(params)
    -- [验证参数合法性]
    -- 验证参数集
    TDAssertUtils.assertParameters(params)

    -- 参数赋值
    local friendId                  = 0                                                     -- 好友ID
    local friendName                = ""                                                    -- 好友名称
    local friendPoint               = 0                                                     -- 友情点
    local levelId                   = 0                                                     -- 关卡ID
    local difficulty                = 0                                                     -- 关卡难度
    local recordId                  = params.recordId                                       -- 记录ID
    local serid                     = params.serid                                          -- 记录ID
    local mode                      = params.mode                                           -- 战斗模式
    local fnNextView                = params.fnNextView                                     -- 下一跳转视图方法

    -- print("friendId", friendId)
    -- print("friendName", friendName)
    -- print("friendPoint", friendPoint)
    -- print("levelId", levelId)
    -- print("difficulty", difficulty)

    -- 场景转换
    local scene = FightScene.new({ serid = serid, friendId = friendId, friendName = friendName, friendPoint = friendPoint, levelId = levelId, difficulty = difficulty, recordId = recordId, mode = mode, fnNextView = fnNextView, })
    CCBCenter.replaceScene(scene)
end

function M.viewFirstFight( )
    -- 场景转换
    local scene = FightScene.new({ 
        mode = {battle = MODE_BATTLE_PK, pk = MODE_PK_FIRST_FIGHT}, 
        })
    CCBCenter.replaceScene(scene)
end






return M