--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

卡片详情景层类，定义卡片详情场景层相关操作方法及逻辑实现。

-   显示卡片详细信息。

]]

local M = class("CardDetailLayer", function()
    return display.newLayer()
end)



-- [包含文件]
-- ...




-- [公共参数]
-- 操作变量
-- 断言
local TDAssertUtils 		= require(PH_MDL_UTIL .. "TDAssertUtils")
local TDExceptionUtils 		= require(PH_MDL_UTIL .. "TDExceptionUtils")
local TDPointUtils 			= require(PH_MDL_UTIL .. "TDPointUtils")
local TDTouchUtils 			= require(PH_MDL_UTIL .. "TDTouchUtils")
local acp 					= TDPointUtils.point

-- 常量
local C_BACKGROUND 			= ccc4(0, 0, 0, 180)
local M_INTERSPACE			= acp(80, 0)
local M_SKILL		 		= acp(0, -60)
local M_SKILL_INFO		 	= acp(0, -28)
local FZ_LABEL_NAME 		= (28 * display.contentScaleFactor)
local FZ_LABEL_ETHNICITY 	= (17 * display.contentScaleFactor)
local FZ_LABEL_LEVEL 		= (17 * display.contentScaleFactor)
local FZ_LABEL_SKILL_NAME 	= (17 * display.contentScaleFactor)
local FZ_LABEL_SKILL_INFO 	= (15 * display.contentScaleFactor)
local FZ_TIP			 	= (20 * display.contentScaleFactor)
local P_LABEL_NAME 			= acp(600, 500)
local P_LABEL_ETHNICITY 	= acp(520, 450)
local P_LABEL_LEVEL 		= acp(680, 450)
local P_LABEL_SKILL_START 	= acp(520, 400)
local O_TIP_TOUCH_FOR_CLOSE = acp(0, 20)
local DMS_LABEL_SKILL_INFO  = CCSize(display.width * .25, 40 * display.contentScaleFactor)

-- 局部变量
-- ..
-- 枚举
-- ..










-- [构造方法]
--[[--
    创建实例
]]
function M:ctor(params)
	-- 验证参数合法性
	-- TDAssertUtils.assertClassname(poCards, UserPo)

	-- 参数赋值
    self.poCard       			= params.poCard 							-- 卡片PO

    -- 配置视图窗口
    -- self:configWindow()

	







	-- [辅助变量]

	







	-- [私有方法声明]
	-- [渲染视图]
	local viewRender
	-- 加载背景
	local loadBackground
	-- 加载卡片
	local loadCard
	-- 加载标签
	local loadLabels
	-- 加载标签[名称]
	local loadLabelName
	-- 加载标签[种族]
	local loadLabelEthnicity
	-- 加载标签[等级]
	local loadLabelLevel
	-- 加载技能
	local loadSkills
	-- 加载点击提示
	local loadTipTouchForClose
	-- 加载渲染以外的其他操作
	local loadRenderOtherwise
	-- 绑定点击事件侦听器
	local bindTouchListener
		

		

		





	-- [私有方法定义]
	-- [渲染视图]
	-- 视图渲染
	function viewRender()
		-- 加载背景
		loadBackground()

		-- 加载卡片
		loadCard()
		-- 加载标签
		loadLabels()
		-- 加载标签[名称]
		loadLabelName()
		-- 加载标签[种族]
		loadLabelEthnicity()
		-- 加载标签[等级]
		loadLabelLevel()
		-- 加载技能
		loadSkills()
		-- 加载点击提示
		loadTipTouchForClose()

		-- 加载渲染以外的其他操作
		loadRenderOtherwise()
	end

	-- 加载背景
	function loadBackground()
		local sprite = widget.newColorLayer(C_BACKGROUND)
		self:addChild(sprite)
	end

	-- 加载卡片
	function loadCard()
		local sprite = display.newSprite(self.poCard:getImageEx())
		self:addChild(sprite)

		util.alignNode(sprite, self:getContentSize(), display.CENTER)
		sprite:setPosition(ccp(sprite:getPositionX() - sprite:getContentSize().width / 2 - M_INTERSPACE.x, sprite:getPositionY() + M_INTERSPACE.y))
	end

	-- 加载标签
	function loadLabels()
		-- 加载标签[名称]
		loadLabelName()
		-- 加载标签[种族]
		loadLabelEthnicity()
		-- 加载标签[等级]
		loadLabelLevel()
	end
	-- 加载标签[名称]
	function loadLabelName()
        local sprite = ui.newTTFLabelWithShadow({
            text = self.poCard:getName(),
            size = FZ_LABEL_NAME,
        })
        self:addChild(sprite)

        -- 定位
        sprite:setPosition(P_LABEL_NAME)
	end
	-- 加载标签[种族]
	function loadLabelEthnicity()
        local sprite = ui.newTTFLabelWithShadow({
            text = "种族: " .. self.poCard:getEthnicityName(),
            size = FZ_LABEL_ETHNICITY,
        })
        self:addChild(sprite)

        -- 定位
        sprite:setPosition(P_LABEL_ETHNICITY)
	end
	-- 加载标签[等级]
	function loadLabelLevel()
        local sprite = ui.newTTFLabelWithShadow({
            text = "Lv: " .. self.poCard:getLv(),
            size = FZ_LABEL_LEVEL,
        });
        self:addChild(sprite)

        -- 定位
        sprite:setPosition(P_LABEL_LEVEL)
	end

	-- 加载技能
	function loadSkills()
		local posSkill = self.poCard:getSkills()

		if posSkill ~= nil and #posSkill > 0 then
			-- 记录前一条记录位置
			local positionPrev

			-- 生成技能标签
			for i = 1, #posSkill do
				-- 获得PO对象
				local poSkill = posSkill[i]


				-- [名称]
				-- 渲染输出
		        local labelName = ui.newTTFLabelWithShadow({
		            text = poSkill:getName() .. (poSkill:isIsLearned() and "" or "(" .. poSkill:getUnlockLevel() .. "级解锁)"),
		            size = FZ_LABEL_SKILL_NAME,
		        });
		        self:addChild(labelName)

		        -- 定位坐标
		        if positionPrev == nil then 
		        	labelName:setPosition(P_LABEL_SKILL_START)
		        else
		        	labelName:setPosition(ccp(positionPrev.x + M_SKILL.x, positionPrev.y + M_SKILL.y))
		        end
		        -- 存储当前坐标以供后续标签参考定位
		        positionPrev = ccp(labelName:getPosition())


				-- [描述]
				-- 渲染输出
		        local labelInfo = ui.newTTFLabelWithShadow({
		            text 		= poSkill:getInfo(),
		            size 		= FZ_LABEL_SKILL_INFO,
		            dimensions 	= DMS_LABEL_SKILL_INFO,
		            valign 		= ui.TEXT_VALIGN_TOP,
		        });
		        self:addChild(labelInfo)

		        -- 定位坐标
		        labelInfo:setPosition(ccp(positionPrev.x + M_SKILL_INFO.x, positionPrev.y + M_SKILL_INFO.y))
		        -- 存储当前坐标以供后续标签参考定位
		        positionPrev = ccp(labelInfo:getPosition())
			end
		end
	end

	-- 加载点击提示
	function loadTipTouchForClose()
        local sprite = ui.newTTFLabelWithShadow({
            text = "请点击屏幕－关闭",
            size = FZ_TIP,
        });
        self:addChild(sprite)

        -- 定位
        util.alignNode(sprite, self:getContentSize(), display.BOTTOM_CENTER)
        sprite:setPosition(sprite:getPositionX() + O_TIP_TOUCH_FOR_CLOSE.x, sprite:getPositionY() + O_TIP_TOUCH_FOR_CLOSE.y)
	end

	-- 加载渲染以外的其他操作
	function loadRenderOtherwise()
		-- 绑定点击事件侦听器
		bindTouchListener()
	end

	-- 绑定点击事件侦听器
	function bindTouchListener()
		TDTouchUtils.bindLayerTouchEvent({ layer = self, priority = TDTouchUtils.PRIORITY_MASK, handler = function()
			self:removeSelf()
		end})
	end


	---------


	-- [事件处理]
	-- ..


	---------


	-- [辅助方法]
	-- ..


	---------


	-- [工具方法]
	-- ..
		

		

		





	-- [实例化对象]
	viewRender()
end




return M