--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗按钮层类，定义战斗按钮层相关操作方法及逻辑实现。

-   存储战斗按钮户相关信息。

]]

local M = class("FightButtonLayer", TDLayer)



-- [包含文件]
-- ...



-- [公共参数]
-- [常量]
local M_BUTTON                  = ccp(-70, 89)
local S_BUTTON                  = 1
local C_BUTTON_ENABLE           = ccc3(255, 255, 255)
local C_BUTTON_DISABLE          = ccc3(120, 120, 120)


-- [操作变量]
-- ..


-- [枚举]
-- 结点枚举
M.NODETAG__LOWERBOUND           = 500
M.NODETAG_BUTTON_AUTOFIGHT      = M.NODETAG__LOWERBOUND + 10
M.NODETAG_BUTTON_STARTROUND     = M.NODETAG__LOWERBOUND + 20
-- 模式
M.MODE_DEATH                    = 0
M.MODE_FIGHT                    = 1
M.MODE_INTERACTIVE              = 2
M.MODE_END_NONE                 = 3
M.MODE_END_JUMP                 = 4
M.MODE_END_DIAMOND_SKIP         = 5
M.MODE_DIE                      = 6










-- [私有方法声明]
-- [实例方法][视图渲染]
-- ..
-- [视图析构]
-- ..

    


    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [合法性验证]
    self:assertParameters(params)



    -- [参数赋值]
    self._hasAlertOpen                          = false         -- 判断是否有弹出框存在（钻石购买，提示等）


    
    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendLayer.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][视图渲染]
-- 视图渲染
function M:viewRender()
    -- 加载按钮[自动战斗]
    self:loadButtonAutoFight()
    -- 加载按钮[回合结束]
    self:loadButtonEndRound()
    -- 加载按钮[跳过]
    self:loadButtonJump()
    -- 加载按钮[钻石跳过]
    self:loadButtonDiamondSkip()

    -- 加载渲染以外其他操作
    self:loadRenderOtherwise()
end

-- 加载按钮[自动战斗]
function M:loadButtonAutoFight()
    -- 定义参数
    local params = {
        text              = _s("FIGHT_AUTO"),
        fileNormal        = "f_btn_bg_orange_n.png",
        fileSelect        = "f_btn_bg_orange_s.png",
        fileDisable       = "f_btn_bg_gray_d.png",
        handler           = self.onButtonAutoFight,
        x                 = 960 + M_BUTTON.x,               -- display.width + M_BUTTON.x,
        y                 = 320 + M_BUTTON.y,               -- display.cy + M_BUTTON.y,
        nodeTag           = M.NODETAG_BUTTON_AUTOFIGHT,
    }

    -- 加载实体
    self.buttonAutoFight = self:loadButton(params)
end

-- 加载按钮[回合结束]
function M:loadButtonEndRound()
    -- 定义参数
    local params = {
        text              = _s("FIGHT_ROUND_END"),
        fileNormal        = "f_btn_bg_blue_n.png",
        fileSelect        = "f_btn_bg_blue_s.png",
        fileDisable       = "f_btn_bg_gray_d.png",
        handler           = self.onButtonEndRound,
        x                 = 960 + M_BUTTON.x,               -- display.width + M_BUTTON.x,
        y                 = 320 - M_BUTTON.y,               -- display.cy - M_BUTTON.y,
        nodeTag           = M.NODETAG_BUTTON_STARTROUND,
    }

    -- 加载实体
    self.buttonEndRound = self:loadButton(params)
end

-- 加载按钮[跳过]
function M:loadButtonJump()
    -- 定义参数
    local params = {
        text              = _s("FIGHT_SKIP"),
        fileNormal        = "f_btn_bg_blue_n.png",
        fileSelect        = "f_btn_bg_blue_s.png",
        fileDisable       = "f_btn_bg_gray_d.png",
        handler           = self.onButtonJump,
        x                 = 960 + M_BUTTON.x,               -- display.width + M_BUTTON.x,
        y                 = 320 - M_BUTTON.y,               -- display.cy - M_BUTTON.y,
        nodeTag           = M.NODETAG_BUTTON_STARTROUND,
    }

    -- 加载实体
    self.buttonJump = self:loadButton(params)
end

-- 加载按钮[钻石跳过]
function M:loadButtonDiamondSkip()
    -- 定义参数
    local params = {
        text              = _s("FIGHT_SKIP"),
        fileNormal        = "f_btn_bg_blue_n.png",
        fileSelect        = "f_btn_bg_blue_s.png",
        fileDisable       = "f_btn_bg_gray_d.png",
        handler           = self.onButtonDiamondSkip,
        x                 = 960 + M_BUTTON.x,               -- display.width + M_BUTTON.x,
        y                 = 320 - M_BUTTON.y,               -- display.cy - M_BUTTON.y,
        nodeTag           = M.NODETAG_BUTTON_STARTROUND,
        padding           = { top = 18},
    }

    -- 加载实体
    self.buttonDiamondSkip = self:loadButton(params)

    local sprite = widget.newSprite{
        file = "shop/titlepic12.png",
        x = 30, 
        y = 50,
    }
    sprite:setScale(0.5)
    self.buttonDiamondSkip:addChild(sprite, 1000, 1000)

    -- [钻石跳过标签]
    -- 定义配置参数
    local params = {
        text        = GAMERATE:getJumpDiamond(),        -- 标签文字
        parentNode  = self.buttonDiamondSkip,           -- 父结点
        align       = display.CENTER,                   -- 对齐方式
        anchor      = display.CENTER,                   -- 锚点
        -- offset      = ccp(16, 11),                      -- 偏移量
        fontSize    = 18,
        mode        = TDUIUtils.MODE_LABEL_TTF_SHADOW,  -- 模式
        z           = 10,
    }
    -- 生成标签
    local label = TDUIUtils.loadLabelTTF(params)
    label:setPosition(15, 6)
end

-- 加载渲染以外其他操作
function M:loadRenderOtherwise()
    -- 切换按钮模式[战斗中]
    self:swithMode(M.MODE_DEATH)
end


---------


-- [实例方法][视图析构]
-- 视图析构
function M:viewDestructor()

end


---------


-- [实例方法][工具]
function M:loadButton(params)
    -- 参数赋值
    local text              = params.text               or ""
    local fileNormal        = params.fileNormal
    local fileSelect        = params.fileSelect
    local fileDisable       = params.fileDisable
    local handler           = params.handler
    local x                 = params.x                  or 0
    local y                 = params.y                  or 0
    local nodeTag           = params.nodeTag
    local z                 = params.z                  or 0
    local scale             = params.scale              or S_BUTTON

    -- 加载实体
    local button = self:newButton(text, fileNormal, fileSelect, fileDisable, handler, params.padding)
    button:setScale(scale)
    self:addChild(button, 0, nodeTag)

    -- 定位
    display.align(button, display.CENTER, x, y)

    return button
end

-- 实例化按钮
function M:newButton(text, fileNormal, fileSelect, fileDisable, handler, textPadding)
    -- 定义参数
    local params = {
        normalImage             = PHI_FIGHT_BUTTON .. fileNormal,
        selectImage             = PHI_FIGHT_BUTTON .. fileSelect,
        disableImage            = PHI_FIGHT_BUTTON .. fileDisable,
        listener                = function(sender) handler(self, sender) end,
        discolor                = ccc3(49, 24, 11) ,
        labelParams             = {
            text = text, 
            size = 20,
            textAlign = ui.TEXT_ALIGN_CENTER,
            padding = textPadding,
            shadow = true,
        },
    }

    -- 创建对象
    return widget.newButton(params)
end


---------


-- [实例方法][事件侦听]
-- 处理按钮事件响应[自动战斗]
function M:onButtonAutoFight(sender)
    -- [打印]
    -- print("onButtonAutoFight")


    -- [过滤判断]
    if self:isButtonAutoFightEnable() == false then
        -- print("[Button]AutoFight is disable!")
        return
    end


    -- [播放音效]
    self:playSound()


    -- [按钮状态]
    self:setAllButtonsDisable()
    -- [卡片状态]
    self:cleanAllCardsReady()


    -- [战斗逻辑]
    -- 获得顶层视图
    local view = self:getTopView()
    -- 发送请求
    view._flag = 1
    view:netSendAutoFight()
end

-- 处理按钮事件响应[回合结束]
function M:onButtonEndRound(sender)
    -- [打印]
    -- print("onButtonEndRound")
    

    -- 过滤判断
    if self:isButtonEndRoundEnable() == false then
        -- print("[Button]EndRound is disable!")
        return
    end


    -- [播放音效]
    self:playSound()

    -- [按钮状态]
    self:setAllButtonsDisable()
    -- [卡片状态]
    self:cleanAllCardsReady()


    -- [战斗逻辑]
    -- 获得顶层视图
    local view = self:getTopView()
    -- 发送请求
    view:netSendEndRound()
end

-- 处理按钮事件响应[跳过]
function M:onButtonJump(sender)
    -- [打印]
    -- print("onButtonJump") 

    -- 过滤判断
    if self:isButtonJumpEnable() == false then
        -- print("[Button]Jump is disable!")
        return
    end


    -- [播放音效]
    self:playSound()

    -- [按钮状态]
    self:setAllButtonsDisable()
    -- [卡片状态]
    self:cleanAllCardsReady()


    -- [战斗逻辑]
    -- 获得顶层视图
    local view = self:getTopView()
    -- 跳至结果
    view:skipToResult()
end

-- 处理按钮事件响应[钻石跳过]
function M:onButtonDiamondSkip(sender)
    -- [打印]
    -- print("onButtonDiamondSkip")
    

    -- 过滤判断
    if self:isButtonDiamondSkipEnable() == false then
        -- print("[Button]DiamondSkip is disable!")
        return
    end
    if ME:getDiamond() < tonumber(GAMERATE:getJumpDiamond()) then
        -- 设置弹出层标识
        self._hasAlertOpen = true
        -- 显示[钻石不足]对话框
        AlertLayer.new{
            titleStr    = _s("COMMON_NEWS_TIPS"),                                       -- 标题
            content     = _s("SHOP_DIAMOND_NOT_ENOUGH"),                                -- 内容
            btnStr      = { _s("COMMON_OK"), },                                         -- 按钮文字数组
            btnColor    = { AlertLayer.BTN_COLOR_YELLOW, },
            -- 按钮颜色数组
            btnClickEvent = {                   -- 按钮事件数组
                function()                      -- 按钮[确定]回调
                    -- 设置弹出层标识
                    self._hasAlertOpen = false
                end,
            }, 
        }
        return
    end


    -- [播放音效]
    self:playSound()


    -- [标识设置]
    -- 设置弹出层标识（为防止弹出对话框时不点确定，结果页出现的Bug）
    self._hasAlertOpen = true

    -- [网络请求]
    -- 获得网络请求数据
    local snParams   = TDStringUtils.split(self:getParent():getLevelId(), "-")
    local sn         = snParams[1]
    local subsn      = snParams[2]
    local star       = self:getParent():getDifficulty()

    -- 方法定义 
    local function success()
        self:setButtonDiamondSkipEnable()
        self:swithMode(M.MODE_END_JUMP)
        self:onButtonJump(sender)
    end
    local function fail()
        -- print("按钮[钻石跳过]网络请求失败")
        self:setButtonDiamondSkipEnable()
    end

    self:setButtonDiamondSkipDisable()
    
    -- if game.getGameState("SKIP_FIGHT_WITHOUT_ALERT") then
    --     -- 设置弹出层标识
    --     self._hasAlertOpen = false
    --     -- 发起网络请求
    --     NetHelper.createRequest(
    --        ACTID.MSG_SKIP_MAP,                      -- 请求ID
    --        success,                                 -- 回调函数
    --        {                                        -- 请求信息实体或参数Table
    --             sn      = sn,                       -- 关卡ID
    --             subsn   = subsn,                    -- 子关卡ID
    --             star    = star,                     -- 关卡星级
    --        },
    --        {
    --             failedCb = fail,                    -- 请求失败回调函数
    --         }
    --     )
    -- else
        -- 显示[钻石跳过]对话框
        AlertLayer.new{
            titleStr    = _s("COMMON_NEWS_TIPS"),                                       -- 标题
            content     = format(_s("FIGHT_BUY_JD_TIP"), GAMERATE:getJumpDiamond()),    -- 内容
            btnStr      = { _s("COMMON_OK"), _s("COMMON_CANCEL") },                     -- 按钮文字数组
            btnColor    = { AlertLayer.BTN_COLOR_YELLOW, AlertLayer.BTN_COLOR_YELLOW },
            isUserDiamond = true,
            diamondIndex = 7,
            -- 按钮颜色数组
            btnClickEvent = {                   -- 按钮事件数组
                function()                      -- 按钮[确定]回调
                    -- game.addGameState( "SKIP_FIGHT_WITHOUT_ALERT", true, true )
                    --打塔
                    if self:getParent():getMode().battle == MODE_BATTLE_EXPFBFLIGHT then 
                        -- 设置弹出层标识
                        self._hasAlertOpen = false
                        -- 发起网络请求
                        NetHelper.createRequest(
                           ACTID.MSG_SKIP_MAP,                      -- 请求ID
                           success,                                 -- 回调函数
                           {                                        -- 请求信息实体或参数Table
                                sn      = "0",                       -- 关卡ID
                                subsn   = "0",                      -- 子关卡ID
                                star    = 0,                        -- 关卡星级
                           },
                           {
                                failedCb = fail,                    -- 请求失败回调函数
                            }
                        )
                    else 
                        -- 设置弹出层标识
                        self._hasAlertOpen = false
                        -- 发起网络请求
                        NetHelper.createRequest(
                           ACTID.MSG_SKIP_MAP,                      -- 请求ID
                           success,                                 -- 回调函数
                           {                                        -- 请求信息实体或参数Table
                                sn      = sn,                       -- 关卡ID
                                subsn   = subsn,                    -- 子关卡ID
                                star    = star,                     -- 关卡星级
                           },
                           {
                                failedCb = fail,                    -- 请求失败回调函数
                            }
                        )
                    end
                    
                end,
                function()                      -- 按钮[取消]回调
                    -- 设置弹出层标识
                    self._hasAlertOpen = false
                    fail()
                end,
            }, 
        }
    -- end
end


---------


-- [实例方法][模式]
-- 处理模式
function M:swithMode(mode)
    if     mode == M.MODE_DEATH then 
        self:handleModeDeath()
    elseif mode == M.MODE_FIGHT then 
        self:handleModeFight()
    elseif mode == M.MODE_INTERACTIVE then 
        self:handleModeInteractive()
    elseif mode == M.MODE_END_NONE then 
        self:handleModeEndNone()
    elseif mode == M.MODE_END_JUMP then 
        self:handleModeEndJump()
    elseif mode == M.MODE_END_DIAMOND_SKIP then 
        self:handleModeEndDiamondSkip()
    elseif mode == M.MODE_DIE then 
        self:handleModeDie()
    end
end

-- 处理模式[死亡]
function M:handleModeDeath()
    -- [自动战斗]可见、禁用
    self:showButtonAutoFight()
    self:setButtonAutoFightDisable()
    -- [回合结束]可见、禁用
    self:showButtonEndRound()
    self:setButtonEndRoundDisable()
    -- [跳过]不见、禁用
    self:hideButtonJump()
    self:setButtonJumpDisable()
    -- [钻石跳过]不见、禁用
    self:hideButtonDiamondSkip()
    self:setButtonDiamondSkipDisable()
end

-- 处理模式[战斗中]
function M:handleModeFight()
    -- [自动战斗]可见、可用
    self:showButtonAutoFight()
    self:setButtonAutoFightEnable()
    -- [回合结束]可见、禁用
    self:showButtonEndRound()
    self:setButtonEndRoundDisable()
    -- [跳过]不见、禁用
    self:hideButtonJump()
    self:setButtonJumpDisable()
    -- [钻石跳过]不见、禁用
    self:hideButtonDiamondSkip()
    self:setButtonDiamondSkipDisable()
end

-- 处理模式[交互]
function M:handleModeInteractive(isSwithTexture)
    -- HelperView2.showNext(80, 200) ===

    -- 处理图片切换
    isSwithTexture = type(isSwithTexture) == "boolean" and isSwithTexture or false
    if isSwithTexture then 
        -- 开始攻击 按键
        self.buttonEndRound.setStateImages(
            { PHI_FIGHT_BUTTON .. "f_btn_bg_red_n.png", 
              PHI_FIGHT_BUTTON .. "f_btn_bg_red_s.png",
              PHI_FIGHT_BUTTON .. "f_btn_bg_gray_d.png",
            }, 
            true
        )
        self.buttonEndRound.setTextString(_s("IMAGE_FIGHT_BTN_STARTFIGHT"))
    else
        -- 回合结束按键
        self.buttonEndRound.setStateImages(
            { PHI_FIGHT_BUTTON .. "f_btn_bg_blue_n.png", 
              PHI_FIGHT_BUTTON .. "f_btn_bg_blue_s.png",
              PHI_FIGHT_BUTTON .. "f_btn_bg_gray_d.png",
            }, 
            true
        )
        self.buttonEndRound.setTextString(_s("FIGHT_ROUND_END"))
    end

    -- [自动战斗]可见、可用
    self:showButtonAutoFight()
    self:setButtonAutoFightEnable()
    -- [回合结束]可见、可用
    self:showButtonEndRound()
    self:setButtonEndRoundEnable()
    -- [跳过]不见、禁用
    self:hideButtonJump()
    self:setButtonJumpDisable()
    -- [钻石跳过]不见、禁用
    self:hideButtonDiamondSkip()
    self:setButtonDiamondSkipDisable()
end

-- 处理模式[结束(不跳过)]
function M:handleModeEndNone()
    -- [自动战斗]可见、禁用
    self:showButtonAutoFight()
    self:setButtonAutoFightDisable()
    -- [回合结束]可见、禁用
    self:showButtonEndRound()
    self:setButtonEndRoundDisable()
    -- [跳过]不见、禁用
    self:hideButtonJump()
    self:setButtonJumpDisable()
    -- [钻石跳过]不见、禁用
    self:hideButtonDiamondSkip()
    self:setButtonDiamondSkipDisable()
end

-- 处理模式[结束(可跳过)]
function M:handleModeEndJump()
    -- [自动战斗]可见、禁用
    self:showButtonAutoFight()
    self:setButtonAutoFightDisable()
    -- [回合结束]不见、禁用
    self:hideButtonEndRound()
    self:setButtonEndRoundDisable()
    -- [跳过]可见、可用
    self:showButtonJump()
    self:setButtonJumpEnable()
    -- [钻石跳过]不见、禁用
    self:hideButtonDiamondSkip()
    self:setButtonDiamondSkipDisable()
end

-- 处理模式[结束(钻石跳过)]
function M:handleModeEndDiamondSkip()
    -- [自动战斗]可见、禁用
    self:showButtonAutoFight()
    self:setButtonAutoFightDisable()
    -- [回合结束]不见、禁用
    self:hideButtonEndRound()
    self:setButtonEndRoundDisable()
    -- [跳过]不见、禁用
    self:hideButtonJump()
    self:setButtonJumpDisable()
    -- [钻石跳过]可见、可用
    self:showButtonDiamondSkip()
    self:setButtonDiamondSkipEnable()
end

-- 处理模式[死亡]
function M:handleModeDie()
    -- [自动战斗]可见、禁用
    self:setButtonAutoFightDisable()
    -- [回合结束]不见、禁用
    self:setButtonEndRoundDisable()
    -- [跳过]不见、禁用
    self:setButtonJumpDisable()
    -- [钻石跳过]可见、可用
    self:setButtonDiamondSkipDisable()

    -- 处理弹出层销毁
    if self._hasAlertOpen then 
        CCBCenter.popLastLevelCCBNode()
    end
end


---------


-- [实例方法][按钮操作]
-- 判断按钮是否可用[自动战斗]
function M:isButtonAutoFightEnable()
    return self:isButtonEnable(self.buttonAutoFight)
end

-- 判断按钮是否可用[回合结束]
function M:isButtonEndRoundEnable()
    return self:isButtonEnable(self.buttonEndRound)
end

-- 判断按钮是否可用[跳过]
function M:isButtonJumpEnable()
    return self:isButtonEnable(self.buttonJump)
end

-- 判断按钮是否可用[钻石跳过]
function M:isButtonDiamondSkipEnable()
    return self:isButtonEnable(self.buttonDiamondSkip)
end

-- 判断按钮是否可用
function M:isButtonEnable(button)
    return button:getState() ~= widget.STATE_BUTTON_DISABLE
end

-- 设置按钮状态可用[自动战斗]
function M:setButtonAutoFightEnable()
    self:setButtonEnable(self.buttonAutoFight)
end

-- 设置按钮状态可用[回合结束]
function M:setButtonEndRoundEnable()
    self:setButtonEnable(self.buttonEndRound)
end

-- 设置按钮状态可用[跳过]
function M:setButtonJumpEnable()
    self:setButtonEnable(self.buttonJump)
end

-- 设置按钮状态可用[钻石跳过]
function M:setButtonDiamondSkipEnable()
    self:setButtonEnable(self.buttonDiamondSkip)
end

-- 设置按钮状态可用
function M:setButtonEnable(button)
    button.setStateDisabled(false)
end

-- 设置按钮状态不可用[自动战斗]
function M:setButtonAutoFightDisable()
    self:setButtonDisable(self.buttonAutoFight)
end

-- 设置按钮状态不可用[回合结束]
function M:setButtonEndRoundDisable()
    self:setButtonDisable(self.buttonEndRound)
end

-- 设置按钮状态不可用[跳过]
function M:setButtonJumpDisable()
    self:setButtonDisable(self.buttonJump)
end

-- 设置按钮状态不可用[钻石跳过]
function M:setButtonDiamondSkipDisable()
    self:setButtonDisable(self.buttonDiamondSkip)
end

-- 设置按钮状态不可用
function M:setButtonDisable(button)
    button.setStateDisabled(true)
end

-- 设置所有按钮状态可用
function M:setAllButtonsEnable()
    self:setButtonAutoFightEnable()
    self:setButtonEndRoundEnable()
    self:setButtonJumpEnable()
    self:setButtonDiamondSkipEnable()
end

-- 设置所有按钮状态可用
function M:setAllButtonsDisable()
    self:setButtonAutoFightDisable()
    self:setButtonEndRoundDisable()
    self:setButtonJumpDisable()
    self:setButtonDiamondSkipDisable()
end

-- 设置按钮状态可见[自动战斗]
function M:showButtonAutoFight()
    self:showButton(self.buttonAutoFight)
end

-- 设置按钮状态不可见[自动战斗]
function M:hideButtonAutoFight()
    self:hideButton(self.buttonAutoFight)
end

-- 设置按钮状态可见[回合结束]
function M:showButtonEndRound()
    self:showButton(self.buttonEndRound)
end

-- 设置按钮状态不可见[回合结束]
function M:hideButtonEndRound()
    self:hideButton(self.buttonEndRound)
end

-- 设置按钮状态可见[跳过]
function M:showButtonJump()
    self:showButton(self.buttonJump)
end

-- 设置按钮状态不可见[跳过]
function M:hideButtonJump()
    self:hideButton(self.buttonJump)
end

-- 设置按钮状态可见[钻石跳过]
function M:showButtonDiamondSkip()
    self:showButton(self.buttonDiamondSkip)
end

-- 设置按钮状态不可见[钻石跳过]
function M:hideButtonDiamondSkip()
    self:hideButton(self.buttonDiamondSkip)
end

-- 设置按钮状态可见
function M:showButton(button)
    button:setVisible(true)
end

-- 设置按钮状态不可见
function M:hideButton(button)
    button:setVisible(false)
end

---------


-- [实例方法][辅助方法]
-- 清除所有卡片就绪状态
function M:cleanAllCardsReady()
    self:getTopView():cleanAllPlayerCardsReady()
end

---------


-- [实例方法][声音]
function M:playSound()
    SOUND:playEffectButton2()
end


---------


-- [实例方法][快捷引用]
-- 获得数据[我方玩家]
function M:getPlayerMe()
    return self:getParent():getPlayerMe()
end
-- 获得数据[敌方玩家]
function M:getPlayerOpponent()
    return self:getParent():getPlayerOpponent()
end

-- 获得数据[总]
function M:getData()
    return self:getParent():getData()
end

-- 获得数据[总]
function M:getRound()
    return self:getParent():getRound()
end

-- 获得顶层视图
function M:getTopView()
    return self:getParent()
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    -- TDAssertUtils.assertParameters(params)
end



return M