--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗卡组场景层类，定义战斗卡组场景层相关操作方法及逻辑实现。

-   存储卡牌相关信息。
-   提供卡牌与玩家数据之间交互的渠道。
-   调用卡牌战斗、技能渲染等操作。

]]

local M = class("FightCardGroupLayer", function()
    return display.newLayer()
end)



-- [包含文件]
-- ...



-- [操作变量]
-- 实体[英雄]
local EntityHero            = requireEN("EntityHero")
-- 实体[卡片]
local EntityCard            = requireEN("EntityCard")

-- 卡牌详细信息层
local CardDetailLayer       = requireL("CardDetailLayer")




-- [公共参数]
-- 常量
local NUM_VIEWED_CARDS          = 6                         -- 滚动层显示的卡牌数量
local M_CARD                    = acp(8, 0)                 -- 外边距[卡牌]
local A_TIME1                   = .30                       -- 动画时间1
local A_TIME2                   = .30                       -- 动画时间2
local C_ON_STAGE                = ccc3(120, 120, 120)       -- 颜色[卡牌上场]
local C_OFF_STAGE               = ccc3(255, 255, 255)       -- 颜色[卡牌下场]
local T_DELAYOPERATE            = 1.0                       -- 自动操作每张卡牌延迟时间

-- 枚举
M.NODETAG__LOWERBOUND           = 500
M.NODETAG_CEMETERY              = M.NODETAG__LOWERBOUND + 10










-- [构造方法]
--[[--
    创建实例
]]
function M:ctor(params)
    -- [验证参数合法性]
    TDAssertUtils.assertParameters(params)



    -- [参数赋值]
    local _owner                    = params.owner                                      -- 卡牌拥有者标识
    local _posCard                  = params.posCard                    or {}           -- 卡牌PO集合
    local _positionBeginStageDown   = params.positionBeginStageDown     or P_ZERO       -- 卡牌起始位置[下场卡牌]
    local _positionBeginStageUp     = params.positionBeginStageUp       or P_ZERO       -- 卡牌起始位置[上场卡牌]



    -- [辅助变量]
    local _cardsStageDown           = {}                                                -- 卡牌集[下场卡牌]
    local _cardsStageUp             = {}                                                -- 卡牌集[上场卡牌]
    local _cardsFight               = {}                                                -- 卡牌集[战斗]
    local _sizeStageDown                                                                -- 卡牌尺寸[下场卡牌]
    local _sizeStageUp                                                                  -- 卡牌尺寸[上场卡牌]
    local _positionCemetery         = P_ZERO                                            -- 卡牌位置[墓地]
    local _positionsStageDown       = {}                                                -- 卡牌位置集[下场卡牌]
    local _positionsStageUp         = {}                                                -- 卡牌位置集[上场卡牌]
    local _positionsFight           = {}                                                -- 卡牌位置集[战斗]

    







    -- [私有方法声明]
    -- [渲染视图]
    local viewRender
    -- 加载状态变量
    local loadStatuses
    -- 加载尺寸
    local loadSizes
    -- 加载尺寸[下场卡牌]
    local loadSizeIcon
    -- 加载尺寸[上场卡牌]
    local loadSizeImage
    -- 加载卡牌位置
    local loadPositions
    -- 加载卡牌位置[下场卡牌]
    local loadPositionsStageDown
    -- 加载卡牌位置[上场卡牌]
    local loadPositionsStageUp
    -- 加载卡牌位置[墓地]
    local loadPositionCemetery
     -- 加载卡牌位置集[战斗]
    local loadPositionsFight
    -- 加载卡牌背景
    local loadBackgrounds
    -- 加载卡牌背景[下场卡牌]
    local loadBackgroundsStageDown
    -- 加载卡牌背景[上场卡牌]
    local loadBackgroundsStageUp
    -- 加载卡牌背景[墓地]
    local loadBackgroundCemetery
    -- 加载卡牌
    local loadCards
    -- 加载渲染视图其他操作
    local loadRenderOtherwise
    -- 设置卡片可见性
    local setCardsVisiblitiy

    -- [事件侦听]
    local onFilterCardDown
    local onFilterCardUp
    local onTouchCardDown
    local onTouchCardUp

    -- [辅助方法]
    -- 获得卡牌尺寸
    local getCardSize
    -- 根据卡牌类型标识获得卡牌文件名[图标/图片](用于计算卡牌尺寸)
    local getCardTempFileName
    -- 根据卡牌类型标识获得卡牌背景文件名[图标/图片]
    local getBackgroundFileName
    -- 根据卡牌实体、实体索引、卡牌类型[图标/图片]排列卡牌位置
    local locateCard
        

        

        





    -- [私有方法定义]
    -- [渲染视图]
    -- 视图渲染
    function viewRender()
        -- 加载状态变量
        loadStatuses()

        -- 加载卡牌背景
        loadBackgrounds()
        -- 加载卡牌
        loadCards()

        -- 加载渲染视图其他操作
        loadRenderOtherwise()
    end

    -- 加载状态变量
    function loadStatuses()
        -- 加载尺寸
        loadSizes()
        -- 加载卡牌位置
        loadPositions()
    end

    -- 加载尺寸
    function loadSizes()
        -- 加载尺寸[下场卡牌]
        loadSizeStageDown()
        -- 加载尺寸[上场卡牌]
        loadSizeStageUp()
    end

    -- 加载尺寸[下场卡牌]
    function loadSizeStageDown()
        _sizeIcon = getCardSize(TYPE_CARD_ICON)
    end

    -- 加载尺寸[上场卡牌]
    function loadSizeStageUp()
        _sizeImage = getCardSize(TYPE_CARD_IMAGE)
    end

    -- 加载卡牌位置
    function loadPositions()
        -- 加载卡牌位置[下场卡牌]
        loadPositionsStageDown()
        -- 加载卡牌位置[上场卡牌]
        loadPositionsStageUp()
        -- 加载卡牌位置[墓地]
        loadPositionCemetery()

        -- 加载卡牌位置集[战斗]
        loadPositionsFight()
    end

    -- 加载卡牌位置[下场卡牌]
    function loadPositionsStageDown()
        -- 如果卡牌尺寸未设置则进行设置
        local positions     = {}
        local cardType      = TYPE_CARD_ICON
        local cardSize      = _sizeIcon
        local offset        = _positionBeginStageDown

        for i = 1, NUM_VIEWED_CARDS do
            local position = ccp(cardSize.width * (i - 1) + M_CARD.x * (i - 2), 0)
            position = ccpAdd(position, offset)

            positions[#positions + 1] = position
        end
        _positionsStageDown = positions
    end

    -- 加载卡牌位置[上场卡牌]
    function loadPositionsStageUp()
        local positions     = {}
        local cardType      = TYPE_CARD_IMAGE
        local cardSize      = _sizeImage
        local offset        = _positionBeginStageUp

        for i = 1, NUM_VIEWED_CARDS do
            local position = ccp(cardSize.width * (i - 1) + M_CARD.x * (i - 2), 0)
            position = ccpAdd(position, offset)

            positions[#positions + 1] = position
        end
        _positionsStageUp = positions
    end

    -- 加载卡牌位置[下场卡牌]
    function loadPositionCemetery()
        -- 配置属性
        local cardType      = TYPE_CARD_ICON
        local cardSize      = _sizeIcon
        local offset        = _positionBeginStageDown

        local i = NUM_VIEWED_CARDS + 1                                      -- 位置索引
        local position = ccp(cardSize.width * (i - 1) + M_CARD.x * (i - 2), 0)
        position = ccpAdd(position, offset)

        _positionCemetery = position
    end

    -- 加载卡牌位置集[战斗]
    function loadPositionsFight()
        local positions = {}

        -- 获得位置集[下场卡牌]
        local positionsStageDown    = _positionsStageDown
        -- 获得位置集[上场卡牌]
        local positionsStageUp      = _positionsStageUp


        -- 加载未出场位置
        positions[0] = positionsStageDown[1]

        -- 加载下场卡牌
        for i = 1, #positionsStageDown do
            positions[i] = positionsStageDown[i]
        end
        -- 加载上场卡牌
        for i = 1, #positionsStageUp do
            positions[IDX_STAGEUP_INC + i] = positionsStageUp[i]
        end
        -- 加载墓地卡片
        positions[IDX_CEMETERY] = _positionCemetery

        _positionsFight = positions
    end

    -- 加载卡牌背景
    function loadBackgrounds()
        -- 加载卡牌背景[下场卡牌]
        loadBackgroundsStageDown()
        -- 加载卡牌背景[上场卡牌]
        loadBackgroundsStageUp()

        -- 加载卡牌背景[墓地]
        loadBackgroundCemetery()
    end

    -- 加载卡牌背景[下场卡牌]
    function loadBackgroundsStageDown()
        -- 配置属性
        local cardType = TYPE_CARD_ICON

        -- 生成实体
        local fileName, sprite
        for i = 1, NUM_VIEWED_CARDS do
            -- 实例对象
            fileName = getBackgroundFileName(cardType)                      -- 获得卡牌背景文件名
            sprite = display.newSprite(fileName)                            -- 实例化卡牌背景

            -- 添加到容器
            self:addChild(sprite)

            -- 设置参数
            locateCard(sprite, i, cardType)                                 -- 定位位置
        end
    end

    -- 加载卡牌背景[上场卡牌]
    function loadBackgroundsStageUp()
        -- 配置属性
        local cardType = TYPE_CARD_IMAGE

        -- 生成实体
        local fileName, sprite
        for i = 1, NUM_VIEWED_CARDS do
            -- 实例对象
            fileName = getBackgroundFileName(cardType)                      -- 获得卡牌背景文件名
            sprite = display.newSprite(fileName)                            -- 实例化卡牌背景

            -- 添加到容器
            self:addChild(sprite)

            -- 设置参数
            locateCard(sprite, i, cardType)                                 -- 定位位置
        end
    end

    -- 加载卡牌背景[墓地]
    function loadBackgroundCemetery()
        -- 配置属性
        local cardType = TYPE_CARD_ICON

        -- 生成实体
        local i = NUM_VIEWED_CARDS + 1                                      -- 位置索引
        local sprite = display.newSprite(PHI_FIGHT .. "Cemetery.png")        -- 实例化卡牌

        -- 添加到容器
        self:addChild(sprite, 0, M.NODETAG_CEMETERY)

        -- 设置参数
        locateCard(sprite, i, cardType)                                     -- 定位位置
    end

    -- 加载卡牌
    function loadCards() 
        for i = 1, #_posCard do
            local params = {
                po                      = _posCard[i],                  -- PO对象
                owner                   = _owner,                       -- 拥有者标识
                positions               = _positionsFight,              -- 位置集
                index                   = 0,                            -- 位置索引
                isPositionAutoCorrect   = true,                         -- 自动校正原型(如果自动校正，则设置_index时会自动将位置、旋转角度、缩放比例等定位到指定索引位置)
            }
            local up    = CardFactory.newStageUp(params)                -- 下场卡片
            local down  = CardFactory.newStageDown(params)              -- 上场卡片

            up:setAssociatedCard(down)
            down:setAssociatedCard(up)
 
            self:addChild(up)
            self:addChild(down)

            _cardsStageUp[#_cardsStageUp + 1] = up
            _cardsStageDown[#_cardsStageDown + 1] = down
        end
    end
    
    -- 加载渲染视图其他操作
    function loadRenderOtherwise() 
        -- 设置卡片初始状态
        -- setCardInitializeStates()
        -- 设置卡片可见性
        setCardsVisiblitiy()
    end

    -- 设置卡片可见性
    function setCardsVisiblitiy()
        for _, card in pairs(_cardsStageDown) do
            card:setVisible(false)
            card:getAssociatedCard():setVisible(false)
        end
    end


    ---------


    -- [辅助方法]
    -- 获得卡牌尺寸[图标/图片]
    function getCardSize(cardType)
        -- 获得临时卡牌文件名
        local tempFileName = getCardTempFileName(cardType)
        -- 加载临时卡牌
        local tempCard = display.newSprite(tempFileName)
        -- 合法性验证
        TDAssertUtils.assertNotNil(tempCard)

        return tempCard:getContentSize()
    end

    -- 根据卡牌类型标识获得卡牌文件名[图标/图片](用于计算卡牌尺寸)
    function getCardTempFileName(cardType)
        local fileName

        if cardType == TYPE_CARD_ICON then
            fileName = string.gsub(TPL_CARD_ICON, "{0}", 1)
        elseif cardType == TYPE_CARD_IMAGE then
            fileName = string.gsub(TPL_CARD_IMAGE, "{0}", 1)
        else
            TDExceptionUtils.raiseUnRunHereException()
        end

        return fileName
    end

    -- 根据卡牌类型标识获得卡牌背景文件名[图标/图片]
    function getBackgroundFileName(cardType)
        local fileName

        if cardType == TYPE_CARD_ICON then
            fileName = PHI_FIGHT .. "bg_card_icon.png"
        elseif cardType == TYPE_CARD_IMAGE then
            fileName = PHI_FIGHT .. "bg_card_image.png"
        else
            TDExceptionUtils.raiseUnRunHereException()
        end

        return fileName
    end

    -- 根据卡牌实体、实体索引、卡牌类型[图标/图片]排列卡牌位置
    function locateCard(entity, index, cardType)
        -- 如果卡牌尺寸未设置则进行设置
        local cardSize = getCardSize(cardType)

        local position = ccp(cardSize.width * (index - 1) + M_CARD.x * (index - 2), 0)
        local offset = getCardPositionBegin(cardType)

        position = ccpAdd(position, offset)

        entity:setPosition(position)
    end

    -- 根据卡牌类型标识获得卡牌起始位置[图标/图片]
    function getCardPositionBegin(cardType)
        local position

        if cardType == TYPE_CARD_ICON then
            position = _positionBeginStageDown
        elseif cardType == TYPE_CARD_IMAGE then
            position = _positionBeginStageUp
        else
            TDExceptionUtils.raiseUnRunHereException()
        end

        return position
    end

    -- 获得卡牌集
    function self:getCards()
        return _cardsStageDown
    end
    -- 设置位置集
    function self:getPositions()
        return _positionsFight
    end












































    -- 加载卡牌[上场卡牌]
    function loadCardsImage()
        -- 配置属性
        local cardType = TYPE_CARD_IMAGE
        local handler  = onImageTouched

        -- 生成实例
        for i = 1, NUM_VIEWED_CARDS do
            -- 实例对象
            local params = {
                po          = self.poCards[i],                              -- 卡牌PO
                tagOwner    = self.owner,                                   -- 卡牌类型（图片或图标）
                ["handler"] = handler,                                      -- 点击触碰函数
            }
            local sprite = CardFacade.newCardOnStage(params)                -- 根据卡牌PO实例化卡牌对象
            display.align(sprite, display.CENTER)

            -- 添加到容器
            self:addChild(sprite)
            self.images[#self.images + 1] = sprite

            -- 设置参数
            alignCardEntity(sprite, i, cardType)                            -- 定位位置
        end
    end


    ---------


    -- [卡牌集调度]
    -- ..


    ---------


    -- [数据源]
    -- 随机获得一个卡牌PO
    function getRandomCardPo()
        -- 创建一个卡牌PO对象
        local po = CardPo.new()

        -- 配置属性
        po.name              = po.name                           -- 名称
        po.icon              = po.icon                           -- 图标
        po.image             = po.image                          -- 图片
        po.star              = po.star                           -- 星级
        po.level             = po.level                          -- 卡牌等级
        po.cost              = po.cost                           -- COST值（上场值）
        po.ethnicity         = po.ethnicity                      -- 种族
        po.waitRounds        = po.waitRounds                     -- 等待回合数
        po.hp                = po.hp                             -- 生命值HP
        po.attack            = po.attack                         -- 攻击力
        po.defense           = po.defense                        -- 防御力
        po.skills            = po.skills                         -- 技能集合ctor
        po.waitRoundsCurrent = waitRounds                        -- 等待回合数（当前）

        return po
    end


    ---------


    -- [自动操作]
    -- 卡牌自动上场
    function autoOnStage()
        -- print("autoOnStage()")

        -- 过滤判断
        if TDArrayUtils.isEmpty(self.iconsFighting) then 
            return
        end


        -- 执行操作
        for _,v in ipairs(self.iconsFighting) do
            if v.isReady() and (v.isOnStage == false) then
                onIconTouched(v)
            end
        end
    end

    -- 卡牌自动战斗
    function autoFight()
        -- print("owner: " .. self:getParent().owner .. ", " .. "autoFight()")

        -- 过滤判断
        if TDArrayUtils.isEmpty(self.imagesFighting) then 
            -- 回调处理所有攻击结束后的逻辑
            callbackAllFightEnded()
        else
            -- 映射战斗序列
            mapImagesFightingSequence()
            -- 延迟执行卡牌序列战斗操作
            delayToRunImagesFightingSequenceFight()
        end
    end


    ---------


    -- [战斗逻辑]
    -- 映射战斗序列
    function mapImagesFightingSequence()
        self.imagesFightingSequence = {}

        -- 序列源
        local src = self.imagesFighting
        local dst = self.imagesFightingSequence

        if TDArrayUtils.isNotEmpty(src) then
            for i = 1,#src do
                dst[i] = src[i]
                dst[i].sequenceId = i
            end
        end

        return dst
    end

    -- 延迟执行卡牌序列战斗操作
    function delayToRunImagesFightingSequenceFight() 
        -- 配置参数
        local delay = 1.5


        -- 创建Action序列
        local action = transition.sequence({
            CCDelayTime:create(delay),                                      -- 延迟
            CCCallFunc:create(function()                                    -- 回调函数
                fightSequence()
            end),   
        })
        self:runAction(action)
    end

    -- 依照战斗序列依次执行卡牌攻击逻辑
    function fightSequence()
        ------------------------------------------------------------
        -- 意图：
        --      依次取出并删除战斗序列头部第一个元素，执行战斗逻辑
        ------------------------------------------------------------
        -- 获得序列
        local sequence = self.imagesFightingSequence

        -- 过滤判断
        -- 如果序列为空，则切换回合
        if TDArrayUtils.isEmpty(sequence) then
            callbackAllFightEnded()
            return
        end


        -- 获得卡牌[攻击方]
        local fighter = popupSequenceCard()
        -- 获得卡牌[受伤方]
        local injuryer, injuryerType = judgeInjuryer(fighter.sequenceId)


        -- 执行卡牌攻击
        local params = {
            ["injuryer"]        = injuryer,
            ["injuryerType"]    = injuryerType,
            ["delay"]           = delay,
            ["callback"]        = fightSequence,
        }
        fighter:fight(params)
    end

    -- 取出并移除对战卡牌序列第一张卡牌
    function popupSequenceCard()
        -- 获得序列
        local sequence = self.imagesFightingSequence
        -- 存储第一个元素
        local first = sequence[1]
        -- 删除第一个元素
        removeTableAtIndex(sequence, 1)

        return first
    end


    ---------


    -- [回调处理]

    -- 回调处理所有攻击结束后的逻辑
    function callbackAllFightEnded()
        -- 执行下一回合
        self:getParent():getParent():turnRound()
    end


    ---------


    -- [事件处理]
    -- 处理点击事件响应[卡牌图标]
    function onIconTouched(sender)
        -- 点击过滤
        if sender:isVisible() == false then
            -- print("onIconTouched: visible hidden!")
            return 
        end
        if sender.isTouchLock == true then
            -- print("onIconTouched: isTouchLock!")
            return 
        end
        if sender.isReady() == false then 
            -- print("onIconTouched: unReady!")
            -- 显示卡牌详细信息
            showCardDetail(sender.po)
            return 
        end

        -- print("onIconTouched")


        -- 判断卡牌是否已经上场
        local isOnStage = sender.isOnStage
        -- 处理上/下场逻辑
        if isOnStage == true then 
            handleOffStage(sender)
            sender.isOnStage = false
        else
            handleOnStage(sender)
            sender.isOnStage = true
        end
    end
    -- 处理点击事件响应[卡牌图片]
    function onImageTouched(sender)
        -- 点击过滤
        if sender:isVisible() == false then
            -- print("onImageTouched: visible hidden!")
            return 
        end
        if self:isLockCard(sender) then
            -- print("onImageTouched: isTouchLock!")
            return 
        end
        
        -- print("onImageTouched")

        -- 显示卡牌详细信息
        showCardDetail(sender.po)
    end


    ---------


    -- [辅助方法]
    -- 获得攻击到敌方的受伤对象
    function judgeInjuryer(index, opponent)
        -- 返回值定义
        local injuryer = nil
        local injuryerType = -1


        -- 默认赋值
        opponent = opponent or self:getParent().opponent


        -- [逻辑判断]
        local opponentImagesFighting = opponent:cardImagesFighting()    -- 敌方卡牌
        if opponentImagesFighting ~= nil and index <= #opponentImagesFighting then 
            injuryer = opponentImagesFighting[index]
            injuryerType = TYPE_ROLE_CARD
        else
            injuryer = opponent:blood()
            injuryerType = TYPE_ROLE_HERO
        end

        return injuryer, injuryerType
    end

    -- 根据卡牌体索引获得卡牌[下场卡牌]位置
    function positionIconByLocationNo(no)
        return positionByLocationNo(no, TYPE_CARD_ICON)
    end

    -- 根据卡牌体索引获得卡牌[上场卡牌]位置
    function positionImageByLocationNo(no)
        return positionByLocationNo(no, TYPE_CARD_IMAGE)
    end

    -- 根据卡牌体索引获得卡牌位置
    function positionByLocationNo(no, cardType)
        if isGeneratedPositionCards() == false then 
            generatePositionCards()
        end

        return (cardType == TYPE_CARD_ICON) and self.positionsIcon[no] or self.positionsImage[no]
    end

    -- 判断是否已经生成卡牌坐标位置（图标/图片）
    function isGeneratedPositionCards()
        return self.positionsIcon ~= nil and self.positionsImage ~= nil
    end

    -- 生成卡牌坐标位置（图标/图片）
    function generatePositionCards()
        -- [下场卡牌]
        -- 如果卡牌尺寸未设置则进行设置
        local positions = {}
        local cardType = TYPE_CARD_ICON
        local cardSize = getCardSize(cardType)
        local offset = getPositionStartByTypeCard(cardType)

        for i = 1, #self.poCards do
            local position = ccp(cardSize.width * (i - 1) + M_CARD.x * (i - 2), 0)
            position = ccpAdd(position, offset)

            positions[#positions + 1] = position
        end
        self.positionsIcon = positions



        -- [上场卡牌]
        local positions = {}
        local cardType = TYPE_CARD_IMAGE
        local cardSize = getCardSize(cardType)
        local offset = getPositionStartByTypeCard(cardType)

        for i = 1, #self.poCards do
            local position = ccp(cardSize.width * (i - 1) + M_CARD.x * (i - 2), 0)
            position = ccpAdd(position, offset)

            positions[#positions + 1] = position
        end
        self.positionsImage = positions
    end

    -- 获得墓地位置
    function positionCemetery()
        local node = self:getChildByTag(M.NODETAG_CEMETERY)
        return ccp(node:getPosition())
    end

    -- 处理卡牌上场逻辑
    function handleOnStage(sender, callback)
        -- 获得卡牌实体[上场卡牌]
        local imageEntity = sender.imageEntity
        -- 存储当前图片到战斗栈imagesFighting中
        self.imagesFighting[#self.imagesFighting + 1] = imageEntity
        -- 设置卡牌实体颜色[下场卡牌]
        sender:setColor(C_ON_STAGE)


        -- 动画显示
        -- 动画属性
        local positionTarget    = positionImageByLocationNo(#self.imagesFighting)
        -- 初始设定
        imageEntity:setVisible(true)
        imageEntity:setPosition(sender:getPosition())
        imageEntity:setScaleY(0)
        -- 停止当前动作
        imageEntity:stopAllActions()
        -- 创建Action序列
        local action = transition.sequence({
            CCScaleTo:create(A_TIME1, 1.0),                                 -- 还原比例
            CCMoveTo:create(A_TIME2, positionTarget),                       -- 向上移动
            CCCallFunc:create(function()                                    -- 回调函数
                -- 锁定点击
                imageEntity.isTouchLock = false
                -- 刷新界面卡牌显示
                refleshView()
                -- 执行回调处理   
                if callback ~= nil then 
                    -- 调用回调函数
                    callback()
                    -- 重新排列好友序列 
                    realignCardsSequence()
                end
            end),   
        })
        imageEntity:runAction(action)


        -- 显示卡牌图标Ready
        sender:hideReady()
    end

    -- 处理卡牌下场逻辑
    function handleOffStage(sender) 
        -- 获得卡牌实体[上场卡牌]
        local imageEntity = sender.imageEntity
        -- 移除战斗栈图标imagesFighting
        removeTableAtEntity(self.imagesFighting, imageEntity)
        -- 设置卡牌实体颜色[下场卡牌]
        sender:setColor(C_OFF_STAGE)


        -- 动画显示
        -- 动画属性
        local positionTarget    = sender:getPosition()
        -- 初始设定
        imageEntity:setVisible(true)
        -- 停止当前动作
        imageEntity:stopAllActions()
        -- 创建Action序列
        local action = transition.sequence({
            CCMoveTo:create(A_TIME1, ccp(sender:getPositionX(), sender:getPositionY())),                -- 向下移动
            CCScaleTo:create(A_TIME2, 1.0, 0),                                                          -- 缩放比例
            CCCallFunc:create(function()                                                                -- 回调函数
                imageEntity:setVisible(false)
            end),
        })
        imageEntity:runAction(action)


        -- 显示卡牌图标Ready
        sender:showReady()
        -- 刷新界面卡牌显示
        refleshView()
    end

    -- 刷新界面卡牌显示
    function refleshView()
        -- [上场卡牌]
        local position
        for i,v in ipairs(self.imagesFighting) do
            -- 停止所有动作
            v:stopAllActions()

            -- 动画显示
            -- 动画属性
            position = self.positionsImage[i]
            -- 初始设定
            v:setVisible(true)
            -- 创建Action序列
            local action = transition.sequence({
                CCScaleTo:create(A_TIME1, 1.0),                                                             -- 还原比例
                CCMoveTo:create(A_TIME2, position),                                                         -- 向下移动
            })

            v:runAction(action)
        end
    end

    -- 显示卡牌详细信息
    function showCardDetail(po)
        local topView = self:getParent():getParent()

        local layer = CardDetailLayer.new({poCard = po})
        topView:addChild(layer)
    end

    -- 判断是否所有卡牌已经全部死亡
    function isAllCardsDie()
        -- 1、判断场上卡牌是否已经全灭
        local aliveOnStage = self.iconsFighting
        if (#aliveOnStage > 0) then
            return false
        end

        -- 2、判断场下卡牌是否已经全灭
        local aliveOffStage = self.icons
        for _,v in ipairs(aliveOffStage) do
            if (#v.isOnStage == false) then
                return false
            end
        end

        return true
    end
    


    ---------


    -- [工具方法]
    -- 根据实体删除表格中的对象
    function removeTableAtEntity(aTable, entity)
        for i,v in ipairs(aTable) do
            if entity == v then
                removeTableAtIndex(aTable, i)
                break
            end
        end
    end 
    -- 根据索引删除表格中的对象
    function removeTableAtIndex(aTable, index)
        table.remove(aTable, index)
    end
    -- 判断表格中是否包含有实例
    function isContainsEntity(aTable, entity)
        local isContains = false
        local index = 0

        for i,v in pairs(aTable) do
            if v == entity then 
                isContains = true
                index = i
                break
            end
        end

        return isContains, index
    end
        

        

        





    -- [实例化对象]
    viewRender()

    -- 锁定卡牌点击
    function self:lockCard(card)

    end
    -- 解锁卡牌点击
    function self:unlockCard(card)

    end
    -- 判断卡牌是否锁定
    function self:isLockCard(card)

    end

    -- 获得战斗卡牌集
    function self:cardImagesFighting()
        return self.imagesFighting
    end

    -- 递减已上场卡牌的等待回合数
    function self:decreaseWaitRounds()
        for i,v in ipairs(self.iconsFighting) do
            v.waitRoundsDecrease()
        end
    end

    -- 自动操作
    function self:autoOperate()
        -- print("autoOperate()")

        -- 卡牌自动上场
        autoOnStage()
        -- 卡牌自动战斗
        autoFight()
    end

    -- 自动操作
    function self:autoFight()
        -- print("autoOperate()")

        -- 卡牌自动战斗
        autoFight()
    end

    -- 生成卡牌
    function self:spawnCard()
        -- 条件过滤
        -- 当索引超时卡牌数量时退出
        if #self.icons + 1 > #self.poCards then 
            -- print("overflow poCards index!")
            return
        end
        -- 当场上卡牌数超过设定的最大数目时等待
        if #self.icons + 1 > NUM_VIEWED_CARDS then 
            -- print("cards on stage is full!")
            return
        end





        -- [下场卡牌]
        -- 配置属性
        local cardType      = TYPE_CARD_ICON
        local handler       = onIconTouched
        local indexStart    = 1
        local index         = self:spawnCardIndex()

        -- 生成实例
        local params = {
            po          = self.poCards[index],                          -- 卡牌PO
            tagOwner    = self.owner,                                   -- 卡牌类型（图片或图标）
            ["handler"] = handler,                                      -- 点击触碰函数
        }
        local sprite = CardFacade.newCardOffStage(params)               -- 根据卡牌PO实例化卡牌对象

        -- 锁定实体
        sprite.isLockTouch = false                                      -- 锁定点击操作
        -- 添加到容器
        self:addChild(sprite)
        self.icons[index] = sprite
        self.iconsFighting[#self.iconsFighting + 1] = sprite


        -- 动画显示
        -- 动画属性
        local positionStart     = positionIconByLocationNo(indexStart)
        local positionTarget    = positionIconByLocationNo(index)
        -- 初始设定
        sprite:setPosition(positionStart)
        sprite:setScaleX(0)
        -- 创建Action序列
        local action = transition.sequence({
            CCScaleTo:create(A_TIME1, 1.0),             -- 还原比例
            CCMoveTo:create(A_TIME2, positionTarget),   -- 向右移动
            CCCallFunc:create(function() local entity = sprite; entity.isLockTouch = true; end),  -- 回调函数
        })
        sprite:runAction(action)
        -- 记录图标对象
        local icon = sprite





        -- [上场卡牌]
        -- 配置属性
        cardType        = TYPE_CARD_IMAGE
        handler         = onImageTouched

        -- 实例对象
        params = {
            po          = self.poCards[index],                          -- 卡牌PO
            tagOwner    = self.owner,                                   -- 卡牌类型（图片或图标）
            positionDie = positionCemetery(),                           -- 卡牌死亡位置
            ["handler"] = handler,                                      -- 点击触碰函数
        }
        sprite = CardFacade.newCardOnStage(params)                      -- 根据卡牌PO实例化卡牌对象

        -- 锁定实体
        sprite.isLockTouch = false                                      -- 锁定点击操作
        -- 设置代理对象
        sprite.delegate = self                                          -- 设置代理

        -- 添加到容器
        self:addChild(sprite)
        self.images[#self.images + 1] = sprite





        -- [配置关联性]
        -- 设置image对象隐藏显示
        sprite:setVisible(false)
        -- 绑定image对象到icon上
        icon.imageEntity = sprite
        -- 标识卡牌当前上场状态
        icon.isOnStage = false  
    end

    -- 获得生成卡牌的索引
    function self:spawnCardIndex()
        local index = 0

        if TDArrayUtils.isNotEmpty(self.iconsFighting) then
            for i = 1, #self.iconsFighting do
                local item = self.iconsFighting[i]
                if item.isOnStage == false then 
                    index = index + 1
                end
            end
        end

        return index + 1
    end

    -- 处理卡牌死亡
    function self:handleCardDie(card)
        -- 处理卡牌死亡逻辑
        card:die()
        -- 从卡牌集合中移除死亡卡牌
        removeTableAtEntity(self.imagesFighting, card)

        if isAllCardsDie() then
            -- 顶层视图
            local topView = self:getParent():getParent()
            topView:gameOver()
        end
    end

    -- 判断玩家卡牌是否处于等待上场状态
    function self:isCardsWaitingUpStage()
        local isWaiting = false

        -- 过滤判断
        local icons = self.iconsFighting
        if (TDArrayUtils.isNotEmpty(icons)) then 
            for _,v in ipairs(icons) do
                if (v:waitRoundsCurrent() <= 0) and (v.isOnStage ~= true) then
                    isWaiting = true
                    break
                end
            end
        end

        return isWaiting
    end

    -- 处理回合攻击
    function self:fightRound(opponent)
        -- 重新排列好友序列 
        realignCardsSequence()
        -- 攻击敌方卡牌
        self:fightCards(opponent)
    end

    -- 重新排列好友序列 
    function realignCardsSequence(callback)
        -- print("[重新排列好友序列]realignCardsSequence() called!")

        local iconsFighting = self.iconsFighting
        local indexAlign = 1

        for i = 1, #iconsFighting do
            local item = iconsFighting[i]

            if item.isOnStage then
                item:setVisible(false)
            else
                -- 动画显示
                -- 动画属性
                local positionTarget    = positionIconByLocationNo(indexAlign)
                -- 创建Action序列
                local action = transition.sequence({
                    CCMoveTo:create(A_TIME1, positionTarget),   -- 向右移动
                })
                item:runAction(action)

                -- 排列索引加1
                indexAlign = indexAlign + 1
            end
        end




        local imagesFighting = self.imagesFighting
        local indexAlign2 = 1

        for i = 1, #imagesFighting do
            local item = iconsFighting[i]

            if item.isOnStage then
                item:setVisible(false)
            else
                -- 动画显示
                -- 动画属性
                local positionTarget    = positionImageByLocationNo(indexAlign2)
                -- 创建Action序列
                local action = transition.sequence({
                    CCMoveTo:create(A_TIME1, positionTarget),   -- 向右移动
                })
                item:runAction(action)

                -- 排列索引加1
                indexAlign2 = indexAlign2 + 1
            end
        end
    end

    -- 是否可以操作
    function canOperated()
        return self:getParent():isRoundOperated()
    end

    -- 攻击敌方卡牌
    function self:fightCards(opponent)
        -- 过滤判断
        if TDArrayUtils.isEmpty(self.imagesFighting) then 
            -- 回调处理所有攻击结束后的逻辑
            callbackAllFightEnded()
        else
            -- 映射战斗序列
            mapImagesFightingSequence()
            -- 执行卡牌序列战斗操作
            fightSequence()
        end
    end
end





--[[--
    
    配置场景层容器视图

]]
function M:configWindow()

end

--[[--
    
    获得窗口内容大小

]]
function M:getWindowContentSize()
    -- 获得卡牌尺寸
    local cardSize = M:getCardSize()
    -- 计算窗口尺寸
    local winSize = CCSize( cardSize.width * NUM_VIEWED_CARDS + M_CARD.x * (NUM_VIEWED_CARDS - 1), cardSize.height + M_CARD.y )

    return winSize
end

-- 获得卡牌尺寸
function M:getCardSize()
    local template = (self.typeCard == TYPE_CARD_ICON) and TPL_CARD_ICON or TPL_CARD_IMAGE
    local tempFilename = string.gsub(template, "{0}", 1)
    local tempSprite = display.newSprite(tempFilename)
    local cardSize = tempSprite:getContentSize()

    return cardSize
end



return M