--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗卡片集层类，定义战斗卡片集层相关操作方法及逻辑实现。

-   存储战斗卡片集层相关信息。

]]

local M = class("FightCardSetLayer", TDLayer)



-- [包含文件]
-- ...



-- [公共参数]
-- [常量]
-- 位置[我方]
local P_SO_STAGEUP_ME               = ccp(5, 5)
-- 位置[敌方]
local P_SO_STAGEUP_OPPONENT         = ccp(5, 5)
-- 层深度[浮起]
local Z_CARD_FLOAD                  = 100
-- 层深度[正常]
local Z_CARD_NORMAL                 = 0
-- 层深度[墓地]
local Z_CEMETERY                    = 10


-- [操作变量]
-- ..


-- [枚举]
M.NODETAG__LOWERBOUND           = 500
M.NODETAG_CEMETERY              = M.NODETAG__LOWERBOUND + 10










-- [私有方法声明]
-- [实例方法][视图渲染]
-- ..
-- [视图析构]
-- ..

    


    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [合法性验证]
    self:assertParameters(params)



    -- [参数赋值]
    self._type                      = params.type                   or TYPE_CARD_UNKNOWN    -- 卡牌类型[场下卡片/场上卡片]
    self._posCard                   = params.posCard                or {}                   -- 卡牌PO集合
    self._owner                     = params.owner                                          -- 拥有者标识
    self._opponent                  = nil                                                   -- 对手
    self._cards                     = {}                                                    -- 卡牌集
    self._prototypes                = nil                                                   -- 卡牌集[原型]
    self._size                      = CS_ZERO                                               -- 卡牌集[尺寸]
    self._prototypeCemetery         = nil                                                   -- 卡牌[原型－墓地]
    self._cemetery                  = nil                                                   -- 卡牌[墓地]


    
    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendLayer.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][视图渲染]
-- 视图渲染
function M:viewRender()
    -- 加载状态变量
    self:loadStatuses()

    -- 加载卡牌背景
    self:loadBackgrounds()
    -- 加载卡牌集
    self:loadCards()

    -- 加载渲染视图其他操作
    self:loadRenderOtherwise()
end

-- 加载状态变量
function M:loadStatuses()
    -- 加载卡牌集[原型]
    self:loadPrototypes()
    -- 加载原型[墓地](从原型集中进行映射)
    self:loadPrototypeCemetery()
end

-- 加载背景
function M:loadBackgrounds()
    -- 加载卡牌背景[墓地]
    self:loadBackgroundCemetery()
end

-- 加载卡牌背景[墓地]
function M:loadBackgroundCemetery()
    -- 创建对象
    local sprite = Cemetery.new({ fileName = PHI_FIGHT .. "f_cemetery_bg.png" })        -- 实例化卡牌
    -- 添加到容器
    self:addChild(sprite, Z_CEMETERY, M.NODETAG_CEMETERY)

    -- 定位坐标
    sprite:setScale(.5)
    sprite:setPosition(self._prototypes:getPrototype(IDX_CEMETERY):getPosition())

    -- 存储对象
    self._cemetery = sprite
end

-- 加载原型
function M:loadPrototypes()
    -- 定义原型集
    local prototypes = {}

    -- 设置原型集
    if self._owner == OWNER_ME then 
        -- [我方卡牌]
        -- 校正偏移
        local correct = ccp(0, 0)
        -- 原型集
        prototypes[0]  = CardPrototype.new({ position = ccpAdd(ccp( 96,   4), correct), scale = 1.0, rotation = -12, })
        prototypes[1]  = CardPrototype.new({ position = ccpAdd(ccp(260,  34), correct), scale = 1.0, rotation = -8 , })
        prototypes[2]  = CardPrototype.new({ position = ccpAdd(ccp(380,  47), correct), scale = 1.0, rotation = -4 , })
        prototypes[3]  = CardPrototype.new({ position = ccpAdd(ccp(500,  52), correct), scale = 1.0, rotation =  0 , })
        prototypes[4]  = CardPrototype.new({ position = ccpAdd(ccp(620,  47), correct), scale = 1.0, rotation =  4 , })
        prototypes[5]  = CardPrototype.new({ position = ccpAdd(ccp(738,  34), correct), scale = 1.0, rotation =  8 , })
        prototypes[10] = CardPrototype.new({ position = ccpAdd(ccp(894,  67), correct), scale = 1.0, rotation =  0 , })
        prototypes[11] = CardPrototype.new({ position = ccpAdd(ccp(260, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[12] = CardPrototype.new({ position = ccpAdd(ccp(380, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[13] = CardPrototype.new({ position = ccpAdd(ccp(500, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[14] = CardPrototype.new({ position = ccpAdd(ccp(620, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[15] = CardPrototype.new({ position = ccpAdd(ccp(738, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[16] = CardPrototype.new({ position = ccpAdd(ccp(858, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[17] = CardPrototype.new({ position = ccpAdd(ccp(978, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[18] = CardPrototype.new({ position = ccpAdd(ccp(1098, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[19] = CardPrototype.new({ position = ccpAdd(ccp(1218, 215), correct), scale = 1.0, rotation =  0 , })
    else 
        -- [敌方卡牌]
        -- 校正偏移
        local correct = ccp(0, 0)
        -- 原型集
        prototypes[0]  = CardPrototype.new({ position = ccpAdd(ccp( 96, 635), correct), scale = 1.0, rotation =  12, })
        prototypes[1]  = CardPrototype.new({ position = ccpAdd(ccp(260, 604), correct), scale = 1.0, rotation =  8 , })
        prototypes[2]  = CardPrototype.new({ position = ccpAdd(ccp(380, 592), correct), scale = 1.0, rotation =  4 , })
        prototypes[3]  = CardPrototype.new({ position = ccpAdd(ccp(500, 587), correct), scale = 1.0, rotation =  0 , })
        prototypes[4]  = CardPrototype.new({ position = ccpAdd(ccp(620, 592), correct), scale = 1.0, rotation = -4 , })
        prototypes[5]  = CardPrototype.new({ position = ccpAdd(ccp(738, 604), correct), scale = 1.0, rotation = -8 , })
        prototypes[10] = CardPrototype.new({ position = ccpAdd(ccp(894, 571), correct), scale = 1.0, rotation =  0 , })
        prototypes[11] = CardPrototype.new({ position = ccpAdd(ccp(260, 425), correct), scale = 1.0, rotation =  0 , })
        prototypes[12] = CardPrototype.new({ position = ccpAdd(ccp(380, 425), correct), scale = 1.0, rotation =  0 , })
        prototypes[13] = CardPrototype.new({ position = ccpAdd(ccp(500, 425), correct), scale = 1.0, rotation =  0 , })
        prototypes[14] = CardPrototype.new({ position = ccpAdd(ccp(620, 425), correct), scale = 1.0, rotation =  0 , })
        prototypes[15] = CardPrototype.new({ position = ccpAdd(ccp(738, 425), correct), scale = 1.0, rotation =  0 , })
        prototypes[16] = CardPrototype.new({ position = ccpAdd(ccp(858, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[17] = CardPrototype.new({ position = ccpAdd(ccp(978, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[18] = CardPrototype.new({ position = ccpAdd(ccp(1098, 215), correct), scale = 1.0, rotation =  0 , })
        prototypes[19] = CardPrototype.new({ position = ccpAdd(ccp(1218, 215), correct), scale = 1.0, rotation =  0 , })
    end

    self._prototypes = CardPrototypeSet.new({ prototypes = prototypes })
end

-- 加载原型[墓地](从原型集中进行映射)
function M:loadPrototypeCemetery()
    -- 设置位置[墓地]
    self._prototypeCemetery = self._prototypes:getPrototype(IDX_CEMETERY)
end

-- 加载卡牌集
function M:loadCards() 
    for i = 1, #self._posCard do
        local sn     = self._posCard[i]:getSn()
        local params = {
            po                      = self._posCard[i],             -- PO对象
            owner                   = self._owner,                  -- 拥有者标识
            prototypes              = self._prototypes,             -- 原型集
            index                   = 0,                            -- 位置索引
            isPrototypeAutoCorrect  = true,                         -- 自动校正原型(如果自动校正，则设置_index时会自动将位置定位到指定索引位置)
            offsetShadow            = self:getOffsetShadow(),       -- 阴影偏移量
        }

        local card  = self:newCard(params)
        self:addChild(card, Z_CARD_NORMAL)
        self._cards[sn] = card
    end
end
    
-- 加载渲染视图其他操作
function M:loadRenderOtherwise()
    -- 设置卡片可见性
    self:setCardsVisiblitiy()
end

-- 设置卡牌可见性
function M:setCardsVisiblitiy()
    for _, card in pairs(self._cards) do
        card:setVisible(false)
    end
end


---------


-- [实例方法][视图析构]
-- 视图析构
function M:viewDestructor()

end


---------


-- [实例方法][外部调用]
-- 清除所有卡片状态[就绪]
function M:clearAllStateReady()
    for _, card in pairs(self._cards) do
        card:hideBoxReady()
    end
end

-- 重新排列卡牌顺序
function M:reorderCard(card)
    -- [Version1]
    --[[
    ]]
    for k in pairs(self._cards) do
        local thisCard = self._cards[k]
        if card == thisCard then
            self:reorderChild(thisCard, Z_CARD_FLOAD)
        else
            self:reorderChild(thisCard, Z_CARD_NORMAL)
        end
    end

    -- [Version2]
    --[[
    for k in pairs(self._cards) do
        local thisCard = self._cards[k]
        if card == thisCard then 
            thisCard:setZOrder(Z_CARD_FLOAD)
        else
            thisCard:setZOrder(Z_CARD_NORMAL)
        end
    end
    -- self:sortAllChildren()
    ]]
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    TDAssertUtils.assertParameters(params)
end


---------


-- [辅助方法]
-- 根据卡牌类型标识获得卡牌文件名[图标/图片](用于计算卡牌尺寸)
function M:getCardTempFileName()
    local fileName

    if self._type == TYPE_CARD_ICON then
        fileName = string.gsub(TPL_CARD_ICON, "{0}", 1)
    elseif self._type == TYPE_CARD_IMAGE then
        fileName = string.gsub(TPL_CARD_IMAGE, "{0}", 1)
    else
        TDExceptionUtils.raiseUnRunHereException()
    end

    return fileName
end

-- 根据拥有者标识获得阴影偏移量
function M:getOffsetShadow()
    local offset = P_ZERO

    if self._owner == OWNER_ME then 
        offset = P_SO_STAGEUP_ME
    else
        offset = P_SO_STAGEUP_OPPONENT
    end

    return offset
end

-- 获得卡牌尺寸
function M:getCardSize()
    return display.newSprite(self:getCardTempFileName()):getContentSize()
end

-- 实例化卡牌
function M:newCard(params)
    local card
    
    if self._type == TYPE_CARD_ICON then
        card = CardFactory.newStageDown(params)
    elseif self._type == TYPE_CARD_IMAGE then
        card = CardFactory.newStageUp(params)
    else
        TDExceptionUtils.raiseUnRunHereException()
    end

    return card
end

-- 动画发光墓地对象
function M:glitterCemetery()
    self._cemetery:animate()
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得卡牌类型[场下卡片/场上卡片]
function M:getType()
    return self._type
end
-- 设置卡牌类型[场下卡片/场上卡片]
function M:setType(owner)
    self._type = owner
end

-- 获得拥有者标识
function M:getOwner()
    return self._owner
end
-- 设置拥有者标识
function M:setOwner(owner)
    self._owner = owner
end

-- 获得对手
function M:getOpponent()
    return self._opponent
end
-- 设置对手
function M:setOpponent(opponent)
    self._opponent = opponent
end

-- 获得卡牌集
function M:getCards()
    return self._cards
end
-- 设置卡牌集
function M:setCards(cards)
    self._cards = cards
end

-- 获得卡牌集[原型]
function M:getPrototypes()
    return self._prototypes
end
-- 设置卡牌集[原型]
function M:setPrototypes(prototypes)
    self._prototypes = prototypes
end

-- 获得卡牌[尺寸]
function M:getSize()
    return self._size
end
-- 设置卡牌[尺寸]
function M:setSize(size)
    self._size = size
end

-- 获得卡牌[原型－墓地]
function M:getPrototypeCemetery()
    return self._prototypeCemetery
end
-- 设置卡牌[原型－墓地]
function M:setPrototypeCemetery(prototype)
    self._prototypeCemetery = prototype
end





return M