--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗界面英雄层类，定义战斗界面英雄层相关操作方法及逻辑实现。

-   存储英雄层相关信息。
-   实现英雄动画效果。

]]

local M = class("FightHerosLayer", TDLayer)



-- [包含文件]
-- ...



-- [公共参数]
-- [常量]
-- .. 


-- [操作变量]
-- ..


-- [枚举]
-- ..

    


    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [合法性验证]
    self:assertParameters(params)



    -- [参数赋值]
    self._mePo                      = params.mePo                       -- [我]PO
    self._mePositionHero            = params.mePositionHero             -- [我]英雄位置
    self._mePositionBlood           = params.mePositionBlood            -- [我]血量位置
    self._mePositionName            = params.mePositionName             -- [我]名字位置
    self._mePositionLevel           = params.mePositionLevel            -- [我]等级位置
    self._oppPo                     = params.oppPo                      -- [敌]PO
    self._oppPositionHero           = params.oppPositionHero            -- [敌]英雄位置
    self._oppPositionBlood          = params.oppPositionBlood           -- [敌]血量位置
    self._oppPositionName           = params.oppPositionName            -- [敌]名字位置
    self._oppPositionLevel          = params.oppPositionLevel           -- [敌]等级位置
    self._mode                      = params.mode                       -- 战斗模式


    
    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendLayer.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][视图渲染]
-- 视图渲染
function M:viewRender()
    -- 加载滑动栏
    self:loadSlider()
    -- 加载英雄
    self:loadHeros()
end

-- 加载滑动栏
function M:loadSlider()
    -- 加载
    local sprite = display.newSprite(PHI_FIGHT .. "f_hero_slider.png")
    self:addChild(sprite)

    -- 配置实体
    sprite:setPosition(0, 320)
    display.align(sprite, display.CENTER_LEFT)
end

-- 加载英雄
function M:loadHeros()
    -- 加载英雄[我方]
    self:loadHero({ po = self._mePo,  positionHero = self._mePositionHero,  positionBlood = self._mePositionBlood,  positionName = self._mePositionName,  positionLevel = self._mePositionLevel,  owner = OWNER_ME, })
    -- 加载英雄[敌方]
    self:loadHero({ po = self._oppPo, positionHero = self._oppPositionHero, positionBlood = self._oppPositionBlood, positionName = self._oppPositionName, positionLevel = self._oppPositionLevel, owner = OWNER_OPPONENT, })
end

-- 加载英雄[我方]
function M:loadHero(params)
    -- 加载血量
    self:loadHeroBlood(params)
    -- 加载卡牌
    self:loadHeroCard(params)
    -- 加载名字
    self:loadHeroName(params)
    -- 加载等级
    self:loadHeroLevel(params)
end

-- 加载血量
function M:loadHeroBlood(params)
    -- [参数赋值]
    local po                      = params.po
    local positionHero            = params.positionHero
    local positionBlood           = params.positionBlood
    local positionName            = params.positionName
    local positionLevel           = params.positionLevel
    local owner                   = params.owner


    -- [加载实体]
    local sprite = Blood.new({ poHero = po, fileName = PHI_FIGHT .. "f_blood_bg.png", owner = owner })
    self:addChild(sprite, 1)

    sprite:setPosition(positionBlood)
    -- sprite:setScale(.5)


    -- [存储属性]
    if owner == OWNER_ME then 
        self._spriteMeHeroBlood  = sprite
    else 
        self._spriteOppHeroBlood = sprite
    end
end

-- 加载卡牌
function M:loadHeroCard(params)
    -- [参数赋值]
    local po                      = params.po
    local positionHero            = params.positionHero
    local positionBlood           = params.positionBlood
    local positionName            = params.positionName
    local positionLevel           = params.positionLevel
    local owner                   = params.owner
    local mode                    = self._mode


    -- [加载实体]
    local sprite = CardFactory.newHero({ po = po, fileName = PHI_FIGHT .. "f_hero_box.png", owner = owner, mode = mode, })
    self:addChild(sprite)

    sprite:setPosition(positionHero)
    -- sprite:setScale(.5)


    -- [存储属性]
    if owner == OWNER_ME then 
        self._spriteMeHeroCard  = sprite
    else 
        self._spriteOppHeroCard = sprite
    end
end

-- 加载名字
function M:loadHeroName(params)
    -- [参数赋值]
    local po                      = params.po
    local positionHero            = params.positionHero
    local positionBlood           = params.positionBlood
    local positionName            = params.positionName
    local positionLevel           = params.positionLevel
    local owner                   = params.owner


    -- [加载实体]
    local sprite = HeroName.new({ text = po:getName(), fileName = PHI_FIGHT .. "f_heroname_box.png", owner = owner })
    self:addChild(sprite)

    sprite:setPosition(positionName)
    -- sprite:setScale(.5)


    -- [存储属性]
    if owner == OWNER_ME then 
        self._spriteMeHeroName  = sprite
    else 
        self._spriteOppHeroName = sprite
    end
end

-- 加载等级
function M:loadHeroLevel(params)
    -- [参数赋值]
    local po                      = params.po
    local positionHero            = params.positionHero
    local positionBlood           = params.positionBlood
    local positionName            = params.positionName
    local positionLevel           = params.positionLevel
    local owner                   = params.owner


    -- [加载实体]
    local sprite = HeroLevel.new({ text = po:getLv(), fileName = PHI_FIGHT .. "f_herolv_box.png", owner = owner })
    self:addChild(sprite)

    sprite:setPosition(positionLevel)
    -- sprite:setScale(.5)


    -- [存储属性]
    if owner == OWNER_ME then 
        self._spriteMeHeroName  = sprite
    else 
        self._spriteOppHeroName = sprite
    end
end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()

end


---------


-- [实例方法][视图析构]
-- 视图析构
function M:viewDestructor()

end


---------


-- [实例方法][外部调用]
-- 获得血量[我方]
function M:getBloodMe()
    return self:getBlood(OWNER_ME)
end

-- 获得血量[敌方]
function M:getBloodOpponent()
    return self:getBlood(OWNER_OPPONENT)
end

-- 获得血量
function M:getBlood(owner)
    local sprite

    if owner == OWNER_ME then 
        sprite = self._spriteMeHeroBlood
    else
        sprite = self._spriteOppHeroBlood
    end

    return sprite
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    TDAssertUtils.assertParameters(params)
end


---------


-- [实例方法][属性Getter/Setter]
-- 设置拥有者标识
function M:setOwner(owner)
    self._owner = owner
end
-- 获得拥有者标识
function M:getOwner()
    return self._owner
end



return M