--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

对战玩家场景层类，定义对战玩家场景层相关操作方法及逻辑实现。

-   存储卡牌相关信息。
-   提供卡牌与玩家数据之间交互的渠道。

]]

local M = class("FightPlayerLayer", TDLayer)



-- [包含文件]
-- ...



-- [操作变量]
-- 层[卡组]
local FightCardGroupLayer       = requireL("FightCardGroupLayer")

-- 实体[英雄]
local EntityHero                = requireEN("EntityHero")
-- 实体[卡牌]
local EntityCard                = requireEN("EntityCard")

-- 精灵[英雄血量]
local Blood                     = requireS("Blood")



-- [公共参数]
-- 常量
-- ...










-- [构造方法]
--[[--
    创建实例
]]
function M:ctor(params)
    -- 验证参数合法性
    -- TDAssertUtils.assertClassname(poCards, UserPo)

    -- 参数赋值
    local _poHero                   = params.poHero                                     -- 英雄PO
    local _posCard                  = params.posCard                or {}               -- 卡牌PO集合
    local _positionHero             = params.positionHero           or ccp(0, 0)        -- 用户位置
    local _positionBlood            = params.positionBlood          or ccp(0, 0)        -- 血量位置
    local _positionBeginStageDown   = params.positionBeginStageDown or ccp(0, 0)        -- 卡牌集合起始位置[图标]
    local _positionBeginStageUp     = params.positionBeginStageUp   or ccp(0, 0)        -- 卡牌集合起始位置[图片]
    local _owner                    = params.owner                                      -- 拥有者标识
    local _opponent                 = nil                                               -- 对手
    local _heroHp                   = nil                                               -- 英雄血量
    local _cardGroupLayer           = nil                                               -- 卡组层

    







    -- [辅助变量]

    







    -- [私有方法声明]
    -- [渲染视图]
    -- 视图渲染
    local viewRender
    -- 加载英雄
    local loadHero
    -- 加载英雄血量
    local loadHeroBlood
    -- 加载英雄卡牌
    local loadHeroCard
    -- 加载卡组层（卡组层中包含有“未上场卡牌”和“上场卡牌”组）
    local loadCardGroupLayer

    -- [视图析构]
    -- 视图析构
    local viewDestructor
    

    

    





    -- [私有方法定义]
    -- [渲染视图]
    -- 视图渲染
    function viewRender()
        -- 加载英雄
        -- loadHero()
        -- 加载卡组层（卡组层中包含有“未上场卡牌”和“上场卡牌”组）
        loadCardGroupLayer()
    end 

    -- 加载英雄
    function loadHero()
        -- 加载英雄血量
        loadHeroBlood()
        -- 加载英雄卡牌
        loadHeroCard()
    end

    -- 加载英雄血量
    function loadHeroBlood()
        local sprite = Blood.new({ poHero = _poHero, fileName = PHI_FIGHT .. "f_blood_bg.png", owner = _owner })
        self:addChild(sprite)

        sprite:setPosition(_positionBlood)
        sprite:setScale(.5)

        -- 存储属性
        _heroHp = sprite
    end

    -- 加载英雄卡牌
    function loadHeroCard()
        local sprite = CardFactory.newHero({ po = _poHero, owner = _owner })
        self:addChild(sprite)

        display.align(sprite, display.LEFT_BOTTOM, _positionHero.x, _positionHero.y)
        sprite:setScale(.5)
    end

    -- 加载卡组层（卡组层中包含有“未上场卡牌”和“上场卡牌”组）
    function loadCardGroupLayer()
        -- 定义参数
        local params = {
            owner                   = _owner,
            posCard                 = _posCard,
            positionBeginStageDown  = _positionBeginStageDown,
            positionBeginStageUp    = _positionBeginStageUp,
        }

        -- 实例化对象
        local layer = FightCardGroupLayer.new(params)
        -- 添加对象
        self:addChild(layer)

        -- 存储属性
        _cardGroupLayer = layer
    end


    ---------


    -- [视图析构]
    -- 视图析构
    function viewDestructor()

    end


    ---------


    -- [全局访问]
    -- 获得英雄血量
    function self:getHeroHp()
        return _heroHp
    end

    -- 获得英雄血量值
    function self:getHeroHpValue()
        return self:getHeroHp():getValue()
    end
    -- 设置英雄血量值
    function self:setHeroHpValue(value)
        return self:getHeroHp():setValue(value)
    end

    -- 获得对手
    function self:getOpponent()
        return _opponent
    end
    -- 设置对手
    function self:setOpponent(player)
        _opponent = player
    end

    -- 获得卡牌集
    function self:getCards()
        return _cardGroupLayer:getCards()
    end
    -- 设置位置集
    function self:getPositions()
        return _cardGroupLayer:getPositions()
    end











    -- 攻击敌方卡牌
    function self:fightRound(opponent)
        _opponent = opponent or self:getOpponent()
        _cardGroupLayer:fightRound(opponent)
    end

    -- 获得战斗卡牌集
    function self:cardImagesFighting()
        return _cardGroupLayer:cardImagesFighting()
    end


    -- 生成卡牌
    function self:spawnCard()
        _cardGroupLayer:spawnCard()
    end

    -- 递减已上场卡牌的等待回合数
    function self:decreaseWaitRounds()
        _cardGroupLayer:decreaseWaitRounds()
    end

    -- 自动操作
    function self:autoOperate()
        _cardGroupLayer:autoOperate()
    end

    -- 自动战斗
    function self:autoFight()
        _cardGroupLayer:autoFight()
    end

    -- 判断玩家卡牌是否处于等待上场状态
    function self:isCardsWaitingUpStage()
        return _cardGroupLayer:isCardsWaitingUpStage()
    end






    -- [具体操作]
    viewRender()
end



return M