--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

提示场景层类，定义提示场景层相关操作方法及逻辑实现。

-   显示提示详细信息。

]]

local M = class("FightResultLayer", TDLayer)



-- [包含文件]
-- ...




-- [公共参数]
-- 操作变量
-- 断言
local TDAssertUtils 		= require(PH_MDL_UTIL .. "TDAssertUtils")
local TDExceptionUtils 		= require(PH_MDL_UTIL .. "TDExceptionUtils")
local TDPointUtils 			= require(PH_MDL_UTIL .. "TDPointUtils")
local TDTouchUtils 			= require(PH_MDL_UTIL .. "TDTouchUtils")
local acp 					= TDPointUtils.point

-- 常量
local C_BACKGROUND 			= ccc4(0, 0, 0, 180)
local M__ITEM		 		= acp(0, -30)
local M_LABEL_VALUE		 	= acp(10, 0)
local FZ_LABEL_TITLE 		= (28 * display.contentScaleFactor)
local FZ_LABEL_GOLD 		= (20 * display.contentScaleFactor)
local FZ_LABEL_EXPERIENCE 	= (20 * display.contentScaleFactor)
local P_LABEL_TITLE 		= acp(display.cx, 500)
local P_LABEL_GOLD 			= acp(360, 420)
local P_LABEL_EXPERIENCE	= acp(360, 360)
local P_LABEL_VALUE 		= acp(460, -1)
local O_TIP_TOUCH_FOR_CLOSE = acp(0, 20)

-- 局部变量
-- ..
-- 枚举
-- ..










-- [构造方法]
--[[--
    创建实例
]]
function M:ctor(params)
    -- [合法性验证]
    self:assertParameters(params)



    -- [参数赋值]
    self._title       			= params.title 							-- 标题
    self._gold       			= params.gold 							-- 金币
    self._experience       		= params.experience 					-- 经验值
    self._viewToJump       		= params.viewToJump or nil 				-- 跳转视图


    
    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendLayer.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][视图渲染]
-- 视图渲染
function M:viewRender()
	-- 加载背景
	self:loadBackground()

	-- 加载标签
	self:loadLabels()
	-- 加载点击提示
	self:loadTipTouchForClose()

	-- 加载渲染以外的其他操作
	self:loadRenderOtherwise()
end

-- 加载背景
function M:loadBackground()
	local sprite = widget.newColorLayer(C_BACKGROUND)
	self:addChild(sprite)
end

-- 加载标签
function M:loadLabels()
	-- 加载标签[标题]
	self:loadLabelTitle()
	-- 加载标签[金币]
	self:loadLabelGold()
	-- 加载标签[经验值]
	self:loadLabelExperience()
end
-- 加载标签[标题]
function M:loadLabelTitle()
    local sprite = ui.newTTFLabelWithShadow({
        text = self._title,
        size = FZ_LABEL_TITLE,
    });
    self:addChild(sprite)

    -- 定位
    sprite:setPosition(P_LABEL_TITLE)
end
-- 加载标签[金币]
function M:loadLabelGold()
	-- 提示语
    local label = ui.newTTFLabelWithShadow({
        text = "经验值: ",
        size = FZ_LABEL_GOLD,
    })
    self:addChild(label)
    -- 定位
    label:setPosition(P_LABEL_GOLD)



    -- 数值
    local value = ui.newTTFLabelWithShadow({
        text = tostring(self._gold),
        size = FZ_LABEL_GOLD,
    })
    self:addChild(value)
    -- 定位
    value:setPosition(ccp(P_LABEL_VALUE.x, P_LABEL_GOLD.y + M_LABEL_VALUE.y))
end
-- 加载标签[经验]
function M:loadLabelExperience()
	-- 提示语
    local label = ui.newTTFLabelWithShadow({
        text = "获得金币: ",
        size = FZ_LABEL_EXPERIENCE,
    })
    -- 定位
    label:setPosition(P_LABEL_EXPERIENCE)
    self:addChild(label)



    -- 数值
    local value = ui.newTTFLabelWithShadow({
        text = tostring(self._experience),
        size = FZ_LABEL_EXPERIENCE,
    })
    self:addChild(value)
    -- 定位
    value:setPosition(ccp(P_LABEL_VALUE.x, P_LABEL_EXPERIENCE.y + M_LABEL_VALUE.y))
end

-- 加载点击提示
function M:loadTipTouchForClose()
    local sprite = ui.newTTFLabelWithShadow({
        text = "请点击屏幕－关闭",
        size = FZ_TIP,
    })
    self:addChild(sprite)

    -- 定位
    util.alignNode(sprite, self:getContentSize(), display.BOTTOM_CENTER)
    sprite:setPosition(sprite:getPositionX() + O_TIP_TOUCH_FOR_CLOSE.x, sprite:getPositionY() + O_TIP_TOUCH_FOR_CLOSE.y)
end

-- 加载渲染以外的其他操作
function M:loadRenderOtherwise()
	-- 绑定点击事件侦听器
	self:bindTouchListener()
end

-- 绑定点击事件侦听器
function M:bindTouchListener()
	TDTouchUtils.bindLayerTouchEvent({ layer = self, priority = TDTouchUtils.PRIORITY_MASK, handler = function()
		-- 清理自身
		self:removeSelf()
		-- 视图跳转
		-- print("跳转视图")
	end})
end


---------


-- [实例方法][视图析构]
-- 视图析构
function M:viewDestructor()

end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    TDAssertUtils.assertParameters(params)
end




return M