--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗界面欢迎层类，定义战斗界面欢迎层相关操作方法及逻辑实现。

-   存储欢迎层相关信息。
-   实现欢迎动画效果。

]]

local M = class("FightWelcomeLayer", TDLayer)



-- [包含文件]
-- ..










-- [公共参数]
-- [常量]
-- 位置[V]
M.O_LABEL_V                     = ccp(-30, 0)
-- 位置[S]
M.O_LABEL_S                     = ccp(50, 0)
-- 触碰优先级
M.TOUCH_PRIORITY                = -100000


-- [操作变量]
-- ..


-- [枚举]
-- ..

    


    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [合法性验证]
    self:assertParameters(params)



    -- [参数赋值]
    self._mePo                      = params.mePo                                   -- [我]PO
    self._mePositionStart           = params.mePositionStart                        -- [我]英雄位置[起始]
    self._mePositionEnd             = params.mePositionEnd                          -- [我]英雄位置[结束]
    self._oppPo                     = params.oppPo                                  -- [敌]PO
    self._oppPositionStart          = params.oppPositionStart                       -- [敌]英雄位置[起始]
    self._oppPositionEnd            = params.oppPositionEnd                         -- [敌]英雄位置[结束]
    self._touchPriority             = params.touchPriority  or M.TOUCH_PRIORITY     -- 触碰优先级
    self._delegate                  = params.delegate                               -- 代理对象


    
    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendLayer.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][视图渲染]
-- 视图渲染
function M:viewRender()
    -- 加载遮罩
    self:loadMask()
    -- 加载英雄
    self:loadHeros()
    -- 加载标签
    self:loadLabels()

    -- 加载渲染以外其他操作
    self:loadRenderOtherwise()
end

-- 加载遮罩
function M:loadMask()
    -- [加载实体]
    local sprite = CCLayerColor:create(ccc4(0, 0, 0, 180))
    self:addChild(sprite)


    -- [存储属性]
    self._layerMask = sprite
end

-- 加载英雄
function M:loadHeros()
    -- 加载英雄[我方]
    self:loadHero({ po = self._mePo,  position = self._mePositionStart,  owner = OWNER_ME, })
    -- 加载英雄[敌方]
    self:loadHero({ po = self._oppPo, position = self._oppPositionStart, owner = OWNER_OPPONENT, })
end

-- 加载英雄[我方]
function M:loadHero(params)
    -- [参数赋值]
    local po                      = params.po
    local position                = params.position
    local owner                   = params.owner


    -- [加载实体]
    local sprite = WelcomeHero.new({ po = po, fileName = PHI_FIGHT .. "f_wlc_board_avatar.png", owner = owner })
    self:addChild(sprite)

    sprite:setPosition(position)


    -- [存储属性]
    if owner == OWNER_ME then 
        self._spriteMe  = sprite
    else 
        self._spriteOpp = sprite
    end
end

-- 加载标签
function M:loadLabels()
    -- 加载标签[V]
    self:loadLabelV()
    -- 加载标签[S]
    self:loadLabelS()
end

-- 加载标签[V]
function M:loadLabelV()
    -- [构建实体]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT .. "f_wlc_lbl_v.png",                    -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        offset      = M.O_LABEL_V,                                      -- 偏移量
    }
    -- 生成精灵
    local sprite = TDUIUtils.loadSprite(params)

    -- 存储状态
    self._labelV = sprite
end

-- 加载标签[S]
function M:loadLabelS()
    -- [构建实体]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT .. "f_wlc_lbl_s.png",                    -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        offset      = M.O_LABEL_S,                                      -- 偏移量
    }
    -- 生成精灵
    local sprite = TDUIUtils.loadSprite(params)

    -- 存储状态
    self._labelS = sprite
end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()
    -- 绑定点击事件
    self:bindTouchEvent()
    -- 动画显示
    self:animate()
end

-- 绑定点击事件
function M:bindTouchEvent()
    -- 绑定点击事件
    widget.bindTouchEvent{
        layer           = self,
        clickListener   = nil,
        priority        = self._touchPriority,
        swallowsTouches = true,
    }
end


---------


-- [实例方法][视图析构]
-- 视图析构
function M:viewDestructor()

end


---------


-- [实例方法][动画]
-- 动画显示
function M:animate()
    -- 我方英雄
    if true then 
        -- [参数定义]
        local node          = self._spriteMe
        local time1         = .1
        local delay         = 0
        local opacity0      = 50
        local opacity1      = 255
        local scale0        = 3.0
        local scale1        = node:getScale()
        local rate          = .3


        -- [动作定义]
        local array     = {}
        -- 初始设置
        node:setOpacity(opacity0)
        node:setScale(scale0)
        -- 延迟
        if delay ~= 0 then 
            array[#array + 1] = CCDelayTime:create(delay)
        end
        -- 缩放比例＋透明度
        array[#array + 1] = CCEaseIn:create(TDAnimationUtils.spawn({
            CCScaleTo:create(time1, scale1),
            CCFadeTo:create(time1, opacity1),
            CCCallFunc:create(function()
                -- 获得头像
                local avatar = node:getAvatar()
                -- 动画头像[摇晃]
                local time    = .1
                local rotate1 = 10
                local rotate2 = -rotate1
                local action = TDAnimationUtils.sequence({
                    CCRotateTo:create(time, rotate1),
                    CCRotateTo:create(time, 0),
                    CCRotateTo:create(time, rotate2),
                    CCRotateTo:create(time, 0),
                })
                avatar:runAction(action)
            end),
        }), rate)

        node:runAction(TDAnimationUtils.sequence(array))
    end

    -- 他方英雄
    if true then 
        -- [参数定义]
        local node          = self._spriteOpp
        local time1         = .1
        local delay         = .1
        local opacity0      = 50
        local opacity1      = 255
        local scale0        = 3.0
        local scale1        = node:getScale()
        local rate          = .3


        -- [动作定义]
        local array     = {}
        -- 初始设置
        node:setOpacity(opacity0)
        node:setScale(scale0)
        -- 延迟
        if delay ~= 0 then 
            array[#array + 1] = CCDelayTime:create(delay)
        end
        -- 缩放比例＋透明度
        array[#array + 1] = CCEaseIn:create(TDAnimationUtils.spawn({
            CCScaleTo:create(time1, scale1),
            CCFadeTo:create(time1, opacity1),
            CCCallFunc:create(function()
                -- 获得头像
                local avatar = node:getAvatar()
                -- 动画头像[摇晃]
                local time    = .1
                local rotate1 = 10
                local rotate2 = -rotate1
                local action = TDAnimationUtils.sequence({
                    CCRotateTo:create(time, rotate1),
                    CCRotateTo:create(time, 0),
                    CCRotateTo:create(time, rotate2),
                    CCRotateTo:create(time, 0),
                })
                avatar:runAction(action)
            end),
        }), rate)

        node:runAction(TDAnimationUtils.sequence(array))
    end

    -- 标签[V]
    if true then 
        -- [参数定义]
        local node          = self._labelV
        local time1         = .1
        local delay         = .3
        local scale0        = 4.0
        local scale1        = node:getScale()
        local rate          = .3
        local texture0      = self:textureVBlur()
        local texture1      = self:textureV()


        -- [动作定义]
        local array     = {}
        -- 初始设置
        node:setScale(scale0)
        node:setTexture(texture0)
        -- 隐藏
        array[#array + 1] = CCHide:create()
        -- 延迟
        if delay ~= 0 then 
            array[#array + 1] = CCDelayTime:create(delay)
        end
        -- 显示
        array[#array + 1] = CCShow:create()
        -- 缩放比例＋透明度
        array[#array + 1] = TDAnimationUtils.spawn({
            CCEaseBackOut:create(CCScaleTo:create(time1, scale1)),
            CCCallFunc:create(function()
                node:setTexture(texture1)
            end),
        })

        node:runAction(TDAnimationUtils.sequence(array))
    end

    -- 标签[S]
    if true then 
        -- [参数定义]
        local node          = self._labelS
        local time1         = .1
        local delay         = .4
        local scale0        = 4.0
        local scale1        = node:getScale()
        local rate          = .3
        local texture0      = self:textureSBlur()
        local texture1      = self:textureS()


        -- [动作定义]
        local array     = {}
        -- 初始设置
        node:setScale(scale0)
        node:setTexture(texture0)
        -- 隐藏
        array[#array + 1] = CCHide:create()
        -- 延迟
        if delay ~= 0 then 
            array[#array + 1] = CCDelayTime:create(delay)
        end
        -- 显示
        array[#array + 1] = CCShow:create()
        -- 缩放比例＋透明度
        array[#array + 1] = TDAnimationUtils.spawn({
            CCEaseBackOut:create(CCScaleTo:create(time1, scale1)),
            CCCallFunc:create(function()
                node:setTexture(texture1)
                self:animateJumpToTarget()
            end),
        })

        node:runAction(TDAnimationUtils.sequence(array))
    end
end

-- 动画显示[跳至目标位置]
function M:animateJumpToTarget()
    -- 公共常量
    local DELAY_VS          = .7

    -- V
    if true then 
        -- [操作结点]
        local node          = self._labelV



        -- [参数定义]
        local delay         = DELAY_VS
        local time1         = .8
        local position0     = ccp(node:getPosition())
        local position1     = ccp(position0.x, 960 + 100)        -- ccp(position0.x, display.height + 100)
        local rate          = .3
        local array         = {}

        -- 初始设置
        -- ..
        -- 延迟
        if delay ~= 0 then 
            array[#array + 1] = CCDelayTime:create(delay)
        end
        -- 移动
        array[#array + 1] = CCEaseBackOut:create(CCMoveTo:create(time1, position1))

        node:runAction(TDAnimationUtils.sequence(array))
    end
    -- S
    if true then 
        -- [操作结点]
        local node          = self._labelS



        -- [参数定义]
        local delay         = DELAY_VS
        local time1         = .8
        local position0     = ccp(node:getPosition())
        local position1     = ccp(position0.x, 0 - 100)
        local rate          = .3
        local array         = {}

        -- 初始设置
        -- ..
        -- 延迟
        if delay ~= 0 then 
            array[#array + 1] = CCDelayTime:create(delay)
        end
        -- 移动
        array[#array + 1] = CCEaseBackOut:create(CCMoveTo:create(time1, position1))

        node:runAction(TDAnimationUtils.sequence(array))
    end

    -- 我方英雄
    if true then 
        -- [状态常量]
        local SCALE_AVATAR  = .5
        -- [操作结点]
        local node          = self._spriteMe
        -- [创建头像副本]
        local sprite        = display.newSprite(node:getFrameNameAvatar())       -- node:getPo():getHead()
        self:addChild(sprite)
        sprite:setPosition(ccp(node:getPositionX() + node:getAvatarOffset().x, node:getPositionY() + node:getAvatarOffset().y))
        -- sprite:setScale(SCALE_AVATAR)
        -- [存储变量]
        self._spriteMeAvatarCopied = sprite



        -- [参数定义]
        local avatar        = sprite
        local time1         = .2
        local time2         = .3
        local time3         = .5
        local delay         = .2
        local tint1         = ccc3(110, 110, 110)
        local scaleNode     = node:getScale()
        local scale1        = scaleNode * .8
        local scale1A       = avatar:getScale() * 1.5
        local scale3A       = avatar:getScale() * (1 / .8)
        local position3     = self._mePositionEnd
        local jumpHeight3   = 30
        local jumptTimes3   = 1
        local rate          = .3



        -- [动作定义]
        -- 整体
        if true then 
            local target    = node
            local array     = {}
            -- 初始设置
            -- ..
            -- 缩放比例＋透明度
            array[#array + 1] = CCEaseOut:create(TDAnimationUtils.spawn({
                CCScaleTo:create(time1, scale1),
                CCTintTo:create(time1, tint1.r, tint1.g, tint1.b),
            }), rate)
            -- 延迟
            if delay ~= 0 then 
                array[#array + 1] = CCDelayTime:create(delay)
            end
            -- 渐隐
            array[#array + 1] = CCCallFunc:create(function() 
                node:getAvatar():setVisible(false)
                node:fadeOutWithChildren({ time = time3, opacity = 0, delay = 0 })
            end)

            target:runAction(TDAnimationUtils.sequence(array))
        end

        -- 头像
        if true then 
            local target    = avatar
            local array     = {}
            -- 初始设置
            -- ..
            -- 缩放比例＋透明度
            array[#array + 1] = CCEaseOut:create(TDAnimationUtils.spawn({
                CCScaleTo:create(time1, scale1A),
                -- TDAnimationUtils.sequence({
                --     CCTintTo:create(time1 / 2, 255, 255, 255),
                --     CCTintTo:create(time1 / 2, 0, 0, 0),
                -- }),
            }), rate)
            -- 延迟
            if delay ~= 0 then 
                array[#array + 1] = CCDelayTime:create(delay)
            end
            -- 跳跃
            array[#array + 1] = TDAnimationUtils.spawn({
                CCJumpTo:create(time3, position3, jumpHeight3, jumptTimes3),
                CCScaleTo:create(time3, scale3A),
            })

            target:runAction(TDAnimationUtils.sequence(array))
        end
    end


    -- 他方英雄
    if true then 
        -- [状态常量]
        local SCALE_AVATAR  = .5
        -- [操作结点]
        local node          = self._spriteOpp
        -- [创建头像副本]
        local sprite        = display.newSprite(node:getFrameNameAvatar())       -- node:getPo():getHead()
        self:addChild(sprite)
        sprite:setPosition(ccp(node:getPositionX() + node:getAvatarOffset().x, node:getPositionY() + node:getAvatarOffset().y))
        -- sprite:setScale(SCALE_AVATAR)
        -- [存储变量]
        self._spriteOppAvatarCopied = sprite



        -- [参数定义]
        local avatar        = sprite
        local time1         = .2
        local time2         = .3
        local time3         = .5
        local delay         = .2
        local tint1         = ccc3(110, 110, 110)
        local scaleNode     = node:getScale()
        local scale1        = scaleNode * .8
        local scale1A       = avatar:getScale() * 1.5
        local scale3A       = avatar:getScale() * (1 / .8)
        local position3     = self._oppPositionEnd
        local jumpHeight3   = 30
        local jumptTimes3   = 1
        local rate          = .3



        -- [动作定义]
        -- 整体
        if true then 
            local target    = node
            local array     = {}
            -- 初始设置
            -- ..
            -- 缩放比例＋透明度
            array[#array + 1] = CCEaseOut:create(TDAnimationUtils.spawn({
                CCScaleTo:create(time1, scale1),
                CCTintTo:create(time1, tint1.r, tint1.g, tint1.b),
            }), rate)
            -- 延迟
            if delay ~= 0 then 
                array[#array + 1] = CCDelayTime:create(delay)
            end
            -- 渐隐
            array[#array + 1] = CCCallFunc:create(function() 
                node:getAvatar():setVisible(false)
                node:fadeOutWithChildren({ time = time3, opacity = 0, delay = 0 })
            end)

            target:runAction(TDAnimationUtils.sequence(array))
        end

        -- 头像
        if true then 
            local target    = avatar
            local array     = {}
            -- 初始设置
            -- ..
            -- 缩放比例＋透明度
            array[#array + 1] = CCEaseOut:create(TDAnimationUtils.spawn({
                CCScaleTo:create(time1, scale1A),
                -- TDAnimationUtils.sequence({
                --     CCTintTo:create(time1 / 2, 255, 255, 255),
                --     CCTintTo:create(time1 / 2, 0, 0, 0),
                -- }),
            }), rate)
            -- 延迟
            if delay ~= 0 then 
                array[#array + 1] = CCDelayTime:create(delay)
            end
            -- 跳跃
            array[#array + 1] = TDAnimationUtils.spawn({
                CCJumpTo:create(time3, position3, jumpHeight3, jumptTimes3),
                CCScaleTo:create(time3, scale3A),

                -- FIXME: 终止
                CCCallFunc:create(function()
                    self:animateEnding()
                end),
            })

            target:runAction(TDAnimationUtils.sequence(array))
        end
    end
end

-- 动画显示[结束]
function M:animateEnding()
    local TIME = 1.6

    -- 我方英雄
    if true then 
        -- [参数定义]
        local node          = self._spriteMeAvatarCopied
        local time          = TIME
        local rate          = .3


        -- [动作定义]
        local array     = {}
        -- 缩放比例＋透明度
        array[#array + 1] = CCEaseIn:create(TDAnimationUtils.spawn({
            CCFadeOut:create(time),
        }), rate)

        node:runAction(TDAnimationUtils.sequence(array))
    end

    -- 他方英雄
    if true then 
        -- [参数定义]
        local node          = self._spriteOppAvatarCopied
        local time          = TIME
        local rate          = .3


        -- [动作定义]
        local array     = {}
        -- 缩放比例＋透明度
        array[#array + 1] = CCEaseIn:create(TDAnimationUtils.spawn({
            CCFadeOut:create(time),
        }), rate)

        node:runAction(TDAnimationUtils.sequence(array))
    end

    -- 背景
    if true then 
        -- [参数定义]
        local node          = self._layerMask
        local time          = TIME
        local rate          = .3


        -- [动作定义]
        local array     = {}
        -- 缩放比例＋透明度
        array[#array + 1] = CCEaseIn:create(TDAnimationUtils.spawn({
            CCFadeOut:create(time),
        }), rate)
        -- FIXME:
        -- 回调函数
        array[#array + 1] = 
            CCCallFunc:create(function()
                self._delegate:loadStep2()
                self:removeFromParentAndCleanup(true)
            end)
            

        node:runAction(TDAnimationUtils.sequence(array))
    end
end


---------


-- [实例方法][内部调用]
-- 获得标签V图片[正常]
function M:textureV()
    return CCTextureCache:sharedTextureCache():addImage(PHI_FIGHT .. "f_wlc_lbl_v.png")
end
-- 获得标签V图片[模糊]
function M:textureVBlur()
    return CCTextureCache:sharedTextureCache():addImage(PHI_FIGHT .. "f_wlc_lbl_v_blur.png")
end

-- 获得标签V图片[正常]
function M:textureS()
    return CCTextureCache:sharedTextureCache():addImage(PHI_FIGHT .. "f_wlc_lbl_s.png")
end
-- 获得标签V图片[模糊]
function M:textureSBlur()
    return CCTextureCache:sharedTextureCache():addImage(PHI_FIGHT .. "f_wlc_lbl_s_blur.png")
end


---------


-- [实例方法][外部调用]
-- ..


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    TDAssertUtils.assertParameters(params)
end


---------


-- [实例方法][属性Getter/Setter]
-- ..



return M