--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗场景类，定义战斗场景相关操作方法及逻辑实现。

-   存储用户相关信息。

]]

local M = class("FightScene", TDScene)



-- [包含文件]
require(PH_MDL_CONSTANT .. "Include")



-- [操作变量]
-- 层[测试－窗口视图]
local DemoViewWindowLayer   = requireL("DemoViewWindowLayer")
-- 层[测试－特效]
local DemoEffectLayer       = requireL("DemoEffectLayer")



-- [公共参数]
-- [常量]
local P_ME_WELCOME                  = ccp(480 - 200, 320)     -- ccp(display.cx - 200, display.cy)
local P_OPP_WELCOME                 = ccp(480 + 200, 320)     -- ccp(display.cx + 200, display.cy)
local P_ME_HERO                     = ccp(91, 101)
local P_OPP_HERO                    = ccp(P_ME_HERO.x, 541)
local P_ME_BLOOD                    = ccp(94, 236)
local P_OPP_BLOOD                   = ccp(P_ME_BLOOD.x, 405)
local P_ME_NAME                     = ccp(91, 24.4)
local P_OPP_NAME                    = ccp(P_ME_NAME.x, 617)
local P_ME_LEVEL                    = ccp(140, 151)
local P_OPP_LEVEL                   = ccp(P_ME_LEVEL.x, 487)
local P_B_STAGEDOWN_ME              = ccp(200, 80)
local P_B_STAGEUP_ME                = ccp(200, 230)
local P_SO_STAGEDOWN_ME             = ccp(40, 40)
local P_B_STAGEDOWN_OPPONENT        = ccp(200, 560)
local P_B_STAGEUP_OPPONENT          = ccp(200, 410)
local P_SO_STAGEUP_OPPONENT         = ccp(40, -40)

-- [枚举]
-- 结点枚举
M.NODETAG__LOWERBOUND               = 100
M.NODETAG_LAYER_MAIN                = M.NODETAG__LOWERBOUND + 10
M.NODETAG_LAYER_BUTTON              = M.NODETAG__LOWERBOUND + 20
M.NODETAG_LAYER_HEROS               = M.NODETAG__LOWERBOUND + 30
M.NODETAG_LAYER_M_CARDS_U           = M.NODETAG__LOWERBOUND + 40
M.NODETAG_LAYER_M_CARDS_D           = M.NODETAG__LOWERBOUND + 50
M.NODETAG_LAYER_O_CARDS_U           = M.NODETAG__LOWERBOUND + 60
M.NODETAG_LAYER_O_CARDS_D           = M.NODETAG__LOWERBOUND + 70
M.NODETAG_LAYER_SPEED               = M.NODETAG__LOWERBOUND + 80
M.NODETAG_LAYER_MASK                = M.NODETAG__LOWERBOUND + 90
M.NODETAG_LAYER_TIP                 = M.NODETAG__LOWERBOUND + 100
-- 层深度枚举
M.DEPTH_LAYER_BACKGROUND            = 0
M.DEPTH_LAYER_MAIN                  = 100
M.DEPTH_LAYER_BUTTON                = 10
M.DEPTH_LAYER_HEROS                 = 30
M.DEPTH_LAYER_UP                    = 40
M.DEPTH_LAYER_DOWN                  = 20
M.DEPTH_LAYER_SPEED                 = 666
M.DEPTH_LAYER_WELCOME               = 777
M.DEPTH_LAYER_MASK                  = 788
M.DEPTH_LAYER_DOOR                  = 799
M.DEPTH_LAYER_CARD_DETAIL           = 888
M.DEPTH_LAYER_RESULT                = 999
M.DEPTH_LAYER_TIP                   = 1000
-- 上下枚举
M.UD_UP                             = 1
M.UD_DOWN                           = 2
-- 添加移除枚举
M.AR_ADD                            = 1
M.AR_REMOVE                         = 2
-- 获取枚举
M.GS_GET                            = 1
M.GS_SET                            = 2









-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [合法性验证]
    self:assertParameters(params)



    -- [参数赋值]
    self._friendId                  = params.friendId       or 0                    -- 好友ID
    self._tid                       = params.tid            or 0                    -- 队伍ID
    self._serid                     = params.serid          or 0                    -- 服务器ID
    self._typ                       = params.typ            or 0                    -- 类型 1 金币 2 经验卡。
    self._no                        = params.no             or 0                    -- 编号
    self._friendName                = params.friendName     or ""                   -- 好友名称
    self._friendPoint               = params.friendPoint    or 0                    -- 友情点
    self._levelId                   = params.levelId        or 0                    -- 关卡ID
    self._difficulty                = params.difficulty     or 0                    -- 关卡难度
    self._difficultyMax             = params.difficultyMax  or 0                    -- 关卡难度[最大]
    self._recordId                  = params.recordId       or 0                    -- 记录ID（当mode为MODE_BATTLE_RECORD时有效）
    self._isBreak                   = params.isBreak        or false                -- 是否是最新关卡（如果胜利则展开新关卡）
    self._mode                      = params.mode           or { battle = MODE_BATTLE_LEVEL, pk = MODE_PK_LEARN }    -- 战斗模式
    self._fnNextView                = params.fnNextView     or nil                  -- 下一跳转视图方法
    self._isAuto                    = params.isAuto         or false                -- 自动战斗标识 
    self._againDATA                 = params.data           or nil                      -- 再来一次参数


    -- [操作变量]
    self._data                      = nil                                           -- 网络交互的数据对象
    self._dataFollow                = nil                                           -- 网络交互的数据对象[后续.说明: 先解析_data，若存在_dataFollow数据，在执行完_data后继而解析_dataFollow，此_dataFollow必为自动战斗数据]
    self._poHeroMe                  = nil                                           -- 英雄信息PO
    self._poHeroOpponent            = nil                                           -- 英雄信息PO
    self._posCardMe                 = nil                                           -- 英雄信息PO
    self._posCardOpponent           = nil                                           -- 卡牌信息PO
    self._talentMe                  = nil                                           -- 天赋信息
    self._talentOpponent            = nil                                           -- 天赋信息
    self._equipEffectMe             = nil                                           -- 装备信息
    self._equipEffectOpponent       = nil                                           -- 装备信息
    self._isGameEnded               = false                                         -- 是否游戏结束
    self._round                     = 1                                             -- 当前回合数
    self._cardsUpMe                 = {}                                            -- 卡片集[我方][场上]
    self._cardsDownMe               = {}                                            -- 卡片集[我方][场下]
    self._cardsUpOpp                = {}                                            -- 卡片集[敌方][场上]
    self._cardsDownOpp              = {}                                            -- 卡片集[敌方][场下]
    self._cardsNet                  = {}                                            -- 卡片集[网络]
    self._statePlayer               = nil                                           -- 状态[玩家]
    self._runningAction             = nil                                           -- 动作[进行中..]
    self._isInteractive             = false                                         -- 交互状态[战斗进行时状态为DONE，战斗等待玩家进行交互时状态为DOING]
    self._indexBase                 = 0
    self._isHelpGuide               = false                                         -- 是否正在进行新手引导
    self._DEBUG                     = false                                         -- 调试模式
    self._firstFightData            = nil
    self._firstFightDataIndex       = nil
    self._flag                       = 1                                             -- 初始化flag为1,自动战斗分窜处理标示

    
    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendScene.extendRenderDestuctor(self)
    -- 记录信息
    self:record()
end



local function createSkipButton4Helper(self)
    if self:getChildByTag(9999) then return end
    local skipBtn = widget.newButton{
        normalImage = "help/btn_skip.png",
        contentSize = CCSize(200, 80),
        rect = CCRect(0, 0, 70, 80),
        scale = 1.1,
        labelParams = {
            text = _s("FIGHT_SKIP"),
            size = 40,
        }, 
        listener = function ( button )
            if not self.SKIP_CLICKED  and not HelperView2.isShowing() then 
                self.SKIP_CLICKED = true
                self:pauseAll()
                HelperView2.skipTo(300, 11)
                self:performWithDelay(function ()
                    CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
                end, 1.0)
            end
        end
    }
    self:addChild(skipBtn, M.DEPTH_LAYER_DOOR - 1, 9999)
    util.alignNode(skipBtn, self, display.TOP_RIGHT, 20)
end



-- [实例方法]
-- [实例方法][视图渲染]
-- 视图渲染
function M:viewRender()
    -- 加载关门层
    self:loadLayerDoor()
end

-- 视图渲染
function M:doRender()
    -- 播放音乐
    self:playMusic()

    -- 加载资源
    self:loadResources()

    -- 加载状态
    self:loadState()
    -- 加载数据
    self:loadData()

    -- 加载阶段1
    self:loadStep1()

    -----------------
    -- 加载测试
    -- self:loadTest()
    -----------------
end

-- 加载阶段1
function M:loadStep1()
    -- 加载遮罩
    self:loadMask()
    -- 加载层
    self:loadLayers()
end

function M:loadStep2()
    if not self:isHelpGuide() then 
        -- 加载渲染以外其他操作
        self:loadRenderOtherwise()
    end
end


---------


-- [实例方法][新手帮助]

-- 处理新手引导回调[通用步骤]
function M:helpJudgeCallback(index)
    
end

-- 处理新手引导回调[回合结束]
function M:helpJudgeEndRound()
end

-- 处理新手引导回调[渲染卡牌上场]
function M:helpJudgeStageDownCardRender()
    
end

-- 是否正在进行新手引导
function M:isHelpGuide()
    -- return self._isHelpGuide
    return false
end


---------


-- [实例方法][视图析构]
-- 视图析构
function M:viewDestructor()
    -- 卸载资源
    self:unloadResources()
end


---------


-- [实例方法][记录]
-- 记录信息
function M:record()
    -- 记录当前关卡信息（用于战斗结束地图跳转）
    GSVALUES.levelIdLast  = self._levelId
    GS.save(GSVALUES)
end


---------


-- [实例方法][网络]
-- 发送网络请求[开始游戏]
function M:netSendStart()
    -- [正式请求]
    --[[
    --]]
    -- 参数赋值
    local sns           = {}                        -- 上场卡牌sn数组(卡牌1，2，3...)
    local operation     = N_FIGHT_OP_START          -- 操作标识
    local callback      = self.netReceiveStart      -- 请求回调函数
    local isHideLoading = true                      -- 是否隐藏Loading
--print("22222222")
    -- 发送请求
    self:netSend({ flag = self._flag, sns = sns, operation = operation, callback = callback, isHideLoading = isHideLoading, })

    -- [模拟请求]
    --[[
    local properties = Json.decode(ROUND_OVER5_TEST_1)  -- ROUND_OVER5_TEST_1 ROUND_AUTO_TEST_10
    local markFields = {
        self = self,
        mark1 = 1,
        mark2 = "mark2",
    }

    self.netReceiveStart(properties, markFields)
    --]]
end

-- 发送网络请求[回合结束]
function M:netSendEndRound()
    -- 参数赋值
    local sns           = self:getCardsNetSns()     -- 上场卡牌sn数组(卡牌1，2，3...)
    local operation     = (TDArrayUtils.count(sns) == 0) and N_FIGHT_OP_ROUND_UNDO or  N_FIGHT_OP_ROUND_DO    -- 操作标识
    local callback      = self.netReceiveEndRound   -- 请求回调函数
--print("111111")
    -- 发送请求
    self:netSend({ sns = sns, operation = operation, callback = callback, })
end

-- 发送网络请求[自动战斗]
function M:netSendAutoFight(params)
	HelperView2.showNext(657, 870)
    -- print("发送请求........")
    -- 参数赋值
    local sns           = self:getCardsNetSns()     -- 上场卡牌sn数组(卡牌1，2，3...)
    local operation     = N_FIGHT_OP_FIGHT_AUTO     -- 操作标识
    local callback      = self.netReceiveAutoFight  -- 请求回调函数

    -- 发送请求
    self:netSend({ flag = self._flag, sns = sns, operation = operation, callback = callback, })
end

-- 接收网络请求[开始游戏]
function M.netReceiveStart(properties, markFields)
   --print("游戏结果:"..json.encode(properties))
    -- 获得当前场景
    local self = markFields.self

    -- 设置数据对象
    self._data = EntityRecFight.new(properties)
    self._flag = self._data:getResult():getFlag()
    -- 视图渲染
    -- print("视图渲染")
    self:doRender()
    -- 卸载层[门]
    self:unloadLayerDoor()
end

-- 接收网络请求[回合结束]
function M.netReceiveEndRound(properties, markFields)
    --print("结果。。。。。。。。"..json.encode(properties))
    -- 获得当前场景
    local self = markFields.self

    -- 设置数据对象
    self._data = EntityRecFight.new(properties)
    
    -- print("解析结束")

    -- 执行战斗
    self:fight()
    
    -- 重置网络请求回调状态
    self:netReceiveReset()
end

-- 接收网络请求[自动战斗]
function M.netReceiveAutoFight(properties, markFields)
    TDPerformanceTestingUtils.clockBegin("FightScene:netReceiveAutoFight(..)")
  --print("战斗结果:"..json.encode(properties))


    -- 获得当前场景
    local self = markFields.self

    -- 设置自动战斗标识
    self._isAuto = true
-- print("开始解析")
    -- 获得战斗数据
    local data = EntityRecFight.new(properties)

    -- print("解析结束")
-- print("战斗结果:"..data:getResult():getFlag())
    --设置自动战斗窜节点  为-1的时候战斗结束
    self._flag = data:getResult():getFlag()  



    -- 判断交互状态
    if self:isInteractive() then 
        -- print("自动战斗")
        -- 处理自动战斗[交互]
        self:handleAutoFightInteractive(data)
    else
        -- 处理自动战斗[非交互]
        self:handleAutoFightNonInteractive(data)
    end

    -- 重置网络请求回调状态
    self:netReceiveReset()

    TDPerformanceTestingUtils.clockEnd("FightScene:netReceiveAutoFight(..)")
end

local function getFirstFightData(self)
    if not self._firstFightData then 
        self._firstFightData = require("static.first_fight_records")
        self._firstFightDataIndex = 1
    end
    local result = self._firstFightData[self._firstFightDataIndex].DetailResponse[1] -- self._firstFightData[self._firstFightDataIndex]
    result.info.me.hero.name = _s("HELPER_INFO_ROLE_UNCLE") --"维卡大叔"
    result.info.opp.hero.name = _s("HELPER_INFO_ROLE_DEMON_KING") -- "魔族杀手"
    self._firstFightDataIndex = self._firstFightDataIndex + 1
    return result
end

-- 接收网络请求[回合结束]
function M:netSend(params)
    TDPerformanceTestingUtils.clockBegin("FightScene:netSend(..)")


    -- 参数赋值
    local sns           = params.sns                -- 上场卡牌sn数组(卡牌1，2，3...)
    local operation     = params.operation          -- 操作标识
    local callback      = params.callback           -- 请求回调函数
    local isHideLoading = params.isHideLoading      -- 是否隐藏Loading
    local flag          = params.flag or 1
    -- 更多参数
    local mode          = self._mode                -- 战斗模式
    local friendId      = self._friendId            -- 好友ID
    local levelId       = self._levelId             -- 关卡ID
    local difficulty    = self._difficulty          -- 难度
    local pkAction      = self:getPkAction()        -- PKAction
    local recordId      = self._recordId            -- 记录ID
    local tid           = self._tid
    local serid         = self._serid
    local recordAction  = self:getRecordAction()    -- 记录Action


    -- 请求动作
    if mode.battle == MODE_BATTLE_LEVEL then
        NetHelper.createRequest(
           ACTID.MSG_LEVEL_BATTLE,                  -- 请求ID
           callback,                                -- 回调函数
           {                                        -- 请求信息实体或参数Table
                fid         = friendId,                 -- 好友ID
                lid         = levelId,                  -- 关卡ID
                mid         = difficulty,               -- 难度
                op          = operation,                -- 操作标识
                fcards      = sns,                      -- 上场卡牌sn数组(卡牌1，2，3...)
                flag        = self._flag,
           },
           {
                failedCb    = netReceiveFail,          -- 请求失败回调函数
                cbParams    = { self = self },         -- 回调参数
                showLoading = not isHideLoading,       -- 是否显示Loading
            }
        )
    elseif mode.battle == MODE_BATTLE_EXPFBFLIGHT then
        NetHelper.createRequest(
           ACTID.MSG_GET_EXPFBFLIGHT,                  -- 请求ID
           callback,                                -- 回调函数
           {                                        -- 请求信息实体或参数Table
                typ         = self._typ,                 -- 类型 1 金币 2 经验卡。
                no         = self._no,                  -- 编号
                op          = operation,                -- 操作标识
                fcards      = sns,                      -- 上场卡牌sn数组(卡牌1，2，3...)
                flag        = self._flag,
           },
           {
                failedCb    = netReceiveFail,          -- 请求失败回调函数
                cbParams    = { self = self },         -- 回调参数
                showLoading = not isHideLoading,       -- 是否显示Loading
            }
        )
    elseif mode.battle == MODE_BATTLE_LADDERLIST then-- MSG_GET_LADDERLIST    幽冥魔塔战斗
        NetHelper.createRequest(
           ACTID.MSG_GET_LADDERLIST,                  -- 请求ID
           callback,                                -- 回调函数
           {                                        -- 请求信息实体或参数Table
                no         = self._no,                  -- 编号
                op          = operation,                -- 操作标识
                fcards      = sns,                      -- 上场卡牌sn数组(卡牌1，2，3...)
                flag        = self._flag,
           },
           {
                failedCb    = netReceiveFail,          -- 请求失败回调函数
                cbParams    = { self = self },         -- 回调参数
                showLoading = not isHideLoading,       -- 是否显示Loading
            }
        )
    
    elseif mode.battle == MODE_BATTLE_PK then 
        if mode.pk == MODE_PK_FIRST_FIGHT then
            self._flag = 0
            -- HelperView2.showNext(90, 130, 161, 170, 210) ===
            callback(getFirstFightData(self), { self = self })
        elseif mode.pk == MODE_PK_TEAM_FIGHT then
            NetHelper.createRequest(
               pkAction,                            -- 请求ID
               callback,                                -- 回调函数
               {                                        -- 请求信息实体或参数Table
                    tid         = tid,                 -- 记录ID
                    serid         = serid,                 -- 记录ID
               },
               {
                    failedCb    = netReceiveFail,          -- 请求失败回调函数
                    cbParams    = { self = self },         -- 回调参数
                    showLoading = not isHideLoading,       -- 是否显示Loading
                }
            )
        else
            -- print("PK>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
            NetHelper.createRequest(
               pkAction,                                -- 请求ID
               callback,                                -- 回调函数
               {                                        -- 请求信息实体或参数Table
                    oid         = friendId,                 -- 好友ID
                    kind        = mode.pk,                  -- 竞技类型
                    op          = operation,                -- 操作 
                    fcards      = sns,                      -- 上场卡牌sn数组(卡牌1，2，3...)
                    flag        = self._flag,
               },
               {
                    failedCb    = netReceiveFail,          -- 请求失败回调函数
                    cbParams    = { self = self },         -- 回调参数
                    showLoading = not isHideLoading,       -- 是否显示Loading
                }
            )
        end
    elseif mode.battle == MODE_BATTLE_BOSS then 
       
           ---------------BOSS战-----------------------------------------------------------
            NetHelper.createRequest(
               ACTID.MSG_BOSS_FIGHT,                                -- 请求ID
               callback,                                -- 回调函数
               {                                        -- 请求信息实体或参数Table
                    oid         = friendId,                 -- 好友ID
                    kind        = mode.pk,                  -- 竞技类型
                    op          = operation,                -- 操作 
                    fcards      = sns,                      -- 上场卡牌sn数组(卡牌1，2，3...)
                    flag        = self._flag,
               },
               {
                    failedCb    = netReceiveFail,          -- 请求失败回调函数
                    cbParams    = { self = self },         -- 回调参数
                    showLoading = not isHideLoading,       -- 是否显示Loading
                }
            )
       
    elseif mode.battle == MODE_BATTLE_RECORD then 
        NetHelper.createRequest(
           recordAction,                            -- 请求ID
           callback,                                -- 回调函数
           {                                        -- 请求信息实体或参数Table
                rid         = recordId,                 -- 记录ID
                serid       = serid,
           },
           {
                failedCb    = netReceiveFail,          -- 请求失败回调函数
                cbParams    = { self = self },         -- 回调参数
                showLoading = not isHideLoading,       -- 是否显示Loading
            }
        )
    elseif mode.battle == MODE_BATTLE_ARMY_BOSS then 

        NetHelper.createRequest(
           ACTID.MSG_ARMY_ATTACK,                            -- 请求ID
           callback,                                -- 回调函数
           {                                        -- 请求信息实体或参数Table
                armybossno         = self._no
           },
           {
                failedCb    = netReceiveFail,          -- 请求失败回调函数
                cbParams    = { self = self },         -- 回调参数
                showLoading = not isHideLoading,       -- 是否显示Loading
            }
        )
    end


    TDPerformanceTestingUtils.clockEnd("FightScene:netSend(..)")
end

-- 接收网络请求[失败]
function netReceiveFail(params, markFields)
    -- 参数赋值
    local self = markFields.self

    -- 显示[购买体力]对话框
    -- AlertLayer.new{
    --     titleStr    = _s("COMMON_NEWS_TIPS"),                       -- 标题
    --     content     = _s("NETWORK_FIGHT_CONNECT_FAILED"),   -- 内容
    --     btnStr      = { _s("COMMON_OK"), _s("COMMON_BACK") },        -- 按钮文字数组
    --     btnColor    = { AlertLayer.BTN_COLOR_YELLOW, AlertLayer.BTN_COLOR_YELLOW },
    --     -- 按钮颜色数组
    --     btnClickEvent = {                   -- 按钮事件数组
    --         function()                      -- 按钮[确定]回调
    --             -- print("出错[重新战斗]回调")
    --             -- 重新开始游戏
    --             self:gameRestart()
    --         end,
    --         function()                      -- 按钮[取消]回调
    --             -- print("出错[取消战斗]回调")
    --             -- 场景转换
                
    --         end,
    --     }, 
    -- }
    if self._mode.battle == MODE_BATTLE_LADDERLIST then
        CCBCenter.replaceLayer("MSG_MAPCONTAINERLAYER_MODEL", "ccb.c.map.MapContainerLayer", {fromMain = false, }, function (node)
                            node:performWithDelay(function ( ... )
                                game.requestDateForLoad( node )
                            end, 0.01)
                            node.onExit = function ( self )
                                util.removeUnusedTextures()
                            end
                        end,  nil)
        CCBCenter.loadCCB("MSG_TOWERLAYER_MODEL", "ccb.c.tower.TowerLayer",{fromMap = true})                      
                    
    elseif self._mode.battle == MODE_BATTLE_ARMY_BOSS then
        CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
        CCBCenter.loadCCB("MSG_ARMYMAINLAYER_MODEL", "ccb.c.army.ArmyMainLayer")
        -- CCBCenter.loadCCB("MSG_ARMYBOSSLAYER_MODEL", "ccb.c.army.ArmyBossLayer")
    else
        CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
    end
end

-- 获得PKAction
function M:getPkAction()
    local action = IDX_UNKNOWN

    local battle = self._mode.battle
    local pk     = self._mode.pk
    if battle == MODE_BATTLE_PK then 
        if pk == MODE_PK_LEARN or pk == MODE_PK_RANK_WAR then 
            action = ACTID.MSG_PK_BATTLE
        elseif pk == MODE_PK_COMPETITIVE_RACE then 
            action = ACTID.MSG_WEEK_BATTLE
        elseif pk == MODE_PK_TEAM_FIGHT then  
            action = ACTID.MSG_TEAM_PK
        end
    end

    return action
end

-- 获得记录Action
function M:getRecordAction()
    local action = IDX_UNKNOWN

    local battle = self._mode.battle
    local record = self._mode.record
    if battle == MODE_BATTLE_RECORD then 
        if record == MODE_RECODE_RANK_WAR then 
            action = ACTID.MSG_PLAY_TEAMRECORD
        elseif record == MODE_RECODE_COMPETITIVE_RACE then 
            action = ACTID.MSG_PLAY_WEEK_RECORD  
        end
    end

    return action
end


---------


-- [资源]
-- 加载资源
function M:loadResources()
    TDAnimationUtils.addSpriteFramesWithResource(PHI_FIGHT .. "f_c_ready")
    TDAnimationUtils.addSpriteFramesWithResource(PHI_FIGHT_EFFECT_OTH .. "e_f_oth_death")
end

-- 卸载资源
function M:unloadResources()

end


---------


-- [状态]
-- 加载状态
function M:loadState()
    -- 存储玩家状态
    self:storeStatePlayer()
end

-- 存储玩家状态
function M:storeStatePlayer()
    local properties = {
        lv          = ME:getLv(),                   -- 等级
        hp          = ME:getHp(),                   -- 血量
        totalcost   = ME:getTotalcost(),            -- 召唤值
        ccount      = ME:getCcount(),               -- 卡牌上限
        gcount      = ME:getGcount(),               -- 卡组上限
        tcount      = ME:getTcount(),               -- 好友上限
        curexp      = ME:getCurexp(),               -- 当前经验
        gold        = ME:getGold(),                 -- 金币
        diamond     = ME:getDiamond(),              -- 钻石
        cursp       = ME:getCursp(),                -- 当前疲劳
        totalexp    = ME:getTotalexp(),             -- 总经验值
        mname       = ME:getName(),                 -- 姓名[我]
        mp          = self._friendPoint,            -- 友情点[我]
        fname       = self._friendName,             -- 姓名[好友]
        fp          = self._friendPoint,            -- 友情点[好友]
    }

    self._statePlayer = EntityFightState.new(properties)
end


---------


-- [声音]
-- 播放音乐
function M:playMusic()
   
    -- 播放背景音乐
    SOUND:playMusicFight()
    
end


---------


-- [数据]
-- 加载数据
function M:loadData()
    -- 加载数据[天赋]
    self:loadDataTalent()
    -- 加载数据[装备]
    self:loadDataEquipEffect()
    -- 加载数据[英雄PO]
    self:loadDataPoHero()
    -- 加载数据[卡牌PO]
    self:loadDataPosCard()
end
-- 加载数据PO[英雄信息]
function M:loadDataPoHero()
    -- 我方用户数据
    self._poHeroMe           = self:poHeroMe()
    -- 敌方用户数据
    self._poHeroOpponent     = self:poHeroOpponent()

    -- 包装PO
    self._poHeroMe:setOwner(OWNER_ME)
    -- self._poHeroMe:setSex(ME:getSex())
    self._poHeroOpponent:setOwner(OWNER_OPPONENT)

    if self._mode.battle == MODE_BATTLE_LEVEL then 
        -- local heroId, heroName = require("db.leveldb").getHeroInfo(self._levelId)
        -- self._poHeroOpponent:setId(heroId)
        -- self._poHeroOpponent:setName(heroName)
    end
    -- self._poHeroOpponent:setSex(tonumber(self._poHeroOpponent:getHead()))
    -- local cards = self._data:getInfo():getOpp():getCards()
    -- self._poHeroOpponent:setId(cards[1]:getId())
end
-- 加载数据PO[卡牌信息]
function M:loadDataPosCard()
    -- 我方卡牌数据
    self._posCardMe          = self:posCardMe()
    -- 敌方卡牌数据
    self._posCardOpponent    = self:posCardOpponent()
end

-- 加载数据[天赋]
function M:loadDataTalent()
    -- 我方天赋信息
    self._talentMe          = self._data:getInfo():getMe():getTalent()
    -- 敌方天赋信息                                   
    self._talentOpponent    = self._data:getInfo():getOpp():getTalent()
end

-- 加载数据[装备]
function M:loadDataEquipEffect()
    -- 我方装备信息
    self._equipEffectMe          = self._data:getInfo():getMe():getHero():getEquipEffect()
    -- 敌方装备信息                                   
    self._equipEffectOpponent    = self._data:getInfo():getOpp():getHero():getEquipEffect()
end

-- 加载数据PO[英雄信息][我方英雄]
function M:poHeroMe()
    return self:daoHero(self._data:getInfo():getMe():getHero())
end
-- 加载数据PO[英雄信息][敌方英雄]
function M:poHeroOpponent()
    return self:daoHero(self._data:getInfo():getOpp():getHero())
end

-- 加载数据PO[卡牌信息][我方用户]
function M:posCardMe()
    return self:daoCards(self._data:getInfo():getMe():getCards(), OWNER_ME)
end
-- 加载数据PO[卡牌信息][敌方用户]
function M:posCardOpponent()
    return self:daoCards(self._data:getInfo():getOpp():getCards(), OWNER_OPPONENT)
end

-- 从数据库中查询英雄PO的详细信息     TODO:
function M:daoHero(po)
    return po
end
-- 从数据库中查询卡牌PO的详细信息     TODO:
function M:daoCards(pos, owner)
    -- [Version1]
    --[[
    -- 常量定义
    local TB_CARD_NAMES     = { "大恶魔", "黑甲铁骑士", "地狱红龙", "大天使", "哥布林", "坠落精灵", "荣耀巨人", "暗影巨蟒", "火焰凤凰", "牛头怪", "万龙", "斌哥怪", "大李冰", "种马", "野猪人" }
    local N_CARD_NAMES      = #TB_CARD_NAMES


    -- 方法定义
    -- 随机数字
    local function randomNum(min, max)
        return math.random(min, max)
    end

    -- 随机技能PO集合
    local function randomSkills()
        local skills = {}

        -- 技能1
        local skill1 = EntitySkill.new(SKILL_1)
        -- 技能2
        local skill2 = EntitySkill.new(SKILL_2)
        -- 技能3
        local skill3 = EntitySkill.new(SKILL_3)

        -- 添加
        skills[#skills + 1] = skill1
        skills[#skills + 1] = skill2
        skills[#skills + 1] = skill3

        return skills
    end



    if TDArrayUtils.isNotEmpty(pos) then    
        for _, po in pairs(pos) do

            -- 设置名称
            po:setName(TB_CARD_NAMES[randomNum(1, N_CARD_NAMES)])
            -- 卡牌星级
            po:setStar(randomNum(1, 5))
            -- 设置经验值
            po:setExp(randomNum(1, CARD_EXPERENCE_MAX))
            -- 设置COST值（上场值）
            po:setCost(randomNum(CARD_COST_MIN, CARD_COST_MAX))
            -- 设置种族
            po:setEthnicity(randomNum(1, CARD_ETHNICITY_NUM))
            -- 设置等待回合数
            po:setWaitRounds(randomNum(CARD_WAITROUNDS_MIN, CARD_WAITROUNDS_MAX))
            -- 设置生命值HP
            po:setHp(randomNum(CARD_HP_MIN, CARD_HP_MAX))
            -- 设置攻击力
            po:setAttack(randomNum(CARD_ATTACK_MIN, CARD_ATTACK_MAX))
            -- 设置技能集合
            po:setSkills(randomSkills())
        end
    end
    --]]
   
    -- [Version2]
    local talent = (owner == OWNER_ME) and self._talentMe or self._talentOpponent
    local equipEffect = (owner == OWNER_ME) and self._equipEffectMe or self._equipEffectOpponent
    for _, po in ipairs(pos) do
        po:inject({ talent = talent, equipEffect = equipEffect})
    end
    return pos
end


-- 从数据库中查询卡牌碎片PO的详细信息 
function M:daoChips(pos)
    
    for _, po in ipairs(pos) do
        po:inject()
    end
    return pos
end

-- 重置战斗数据
function M:resetData()
    self._data = nil

    if self._dataFollow ~= nil then 
        self._data = self._dataFollow
        self._dataFollow = nil
    end
end

-- 是否获得所有战斗数据
function M:isDataReceiveAll()
    -------------------------------------------------
    -- 说明：
    --     通过判断结束标识res来判断是否获得全部结果数据
    -------------------------------------------------
    local isAll = false
    
    if     self._dataFollow ~= nil then             -- 判断数据[_dataFollow]
        isAll = self._dataFollow:getResult():getRes() ~= 0
    elseif self._data ~= nil then                   -- 判断数据[_data]
        isAll = self._data:getResult():getRes() ~= 0
    end
    
    return isAll
end


---------


-- [层]
-- 加载层
function M:loadLayers()
    -- 加载层[欢迎]
    -- self:loadLayerWelcome()
    -- 加载层[背景]
    self:loadLayerBackground()
    -- 加载层[主场景]
    self:loadLayerMain()

    -- 加载层[按钮]
    self:loadLayerButton()
    -- 加载层[英雄]
    self:loadLayerHeros()
    
    -- 加载层[敌方玩家]
    self:loadLayerOpponent()
    
    -- 加载层[我方玩家]
    self:loadLayerMe()

    -- 加载层[速度]
    if TDHookUtils.existsFight() then 
    -- if true then 
        self:loadLayerSpeed()
    end
end

-- 加载层[欢迎]
function M:loadLayerWelcome()
    local params  = {
        mePo                    = self._poHeroMe,
        mePositionStart         = P_ME_WELCOME,
        mePositionEnd           = P_ME_HERO,
        oppPo                   = self._poHeroOpponent,
        oppPositionStart        = P_OPP_WELCOME,
        oppPositionEnd          = P_OPP_HERO,
        delegate                = self,
    }
    local layer = FightWelcomeLayer.new(params)
    self:addChild(layer, M.DEPTH_LAYER_WELCOME)
end

-- 加载层[门]
function M:loadLayerDoor()
    CCBCenter.loadCCB("MSG_DOORLAYER_MODEL", "ccb.c.fight.DoorLayer", {}, function(layer)
        self:addChild(layer, M.DEPTH_LAYER_DOOR)
    end)
end

-- 加载层[背景]
function M:loadLayerBackground()
    self:addLayer("FightBackgroundLayer", M.DEPTH_LAYER_BACKGROUND)
end

-- 加载层[主场景]
function M:loadLayerMain()
    self:addLayer("FightMainLayer", M.DEPTH_LAYER_MAIN, M.NODETAG_LAYER_MAIN)
end

-- 加载层[按钮]
function M:loadLayerButton()
    local layer = self:addLayer("FightButtonLayer", M.DEPTH_LAYER_BUTTON, M.NODETAG_LAYER_BUTTON)
    if layer and self._mode.battle == MODE_BATTLE_PK and self._mode.pk == MODE_PK_FIRST_FIGHT then 
        -- 首战禁用按键
        layer.setButtonAutoFightEnable_ = layer.setButtonAutoFightEnable
        layer.setButtonAutoFightEnable = function ( self )
            self:setButtonAutoFightDisable()
        end

        layer.setButtonJumpEnable_ = layer.setButtonJumpEnable
        layer.setButtonJumpEnable = function ( self )
            self:setButtonJumpDisable()
        end
        -- 在这里加上跳过按钮
        createSkipButton4Helper(self)
    end
    if layer and (HelperView2.isNextIndex(640) or HelperView2.isNextIndex(850)) then 
    	-- 1-1, 1-2引导时控制
        layer.setButtonAutoFightEnable_ = layer.setButtonAutoFightEnable_ or layer.setButtonAutoFightEnable
        layer.setButtonAutoFightEnable = function ( self )
            self:setButtonAutoFightDisable()
        end
        self._buttonLayer = layer
    end
end

-- 加载层[英雄]
function M:loadLayerHeros()
    local nodeTag = M.NODETAG_LAYER_HEROS
    local params  = {
        mePo                    = self._poHeroMe,
        mePositionHero          = P_ME_HERO,
        mePositionBlood         = P_ME_BLOOD,
        mePositionName          = P_ME_NAME,
        mePositionLevel         = P_ME_LEVEL,
        oppPo                   = self._poHeroOpponent,
        oppPositionHero         = P_OPP_HERO,
        oppPositionBlood        = P_OPP_BLOOD,
        oppPositionName         = P_OPP_NAME,
        oppPositionLevel        = P_OPP_LEVEL,
        mode                    = self._mode,
    }
    local layer = FightHerosLayer.new(params)
    self:addChild(layer, M.DEPTH_LAYER_HEROS, nodeTag)
end

-- 加载层[我方玩家]
function M:loadLayerMe()
    -- 加载层[我方玩家][场上卡牌]
    self:loadLayerMeCardsUp()
    -- 加载层[我方玩家][场下卡牌]
    self:loadLayerMeCardsDown()
end

-- 加载层[敌方玩家]
function M:loadLayerOpponent()
    -- 加载层[敌方玩家][场上卡牌]
    self:loadLayerOpponentCardsUp()
    -- 加载层[敌方玩家][场下卡牌]
    self:loadLayerOpponentCardsDown()
end

-- 加载层[我方玩家][场上卡牌]
function M:loadLayerMeCardsUp()
    local nodeTag = M.NODETAG_LAYER_M_CARDS_U
    local depth   = M.DEPTH_LAYER_UP
    local params  = {
        posCard                 = self._posCardMe,
        owner                   = OWNER_ME,
        isPrototypeAutoCorrect  = true,
    }
    local layer = FightCardsUpLayer.new(params)
    self:addChild(layer, depth, nodeTag)
    self._meCardsUp = layer
end

-- 加载层[我方玩家][场下卡牌]
function M:loadLayerMeCardsDown()
    local nodeTag = M.NODETAG_LAYER_M_CARDS_D
    local depth   = M.DEPTH_LAYER_DOWN
    local params  = {
        posCard                 = self._posCardMe,
        owner                   = OWNER_ME,
        isPrototypeAutoCorrect  = true,
    }
    local layer = FightCardsDownLayer.new(params)
    self:addChild(layer, depth, nodeTag)
    self._meCardsDown = layer
end

-- 加载层[敌方玩家]
function M:loadLayerOpponentCardsUp()
    local nodeTag = M.NODETAG_LAYER_O_CARDS_U
    local depth   = M.DEPTH_LAYER_UP
    local params  = {
        posCard                 = self._posCardOpponent,
        owner                   = OWNER_OPPONENT,
        isPrototypeAutoCorrect  = true,
    }
    local layer = FightCardsUpLayer.new(params)
    self:addChild(layer, depth, nodeTag)
    self._oppCardsUp = layer
end

-- 加载层[敌方玩家]
function M:loadLayerOpponentCardsDown()
    local nodeTag = M.NODETAG_LAYER_O_CARDS_D
    local depth   = M.DEPTH_LAYER_DOWN
    local params  = {
        posCard                 = self._posCardOpponent,
        owner                   = OWNER_OPPONENT,
        isPrototypeAutoCorrect  = true,
    }
    local layer = FightCardsDownLayer.new(params)
    self:addChild(layer, depth, nodeTag)
    self._oppCardsDown = layer
end

-- 加载层[变速]
function M:loadLayerSpeed()
    local nodeTag = M.NODETAG_LAYER_SPEED
    local depth   = M.DEPTH_LAYER_SPEED
    local layer = FightSpeedLayer.new(params)
    self:addChild(layer, depth, nodeTag)
end

-- 卸载层[门]
function M:unloadLayerDoor()
    CCBCenter.sendLuaMessage(MSG_ID.MSG_DOORLAYER_OPENDOOR)
end


---------


-- [渲染其他]
-- 加载视图以外其他操作
function M:loadRenderOtherwise()
    TDPerformanceTestingUtils.clockBegin("FightScene:loadRenderOtherwise(..)")


    -- 链接卡牌(设定场上、场下卡牌关联)
    self:linkCards()
    -- 设置按钮状态
    self:setButtonStatusNetReceive()
    -- 开启回合战斗
    self:fight()


    TDPerformanceTestingUtils.clockEnd("FightScene:loadRenderOtherwise(..)")
end

-- 链接卡牌(设定场上、场下卡牌关联)
function M:linkCards()
    -- 获得卡牌层
    local meCardsUp          = self:getLayerMeCardsUp():getCards()
    local meCardsDown        = self:getLayerMeCardsDown():getCards()
    local opponentCardsUp    = self:getLayerOpponentCardsUp():getCards()
    local opponentCardsDown  = self:getLayerOpponentCardsDown():getCards()

    -- 合法性验证
    TDAssertUtils.assertNotNil(meCardsUp)
    TDAssertUtils.assertNotNil(meCardsDown)
    TDAssertUtils.assertNotNil(opponentCardsUp)
    TDAssertUtils.assertNotNil(opponentCardsDown)

    -- 设置链接
    -- 我方卡牌
    for i in pairs(meCardsUp) do
        local up        = meCardsUp[i]
        local down      = meCardsDown[i]

        up:setAssociatedCard(down)
        down:setAssociatedCard(up)
    end
    -- 敌方卡牌
    for i in pairs(opponentCardsUp) do
        local up        = opponentCardsUp[i]
        local down      = opponentCardsDown[i]

        up:setAssociatedCard(down)
        down:setAssociatedCard(up)
    end
end

local function checkHelperInfo( self )
    local function helperCallback( index )
        -- if index == 60 then
        -- elseif index == 90 then
        -- elseif index == 170 then
        -- elseif index == 210 then
        -- elseif index == 220 then
        -- elseif index == 240 then
        -- elseif index == 250 then 
        --     CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
        --     return 
        -- -- elseif index == 70 or index == 110 or index == 190 then 
        -- --     HelperView2.showNext() 
        -- else 
        --     return 
        -- end
        -- self:resumeAll()

        if index == 50 or index == 60 then 
            self:resumeAll()
        elseif index == 70 then 
            CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
        end
    end
    HelperView2.showBetween(self, helperCallback, {50, 70}) 
end

local function isInStep( p, roundNo, i, helpStep )
    if helpStep and helpStep ~= HelperView2.getCurIndex() then return false end 
    return p.roundNo == roundNo and p.i == i 
end

-- 开启回合战斗
function M:fight()
    -- 设置交互状态[未交互]
    self:setInteractive(false)

    -- 解析战斗逻辑
    local params = {
        data                 = self._data,                          -- 数据
        playerMe             = self:getPlayerMe(),                  -- 玩家[我方玩家]
        playerOpponent       = self:getPlayerOpponent(),            -- 玩家[敌方玩家]
        round                = self:getRound(),                     -- 回合
        delegate             = self,                                -- 代理对象
    } 
    if HelperView2.isEnabled() and 50 <= HelperView2.getCurIndex() and HelperView2.getCurIndex() <= 70 then 
        --[[
        roundNo     回合数
        turn
        actionId    动作ID, 参看 FightActionEnum 的 ACTIONS
        node        对应的卡片
        index       po,位置
        sn          敌我位置表达,m1 m2... ,o1,o2...
        i           在此回合内的动作下标
        --]]
        
        params.callfront = function ( p )
            -- print("***************")
            -- print("Before run : Round", p.roundNo, "ActionId", p.actionId, "ActionIndex", p.i)
            -- print("***************")
            -- if isInStep(p, 5, 1, 70) then 
            --     -- 70[5, 1前, 定点70](第5回合，维卡卡牌可上场, 点击可上场卡牌)
            --     checkHelperInfo( self )
            --     return 
            -- elseif isInStep(p, 6, 7) then
            --     -- 90 [6, 7前](第6回合，魔族杀手卡牌上场)
            -- elseif isInStep(p, 12, 6) then
            --     -- 170 [12, 6前](第12回合，杀戮者上场，发动毁灭技能)
            -- elseif isInStep(p, 13, 7) then
            --     -- 210 [13, 7前](第13回合，杀戮者被秒后)
            -- elseif isInStep(p, 14, 1) then
            --     -- 220 [14, 1前](第14回合，拦截者准备回合)
            -- else
            --     return 
            -- end
            -- self:pauseAll()
            -- checkHelperInfo( self )

            -- 50 [2, 1前] 大叔咆哮~~~
            -- 60 [3, 1前] 魔君泼冷水...
            if isInStep(p, 2, 1) or isInStep(p, 3, 1) then 
                self:pauseAll()
                checkHelperInfo( self )
            end
        end

        params.callback = function ( p )
            -- print("===============")
            -- print("After run : Round", p.roundNo, "ActionId", p.actionId, "ActionIndex", p.i)
            -- print("===============")
            -- if isInStep(p, 4, 2) then 
            --     -- 50 [4, 2后](双方抽牌至第4回合,魔族杀手卡牌上场) 
            -- elseif isInStep(p, 14, 1) then
            --     -- 230 [14, 1后](第14回合，隐刃上场后)
            -- elseif isInStep(p, 14, 10) then
            --     -- 250 [14, 10后](战斗结束，维卡战败时,点击后直接回到首页)
            -- else
            --     return 
            -- end
            -- self:pauseAll()
            -- checkHelperInfo( self )
            if isInStep(p, 5, 21) then 
                --60后 [5, 21后](战斗结束，维卡战败时,点击后直接回到首页)
                self:pauseAll()
                checkHelperInfo( self )
            end
        end
    end
    FightInterpreter:interpret(params)
    -- print("开始fight()")
    if self._flag >= 1 then 
        self:netSendAutoFight()
    end
end


---------


-- [遮罩]
-- 加载遮罩
function M:loadMask()
    local layer = widget.newColorLayer(ccc4(0, 0, 0, 255))
    self:addChild(layer, M.DEPTH_LAYER_MASK, M.NODETAG_LAYER_MASK)
end

-- 卸载遮罩
function M:unloadMask()
    local layer = self:getChildByTag(M.NODETAG_LAYER_MASK)
    layer:removeFromParentAndCleanup(true)
end


---------


-- [自动战斗]
-- 处理自动战斗[交互]
function M:handleAutoFightInteractive(data)
    -- 设置数据对象
    self._data = data
    -- 执行战斗
    self:fight()
end

-- 处理自动战斗[非交互]
function M:handleAutoFightNonInteractive(data)
    self._dataFollow = data
end


---------


-- [快捷调用]
-- 获得层[英雄]
function M:getLayerHeros()
    return self:getLayer(M.NODETAG_LAYER_HEROS, FightHerosLayer)
end

-- 获得层[场上卡牌]
function M:getLayerCardsUp(owner)
    return (owner == OWNER_ME) and self:getLayerMeCardsUp() or self:getLayerOpponentCardsUp()
end

-- 获得层[场下卡牌]
function M:getLayerCardsDown(owner)
    return (owner == OWNER_ME) and self:getLayerMeCardsDown() or self:getLayerOpponentCardsDown()
end

-- 获得层[我方][场上卡牌]
function M:getLayerMeCardsUp()
    return self:getLayer(M.NODETAG_LAYER_M_CARDS_U, FightCardsUpLayer)
end

-- 获得层[我方][场下卡牌]
function M:getLayerMeCardsDown()
    return self:getLayer(M.NODETAG_LAYER_M_CARDS_D, FightCardsDownLayer)
end

-- 获得层[敌方][场上卡牌]
function M:getLayerOpponentCardsUp()
    return self:getLayer(M.NODETAG_LAYER_O_CARDS_U, FightCardsUpLayer)
end

-- 获得层[敌方][场下卡牌]
function M:getLayerOpponentCardsDown()
    return self:getLayer(M.NODETAG_LAYER_O_CARDS_D, FightCardsDownLayer)
end

-- 获得层[按钮]
function M:getLayerButton()
    return self:getLayer(M.NODETAG_LAYER_BUTTON, FightButtonLayer)
end

-- 获得层
function M:getLayer(nodeTag, clazz)
    local layer = self:getChildByTag(nodeTag)
    TDAssertUtils.assertInherit(layer, clazz)

    return layer
end

-- 获得天赋
function M:getTalent(owner)
    return (owner == OWNER_ME) and self._talentMe or self._talentOpponent
end
-- 获得装备效果
function M:getEquipEffect(owner)
    return (owner == OWNER_ME) and self._equipEffectMe or self._equipEffectOpponent
end

---------


-- [按钮操作]
-- 设置按钮状态[网络接收]
function M:setButtonStatusNetReceive()
    local isReceiveAll  = self:isDataReceiveAll()
    local layer         = self:getLayerButton()
    local showJump      = self:isShowButtonJump()
    local isHelpGuide   = self._isHelpGuide

    if isReceiveAll then 
        -- layer:swithMode(showJump and FightButtonLayer.MODE_END_JUMP or FightButtonLayer.MODE_END_NONE)
        layer:swithMode(showJump and FightButtonLayer.MODE_END_JUMP or (isHelpGuide and FightButtonLayer.MODE_END_NONE or FightButtonLayer.MODE_END_DIAMOND_SKIP))
    else
        layer:swithMode(FightButtonLayer.MODE_FIGHT)
    end
end

-- 设置按钮状态[交互]
function M:setButtonStatusInteractive(isTextureStartFight)
    -- self:getLayerButton():swithMode(FightButtonLayer.MODE_INTERACTIVE)
    self:getLayerButton():handleModeInteractive(#self._cardsNet > 0)
    local function callback( index )
        if 640 == index or 850 == index then 
            if self._buttonLayer then 
                self._buttonLayer.setButtonAutoFightEnable = self._buttonLayer.setButtonAutoFightEnable_
                self._buttonLayer:setButtonAutoFightEnable()
            end
        end
    end
    HelperView2.showBetween(self, callback, {640, 640}, {850, 850}) 
end

-- 设置按钮状态[死亡]
function M:setButtonStatusDie()
    self:getLayerButton():handleModeDie()
end

-- 是否显示跳过按钮
function M:isShowButtonJump()
    local isBattletNotLevel = self._mode.battle ~= MODE_BATTLE_LEVEL                                                -- 判断非关卡
    local isOverDifficulty  = self._mode.battle == MODE_BATTLE_LEVEL and self._difficulty <= self._difficultyMax    -- 判断难度 
    local isHook            = TDHookUtils.existsFight()

    if self._mode.battle == MODE_BATTLE_EXPFBFLIGHT then
        return false
    else
        return isBattletNotLevel or isOverDifficulty or isHook    
    end

    
end


---------


-- [代理]
-- 执行代理逻辑[动作运行]
function M:delActionRun(action, params)
    -- [记录动作]
    self._runningAction = action


    -- [调试日志]
    if self._DEBUG then
        -- 初始化动作调试对象
        if not self._debugActions then 
            local LINE_COUNT        = 5
            local FONG_SIZE         = 20
            local OFFSET_START      = acp(200, 200)
            local OFFSET_ITEM       = acp(0, -30)

            self._debugLebels       = {}    -- 标签集
            self._debugActions      = {}    -- 动作集
            self._debugActionIndex  = 1     -- 动作索引
            for i = 1, LINE_COUNT do
                -- 定义配置参数
                local params = {
                    text        = tostring(i),                  -- 标签文字
                    fontSize    = FONG_SIZE,                    -- 字体大小
                    parentNode  = self,                         -- 父结点
                    align       = display.CENTER,               -- 对齐方式
                    anchor      = display.CENTER_LEFT,          -- 锚点
                    offset      = ccpAdd(OFFSET_START, ccp(OFFSET_ITEM.x * (i - 1), OFFSET_ITEM.y * (i - 1))),        -- 偏移量
                    z           = 9999,
                }
                -- 生成标签
                local label = TDUIUtils.loadLabelTTF(params)
                display.align(label, display.CENTER_LEFT)
                self._debugLebels[i] = label
            end
        end


        -- 处理调试显示
        local className = action:getClassName()
        local naming    = ActionClassNamingMapper[className]

        if #self._debugActions >= #self._debugLebels then 
            table.remove(self._debugActions, 1)
        end
        self._debugActions[#self._debugActions + 1] = format("%d %s", self._debugActionIndex, naming)

        for i = 1, #self._debugActions do
            local label = self._debugLebels[i]
            local text  = self._debugActions[i]
            local color = string.find(text, "[系统]") ~= nil and ccc3(255, 0, 0) or ccc3(255, 255, 255)

            label:setString(text)
            -- label:setColor(color)
        end

        self._debugActionIndex = self._debugActionIndex + 1
    end
end


---------


-- [工具方法]
-- 添加层
function M:newLayer(className)
    return requireL(className).new()
end
-- 添加层
function M:addLayer(className, z, tag)
    -- 默认赋值
    z   = z or 0 

    -- 逻辑处理
    local layer = self:newLayer(className)
    if not tag then
        self:addChild(layer, z)
    else
        self:addChild(layer, z, tag)
    end
    return layer
end


---------


-- [全局访问]
-- 游戏结束
function M:gameOver()
    -- print("游戏结束")
    -- 设置标识 
    _isGameEnded = true
    -- 加载结果视图
    self:addLayerResult()
end
-- 判断是否游戏结束
function M:isGameOver()
    return _isGameEnded == true
end

-- 重新开始游戏
function M:gameRestart()
    
    if self._mode.battle == MODE_BATTLE_EXPFBFLIGHT then
          local scene = require("view.Fight.Scene.FightScene").new({  mode = {battle = MODE_BATTLE_EXPFBFLIGHT},typ = self._typ,no = self._no,fnNextView = function ( )        
                                    CCBCenter.replaceLayer("MSG_MAPCONTAINERLAYER_MODEL", "ccb.c.map.MapContainerLayer", {fromMain = false, }, function (node)
                                        node:performWithDelay(function ( ... )
                                            game.requestDateForLoad( node )
                                        end, 0.01)
                                        node.onExit = function ( self )
                                            util.removeUnusedTextures()
                                        end
                                    end,  nil)
                                    CCBCenter.loadCCB("MSG_SPECIALMAPLAYER_MODEL", "ccb.c.specialMap.SpecialMapLayer",{specialMapType = 1,mapTyp = self._typ,isCheckpoint = 1, fromMap = true})                      
                            end, })
                CCBCenter.replaceScene(scene)
    elseif self._mode.battle == MODE_BATTLE_LADDERLIST then

        local scene = require("view.Fight.Scene.FightScene").new({  mode = {battle = MODE_BATTLE_LADDERLIST},no = self._no,fnNextView = function ( )        
                                        CCBCenter.replaceLayer("MSG_MAPCONTAINERLAYER_MODEL", "ccb.c.map.MapContainerLayer", {fromMain = false, }, function (node)
                                            node:performWithDelay(function ( ... )
                                                game.requestDateForLoad( node )
                                            end, 0.01)
                                            node.onExit = function ( self )
                                                util.removeUnusedTextures()
                                            end
                                        end,  nil)
                                        CCBCenter.loadCCB("MSG_TOWERLAYER_MODEL", "ccb.c.tower.TowerLayer",{fromMap = true})                      
                                end, })
                    CCBCenter.replaceScene(scene)
    elseif self._mode.battle == MODE_BATTLE_ARMY_BOSS then

        local scene = require("view.Fight.Scene.FightScene").new({  mode = {battle = MODE_BATTLE_ARMY_BOSS},no = self._no,fnNextView = function ( )        
                                    CCBCenter.replaceScene(MAINLAYER_ID, MAINLAYER_PATH)
                                    CCBCenter.loadCCB("MSG_ARMYMAINLAYER_MODEL", "ccb.c.army.ArmyMainLayer")
                                    CCBCenter.loadCCB("MSG_ARMYBOSSLAYER_MODEL", "ccb.c.army.ArmyBossLayer")
                                end, })
                    CCBCenter.replaceScene(scene)


    --去掉钻石跳过的判断
    elseif self._againDATA then
        local function isSpEmpty( )
            local costSp = math.ceil(self._againDATA.po.sp / GAMERATE:getSpRate())
            return costSp > ME:getCursp()
        end
        -- 过滤判断
        if isSpEmpty() then 
            -- 显示对话框[体力不足]
            AlertLayer.new4OK({
                content     = _s("MAP_SP_NOT_ENOUGH")
            })
            return
        end
        self._againDATA.po.star = self._againDATA.po.difficulty
        self._againDATA.ynShowCloseBtn = true
        if self._againDATA.po.difficulty == DIFFICULTY_DIFFICULT then
            local friendId          = 0                                 	-- 好友ID
            local friendName        = ""                                -- 好友名称
            local friendPoint       = 0                                 -- 友情点
            local levelId           = self:getLevelId(self._againDATA.po)    -- 关卡ID
            local difficulty        = self._againDATA.po.difficulty     -- 关卡难度
            local difficultyMax     = self._againDATA.po.star           -- 关卡难度[最大]
            local isBreak           = self._againDATA.po.isBreak        -- 是否是最新关卡（如果胜利则展开新关卡）
            local mode              = { battle = MODE_BATTLE_LEVEL }    -- 战斗模式
            -- 场景转换
            local scene = require("view.Fight.Scene.FightScene").new{ 
                friendId = friendId, 
                friendName = friendName, 
                friendPoint = friendPoint, 
                levelId = levelId, 
                difficulty = difficulty, 
                difficultyMax = difficultyMax, 
                isBreak = isBreak, 
                mode = mode,
                data = self._againDATA,
            }
            CCBCenter.replaceScene(scene)
        else
        

            -- 要再请求一次好友数据啊,cow
            local function netReceiveFriends( properties,  markFields)
                -- 实体
                local EntityFriends = require("ccb.m.friend.EntityFriends")
                local whole = EntityFriends.new(properties)
                -- 操作变量
                local countFriends  = whole:getFcount()                 -- 好友数
                local friends       = whole:getFriends()                -- 好友实体
                self._againDATA.dataFriends = friends
                NetHelper.hideLoading(true)
                CCBCenter.replaceScene("MSG_MAPPOPUPFRIENDLAYER_MODEL", "ccb.c.map.MapPopupFriendLayer",self._againDATA)
            end
            NetHelper.createRequest(
                   ACTID.MSG_INVITE_FRIPOINT_LIST,          -- 请求ID
                   netReceiveFriends,                       -- 回调函数
                   {                                        -- 请求信息实体或参数Table
                   }
                )
        end
        
        -- CCBCenter.replaceLayer("MSG_MAPPOPUPFRIENDLAYER_MODEL", "ccb.c.map.MapPopupFriendLayer",self._againDATA)
    else
        -- 场景转换
        local scene = require("view.Fight.Scene.FightScene").new({ 
            
            friendId        = self._friendId,                       -- 好友ID
            friendName      = self._friendName,                     -- 好友名称
            friendPoint     = self._friendPoint,                    -- 友情点
            levelId         = self._levelId,                        -- 关卡ID
            difficulty      = self._difficulty,                     -- 关卡难度
            difficultyMax   = self._difficulty,                  -- 关卡难度[最大] 最大和难度等值，这样避免重复钻石跳过
            mode            = self._mode,                           -- 战斗模式
        })
        CCBCenter.replaceScene(scene)
    end
end

-- 判断是否是最后一轮
function M:isLastRound(round)
    local isLast = true

    local data = (self._dataFollow ~= nil) and self._dataFollow or self._data
    local fights = data:getFight()

    if TDArrayUtils.isNotEmpty(fights) then 
        local lastRound = fights[#fights]:getRound()
        isLast = round == lastRound
    end

    return isLast
end

-- 设置交互状态
function M:isInteractive()
    return self._isInteractive
end

-- 设置交互状态
function M:setInteractive(interactive)
    self._isInteractive = interactive
end

-- 获得关卡ID
function M:getLevelId()
    return self._levelId
end

function M:getMode( )
    return self._mode
end

-- 获得关卡难度
function M:getDifficulty()
    return self._difficulty
end

-- 获得数据对象
function M:getData()
    return self._data
end

-- 获得场景层[我方玩家]
function M:getPlayerMe()
    return self:getPlayer(OWNER_ME)
end
-- 获得场景层[敌方玩家]
function M:getPlayerOpponent()
    return self:getPlayer(OWNER_OPPONENT)
end

-- 获得玩家数据
function M:getPlayer(owner)
    local cards = self:getPlayerCards(owner)

    -- 定义参数
    local params = {
        heroHp      = self:getPlayerBlood(owner),                -- 英雄血量
        cards       = self:getPlayerCards(owner),                -- 卡牌集
        prototypes  = self:getPlayerPrototypes(owner),           -- 原型集
    }

    return CardPlayer.new(params)
end

-- 获得玩家数据[位置集]
function M:getPlayerBlood(owner)
    local layerHeros = self:getLayerHeros()
    return layerHeros:getBlood(owner)
end

-- 获得玩家数据[卡牌集]
function M:getPlayerCards(owner)
    local cards = {}

    -- 获得卡牌集
    local cardsDown = self:getLayerCardsDown(owner):getCards()

    -- 生成卡牌集
    for i, card in pairs(cardsDown) do
        cards[i] = card
    end
    
    return cards
end

-- 获得玩家数据[原型集]
function M:getPlayerPrototypes(owner)
    return self:getLayerCardsDown(owner):getPrototypes()
end

-- 添加卡片详情层
function M:addLayerCardDetail(po)
    local layer = CardDetailLayer.new({poCard = po})
    self:addChild(layer, M.DEPTH_LAYER_CARD_DETAIL)
end

-- 添加游戏结果层
function M:addLayerResult()
    if HelperView2.isNextIndex(300) then 
        return
    end
    -- 获得结束请求数据
    
    -- print("_data:po")
    -- dump(self._data)
    local data = (self._dataFollow ~= nil) and self._dataFollow or self._data
    local po   = data:getResult()
    -- print("addLayerResult:po"..po:getBrlist()[1])
    --  dump(po)
    -- dump(po:getBrList())
    -- 参数赋值
    local result        = po:getRes()                       -- 战斗结果（成功、失败）
    local diamond       = po:getDiamond()                   -- 钻石
    local gold          = po:getGold()                      -- 金币
    local experience    = po:getExp()                       -- 经验
    local friendPoint   = po:getFp()                        -- 友情点
    
    -- 战斗结果换成 brlist
    -- local cards         = self:daoCards(po:getCards())      -- 获得卡牌数组，不超过5张
    -- local chips         = self:daoChips(po:getChips())      -- 获得碎片数组，不超过5张

    local brlist        = po:getBrlist()                    -- 掉落物数组

    local star          = po:getStar()                      -- 获得星星
    local jjcScore      = po:getJjcScore()                  -- 获得竞技场分数
    local rank          = po:getRank()                      -- 获得排名
    local rankvar       = po:getRankvar()                   -- 获得排名变化值
    local isWin         = result == RES_WIN                 -- 战斗成功标识 
    local mode          = self._mode                        -- 对战模式
    local fnNextView    = self._fnNextView                  -- 下一跳转视图方法
    local levelId       = self._levelId                     -- 关卡ID
    local difficulty    = self._difficulty                  -- 关卡难度
    local isHelpGuide   = self:isHelpGuide()                -- 是否正在进行新手引导
    local isBreak       = isWin and self._isBreak           -- 是否是最新关卡（如果胜利则展开新关卡）
    local levels        = po:getLevels()                    -- 取得最新的地图信息

    local params = {
        title                 = isWin and _s("FIGHT_WIN") or _s("FIGHT_FAILED"),                        -- 战斗结果（成功、失败）
        result                = result,                                                 -- 战斗成功标识 
        gold                  = tostring(gold),                                         -- 金币
        diamond               = tostring(diamond),                                      -- 钻石
        experience            = tostring(experience),                                   -- 经验值
        cards                 = cards,                                                  -- 获得卡片集
        chips                 = chips,                                                  -- 获得碎片集
        brlist                = brlist,                                                  -- 物品集合
        star                  = star,                                                   -- 获得星星
        jjcScore              = jjcScore,                                               -- 获得竞技场分数
        rank                  = rank,                                                   -- 获得排名
        rankvar               = rankvar,                                                -- 获得排名变化值
        stateBefore           = self._statePlayer,                                      -- 状态[战斗前]
        levelId               = levelId,                                                -- 关卡ID
        difficulty            = difficulty,                                             -- 关卡难度
        mode                  = mode,                                                   -- 对战模式
        fnNextView            = fnNextView,                                             -- 下一跳转视图方法
        isHelpGuide           = isHelpGuide,                                            -- 是否正在进行新手引导
        isBreak               = isBreak,                                                -- 是否是最新关卡（如果胜利则展开新关卡）
        levels                = levels,
    }
    -- dump(params)

    -- 关闭背景音乐
    SOUND:playMusicStop()
    -- 设置按钮层
    self:setButtonStatusDie()

    CCBCenter.loadCCB("MSG_FIGHTRESULT3LAYER_MODEL", "ccb.c.fight.FightResult3Layer", params, function(layer)
        util.enableCustomTouch(self._meCardsUp, false)
        util.enableCustomTouch(self._meCardsDown, false)
        util.enableCustomTouch(self._oppCardsUp, false)
        util.enableCustomTouch(self._oppCardsDown, false)
        self:addChild(layer, 100000)
    end)
end

-- 获得回合对象
function M:getRound()
    local node = self:getChildByTag(M.NODETAG_LAYER_MAIN)
    TDAssertUtils.assertInherit(node, requireL("FightMainLayer"))

    return node:getRound()
end

-- 获得是否自动战斗标识
function M:isAutoFight()
    return self._isAuto
end

-- 获得卡牌集[场上]
function M:getCardsUp(owner)
    return self:getOrSetCards(owner, nil, M.UD_UP, M.GS_GET)
end
-- 设置卡牌集[场上]
function M:setCardsUp(owner, cards)
    self:getOrSetCards(owner, cards, M.UD_UP, M.GS_SET)
end

-- 获得卡牌集[场下]
function M:getCardsDown(owner)
    return self:getOrSetCards(owner, nil, M.UD_DOWN, M.GS_GET)
end
-- 设置卡牌集[场下]
function M:setCardsDown(owner, cards)
    self:getOrSetCards(owner, cards, M.UD_DOWN, M.GS_SET)
end

function M:getOrSetCards(owner, cards, upOrDown, getOrSet)
    if getOrSet == M.GS_GET then 
        local cards
        if owner == OWNER_ME then 
            if upOrDown == M.UD_UP then 
                cards = self._cardsUpMe
            else
                cards = self._cardsDownMe
            end
        else
            if upOrDown == M.UD_UP then 
                cards = self._cardsUpOpp
            else
                cards = self._cardsDownOpp
            end
        end

        return cards
    else 
        if owner == OWNER_ME then 
            if upOrDown == M.UD_UP then 
                self._cardsUpMe = cards
            else
                self._cardsDownMe = cards
            end
        else
            if upOrDown == M.UD_UP then 
                self._cardsUpOpp = cards
            else
                self._cardsDownOpp = cards
            end
        end
    end
end

function M:getIndexBase()

    return self:getOrSetIndexBase(0, M.GS_GET)
end

function M:setIndexBase(index)
    return self:getOrSetIndexBase(index, M.GS_SET)
end

function M:getOrSetIndexBase(index, getOrSet)
    if getOrSet == M.GS_GET then 
        return self._indexBase
    else 
        self._indexBase = index
    end
end

-- 获得卡牌集[网络]
function M:getCardsNet()
    return self._cardsNet
end

-- 添加卡片至卡牌集[网络]
function M:addToCardsNet(card)
    -- self._cardsNet[card] = true
    TDArrayUtils.addItemOrder(self._cardsNet, card)
end

-- 移除卡片从卡牌集[网络]
function M:removeFromCardsNet(card, owner)
    TDArrayUtils.removeItemOrder(self._cardsNet, card)
end

-- 添加卡片至卡牌集[场上]
function M:addToCardsUp(owner, card)
    self:addOrRemoveCard(owner, card, M.UD_UP, M.AR_ADD)
end

-- 移除卡片从卡牌集[场上]
function M:removeFromCardsUp(owner, card)
    self:addOrRemoveCard(owner, card, M.UD_UP, M.AR_REMOVE)
end

-- 添加卡片至卡牌集[场下]
function M:addToCardsDown(owner, card)
    self:addOrRemoveCard(owner, card, M.UD_DOWN, M.AR_ADD)
end

-- 移除卡片从卡牌集[场下]
function M:removeFromCardsDown(owner, card)
    self:addOrRemoveCard(owner, card, M.UD_DOWN, M.AR_REMOVE)
end

function M:addOrRemoveCard(owner, card, upOrDown, addOrRemove)
    local cards

    if owner == OWNER_ME then 
        if upOrDown == M.UD_UP then 
            cards = self._cardsUpMe
        else 
            cards = self._cardsDownMe
        end
    else
        if upOrDown == M.UD_UP then 
            cards = self._cardsUpOpp
        else 
            cards = self._cardsDownOpp
        end
    end

    if addOrRemove == M.AR_ADD then 
        TDArrayUtils.addItemOrder(cards, card)
    else
        TDArrayUtils.removeItemOrder(cards, card)
    end
end

-- 重置卡片从卡牌集[网络]
function M:resetCardsNet()
    self._cardsNet = {}
end

-- 重新排列卡牌集[场上]
function M:resortCardsUp( MEorOPP )
 

    -- 获得卡牌原型集
    local cards;
   
    local prototypes;

    if MEorOPP == WONER_ME then
        cards       = self._cardsUpMe
        prototypes  = self:getPlayerPrototypes(OWNER_ME)
    else
        cards       = self._cardsUpOpp
        prototypes  = self:getPlayerPrototypes(OWNER_OPPONENT)
    end

    if TDArrayUtils.isNotEmpty(cards) then
        for i in ipairs(cards) do

            if  i > table.nums(cards)-5 then
                
                local cardI = i-(table.nums(cards)-5) --多于5张
                if table.nums(cards) < 5 then 
                    cardI = i  --少于5张
                end
                
                local card = cards[i]:getAssociatedCard()
                 
                local positionTarget  = prototypes:getPrototype(IDX_STAGEUP_INC + cardI):getPosition()
                -- local positionCurrent = ccp(card:getPosition())
                local time = FightActionHelper:time(.3)
                card:stopAllActions()
                card:setIndex(IDX_STAGEUP_INC + cardI)
                
                --如果这张卡牌是隐藏的，显示的时候采用渐显
                if not card:isVisible() then
                    card:setPosition(ccp(positionTarget.x-50, positionTarget.y))

                    local action = transition.sequence({
                        CCCallFunc:create(function() card:showAll() end), 
                        CCSpawn:createWithTwoActions(
                            CCFadeIn:create(0.4),
                            CCMoveTo:create(0.4, positionTarget)
                        ),
                    })
                    card:runAction(action)
                else 
                    local action = transition.sequence({
                        CCMoveTo:create(time, positionTarget),
                    })
                    card:runAction(action)    
                end
                
            end
        end
    end
end

-- 重置网络请求回调状态
function M:netReceiveReset()
    -- 重置卡片从卡牌集[网络]
    self:resetCardsNet()
    -- 重置上场卡牌对应的下场卡牌的状态
    self:resetStateCardsDown()
    -- 设置按钮状态
    self:setButtonStatusNetReceive()
end

-- 重置上场卡牌对应的下场卡牌的状态(将下场卡牌状态还原)
function M:resetStateCardsDown()
    local cardsUp = self._cardsUpMe

    if TDArrayUtils.isNotEmpty(cardsUp) then

        for _ in pairs(cardsUp) do
            local down = cardsUp[_]
            down:resetColor()
            down:hide()
        end
    end
end

-- 获得发送网络请求的卡片集Sn集合
function M:getCardsNetSns()
    local sns = {}

    local cards = self._cardsNet
    
    for k in ipairs(cards) do
        if cards[k] then 
            sns[#sns + 1] = cards[k]._po:getSn()
        end
    end

    return sns
end

-- 获得卡牌集[所有－包括敌我双方，返回形式为字典]
function M:getAllPlayerCards()
    return {
        [OWNER_ME]          = self:getPlayerCards(OWNER_ME),
        [OWNER_OPPONENT]    = self:getPlayerCards(OWNER_OPPONENT)
    }
end

-- 请除所有卡牌集就绪状态
function M:cleanAllPlayerCardsReady()
    self:getLayerMeCardsDown():clearAllStateReady()
    self:getLayerOpponentCardsDown():clearAllStateReady()
end

-- 获得玩家状态
function M:getStatePlayer()
    return self._statePlayer
end

-- 跳转至结果页
function M:skipToResult()
    TDPerformanceTestingUtils.clockBegin("FightScene:skipToResult(..)")


    -- 停止当前执行动作
    self._runningAction:stop()

    -- 添加结果展示页
    self:addLayerResult()


    TDPerformanceTestingUtils.clockEnd("FightScene:skipToResult(..)")
end


---------


-- [合法性验证]
-- 验证参数合法性
function M:assertParameters(params)
    -- 验证参数集
    -- TDAssertUtils.assertParameters(params)
end


---------


-- [暂停/恢复]
function M:pauseAll()
    local nodes = {}
    self:iterateAddNode(nodes, self)
    TDAnimationUtils.pauseActionsInTargets(nodes)
end

function M:resumeAll()
    local nodes = {}
    self:iterateAddNode(nodes, self)
    TDAnimationUtils.resumeActionsInTargets(nodes)
end

function M:iterateAddNode(nodes, node)
    nodes[#nodes + 1] = node

    local children = node:getChildren()
    -- 处理子结点关闭动画
    if children ~= nil and children:count() > 0 then 
        for i = 0, children:count() - 1 do
            self:iterateAddNode(nodes, util.toCCNode(children:objectAtIndex(i)))
        end
    end
end


---------


-- [测试]
-- 加载测试
function M:loadTest()
    -- 加载测试[PO]
    -- self:loadTestPo()

    -- 加载测试[层]
    self:loadTestLayer()

    -- 加载测试[效果]
    -- self:loadTestEffect()

    -- 加载测试[粒子效果]
    -- self:loadTestParticle()

    -- 加载测试[控件]
    -- self:loadTestControl()

    -- 加载测试[其他]
    -- self:loadTestOther()
end

-- 加载测试[PO]
function M:loadTestPo()
    -- Hero
    local propertiesUser =
        {
            name  = "AC",
            head  = "1",        
            lv    = 12,                           
            hp    = 4000,                         
            exp   = 50000,
        }
    -- print(Json.encode(propertiesUser))


    local poHero = EntityHero.new(propertiesUser)
    local spriteHero =  CardFactory.newHero(poHero)
    self:addChild(spriteHero)
    spriteHero:setPosition(ccp(100, 100))

    
    -- Card
    local propertiesCard =
        {
            id              = 1,
            lv              = 10,
            star            = 3,
            sn              = "m1",

            name            = "大香蕉",
            exp             = 10000,
            cost            = 6,
            ethnicity       = 1,
            waitRounds      = 3,
            hp              = 300,
            attack          = 1000,
            defense         = 2000,
            skills          = {
                {
                    id              = 1,
                    name            = "污染3",
                    info            = "当攻击目标种族为森林时，攻击力加成60％。",
                    unlockLevel     = 0,
                    isLearned       = true,
                },
                {
                    id              = 2,
                    name            = "瘟疫5",
                    info            = "使对方所有卡牌灭丧失25点攻击力和生命值。",
                    unlockLevel     = 5,
                    isLearned       = true,
                },
                {
                    id              = 3,
                    name            = "地狱守护4",
                    info            = "增加场上除自身外我方其他地狱卡牌200点生命值。",
                    unlockLevel     = 10,
                    isLearned       = false,
                },
            }
        }
    local poCard = EntityCard.new(propertiesCard)
    -- print(Json.encode(propertiesCard))
    local spriteCard =  CardFactory.newCardViewed({ po = poCard, viewType = VIEWTYPE_CARD_IMAGE, viewDetailParentNode = self })
    self:addChild(spriteCard)
    spriteCard:setPosition(ccp(200, 200))
end

-- 加载测试[层]
function M:loadTestLayer()
    -- 1、窗口视图
    --[[
    local layer = DemoViewWindowLayer.new()
    self:addChild(layer)
    --]]

    -- 2、效果视图
    --[[
    local layer = DemoEffectLayer.new()
    self:addChild(layer)
    --]]

    -- 3、欢迎视图
    --[[
    local params  = {
        mePo                    = self._poHeroMe,
        mePositionStart         = P_ME_WELCOME,
        mePositionEnd           = P_ME_HERO,
        oppPo                   = self._poHeroOpponent,
        oppPositionStart        = P_OPP_WELCOME,
        oppPositionEnd          = P_OPP_HERO,
    }
    local layer = FightWelcomeLayer.new(params)
    self:addChild(layer, 100000)
    --]]
end

-- 加载测试[效果]
function M:loadTestEffect()
    -- 1、CCWaves
    -- 1、CCWaves:create(float duration, const CCSize& gridSize, unsigned int waves, float amplitude, bool horizontal, bool vertical
    -- CCActionInterval* waves = CCWaves::actionWithWaves(5, 20, true, false, ccg(15,10), 5);  


    -- 2、CCShaky3D


    -- 3、Death
    local PH            = "fight/"
    local PH_CARD       = PH .. "card/"
    local PH_PARTICLE   = PH .. "particle/"

    -- 加载卡片
    local card = display.newSprite(PH_CARD .. "f_c_image.png")
    self:addChild(card, 999999)
    card:setPosition(ccp(display.cx, display.cy))
end

-- 加载测试[粒子效果]
function M:loadTestParticle()
    -- 常量
    -- 路径
    local PH            = "fight/"
    local PH_CARD       = PH .. "card/"
    local PH_PARTICLE   = PH .. "particle/"

    -- 加载卡片
    local card = display.newSprite(PH_CARD .. "f_c_image.png")
    self:addChild(card, 999999)
    card:setPosition(ccp(display.cx, display.cy))



    --[[
    /* BlendingFactorDest */
    #define GL_ZERO                           0
    #define GL_ONE                            1
    #define GL_SRC_COLOR                      0x0300
    #define GL_ONE_MINUS_SRC_COLOR            0x0301
    #define GL_SRC_ALPHA                      0x0302
    #define GL_ONE_MINUS_SRC_ALPHA            0x0303
    #define GL_DST_ALPHA                      0x0304
    #define GL_ONE_MINUS_DST_ALPHA            0x0305

    /* BlendingFactorSrc */
    /*      GL_ZERO */
    /*      GL_ONE */
    #define GL_DST_COLOR                      0x0306
    #define GL_ONE_MINUS_DST_COLOR            0x0307
    #define GL_SRC_ALPHA_SATURATE             0x0308
    --]]



    -- 加载粒子效果
    -- 1、水珠上浮
    --[[
    local emitter = CCParticleSystemQuad:create(PH_PARTICLE .. "f_p_bubble.plist")
    local batch = CCParticleBatchNode:createWithTexture(emitter:getTexture())
    batch:addChild(emitter)
    card:addChild(batch, 10)

    -- emitter:setBlendAdditive(true)
    local tBlendFunc = ccBlendFunc:new()
    tBlendFunc.src = 0x0302 --GL_ONE
    tBlendFunc.dst = 0x0303 --GL_ONE_MINUS_SRC_ALPHA
    emitter:setBlendFunc(tBlendFunc)
    -- emitter1:setPosition(ccp(card:getContentSize().width / 2, card:getContentSize().height / 2))
    --]]
end

-- 加载测试[控件]
function M:loadTestControl()
    
end

-- 加载测试[其他]
function M:loadTestOther()
    -- 1、类
    -- 方式一：
    --[[
    local action1 = FightActionPosition.new()
    action1:setPositionBegin(ccp(10, 10))
    action1:setPositionBegin(ccp(20, 20))

    local action2 = FightActionPosition.new()
    action2:setPositionBegin(ccp(100, 100))
    action2:setPositionBegin(ccp(200, 200))


    print("================================")
    print("action1: ", action1:getPositionBegin().x)
    print("action2: ", action2:getPositionBegin().x)
    print("================================")
    --]]


    -- 方式二：
    --[[
    local action3 = FightActionPosition.new({ positionBegin = ccp(10, 10), positionEnd = ccp(20, 20), })
    local action4 = FightActionPosition.new({ positionBegin = ccp(100, 100), positionEnd = ccp(200, 200), })
    print("================================")
    print("action3: ", action3:getPositionBegin().x)
    print("action4: ", action4:getPositionBegin().x)
    print("================================")
    --]]






    -- 2、Action序列[例：模拟动作]
    --[[
    local sprite1 = display.newSprite("img_photoCard_1.jpg")
    self:addChild(sprite1)
    sprite1:setPosition(ccp(display.cx, display.cy))

    local params1 = {
        node            = sprite1,
        positionBegin   = ccp(sprite1:getPosition()),
        positionEnd     = ccp(100, 100),
        delayTime       = 3.0,
        callback        = function(params) print(params) end,
        callbackParams  = { 1, 2 },
        positionRecovery= ccp(500, 200),
        aTime           = 2,
        isStopAllActions= false,
    }
    local actionPosition1 = requireA("FightActionPosition").new(params1)


    local sprite2 = display.newSprite("img_photoCard_1.jpg")
    self:addChild(sprite2)
    sprite2:setPosition(ccp(20, 20))

    local params2 = {
        node            = sprite2,
        positionBegin   = ccp(40, 40),
        positionEnd     = ccp(200, 200),
        delayTime       = 3.0,
        callback        = function(params) print(params) end,
        callbackParams  = { 3, 4 },
        positionRecovery= ccp(display.cx, display.cy),
        aTime           = 2,
        isStopAllActions= true,
    }
    local actionPosition2 = requireA("FightActionPosition").new(params2)


    actionPosition1:setSuccessor(actionPosition2)
    local actionStart = actionPosition1
    actionStart:run()
    --]]






    -- 3、Action序列[例：上场、下场]
    --[[
    local positions = {
        [1]         = ccp(100, 100),
        [2]         = ccp(300, 100),
        [11]        = ccp(100, 500),
        [12]        = ccp(300, 500),
    }
    local sprite = display.newSprite("img_photoCard_1.jpg")
    self:addChild(sprite)
    sprite:setPosition(ccp(display.cx, display.cy))


    local params = {
        node            = sprite,
        positions       = positions,
        indexBegin      = 2,
        indexEnd        = 12,
        delayTime       = 1.0,
        aTime           = 2,
        isStopAllActions= true,
    }
    local action = requireA("ActionStageMovement").new(params)
    action:run()
    ]]





    -- 4、精灵
    --[[
    local positions = {
        [1]         = ccp(100, 100),
        [2]         = ccp(300, 100),
        [11]        = ccp(100, 500),
        [12]        = ccp(300, 500),
    }
    local card1 = AbstractCard.new({ fileName = "img_photoCard_1.jpg", index = 1, positions = positions, owner = OWNER_ME })
    self:addChild(card1)
    card1:setPosition(ccp(100, 100))

    local card2 = AbstractCard.new({ fileName = "img_photoCard_2.jpg", index = 11, positions = positions, owner = OWNER_ME })
    self:addChild(card2)
    card2:setPosition(ccp(200, 200))

    print("position1: " .. card1:getPosition())
    print("position1: " .. card1:getPosition())
    --]]






    -- 5、Action[攻击]
    --[[
    local positions = {
        [1]         = ccp(100, 100),
        [2]         = ccp(300, 100),
        [11]        = ccp(100, 500),
        [12]        = ccp(300, 500),
    }
    local sprite = AbstractCard.new({ fileName = "img_photoCard_1.jpg", index = 2, positions = positions, owner = OWNER_ME })
    self:addChild(sprite)
    sprite:setPosition(ccp(display.cx, display.cy))

    local params = {
        node            = sprite,
        delayTime       = 1.0,
    }
    local action = requireA("ActionAtkNormal").new(params)
    action:run()
    --]]






    -- x、LocationCard
    --[[
    local positions = {
        [1]         = ccp(100, 100),
        [2]         = ccp(300, 100),
        [11]        = ccp(100, 500),
        [12]        = ccp(300, 500),
    }
    local sprite = LocationCard.new({ fileName = "img_photoCard_1.jpg", index = 2, positions = positions, owner = OWNER_OPPONENT })
    self:addChild(sprite)
    sprite:setPosition(ccp(display.cx, display.cy))

    print("index:", sprite:getIndex())
    --]]






    -- x、HeroCard
    --[[
    local po = poHeroMe()
    local sprite = CardFactory.newHero({ po = poHeroMe(), owner = OWNER_ME })
    self:addChild(sprite)
    sprite:setPosition(ccp(display.cx, display.cy))
    --]]






    -- x、IconCard
    --[[ 
    local positions = {
        [1]         = ccp(100, 100),
        [2]         = ccp(300, 100),
        [11]        = ccp(100, 500),
        [12]        = ccp(300, 500),
    }
    local pos = posCardMe()
    local sprite = CardFactory.newIcon({ po = pos[1], owner = OWNER_ME, positions = positions, index = 1, isPositionAutoCorrect = true, handler = nil })
    self:addChild(sprite)
    --]]






    -- x、ImageCard
    --[[
    local positions = {
        [1]         = ccp(100, 100),
        [2]         = ccp(300, 100),
        [11]        = ccp(100, 500),
        [12]        = ccp(300, 500),
    }
    local pos = posCardMe()
    local sprite = CardFactory.newImage({ po = pos[1], owner = OWNER_ME, positions = positions, index = 11, isPositionAutoCorrect = true, handler = nil })
    self:addChild(sprite)
    --]]
    






    -- x、StageDownCard
    --[[ 
    local positions = {
        [1]         = ccp(100, 100),
        [2]         = ccp(300, 100),
        [11]        = ccp(100, 500),
        [12]        = ccp(300, 500),
    }
    local pos = posCardMe()
    local sprite = CardFactory.newStageDown({ po = pos[1], owner = OWNER_ME, positions = positions, index = 12, isPositionAutoCorrect = true, handler = nil })
    self:addChild(sprite)
    --]]
    






    -- x、CutOff
    --[[
    -- 定义配置参数
    local fileName = "img_middleCard_1.jpg"
    local node = display.newSprite(fileName)
    self:addChild(node)
    node:setPosition(display.cx, display.cy)

    -- 生成精灵[左]
    local params = {
        fileName    = fileName,                                     -- 文件名
        parentNode  = node,                                         -- 父结点
        align       = display.CENTER_LEFT,                               -- 对齐方式
        anchor      = display.CENTER_LEFT,                          -- 锚点
    }
    local left = TDUIUtils.loadSprite(params)
    left:setColor(ccc3(255, 0, 0))
    -- 生成精灵[右]
    local params = {
        fileName    = fileName,                                     -- 文件名
        parentNode  = node,                                         -- 父结点
        align       = display.CENTER_RIGHT,                               -- 对齐方式
        anchor      = display.CENTER_LEFT,                          -- 锚点
    }
    local right = TDUIUtils.loadSprite(params)
    right:setColor(ccc3(0, 0, 255))

    local width, height = node:getContentSize().width, node:getContentSize().height
    left:setTextureRect(CCRect(0, 0, width / 2, height))
    right:setTextureRect(CCRect(width / 2, 0, width / 2, height))
    --]]
    






    -- x、结果页
    --[[
    local function onLayerLoaded(layer)
        self:addChild(layer, 100000)
    end
    CCBCenter.loadCCB("MSG_FIGHTRESULTLAYER_MODEL", "ccb.c.fight.FightResultLayer", {}, onLayerLoaded)
    --]]
    






    -- x、英雄[欢迎场景]
    --[[
    local po = {
        id          = 1,
        name        = "AC",                             -- 名字。
        head        = PHI_FIGHT .. "f_hero_tmp.png",     -- 头像。
    }
    local entity = EntityHero.new(po)
    local hero = WelcomeHero.new({ fileName = PHI_FIGHT .. "f_wlc_board_avatar.png", po = entity, owner = OWNER_ME, })
    self:addChild(hero, 100000)
    hero:setPosition(ccp(display.cx, display.cy))
    --]]
end



return M