--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

英雄血量类，定义游戏英雄血量相关操作方法及逻辑实现。

-   指定英雄血量值。

]]

local M = class("Blood", TDSprite)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 标签字体大小
local FZ_LABEL      = (13 * display.contentScaleFactor)
-- 偏移量[当前血量]
local O_HP_CURRENT  = ccp(-15, 0)
-- 偏移量[总血量]
local O_HP_TOTAL    = ccp(15, 0)
-- 动画次数[血]
local A_RUN_HP_TMS  = 20
-- 动画时间[血]
local A_RUN_HP_T    = .03

-- 操作变量
-- ..










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    self:assertParameters(params)



    -- [参数赋值]
    self._poHero            = params.poHero                             -- 英雄PO
    self._fileName          = params.fileName                           -- 文件名[图片]
    self._owner             = params.owner                              -- 拥有者标识
    self._label             = nil                                       -- 血量标签
    self._lineWidth         = 0                                         -- 血量线宽度
    self._lineHeight        = 0                                         -- 血量线高度


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载线条
    self:loadLine()
    -- 加载标签
    self:loadLabels()
    -- 加载闪光
    self:loadLight()

    -- 加载实体渲染其他操作
    self:loadRenderOtherwise()
end

-- 加载线条
function M:loadLine()
    local sprite = display.newSprite(PHI_FIGHT .. "f_blood_line.png")
    self:addChild(sprite, -1)

    -- 定位
    util.alignNode(sprite, self:getContentSize(), display.CENTER)
    sprite:setPosition(ccp(sprite:getPositionX(), sprite:getPositionY() - sprite:getContentSize().height / 2))
    sprite:setFlipY(true)
    display.align(sprite, display.BOTTOM_CENTER)

    -- 标记
    self.line = sprite
end

-- 加载标签
function M:loadLabels()
    -- 加载标签[当前血量]
    self:loadLabelHpCurrent()
end

-- 加载标签[当前血量]
function M:loadLabelHpCurrent()
    -- 定义配置参数
    local params = {
        text        = tostring(self:getHpTotal()),          -- 标签文字
        fontName    = FONT_1,                               -- 字体名称
        fontSize    = FZ_LABEL,                             -- 字体大小
        parentNode  = self,                                 -- 父结点
        align       = display.CENTER,                       -- 对齐方式
        anchor      = display.CENTER,                       -- 锚点
        mode        = TDUIUtils.MODE_LABEL_TTF_SHADOW,      -- 文字模式
    }
    -- 生成标签
    local label = TDUIUtils.loadLabelTTF(params)

    -- 标记
    self._labelHpCurrent = label
end

-- 加载光效
function M:loadLight()
    local sprite = display.newSprite(PHI_FIGHT .. "f_blood_light.png")
    self:addChild(sprite)

    -- 定位
    util.alignNode(sprite, self:getContentSize(), display.CENTER)

    -- 标记
    self.light = sprite
end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()
    -- 记录血条尺寸
    self:markLineSize()
    -- 隐藏光效
    self:hideLight()
end

-- 记录血条尺寸
function M:markLineSize()
    self._lineWidth  = self.line:getContentSize().width
    self._lineHeight = self.line:getContentSize().height
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()

end


---------


-- [实例方法][外部调用]
-- 获得血量
function M:getValue()
    if (self._labelHpCurrent) then
        return tonumber(self._labelHpCurrent.label:getString())
    end
    return 0
end

-- 设置血量
function M:setValue(hp)
    if (self._labelHpCurrent) then
        -- 设置文字 
        self._labelHpCurrent:setString(tostring(hp))

        -- 设置血条长度
        local line = self.line
        line:setTextureRect(CCRect(0, 0, self._lineWidth, hp / self:getHpTotal() * self._lineHeight))
    end
end

-- 设置血量[动画显示]
function M:animateHp(value)
    local current = self:getValue()
    local delta   = math.floor((value - current) / A_RUN_HP_TMS)

    local target  = self._labelHpCurrent
    target:stopAllActions()

    local actions = {}
    for i = 1, A_RUN_HP_TMS do
        if i ~= A_RUN_HP_TMS then 
            actions[#actions + 1] = self:actionRunToHp(current + delta * i)
        else
            actions[#actions + 1] = self:actionRunToHp(value)
            self:animateHpEnd()
        end
    end

    target:runAction(TDAnimationUtils.sequence(actions))
end

function M:actionRunToHp(value)
    local actions = {}

    actions[#actions + 1] = CCCallFunc:create(function() self:setValue(value) end)
    actions[#actions + 1] = CCDelayTime:create(A_RUN_HP_T)

    return TDAnimationUtils.sequence(actions)
end

function M:animateHpEnd()
    local target = self.light
    local time1  = FightActionHelper:time(.2)
    local time2  = FightActionHelper:time(.2)
    local opacity = 80

    local actions = {}
    -- 设置透明度
    actions[#actions + 1] = CCCallFunc:create(function() target:setOpacity(0) end)
    -- 设置可见性
    actions[#actions + 1] = CCShow:create()
    -- 动画闪烁
    actions[#actions + 1] = CCFadeTo:create(time1, opacity)
    -- 回调处理
    actions[#actions + 1] = CCFadeOut:create(time2)
    -- 设置可见性
    actions[#actions + 1] = CCHide:create()

    target:runAction(TDAnimationUtils.sequence(actions))
end

-- 获得血量[总值]
function M:getHpTotal()
    return self._poHero:getHp()
end

-- 获得血量[当前值]
function M:getHpCurrent()
    return self:getValue()
end

-- 获得拥有者标识
function M:getOwner()
    return self._owner
end


---------


-- [实例方法][显示隐藏]
-- 显示光效
function M:showLight()
    self.light:setVisible(true)
end

-- 隐藏光效
function M:hideLight()
    self.light:setVisible(false)
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数合法性(参数集)
    TDAssertUtils.assertParameters(params)
    -- 验证参数合法性(文件名)
    TDAssertUtils.assertTypeString(params.fileName)
    -- 验证参数合法性(血量)
    TDAssertUtils.assertNotNil(params.poHero:getHp())
end





return M