--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

墓地类，定义墓地相关操作方法及逻辑实现。

-   定义墓地方法及成员变量。

]]

local M = class("Cemetery", TDSprite)



-- [包含文件]
-- ..


-- [操作变量]
-- ..


-- [公共参数]
-- [常量]
-- 动画时间
M.O_EYE                         = ccp(0, 6)
-- 动画时间
M.A_GLITTER_T                   = 1.0

    

    





-- [构造方法]
-- 构造方法，定义视图加载逻辑
function M:ctor(params)
    -- [验证参数合法性]
    self:assertParameters(params)



    -- [参数赋值]
    -- [标识]
    self._owner         = params.owner  or OWNER_ME         -- 拥有者标识(我方/敌方)
    -- [文件名/帧]
    self._fileName      = params.fileName                   -- 文件名
    -- [动画]
    self._actionBoardGlitter    = nil
    self._actionEyeGlitter      = nil


    
    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载闪光[面板]
    self:loadBoardGlitter()
    -- 加载面板
    self:loadBoard()
    -- 加载闪光[眼睛]
    self:loadEyeGlitter()
    -- 加载眼睛
    self:loadEye()
    
    -- 加载实体渲染其他操作
    self:loadRenderOtherwise()
end

-- 加载闪光[面板]
function M:loadBoardGlitter()
    -- [生成对象]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT .. "f_cemetery_boardg.png",          -- 文件名
        parentNode  = self,                                         -- 父结点
        align       = display.CENTER,                               -- 对齐方式
        anchor      = display.CENTER,                               -- 锚点
        z           = 1,                                            -- 层深度
    }
    -- 生成精灵
    self._boardGlitter = TDUIUtils.loadSprite(params)
end

-- 加载面板
function M:loadBoard()
    -- [生成对象]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT .. "f_cemetery_board.png",           -- 文件名
        parentNode  = self,                                         -- 父结点
        align       = display.CENTER,                               -- 对齐方式
        anchor      = display.CENTER,                               -- 锚点
        z           = 2,                                            -- 层深度
    }
    -- 生成精灵
    self._board = TDUIUtils.loadSprite(params)
end

-- 加载闪光
function M:loadEyeGlitter()
    -- [生成对象]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT .. "f_cemetery_eyeg.png",            -- 文件名
        parentNode  = self,                                         -- 父结点
        align       = display.CENTER,                               -- 对齐方式
        anchor      = display.CENTER,                               -- 锚点
        z           = 3,                                            -- 层深度
        offset      = M.O_EYE,                                      -- 偏移量
    }
    -- 生成精灵
    self._eyeGlitter = TDUIUtils.loadSprite(params)
end

-- 加载眼睛
function M:loadEye()
    -- [生成对象]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT .. "f_cemetery_eye.png",             -- 文件名
        parentNode  = self,                                         -- 父结点
        align       = display.CENTER,                               -- 对齐方式
        anchor      = display.CENTER,                               -- 锚点
        z           = 4,                                            -- 层深度
        offset      = M.O_EYE,                                      -- 偏移量
    }
    -- 生成精灵
    self._eye = TDUIUtils.loadSprite(params)
end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()
    -- HOOK:
    self:setOpacity(0)
    self._boardGlitter:setVisible(false)
    self._eyeGlitter:setVisible(false)
    -- 自动动画
    -- self:autoAnimate()
    -- 预加载动画动作
    self:preloadActionsIfEmpty()
end

-- 预加载动画动作
function M:preloadActionsIfEmpty()
    if self._actionBoardGlitter == nil or self._actionEyeGlitter == nil then 
        self._actionBoardGlitter    = transition.sequence({
            CCShow:create(),
            CCFadeIn:create(M.A_GLITTER_T),                   -- Fade in 
            CCFadeOut:create(M.A_GLITTER_T),                  -- Fade out
            CCHide:create(),
        })
        self._actionEyeGlitter      = transition.sequence({
            CCShow:create(),
            CCFadeIn:create(M.A_GLITTER_T),                   -- Fade in 
            CCFadeOut:create(M.A_GLITTER_T),                  -- Fade out
            CCHide:create(),
        })
    end
end

-- 自动动画
function M:autoAnimate()
    local action = CCRepeatForever:create(transition.sequence({
        CCFadeIn:create(M.A_GLITTER_T),                   -- Fade in 
        CCFadeOut:create(M.A_GLITTER_T),                  -- Fade out
    }))

    self._boardGlitter:runAction(action)

    action = CCRepeatForever:create(transition.sequence({
        CCFadeIn:create(M.A_GLITTER_T),                   -- Fade in 
        CCFadeOut:create(M.A_GLITTER_T),                  -- Fade out
    }))

    self._eyeGlitter:runAction(action)
end


---------


-- [实例方法][显隐]
function M:animate()
    -- 预加载动画动作
    -- self:preloadActionsIfEmpty()

    -- print(self._actionBoardGlitter)
    self._boardGlitter:stopAllActions()
    self._boardGlitter:runAction(transition.sequence({
            CCShow:create(),
            CCFadeIn:create(M.A_GLITTER_T),                   -- Fade in 
            CCFadeOut:create(M.A_GLITTER_T),                  -- Fade out
            CCHide:create(),
        }))

    -- print(self._actionEyeGlitter)
    self._eyeGlitter:stopAllActions()
    self._eyeGlitter:runAction(transition.sequence({
            CCShow:create(),
            CCFadeIn:create(M.A_GLITTER_T),                   -- Fade in 
            CCFadeOut:create(M.A_GLITTER_T),                  -- Fade out
            CCHide:create(),
        }))
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()
    
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数集
    -- TDAssertUtils.assertParameters(params)
end


---------


-- [实例方法][属性Getter/Setter]
-- 设置拥有者标识
function M:setOwner(owner)
    self._owner = owner
end
-- 获得拥有者标识
function M:getOwner()
    return self._owner
end

-- 获得文件名
function M:getFileName()
    return self._fileName
end





return M