--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗CD类，定义游戏战斗CD相关操作方法及逻辑实现。

-   指定游戏战斗CD。
-   自动切换战斗CD。

- 示例：

local FightCD = require( PH_MDL_SPRITE .."FightCD")
local node = FightCD.new({ value = 2, fileName = PHI_FIGHT_CARD .. "f_c_cd_funnel.png", offsetCD = ccp(30, 0) })
self:addChild(node, 9999)
node:setPosition(ccp(300, 300))

]]

local M = class("FightCD", TDSprite)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 偏移量[CD]
local O_CD                      = ccp(24, 0)
-- 资源名称
local F_RESOURCE                = "f_c_cd_n"
-- 资源前缀
local F_PREFIX                  = "f_c_cd_n"

-- 操作变量
-- 断言
local TDAssertUtils             = require(PH_MDL_UTIL .. "TDAssertUtils")
-- 点控制
local TDPointUtils              = require(PH_MDL_UTIL .. "TDPointUtils")
-- 点控制
local TDUIUtils                 = require(PH_MDL_UTIL .. "TDUIUtils")

local LinearNum                 = require("cardui.LinearNum")










-- [辅助方法]










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    self:assertParameters(params)



    -- [参数赋值]
    self._value             = params.value              or 999              -- 编号
    self._fileName          = params.fileName                               -- 文件名[图片]
    self._fileNameBox       = params.fileNameBox                            -- 文件名[CD盒子]
    self._x, self._y        = params.x or 0, params.y   or 0                -- 坐标x, y


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载CD盒子
    self:loadCDBox()
    -- 加载CD值
    self:loadCD()
end

-- 加载CD盒子
function M:loadCDBox()
    -- [构建实体]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT_CARD .. "f_c_cd_box.png",               -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        offset      = O_CD,                                             -- 偏移量
        z           = -2,                                               -- 层深度
    }
    -- 生成精灵
    local node = TDUIUtils.loadSprite(params)

    -- 存储变量
    self._cdBox = node
end

-- 加载CD值
function M:loadCD()
    -- [构建实体]
    local node = LinearNum.new({
        resource               = PHI_FIGHT_NUM .. F_RESOURCE,           -- 资源名(不包含后缀名的前缀，如要缓存要引入num.plist和num.png，则resource为num)
        prefix                 = F_PREFIX,                              -- 数字及符号前缀
        value                  = self._value,                           -- 值
        isShowPlus             = false,                                 -- 显示＋标识
        isShowMinus            = false,                                 -- 显示－标识
        zeroSymbol             = false,                                 -- 数字0显示符号 可选值：+或-
        margin                 = ccp(0, 0),                             -- 项目间隔
        marginSymbol           = ccp(0, 0),
        align                  = LinearNum.ALIGN_CENTER,                -- 项目对齐方式
    })
    self:addChild(node, -1)
    TDUIUtils.alignNode(node, self, display.CENTER, display.CENTER, O_CD)

    -- 存储变量
    self._cd = node
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()

end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数合法性(参数集)
    TDAssertUtils.assertParameters(params)
    -- 验证参数合法性(文件名)
    TDAssertUtils.assertTypeString(params.fileName)
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得回合数
function M:getValue()
    if self._cd ~= nil then
        return tonumber(self._cd:getValue())
    end
    return 0
end

-- 设置回合数
function M:setValue(value)
    if self._cd ~= nil then
        self._cd:update({
            prefix = F_PREFIX,
            value  = value,
            align  = LinearNum.ALIGN_CENTER,
        })
    end
end





return M