--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗CD类，定义游戏战斗CD相关操作方法及逻辑实现。

-   指定游戏战斗CD。
-   自动切换战斗CD。

- 示例：

local FightTip = require( PH_MDL_SPRITE .."FightTip")
local node = FightTip.new({ text = "超级隐藏关卡001" })
self:addChild(node, 9999)
node:setPosition(ccp(300, 300))

]]

local M = class("FightTip", TDSprite)



-- [包含文件]
-- ...




-- [公共参数]
-- [常量]
local FN                        = FONT_1
local FZ                        = 35
local P_HIDE                    = ccp(-240, 595)
local P_SHOW                    = ccp(216, 595)
local O_LABEL                   = ccp(10, 20)

-- 操作变量
-- 动画
local TDAnimationUtils          = require(PH_MDL_UTIL .. "TDAnimationUtils")
-- 断言
local TDAssertUtils             = require(PH_MDL_UTIL .. "TDAssertUtils")
-- 点控制
local TDUIUtils                 = require(PH_MDL_UTIL .. "TDUIUtils")










-- [辅助方法]










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    self:assertParameters(params)



    -- [参数赋值]
    self._text              = params.text               or ""               -- 文本
    self._fileName          = params.fileName                               -- 文件名[图片]
    self._x, self._y        = params.x or 0, params.y   or 0                -- 坐标x, y


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载标签
    self:loadLabel()
    -- 加载粒子效果
    self:loadParticular()

    -- 加载实体渲染其他操作
    self:loadRenderOtherwise()
end

-- 加载标签
function M:loadLabel()
    -- [构建实体]
    -- 定义配置参数
    local params = {
        text        = self._text,                   -- 标签文字
        fontName    = FN,                           -- 字体名称
        fontSize    = FZ,                           -- 字体大小
        color       = ccc3(95, 22, 17),             -- 字体颜色
        parentNode  = self,                         -- 父结点
        align       = display.CENTER,               -- 对齐方式
        anchor      = display.CENTER,               -- 锚点
        offset      = O_LABEL,                      -- 偏移量
    }
    -- 生成标签
    local label = TDUIUtils.loadLabelTTF(params)

    -- 存储变量
    self._label = node
end

-- 加载粒子效果
function M:loadParticular()

end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()
    self:setPosition(P_HIDE)
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()

end


---------


-- [实例方法][外部调用]
-- 动画显示
function M:show()
    local node          = self
    local array         = {}
    local time1         = .2
    local time2         = 1
    local time3         = .3
    local position0     = P_HIDE
    local position1     = P_SHOW
    local position2     = P_HIDE
    local rate          = 8


    -- 初始设置
    node:setPosition(position0)
    -- 位置移动 
    array[#array + 1] = CCEaseBackOut:create(CCMoveTo:create(time1, position1))
    -- 处理回调函数
    array[#array + 1] = CCCallFunc:create(function() SOUND:playEffectFightUnlock(); self:onPlayParticular() end)
    -- 延迟
    array[#array + 1] = CCDelayTime:create(time2)
    -- 位置移动 
    array[#array + 1] = CCEaseIn:create(CCMoveTo:create(time3, position2), rate)

    node:runAction(TDAnimationUtils.sequence(array))
end

function M:playParticular()
    local emitter = CCParticleSystemQuad:create(PHI_FIGHT_RESULT .. "p_fight_tip.plist")
    local spriteBatch = CCParticleBatchNode:createWithTexture(emitter:getTexture())
    spriteBatch:addChild(emitter)
    self:addChild(spriteBatch, -1)
    spriteBatch:setPosition(ccp(0, 0))

    self._particles = spriteBatch
end
function M:stopParticular()
    self._particles:removeFromParentAndCleanup(true)
end

-- 播放粒子效果
function M:onPlayParticular()
    self:playParticular()
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数合法性(参数集)
    TDAssertUtils.assertParameters(params)
    -- 验证参数合法性(文件名)
    TDAssertUtils.assertTypeString(params.fileName)
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得文本
function M:getText()
    return self._text
end





return M