--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

位置卡片类，定义位置卡片公用操作方法及逻辑实现。

-   附加卡片位置信息。

]]

local M = class("HeroCard", AbstractCard)



-- [包含文件]
-- .. 




-- [公共参数]
-- 常量
-- 偏移量[名字]
local OF_NAME                   = ccp(0, 25)
-- 偏移量[等级]
local OF_LEVEL                  = ccp(-25, -20)

-- 字体尺寸[名字]
local FZ_NAME                   = (30 * display.contentScaleFactor)
-- 字体尺寸[等级]
local FZ_LEVEL                  = (24 * display.contentScaleFactor)

-- 字体颜色[名字]
local C_NAME                    = display.COLOR_WHITE
-- 字体颜色[等级]
local C_LEVEL                   = ccc3(255, 255, 0)

-- 枚举
M.OFFSET_OWNERS = {
    ccp(0, FZ_NAME),                -- TAG_OWNER_ME
    ccp(0, 0),                      -- TAG_OWNER_OPPONENT
}

-- 操作变量
-- ..










-- [实例方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    self._po                = params.po                                 -- PO对象
    self._isShowName        = params.isShowName         or false        -- 显示标识[姓名]
    self._isShowLevel       = params.isShowLevel        or false        -- 显示标识[等级]
    self._mode              = params.mode                               -- 战斗模式
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载头像
    self:loadAvatar()
    -- 加载标签
    self:loadLabels()

    -- 加载渲染以外的其他操作
    self:loadRenderOtherwise()
end

-- 加载头像
function M:loadAvatar()
    if self._mode.battle == MODE_BATTLE_PK and self._mode.pk == MODE_PK_FIRST_FIGHT then 
        if self._po:getOwner() == OWNER_ME then 
            -- 加维卡大叔
            local sprite = widget.newSprite{
                file = "fight/heads/fight_hero_head_uncle.png",
            }
            self:addChild(sprite)
            util.alignNode(sprite, self:getContentSize(), display.CENTER)
        else
            -- 加杀手同志
            local sprite = widget.newSprite{
                file = "help/h_head_demon_king.png",
            }
            sprite:setScale(0.6125)
            self:addChild(sprite)
            util.alignNode(sprite, self:getContentSize(), display.CENTER)
        end
        return
    elseif HelperView2.isNextIndex(640) then
        -- 新手引导到 1 - 1
        if self._po:getOwner() ~= OWNER_ME then 
            -- 加拦截者,我导弹呢
            local sprite = widget.newSprite{
                file = "fight/heads/fight_hero_head_enemy.png",
            }
            self:addChild(sprite)
            util.alignNode(sprite, self:getContentSize(), display.CENTER)
            self._po:setName(_s("HELPER_INFO_ROLE_ENEMIES"))
            return 
        end
    end
    -- [Version1]
    --[[
    local sprite = display.newSprite(self:getFileName())      -- TODO: self._po:getHead()
    self:addChild(sprite)

    -- 定位
    display.align(sprite, display.CENTER)
    util.alignNode(sprite, self:getContentSize(), display.CENTER)
    ]]

    -- [Version2]
    local fileName = self:getFileName()
    local fileNameRoleM = format("%sr_ico_%s.png", PHI_ROLE, "m")
    local fileNameRoleF = format("%sr_ico_%s.png", PHI_ROLE, "fm")
    if fileName == fileNameRoleM or fileName == fileNameRoleF then 
        -- [玩家]
        local sprite = nil
        local owner  = self._po:getOwner()
        local OFFSET = ccp(0, 0)
        if owner == OWNER_ME then 
            sprite = self._po:createPlayerHeadRectView()
            OFFSET = ccp(-5, -8)
        else 
            sprite = self._po:createPlayerHeadRectView()
            OFFSET = ccp(-5, -82)
        end
        sprite:setScale((self:getContentSize().width - 10) / sprite:getContentSize().width)
        self:addChild(sprite)
        -- sprite:setScale(1.2)

        -- 定位
        display.align(sprite, display.CENTER)
        util.alignNode(sprite, self:getContentSize(), display.CENTER)
        -- sprite:setPosition(ccp(sprite:getPositionX() + OFFSET.x, sprite:getPositionY() + OFFSET.y))
    else
        local sprite, file
        local levelId = GSVALUES.levelIdLast
        if levelId == "2-10" or levelId == "10-5" then 
            -- 莫卡
            file = "help/h_head_demon_general_m.png"
        elseif levelId == "4-10" or levelId == "10-6" then 
            -- 朱旺
            file = "help/h_head_demon_general_z.png"
        elseif levelId == "6-10" or levelId == "10-7" then 
            -- 毕卡斯
            file = "help/h_head_demon_general_b.png"
        elseif levelId == "8-10" or levelId == "10-8" then 
            -- 斯齐亚
            file = "help/h_head_demon_general_s.png"
        elseif levelId == "10-9" or levelId == "10-10" then 
            -- 魔君
            file = "help/h_head_demon_king.png"
        end

        if file then 
            sprite = display.newSprite(file)
            sprite:setScale(0.675)
        else
            -- [关卡]
            sprite = display.newSprite(self:getFileName())      -- TODO: self._po:getHead()
        end
        self:addChild(sprite)

        -- 定位
        display.align(sprite, display.CENTER)
        util.alignNode(sprite, self:getContentSize(), display.CENTER)
    end

    -- 标记
    self.avatar = sprite
end

-- 加载标签
function M:loadLabels()
    -- 加载标签[名字]
    self:loadLabelName()
    -- 加载标签[等级]
    self:loadLabelLevel()
end

-- 加载标签[名字]
function M:loadLabelName()
    -- 定义配置参数
    local params = {
        text        = self._po:getName(),           -- 标签文字
        fontSize    = FZ_NAME,                      -- 字体大小
        color       = C_NAME,                       -- 颜色
        parentNode  = self,                         -- 父结点
        align       = display.BOTTOM_CENTER,        -- 对齐方式
        offset      = OF_NAME,                      -- 偏移量
    }
    -- 生成标签
    local sprite = TDUIUtils.loadLabelTTF(params)

    -- 标记
    self.labelName = sprite
end
-- 加载标签[等级]
function M:loadLabelLevel()
    -- 定义配置参数
    local params = {
        text        = tostring(self._po:getLv()),   -- 标签文字
        fontSize    = FZ_LEVEL,                     -- 字体大小
        color       = C_LEVEL,                      -- 颜色
        parentNode  = self,                         -- 父结点
        align       = display.RIGHT_TOP,            -- 对齐方式
        offset      = OF_LEVEL,                     -- 偏移量
    }
    -- 生成标签
    local sprite = TDUIUtils.loadLabelTTF(params)

    -- 标记
    self.labelLevel = sprite
end

-- 加载渲染以外的其他操作
function M:loadRenderOtherwise()
    -- 设置可见性[名字]
    self:setVisibilityName()
    -- 设置可见性[等级]
    self:setVisibilityLevel()
end


-- [实例方法][内部调用]
-- 设置可见性[名字]
function M:setVisibilityName()
    self:setVisibility(self.labelName, self._isShowName)
end

-- 设置可见性[等级]
function M:setVisibilityLevel()
    self:setVisibility(self.labelLevel, self._isShowLevel)
end

-- 设置可见性
function M:setVisibility(node, isVisible)
    node:setVisible(isVisible)
end


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 调用超类验证逻辑
    self.super:assertParameters(params)

    -- 验证图像文件名
    TDAssertUtils.assertTypeString(params.fileName)
end


-- [实例方法][图片]
-- 获得图片
-- function M:getFileName()
--     local mode  = self._mode
--     local owner = self._po:getOwner()
--     local sex = self._po:getSex()
--     local idd = self._po:getId()

--     if mode.battle == MODE_BATTLE_LEVEL and owner ~= OWNER_ME then 
--         local entity = CardEntity.new({ id = idd })
--         return entity:getCardIcon()
--     else
--         return format("%sr_ico_%s.png", PHI_ROLE, (sex == GENDER_MALE) and "m" or "fm")
--     end
-- end

function M:getFileName()
    local mode  = self._mode
    local owner = self._po:getOwner()
    local sex = self._po:getSex()
    local idd = self._po:getId()
    if (mode.battle == MODE_BATTLE_LEVEL or mode.battle == MODE_BATTLE_BOSS or mode.battle == MODE_BATTLE_EXPFBFLIGHT or mode.battle == MODE_BATTLE_LADDERLIST or mode.battle == MODE_BATTLE_ARMY_BOSS) and owner ~= OWNER_ME then 
        -- local entity = CardEntity.new({ id = idd })
        -- return entity:getCardIcon()
        local head = self._po:getHead()
        if head and "" ~= head then 
            if string.endwith(head, "%.png") then 
                return head
            else
                return string.format("photoCard/img_photoCard_sp_%s.png", head)
            end
        else
            return string.format(CARD_ICON, sex)
        end
    else
        return format("%sr_ico_%s.png", PHI_ROLE, (sex == GENDER_MALE) and "m" or "fm")
    end
end


-- [实例方法][属性Getter/Setter]
-- 获得PO对象
function M:getPo()
    return self._po
end
-- 设置PO对象
function M:setPo(po)
    self._po = po
end

-- 获得显示标识[姓名]
function M:isShowName()
    return self._isShowName
end
-- 设置显示标识[姓名]
function M:setShowName(isShow)
    self._isShowName = isShow
end

-- 获得显示标识[等级]
function M:isShowLevel()
    return self._isShowLevel
end
-- 设置显示标识[等级]
function M:setShowLevel(isShow)
    self._isShowLevel = isShow
end





return M