--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

图标卡片类，定义卡片公用操作方法及逻辑实现。

-   扩展位置卡片。
-   附加公共图标卡片展示逻辑。

]]

local M = class("LocationCard", PrototypeCard)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 偏移量[名字]
local OF_D_ATTACK               = ccp(-5, 25)
local OF_D_HP                   = ccp(-5, 5)
local OF_D_WAITROUNDS           = ccp(5, -5)     -- ccp(5, 5)
local OF_U_ATTACK               = ccp(-5, -25)
local OF_U_HP                   = ccp(-5, -5)
local OF_U_WAITROUNDS           = ccp(5, 7)    -- ccp(5, -7)
local OF_DETAIL                 = ccp(5, -10)

-- 字体名称[名字]
local FN                        = FONT_1

-- 字体尺寸[名字]
local FZ_ATTACK                 = (15 * display.contentScaleFactor)
local FZ_HP                     = FZ_ATTACK
local FZ_WAITROUNDS             = (15 * display.contentScaleFactor)
local FZ_DETAIL                 = (15 * display.contentScaleFactor)

-- 字体颜色[名字]
local C_ATTACK                  = display.COLOR_WHITE
local C_HP                      = display.COLOR_WHITE
local C_WAITROUNDS              = display.COLOR_WHITE
local C_DETAIL                  = display.COLOR_WHITE

-- 层深度[框架]
local Z_FRAME                   = 10

-- 标签文本
local TXT_DETAIL                = "详细信息"

-- 操作变量
-- ..










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    -- [数据源]
    self._po                    = params.po                                     -- PO对象
    -- [显示文字]
    self._textDetail            = params.textDetail         or TXT_DETAIL       -- 文本.详细信息
    -- [文件名]
    self._fileNameBorder        = params.fileNameBorder                         -- 边框
    -- [侦听事件]
    self._filter                = params.filter             or nil              -- 点击事件过滤函数(当其返回值为true时才允许handler被调用)
    self._handler               = params.handler                                -- 点击事件处理函数
    self._handlerDetail         = params.handlerDetail                          -- 点击事件处理函数[详细信息]
    -- [其他]
    self._isShowWaitRounds      = params.isShowWaitRounds   or false            -- 是否显示等待回合
    self._isShowDetail          = params.isShowDetail       or false            -- 是否显示详细信息
    self._isShowBorder          = params.isShowBorder       or false            -- 是否显示边框



    -- [操作变量]
    self._waitRoundsTotal       = self._po:getWaitRounds()                      -- 回合数[总数]
    self._waitRoundsCurrent     = self._waitRoundsTotal                         -- 回合数[当前]


    -- [属性]
    local po                    = self._po                                      -- PO对象
    local name                  = po:getName()                                  -- 名称
 
    local icon                  = po:getIcon()                                  -- 图标
    local image                 = po:getImage()                                 -- 图片
    local star                  = po:getStar()                                  -- 星级
    local level                 = po:getLv()                                    -- 卡片等级
    local cost                  = po:getCost()                                  -- COST值（上场值）
    local ethnicity             = po:getEthnicity()                             -- 种族
    local waitRounds            = po:getWaitRounds()                            -- 等待回合数
    local hp                    = po:getHp()                                    -- 生命值HP
    local attack                = po:getAttack()                                -- 攻击力
    local defense               = po:getDefense()                               -- 防御力
    local skills                = po:getSkills()                                -- 技能集合ctor


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    -- TDExtendNode.extendRenderDestuctor(self)
end


 





-- [实例方法]
-- [实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载框架
    self:loadFrame()
    -- 加载标签
    self:loadLabels()
    -- 加载按钮
    self:loadButtons()
    -- 加载边框
    self:loadBorder()

    -- 加载渲染以外的其他操作
    self:loadRenderOtherwise()
end

-- 加载框架
function M:loadFrame()
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT_CARD .. "f_c_ico_box.png",          -- 文件名
        parentNode  = self,                                         -- 父结点
        align       = display.CENTER,                               -- 对齐方式
        anchor      = display.CENTER,                               -- 锚点
        z           = Z_FRAME,                                      -- 层深度
    }

    -- 生成精灵
    self._frame = TDUIUtils.loadSprite(params)
end

-- 加载标签
function M:loadLabels()
    -- 加载标签[攻击力]
    self:loadLabelAttack()
    -- 加载标签[血量]
    self:loadLabelHp()
    -- 加载标签[等待回合]
    self:loadLabelWaitRounds()
end

-- 加载标签[攻击力]
function M:loadLabelAttack()
    -- 参数赋值
    local owner     = self:getOwner()
    local align     = (owner == OWNER_ME) and display.RIGHT_TOP or display.RIGHT_BOTTOM
    local anchor    = display.RIGHT_CENTER
    local offset    = (owner == OWNER_ME) and OF_U_ATTACK       or OF_D_ATTACK

    -- 定义配置参数
    local params = {
        text        = "AT: " .. tostring(self._po:getAttack()),     -- 标签文字
        fontName    = FN,                                           -- 字体名称
        fontSize    = FZ_ATTACK,                                    -- 字体大小
        color       = C_ATTACK,                                     -- 颜色
        parentNode  = self,                                         -- 父结点
        align       = align,                                        -- 对齐方式
        offset      = offset,                                       -- 偏移量
        anchor      = anchor,                                       -- 锚点
        mode        = TDUIUtils.MODE_LABEL_TTF_OUTLINE,              -- 文字模式
    }

    -- 生成标签
    self._nodeAttack = TDUIUtils.loadLabelTTF(params)
end
-- 加载标签[血量]
function M:loadLabelHp()
    -- 参数赋值
    local owner     = self:getOwner()
    local align     = (owner == OWNER_ME) and display.RIGHT_TOP or display.RIGHT_BOTTOM
    local anchor    = display.RIGHT_CENTER
    local offset    = (owner == OWNER_ME) and OF_U_HP           or OF_D_HP

    -- 定义配置参数
    local params = {
        text        = "HP: " .. tostring(self._po:getHp()),         -- 标签文字
        fontName    = FN,                                           -- 字体名称
        fontSize    = FZ_HP,                                        -- 字体大小
        color       = C_HP,                                         -- 颜色
        parentNode  = self,                                         -- 父结点
        align       = align,                                        -- 对齐方式
        offset      = offset,                                       -- 偏移量
        anchor      = anchor,                                       -- 锚点
        mode        = TDUIUtils.MODE_LABEL_TTF_OUTLINE,              -- 文字模式
    }

    -- 生成标签
    self._nodeHp = TDUIUtils.loadLabelTTF(params)
end
-- 加载标签[等待回合]
function M:loadLabelWaitRounds()
    -- 参数赋值
    local owner     = self:getOwner()
    local align     = (owner == OWNER_ME) and display.LEFT_BOTTOM  or display.LEFT_TOP
    local anchor    = display.CENTER
    local offset    = (owner == OWNER_ME) and OF_U_WAITROUNDS   or OF_D_WAITROUNDS

    -- 定义配置参数
    local FightCD = require( PH_MDL_SPRITE .."FightCD")
    local node = FightCD.new({ value = self._waitRoundsTotal, fileName = PHI_FIGHT_CARD .. "f_c_cd_funnel.png" })
    self:addChild(node)
    TDUIUtils.alignNode(node, self, align, anchor, offset)

    -- 生成标签
    self._nodeWaitRounds = node
end

-- 加载按钮
function M:loadButtons()
    -- 加载按钮[详细信息]
    self:loadButtonDetail()
end

-- 加载按钮[详细信息]
function M:loadButtonDetail()
    -- 合法性判断 
    if not self._textDetail then
        return 
    end

    -- TODO: JUMP ALL CASE!
    if true then return false end

    -- 定义配置参数
    local params = {
        text        = self._textDetail,                             -- 按钮文字
        fontSize    = FZ_DETAIL,                                    -- 字体大小
        color       = C_DETAIL,                                     -- 颜色
        parentNode  = self,                                         -- 父结点
        align       = display.LEFT_TOP,                             -- 对齐方式
        offset      = OF_DETAIL,                                    -- 偏移量
        anchor      = display.CENTER,                               -- 锚点
    }

    -- 生成按钮
    self._buttonDetail = TDUIUtils.loadButton(params)
end

-- 加载边框
function M:loadBorder()
    -- 合法性判断 
    if not self._fileNameBorder then
        return 
    end

    -- 定义配置参数
    local params = {
        fileName    = self._fileNameBorder,                         -- 文件名
        parentNode  = self,                                         -- 父结点
        align       = display.CENTER,                               -- 对齐方式
        anchor      = display.CENTER,                               -- 锚点
    }

    -- 生成精灵
    self._border = TDUIUtils.loadSprite(params)
end

-- 加载渲染以外的其他操作
function M:loadRenderOtherwise()
    -- 进行位置校正
    self:correctPrototype()
    -- 设置可见性
    self:setVisiblities()
    -- 加载事件侦听器
    self:loadEventListener()
end

-- 设置可见性
function M:setVisiblities()
    -- 判断是否显示等待回合
    self:setVisiblityWaitRounds()
    -- 判断是否显示详细信息
    self:setVisiblityDetail()
    -- 判断是否显示边框
    self:setVisiblityBorder()
end

-- 判断是否显示等待回合
function M:setVisiblityWaitRounds()
    if self._nodeWaitRounds then
        self._nodeWaitRounds:setVisible(self._isShowWaitRounds)
    end
end

-- 判断是否显示详细信息
function M:setVisiblityDetail()
    if self._buttonDetail then 
        self._buttonDetail:setVisible(self._isShowDetail)
    end
end

-- 判断是否显示边框
function M:setVisiblityBorder()
    if self._border then 
        self._border:setVisible(self._isShowBorder)
    end
end

-- 加载事件侦听器
function M:loadEventListener()
    if self._handler then
        TDTouchUtils.bindNodeTouchEvent({ node = self, filter = self._filter, handler = self._handler })
    end
end


---------


-- [实例方法][外部调用]
-- 设置回合数
function M:setWaitRounds(no)
    self._waitRoundsCurrent = no
    self._nodeWaitRounds:setValue(no)
end

-- 递减回合数
function M:decreaseWaitRounds()
    local round = self._waitRoundsCurrent - 1
    self:setWaitRounds(round)
end

-- 获得回合数[当前]
function M:getWaitRoundsCurrent()
    return self._waitRoundsCurrent
end

-- 获得回合数[总数]
function M:getWaitRoundsTotal()
    return self._waitRoundsTotal
end

-- 隐藏回合数标签
function M:hideWaitRounds()
    return self._nodeWaitRounds:setVisible(false)
end

-- 显示回合数标签
function M:showWaitRounds()
    return self._nodeWaitRounds:setVisible(true)
end

-- 渲染当前状态
function M:render(params)
    -- Nothing to do, it will override by subclass
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得点击事件处理函数
function getHandler()
    return self._handler
end
-- 设置点击事件处理函数
function setHandler(handler)
    self._handler = handler
end

-- 获得点击事件过滤函数(当其返回值为true时才允许handler被调用)
function getFilter()
    return self._filter
end
-- 设置点击事件过滤函数(当其返回值为true时才允许handler被调用)
function setFilter(filter)
    self._filter = filter
end

-- 获得点击事件处理函数[详细信息]
function getHandlerDetail()
    return self._handlerDetail
end
-- 设置点击事件处理函数[详细信息]
function setHandlerDetail(handler)
    self._handlerDetail = handler
end

-- 获得是否显示等待回合标识
function isShowWaitRounds()
    return self._isShowWaitRounds
end
-- 设置是否显示等待回合标识
function setShowWaitRounds(isShow)
    self._isShowWaitRounds = isShow
end

-- 获得是否显示详细信息标识
function isShowDetail()
    return self._isShowDetail
end
-- 设置是否显示详细信息标识
function setShowDetail(isShow)
    self._isShowDetail = isShow
end

-- 获得是否显示边框标识
function isShowBorder()
    return self._isShowBorder
end
-- 设置是否显示边框标识
function setShowBorder(isShow)
    self._isShowBorder = isShow
end





return M