--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

图像卡片类，定义图像卡片公用操作方法及逻辑实现。

-   扩展位置卡片。
-   附加公共图像卡片展示逻辑。

]]

local M = class("ImageCard", PrototypeCard)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 偏移量
local OF_NAME                   = ccp(2, 9)
local OF_LEVEL                  = ccp(17.5, -18.5)
local OF_HP                     = ccp(-10, 10)
local OF_ATTACK                 = ccp(-10, 27)
local OF_WAITROUNDS             = ccp(5, 5)
local OF_DETAIL                 = ccp(5, -10)
local OF_NUM_NAME               = ccp(10, 8)
local OF_NUM_LEVEL              = ccp(12, -12)
local OF_NUM_HP                 = ccp(35, 8)
local OF_NUM_ATTACK             = ccp(-35, 8)
local OF_STAR                   = ccp(4, 50)
local M_LBL                     = 1
local M_STAR                    = ccp(0, 0)

-- 字体名称
local FN                        = FONT_1

-- 字体尺寸
local FZ_NAME                   = (15 * display.contentScaleFactor)
local FZ_LEVEL                  = (18 * display.contentScaleFactor)
local FZ_ATTACK                 = (13.5 * display.contentScaleFactor)
local FZ_HP                     = (13.5 * display.contentScaleFactor)
local FZ_WAITROUNDS             = (15 * display.contentScaleFactor)
local FZ_DETAIL                 = (15 * display.contentScaleFactor)

-- 字体颜色
local C_NAME                    = ccc3(255, 255, 255)
local C_LEVEL                   = ccc3(255, 255, 0)
local C_NUM_NAME                = ccc3(255, 255, 255)   -- ccc3(255, 255, 255)
local C_NUM_LEVEL               = ccc3(0, 0, 0)   -- ccc3(255, 255, 0)




local C_ATTACK                  = display.COLOR_WHITE   -- display.COLOR_WHITE
local C_HP                      = display.COLOR_WHITE   -- display.COLOR_WHITE
local C_WAITROUNDS              = ccc3(0, 0, 0)   -- display.COLOR_WHITE
local C_DETAIL                  = ccc3(0, 0, 0)   -- display.COLOR_WHITE

-- 标签文本
local TXT_DETAIL                = "详细信息"

-- 操作变量
-- ..










-- [辅助方法]










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    -- [数据源]
    self._po                    = params.po                                     -- PO对象
    -- [显示文字]
    self._textDetail            = params.textDetail         or TXT_DETAIL       -- 文本.详细信息
    -- [文件名]
    self._fileNameBorder        = params.fileNameBorder                         -- 边框
    -- [侦听事件]
    self._filter                = params.filter             or nil              -- 点击事件过滤函数(当其返回值为true时才允许handler被调用)
    self._handler               = params.handler                                -- 点击事件处理函数
    self._handlerDetail         = params.handlerDetail                          -- 点击事件处理函数[详细信息]
    -- [其他]
    self._isShowWaitRounds      = params.isShowWaitRounds   or false            -- 是否显示等待回合
    self._isShowDetail          = params.isShowDetail       or false            -- 是否显示详细信息
    self._isShowBorder          = params.isShowBorder       or false            -- 是否显示边框





    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    -- TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载图片
    self:loadImage()
    --加载进阶等级
    self:loadEvoLvImage()
    -- 加载标签
    self:loadLabels()
    -- 加载数字
    self:loadNums()

    -- 加载按钮
    self:loadButtons()
    -- 加载星星
    self:loadStar()
    -- 加载边框
    self:loadBorder()

    -- 加载渲染以外的其他操作
    self:loadRenderOtherwise()
end
local IMAGE_SIZE = {
    LARGE = "l",
    SMALL = "s",
    MIDDLE = "m",
}
function M:loadEvoLvImage( )


    local evolv = self._po:getEvolv()

    if evolv > 1 then
        evolv = evolv - 1
    
        local size =  IMAGE_SIZE.MIDDLE

        local numFomat = "#img_evo_num_%s_%s.png"
        local layer = widget.newLinearLayer()
        local evoStr = tostring(evolv)
        local numLen = string.len(evoStr)
        display.addSpriteFramesWithFile("card/img_evo_num.plist", "card/img_evo_num.png")
        local addSprite = widget.newSprite{file = string.format(numFomat, size, "plus"),}
        layer:addChild(addSprite)
        for i = 1, numLen do
            local num = string.byte(evoStr, i)
            local numSprite = widget.newSprite{file = string.format(numFomat, size, string.char(num)),}
            layer:addChild(numSprite)
        end
        layer:setCascadeColorEnabled(true)
        layer:setColor(ccc3(255, 255, 255))
        -- layer:setScale(0.6)
        util.alignNodeWithPadding(layer, self:getContentSize(), display.RIGHT_TOP, {top = 8, right = 4})
        self:addChild(layer,99)

        local evoBg = widget.newSprite{
            file = string.format("card/bg_card_evo_s_%d.png", evolv),
        }
        evoBg:setScale(0.8)
        util.alignNodeWithPadding(evoBg, self:getContentSize(), display.RIGHT_TOP, {top = 4, right = 4})
        self:addChild(evoBg,98)
    end
end


-- 加载图片
function M:loadImage()
    -- 定义配置参数
    local params = {
        fileName    = self._po:getImage(),                              -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        z           = -1,                                               -- 层深度
    }
    -- 生成精灵
    self._spriteImage = TDUIUtils.loadSprite(params)
end

-- 加载标签
function M:loadLabels()
    --背景
    self:loadLabelsBack()
    -- 加载标签[名称]
    -- self:loadLabelName()
    
    -- 加载标签[攻击力]
    self:loadLabelAttack()
    -- 加载标签[血量]
    self:loadLabelHp()

    -- 加载标签[测试]
    -- self:loadLabelTest()
end

--加载血量背景
function M:loadLabelsBack()
    -- 定义配置参数
   
    local params = {
        fileName    = "fight/card/f_c_img_bloodBack.png",                              -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.LEFT_BOTTOM,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        
        offset      = ccp(5,10),
    }
    local sprite = TDUIUtils.loadSprite(params)
    -- self:addChild(sprite)
end

-- 加载标签[名称]
function M:loadLabelName()
    -- 定义配置参数
    local params = {
        text        = tostring(self._po:getName()),             -- 标签文字
        fontName    = FONT_1,                                   -- 字体名称
        fontSize    = FZ_NAME,                                  -- 字体大小
        color       = C_NAME,                                   -- 颜色
        parentNode  = self,                                     -- 父结点
        align       = display.CENTER_TOP,                       -- 对齐方式
        offset      = OF_NAME,                                  -- 偏移量
        mode        = TDUIUtils.MODE_LABEL_TTF,          -- 文字模式
    }

    -- 生成标签
    self._labelName = TDUIUtils.loadLabelTTF(params)
end
-- 加载标签[等级]
function M:loadLabelLevel()
    -- 定义配置参数
    local params = {
        text        = "Lv: " .. tostring(self._po:getLv()),     -- 标签文字
        fontName    = FN,                                       -- 字体名称
        fontSize    = FZ_LEVEL,                                 -- 字体大小
        color       = C_NUM_LEVEL,                              -- 颜色
        parentNode  = self,                                     -- 父结点
        align       = display.BOTTOM_LEFT,                      -- 对齐方式
        offset      = OF_NUM_LEVEL,                             -- 偏移量
    }

    -- 生成标签
    self._labelLevel = TDUIUtils.loadLabelTTF(params)
end
-- 加载标签[攻击力]
function M:loadLabelAttack()
    -- [值]
    -- 定义配置参数
    local params = {
        text        = tostring(self._po:getAttack()),           -- 标签文字
        fontName    = FN,                                       -- 字体名称
        fontSize    = FZ_ATTACK,                                -- 字体大小
        color       = C_ATTACK,                                 -- 颜色
        parentNode  = self,                                     -- 父结点
        align       = display.RIGHT_BOTTOM,                     -- 对齐方式
        anchor      = display.RIGHT_CENTER,                     -- 锚点
        offset      = OF_ATTACK,                                -- 偏移量
        mode        = TDUIUtils.MODE_LABEL_TTF_OUTLINE,          -- 文字模式
    }

    -- 生成标签
    local value = TDUIUtils.loadLabelTTF(params)

    self._labelAttack = value
end
-- 加载标签[血量]
function M:loadLabelHp()
    -- 定义配置参数
    local params = {
        text        = tostring(self._po:getHp()),               -- 标签文字
        fontName    = FN,                                       -- 字体名称
        fontSize    = FZ_HP,                                    -- 字体大小
        color       = C_HP,                                     -- 颜色
        parentNode  = self,                                     -- 父结点
        align       = display.RIGHT_BOTTOM,                     -- 对齐方式
        anchor      = display.RIGHT_CENTER,                     -- 锚点
        offset      = OF_HP,                                    -- 偏移量
        mode        = TDUIUtils.MODE_LABEL_TTF_OUTLINE,          -- 文字模式
    }

    -- 生成标签
    local value = TDUIUtils.loadLabelTTF(params)

    self._labelHp = value
end

-- 加载标签[测试]
function M:loadLabelTest()
    -- [种族]
    -- 定义配置参数
    local params = {
        text        = "种族: " .. tostring(self._po:getEthnicityName()),    -- 标签文字
        fontSize    = FZ_HP,                                    -- 字体大小
        color       = C_HP,                                     -- 颜色
        parentNode  = self,                                     -- 父结点
        align       = display.CENTER,                           -- 对齐方式
        offset      = ccp(0, 55),                               -- 偏移量
    }

    -- 生成标签
    self._labelEthnicity = TDUIUtils.loadLabelTTF(params)

    -- [技能]
    -- 技能)
    local skills = self._po:getSkills()
    for i = 1, 3 do
        local name = ""
        local offset
        if      (i == 1) then
            name   = (skills.skill1 ~= nil) and skills.skill1:getName() or ""
            offset = ccp(0, 20) 
        elseif  (i == 2) then
            name   = (skills.skill2 ~= nil) and skills.skill2:getName() or ""
            offset = ccp(0, 0) 
        elseif  (i == 3) then
            name   = (skills.skill3 ~= nil) and skills.skill3:getName() or ""
            offset = ccp(0, -20) 
        end

        -- 定义配置参数
        local params = {
            text        = format("技能%d: ", i) .. name,             -- 标签文字
            fontSize    = FZ_HP,                                    -- 字体大小
            color       = C_HP,                                     -- 颜色
            parentNode  = self,                                     -- 父结点
            align       = display.CENTER,                           -- 对齐方式
            offset      = offset,                                   -- 偏移量
        }

        -- 生成标签
        self._labelEthnicity = TDUIUtils.loadLabelTTF(params)
    end
end

-- 加载数字
function M:loadNums()
    -- 加载数字[等级]
    self:loadNumLevel()
    -- 加载数字[攻击力]
    -- self:loadNumAttack()
    -- 加载数字[血量]
    -- self:loadNumHp()
    -- 加载数字背景
    self:loadNumsBack()
end

function M:loadNumsBack()
    -- 定义配置参数

    --介数
    local evolv = self._po:getEvolv() > 1 and 2 or 1

    local params = {
        fileName    = format("%sf_c_img_num%d%d.png", PHI_FIGHT_CARD, self._po:getEthnicity(),evolv),                              -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.LEFT_TOP,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        z           = 1,                                               -- 层深度
        offset      = ccp(-2,0),
    }

    local sprite = TDUIUtils.loadSprite(params)
    sprite:setScale(0.9)
    -- self:addChild(sprite)
end
-- 加载数字[等级]
function M:loadNumLevel()
    local node = LinearNum.new({
        resource               = PHI_FIGHT_NUM .. "f_c_cd_n",           -- 资源名(不包含后缀名的前缀，如要缓存要引入num.plist和num.png，则resource为num)
        prefix                 = "f_c_cd_n",                            -- 数字及符号前缀
        value                  = self._po:getLv(),                      -- 值
        isShowPlus             = false,                                 -- 显示＋标识
        isShowMinus            = false,                                 -- 显示－标识
        zeroSymbol             = false,                                 -- 数字0显示符号 可选值：+或-
        margin                 = ccp(-2, 0),                            -- 项目间隔
        marginSymbol           = ccp(10, 0),
        align                  = LinearNum.ALIGN_CENTER,                -- 项目对齐方式
    })
    node:setScale(0.6)
    self:addChild(node,2,2)

    --介数2 调整位置
    -- local evolv = self._po:getEvolv() > 2 and 2 or self._po:getEvolv()
    -- if evolv == 2 then
    --     OF_LEVEL = ccp(22,-25)
    -- end
    

    TDUIUtils.alignNode(node, self, display.LEFT_TOP, display.CENTER, ccp(23,-18))
end

-- 加载数字[攻击力]
function M:loadNumAttack()
    local node = LinearNum.new({
        resource               = PHI_FIGHT_NUM .. "f_c_v",              -- 资源名(不包含后缀名的前缀，如要缓存要引入num.plist和num.png，则resource为num)
        prefix                 = "f_c_vw",                              -- 数字及符号前缀
        value                  = self._po:getAttack(),                  -- 值
        isShowPlus             = false,                                 -- 显示＋标识
        isShowMinus            = false,                                 -- 显示－标识
        zeroSymbol             = false,                                 -- 数字0显示符号 可选值：+或-
        margin                 = ccp(0, 0),                             -- 项目间隔
        marginSymbol           = ccp(10, 0),
        align                  = LinearNum.ALIGN_LEFT,                  -- 项目对齐方式
    })
    self:addChild(node)
    TDUIUtils.alignNode(node, self, display.BOTTOM_CENTER, display.CENTER, OF_NUM_ATTACK)

    -- 存储结点
    self._numAttack = node
end

-- 加载数字[血量]
function M:loadNumHp()
    local node = LinearNum.new({
        resource               = PHI_FIGHT_NUM .. "f_c_v",              -- 资源名(不包含后缀名的前缀，如要缓存要引入num.plist和num.png，则resource为num)
        prefix                 = "f_c_vw",                              -- 数字及符号前缀
        value                  = self._po:getHp(),                      -- 值
        isShowPlus             = false,                                 -- 显示＋标识
        isShowMinus            = false,                                 -- 显示－标识
        zeroSymbol             = false,                                 -- 数字0显示符号 可选值：+或-
        margin                 = ccp(0, 0),                             -- 项目间隔
        marginSymbol           = ccp(10, 0),
        align                  = LinearNum.ALIGN_RIGHT,                 -- 项目对齐方式
    })
    self:addChild(node)
    TDUIUtils.alignNode(node, self, display.BOTTOM_CENTER, display.CENTER, OF_NUM_HP)

    -- 存储结点
    self._numHp = node
end

-- 加载按钮
function M:loadButtons()
    -- 加载按钮[详细信息]
    self:loadButtonDetail()
end

-- 加载按钮[详细信息]
function M:loadButtonDetail()
    -- 合法性判断 
    if not self._textDetail then
        return 
    end

    -- TODO: JUMP ALL CASE!
    if true then return false end

    -- 定义配置参数
    local params = {
        text        = self._textDetail,                             -- 按钮文字
        fontSize    = FZ_DETAIL,                                    -- 字体大小
        color       = C_DETAIL,                                     -- 颜色
        parentNode  = self,                                         -- 父结点
        align       = display.LEFT_TOP,                             -- 对齐方式
        offset      = OF_DETAIL,                                    -- 偏移量
        anchor      = display.CENTER,                               -- 锚点
    }

    -- 生成按钮
    self._buttonDetail = TDUIUtils.loadButton(params)
end

-- 加载星星
function M:loadStar()
    local sprite = LinearStar.new({ fileName = PHI_FIGHT_CARD .. "f_c_star.png", count = self._po:getStar(), margin = M_STAR, })
    self:addChild(sprite)
    TDUIUtils.alignNode(sprite, self, display.LEFT_BOTTOM, display.CENTER, OF_STAR)
end

-- 加载边框
function M:loadBorder()
    -- 合法性判断 
    if not self._fileNameBorder then
        return 
    end

    -- 定义配置参数
    local params = {
        fileName    = self._fileNameBorder,                         -- 文件名
        parentNode  = self,                                         -- 父结点
        align       = display.CENTER,                               -- 对齐方式
        anchor      = display.CENTER,                               -- 锚点
    }

    -- 生成精灵
    self._border = TDUIUtils.loadSprite(params)
end

-- 加载渲染以外的其他操作
function M:loadRenderOtherwise()
    -- 进行位置校正
    self:correctPrototype()
    -- 设置可见性
    self:setVisiblities()
    -- 加载事件侦听器
    self:loadEventListener()
end

-- 设置可见性
function M:setVisiblities()
    -- 判断是否显示等待回合
    self:setVisiblityWaitRounds()
    -- 判断是否显示详细信息
    self:setVisiblityDetail()
    -- 判断是否显示边框
    self:setVisiblityBorder()
end

-- 判断是否显示等待回合
function M:setVisiblityWaitRounds()
    if self._labelWaitRounds then
        self._labelWaitRounds:setVisible(self._isShowWaitRounds)
    end
end

-- 判断是否显示详细信息
function M:setVisiblityDetail()
    if self._buttonDetail then 
        self._buttonDetail:setVisible(self._isShowDetail)
    end
end

-- 判断是否显示边框
function M:setVisiblityBorder()
    if self._border then 
        self._border:setVisible(self._isShowBorder)
    end
end

-- 加载事件侦听器
function M:loadEventListener()
    if self._handler then
        TDTouchUtils.bindNodeTouchEvent({ node = self, filter = self._filter, handler = self._handler })
    end
end


---------


-- [外部调用]
-- 获得点击事件处理函数
function getHandler()
    return self._handler
end
-- 设置点击事件处理函数
function setHandler(handler)
    self._handler = handler
end

-- 获得点击事件过滤函数(当其返回值为true时才允许handler被调用)
function getFilter()
    return self._filter
end
-- 设置点击事件过滤函数(当其返回值为true时才允许handler被调用)
function setFilter(filter)
    self._filter = filter
end

-- 获得点击事件处理函数[详细信息]
function getHandlerDetail()
    return self._handlerDetail
end
-- 设置点击事件处理函数[详细信息]
function setHandlerDetail(handler)
    self._handlerDetail = handler
end

-- 获得是否显示等待回合标识
function isShowWaitRounds()
    return self._isShowWaitRounds
end
-- 设置是否显示等待回合标识
function setShowWaitRounds(isShow)
    self._isShowWaitRounds = isShow
end

-- 获得是否显示详细信息标识
function isShowDetail()
    return self._isShowDetail
end
-- 设置是否显示详细信息标识
function setShowDetail(isShow)
    self._isShowDetail = isShow
end

-- 获得是否显示边框标识
function isShowBorder()
    return self._isShowBorder
end
-- 设置是否显示边框标识
function setShowBorder(isShow)
    self._isShowBorder = isShow
end

-- 获得HP
function M:getHp()
    return tonumber(self._labelHp:getString())
end
-- 设置HP
function M:setHp(hp)
    self._labelHp:setString(hp)
end

-- 获得Po
function M:getPo()
    return self._po
end
-- 设置Po
function M:setPo(po)
    self._po = po
end





return M