--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

位置卡片类，定义位置卡片公用操作方法及逻辑实现。

-   附加卡片位置信息。

]]

local M = class("LocationCard", AbstractCard)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 标签字体大小
local CARD_A_CORRECT_TIME           = 1.0

-- 操作变量
-- ..










-- [辅助方法]










-- [实例方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- 加载父类的构造方法
    M.super.ctor(self, params)



    -- 提取配置参数
    -- [位置]
    self._positions                 = params.positions                  or {}               -- 位置集
    self._index                     = params.index                      or 0                -- 位置索引
    self._isPositionAutoCorrect     = params.isPositionAutoCorrect      or false            -- 自动校正原型(如果自动校正，则设置_index时会自动将位置、旋转角度、缩放比例等定位到指定索引位置)
    self._isPositionCorrectAnimate  = params.isPositionCorrectAnimate   or false            -- 是否校正位置时动画过渡
end










-- [实例方法]
-- [实例方法][外部调用]
-- 获得期待位置
function M:getPositionExpect()
    -- 验证合法性
    M:assertLocation()

    -- 获得位置
    return self._positions[self._index]
end

-- 获得实际位置
function M:getPositionActual()
    return self:getPosition()
end

-- 校正位置
function M:correctPosition(isAnimate)
    -- 是否动画过渡
    isAnimate = self._isPositionCorrectAnimate or false
    -- 目标位置
    local positionTarget = self:getPositionExpect()

    -- 处理逻辑
    if not isAnimate then
        self:setPosition(positionTarget)
    else
        local action = transition.moveTo(sprite, {time = CARD_A_CORRECT_TIME, x = positionTarget.x, y = positionTarget.y})
        self:stopAllActions()
        self:runAction(action)
    end
end

-- 设置位置索引
function M:setIndex(index)
    -- 设置索引
    self._index = index
    
    -- 是否进行位置自动校正
    if self:isPositionAutoCorrect() then
        self:setPosition(self:getPositionExpect())
    end
end


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 调用超类验证逻辑
    self.super:assertParameters(params)

    -- 验证位置
    self:assertLocation(params)
end

-- 验证合法性[位置]
function M:assertLocation(params)
    -- 默认赋值
    local args = params or { positions = self._positions, index = self._index, }

    -- 验证合法性
    TDAssertUtils.assertNotNil(args.positions)
    TDAssertUtils.assert(args.index >= 0, "Card index must be greater than or equal to zero!")
end


-- [实例方法][属性Getter/Setter]
-- 设置位置集
function M:setPositions(positions)
    self._positions = positions
end
-- 获得位置集
function M:getPositions()
    return self._positions
end

-- 获得位置索引
function M:getIndex()
    return self._index
end

-- 设置是否自动校正位置
function M:setPositionAutoCorrect(isAutoCorrect)
    self._isPositionAutoCorrect = isAutoCorrect
end
-- 获得是否自动校正位置
function M:isPositionAutoCorrect()
    return self._isPositionAutoCorrect
end

-- 设置是否校正位置时动画过渡
function M:setPositionCorrectAnimate(isAutoCorrect)
    self._isPositionCorrectAnimate = isAutoCorrect
end
-- 获得是否校正位置时动画过渡
function M:isPositionCorrectAnimate()
    return self._isPositionCorrectAnimate
end





return M