--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

原型卡牌类，定义原型卡牌公用操作方法及逻辑实现。

-   附加卡牌位置、旋转角度、缩放比例等相关信息。
-   根据判定是否自动进行原型信息的注入操作。

]]

local M = class("PrototypeCard", AbstractCard)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- ..

-- 操作变量
-- ..










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    -- [位置]
    self._prototypes                = params.prototypes                 or {}               -- 原型集
    self._index                     = params.index                      or 0                -- 位置索引
    self._indexActual               = 0                                                     -- 位置索引［实际］
    self._isPrototypeAutoCorrect    = params.isPrototypeAutoCorrect     or false            -- 自动校正原型信息(如果自动校正，则设置_index时会自动将位置定位到指定索引位置)
    self._isPrototypeCorrectAnimate = params.isPrototypeCorrectAnimate  or false            -- 是否校正原型信息时动画过渡
end










-- [实例方法]
-- [实例方法][外部调用]
-- 获得期待原型信息
function M:getPrototypeExpect()
    -- 验证合法性
    self:assertPrototypes()
    -- 获得原型信息
    return self._prototypes:getPrototype(self._indexActual)
end

-- 获得期待位置
function M:getPositionExpect()
    return self:getPrototypeExpect():getPosition()
end
-- 获得期待旋转角度
function M:getRotationExpect()
    return self:getPrototypeExpect():getRotation()
end
-- 获得期待缩放比例
function M:getScaleExpect()
    return self:getPrototypeExpect():getScale()
end

-- 获得期待位置
function M:getPositionExpect()
    return self:getPrototypeExpect():getPosition()
end
-- 获得实际位置
function M:getPositionActual()
    return self:getPosition()
end
-- 获得实际旋转角度
function M:getRotationActual()
    return self:getRotation()
end
-- 获得实际缩放比例
function M:getScaleActual()
    return self:getScale()
end

-- 校正位置
function M:correctPrototype(isAnimate)
    -- 判断是否自动校正位置
    if self:isPrototypeAutoCorrect() then
        -- 过滤判断
        if self:getIndexActual() % IDX_STAGEUP_INC > NUM_FIGHT_VIEW then 
            local action = TDAnimationUtils.spawn({
                CCHide:create(),       -- hide
            })
            self:stopAllActions()
            self:runAction(action)
            return
        end

        -- 是否动画过渡
        isAnimate = self._isPrototypeCorrectAnimate or false
        -- 目标位置
        local positionTarget = self:getPositionExpect()
        -- 目标旋转角度
        local rotationTarget = self:getRotationExpect()
        -- 目标缩放比例
        local scaleTarget = self:getScaleExpect()

        -- 处理逻辑
        if not isAnimate then
            self:setPosition(positionTarget)
            self:setRotation(rotationTarget)
            self:setScale(scaleTarget)
        else
            local action = TDAnimationUtils.spawn({
                CCShow:create(),                                            -- show
                CCMoveTo:create(CARD_A_CORRECT_TIME, positionTarget),       -- moving
                CCRotateTo:create(CARD_A_CORRECT_TIME, rotationTarget),     -- rotating
                CCScaleTo:create(CARD_A_CORRECT_TIME, scaleTarget),         -- scaling
            })
            self:stopAllActions()
            self:runAction(action)
        end
    end
end

-- 设置位置索引
function M:setIndex(index)
    -- print("设置索引"..index)
    -- 设置索引
    self._index = index
end
-- 设置位置索引[pubf]
function M:setIndexActual(index)
    -- print("设置setIndexActual"..index)
    self._indexActual = index
    -- 进行位置校正
    -- self:correctPrototype()
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 调用超类验证逻辑
    M.super.assertParameters(self, params)

    -- 验证合法性[原型集]
    self:assertPrototypes(params)
    -- 验证合法性[索引]
    self:assertIndex(params)
end

-- 验证合法性[原型集]
function M:assertPrototypes(params)
    -- 默认赋值
    local args = params or { prototypes = self._prototypes, }
    -- 验证
    TDAssertUtils.assertNotNil(args.prototypes)
end

-- 验证合法性[索引]
function M:assertIndex(params)
    -- 验证
    TDAssertUtils.assert(params.index >= 0, "Card index must be greater than or equal to zero!")
end


---------


-- [实例方法][属性Getter/Setter]
-- 设置原型集
function M:setPrototypes(prototypes)
    self._prototypes = prototypes
end
-- 获得原型集
function M:getPrototypes()
    return self._prototypes
end

-- 获得位置索引
function M:getIndex()
    -- print("获得索引"..self._index)
    return self._index
end
-- 获得位置索引
function M:getIndexActual()
    return self._indexActual
end

-- 设置是否自动校正原型
function M:setPrototypeAutoCorrect(isAutoCorrect)
    self._isPrototypeAutoCorrect = isAutoCorrect
    self:correctPrototype()
end
-- 获得是否自动校正原型
function M:isPrototypeAutoCorrect()
    return self._isPrototypeAutoCorrect
end

-- 设置是否校正原型时动画过渡
function M:setPrototypeCorrectAnimate(isAutoCorrect)
    self._isPrototypeCorrectAnimate = isAutoCorrect
end
-- 获得是否校正原型时动画过渡
function M:isPrototypeCorrectAnimate()
    return self._isPrototypeCorrectAnimate
end





return M