--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

游戏回合类，定义游戏回合相关操作方法及逻辑实现。

-   指定游戏回合数。
-   自动切换回合。

]]

local M = class("Round", TDSprite)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 回合数起始索引
local AC_ROUND_START            = 1
-- 字体大小[标签文字]
local FZ_LABEL_TEXT             = (22 * display.contentScaleFactor)
-- 字体大小[标签值]
local FZ_LABEL_VALUE            = (40 * display.contentScaleFactor)
-- 字体颜色[标签文字]
local C_LABEL_TEXT              = ccc3(162, 94, 37)
-- 字体颜色[标签值]
local C_LABEL_VALUE             = ccc3(162, 94, 37)
-- 偏移量[标签文字]
local O_LABEL_TEXT              = ccp(40, 0)
-- 偏移量[标签值]
local O_LABEL_VALUE             = ccp(-10, 0)

-- 操作变量
-- 断言
local TDAssertUtils             = requireU("TDAssertUtils");
-- 点控制
local TDPointUtils              = requireU("TDPointUtils");










-- [辅助方法]










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    self:assertParameters(params)



    -- [参数赋值]
    self._no                = params.no         or AC_ROUND_START           -- 编号
    self._fileName          = params.fileName                               -- 文件名[图片]
    self._x, self._y        = params.x or 0, params.y or 0                  -- 坐标x, y


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载标签
    self:loadLabels()
end

-- 加载标签
function M:loadLabels()
    -- 加载标签[文字]
    self:loadLabelText()
    -- 加载标签[值]
    self:loadLabelValue()
end

-- 加载标签[文字]
function M:loadLabelText()
    -- 创建回合数
    local label = ui.newTTFLabel({
        text = _s("FIGHT_ROUND"),
        size = FZ_LABEL_TEXT,
        color = C_LABEL_TEXT,
        font  = FONT_1,
    })
    -- 定位
    util.alignNode(label, self:getContentSize(), display.CENTER);
    label:setPosition(ccp(label:getPositionX() + O_LABEL_TEXT.x, label:getPositionY() + O_LABEL_TEXT.y));

    self:addChild(label)


    -- 标记
    self.label = label
end

-- 加载标签[值]
function M:loadLabelValue()
    -- 创建回合数
    local label = ui.newTTFLabel({
        text = tostring(self._no),
        size = FZ_LABEL_VALUE,
        color = C_LABEL_VALUE,
        font  = FONT_1,
    })
    -- 定位
    util.alignNode(label, self:getContentSize(), display.CENTER);
    label:setPosition(ccp(label:getPositionX() + O_LABEL_VALUE.x, label:getPositionY() + O_LABEL_VALUE.y));

    self:addChild(label)


    -- 标记
    self.label = label
end

-- 获得顶层视图
function M:getTopView()
    return self:getParent():getParent()
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()

end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数合法性(参数集)
    TDAssertUtils.assertParameters(params)
    -- 验证参数合法性(文件名)
    TDAssertUtils.assertTypeString(params.fileName)
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得回合数
function M:getNo()
    if self.label ~= nil then
        return tonumber(self.label:getString())
    end
    return 0
end

-- 设置回合数
function M:setNo(index)
    if self.label ~= nil then
        self.label:setString(tostring(index))
    end
end

-- 设置回合标签
function M:getLabel()
    return self.label
end





return M