--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

下场卡片类，定义卡片公用操作方法及逻辑实现。

-   扩展位置卡片。
-   附加战斗下场卡片展示逻辑。

]]

local M = class("StageDownCard", IconCard)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 就绪渐显渐隐时间
local T_READY_FADE 		= 1.0

-- 操作变量
-- ..










-- [辅助方法]










-- [实例方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    -- [卡牌]
    self._associatedCard 	= nil											                     -- 关联的上场卡牌

    -- [侦听事件]
    self._filter            = self.onFilter                                                      -- 过滤事件
    self._handler           = self.onTouch                                                       -- 侦听事件

    -- [其他]
    self._isShowWaitRounds  = true                                                               -- 是否显示等待回合
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
	-- 加载就绪盒子
	self:loadBoxReady()
    -- 加载精灵配置
    self:loadSpriteConfig()

    -- [加载超类方法]
	M.super.entityRender(self)
end

-- 加载就绪盒子
function M:loadBoxReady()
    -- 定义配置参数
    local params = {
        fileName    = TDAnimationUtils.frameNameAtFirst("#f_c_ready"),  -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        z 			= -1, 											    -- 层深度
    }

    -- 生成精灵
    self._boxReady = TDUIUtils.loadSprite(params)
end

-- 加载精灵配置
function M:loadSpriteConfig()
    -- 抗锯齿
    self:getTexture():setAntiAliasTexParameters()
    -- 不抗锯齿
    -- self:getTexture():setAliasTexParameters()
end

-- 加载渲染以外的其他操作
function M:loadRenderOtherwise()
    -- 隐藏就绪盒子
    self:hideBoxReady()

    -- [加载超类方法]
	M.super.loadRenderOtherwise(self)
end


---------


-- [实例方法][外部调用]
-- 卡牌上场 
function M:stageUp()
    -- 获得顶层视图
    local view  = self:getTopView()
    -- 获得我方回合位置
    local index = self:getStageUpPosition()

    -- 显示上场卡牌
    self:getAssociatedCard():setVisible(true)

    local params = {
        node            = self,
        prototypes      = self:getPrototypes(),
        index           = index,
        sn              = self._po:getSn(),
        action          = ActionEnum.STAGE_UP,
        varHp           = varHp,
        varAtk          = varAtk,
        curHp           = curHp,
        curAtk          = curAtk,
        isKeepNodeDown  = true,
        callback        = function()
            -- 添加上场卡牌
            view:addToCardsNet(self)
            -- 更新按钮状态（开始战斗/回合结束）
            view:setButtonStatusInteractive()
        end,
    }
    requireA("ActionStageUp").new(params):run()
end

-- 卡牌下场 
function M:stageDown()
    -- 获得顶层视图
    local view  = self:getTopView()
    -- 获得我方回合位置
    local index = self:getStageDownPosition()

    -- 更新按钮状态（开始战斗/回合结束）
    view:setButtonStatusInteractive()

    local params = {
        node            = self,
        prototypes      = self:getPrototypes(),
        index           = index,
        sn              = self._po:getSn(),
        action          = ActionEnum.STAGE_DOWN,
        varHp           = varHp,
        varAtk          = varAtk,
        curHp           = curHp,
        curAtk          = curAtk,
        callback        = function()
            -- 移除上场卡牌
            view:removeFromCardsNet(self)
            view:resortCardsUp(WONER_ME)
            view:resortCardsUp()
            -- 更新按钮状态（开始战斗/回合结束）
            view:setButtonStatusInteractive()
        end,
    }
    requireA("ActionStageDown").new(params):run()
end

-- 获得上场索引位置
function M:getStageUpPosition()
    -- 获得顶层视图
    local view  = self:getTopView()
    -- 获得索引位置
    return TDArrayUtils.count(view:getCardsUp(OWNER_ME)) + IDX_STAGEUP_INC + 1
end

-- 获得下场索引位置
function M:getStageDownPosition()
    local position

    -- [Version1]
    --[[
    -- 获得顶层视图
    local view  = self:getTopView()
    local dataFight = view:getData():getFight()
    local dataFightLastRound = dataFight[#dataFight]

    local snMe    = self._po:getSn()
    local poCards = dataFightLastRound:getFme()
    for _, poCard in pairs(poCards) do
        local sn       = poCard:getSn()

        if sn == snMe then 
            position = poCard:getPo()
            break
        end
    end
    --]]

    -- [Version2]
    position = self:getIndex()

    return position
end

-- 获得上场索引位置
function M:getPrototypes()
    return self:getTopView():getPlayerPrototypes(self._owner)
end

-- 渲染当前状态
function M:render(params)
    -- [逻辑过滤]
    if not self:isVisible() or self:isDie() then
        return
    end


    -- [参数赋值]
    local isShowBoxReady        = params.isShowBoxReady or false                        -- 是否显示就绪盒子
    local waitRounds            = params.waitRounds     or self:getWaitRoundsCurrent()  -- 当前等待回合数
    local topView               = self:getTopView()                                     -- 获得顶层视图

    -- [逻辑]
    -- 重置回合数
    self:setWaitRounds(waitRounds)
    -- 处理回合数＝0
    if self:getWaitRoundsCurrent() == 0 then 
        -- 判断是否显示就绪盒子
        if isShowBoxReady then
            -- 隐藏回合数标签
            self:hideWaitRounds()
            -- 显示就绪盒子
            self:showBoxReady()
            -- 设置按钮[回合结束]状态可用
            if self._owner == OWNER_ME then
                topView:setButtonStatusInteractive()
                topView:helpJudgeStageDownCardRender()
            end
        else
            -- 显示回合数标签
            self:showWaitRounds()
        end
    end
end

-- 获得颜色[活]
function M:getColorAlive()
    return ccc3(255, 255, 255)
end

-- 获得颜色[活]
function M:getColorStateUp()
    return ccc3(140, 140, 140)
end

-- 获得颜色[死]
function M:getColorDie()
    return ccc3(100, 100, 100)
end

-- 获得位置[墓地]
function M:getPositionCemetery()
    TDAssertUtils.assertNotNil(self._positions, "Positions is nil!")
    return self._positions[IDX_CEMETERY]
end

-- 重置状态
function M:reset()
    -- print("StageDownCard:reset()")

    -- 重置状态[颜色]
    self:resetColor()
    -- 重置状态[CD]
    self:resetCD()
    -- 重置状态[数值]
    self:resetValue()

    -- 重置状态[清除所有动作]
    self:resetActions()
end

-- 重置状态[颜色]
function M:resetColor()
    -- 重置状态[颜色]
    self:setColor(self:getColorAlive())
end

-- 重置状态[CD]
function M:resetCD()
    local cd = self:getWaitRoundsTotal()
    self:showWaitRounds()
    self:setWaitRounds(cd)
end

-- 重置状态[数值]
function M:resetValue()
    local associatedCard = self:getAssociatedCard()
    associatedCard:reload()
end

-- 重置状态[清除所有动作]
function M:resetActions()
    local actions = self:getAssociatedCard():getActions()
    if TDArrayUtils.isNotEmpty(actions) then 
        for _, effects in pairs(actions) do
            for k in pairs(effects) do
                effects[k]:removeFromParentAndCleanup(true)
            end
        end
    end
end

-- 卡牌浮动到其卡牌层最上方
function M:float()
    self:getParent():reorderCard(self)
end



---------


-- [实例方法][内部调用]
-- 显示就绪盒子
function M:showBoxReady()
    local entity = self._boxReady

    -- 显示
    entity:setVisible(true)

    -- 动画
    self._boxReady:stopAllActions()
    -- 构造动作
    local params = {
        prefix      = "f_c_ready",
        frameCount  = 9,
        interval    = (1 / 18),
    }
    local action = TDAnimationUtils.animateWithFrameRepeat(params)
    -- 执行动作
    entity:runAction(action)
end

-- 隐藏就绪盒子
function M:hideBoxReady()
    self._boxReady:stopAllActions()
    self._boxReady:setVisible(false)
end

-- 判断是否准备就绪(就绪盒子显示，冷却时间为0)
function M:isReady()
    return self._boxReady:isVisible()
end

-- 判断卡牌是否已经死亡
function M:isDie()
    local color         = self:getColor()
    local colorCompare  = self:getColorDie()
    return color.r == colorCompare.r and color.g == colorCompare.g and color.b == colorCompare.b
end

-- 判断是否为状态[上场]
function M:isStateToUp()
    return self:isReady()
end

-- 判断是否为状态[下场]
function M:isStateToDown()
    local color         = self:getColor()
    local colorCompare  = self:getColorStateUp()
    return color.r == colorCompare.r and color.g == colorCompare.g and color.b == colorCompare.b
end

-- 获得顶层视图
function M:getTopView()
    return self:getParent():getParent()
end


---------


-- [实例方法][侦听事件]
-- 处理点击事件处理函数
function M:onFilter(x, y) 
    -- 显示卡片详细信息
    return self:isVisible() and util.isPointInNode(self, x, y)
end

-- 处理点击事件处理函数
function M:onTouch()
    -- 判断卡牌是否已经死亡
    if self:isDie() then
        return
    end

    -- 判断卡片是否已经准备就绪
    if self:isReady() then
        -- 卡片上场
        self:stageUp()
    else
        if self:isStateToDown() then 
            -- 卡片下场
            self:stageDown()
        else
            -- 显示卡片详细信息
            self:showCardDetail()
        end
    end
end

-- 显示卡片详细信息
function M:showCardDetail()
    -- [Version1]
    --[[
    local topView = self:getTopView()
    topView:addLayerCardDetail(self._po)
    --]]

    -- [Version2]
    -- 暂停所有调度器和动作
    local topView = self:getTopView() 
    topView:pauseAll()
    local entity = CardEntity.new({ id = self._po:getId(), evolv = self._po:getEvolv(), lv = self._po:getLv(), talent = topView:getTalent(self._owner), equipEffect = topView:getEquipEffect(self._owner), addvalue = self._po:getAddvalue()})
    entity.onConfirm = function ( )
        topView:resumeAll()
    end
    CCBCenter.loadCCB("MSG_CARDDETAILLAYER_MODEL", "ccb.c.cardgroup.CardDetailLayer", entity
    --     , function(layer)
    --     -- 绑定点击事件
    --     widget.bindTouchEvent{
    --         layer =  util.toCCLayer(layer),
    --         clickListener = function() layer:removeFromParentAndCleanup(true); topView:resumeAll() end,
    --         priority = -9999,
    --         swallowsTouches = true,
    --     }

    --     -- 获得顶层视图
    --     local topView = self:getTopView()
    --     -- 添加详情页
    --     topView:addChild(layer, 9999)
    -- end
    )
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得关联的下场卡牌
function M:getAssociatedCard()
    return self._associatedCard
end
-- 设置关联的下场卡牌
function M:setAssociatedCard(card)
    self._associatedCard = card
end





return M