--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

上场卡片类，定义卡片公用操作方法及逻辑实现。

-   扩展位置卡片。
-   附加战斗上场卡片展示逻辑。

]]

local M = class("StageUpCard", ImageCard)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 文本动画时间
M.A_TEXT_T              = .5
-- 文本动画过渡次数
M.A_TEXT_TMS            = 20

-- 操作变量
-- ..










-- [实例方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    M.super.ctor(self, params)



    -- [参数赋值]
    -- [卡牌]
    self._associatedCard        = nil                                                   -- 关联的下场卡牌
    -- [实体]
    self._offsetShadow          = params.offsetShadow           or P_ZERO               -- 偏移量[阴影]
    -- [侦听事件]
    self._filter                = self.onFilter                                         -- 过滤事件
    self._handler               = self.onTouch                                          -- 侦听事件
    -- [动作]
    self._actions               = {}                                                    -- 动作集
end










-- [实例方法]
-- [实例方法][实体渲染]
-- 实体渲染
function M:entityRender()
    -- [加载超类方法]
    M.super.entityRender(self)

    -- 加载背景
    self:loadShadow()
end

-- 加载阴影
function M:loadShadow()
    -- 构建实体
    local sprite = display.newSprite(PHI_FIGHT .. "f_c_img_shadow.png")
    self:addChild(sprite, -99)

    -- 设置位置
    util.alignNodeWithPadding(sprite, self:getContentSize(), display.CENTER, {left = self._offsetShadow.x, top = self._offsetShadow.y})
    -- print("_offsetShadow"..self._offsetShadow.x)
    -- sprite:setPosition(ccp(sprite:getPositionX() + self._offsetShadow.x, sprite:getPositionY() + self._offsetShadow.y))
end


---------


-- [外部调用]
-- 动画显示标签[HP]
function M:animateHp(value, color0, color1)
    -- [Version1]
    local params = {
        label   = self._labelHp,
        tag     = "HP",
        valueTo = value,
        color0  = color0, 
        color1  = color1, 
    }

    self:animateText(params)

    -- [Version2]
    --[[
    local params = {
        label   = self._numHp,
        tag     = "HP",
        valueTo = value,
        align   = LinearNum.ALIGN_RIGHT,
        color0  = color0, 
        color1  = color1, 
    }

    self:animateNum(params)
    --]]
end

-- 动画显示标签[攻击力]
function M:animteAttack(value, color0, color1)
    -- [Version1]
    -- 如果减攻击后的数值和原攻击一样，颜色变为白色
    if self._po:getAttack() == value then
        color1 = FA__OPT_VALUE__C_ATK_WH
    end
    local params = {
        label   = self._labelAttack,
        tag     = "AT",
        valueTo = value,
        color0  = color0, 
        color1  = color1, 
    }

    self:animateText(params)

    -- [Version2]
    --[[
    local params = {
        label   = self._numAttack,
        tag     = "AT",
        valueTo = value,
        align   = LinearNum.ALIGN_LEFT,
        color0  = color0, 
        color1  = color1, 
    }

    self:animateNum(params)
    --]]
end

-- 动画显示标签文本
function M:animateText(params)
    -- [参数赋值]
    local label             = params.label                              -- 标签对象
    local tag               = params.tag                                -- 内容标识
    local valueTo           = params.valueTo                            -- 目标值
    local color0            = params.color0                             -- 颜色值[初始]
    local color1            = params.color1                             -- 颜色值[结束]


    -- [动画参数]
    local valueFrom         = tonumber(label:getString())               -- 初始值
    local difference        = (valueTo - valueFrom)                     -- 变化值
    local time              = M.A_TEXT_T                                -- 动画时间
    local times             = M.A_TEXT_TMS                              -- 动画次数

    -- [格式化]
    if math.abs(difference) < times then 
        times = math.abs(difference)
    end
    -- [过滤判断]
    if times == 0 then 
        -- print("[WARNING!!!!]times == 0")
        -- print("[WARNING!!!!]times == 0")
        -- print("[WARNING!!!!]times == 0")
        return
    end
    local increase          = math.floor(difference / times)
    local interval          = FightActionHelper:time(time / times)


    -- [调试日志]
    -- print("valueTo", valueTo)
    -- print("valueFrom", valueFrom)
    -- print("difference", difference)
    -- print("increase", increase)
    -- print("interval", interval)



    -- [动画]
    -- [动画初始化]
    label:stopAllActions()
    label:setColor(color0)
    label:setString(valueFrom)

    -- [动画执行]
    local actions = {}
    for i = 1, times do
        if i ~= times then 
            actions[#actions + 1] = CCCallFunc:create(function() label:setString(valueFrom + i * increase) end)
        else
            actions[#actions + 1] = CCCallFunc:create(function() label:setString(valueTo);label:setColor(color1) end)
        end
        
        actions[#actions + 1] = CCDelayTime:create(interval)
    end
    -- 运行动作
    label:runAction(TDAnimationUtils.sequence(actions))
end


-- 动画显示数字
function M:animateNum(params)
    -- [参数赋值]
    local label             = params.label                              -- 标签对象
    local tag               = params.tag                                -- 内容标识
    local valueTo           = params.valueTo                            -- 目标值
    local align             = params.align                              -- 数字排列方式
    local color0            = params.color0                             -- 颜色值[初始]
    local color1            = params.color1                             -- 颜色值[结束]


    -- [动画参数]
    local valueFrom         = label:getValue()                          -- 初始值
    local difference        = (valueTo - valueFrom)                     -- 变化值
    local time              = M.A_TEXT_T                                -- 动画时间
    local times             = M.A_TEXT_TMS                              -- 动画次数

    -- [格式化]
    if math.abs(difference) < times then 
        times = math.abs(difference)
    end
    -- [过滤判断]
    if times == 0 then 
        -- print("[WARNING!!!!]times == 0")
        -- print("[WARNING!!!!]times == 0")
        -- print("[WARNING!!!!]times == 0")
        return
    end

    local increase          = math.floor(difference / times)
    local interval          = FightActionHelper:time(time / times)


    -- [调试日志]
    -- print("valueTo", valueTo)
    -- print("valueFrom", valueFrom)
    -- print("difference", difference)
    -- print("increase", increase)
    -- print("interval", interval)



    -- [动画]
    local function colorPrefix(color)
        -- [攻击力]
        -- 颜色[攻击力－增长]
        local colorToCompare = FA__OPT_VALUE__C_ATK_U
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vb"
        end
        -- 颜色[攻击力－增长结束]
        local colorToCompare = FA__OPT_VALUE__C_ATK_UE
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vb"
        end

        -- 颜色[攻击力－减少]
        local colorToCompare = FA__OPT_VALUE__C_ATK_D
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vw"
        end
        -- 颜色[攻击力－减少结束]
        local colorToCompare = FA__OPT_VALUE__C_ATK_DE
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vw"
        end


        -- [血量]
        -- 颜色[血量－增长]
        local colorToCompare = FA__OPT_VALUE__C_HP_U
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vg"
        end
        -- 颜色[血量－增长结束]
        local colorToCompare = FA__OPT_VALUE__C_HP_UE
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vw"
        end

        -- 颜色[血量－减少]
        local colorToCompare = FA__OPT_VALUE__C_HP_D
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vr"
        end
        -- 颜色[血量－减少结束]
        local colorToCompare = FA__OPT_VALUE__C_HP_DE
        if color.r == colorToCompare.r and color.g == colorToCompare.g and color.b == colorToCompare.b then
            return "f_c_vw"
        end
    end

    -- [动画初始化]
    local prefix0 = colorPrefix(color0)
    local prefix1 = colorPrefix(color1)
    -- [动画初始化]
    label:stopAllActions()

    -- [动画执行]
    local actions = {}
    for i = 1, times do
        if i ~= times then 
            actions[#actions + 1] = CCCallFunc:create(function() 
                label:update({
                    prefix = prefix0,
                    value  = valueFrom + i * increase,
                    align  = align,
                })
            end)
        else
            actions[#actions + 1] = CCCallFunc:create(function() 
                label:update({
                    prefix = prefix1,
                    value  = valueTo,
                    align  = align,
                })
            end)
        end
        
        actions[#actions + 1] = CCDelayTime:create(interval)
    end
    -- 运行动作
    label:runAction(TDAnimationUtils.sequence(actions))
end

-- 卡牌浮动到其卡牌层最上方
function M:float()
    self:getParent():reorderCard(self)
end


---------


-- [实例方法][内部调用]
-- 获得顶层视图
function M:getTopView()
    return self:getParent():getParent()
end


---------


-- [实例方法][侦听事件]
-- 处理点击事件处理函数
function M:onFilter(x, y) 
    -- 显示卡片详细信息
    return self:isVisible() and util.isPointInNode(self, x, y)
end

-- 处理点击事件处理函数
function M:onTouch() 
    -- 显示卡片详细信息
    self:showCardDetail()
end

-- 显示卡片详细信息
function M:showCardDetail()
    -- [Version1]
    --[[
    local topView = self:getTopView()
    topView:addLayerCardDetail(self._po)
    --]]

    -- [Version2]
    -- 暂停所有调度器和动作
    local topView = self:getTopView() 
    topView:pauseAll()

    local entity = CardEntity.new({ evolv = self._po:getEvolv(), id = self._po:getId(), lv = self._po:getLv(), talent = topView:getTalent(self._owner), equipEffect = topView:getEquipEffect(self._owner), addvalue = self._po:getAddvalue()})
    entity.onConfirm = function ( )
        topView:resumeAll()
    end
    CCBCenter.loadCCB("MSG_CARDDETAILLAYER_MODEL", "ccb.c.cardgroup.CardDetailLayer", entity
    --     , function(layer)
    --     -- 绑定点击事件
    --     widget.bindTouchEvent{
    --         layer =  util.toCCLayer(layer),
    --         clickListener = function() layer:removeFromParentAndCleanup(true); topView:resumeAll() end,
    --         priority = -9999,
    --         swallowsTouches = true,
    --     }

    --     -- 获得顶层视图
    --     local topView = self:getTopView()
    --     -- 添加详情页
    --     topView:addChild(layer, 9999)
    -- end
    )
end


---------


-- [状态]
-- 重新加载
function M:reload()
    self._labelHp:removeFromParentAndCleanup(true)
    self._labelAttack:removeFromParentAndCleanup(true)

    self:loadLabelHp()
    self:loadLabelAttack()
end

function M:hideAll()
    self:setVisible(false)
    self._labelHp:setVisible(false)
    self._labelAttack:setVisible(false)
    -- self._labelName:setVisible(false)
    self._spriteImage:setVisible(false)
end

function M:showAll()
    -- if not self:isVisible() then

        self:setVisible(true)
        self._labelHp:setVisible(true)
        self._labelAttack:setVisible(true)
        -- self._labelName:setVisible(true)
        self._spriteImage:setVisible(true) 
        -- transition.fadeIn(self._spriteImage , {time = 0.4}) 
    -- end
end


---------


-- [实例方法][属性Getter/Setter]
-- 获得关联的下场卡牌
function M:getAssociatedCard()
    return self._associatedCard
end
-- 设置关联的下场卡牌
function M:setAssociatedCard(card)
    self._associatedCard = card
end

-- 获得动作集
function M:getActions()
    return self._actions
end
-- 设置动作集
function M:setActions(actions)
    self._actions = actions
end





return M