--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

战斗欢迎场景英雄类，定义战斗欢迎场景英雄相关操作方法及逻辑实现。

-   指定战斗欢迎场景英雄相关方法。

]]

local M = class("WelcomeHero", TDSprite)



-- [包含文件]
-- ...




-- [公共参数]
-- 常量
-- 标签字体大小
local FZ_LABEL      = (25 * display.contentScaleFactor)
-- 偏移量[底框]
local O_FRAME       = ccp(0, 20)
-- 偏移量[头像]
local O_AVATAR      = O_FRAME
-- 偏移量[姓名]
local O_NAME        = ccp(0, -63)
-- 缩放比例[头像]
local S_AVATAR      = .5

-- 操作变量
-- ..
local TDAnimationUtils              = require(PH_MDL_UTIL .. "TDAnimationUtils")

-- [枚举]
-- 结点枚举
M.NODETAG__LOWERBOUND               = 100
M.NODETAG_FRAME                     = M.NODETAG__LOWERBOUND + 10
M.NODETAG_AVATAR                    = M.NODETAG__LOWERBOUND + 20
M.NODETAG_LABEL_NAME                = M.NODETAG__LOWERBOUND + 30










-- [构造方法]
--[[--
    创建对象
]]
function M:ctor(params)
    -- [加载超类构造函数]
    self:assertParameters(params)



    -- [参数赋值]
    self._po                = params.po                                 -- 英雄PO
    self._fileName          = params.fileName                           -- 文件名[图片]
    self._owner             = params.owner                              -- 拥有者标识


    


    -- [具体操作]
    -- 扩展onEnter/onExit事件处理
    TDExtendNode.extendRenderDestuctor(self)
end










-- [实例方法]
-- [实体渲染]
-- 实体渲染
function M:entityRender()
    -- 加载底框
    self:loadFrame()
    -- 加载头像
    self:loadAvatar()
    -- 加载标签
    self:loadLabels()

    -- 加载实体渲染其他操作
    self:loadRenderOtherwise()
end

-- 加载底框
function M:loadFrame()
    -- [构建实体]
    -- 定义配置参数
    local params = {
        fileName    = PHI_FIGHT .. "f_wlc_box_avatar.png",               -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        offset      = O_FRAME,                                          -- 偏移量
    }
    -- 生成精灵
    local sprite = TDUIUtils.loadSprite(params)

    -- 存储状态
    self.frame = sprite
end

-- 加载头像
function M:loadAvatar()
    -- [构建实体]
    -- 定义配置参数
    local params = {
        fileName    = self:getFrameNameAvatar(),                        -- 文件名
        parentNode  = self,                                             -- 父结点
        align       = display.CENTER,                                   -- 对齐方式
        anchor      = display.CENTER,                                   -- 锚点
        offset      = O_AVATAR,                                         -- 偏移量
        z           = 1,                                                -- 层深度
    }
    -- 生成精灵
    local sprite = TDUIUtils.loadSprite(params)
    -- sprite:setScale(S_AVATAR)

    -- 存储状态
    self.avatar = sprite
end

-- 加载标签
function M:loadLabels()
    -- 加载标签[姓名]
    self:loadLabelName()
end

-- 加载标签[姓名]
function M:loadLabelName()
    -- 定义配置参数
    local params = {
        text        = self._po:getName(),           -- 标签文字
        fontSize    = FZ_LABEL,                     -- 字体大小
        parentNode  = self,                         -- 父结点
        align       = display.CENTER,               -- 对齐方式
        anchor      = display.CENTER,               -- 锚点
        offset      = O_NAME,                       -- 偏移量
    }
    -- 生成标签
    local label = TDUIUtils.loadLabelTTF(params)

    -- 标记
    self._labelName = label
end

-- 加载实体渲染其他操作
function M:loadRenderOtherwise()
    
end


---------


-- [实例方法][实体析构]
-- 实体析构
function M:entityDestructor()

end


---------


-- [实例方法][外部调用]
-- 动画显示
function M:fadeOutWithChildren(params)
    local time              = params.time                   or .5
    local opacity           = params.opacity                or 0
    local delay             = params.delay                  or 0
    local exceptNodeTags    = params.exceptNodeTags         or nil
    local node              = self
    local time1             = time
    local opacity1          = opacity
    local children          = node:getChildren()



    -- [本结点]
    -- 目标结点
    local node      = self
    local array     = {}
    -- 延迟时间
    if delay then 
        array[#array + 1] = CCDelayTime:create(delay)
    end
    -- 透明度
    array[#array + 1] = CCFadeTo:create(time1, opacity1)

    -- 处理回调函数
    if i == children:count() - 1 then 
        if callback then 
            array[#array + 1] = CCCallFunc:create(callback)
        end
    end

    node:runAction(TDAnimationUtils.sequence(array))



    -- [子结点]
    if children:count() > 0 then
        for i = 0, children:count() - 1 do
            -- 目标结点
            local childNode = util.toCCSprite(children:objectAtIndex(i))
            local array     = {}

            -- 判断是否是过渡结点
            local isSkip    = false
            if TDArrayUtils.isNotEmpty(exceptNodeTags) then  
                for i = 1, #exceptNodeTags do
                    if childNode:getTag() == exceptNodeTags[i] then 
                        isSkip = true
                        break
                    end
                end
            end

            if not isSkip then 
                -- 初始设置
                -- ..
                -- 延迟时间
                if delay then 
                    array[#array + 1] = CCDelayTime:create(delay)
                end
                -- 透明度
                array[#array + 1] = CCFadeTo:create(time1, opacity1)

                -- 处理回调函数
                if i == children:count() - 1 then 
                    if callback then 
                        array[#array + 1] = CCCallFunc:create(callback)
                    end
                end

                childNode:runAction(TDAnimationUtils.sequence(array))
            end
        end
    end
end

-- 获得血量[PO]
function M:getPo()
    return self._po
end

-- 获得文件名[头像]
function M:getFrameNameAvatar()
    local owner = self._po:getOwner()
    local sex = self._po:getSex()
    local idd = self._po:getId()

    if owner == OWNER_ME then 
        return format("%sr_ico_%s.png", PHI_ROLE, (sex == GENDER_MALE) and "m" or "fm")
    else
        local entity = CardEntity.new({ id = idd })
        return entity:getCardIcon()
    end
end

-- 获得血量[当前值]
function M:getName()
    return self:getPo():getName()
end

-- 获得拥有者标识
function M:getOwner()
    return self._owner
end

-- 获得头像
function M:getAvatar()
    return self.avatar
end

-- 获得头像偏移量
function M:getAvatarOffset()
    return O_AVATAR
end


---------


-- [实例方法][验证]
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
    -- 验证参数合法性(参数集)
    TDAssertUtils.assertParameters(params)
    -- 验证参数合法性(文件名)
    TDAssertUtils.assertTypeString(params.fileName)
end





return M