--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

动画工具类，定义动画相关操作方法及逻辑实现。

-   指定动画逻辑。
-   提供便捷生成动画接口。

]]

local M = {}



-- [公共参数]
-- 全局变量
local _actionsToPause   = nil
local _schedulesToPause = nil

-- 常量
-- ..










-- [工具方法]
-- [Animate & Animation]
-- [Animate & Animation][Animate]
--[[
/** 
 *
 * @Description: 使用多张图片文件生成一个动画(CCAnimation)对象
 * @Notice: 使用animationWithFile这个方法需要满足两个前提: 
 *    1. 动画帧文件的名字带有从0开始计数的连续数字。 
 *    2. 动画帧文件必须是.png文件。
 *
 * @param prefix: 动画文件前缀名称
 * @param frameCount: 动画帧数
 * @param delay: 动画帧间隔
 * @return: CCAnimation动画对象
 *
 * @author: AC    @date: 2012-04-19 16:40:00
 *
 */
--]]
function M.animationWithFile(params)
    -- [参数赋值]
    local prefix 			= params.prefix 							-- 帧前缀
    local frameCount 		= params.frameCount 						-- 动画帧数
    local interval          = params.interval     or 1.0 / frameCount   -- 帧间隔


    -- [构建帧集合]
    -- 把动画帧作为贴图进行加载,然后生成精灵动画帧
    local frames = CCArray:create()
    for i = 1, frameCount do
        -- 1.获得图片文件名
    	local fileName = format("%s%d.png", prefix, i)
        -- 2.生成图片Texture2D对象
        local texture = M.addTextureCache(fileName)
        -- 3.获得图片Texture2D对象的rect
        local texSize = texture:getContentSize()
        local texRect = CCRect(0, 0, texSize.width, texSize.height)
        -- 2.生成CCSpriteFrame
        local frame = CCSpriteFrame:create(texture, texRect)
        frames:addObject(frame)
    end

    
    -- [生成动画对象]
    -- 使用所有的精灵动画帧,返回一个动画对象
    return CCAnimation:createWithSpriteFrames(frames, interval)
end

--[[
/** 
 *
 * @Description: 使用单张图片文件生成一个动画(CCAnimation)对象
 * @Notice: 使用animationWithFrame这个方法需要满足两个前提: 
 *    1. 动画帧文件的名字带有从0开始计数的连续数字。 
 *    2. 动画帧文件必须是.png文件。
 *
 * @param prefix: 动画帧前缀名称
 * @param frameCount: 动画帧数
 * @param delay: 动画帧间隔
 * @return: CCAnimation动画对象
 *
 * @author: AC    @date: 2012-04-19 16:40:00
 *
 */
 --]]
function M.animationWithFrame(params)
    -- [参数赋值]
    local prefix 			= params.prefix 							-- 帧前缀
    local frameCount 		= params.frameCount 						-- 动画帧数
    local interval 			= params.interval     or 1.0 / frameCount 	-- 帧间隔


    -- [构建帧集合]
    -- 把动画帧作为贴图进行加载,然后生成精灵动画帧
    local frames = CCArray:create()
    for i = 1, frameCount do
        -- 1.获得图片文件名
    	local frameName = format("%s%d.png", prefix, i)
    	local frame 	= display.newSpriteFrame(frameName)
        frames:addObject(frame)
	end
	
    
    -- [生成动画对象]
    -- 使用所有的精灵动画帧,返回一个动画对象
    return CCAnimation:createWithSpriteFrames(frames, interval)
end

--[[
/** 
 *
 * @Description: 使用图片帧名称集合生成一个动画(CCAnimation)对象
 * @Notice: 使用animationWithFrame这个方法需要满足两个前提: 
 *    1. 动画帧文件必须是.png文件。
 *
 * @param frameNames: 动画帧名称集合
 * @param delay: 动画帧间隔
 * @return: CCAnimation动画对象
 *
 * @author: AC    @date: 2012-08-13 16:40:00
 *
 */
 --]]
function M.animationWithFrameNames(params)
    -- [参数赋值]
    local frameNames 		= params.frameNames 						-- 帧名称集
    local interval          = params.interval                           -- 帧间隔


    -- [构建帧集合]
    -- 把动画帧作为贴图进行加载,然后生成精灵动画帧
    local frames = CCArray:create()
    local frameCount = #frameNames
    for i = 1, frameCount do
        -- 1.获得图片文件名
    	local frameName = frameNames[i]
    	local frame 	= display.newSpriteFrame(frameName)
        frames:addObject(frame)
	end
	
    
    -- [生成动画对象]
    -- 使用所有的精灵动画帧,返回一个动画对象
    return CCAnimation:createWithSpriteFrames(frames, interval)
end

-- [Animate & Animation][Animation]
--[[
/** 
 *
 * @Description: 使用多张图片文件生成一个动画(CCAnimate)对象
 * @Notice: 使用animationWithFile这个方法需要满足两个前提: 
 *    1. 动画帧文件的名字带有从0开始计数的连续数字。 
 *    2. 动画帧文件必须是.png文件。
 *
 * @param prefix: 动画文件前缀名称
 * @param frameCount: 动画帧数
 * @param delay: 动画帧间隔
 * @return: CCAnimate动画对象
 *
 * @author: AC    @date: 2012-04-19 16:40:00
 *
 */
 --]]
function M.animateWithFile(params)
    local animation = M.animationWithFile(params)
    return CCAnimate:create(animation)
end

--[[
/** 
 *
 * @Description: 使用单张图片文件生成一个动画(CCAnimate)对象
 * @Notice: 使用animationWithFrame这个方法需要满足两个前提: 
 *    1. 动画帧文件的名字带有从0开始计数的连续数字。 
 *    2. 动画帧文件必须是.png文件。
 *
 * @param prefix: 动画帧前缀名称
 * @param frameCount: 动画帧数
 * @param delay: 动画帧间隔
 * @return: CCAnimate动画对象
 *
 * @author: AC    @date: 2012-04-19 16:40:00
 *
 */
 --]]
function M.animateWithFrame(params)
    local animation = M.animationWithFrame(params)
    return CCAnimate:create(animation)
end

 --[[
/** 
 *
 * @Description: 使用图片帧名称集合生成一个动画(CCAnimate)对象
 * @Notice: 使用animationWithFrame这个方法需要满足两个前提: 
 *    1. 动画帧文件必须是.png文件。
 *
 * @param frameNames: 动画帧名称集合
 * @param delay: 动画帧间隔
 * @return: CCAnimate动画对象
 *
 * @author: AC    @date: 2012-08-13 16:40:00
 *
 */
 --]]
function M.animateWithFrameNames(params)
    local animation = M.animationWithFrameNames(params)
    return CCAnimate:create(animation)
end

--[[
/** 
 *
 * @Description: ［重复］使用多张图片文件生成一个动画(CCAnimate)对象
 * @Notice: 使用animationWithFile这个方法需要满足两个前提: 
 *    1. 动画帧文件的名字带有从0开始计数的连续数字。 
 *    2. 动画帧文件必须是.png文件。
 *
 * @param prefix: 动画文件前缀名称
 * @param frameCount: 动画帧数
 * @param delay: 动画帧间隔
 * @return: CCAnimate动画对象
 *
 * @author: AC    @date: 2012-04-19 16:40:00
 *
 */
 --]]
function M.animateWithFileRepeat(params)
    local animate = M.animateWithFile(params)
    return CCRepeatForever:create(animate)
end

--[[
/** 
 *
 * @Description: ［重复］使用单张图片文件生成一个动画(CCAnimate)对象
 * @Notice: 使用animationWithFrame这个方法需要满足两个前提: 
 *    1. 动画帧文件的名字带有从0开始计数的连续数字。 
 *    2. 动画帧文件必须是.png文件。
 *
 * @param prefix: 动画帧前缀名称
 * @param frameCount: 动画帧数
 * @param delay: 动画帧间隔
 * @return: CCAnimate动画对象
 *
 * @author: AC    @date: 2012-04-19 16:40:00
 *
 */
--]]
function M.animateWithFrameRepeat(params)
    local animate = M.animateWithFrame(params)
    return CCRepeatForever:create(animate)
end

--[[
/** 
 *
 * @Description: ［重复］使用图片帧名称集合生成一个动画(CCAnimate)对象
 * @Notice: 使用animationWithFrame这个方法需要满足两个前提: 
 *    1. 动画帧文件必须是.png文件。
 *
 * @param frameNames: 动画帧名称集合
 * @param delay: 动画帧间隔
 * @return: CCAnimate动画对象
 *
 * @author: AC    @date: 2012-08-13 16:40:00
 *
 */
--]]
function M.animateWithFrameNamesRepeat(params)
    local animate = M.animateWithFrameNames(params)
    return CCRepeatForever:create(animate)
end










-- [Indexing]
--[[
/** 
 *
 * @Description: 获得指定帧的帧名称
 *
 * @param index: 帧索引
 * @param prefix: 帧前缀名称
 * @return: 帧名称
 *
 * @author: AC    @date: 2012-04-25 12:52:03
 *
 */
--]]
function M.frameNameAtIndex(index, prefix)
    return format("%s%d.png", prefix, index)
end

--[[
/** 
 *
 * @Description: 获得第一帧的帧名称
 *
 * @param prefix: 帧前缀名称
 * @return: 帧名称
 *
 * @author: AC    @date: 2012-04-25 12:52:03
 *
 */
--]]
function M.frameNameAtFirst(prefix)
    return M.frameNameAtIndex(1, prefix)
end










-- [缓存Cache]
-- 添加图片进纹理缓存
function M.addTextureCache(fileName)
	return CCTextureCache:sharedTextureCache():addImage(fileName)
end

--[[--

从资源名称中添加多个精灵帧

### Example:

    TDAnimationUtils.addSpriteFramesWithResource("sprites")
    local sprite1 = display.newSprite("#sprite0001.png")

### Parameters:

-   string **resource** 资源路径前缀

]]
-- 添加图片进纹理缓存
function M.addSpriteFramesWithResource(resource)
    local plistFilename = resource .. ".plist"
    local imageName     = resource .. ".png"
    display.addSpriteFramesWithFile(plistFilename, imageName)
end

--[[--

从资源名称中移除多个精灵帧.

Sprite Frames stored in this file will be removed. It is convinient to call this method when a specific texture needs to be removed.

### Parameters:

-   string **resource** 资源路径前缀

]]
function M.removeSpriteFramesWithResource(resource)
    local plistFilename = resource .. ".plist"
    local imageName     = resource .. ".png"
    display.removeSpriteFramesWithFile(plistFilename, imageName)
end

--[[--

从图片及Plist文件中添加多个精灵帧

### Example:

    TDAnimationUtils.addSpriteFramesWithFile("sprites.plist", "sprites.png")
    local sprite1 = display.newSprite("#sprite0001.png")

### Parameters:

-   string **plistFilename** Plist文件名

-   string **image**         图片文件名

]]
-- 添加图片进纹理缓存
function M.addSpriteFramesWithFile(plistFilename, image)
    display.addSpriteFramesWithFile(plistFilename, image)
end

--[[--

从Plist文件中移除多个精灵帧.

Sprite Frames stored in this file will be removed. It is convinient to call this method when a specific texture needs to be removed.

### Parameters:

-   string **plistFilename** Plist文件名

]]
function M.removeSpriteFramesWithFile(plistFilename, imageName)
    display.removeSpriteFramesWithFile(plistFilename, imageName)
end

--[[--
    
]]
function M.removeSpriteFrameByImageName(imageName)
    display.removeSpriteFrameByImageName(imageName)
end










-- [调用控制]
-- 暂停所有调用器和动作
function M.pauseAllSchedulerAndActions()
    -- 暂停所有调用器
    M.pauseAllSchedulers()
    -- 暂停所有动作
    M.pauseAllActions()
end

-- 暂停所有调用器
function M.pauseAllSchedulers()
    _schedulesToPause = CCDirector:sharedDirector():getScheduler():pauseAllTargets()
end

-- 暂停所有动作
function M.pauseAllActions()
    _actionsToPause = CCDirector:sharedDirector():getActionManager():pauseAllRunningActions()
end

-- 暂停指定对象集的动作
function M.pauseActionsInTargets(targets)
    if targets then 
        for i = 1, #targets do
            CCDirector:sharedDirector():getActionManager():pauseTarget(targets[i])
        end
    end
end

-- 恢复所有调用器和动作
function M.resumeAllSchedulerAndActions()
    -- 暂停所有调用器
    M.resumeAllSchedulers()
    -- 暂停所有动作
    M.resumeAllActions()
end

-- 恢复所有调用器
function M.resumeAllSchedulers()
    CCDirector:sharedDirector():getScheduler():resumeTargets(_schedulesToPause)
end

-- 恢复所有动作
function M.resumeAllActions()
    CCDirector:sharedDirector():getActionManager():resumeTargets(_actionsToPause)
end

-- 恢复指定对象集的动作
function M.resumeActionsInTargets(targets)
    if targets then 
        for i = 1, #targets do
            CCDirector:sharedDirector():getActionManager():resumeTarget(targets[i])
        end
    end
end










-- [动作]
-- 生成合并动作
function M.spawn(actions)
    if #actions < 1 then return end
    if #actions < 2 then return actions[1] end

    local prev = actions[1]
    for i = 2, #actions do
        prev = CCSpawn:createWithTwoActions(prev, actions[i])
    end
    return prev
end

-- 生成序列动作
function M.sequence(actions)
	return transition.sequence(actions)
end

-- 生成重复序列动作
function M.sequenceRepeat(actions)
    return CCRepeatForever:create(transition.sequence(actions))
end





return M