--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

集合工具类，定义集合相关操作方法及逻辑实现。

-   指定集合逻辑。
-   公共集合定义。

]]

local M = {}



-- [公共参数]
-- 常量

-- 操作变量
-- 断言
local TDAssertUtils      			= require(PH_MDL_UTIL .. "TDAssertUtils")





-- [私有方法]
-- [声明]

-- [定义]





-- [公有方法]
-- 验证集合是否为空
function M.isEmpty(array, tag)
    -- 默认赋值
    local tag           = tag      or "array";
    local errorMessage  = "[" .. tag .. "] must be not empty!";

    -- 验证合法性
    return (array == nil) or (M.count(array) == 0)
end

-- 验证集合是否不为空
function M.isNotEmpty(array, tag)
    return not M.isEmpty(array, tag)
end

-- 获得数量
function M.count(array)	
	local count = 0

	if array ~= nil then 
		for _ in pairs(array) do
			count = count + 1
		end
	end

	return count
end

-- 添加元素到集合
function M.addItem(array, item)
	if array == nil then 
		return
	end

	array[#array + 1] = item
end

-- 删除元素从集合
function M.removeItem(array, item, isWalkOverAll)
	if array == nil then 
		return
	end

	isWalkOverAll = isWalkOverAll or false
	for i in pairs(array) do
		local thisItem = array[i]
		if thisItem == item then
			-- table.remove(array, i)
			array[i] = nil

			if not isWalkOverAll then 
				break
			end
		end
	end
end

-- 添加元素到集合[索引为有顺索引]
function M.addItemOrder(array, item)
	if array == nil then 
		return
	end

	array[#array + 1] = item
end

-- 删除元素从集合[索引为有顺索引]
function M.removeItemOrder(array, item, isWalkOverAll)
	if array == nil then 
		return
	end

	isWalkOverAll = isWalkOverAll or false
	for i in ipairs(array) do
		local thisItem = array[i]
		if thisItem == item then
			table.remove(array, i)

			if not isWalkOverAll then 
				break
			end
		end
	end
end

-- 推入元素到栈
function M.pushItem(array, item)
	if array == nil then 
		return
	end

	table.insert(array, item)
end

-- 弹出元素从栈
function M.popItem(array)
	if array == nil then 
		return
	end

	local indexTop = table.maxn(array)
	local item = array[indexTop]
	table.remove(array)

	return item
end

-- 合并集体
function M.merge(array1, array2)
	-- 合法性验证
	TDAssertUtils.assertNotNil(array1)

	-- 如果集合2为空，则直接返回集合1
	if array2 == nil then 
		return array1
	end

	for i,v in ipairs(array2) do
		array1[i] = v
	end

	return array1
end

return M;
