--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

断言工具类，定义断言相关操作方法及逻辑实现。

-   指定逻辑断言。
-   公共断言定义。

]]

local M = {}



-- [公共参数]
-- 常量





-- [私有方法]
-- [声明]
local isTypeString

-- [定义]
function isTypeString(str)
    return type(str) == "string"
end





-- [断言方法]
-- 验证参数合法性（是否为表格类型）
function M.assertParameters(params, source, tag)
    -- 默认赋值
    local source        = source    or "Unknown"
    local tag           = tag       or ""

    -- FIXME: 验证优化
    -- 验证合法性
    assert(type(params) == "table" or type(params) == "userdata",
           "[" .. tostring(source) .. "]" .. tag .. " invalid params!")
end

-- 验证是否为字符串
function M.assertTypeString(str, source, tag)
    -- 默认赋值
    local source        = source    or "Unknown"
    local tag           = tag       or ""

    -- 验证合法性
    assert(isTypeString(str),
           "[" .. tostring(source) .. "]" .. tag .. " invalid type \"string\"!")
end

-- 验证是否是某个类
function M.assertInherit(instance, clazz, tag)
    -- 默认赋值
    local classname     = clazz.classname
    local tag           = tag       or ""

    -- FIXME: 逻辑未处理
    -- 验证合法性
    assert(true,
           "[" .. tostring(instance) .. "]" .. tag .. " inequial inherit from class[" .. tostring(classname) .. "] !")
end

-- 验证是否是某个类
function M.assertClassname(instance, clazz, tag)
    -- 默认赋值
    local classname     = clazz.classname
    local tag           = tag       or ""

    -- 验证合法性
    assert(classname == instance.classname,
           "[" .. tostring(instance) .. "]" .. tag .. " inequial classname !")
end

-- 验证参数是否不为空
function M.assertNotNil(instance, tag)
    -- 默认赋值
    local tag           = tag       or ""

    -- 验证合法性
    assert(nil ~= instance,
           "[" .. tostring(instance) .. "]" .. tag .. " must be not nil!")
end

-- 验证参数是否为空
function M.assertNil(instance, tag)
    -- 默认赋值
    local tag           = tag       or ""
    
    -- 验证合法性
    assert(nil == instance,
           "[" .. tostring(instance) .. "]" .. tag .. " must be nil!")
end

-- 验证参数合法性
function M.assert(judge, errorMessage)
    -- 默认赋值
    errorMessage     = errorMessage or "Assert failed! becase judge is [" .. tostring(judge) .. "]";

    -- 验证合法性
    assert(judge, errorMessage)
end

return M