--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

触碰工具类，定义触碰相关操作方法及逻辑实现。

-   公共触碰逻辑定义。

]]

local M = {}



-- [公共参数]
-- 常量
--[[TOUCH类型]]
M.TOUCHBEGAN 			    = "began" 			   -- CCTOUCHBEGAN
M.TOUCHMOVED 			    = "moved" 			   -- CCTOUCHMOVED
M.TOUCHENDED 			    = "ended" 			   -- CCTOUCHENDED
M.TOUCHCANCELLED 		  = "cancelled" 		 -- CCTOUCHCANCELLED
-- [遮罩层]
M.PRIORITY_MASK       = -10000           -- 遮罩层点击优先级

-- 操作变量
-- 断言
local TDAssertUtils 	= require(PH_MDL_UTIL .. "TDAssertUtils")





-- [私有方法]
-- [声明]

-- [定义]





-- [工具方法]
--[[ 

绑定Touch事件

**Parameters:**

-   params:             table               参数
                           --#必填项#
                           CCLayer                  layer                   待绑定的layer
                           --#选填项# 
                           boolean                  isMultiTouches          是否是多点触摸,默认为false
                           int                      priority                优先级,默认为0
                           boolean                  swallowsTouches         是否是截断处理,默认为true
                           fun(x, y)                onTouchBegan
                           fun(x, y)                onTouchMoved
                           fun(x, y)                onTouchEnded
                           fun(x, y)                onTouchCancelled
                           fun(layer)               clickListener           点击处理
                
--]]
function M.bindTouchEvent( params )
	-- 合法性验证
    TDAssertUtils.assertNotNil(params.layer, "invalid params.layer")

    --#必填项#
    local _layer = params.layer 
    --#选填项#
    local _isMultiTouches = params.isMultiTouches or false
    local _priority = params.priority or 0
    local _swallowsTouches = params.swallowsTouches or true
    local _onTouchBegan = params.onTouchBegan
    local _onTouchMoved = params.onTouchMoved
    local _onTouchEnded = params.onTouchEnded
    local _onTouchCancelled = params.onTouchCancelled
    local _clickListener = params.clickListener
    --#内部参数#
    local _beganX, _beganY
    local _ismoved
    local onTouch
    local onTouchBegan
    local onTouchMoved
    local onTouchEnded
    local onTouchCancelled

    function onTouchBegan( x, y )
        _beganX, _beganY = x, y
        if _onTouchBegan then 
            return _onTouchBegan(x, y)
        else
            return util.isPointInNode(_layer, x, y)
        end
    end

    function onTouchMoved( x, y )
        if _onTouchMoved then 
            _onTouchMoved(x, y)
        else
            if _ismoved ~= true and (math.abs(x - _beganX) > 5 or math.abs(y - _beganY) > 5) then
                _ismoved = true
            end
        end
    end

    function onTouchEnded( x, y )
        if _onTouchEnded then 
            _onTouchEnded(x, y)
        else
            if _ismoved ~= true then
                --可以算是点击了,看看点的是谁
                if _clickListener then
                    _clickListener(_layer, x, y)
                end
            end
        end
        _layer:resetTouchStatus(  )
    end

    function onTouchCancelled( x, y )
        if _onTouchCancelled then 
            _onTouchCancelled(x, y)
        end
        _layer:resetTouchStatus(  )
    end

    function onTouch(eventType, x, y)
        --printf("%s newGrid() onTouch: x = %d, y = %d", "TAG", x, y)
        if eventType == M.TOUCHBEGAN then
            return onTouchBegan(x, y)
        elseif eventType == M.TOUCHMOVED then 
            onTouchMoved(x, y)
        elseif eventType == M.TOUCHENDED then
            onTouchEnded(x, y)
        elseif eventType == M.TOUCHCANCELLED then
            onTouchCancelled(x, y)
        end
    end

--[[ ## public fuction --]]
    function _layer:getBeganX(  )
        return _beganX
    end

    function _layer:setBeganX( x )
        _beganX = x
    end

    function _layer:getBeganY(  )
        return _beganY
    end

    function _layer:setBeganY( y )
        _beganY = y
    end

    function _layer:IsMoved(  )
        return _ismoved
    end

    function _layer:setIsMoved( ismoved )
        _ismoved = ismoved
    end

    function _layer:resetTouchStatus(  )
        _beganX, _beganY = 0, 0
        _ismoved = nil
    end

    -- display.extendLayer(_layer)
    CCLayerExtend.extend(_layer)
    _layer:addTouchEventListener(onTouch, _isMultiTouches, _priority, _swallowsTouches)
    _layer:setTouchEnabled(true)
end

--[[ 

绑定结点的Touch事件

**Parameters:**

-   params:             table               参数
                           --#必填项#
                           CCNode                  	node                    待绑定事件的结点
                           --#选填项# 
                           boolean                  isMultiTouches          是否是多点触摸,默认为false
                           int                      priority                优先级,默认为0
                           boolean                  swallowsTouches         是否是截断处理,默认为false
                           fun(x, y)                onTouchBegan
                           fun(x, y)                onTouchMoved
                           fun(x, y)                onTouchEnded
                           fun(x, y)                onTouchCancelled
                           fun(node, touchLayer)    handler           		事件处理函数
                
--]]
function M.bindNodeTouchEvent(params)
	-- 合法性验证
    TDAssertUtils.assertNotNil(params.node, "invalid params.node")

    -- 默认赋值
    local node          = params.node
    local handler 			= params.handler  or nil
    local filter        = params.filter   or nil


    ----------------------------------------------------------------
    -- 意图：
    -- 		向结点中添加一个点击层，点击层的contentSize与结点相同
    ----------------------------------------------------------------
    -- 添加触碰逻辑
    local layerForTouch = display.newLayer()
    layerForTouch:setContentSize(node:getContentSize())
    node:addChild(layerForTouch)

    local onTouchBegan = nil
    if filter then 
      onTouchBegan = function(x, y)
        return filter(node, x, y)
      end
    end

    widget.bindTouchEvent{ 
      layer = layerForTouch, 
      onTouchBegan = onTouchBegan,
      priority = params.priority,
      swallowsTouches = params.swallowsTouches,
      clickInner = true,
      clickListener = function() handler(node) end 
    }
end

--[[ 

绑定层的Touch事件

**Parameters:**

-   params:             table               参数
                           --#必填项#
                           CCLayer                  layer                   待绑定事件的结点
                           --#选填项# 
                           boolean                  isMultiTouches          是否是多点触摸,默认为false
                           int                      priority                优先级,默认为0
                           boolean                  swallowsTouches         是否是截断处理,默认为false
                           fun(x, y)                onTouchBegan
                           fun(x, y)                onTouchMoved
                           fun(x, y)                onTouchEnded
                           fun(x, y)                onTouchCancelled
                           fun(node, touchLayer)    handler                 事件处理函数
                
--]]
function M.bindLayerTouchEvent(params)
    -- 合法性验证
    TDAssertUtils.assertNotNil(params.layer, "invalid params.node")

    -- 默认赋值
    local layer             = params.layer
    local handler           = params.handler or nil
    local priority          = params.priority
    local filter            = params.filter   or nil


    ----------------------------------------------------------------
    -- 意图：
    --      向结点中添加一个点击层，点击层的contentSize与结点相同
    ----------------------------------------------------------------
    local onTouchBegan = nil
    if filter then 
      onTouchBegan = function(x, y)
        if not util.isCustomTouchEnabled(layer) then
            return false
        end
        return filter(layer, x, y)
      end
    end
    -- 添加触碰逻辑
    widget.bindTouchEvent{ 
      layer = layer, 
      onTouchBegan = onTouchBegan, 
      priority = priority, 
      clickInner = true,
      clickListener = function() handler(layer) end 
    }
end



return M