--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

UI工具类，定义UI相关操作方法及逻辑实现。

-   实现UI常用操作。

]]

local M = {}


-- [公共参数]
-- 常量
-- [未设置]
local UNSET_Z 							= 0
local UNSET_TAG 						= -1

-- [默认]标签字体大小
local DEFAULT_LABEL_FONTSIZE 			= (14 * display.contentScaleFactor)
local DEFAULT_LABEL_COLOR				= display.COLOR_WHITE

-- [操作变量]
-- 断言
local TDAssertUtils                     = require(PH_MDL_UTIL .. "TDAssertUtils")

-- [枚举]
-- 标签模式
M.MODE_LABEL_TTF                        = 1
M.MODE_LABEL_TTF_SHADOW                 = 2
M.MODE_LABEL_TTF_OUTLINE                = 3
M.MODE_LABEL_TTF_BMFONT                 = 4
M.LABEL_CLASS_TYPES = {
    ui.newTTFLabel,
    ui.newTTFLabelWithShadow,
    ui.newTTFLabelWithOutline,
    ui.newBMFontLabel,
}










-- [私有方法]
-- [声明]

-- [定义]





-- [操作方法]
-- 加载LabelTTF
function M.loadLabelTTF(params)
    -- 默认赋值
    local text      	= params.text 			or "Unknown"				-- 标签文字
    local fontName      = params.fontName       or ui.DEFAULT_TTF_FONT      -- 字体名称
    local fontSize  	= params.fontSize		or DEFAULT_FZ_LABEL			-- 字体大小
    local color     	= params.color			or DEFAULT_LABEL_COLOR		-- 字体颜色
    local parentNode 	= params.parentNode 	or nil						-- 父结点
    local align     	= params.align			or display.CENTER			-- 对齐方式
    local offset    	= params.offset			or ccp(0, 0)				-- 偏移量
    local anchor        = params.anchor         or display.CENTER           -- 锚点
    local z 			= params.z				or UNSET_Z 					-- 层深度
    local tag 			= params.tag			or UNSET_TAG 				-- 结点标签
    local mode          = params.mode           or M.MODE_LABEL_TTF         -- 模式


    -- 生成标签
    local node = M.LABEL_CLASS_TYPES[mode]({
        text       = text,                               	-- 标签文字
        font       = fontName,                              -- 字体名称
        size       = fontSize,                              -- 字体大小
        color      = color,                                 -- 颜色
    })
    -- 添加到精灵中
    if parentNode then
	    if tag == UNSET_TAG then
	    	parentNode:addChild(node, z)
	    else 
	    	parentNode:addChild(node, z, tag)
	    end
    end
    -- 定位结点
    if mode == M.MODE_LABEL_TTF or mode == M.MODE_LABEL_TTF_BMFONT then 
        M.alignNode(node, parentNode, align, anchor)
    elseif mode == M.MODE_LABEL_TTF_SHADOW then 
        M.alignNode(node.label, parentNode, align, anchor)
        M.alignNode(node.shadow1, parentNode, align, anchor)
        function node:getString() 
            return node.label:getString()
        end
    elseif mode == M.MODE_LABEL_TTF_OUTLINE then 
        M.alignNode(node.label, parentNode, align, anchor)
        M.alignNode(node.shadow1, parentNode, align, anchor)
        M.alignNode(node.shadow2, parentNode, align, anchor)
        M.alignNode(node.shadow3, parentNode, align, anchor)
        M.alignNode(node.shadow4, parentNode, align, anchor)
        function node:getString() 
            return node.label:getString()
        end
    end



    node:setPosition(ccp(node:getPositionX() + offset.x, node:getPositionY() + offset.y))

    return node
end

-- 加载Button
function M.loadButton(params)
    -- 参数赋值
    local fileNormal        = params.fileNormal
    local fileSelect        = params.fileSelect
    local fileDisable       = params.fileDisable
    local handler           = params.handler
    local labelParams       = params.labelParams
    local x                 = params.x                  or 0
    local y                 = params.y                  or 0
    local nodeTag           = params.nodeTag
    local z                 = params.z                  or UNSET_Z                  -- 层深度
    local tag               = params.tag                or UNSET_TAG                -- 结点标签
    local scale             = params.scale              or 1                        -- 缩放比例
    local parentNode        = params.parentNode         or nil                      -- 父结点
    local align             = params.align              or display.CENTER           -- 对齐方式
    local offset            = params.offset             or ccp(0, 0)                -- 偏移量
    local anchor            = params.anchor             or display.CENTER           -- 锚点


    -- 生成标签
    local button = widget.newButton({
        normalImage             = fileNormal,
        selectImage             = fileSelect,
        disableImage            = fileDisable,
        labelParams             = labelParams,
        listener                = function(sender) handler(self, sender) end,
    })
    button:setScale(scale)
    -- 添加到精灵中
    if parentNode then
        if tag == UNSET_TAG then
            parentNode:addChild(node, z)
        else 
            parentNode:addChild(node, z, tag)
        end
    end
    -- 定位结点
    M.alignNode(node, parentNode, align, anchor)
    node:setPosition(ccp(node:getPositionX() + offset.x, node:getPositionY() + offset.y))

    return button
end

-- 加载Sprite
function M.loadSprite(params)
    -- 合法性验证
    TDAssertUtils.assertTypeString(params.fileName)

    -- 默认赋值
    local fileName      = params.fileName                                   -- 文件名
    local color         = params.color          or DEFAULT_LABEL_COLOR      -- 颜色
    local parentNode    = params.parentNode     or nil                      -- 父结点
    local align         = params.align          or display.CENTER           -- 对齐方式
    local offset        = params.offset         or ccp(0, 0)                -- 偏移量
    local anchor        = params.anchor         or display.CENTER           -- 锚点
    local z             = params.z              or UNSET_Z                  -- 层深度
    local tag           = params.tag            or UNSET_TAG                -- 结点标签


    -- 生成标签
    local node = display.newSprite(fileName)
    node:setColor(color)

    -- 添加到精灵中
    if parentNode then
        if tag == UNSET_TAG then
            parentNode:addChild(node, z)
        else 
            parentNode:addChild(node, z, tag)
        end
    end
    -- 定位结点
    M.alignNode(node, parentNode, align, anchor)
    node:setPosition(ccp(node:getPositionX() + offset.x, node:getPositionY() + offset.y))

    return node
end

-- 排列结点
function M.alignNode(node, parentNode, align, anchor, offset)
    align   = align   or display.CENTER
    anchor  = anchor  or display.CENTER
    offset  = offset  or ccp(0, 0)

    local a = display.ANCHOR_POINTS[anchor]
    local sizeParent = M.realSize(parentNode)
    local sizeChild  = M.realSize(node)
    local aL, aR = sizeChild.width * a.x, sizeChild.width * (1 - a.x) * -1
    local aB, aT = sizeChild.height * a.y, sizeChild.height * (1 - a.y) * -1

    local x, y
    local pw, ph = sizeParent.width, sizeParent.height
    local cw, ch = sizeChild.width, sizeChild.height
    if align == display.TOP_LEFT then
        x, y =    0 + aL,   ph + aT
    elseif align == display.CENTER_LEFT then
        x, y =    0 + aL, ph/2 + 0
    elseif align == display.BOTTOM_LEFT then
        x, y =    0 + aL,    0 + aB
    elseif align == display.TOP_CENTER then
        x, y = pw/2 +  0,   ph + aT
    elseif align == display.CENTER then
        x, y = pw/2 +  0, ph/2 + 0
    elseif align == display.BOTTOM_CENTER then
        x, y = pw/2 +  0,    0 + aB
    elseif align == display.TOP_RIGHT then
        x, y =   pw + aR,   ph + aT
    elseif align == display.CENTER_RIGHT then
        x, y =   pw + aR, ph/2 + 0
    else
        x, y =   pw + aR,    0 + aB
    end

    x, y = x + offset.x, y + offset.y
    node:ignoreAnchorPointForPosition(false)
    display.align(node, anchor, x, y)
end

function M.realSize(node)
    local contentSize = node:getContentSize()
    local scale       = M.realScale(node)
    return CCSize(contentSize.width * scale, contentSize.height * scale)
    --[[
    return node:getContentSize()
    --]]
end

function M.realScale(node)
    local scale = node:getScale()
    if node:getParent() ~= nil then 
        scale = scale * M.realScale(node:getParent())
    end

    return scale
end



return M