--[[

Copyright (c) 2012-2013 gao7.com

http://www.gao7.com/LizardMan/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

]]

--[[--

版本工具类，定义版本相关操作方法及逻辑实现。

-   指定版本逻辑。
-   公共版本操作定义。

]]

local M = {}



-- [公共参数]
-- 操作变量
-- 断言
local TDAssertUtils      			= require(PH_MDL_UTIL .. "TDAssertUtils")
-- 字符串
local TDStringUtils                 = require(PH_MDL_UTIL .. "TDStringUtils")

-- 常量
local COMPARE_EQUAL                 = 0
local COMPARE_UP                    = 1
local COMPARE_DOWN                  = -1





-- [私有方法]
-- [声明]

-- [定义]





-- [公有方法]
--[[--

判断版本是否过时

]]
function M.isOutdate(version)
    return M.isDownThan(version)
end

--[[--

判断当前版本是否比比较版本新

]]
function M.isUpThan(version)
    return M.compareWith(version) == COMPARE_UP
end

--[[--

判断当前版本是否比比较版本旧

]]
function M.isDownThan(version)
    return M.compareWith(version) == COMPARE_DOWN
end

--[[--

判断当前版本是否与比较版本相同

]]
function M.isEqualWith(version)
    return M.compareWith(version) == COMPARE_EQUAL
end

--[[--

获得版本比较标识（0＝相同，1＝大于，-1＝小于）

]]
function M.compareWith(version)
    local sign = COMPARE_EQUAL

    local v1Params = TDStringUtils.split(CVERSION, "%.")
    local v2Params = TDStringUtils.split(version, "%.")


    local numBits = math.max(#v1Params, #v2Params)
    if #v1Params < numBits then 
        for i = 1, numBits - #v1Params do
            v1Params[#v1Params + 1] = 0
        end
    end
    if #v2Params < numBits then 
        for i = 1, numBits - #v2Params do
            v2Params[#v2Params + 1] = 0
        end
    end

    for i = 1, numBits do
        local bit1 = tonumber(v1Params[i])
        local bit2 = tonumber(v2Params[i])

        if bit1 > bit2 then
            sign = COMPARE_UP
            break
        elseif bit1 < bit2 then
            sign = COMPARE_DOWN
            break
        end
    end

    return sign
end

--[[--

转换版本（数字）到版本（字符串）

]]
function M.convert(versionNumber)
    if type(versionNumber) ~= "number" then
        -- TODO: error
        return
    end

    local str = ""
    local versionFormat = string.format("%d", versionNumber)
    for i = 1, string.len(versionFormat) do
        str = str .. string.sub(versionFormat, i, i) .. "."
    end

    return string.sub(str, 1, string.len(str) - 1)
end





return M